/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.compilation.utils.popup.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;

import fr.irisa.espresso.sme.compilation.utils.jobs.CompilingMethods;
import fr.irisa.espresso.sme.polychrony.GraphValue;
import fr.irisa.espresso.sme.polychrony.PKOptionManager;
import fr.irisa.espresso.sme.polychrony.PkPlugin;

/**
 * This class describes the general compilation feature
 */
public class GeneralCompilation {
	private boolean	LISGeneration	= false;
	private boolean	SMEGeneration	= false;
	
	public GeneralCompilation(final CompilingMethods compilingMethods, final long finalTree, final IFile sourceFile,
			final GraphValue graphValue, final CompilationUtils compilationUtils) {
		final PKOptionManager optionManager = new PKOptionManager();
		
		Display display = Display.getCurrent();
		final Shell shell = new Shell(display);
		shell.setLayout(new GridLayout(2, false));
		shell.setText("General Compilation");
		
		// Tabs Definitions
		TabFolder tabfolder = new TabFolder(shell, SWT.BORDER);
		TabItem tabGeneralOptions = new TabItem(tabfolder, SWT.BORDER);
		tabGeneralOptions.setText("General Options");
		TabItem tabPartitionning = new TabItem(tabfolder, SWT.BORDER);
		tabPartitionning.setText("Partitioning");
		TabItem exportTools = new TabItem(tabfolder, SWT.BORDER);
		exportTools.setText("Export Tools");
		TabItem tabCodeGeneration = new TabItem(tabfolder, SWT.BORDER);
		tabCodeGeneration.setText("Code Generation");
		TabItem tabSignalForms = new TabItem(tabfolder, SWT.BORDER);
		tabSignalForms.setText("SIGNAL Forms");
		TabItem tabMisc = new TabItem(tabfolder, SWT.BORDER);
		tabMisc.setText("Miscellaneous");
		
		Composite pageGeneralOptions = new Composite(tabfolder, SWT.BORDER);
		tabGeneralOptions.setControl(pageGeneralOptions);
		Composite pagePartitioning = new Composite(tabfolder, SWT.BORDER);
		tabPartitionning.setControl(pagePartitioning);
		Composite pageExportTools = new Composite(tabfolder, SWT.BORDER);
		exportTools.setControl(pageExportTools);
		Composite pageCodeGeneration = new Composite(tabfolder, SWT.BORDER);
		tabCodeGeneration.setControl(pageCodeGeneration);
		Composite pageSignalForms = new Composite(tabfolder, SWT.BORDER);
		tabSignalForms.setControl(pageSignalForms);
		Composite pageMisc = new Composite(tabfolder, SWT.BORDER);
		tabMisc.setControl(pageMisc);
		
		Composite compositeButtons = new Composite(shell, SWT.BORDER);
		Layout buttonsLayout = new GridLayout();
		compositeButtons.setLayout(buttonsLayout);
		
		// General Options
		final Button events2BooleansButton = new Button(pageGeneralOptions, SWT.CHECK);
		events2BooleansButton.setBounds(10, 10, 200, 25);
		events2BooleansButton.setText("Events To Booleans");
		events2BooleansButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (events2BooleansButton.getSelection()) {
						optionManager.setOptGenBF(true);
					}
					else {
						optionManager.setOptGenBF(false);
					}
				}
			}
		});
		
		final Button flatteningButton = new Button(pageGeneralOptions, SWT.CHECK);
		flatteningButton.setBounds(10, 40, 200, 25);
		flatteningButton.setText("Flattening");
		flatteningButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (flatteningButton.getSelection()) {
						optionManager.setOptGenFBF(true);
					}
					else {
						optionManager.setOptGenFBF(false);
					}
				}
			}
		});
		
		final Button sequentializingButton = new Button(pageGeneralOptions, SWT.CHECK);
		sequentializingButton.setBounds(10, 70, 200, 25);
		sequentializingButton.setText("Sequentializing");
		sequentializingButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (sequentializingButton.getSelection()) {
						optionManager.setOptGenSBF(true);
					}
					else {
						optionManager.setOptGenSBF(false);
					}
				}
			}
		});
		
		final Button profilingButton = new Button(pageGeneralOptions, SWT.CHECK);
		profilingButton.setBounds(10, 100, 200, 25);
		profilingButton.setText("Profiling");
		profilingButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (profilingButton.getSelection()) {
						optionManager.setOptProfiling(true);
					}
					else {
						optionManager.setOptProfiling(false);
					}
				}
			}
		});
		
		final Button retimingButton = new Button(pageGeneralOptions, SWT.CHECK);
		retimingButton.setBounds(10, 130, 200, 25);
		retimingButton.setText("Retiming");
		retimingButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (retimingButton.getSelection()) {
						optionManager.setOptRetiming(true);
					}
					else {
						optionManager.setOptRetiming(false);
					}
				}
			}
		});
		
		final Button abstractionButton = new Button(pageGeneralOptions, SWT.CHECK);
		abstractionButton.setBounds(220, 10, 200, 25);
		abstractionButton.setText("Abstraction");
		abstractionButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (abstractionButton.getSelection()) {
						optionManager.setOptAbstraction(true);
					}
					else {
						optionManager.setOptAbstraction(false);
					}
				}
			}
		});
		
		final Button unificationButton = new Button(pageGeneralOptions, SWT.CHECK);
		unificationButton.setBounds(220, 40, 200, 25);
		unificationButton.setText("Signals Unification");
		unificationButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (unificationButton.getSelection()) {
						optionManager.setOptUnification(true);
					}
					else {
						optionManager.setOptUnification(false);
					}
				}
			}
		});
		
		final Button boolean2EventsButton = new Button(pageGeneralOptions, SWT.CHECK);
		boolean2EventsButton.setBounds(220, 70, 200, 25);
		boolean2EventsButton.setText("Booleans To Events");
		boolean2EventsButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (boolean2EventsButton.getSelection()) {
						optionManager.setOptCrew(true);
					}
					else {
						optionManager.setOptCrew(false);
					}
				}
			}
		});
		
		final Button kernelButton = new Button(pageGeneralOptions, SWT.CHECK);
		kernelButton.setBounds(220, 100, 200, 25);
		kernelButton.setText("Kernel Translation");
		kernelButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (kernelButton.getSelection()) {
						optionManager.setOptKernel(true);
					}
					else {
						optionManager.setOptKernel(false);
					}
				}
			}
		});
		
		final Button clockCalculusDepthButton = new Button(pageGeneralOptions, SWT.CHECK);
		clockCalculusDepthButton.setBounds(10, 160, 200, 25);
		clockCalculusDepthButton.setText("Clock Calculus Depth");
		final Text depthValue = new Text(pageGeneralOptions, SWT.BORDER);
		depthValue.setBounds(220, 160, 100, 25);
		depthValue.setEnabled(false);
		clockCalculusDepthButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (clockCalculusDepthButton.getSelection()) {
						depthValue.setEnabled(true);
					}
					else {
						depthValue.setText("");
						depthValue.setEnabled(false);
					}
				}
			}
		});
		
		final Button exclusiveClockButton = new Button(pageGeneralOptions, SWT.CHECK);
		exclusiveClockButton.setBounds(10, 190, 200, 25);
		exclusiveClockButton.setText("Exclusive Clocks");
		final Text exclusiveClockValue = new Text(pageGeneralOptions, SWT.BORDER);
		exclusiveClockValue.setBounds(220, 190, 100, 25);
		exclusiveClockValue.setEnabled(false);
		exclusiveClockButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (exclusiveClockButton.getSelection()) {
						exclusiveClockValue.setEnabled(true);
					}
					else {
						exclusiveClockValue.setText("");
						exclusiveClockValue.setEnabled(false);
					}
				}
			}
		});
		
		// Partitioning Options
		final Button sequentialClusteringButton = new Button(pagePartitioning, SWT.CHECK);
		sequentialClusteringButton.setBounds(10, 10, 210, 25);
		sequentialClusteringButton.setText("Sequential Clustering");
		sequentialClusteringButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (sequentialClusteringButton.getSelection()) {
						optionManager.setOptClustering(true);
					}
					else {
						optionManager.setOptClustering(false);
					}
				}
			}
		});
		
		final Button separateBooleansOthersButton = new Button(pagePartitioning, SWT.CHECK);
		separateBooleansOthersButton.setBounds(10, 40, 210, 25);
		separateBooleansOthersButton.setText("Separate Booleans/Others");
		separateBooleansOthersButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (separateBooleansOthersButton.getSelection()) {
						optionManager.setOptSeparateBooleansOthers(true);
					}
					else {
						optionManager.setOptSeparateBooleansOthers(false);
					}
				}
			}
		});
		
		final Button separateStatesOthersButton = new Button(pagePartitioning, SWT.CHECK);
		separateStatesOthersButton.setBounds(10, 70, 210, 25);
		separateStatesOthersButton.setText("Separate States/Others");
		separateStatesOthersButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (separateStatesOthersButton.getSelection()) {
						optionManager.setOptSeparateStatesOthers(true);
					}
					else {
						optionManager.setOptSeparateStatesOthers(false);
					}
				}
			}
		});
		
		// Export Tools
		final Button sigaliGenButton = new Button(pageExportTools, SWT.CHECK);
		sigaliGenButton.setBounds(10, 10, 200, 25);
		sigaliGenButton.setText("Z/3Z - Sigali");
		sigaliGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (sigaliGenButton.getSelection()) {
						optionManager.setOptGenz3z(true);
					}
					else {
						optionManager.setOptGenz3z(false);
					}
				}
			}
		});
		
		final Button lustreButton = new Button(pageExportTools, SWT.CHECK);
		lustreButton.setBounds(10, 40, 200, 25);
		lustreButton.setText("Lustre Generation");
		lustreButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (lustreButton.getSelection()) {
						optionManager.setOptGenLustre(true);
					}
					else {
						optionManager.setOptGenLustre(false);
					}
				}
			}
		});
		
		final Button syndexButton = new Button(pageExportTools, SWT.CHECK);
		syndexButton.setBounds(10, 70, 200, 25);
		syndexButton.setText("Syndex Generation");
		syndexButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (syndexButton.getSelection()) {
						optionManager.setOptGenSyndex(true);
					}
					else {
						optionManager.setOptGenSyndex(false);
					}
				}
			}
		});
		
		final Button dotButton = new Button(pageExportTools, SWT.CHECK);
		dotButton.setBounds(10, 100, 200, 25);
		dotButton.setText("Dot Generation");
		dotButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (dotButton.getSelection()) {
						optionManager.setOptGenDot(true);
					}
					else {
						optionManager.setOptGenDot(false);
					}
				}
			}
		});
		
		final Button synopticButton = new Button(pageExportTools, SWT.CHECK);
		synopticButton.setBounds(10, 130, 200, 25);
		synopticButton.setText("Synoptic Generation");
		synopticButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (synopticButton.getSelection()) {
						optionManager.setOptSynoptic(true);
					}
					else {
						optionManager.setOptSynoptic(false);
					}
				}
			}
		});
		
		// Code Generation Options
		final Button cMainGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		cMainGenButton.setBounds(220, 10, 70, 25);
		cMainGenButton.setText("Main");
		cMainGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (cMainGenButton.getSelection()) {
						optionManager.setOptGenCMain(true);
					}
					else {
						optionManager.setOptGenCMain(false);
					}
				}
			}
		});
		
		final Button cBodyGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		cBodyGenButton.setBounds(300, 10, 70, 25);
		cBodyGenButton.setText("Body");
		cBodyGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (cBodyGenButton.getSelection()) {
						optionManager.setOptGenCBody(true);
					}
					else {
						optionManager.setOptGenCBody(false);
					}
				}
			}
		});
		
		final Button cIOGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		cIOGenButton.setBounds(380, 10, 70, 25);
		cIOGenButton.setText("I/O");
		cIOGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (cIOGenButton.getSelection()) {
						optionManager.setOptGenCIO(true);
					}
					else {
						optionManager.setOptGenCIO(false);
					}
				}
			}
		});
		
		final Button cGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		cGenButton.setBounds(10, 10, 200, 25);
		cGenButton.setText("C Generation");
		cGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (cGenButton.getSelection()) {
						cMainGenButton.setEnabled(true);
						cIOGenButton.setEnabled(true);
						cBodyGenButton.setEnabled(true);
						cMainGenButton.setSelection(true);
						cIOGenButton.setSelection(true);
						cBodyGenButton.setSelection(true);
						optionManager.setOptGenCMain(true);
						optionManager.setOptGenCBody(true);
						optionManager.setOptGenCIO(true);
					}
					else {
						cMainGenButton.setEnabled(false);
						cMainGenButton.setSelection(false);
						cIOGenButton.setEnabled(false);
						cIOGenButton.setSelection(false);
						cBodyGenButton.setEnabled(false);
						cBodyGenButton.setSelection(false);
						optionManager.setOptGenCMain(false);
						optionManager.setOptGenCBody(false);
						optionManager.setOptGenCIO(false);
					}
				}
			}
		});
		
		final Button cppMainGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		cppMainGenButton.setBounds(220, 40, 70, 25);
		cppMainGenButton.setText("Main");
		cppMainGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (cppMainGenButton.getSelection()) {
						optionManager.setOptGenCPPMain(true);
					}
					else {
						optionManager.setOptGenCPPMain(false);
					}
				}
			}
		});
		
		final Button cppBodyGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		cppBodyGenButton.setBounds(300, 40, 70, 25);
		cppBodyGenButton.setText("Body");
		cppBodyGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (cppBodyGenButton.getSelection()) {
						optionManager.setOptGenCPPBody(true);
					}
					else {
						optionManager.setOptGenCPPBody(false);
					}
				}
			}
		});
		
		final Button cppIOGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		cppIOGenButton.setBounds(380, 40, 70, 25);
		cppIOGenButton.setText("I/O");
		cppIOGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (cppIOGenButton.getSelection()) {
						optionManager.setOptGenCPPIO(true);
					}
					else {
						optionManager.setOptGenCPPIO(false);
					}
				}
			}
		});
		
		final Button cppGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		cppGenButton.setBounds(10, 40, 200, 25);
		cppGenButton.setText("C++ Generation");
		cppGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (cppGenButton.getSelection()) {
						cppMainGenButton.setEnabled(true);
						cppIOGenButton.setEnabled(true);
						cppBodyGenButton.setEnabled(true);
						cppMainGenButton.setSelection(true);
						cppIOGenButton.setSelection(true);
						cppBodyGenButton.setSelection(true);
						optionManager.setOptGenCPPMain(true);
						optionManager.setOptGenCPPBody(true);
						optionManager.setOptGenCPPIO(true);
					}
					else {
						cppMainGenButton.setEnabled(false);
						cppMainGenButton.setSelection(false);
						cppIOGenButton.setEnabled(false);
						cppIOGenButton.setSelection(false);
						cppBodyGenButton.setEnabled(false);
						cppBodyGenButton.setSelection(false);
						optionManager.setOptGenCPPMain(false);
						optionManager.setOptGenCPPBody(false);
						optionManager.setOptGenCPPIO(false);
					}
				}
			}
		});
		
		final Button javaMainGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		javaMainGenButton.setBounds(220, 70, 70, 25);
		javaMainGenButton.setText("Main");
		
		final Button javaBodyGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		javaBodyGenButton.setBounds(300, 70, 70, 25);
		javaBodyGenButton.setText("Body");
		
		final Button javaIOGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		javaIOGenButton.setBounds(380, 70, 70, 25);
		javaIOGenButton.setText("I/O");
		
		javaMainGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (javaMainGenButton.getSelection()) {
						optionManager.setOptGenJava(true);
						optionManager.setOptGenJavaMain(true);
					}
					else {
						if ((!javaBodyGenButton.getSelection()) && (!javaIOGenButton.getSelection())) {
							optionManager.setOptGenJava(false);
							optionManager.setOptGenJavaMain(false);
						}
						else {
							optionManager.setOptGenJavaMain(false);
						}
					}
				}
			}
		});
		
		javaBodyGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (javaBodyGenButton.getSelection()) {
						optionManager.setOptGenJava(true);
						optionManager.setOptGenJavaBody(true);
					}
					else {
						if ((!javaMainGenButton.getSelection()) && (!javaIOGenButton.getSelection())) {
							optionManager.setOptGenJava(false);
							optionManager.setOptGenJavaBody(false);
						}
						else {
							optionManager.setOptGenJavaBody(false);
						}
					}
				}
			}
		});
		
		javaIOGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (javaIOGenButton.getSelection()) {
						optionManager.setOptGenJava(true);
						optionManager.setOptGenJavaIO(true);
					}
					else {
						if ((!javaMainGenButton.getSelection()) && (!javaBodyGenButton.getSelection())) {
							optionManager.setOptGenJava(false);
							optionManager.setOptGenJavaIO(false);
						}
						else {
							optionManager.setOptGenJavaIO(false);
						}
					}
				}
			}
		});
		
		final Button javaGenButton = new Button(pageCodeGeneration, SWT.CHECK);
		javaGenButton.setBounds(10, 70, 200, 25);
		javaGenButton.setText("Java Generation");
		javaGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (javaGenButton.getSelection()) {
						javaMainGenButton.setEnabled(true);
						javaIOGenButton.setEnabled(true);
						javaBodyGenButton.setEnabled(true);
						javaMainGenButton.setSelection(true);
						javaIOGenButton.setSelection(true);
						javaBodyGenButton.setSelection(true);
						optionManager.setOptGenJava(true);
						optionManager.setOptGenJavaIO(true);
						optionManager.setOptGenJavaBody(true);
						optionManager.setOptGenJavaMain(true);
					}
					else {
						javaMainGenButton.setEnabled(false);
						javaMainGenButton.setSelection(false);
						javaIOGenButton.setEnabled(false);
						javaIOGenButton.setSelection(false);
						javaBodyGenButton.setEnabled(false);
						javaBodyGenButton.setSelection(false);
						optionManager.setOptGenJava(false);
						optionManager.setOptGenJavaMain(false);
						optionManager.setOptGenJavaBody(false);
						optionManager.setOptGenJavaIO(false);
					}
				}
			}
		});
		
		final Button checkButton = new Button(pageCodeGeneration, SWT.CHECK);
		checkButton.setBounds(10, 120, 200, 25);
		checkButton.setText("Check");
		checkButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (checkButton.getSelection()) {
						optionManager.setOptCheck(true);
					}
					else {
						optionManager.setOptCheck(false);
					}
				}
			}
		});
		
		final Button forceButton = new Button(pageCodeGeneration, SWT.CHECK);
		forceButton.setBounds(10, 150, 200, 25);
		forceButton.setText("Force");
		forceButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (forceButton.getSelection()) {
						optionManager.setOptAlwaysCode(true);
					}
					else {
						optionManager.setOptAlwaysCode(false);
					}
				}
			}
		});
		
		final Button distButton = new Button(pageCodeGeneration, SWT.CHECK);
		distButton.setBounds(10, 180, 200, 25);
		distButton.setText("Distribution");
		distButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (distButton.getSelection()) {
						optionManager.setOptGenDistrib(true);
					}
					else {
						optionManager.setOptGenDistrib(false);
					}
				}
			}
		});
		
		final Button delayMultiplexingButton = new Button(pageCodeGeneration, SWT.CHECK);
		delayMultiplexingButton.setBounds(220, 120, 200, 25);
		delayMultiplexingButton.setText("Delay Multiplexing");
		delayMultiplexingButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (delayMultiplexingButton.getSelection()) {
						optionManager.setOptDelayMultiplexing(true);
					}
					else {
						optionManager.setOptDelayMultiplexing(false);
					}
				}
			}
		});
		
		final Button delayOptimizingButton = new Button(pageCodeGeneration, SWT.CHECK);
		delayOptimizingButton.setBounds(220, 150, 200, 25);
		delayOptimizingButton.setText("Delay Optimizing");
		delayOptimizingButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (delayOptimizingButton.getSelection()) {
						optionManager.setOptDelayOptimizing(true);
					}
					else {
						optionManager.setOptDelayOptimizing(false);
					}
				}
			}
		});
		
		// SIGNAL Forms Generation Options
		final Button signalLISGenButton = new Button(pageSignalForms, SWT.CHECK);
		signalLISGenButton.setBounds(10, 10, 200, 25);
		signalLISGenButton.setText("Signal LIS Generation");
		signalLISGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (signalLISGenButton.getSelection()) {
						LISGeneration = true;
					}
					else {
						LISGeneration = false;
					}
				}
			}
		});
		
		final Button signalTRAGenButton = new Button(pageSignalForms, SWT.CHECK);
		signalTRAGenButton.setBounds(10, 40, 200, 25);
		signalTRAGenButton.setText("Signal TRA Generation");
		signalTRAGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (signalTRAGenButton.getSelection()) {
						optionManager.setOptGenTrace(true);
					}
					else {
						optionManager.setOptGenTrace(false);
					}
				}
			}
		});
		
		final Button smeGenButton = new Button(pageSignalForms, SWT.CHECK);
		smeGenButton.setBounds(10, 70, 200, 25);
		smeGenButton.setText("SME Generation");
		smeGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (smeGenButton.getSelection()) {
						SMEGeneration = true;
					}
					else {
						SMEGeneration = false;
					}
				}
			}
		});
		
		// Other Options
		final Button traceabilityButton = new Button(pageMisc, SWT.CHECK);
		traceabilityButton.setBounds(10, 10, 200, 25);
		traceabilityButton.setText("Traceability Mode");
		traceabilityButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (traceabilityButton.getSelection()) {
						optionManager.setOptTraceabilityMode(true);
					}
					else {
						optionManager.setOptTraceabilityMode(false);
					}
				}
			}
		});
		
		final Button verboseButton = new Button(pageMisc, SWT.CHECK);
		verboseButton.setBounds(10, 40, 200, 25);
		verboseButton.setText("Verbose");
		verboseButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (verboseButton.getSelection()) {
						optionManager.setOptVerbose(true);
					}
					else {
						optionManager.setOptVerbose(false);
					}
				}
			}
		});
		
		final Button statButton = new Button(pageMisc, SWT.CHECK);
		statButton.setBounds(10, 70, 200, 25);
		statButton.setText("Statistics");
		statButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (statButton.getSelection()) {
						optionManager.setOptStat(true);
					}
					else {
						optionManager.setOptStat(false);
					}
				}
			}
		});
		
		final Button warnButton = new Button(pageMisc, SWT.CHECK);
		warnButton.setBounds(220, 10, 200, 25);
		warnButton.setText("Warnings");
		warnButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (warnButton.getSelection()) {
						optionManager.setOptWarn(true);
					}
					else {
						optionManager.setOptWarn(false);
					}
				}
			}
		});
		
		final Button debugButton = new Button(pageMisc, SWT.CHECK);
		debugButton.setBounds(220, 40, 200, 25);
		debugButton.setText("Debug");
		debugButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (debugButton.getSelection()) {
						optionManager.setOptDebug(true);
					}
					else {
						optionManager.setOptDebug(false);
					}
				}
			}
		});
		
		final Button headerButton = new Button(pageMisc, SWT.CHECK);
		headerButton.setBounds(220, 70, 200, 25);
		headerButton.setText("Header");
		headerButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (headerButton.getSelection()) {
						optionManager.setOptHeader(true);
					}
					else {
						optionManager.setOptHeader(false);
					}
				}
			}
		});
		
		final Button substituteButton = new Button(pageMisc, SWT.CHECK);
		substituteButton.setBounds(10, 100, 200, 25);
		substituteButton.setText("Substitute");
		final Text substituteValue = new Text(pageMisc, SWT.BORDER);
		substituteValue.setBounds(220, 100, 100, 25);
		substituteValue.setEnabled(false);
		substituteButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					if (substituteButton.getSelection()) {
						substituteValue.setEnabled(true);
					}
					else {
						substituteValue.setText("");
						substituteValue.setEnabled(false);
					}
				}
			}
		});
		
		// Compile and Quit buttons.
		final Button compileButton = new Button(compositeButtons, SWT.PUSH);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		compileButton.setLayoutData(gridData);
		compileButton.setText("Compile");
		compileButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					boolean generation = true;
					if (LISGeneration) {
						PkPlugin.getServices().printSignalLIS(finalTree, 0);
					}
					if (SMEGeneration) {
						compilingMethods.generateSME(finalTree, sourceFile.getLocation());
					}
					try {
						if (clockCalculusDepthButton.getSelection()) {
							int value = Integer.parseInt(depthValue.getText());
							if (value < 0) {
								generation = false;
								showMessageBox(shell, "The clock calculus depth value must be positive.");
							}
							else {
								optionManager.setOptSolvingDepth(value);
							}
						}
					}
					catch (NumberFormatException nfe) {
						showMessageBox(shell, "Please enter a positive integer for the clock calculus depth value.");
						generation = false;
					}
					
					try {
						if (exclusiveClockButton.getSelection()) {
							int value = Integer.parseInt(exclusiveClockValue.getText());
							if ((value != 0) && (value != 1) && (value != 2)) {
								generation = false;
								showMessageBox(shell, "The exclusive clocks value must be 0, 1 or 2.");
							}
							else {
								optionManager.setOptExclusiveClock(true, value);
							}
						}
					}
					catch (NumberFormatException nfe) {
						showMessageBox(shell, "Please enter a integer between 0 and 2 for the exclusive clocks value.");
						generation = false;
					}
					
					try {
						if (substituteButton.getSelection()) {
							int value = Integer.parseInt(substituteValue.getText());
							if (value < 0) {
								generation = false;
								showMessageBox(shell, "The substitute value must be positive.");
							}
							else {
								optionManager.setOptSubstitute(value);
							}
						}
					}
					catch (NumberFormatException nfe) {
						showMessageBox(shell, "Please enter a positive integer for the substitute value.");
						generation = false;
					}
					
					if (generation) {
						optionManager.compile(graphValue.getGraph(), sourceFile.getName());
						shell.dispose();
					}
					compilationUtils.refreshContent();
				}
			}
		});
		
		final Button quitButton = new Button(compositeButtons, SWT.PUSH);
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		quitButton.setLayoutData(gridData2);
		quitButton.setText("Quit");
		quitButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					shell.dispose();
				}
			}
		});
		
		cMainGenButton.setEnabled(false);
		cIOGenButton.setEnabled(false);
		cBodyGenButton.setEnabled(false);
		cppMainGenButton.setEnabled(false);
		cppIOGenButton.setEnabled(false);
		cppBodyGenButton.setEnabled(false);
		javaMainGenButton.setEnabled(false);
		javaIOGenButton.setEnabled(false);
		javaBodyGenButton.setEnabled(false);
		
		shell.pack();
		shell.open();
		
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}
	
	/**
	 * Displays a message box with the given message.
	 * 
	 * @param shell
	 *        the shell where the message will be displayed
	 * @param message
	 *        the message to show
	 */
	private void showMessageBox(Shell shell, String message) {
		MessageBox mbox = new MessageBox(shell);
		mbox.setMessage(message);
		mbox.open();
	}
}
