/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.compilation.utils.popup.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import fr.irisa.espresso.sme.compilation.utils.jobs.CompilingMethods;
import fr.irisa.espresso.sme.polychrony.EnvironmentConstants;
import fr.irisa.espresso.sme.polychrony.GraphValue;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyConsole;

/**
 * This class describes the interactive compilation feature
 */
public class InteractiveCompilation {
	/** boolean set to true if a clustering is applied */
	private boolean	clustersApplied;
	/** boolean set to true if an abstraction is applied */
	private boolean	abstractionApplied;
	
	public InteractiveCompilation(final CompilingMethods compilingMethods, final long finalTree,
			final IFile sourceFile, final GraphValue graphValue, final CompilationUtils compilationUtils) {
		Display display = Display.getCurrent();
		final IPath fileLocation = sourceFile.getLocation();
		final IPath fileDirectory = fileLocation.removeFileExtension();
		final Shell shell = new Shell(display);
		
		shell.setLayout(new GridLayout());
		shell.setText("Interactive Compilation");
		
		// Buttons Definition
		
		Group groupOptions = createGroup(shell, "General Options");
		
		final Button events2BooleansButton = createButton(groupOptions, "Events To Booleans");
		final Button unificationButton = createButton(groupOptions, "Signals Unification");
		final Button flatteningButton = createButton(groupOptions, "Flattening");
		final Button clockCalculusButton = createButton(groupOptions, "Clock Calculus");
		final Button sequentializingButton = createButton(groupOptions, "Sequentializing");
		final Button retimingButton = createButton(groupOptions, "Retiming");
		final Button boolean2EventsButton = createButton(groupOptions, "Booleans To Events");
		final Button abstractionButton = createButton(groupOptions, "Abstraction");
		final Button sequentialClusteringButton = createButton(groupOptions, "Sequential Clustering");
		final Button profilingButton = createButton(groupOptions, "Profiling");
		
		Group groupExportTools = createGroup(shell, "Export Tools");
		
		final Button lustreButton = createButton(groupExportTools, "Lustre Generation");
		final Button syndexButton = createButton(groupExportTools, "Syndex Generation");
		final Button sigaliGenButton = createButton(groupExportTools, "Z/3Z - Sigali");
		
		Group groupGeneration = createGroup(shell, "Code Generation");
		
		final Button cGenButton = createButton(groupGeneration, "C Generation");
		final Button cppGenButton = createButton(groupGeneration, "C++ Generation");
		final Button javaGenButton = createButton(groupGeneration, "Java Generation");
		
		Group groupSignalForms = createGroup(shell, "Signal Forms");
		
		final Button signalGenButton = createButton(groupSignalForms, "Signal Generation");
		final Button signalLISGenButton = createButton(groupSignalForms, "Signal LIS Generation");
		final Button signalTRAGenButton = createButton(groupSignalForms, "Signal TRA Generation");
		final Button signalAbstractionButton = createButton(groupSignalForms, "Signal Abstraction");
		final Button smeGenButton = createButton(groupSignalForms, "SME Generation");
		
		final Button quitButton = new Button(shell, SWT.PUSH);
		quitButton.setText("Quit");
		GridData gridDataQuitButton = new GridData();
		gridDataQuitButton.horizontalAlignment = GridData.FILL;
		quitButton.setLayoutData(gridDataQuitButton);
		
		if (sourceFile.getFileExtension().equalsIgnoreCase("sme")) {
			smeGenButton.setEnabled(false);
		}
		else if (sourceFile.getFileExtension().equalsIgnoreCase("sig")) {
			signalGenButton.setEnabled(false);
		}
		sequentialClusteringButton.setEnabled(false);
		sequentializingButton.setEnabled(false);
		flatteningButton.setEnabled(false);
		cGenButton.setEnabled(false);
		cppGenButton.setEnabled(false);
		javaGenButton.setEnabled(false);
		profilingButton.setEnabled(false);
		lustreButton.setEnabled(false);
		syndexButton.setEnabled(false);
		signalAbstractionButton.setEnabled(false);
		
		// Buttons Listeners
		
		events2BooleansButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.bdcPlus(graphValue);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Events to booleans operation done for " + fileLocation);
						retimingButton.setEnabled(false);
						boolean2EventsButton.setEnabled(false);
						unificationButton.setEnabled(false);
						clockCalculusButton.setEnabled(false);
						events2BooleansButton.setEnabled(false);
						abstractionButton.setEnabled(false);
						sequentialClusteringButton.setEnabled(true);
						sequentializingButton.setEnabled(true);
						flatteningButton.setEnabled(true);
						profilingButton.setEnabled(true);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		unificationButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.unify(graphValue);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Signals unifications done for " + fileLocation);
						unificationButton.setEnabled(false);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		flatteningButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.stsFlatten(graphValue);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Flattening operation done for " + fileLocation);
						sequentializingButton.setEnabled(false);
						flatteningButton.setEnabled(false);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		clockCalculusButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.clockCalculus(graphValue);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Clock Calculus done for " + fileLocation);
						clockCalculusButton.setEnabled(false);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		sequentializingButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods
							.sequentializing(graphValue, finalTree, clustersApplied);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Sequentializing operation done for " + fileLocation);
						sequentialClusteringButton.setEnabled(false);
						sequentializingButton.setEnabled(false);
						flatteningButton.setEnabled(false);
						cGenButton.setEnabled(true);
						cppGenButton.setEnabled(true);
						javaGenButton.setEnabled(true);
						lustreButton.setEnabled(true);
						syndexButton.setEnabled(true);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		retimingButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.retiming(graphValue);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Retiming done for " + fileLocation);
						retimingButton.setEnabled(false);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		boolean2EventsButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.boolean2Events(graphValue);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Boolean to events operation done for " + fileLocation);
						boolean2EventsButton.setEnabled(false);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		abstractionButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					abstractionApplied = true;
					IStatus status = compilingMethods.abstraction(graphValue);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Abstraction done for " + fileLocation);
						retimingButton.setEnabled(false);
						boolean2EventsButton.setEnabled(false);
						unificationButton.setEnabled(false);
						clockCalculusButton.setEnabled(false);
						events2BooleansButton.setEnabled(false);
						abstractionButton.setEnabled(false);
						sequentialClusteringButton.setEnabled(true);
						sequentializingButton.setEnabled(true);
						flatteningButton.setEnabled(true);
						profilingButton.setEnabled(true);
						signalAbstractionButton.setEnabled(true);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		sequentialClusteringButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.cluster(graphValue);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Sequential clustering operation done for " + fileLocation);
						sequentialClusteringButton.setEnabled(false);
					}
					clustersApplied = status.isOK();
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		profilingButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateProfiling(graphValue.getGraph());
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Profiling generation done for " + fileLocation);
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		lustreButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateLustre(graphValue.getGraph());
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Lustre generation done for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		syndexButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateSyndex(graphValue.getGraph());
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Syndex generation done for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		sigaliGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateSigali(graphValue.getGraph());
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Sigali generation done for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		cGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateC(graphValue.getGraph(), abstractionApplied);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("C generation done for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		cppGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateCPP(graphValue.getGraph(), abstractionApplied);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("C++ generation done for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		javaGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateJava(graphValue.getGraph(), fileDirectory,
						abstractionApplied);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Java generation done for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		signalGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateSignal(finalTree, fileDirectory);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Signal File Generated for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		signalLISGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateSignalLIS(finalTree);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Signal LIS File Generated for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		signalTRAGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateSignalTRA(graphValue.getGraph());
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Signal TRA File Generated for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		signalAbstractionButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateSignalAbstraction(graphValue.getGraph());
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("Signal Abstraction Generation done for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		smeGenButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					IStatus status = compilingMethods.generateSME(finalTree, fileDirectory);
					if (!status.equals(Status.OK_STATUS)) {
						PolychronyConsole.printError("Error detected : " + status.getMessage());
					}
					else {
						PolychronyConsole.printInfo("SME File Generated for " + fileLocation);
						compilationUtils.refreshContent();
					}
					PolychronyConsole.printNewLine();
				}
			}
		});
		
		quitButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (e.type == SWT.Selection) {
					PkPlugin.getServices().setEnv(EnvironmentConstants.LIBRARY_PATH,
						compilationUtils.getSaveLibraryPath());
					shell.dispose();
				}
			}
		});
		
		shell.pack();
		shell.open();
		
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}
	
	/**
	 * This method creates a button and inserts it in a group
	 * 
	 * @param group
	 *        the group where the button will be inserted
	 * @param buttonName
	 *        the name of the button
	 * @return the button
	 */
	public Button createButton(Group group, String buttonName) {
		final Button button = new Button(group, SWT.PUSH);
		button.setText(buttonName);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		button.setLayoutData(gridData);
		return button;
	}
	
	/**
	 * This method creates a group of buttons and inserts it in a shell
	 * 
	 * @param shell
	 *        the shell where the group of buttons will be inserted
	 * @param groupName
	 *        the name of the group
	 * @return the group
	 */
	public Group createGroup(Shell shell, String groupName) {
		Group group = new Group(shell, SWT.NONE);
		group.setLayout(new GridLayout(2, true));
		group.setText(groupName);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		group.setLayoutData(gridData);
		return group;
	}
}
