/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.compilation.utils.simulation;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import fr.irisa.espresso.sme.compilation.utils.CompilationUtilsPlugin;

/**
 * This class generate the build<language>.xml file to compile and/or execute the code generated by Polychrony. This
 * generated files needs the presence of the PolychronyJavaLibs.jar file inside the source location (for Java).
 * 
 */
public class GenerateBuildFile {
	/**
	 * The directory where the source and library files (such as PolychronyJavaLibs.jar) are
	 */
	private IPath	sourceLocation;
	
	/**
	 * The name of the process to simulate
	 */
	private String	name;
	
	/**
	 * Constructor
	 * 
	 * @param source
	 *        The directory where the source and library files (such as PolychronyJavaLibs.jar) are
	 * @param name
	 *        The name of the process to simulate
	 */
	public GenerateBuildFile(IPath source, String name) {
		this.sourceLocation = source;
		this.name = name;
	}
	
	public void generateJavaFile() {
		if (sourceLocation == null) {
			CompilationUtilsPlugin.getDefault().getLog().log(
				new Status(IStatus.ERROR, CompilationUtilsPlugin.PLUGIN_ID,
					"GenerateBuildFile: The sourceLocation is null"));
		}
		
		// Check that the directory exists and that all directories exists
		File dir = sourceLocation.toFile();
		if (!dir.exists())
			dir.mkdirs();
		
		// The Ant file will be generated only if it does not already exist
		File file = new File(dir, "buildJava.xml");
		if (file.exists())
			return;
		
		PrintWriter pw = null;
		try {
			pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
			
			pw.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			pw.println("<project name=\"" + name + "\" default=\"exec\" basedir=\".\">");
			pw.println();
			pw.println("\t<property name=\"bin\" value=\"${basedir}/bin\"/>");
			pw.println();
			pw.println("\t<!-- =================================");
			pw.println("\t      target: clean");
			pw.println("\t     ================================= -->");
			pw.println("\t<target name=\"clean\" description=\"Delete the binary directory and all its content\">");
			pw.println("\t    <delete dir=\"${bin}\"/>");
			pw.println("\t</target>");
			pw.println();
			pw.println("\t<!-- =================================");
			pw.println("\t      target: build");
			pw.println("\t     ================================= -->");
			pw.println("\t<target name=\"build\" depends=\"clean\" description=\"Compile the " + name
					+ " simulation\">");
			pw.println("\t    <mkdir dir=\"${bin}\" description=\"Create the binary directory\" />");
			pw.println("\t    <javac srcdir=\"${basedir}\" destdir=\"${bin}\" includes=\"*.java\" ");
			pw.println("\t    \tclasspath=\"PolychronyJavaLibs.jar\" debug=\"on\" />");
			pw.println("\t</target>");
			pw.println();
			pw.println("\t<!-- =================================");
			pw.println("\t      target: exec");
			pw.println("\t     ================================= -->");
			pw
					.println("\t<target name=\"exec\" depends=\"build\" description=\"Execute the " + name
							+ " simulation\">");
			pw.println("\t    <java classname=\"" + name + "_main\">");
			pw.println("\t    \t<classpath>");
			pw.println("\t    \t\t<pathelement location=\"${basedir}/PolychronyJavaLibs.jar\"/>");
			pw.println("\t    \t\t<pathelement path=\"${bin}${path.separator}${java.class.path}\"/>");
			pw.println("\t    \t</classpath>");
			pw.println("\t    </java>");
			pw.println("\t</target>");
			pw.println();
			pw.println("</project>");
			
			pw.close();
		}
		catch (IOException ioe) {
			CompilationUtilsPlugin.getDefault().getLog().log(
				new Status(IStatus.ERROR, CompilationUtilsPlugin.PLUGIN_ID, IStatus.ERROR, ioe.getMessage(), ioe));
		}
	}
}
