/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations;

import java.net.URL;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.editor.IConfiguration;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.IPaletteManager;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.graphconf.exceptions.MissingGraphConfFileException;

import fr.irisa.espresso.sme.diagram.clockrelations.edit.AssertionClockConstraintOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.AssertionClockSpeedOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.AutomatonEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.BasicIterateEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockConstraintOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockRelationEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockRelationOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockSpeedOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockedExprEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ConditionEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.DependenceEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.DependenceInOutEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.DependenceOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.InputEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.InputInstanceEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.InputPortEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.IterateEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.LocalEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ModelInstanceEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.NullClockEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.OutputEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.OutputInstanceEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.SignalOrRefEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.SignalRefEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.SubProcessEditPart;

/*
 * A diagram configuration : manages Palette, EditPartFactory for this diagram.
 * 
 * @generated
 */
public class CRD_Configuration implements IConfiguration {
	/**
	 * @generated
	 */
	private CRD_PaletteManager	paletteManager;
	
	/**
	 * @generated
	 */
	private CRD_EditPartFactory	editPartFactory;
	
	/**
	 * @generated
	 */
	private CRD_CreationUtils	creationUtils;
	
	/**
	 * The DiagramGraphConf that contains graphical informations on the configuration
	 * 
	 * @generated
	 */
	private DiagramGraphConf	diagramGraphConf;
	
	/**
	 * Constructor. Initialize Adapter factories.
	 * 
	 * @generated
	 */
	public CRD_Configuration() {
		registerAdapters();
	}
	
	/**
	 * Registers the Adapter Factories for all the EditParts
	 * 
	 * @generated
	 */
	private void registerAdapters() {
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ClockedExprEditPart.class, fr.irisa.espresso.sme.ClockedExpr.class),
			ClockedExprEditPart.class);
		Platform.getAdapterManager()
				.registerAdapters(
					new EditPart2ModelAdapterFactory(DependenceInOutEditPart.class,
						fr.irisa.espresso.sme.DependenceInOut.class), DependenceInOutEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SignalOrRefEditPart.class, fr.irisa.espresso.sme.SignalOrRef.class),
			SignalOrRefEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SignalRefEditPart.class, fr.irisa.espresso.sme.SignalRef.class),
			SignalRefEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(InputEditPart.class, fr.irisa.espresso.sme.Input.class),
			InputEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(OutputEditPart.class, fr.irisa.espresso.sme.Output.class),
			OutputEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(LocalEditPart.class, fr.irisa.espresso.sme.Local.class),
			LocalEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(InputInstanceEditPart.class, fr.irisa.espresso.sme.InputInstance.class),
			InputInstanceEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(OutputInstanceEditPart.class, fr.irisa.espresso.sme.OutputInstance.class),
			OutputInstanceEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(BasicIterateEditPart.class, fr.irisa.espresso.sme.BasicIterate.class),
			BasicIterateEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(IterateEditPart.class, fr.irisa.espresso.sme.Iterate.class),
			IterateEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SubProcessEditPart.class, fr.irisa.espresso.sme.SubProcess.class),
			SubProcessEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(AutomatonEditPart.class, fr.irisa.espresso.sme.Automaton.class),
			AutomatonEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ModelInstanceEditPart.class, fr.irisa.espresso.sme.ModelInstance.class),
			ModelInstanceEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ClockConstraintOperatorEditPart.class,
				fr.irisa.espresso.sme.ClockConstraintOperator.class), ClockConstraintOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(AssertionClockConstraintOperatorEditPart.class,
				fr.irisa.espresso.sme.AssertionClockConstraintOperator.class),
			AssertionClockConstraintOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ClockRelationOperatorEditPart.class,
				fr.irisa.espresso.sme.ClockRelationOperator.class), ClockRelationOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ClockSpeedOperatorEditPart.class,
				fr.irisa.espresso.sme.ClockSpeedOperator.class), ClockSpeedOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(AssertionClockSpeedOperatorEditPart.class,
				fr.irisa.espresso.sme.AssertionClockSpeedOperator.class), AssertionClockSpeedOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(DependenceOperatorEditPart.class,
				fr.irisa.espresso.sme.DependenceOperator.class), DependenceOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(NullClockEditPart.class, fr.irisa.espresso.sme.NullClock.class),
			NullClockEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(InputPortEditPart.class, fr.irisa.espresso.sme.InputPort.class),
			InputPortEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ClockRelationEditPart.class, fr.irisa.espresso.sme.ClockRelation.class),
			ClockRelationEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ConditionEditPart.class, fr.irisa.espresso.sme.Condition.class),
			ConditionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(DependenceEditPart.class, fr.irisa.espresso.sme.Dependence.class),
			DependenceEditPart.class);
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getId()
	 * @generated
	 */
	public String getId() {
		return new String("fr.irisa.espresso.sme.diagram.clockrelations");
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getName()
	 * @generated
	 */
	public String getName() {
		return new String("Clock Relations and Dependences Diagram");
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getEditPartFactory()
	 * @generated
	 */
	public EditPartFactory getEditPartFactory() {
		if (editPartFactory == null) {
			editPartFactory = new CRD_EditPartFactory();
		}
		
		return editPartFactory;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getPaletteManager()
	 * @generated
	 */
	public IPaletteManager getPaletteManager() {
		if (paletteManager == null) {
			paletteManager = new CRD_PaletteManager(getCreationUtils());
		}
		
		return paletteManager;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getCreationUtils()
	 * @generated
	 */
	public ICreationUtils getCreationUtils() {
		if (creationUtils == null) {
			creationUtils = new CRD_CreationUtils(getDiagramGraphConf());
		}
		
		return creationUtils;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getDiagramGraphConf()
	 * @generated
	 */
	public DiagramGraphConf getDiagramGraphConf() {
		if (diagramGraphConf == null) {
			URL url = CRD_Plugin.getDefault().getBundle().getResource(
				"fr/irisa/espresso/sme/diagram/clockrelations/diagram.graphconf");
			if (url != null) {
				URI fileURI = URI.createURI(url.toString());
				ResourceSet resourceSet = new ResourceSetImpl();
				Resource resource = resourceSet.getResource(fileURI, true);
				if (resource != null && resource.getContents().get(0) instanceof DiagramGraphConf) {
					diagramGraphConf = (DiagramGraphConf) resource.getContents().get(0);
				}
			}
			else {
				new MissingGraphConfFileException(
					"The *.diagramgraphconf file can not be retrieved. Check if the path is correct in the Configuration class of your diagram.");
			}
		}
		
		return diagramGraphConf;
	}
	
}
