/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.edit.EListEditPart;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.editor.ModelerEditPartFactory;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.diagram.clockrelations.edit.AssertionClockConstraintOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.AssertionClockSpeedOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.AutomatonEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.BasicIterateEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.CRD_DiagramEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockConstraintOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockRelationEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockRelationOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockSpeedOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ClockedExprEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ConditionEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.DependenceEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.DependenceInOutEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.DependenceOperatorEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.InputEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.InputInstanceEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.InputPortEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.IterateEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.LocalEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.ModelInstanceEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.NullClockEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.OutputEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.OutputInstanceEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.SignalOrRefEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.SignalRefEditPart;
import fr.irisa.espresso.sme.diagram.clockrelations.edit.SubProcessEditPart;
import fr.irisa.espresso.sme.util.SmeSwitch;

/*
 * Part Factory : associates a model object to its controller. <br>
 * 
 * @generated
 */
public class CRD_EditPartFactory extends ModelerEditPartFactory {
	/**
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
	 * @generated
	 */
	public EditPart createEditPart(EditPart context, Object model) {
		if (model instanceof Diagram) {
			return new CRD_DiagramEditPart((Diagram) model);
		}
		else if (model instanceof GraphNode) {
			final GraphNode node = (GraphNode) model;
			EObject element = Utils.getElement(node);
			if (element != null) {
				if ("http://www.irisa.fr/espresso/sme.ecore".equals(element.eClass().getEPackage().getNsURI())) {
					return (EditPart) new NodeSmeSwitch(node).doSwitch(element);
				}
			}
			
			if (node.getSemanticModel() instanceof SimpleSemanticModelElement) {
				// Manage the Element that are not associated with a model object
			}
		}
		else if (model instanceof GraphEdge) {
			final GraphEdge edge = (GraphEdge) model;
			EObject element = Utils.getElement(edge);
			if (element != null) {
				if ("http://www.irisa.fr/espresso/sme.ecore".equals(element.eClass().getEPackage().getNsURI())) {
					return (EditPart) new EdgeSmeSwitch(edge).doSwitch(element);
				}
			}
			
			if (edge.getSemanticModel() instanceof SimpleSemanticModelElement) {
				// Manage the Element that are not associated with a model object
			}
		}
		return super.createEditPart(context, model);
	}
	
	/**
	 * @generated
	 */
	private class NodeSmeSwitch extends SmeSwitch<Object> {
		/**
		 * The graphical node
		 * 
		 * @generated
		 */
		private GraphNode	node;
		
		/**
		 * Constructor
		 * 
		 * @param node
		 *        the graphical node
		 * @generated
		 */
		public NodeSmeSwitch(GraphNode node) {
			this.node = node;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseClockedExpr(fr.irisa.espresso.sme.ClockedExpr)
		 * @generated
		 */
		public Object caseClockedExpr(fr.irisa.espresso.sme.ClockedExpr object) {
			return new ClockedExprEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseDependenceInOut(fr.irisa.espresso.sme.DependenceInOut)
		 * @generated
		 */
		public Object caseDependenceInOut(fr.irisa.espresso.sme.DependenceInOut object) {
			return new DependenceInOutEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSignalOrRef(fr.irisa.espresso.sme.SignalOrRef)
		 * @generated
		 */
		public Object caseSignalOrRef(fr.irisa.espresso.sme.SignalOrRef object) {
			return new SignalOrRefEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSignalRef(fr.irisa.espresso.sme.SignalRef)
		 * @generated
		 */
		public Object caseSignalRef(fr.irisa.espresso.sme.SignalRef object) {
			return new SignalRefEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInput(fr.irisa.espresso.sme.Input)
		 * @generated
		 */
		public Object caseInput(fr.irisa.espresso.sme.Input object) {
			String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
			if (!"".equals(feature)) {
				int featureID = Integer.parseInt(feature);
				return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
			}
			else {
				return new InputEditPart(node);
			}
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOutput(fr.irisa.espresso.sme.Output)
		 * @generated
		 */
		public Object caseOutput(fr.irisa.espresso.sme.Output object) {
			String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
			if (!"".equals(feature)) {
				int featureID = Integer.parseInt(feature);
				return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
			}
			else {
				return new OutputEditPart(node);
			}
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseLocal(fr.irisa.espresso.sme.Local)
		 * @generated
		 */
		public Object caseLocal(fr.irisa.espresso.sme.Local object) {
			String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
			if (!"".equals(feature)) {
				int featureID = Integer.parseInt(feature);
				return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
			}
			else {
				return new LocalEditPart(node);
			}
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInputInstance(fr.irisa.espresso.sme.InputInstance)
		 * @generated
		 */
		public Object caseInputInstance(fr.irisa.espresso.sme.InputInstance object) {
			return new InputInstanceEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOutputInstance(fr.irisa.espresso.sme.OutputInstance)
		 * @generated
		 */
		public Object caseOutputInstance(fr.irisa.espresso.sme.OutputInstance object) {
			return new OutputInstanceEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseBasicIterate(fr.irisa.espresso.sme.BasicIterate)
		 * @generated
		 */
		public Object caseBasicIterate(fr.irisa.espresso.sme.BasicIterate object) {
			return new BasicIterateEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIterate(fr.irisa.espresso.sme.Iterate)
		 * @generated
		 */
		public Object caseIterate(fr.irisa.espresso.sme.Iterate object) {
			String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
			if (!"".equals(feature)) {
				int featureID = Integer.parseInt(feature);
				return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
			}
			else {
				return new IterateEditPart(node);
			}
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSubProcess(fr.irisa.espresso.sme.SubProcess)
		 * @generated
		 */
		public Object caseSubProcess(fr.irisa.espresso.sme.SubProcess object) {
			String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
			if (!"".equals(feature)) {
				int featureID = Integer.parseInt(feature);
				return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
			}
			else {
				return new SubProcessEditPart(node);
			}
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAutomaton(fr.irisa.espresso.sme.Automaton)
		 * @generated
		 */
		public Object caseAutomaton(fr.irisa.espresso.sme.Automaton object) {
			return new AutomatonEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelInstance(fr.irisa.espresso.sme.ModelInstance)
		 * @generated
		 */
		public Object caseModelInstance(fr.irisa.espresso.sme.ModelInstance object) {
			return new ModelInstanceEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseClockConstraintOperator(fr.irisa.espresso.sme.ClockConstraintOperator)
		 * @generated
		 */
		public Object caseClockConstraintOperator(fr.irisa.espresso.sme.ClockConstraintOperator object) {
			return new ClockConstraintOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAssertionClockConstraintOperator(fr.irisa.espresso.sme.AssertionClockConstraintOperator)
		 * @generated
		 */
		public Object caseAssertionClockConstraintOperator(fr.irisa.espresso.sme.AssertionClockConstraintOperator object) {
			return new AssertionClockConstraintOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseClockRelationOperator(fr.irisa.espresso.sme.ClockRelationOperator)
		 * @generated
		 */
		public Object caseClockRelationOperator(fr.irisa.espresso.sme.ClockRelationOperator object) {
			return new ClockRelationOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseClockSpeedOperator(fr.irisa.espresso.sme.ClockSpeedOperator)
		 * @generated
		 */
		public Object caseClockSpeedOperator(fr.irisa.espresso.sme.ClockSpeedOperator object) {
			return new ClockSpeedOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAssertionClockSpeedOperator(fr.irisa.espresso.sme.AssertionClockSpeedOperator)
		 * @generated
		 */
		public Object caseAssertionClockSpeedOperator(fr.irisa.espresso.sme.AssertionClockSpeedOperator object) {
			return new AssertionClockSpeedOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseDependenceOperator(fr.irisa.espresso.sme.DependenceOperator)
		 * @generated
		 */
		public Object caseDependenceOperator(fr.irisa.espresso.sme.DependenceOperator object) {
			return new DependenceOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseNullClock(fr.irisa.espresso.sme.NullClock)
		 * @generated
		 */
		public Object caseNullClock(fr.irisa.espresso.sme.NullClock object) {
			return new NullClockEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInputPort(fr.irisa.espresso.sme.InputPort)
		 * @generated
		 */
		public Object caseInputPort(fr.irisa.espresso.sme.InputPort object) {
			return new InputPortEditPart(node);
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object) {
			return new EMFGraphNodeEditPart(node);
		}
	}
	
	/**
	 * @generated
	 */
	private class EdgeSmeSwitch extends SmeSwitch<Object> {
		/**
		 * The graphical edge
		 * 
		 * @generated
		 */
		private GraphEdge	edge;
		
		/**
		 * Constructor
		 * 
		 * @param edge
		 *        the graphical edge
		 * @generated
		 */
		public EdgeSmeSwitch(GraphEdge edge) {
			this.edge = edge;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseClockRelation(fr.irisa.espresso.sme.ClockRelation)
		 * @generated
		 */
		public Object caseClockRelation(fr.irisa.espresso.sme.ClockRelation object) {
			return new ClockRelationEditPart(edge);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCondition(fr.irisa.espresso.sme.Condition)
		 * @generated
		 */
		public Object caseCondition(fr.irisa.espresso.sme.Condition object) {
			return new ConditionEditPart(edge);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseDependence(fr.irisa.espresso.sme.Dependence)
		 * @generated
		 */
		public Object caseDependence(fr.irisa.espresso.sme.Dependence object) {
			return new DependenceEditPart(edge);
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object) {
			return new EMFGraphEdgeEditPart(edge);
		}
	}
	
}
