/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteStack;
import org.eclipse.gef.requests.CreationFactory;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

import fr.irisa.espresso.sme.AssertionClockConstraintOperator;
import fr.irisa.espresso.sme.AssertionClockSpeedOperator;
import fr.irisa.espresso.sme.ClockConstraintOpKind;
import fr.irisa.espresso.sme.ClockConstraintOperator;
import fr.irisa.espresso.sme.ClockRelationOpKind;
import fr.irisa.espresso.sme.ClockRelationOperator;
import fr.irisa.espresso.sme.ClockSpeedOpKind;
import fr.irisa.espresso.sme.ClockSpeedOperator;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.common.SmeImageRegistry;

/*
 * Generated Palette Manager
 * 
 * @generated
 */
public class CRD_PaletteManager extends ModelerPaletteManager {
	// declare all the palette categories of the diagram
	/**
	 * @generated
	 */
	private PaletteDrawer	identifiersDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	clocksDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	dependencesDrawer;
	
	/**
	 * @generated
	 */
	private ICreationUtils	creationUtils;
	
	/**
	 * The Constructor
	 * 
	 * @param utils
	 *        the creation utils for the tools of the palette
	 * @generated
	 */
	public CRD_PaletteManager(ICreationUtils utils) {
		super();
		this.creationUtils = utils;
	}
	
	/**
	 * Creates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void createCategories() {
		createIdentifiersDrawer();
		createClocksDrawer();
		createDependencesDrawer();
	}
	
	/**
	 * Updates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void updateCategories() {
		// deletion of the existing categories and creation of the updated categories
		
		getRoot().remove(identifiersDrawer);
		createIdentifiersDrawer();
		
		getRoot().remove(clocksDrawer);
		createClocksDrawer();
		
		getRoot().remove(dependencesDrawer);
		createDependencesDrawer();
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createIdentifiersDrawer() {
		identifiersDrawer = new PaletteDrawer("Identifiers", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getLocal(), "default");
		entries.add(new ModelerCreationToolEntry("Local signal", "Local signal", factory, SmeImageRegistry
				.getImageDescriptor("LOCAL"), SmeImageRegistry.getImageDescriptor("LOCAL_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getSignalRef(), "default");
		entries.add(new ModelerCreationToolEntry("Reference to a signal", "Reference to a signal", factory,
			SmeImageRegistry.getImageDescriptor("SIGNALREF"), SmeImageRegistry.getImageDescriptor("SIGNALREF_LARGE")));
		
		identifiersDrawer.addAll(entries);
		getRoot().add(identifiersDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createClocksDrawer() {
		clocksDrawer = new PaletteDrawer("Clocks", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		ModelerCreationToolEntry objectTool;
		PaletteStack stack;
		
		stack = new PaletteStack("Clock Constraint Operator", "Clock Constraint Operator Selection", SmeImageRegistry
				.getImageDescriptor("CLOCKCONSTRAINTOPERATOR"));
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockConstraintOperator()) {
			public EObject getNewModelObject() {
				ClockConstraintOperator cco = (ClockConstraintOperator) super.getNewModelObject();
				cco.setOperator(ClockConstraintOpKind.SYNCHRONIZED_LITERAL);
				return cco;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Synchronization", "Clock Synchronization", factory,
			SmeImageRegistry.getImageDescriptor("CLOCKSYNCHRONIZATION"), SmeImageRegistry
					.getImageDescriptor("CLOCKSYNCHRONIZATION_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockConstraintOperator()) {
			public EObject getNewModelObject() {
				ClockConstraintOperator cco = (ClockConstraintOperator) super.getNewModelObject();
				cco.setOperator(ClockConstraintOpKind.EXCLUDED_LITERAL);
				return cco;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Exclusion", "Clock Exclusion", factory, SmeImageRegistry
				.getImageDescriptor("CLOCKEXCLUSION"), SmeImageRegistry.getImageDescriptor("CLOCKEXCLUSION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockConstraintOperator()) {
			public EObject getNewModelObject() {
				ClockConstraintOperator cco = (ClockConstraintOperator) super.getNewModelObject();
				cco.setOperator(ClockConstraintOpKind.IDENTITY_LITERAL);
				return cco;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Identity", "Clock Identity", factory, SmeImageRegistry
				.getImageDescriptor("CLOCKIDENTITY"), SmeImageRegistry.getImageDescriptor("CLOCKIDENTITY_LARGE"));
		stack.add(objectTool);
		entries.add(stack);
		
		stack = new PaletteStack("Clock Constraint Assertion", "Clock Constraint Assertion Selection", SmeImageRegistry
				.getImageDescriptor("ASSERTIONCLOCKCONSTRAINTOPERATOR"));
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE
				.getAssertionClockConstraintOperator()) {
			public EObject getNewModelObject() {
				AssertionClockConstraintOperator cco = (AssertionClockConstraintOperator) super.getNewModelObject();
				cco.setOperator(ClockConstraintOpKind.SYNCHRONIZED_LITERAL);
				return cco;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Synchronization Assertion", "Clock Synchronization Assertion",
			factory, SmeImageRegistry.getImageDescriptor("ASSERTIONCLOCKSYNCHRONIZATION"), SmeImageRegistry
					.getImageDescriptor("ASSERTIONCLOCKSYNCHRONIZATION_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE
				.getAssertionClockConstraintOperator()) {
			public EObject getNewModelObject() {
				AssertionClockConstraintOperator cco = (AssertionClockConstraintOperator) super.getNewModelObject();
				cco.setOperator(ClockConstraintOpKind.EXCLUDED_LITERAL);
				return cco;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Exclusion Assertion", "Clock Exclusion Assertion", factory,
			SmeImageRegistry.getImageDescriptor("ASSERTIONCLOCKEXCLUSION"), SmeImageRegistry
					.getImageDescriptor("ASSERTIONCLOCKEXCLUSION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE
				.getAssertionClockConstraintOperator()) {
			public EObject getNewModelObject() {
				AssertionClockConstraintOperator cco = (AssertionClockConstraintOperator) super.getNewModelObject();
				cco.setOperator(ClockConstraintOpKind.IDENTITY_LITERAL);
				return cco;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Identity Assertion", "Clock Identity Assertion", factory,
			SmeImageRegistry.getImageDescriptor("ASSERTIONCLOCKIDENTITY"), SmeImageRegistry
					.getImageDescriptor("ASSERTIONCLOCKIDENTITY_LARGE"));
		stack.add(objectTool);
		entries.add(stack);
		
		stack = new PaletteStack("Clock Relation Operator", "Clock Relation Operator Selection", SmeImageRegistry
				.getImageDescriptor("CLOCKRELATIONOPERATOR"));
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockRelationOperator()) {
			public EObject getNewModelObject() {
				ClockRelationOperator cro = (ClockRelationOperator) super.getNewModelObject();
				cro.setOperator(ClockRelationOpKind.UNION_LITERAL);
				return cro;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Union", "Clock Union", factory, SmeImageRegistry
				.getImageDescriptor("CLOCKUNION"), SmeImageRegistry.getImageDescriptor("CLOCKUNION_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockRelationOperator()) {
			public EObject getNewModelObject() {
				ClockRelationOperator cro = (ClockRelationOperator) super.getNewModelObject();
				cro.setOperator(ClockRelationOpKind.INTERSECTION_LITERAL);
				return cro;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Intersection", "Clock Intersection", factory, SmeImageRegistry
				.getImageDescriptor("CLOCKINTERSECTION"), SmeImageRegistry
				.getImageDescriptor("CLOCKINTERSECTION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockRelationOperator()) {
			public EObject getNewModelObject() {
				ClockRelationOperator cro = (ClockRelationOperator) super.getNewModelObject();
				cro.setOperator(ClockRelationOpKind.COMPLEMENTARY_LITERAL);
				return cro;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Complementary", "Clock Complementary", factory,
			SmeImageRegistry.getImageDescriptor("CLOCKCOMPLEMENTARY"), SmeImageRegistry
					.getImageDescriptor("CLOCKCOMPLEMENTARY_LARGE"));
		stack.add(objectTool);
		entries.add(stack);
		
		stack = new PaletteStack("Clock Speed Operator", "Clock Speed Operator Selection", SmeImageRegistry
				.getImageDescriptor("CLOCKSPEEDOPERATOR"));
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockSpeedOperator()) {
			public EObject getNewModelObject() {
				ClockSpeedOperator cso = (ClockSpeedOperator) super.getNewModelObject();
				cso.setOperator(ClockSpeedOpKind.SLOWER_LITERAL);
				return cso;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Slower", "Clock Slower", factory, SmeImageRegistry
				.getImageDescriptor("CLOCKSLOWER"), SmeImageRegistry.getImageDescriptor("CLOCKSLOWER_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockSpeedOperator()) {
			public EObject getNewModelObject() {
				ClockSpeedOperator cso = (ClockSpeedOperator) super.getNewModelObject();
				cso.setOperator(ClockSpeedOpKind.FASTER_LITERAL);
				return cso;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Faster", "Clock Faster", factory, SmeImageRegistry
				.getImageDescriptor("CLOCKFASTER"), SmeImageRegistry.getImageDescriptor("CLOCKFASTER_LARGE"));
		stack.add(objectTool);
		entries.add(stack);
		
		stack = new PaletteStack("Clock Speed Assertion", "Clock Speed Assertion Selection", SmeImageRegistry
				.getImageDescriptor("ASSERTIONCLOCKSPEEDOPERATOR"));
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getAssertionClockSpeedOperator()) {
			public EObject getNewModelObject() {
				AssertionClockSpeedOperator cso = (AssertionClockSpeedOperator) super.getNewModelObject();
				cso.setOperator(ClockSpeedOpKind.SLOWER_LITERAL);
				return cso;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Slower Assertion", "Clock Slower Assertion", factory,
			SmeImageRegistry.getImageDescriptor("ASSERTIONCLOCKSLOWER"), SmeImageRegistry
					.getImageDescriptor("ASSERTIONCLOCKSLOWER_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getAssertionClockSpeedOperator()) {
			public EObject getNewModelObject() {
				AssertionClockSpeedOperator cso = (AssertionClockSpeedOperator) super.getNewModelObject();
				cso.setOperator(ClockSpeedOpKind.FASTER_LITERAL);
				return cso;
			}
		};
		objectTool = new ModelerCreationToolEntry("Clock Faster Assertion", "Clock Faster Assertion", factory,
			SmeImageRegistry.getImageDescriptor("ASSERTIONCLOCKFASTER"), SmeImageRegistry
					.getImageDescriptor("ASSERTIONCLOCKFASTER_LARGE"));
		stack.add(objectTool);
		entries.add(stack);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getNullClock(), "default");
		entries.add(new ModelerCreationToolEntry("Null Clock", "Null Clock", factory, SmeImageRegistry
				.getImageDescriptor("NULLCLOCK"), SmeImageRegistry.getImageDescriptor("NULLCLOCK_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getClockRelation(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("Relation", "Relation", factory, SmeImageRegistry
				.getImageDescriptor("CLOCKRELATION"), SmeImageRegistry.getImageDescriptor("CLOCKRELATION_LARGE")));
		
		clocksDrawer.addAll(entries);
		getRoot().add(clocksDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createDependencesDrawer() {
		dependencesDrawer = new PaletteDrawer("Dependences", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getDependenceOperator(),
			"default");
		entries.add(new ModelerCreationToolEntry("Dependence operator", "Dependence operator", factory,
			SmeImageRegistry.getImageDescriptor("DEPENDENCEOPERATOR"), SmeImageRegistry
					.getImageDescriptor("DEPENDENCEOPERATOR_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getDependence(), "default");
		entries
				.add(new ModelerConnectionCreationToolEntry("Dependence relation", "Dependence relation", factory,
					SmeImageRegistry.getImageDescriptor("DEPENDENCE"), SmeImageRegistry
							.getImageDescriptor("DEPENDENCE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getCondition(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("Condition link", "Condition link", factory,
			SmeImageRegistry.getImageDescriptor("CONDITION"), SmeImageRegistry.getImageDescriptor("CONDITION_LARGE")));
		
		dependencesDrawer.addAll(entries);
		getRoot().add(dependencesDrawer);
	}
	
}
