/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.commands.AbstractRestoreConnectionCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.AssertionClockConstraintOperator;
import fr.irisa.espresso.sme.ClockConstraintOperator;
import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ClockedExpr;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.ModelConnection;

/*
 * ClockedExpr restore connection command
 * 
 * @generated
 */
public class ClockedExprRestoreConnectionCommand extends AbstractRestoreConnectionCommand {
	/**
	 * @param part
	 *        the EditPart that is restored
	 * @generated
	 */
	public ClockedExprRestoreConnectionCommand(EditPart part) {
		super(part);
	}
	
	/**
	 * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
	 * @generated NOT
	 */
	protected void initializeCommands() {
		
		GraphElement elt = getGraphElement();
		EObject eltObject = Utils.getElement(elt);
		
		if (eltObject instanceof ClockedExpr) {
			Iterator<EObject> itDiagContents = getModeler().getActiveDiagram().eAllContents();
			while (itDiagContents.hasNext()) {
				Object obj = itDiagContents.next();
				if (obj instanceof GraphElement
						&& DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null) {
					boolean autoRef = obj.equals(elt);
					GraphElement elt2 = (GraphElement) obj;
					EObject eltObject2 = Utils.getElement(elt2);
					
					if (eltObject2 instanceof ClockedExpr) {
						if (!autoRef) {
							// if the elt is the source of the edge or if it is the target and that the
							// SourceTargetCouple is reversible
							createClockRelationFromClockedExprToClockedExpr_Dst_ClockRelations(elt, elt2);
							// if elt is the target of the edge or if it is the source and that the SourceTargetCouple
							// is reversible
							createClockRelationFromClockedExprToClockedExpr_Dst_ClockRelations(elt2, elt);
						}
					}
					
					if (eltObject2 instanceof InputPort) {
						if (!autoRef) {
							// if the elt is the source of the edge or if it is the target and that the
							// SourceTargetCouple is reversible
							createClockRelationFromClockedExprToInputPort_Dst_ClockRelations(elt, elt2);
						}
					}
					
					if (eltObject2 instanceof ClockConstraintOperator) {
						if (!autoRef) {
							// if the elt is the source of the edge or if it is the target and that the
							// SourceTargetCouple is reversible
							createClockRelationFromClockedExprToClockConstraintOperator_Dst_ClockRelations(elt, elt2);
						}
					}
					
					if (eltObject2 instanceof AssertionClockConstraintOperator) {
						if (!autoRef) {
							// if the elt is the source of the edge or if it is the target and that the
							// SourceTargetCouple is reversible
							createClockRelationFromClockedExprToAssertionClockConstraintOperator_Dst_ClockRelations(
								elt, elt2);
						}
					}
				}
			}
			
			// Add the restore connection commands of all inner objects
			// construct the RestoreConnectionRequest
			RestoreConnectionsRequest request = new RestoreConnectionsRequest();
			
			for (Object obj : getRestoredPart().getChildren()) {
				if (obj instanceof EditPart) {
					// get the Command
					Command command = ((EditPart) obj).getCommand(request);
					if (command != null && command.canExecute())
						add(command);
				}
			}
			
		}
	}
	
	/**
	 * @param srcElt
	 *        the source element
	 * @param targetElt
	 *        the target element
	 * @generated NOT
	 */
	private void createClockRelationFromClockedExprToClockedExpr_Dst_ClockRelations(GraphElement srcElt,
			GraphElement targetElt) {
		ClockedExpr sourceObject = (ClockedExpr) Utils.getElement(srcElt);
		ClockedExpr targetObject = (ClockedExpr) Utils.getElement(targetElt);
		
		EList<ModelConnection> edgeObjectList = ((fr.irisa.espresso.sme.Model) Utils.getDiagramModelObject(srcElt))
				.getConnections();
		for (ModelConnection obj : edgeObjectList) {
			if (obj instanceof ClockRelation) {
				ClockRelation edgeObject = (ClockRelation) obj;
				if (targetObject.equals(edgeObject.getDst()) && sourceObject.equals(edgeObject.getSrc())
						&& sourceObject.getSrc_ClockRelations().contains(edgeObject)
						&& targetObject.getDst_ClockRelations().contains(edgeObject)) {
					// check if the relation does not exists yet
					List<GraphEdge> existing = getExistingEdges(srcElt, targetElt, ClockRelation.class);
					if (!isAlreadyPresent(existing, edgeObject)) {
						ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
						// restore the link with its default presentation
						GraphElement edge = factory.createGraphElement(edgeObject);
						if (edge instanceof GraphEdge) {
							ClockRelationEdgeCreationCommand cmd = new ClockRelationEdgeCreationCommand(
								getEditDomain(), (GraphEdge) edge, srcElt, false);
							cmd.setTarget(targetElt);
							add(cmd);
						}
					}
				}
			}
		}
	}
	/**
	 * @param srcElt
	 *        the source element
	 * @param targetElt
	 *        the target element
	 * @generated NOT
	 */
	private void createClockRelationFromClockedExprToInputPort_Dst_ClockRelations(GraphElement srcElt,
			GraphElement targetElt) {
		ClockedExpr sourceObject = (ClockedExpr) Utils.getElement(srcElt);
		InputPort targetObject = (InputPort) Utils.getElement(targetElt);
		
		EList<ModelConnection> edgeObjectList = ((fr.irisa.espresso.sme.Model) Utils.getDiagramModelObject(srcElt))
				.getConnections();
		for (ModelConnection obj : edgeObjectList) {
			if (obj instanceof ClockRelation) {
				ClockRelation edgeObject = (ClockRelation) obj;
				if (targetObject.equals(edgeObject.getDst()) && sourceObject.equals(edgeObject.getSrc())
						&& sourceObject.getSrc_ClockRelations().contains(edgeObject)
						&& targetObject.getDst_ClockRelations().contains(edgeObject)) {
					// check if the relation does not exists yet
					List<GraphEdge> existing = getExistingEdges(srcElt, targetElt, ClockRelation.class);
					if (!isAlreadyPresent(existing, edgeObject)) {
						ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
						// restore the link with its default presentation
						GraphElement edge = factory.createGraphElement(edgeObject);
						if (edge instanceof GraphEdge) {
							ClockRelationEdgeCreationCommand cmd = new ClockRelationEdgeCreationCommand(
								getEditDomain(), (GraphEdge) edge, srcElt, false);
							cmd.setTarget(targetElt);
							add(cmd);
						}
					}
				}
			}
		}
	}
	/**
	 * @param srcElt
	 *        the source element
	 * @param targetElt
	 *        the target element
	 * @generated NOT
	 */
	private void createClockRelationFromClockedExprToClockConstraintOperator_Dst_ClockRelations(GraphElement srcElt,
			GraphElement targetElt) {
		ClockedExpr sourceObject = (ClockedExpr) Utils.getElement(srcElt);
		ClockConstraintOperator targetObject = (ClockConstraintOperator) Utils.getElement(targetElt);
		
		EList<ModelConnection> edgeObjectList = ((fr.irisa.espresso.sme.Model) Utils.getDiagramModelObject(srcElt))
				.getConnections();
		for (ModelConnection obj : edgeObjectList) {
			if (obj instanceof ClockRelation) {
				ClockRelation edgeObject = (ClockRelation) obj;
				if (targetObject.equals(edgeObject.getDst()) && sourceObject.equals(edgeObject.getSrc())
						&& sourceObject.getSrc_ClockRelations().contains(edgeObject)
						&& targetObject.getDst_ClockRelations().contains(edgeObject)) {
					// check if the relation does not exists yet
					List<GraphEdge> existing = getExistingEdges(srcElt, targetElt, ClockRelation.class);
					if (!isAlreadyPresent(existing, edgeObject)) {
						ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
						// restore the link with its default presentation
						GraphElement edge = factory.createGraphElement(edgeObject);
						if (edge instanceof GraphEdge) {
							ClockRelationEdgeCreationCommand cmd = new ClockRelationEdgeCreationCommand(
								getEditDomain(), (GraphEdge) edge, srcElt, false);
							cmd.setTarget(targetElt);
							add(cmd);
						}
					}
				}
			}
		}
	}
	
	/**
	 * @param srcElt
	 *        the source element
	 * @param targetElt
	 *        the target element
	 * @generated NOT
	 */
	private void createClockRelationFromClockedExprToAssertionClockConstraintOperator_Dst_ClockRelations(
			GraphElement srcElt, GraphElement targetElt) {
		ClockedExpr sourceObject = (ClockedExpr) Utils.getElement(srcElt);
		AssertionClockConstraintOperator targetObject = (AssertionClockConstraintOperator) Utils.getElement(targetElt);
		
		EList<ModelConnection> edgeObjectList = ((fr.irisa.espresso.sme.Model) Utils.getDiagramModelObject(srcElt))
				.getConnections();
		for (ModelConnection obj : edgeObjectList) {
			if (obj instanceof ClockRelation) {
				ClockRelation edgeObject = (ClockRelation) obj;
				if (targetObject.equals(edgeObject.getDst()) && sourceObject.equals(edgeObject.getSrc())
						&& sourceObject.getSrc_ClockRelations().contains(edgeObject)
						&& targetObject.getDst_ClockRelations().contains(edgeObject)) {
					// check if the relation does not exists yet
					List<GraphEdge> existing = getExistingEdges(srcElt, targetElt, ClockRelation.class);
					if (!isAlreadyPresent(existing, edgeObject)) {
						ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
						// restore the link with its default presentation
						GraphElement edge = factory.createGraphElement(edgeObject);
						if (edge instanceof GraphEdge) {
							ClockRelationEdgeCreationCommand cmd = new ClockRelationEdgeCreationCommand(
								getEditDomain(), (GraphEdge) edge, srcElt, false);
							cmd.setTarget(targetElt);
							add(cmd);
						}
					}
				}
			}
		}
	}
	
}
