/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.ClockSpeedOpKind;
import fr.irisa.espresso.sme.ClockSpeedOperator;
import fr.irisa.espresso.sme.common.figures.SignalOperatorFigure;
import fr.irisa.espresso.sme.common.policies.MultipleInputsLayoutEditPolicy;
import fr.irisa.espresso.sme.diagram.clockrelations.figures.ClockSpeedOperatorFigure;
import fr.irisa.espresso.sme.diagram.clockrelations.preferences.CRD_DiagramPreferenceConstants;

/*
 * The ClockSpeedOperator object controller
 * 
 * @generated
 */
public class ClockSpeedOperatorEditPart extends EMFGraphNodeEditPart {
	/**
	 * Constructor
	 * 
	 * @param obj
	 *        the graph node
	 * @generated
	 */
	public ClockSpeedOperatorEditPart(GraphNode obj) {
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
		resizableEditPolicy.setResizeDirections(PositionConstants.NORTH_SOUTH);
		installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);
		
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new MultipleInputsLayoutEditPolicy());
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated
	 */
	protected IFigure createFigure() {
		
		return new ClockSpeedOperatorFigure();
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultBackgroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultBackgroundColor() {
		String backgroundColor = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.CLOCKSPEEDOPERATOR_DEFAULT_BACKGROUND_COLOR);
		if (backgroundColor.length() != 0) {
			return Utils.getColor(backgroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultForegroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String foregroundColor = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.CLOCKSPEEDOPERATOR_DEFAULT_FOREGROUND_COLOR);
		if (foregroundColor.length() != 0) {
			return Utils.getColor(foregroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultFont()
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.CLOCKSPEEDOPERATOR_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
	 * @generated NOT
	 */
	protected void refreshVisuals() {
		super.refreshVisuals();
		
		updateOperator();
		updatePorts();
	}
	
	/**
	 * Update the Clock Constraint operator kind
	 * 
	 * @generated NOT
	 */
	protected void updateOperator() {
		ClockSpeedOpKind op = ((ClockSpeedOperator) getEObject()).getOperator();
		
		ClockSpeedOperatorFigure fig = (ClockSpeedOperatorFigure) getFigure();
		if (fig.getClockSpeedOperator() != op) {
			fig.setClockSpeedOperator(op);
			fig.repaint();
		}
	}
	
	/**
	 * Update the number of InputPorts
	 * 
	 * @generated NOT
	 */
	private void updatePorts() {
		int nbIns = ((ClockSpeedOperator) getEObject()).getInputPorts().size();
		
		SignalOperatorFigure fig = (SignalOperatorFigure) getFigure();
		if (fig.getNbInputPorts() != nbIns) {
			fig.setNbInputPorts(nbIns);
			getGraphNode().setSize(fig.getPreferredSize());
			fig.repaint();
		}
		
		if (nbIns > 0) {
			// place the ports
			int height;
			if (getGraphNode().getSize() == null)
				height = fig.getPreferredSize().height;
			else
				height = getGraphNode().getSize().height;
			
			int nIn = 0;
			int space = height / nbIns;
			int init = (space - SignalOperatorFigure.PORT_SIZE.height) / 2;
			
			for (Object obj : getModelChildren()) {
				GraphElement elt = (GraphElement) obj;
				elt.setPosition(new Point(0, init + (nIn * space)));
				nIn++;
			}
		}
	}
}
