/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerColorConstants;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.diagram.clockrelations.CRD_EdgeObjectConstants;
import fr.irisa.espresso.sme.diagram.clockrelations.figures.DependenceFigure;
import fr.irisa.espresso.sme.diagram.clockrelations.policies.DependenceEdgeObjectUVEditPolicy;
import fr.irisa.espresso.sme.diagram.clockrelations.preferences.CRD_DiagramPreferenceConstants;

/*
 * Dependence controller
 * 
 * @generated
 */
public class DependenceEditPart extends EMFGraphEdgeEditPart {
	
	/**
	 * Constructor
	 * 
	 * @param model
	 *        the graph object
	 * @generated
	 */
	public DependenceEditPart(GraphEdge model) {
		super(model);
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 * @generated
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY, new DependenceEdgeObjectUVEditPolicy());
		
	}
	
	/**
	 * @return the Figure
	 * @generated
	 */
	protected IFigure createFigure() {
		DependenceFigure connection = new DependenceFigure();
		
		createTargetDecoration(connection);
		
		return connection;
	}
	
	/**
	 * @param connection
	 *        the PolylineConnection
	 * @generated
	 */
	private void createTargetDecoration(PolylineConnection connection) {
		
		PolygonDecoration decoration = new PolygonDecoration();
		decoration.setScale(14, 6);
		decoration.setBackgroundColor(ModelerColorConstants.white);
		connection.setTargetDecoration(decoration);
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
	 * @generated
	 */
	public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject) {
		if (CRD_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID.equals(edgeObject.getId())) {
			return ((DependenceFigure) getFigure()).getSrcFieldEdgeObjectFigure();
		}
		if (CRD_EdgeObjectConstants.DSTFIELD_EDGE_OBJECT_ID.equals(edgeObject.getId())) {
			return ((DependenceFigure) getFigure()).getDstFieldEdgeObjectFigure();
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
	 * @generated
	 */
	protected void refreshEdgeObjects() {
		super.refreshEdgeObjects();
		updateSrcFieldLabel();
		updateDstFieldLabel();
	}
	
	/**
	 * Update the srcField Label
	 * 
	 * @generated
	 */
	private void updateSrcFieldLabel() {
		((Label) ((DependenceFigure) getFigure()).getSrcFieldEdgeObjectFigure()).setText(((Dependence) getEObject())
				.getSrcField());
	}
	/**
	 * Update the dstField Label
	 * 
	 * @generated
	 */
	private void updateDstFieldLabel() {
		((Label) ((DependenceFigure) getFigure()).getDstFieldEdgeObjectFigure()).setText(((Dependence) getEObject())
				.getDstField());
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultRouter()
	 * 
	 * @generated
	 */
	protected String getPreferenceDefaultRouter() {
		return getPreferenceStore().getString(CRD_DiagramPreferenceConstants.DEPENDENCE_EDGE_DEFAULT_ROUTER);
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultForegroundColor()
	 * 
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String preferenceForeground = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.DEPENDENCE_EDGE_DEFAULT_FOREGROUND_COLOR);
		if (preferenceForeground.length() != 0) {
			return Utils.getColor(preferenceForeground);
		}
		return null;
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultFont()
	 * 
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.DEPENDENCE_EDGE_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
	}
}
