/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.draw2d.figures.OrientedPortFigure;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.NonResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.common.policies.DeletePortEditPolicy;
import fr.irisa.espresso.sme.diagram.clockrelations.CRD_EditPolicyConstants;
import fr.irisa.espresso.sme.diagram.clockrelations.commands.InputPortRestoreConnectionCommand;
import fr.irisa.espresso.sme.diagram.clockrelations.policies.ClockRelationEdgeCreationEditPolicy;
import fr.irisa.espresso.sme.diagram.clockrelations.preferences.CRD_DiagramPreferenceConstants;

/*
 * The InputPort object controller
 * 
 * @generated
 */
public class InputPortEditPart extends EMFGraphNodeEditPart {
	/**
	 * Constructor
	 * 
	 * @param obj
	 *        the graph node
	 * @generated
	 */
	public InputPortEditPart(GraphNode obj) {
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(CRD_EditPolicyConstants.CLOCKRELATION_EDITPOLICY, new ClockRelationEdgeCreationEditPolicy());
		
		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy() {
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request) {
				return new InputPortRestoreConnectionCommand(getHost());
			}
		});
		
		installEditPolicy(ModelerEditPolicyConstants.DELETE_MODEL_OBJECT_EDITPOLICY, new DeletePortEditPolicy());
		
		/** To avoid the port to be moved * */
		NonResizableEditPolicy policy = new NonResizableEditPolicy();
		policy.setDragAllowed(false);
		installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, policy);
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated NOT
	 */
	protected IFigure createFigure() {
		OrientedPortFigure fig = new OrientedPortFigure();
		fig.setPosition(PositionConstants.LEFT);
		InputPort in = (InputPort) Utils.getElement((GraphElement) this.getModel());
		Label lbl = new Label(in.getName());
		lbl.setParent(fig);
		fig.setToolTip(lbl);
		return fig;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultBackgroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultBackgroundColor() {
		String backgroundColor = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.INPUTPORT_DEFAULT_BACKGROUND_COLOR);
		if (backgroundColor.length() != 0) {
			return Utils.getColor(backgroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultForegroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String foregroundColor = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.INPUTPORT_DEFAULT_FOREGROUND_COLOR);
		if (foregroundColor.length() != 0) {
			return Utils.getColor(foregroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultFont()
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(CRD_DiagramPreferenceConstants.INPUTPORT_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
		
	}
	
	/**
	 * Activate the adapters
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 * @generated NOT
	 */
	public void activate() {
		super.activate();
		
		if (((InputPort) getEObject()).getDst_ClockRelations().size() == 1) {
			ClockRelation clkRel = (ClockRelation) ((InputPort) getEObject()).getDst_ClockRelations().get(0);
			clkRel.eAdapters().add(getModelListener());
		}
	}
	
	/**
	 * Deactivate the adapters
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 * @generated NOT
	 */
	public void deactivate() {
		if (((InputPort) getEObject()).getDst_ClockRelations().size() == 1) {
			ClockRelation clkRel = (ClockRelation) ((InputPort) getEObject()).getDst_ClockRelations().get(0);
			clkRel.eAdapters().remove(getModelListener());
		}
		super.deactivate();
	}
	
	/**
	 * Update model and graphics
	 * 
	 * @param msg
	 *        the Notification
	 */
	protected void handleModelChanged(Notification msg) {
		super.handleModelChanged(msg);
		
		Object newObject = msg.getNewValue();
		Object oldObject = msg.getOldValue();
		
		if (msg.getNotifier() instanceof InputPort) {
			switch (msg.getFeatureID(InputPort.class)) {
				case SmePackage.INPUT_PORT__DST_CLOCK_RELATIONS:
					if (newObject != null && oldObject == null && newObject instanceof ClockRelation) {
						ClockRelation cr = (ClockRelation) newObject;
						if (!cr.eAdapters().contains(getModelListener()))
							cr.eAdapters().add(getModelListener());
					}
					if (newObject == null && oldObject != null && oldObject instanceof ClockRelation) {
						ClockRelation cr = (ClockRelation) oldObject;
						if (cr.eAdapters().contains(getModelListener()))
							cr.eAdapters().remove(getModelListener());
					}
					refreshVisuals();
					break;
			}
		}
	}
}
