/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Input;
import fr.irisa.espresso.sme.Output;
import fr.irisa.espresso.sme.Signal;
import fr.irisa.espresso.sme.SignalRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.common.SignalKind;
import fr.irisa.espresso.sme.diagram.clockrelations.figures.SignalRefFigure;
import fr.irisa.espresso.sme.diagram.clockrelations.preferences.CRD_DiagramPreferenceConstants;

/*
 * The SignalRef object controller
 * 
 * @generated
 */
public class SignalRefEditPart extends SignalOrRefEditPart {
	/**
	 * Constructor
	 * 
	 * @param obj
	 *        the graph node
	 * @generated
	 */
	public SignalRefEditPart(GraphNode obj) {
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated NOT
	 */
	protected IFigure createFigure() {
		IFigure fig = new SignalRefFigure();
		SignalRef sr = (SignalRef) Utils.getElement((GraphElement) this.getModel());
		Label lbl = new Label(sr.toString(true));
		lbl.setParent(fig);
		fig.setToolTip(lbl);
		return fig;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultBackgroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultBackgroundColor() {
		String backgroundColor = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.SIGNALREF_DEFAULT_BACKGROUND_COLOR);
		if (backgroundColor.length() != 0) {
			return Utils.getColor(backgroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultForegroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String foregroundColor = getPreferenceStore().getString(
			CRD_DiagramPreferenceConstants.SIGNALREF_DEFAULT_FOREGROUND_COLOR);
		if (foregroundColor.length() != 0) {
			return Utils.getColor(foregroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultFont()
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(CRD_DiagramPreferenceConstants.SIGNALREF_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
	 * @generated NOT
	 */
	protected void handleModelChanged(Notification msg) {
		IFigure fig = getFigure();
		Label lbl = (Label) fig.getToolTip();
		SignalRef sr = (SignalRef) Utils.getElement((GraphElement) this.getModel());
		if (msg.getNotifier() instanceof SignalRef) {
			switch (msg.getFeatureID(SignalRef.class)) {
				case SmePackage.SIGNAL_REF__SIGNAL:
					lbl.setText(sr.toString(true));
					break;
			}
		}
		else if (msg.getNotifier() instanceof Signal && msg.getNotifier().equals(sr.getSignal()))
			lbl.setText(sr.toString(true));
		super.handleModelChanged(msg);
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
	 * @generated NOT
	 */
	protected void refreshVisuals() {
		super.refreshVisuals();
		
		updateSignalRefKind();
	}
	
	/**
	 * Update the kind of signal referenced by the SignalRef object
	 * 
	 * @generated NOT
	 */
	protected void updateSignalRefKind() {
		Signal sig = ((SignalRef) getEObject()).getSignal();
		SignalKind kind;
		if (sig == null)
			kind = SignalKind.NONE_LITERAL;
		else if (sig instanceof Input)
			kind = SignalKind.INPUT_LITERAL;
		else if (sig instanceof Output)
			kind = SignalKind.OUTPUT_LITERAL;
		else
			// Local signal
			kind = SignalKind.LOCAL_LITERAL;
		
		SignalRefFigure fig = (SignalRefFigure) getFigure();
		if (fig.getSignalRefKind() != kind) {
			fig.setSignalRefKind(kind);
			fig.repaint();
		}
	}
}
