/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.clockrelations.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy;
import org.topcased.modeler.utils.SourceTargetData;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.DependenceInOut;
import fr.irisa.espresso.sme.DependenceOperator;
import fr.irisa.espresso.sme.diagram.clockrelations.commands.DependenceEdgeCreationCommand;

/*
 * Dependence edge creation
 * 
 * @generated
 */
public class DependenceEdgeCreationEditPolicy extends AbstractEdgeCreationEditPolicy {
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#createCommand(org.eclipse.gef.EditDomain,
	 *      org.topcased.modeler.di.model.GraphEdge, org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected CreateTypedEdgeCommand createCommand(EditDomain domain, GraphEdge edge, GraphElement source) {
		return new DependenceEdgeCreationCommand(domain, edge, source);
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkEdge(org.topcased.modeler.di.model.GraphEdge)
	 * @generated
	 */
	protected boolean checkEdge(GraphEdge edge) {
		return Utils.getElement(edge) instanceof Dependence;
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkSource(org.topcased.modeler.di.model.GraphElement)
	 * @generated NOT
	 */
	protected boolean checkSource(GraphElement source) {
		EObject object = Utils.getElement(source);
		return ((object instanceof DependenceInOut) || (object instanceof DependenceOperator));
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkTargetForSource(org.topcased.modeler.di.model.GraphElement,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated NOT
	 */
	protected boolean checkTargetForSource(GraphElement source, GraphElement target) {
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);
		
		if (sourceObject instanceof DependenceOperator && targetObject instanceof DependenceOperator)
			return false;
		if (sourceObject instanceof DependenceInOut && targetObject instanceof DependenceInOut) {
			if (!sourceObject.equals(targetObject))
				return true;
		}
		return false;
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkCommand(org.eclipse.gef.commands.Command)
	 * @generated
	 */
	protected boolean checkCommand(Command command) {
		return command instanceof DependenceEdgeCreationCommand;
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getSourceTargetData(org.topcased.modeler.di.model.GraphElement,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected SourceTargetData getSourceTargetData(GraphElement source, GraphElement target) {
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);
		
		if (sourceObject instanceof fr.irisa.espresso.sme.DependenceInOut
				&& targetObject instanceof fr.irisa.espresso.sme.DependenceInOut) {
			return new SourceTargetData(false, false, SourceTargetData.DIAGRAM, "fr.irisa.espresso.sme.Model",
				"connections", "src", "dst", "src_Dependences", null, "dst_Dependences", null);
		}
		if (sourceObject instanceof fr.irisa.espresso.sme.DependenceInOut
				&& targetObject instanceof fr.irisa.espresso.sme.DependenceOperator) {
			return new SourceTargetData(false, false, SourceTargetData.DIAGRAM, "fr.irisa.espresso.sme.Model",
				"connections", "src", "dst", "src_Dependences", null, "dst_Dependences", null);
		}
		if (sourceObject instanceof fr.irisa.espresso.sme.DependenceOperator
				&& targetObject instanceof fr.irisa.espresso.sme.DependenceInOut) {
			return new SourceTargetData(false, false, SourceTargetData.DIAGRAM, "fr.irisa.espresso.sme.Model",
				"connections", "src", "dst", "src_Dependences", null, "dst_Dependences", null);
		}
		return null;
	}
	
}
