/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow;

import java.net.URL;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.editor.IConfiguration;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.IPaletteManager;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.graphconf.exceptions.MissingGraphConfFileException;

import fr.irisa.espresso.sme.diagram.dataflow.edit.AndEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArithmeticOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArrayEnumerationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArrayProductEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArrayRecoveryEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArrayRestructurationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.AssertionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.AutomatonEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.BasicIterateEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.BooleanExpressionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.BooleanOperatorIEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.CartesianProductEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.CaseConnectionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.CellEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ComparisonOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ComplexOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConcatenationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConditionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConstantAffectationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConstantExprEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConstantRefEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConstantValueEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.CounterEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.DataflowConnectionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.DelayEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.EnumerationTypeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.EnumerationValueEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ExtractionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.IfThenElseEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.IndexEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.InputEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.InputInstanceEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.InputPortEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.IterateEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.IteratorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.LiteralEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.LocalEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.MergeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ModelDeclarationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ModelInstanceEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ModelRefEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ModelTypeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.NotEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.NumericExpressionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.OrEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.OutputEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.OutputInstanceEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.OutputPortEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ParameterEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ParameterInstanceEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.RepetitionOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SequentialDefinitionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SignalExprEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SignalOrRefEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SignalRefEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SubProcessEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SubstitutionTypeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SwitchEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.TranspositionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.TupleOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.TupleTypeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.UnaryMinusEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.UseModuleEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.VariableEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.WindowEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.XorEditPart;

/*
 * A diagram configuration : manages Palette, EditPartFactory for this diagram.
 * 
 * @generated
 */
public class DF_Configuration implements IConfiguration {
	/**
	 * @generated
	 */
	private DF_PaletteManager	paletteManager;
	
	/**
	 * @generated
	 */
	private DF_EditPartFactory	editPartFactory;
	
	/**
	 * @generated
	 */
	private DF_CreationUtils	creationUtils;
	
	/**
	 * The DiagramGraphConf that contains graphical informations on the configuration
	 * 
	 * @generated
	 */
	private DiagramGraphConf	diagramGraphConf;
	
	/**
	 * Constructor. Initialize Adapter factories.
	 * 
	 * @generated
	 */
	public DF_Configuration() {
		registerAdapters();
	}
	
	/**
	 * Registers the Adapter Factories for all the EditParts
	 * 
	 * @generated NOT
	 */
	private void registerAdapters() {
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SubProcessEditPart.class, fr.irisa.espresso.sme.SubProcess.class),
			SubProcessEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(BasicIterateEditPart.class, fr.irisa.espresso.sme.BasicIterate.class),
			BasicIterateEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(IterateEditPart.class, fr.irisa.espresso.sme.Iterate.class),
			IterateEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ModelDeclarationEditPart.class,
				fr.irisa.espresso.sme.ModelDeclaration.class), ModelDeclarationEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ModelRefEditPart.class, fr.irisa.espresso.sme.ModelRef.class),
			ModelRefEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(AutomatonEditPart.class, fr.irisa.espresso.sme.Automaton.class),
			AutomatonEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ModelInstanceEditPart.class, fr.irisa.espresso.sme.ModelInstance.class),
			ModelInstanceEditPart.class);
		Platform.getAdapterManager()
				.registerAdapters(
					new EditPart2ModelAdapterFactory(EnumerationTypeEditPart.class,
						fr.irisa.espresso.sme.EnumerationType.class), EnumerationTypeEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SubstitutionTypeEditPart.class,
				fr.irisa.espresso.sme.SubstitutionType.class), SubstitutionTypeEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(TupleTypeEditPart.class, fr.irisa.espresso.sme.TupleType.class),
			TupleTypeEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ModelTypeEditPart.class, fr.irisa.espresso.sme.ModelType.class),
			ModelTypeEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(UseModuleEditPart.class, fr.irisa.espresso.sme.UseModule.class),
			UseModuleEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(InputPortEditPart.class, fr.irisa.espresso.sme.InputPort.class),
			InputPortEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(AssertionEditPart.class, fr.irisa.espresso.sme.Assertion.class),
			AssertionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(CartesianProductEditPart.class,
				fr.irisa.espresso.sme.CartesianProduct.class), CartesianProductEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SwitchEditPart.class, fr.irisa.espresso.sme.Switch.class),
			SwitchEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SignalExprEditPart.class, fr.irisa.espresso.sme.SignalExpr.class),
			SignalExprEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SignalOrRefEditPart.class, fr.irisa.espresso.sme.SignalOrRef.class),
			SignalOrRefEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SignalRefEditPart.class, fr.irisa.espresso.sme.SignalRef.class),
			SignalRefEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(InputEditPart.class, fr.irisa.espresso.sme.Input.class),
			InputEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(OutputEditPart.class, fr.irisa.espresso.sme.Output.class),
			OutputEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(LocalEditPart.class, fr.irisa.espresso.sme.Local.class),
			LocalEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(InputInstanceEditPart.class, fr.irisa.espresso.sme.InputInstance.class),
			InputInstanceEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(OutputInstanceEditPart.class, fr.irisa.espresso.sme.OutputInstance.class),
			OutputInstanceEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(CellEditPart.class, fr.irisa.espresso.sme.Cell.class), CellEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(CounterEditPart.class, fr.irisa.espresso.sme.Counter.class),
			CounterEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(DelayEditPart.class, fr.irisa.espresso.sme.Delay.class),
			DelayEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(VariableEditPart.class, fr.irisa.espresso.sme.Variable.class),
			VariableEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(WindowEditPart.class, fr.irisa.espresso.sme.Window.class),
			WindowEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ConstantExprEditPart.class, fr.irisa.espresso.sme.ConstantExpr.class),
			ConstantExprEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(OutputPortEditPart.class, fr.irisa.espresso.sme.OutputPort.class),
			OutputPortEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ConstantRefEditPart.class, fr.irisa.espresso.sme.ConstantRef.class),
			ConstantRefEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ParameterEditPart.class, fr.irisa.espresso.sme.Parameter.class),
			ParameterEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ConstantValueEditPart.class, fr.irisa.espresso.sme.ConstantValue.class),
			ConstantValueEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(LiteralEditPart.class, fr.irisa.espresso.sme.Literal.class),
			LiteralEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(EnumerationValueEditPart.class,
				fr.irisa.espresso.sme.EnumerationValue.class), EnumerationValueEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ParameterInstanceEditPart.class,
				fr.irisa.espresso.sme.ParameterInstance.class), ParameterInstanceEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(IndexEditPart.class, fr.irisa.espresso.sme.Index.class),
			IndexEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(IteratorEditPart.class, fr.irisa.espresso.sme.Iterator.class),
			IteratorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ArrayEnumerationEditPart.class,
				fr.irisa.espresso.sme.ArrayEnumeration.class), ArrayEnumerationEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ArrayRecoveryEditPart.class, fr.irisa.espresso.sme.ArrayRecovery.class),
			ArrayRecoveryEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ArrayRestructurationEditPart.class,
				fr.irisa.espresso.sme.ArrayRestructuration.class), ArrayRestructurationEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ConcatenationEditPart.class, fr.irisa.espresso.sme.Concatenation.class),
			ConcatenationEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ExtractionEditPart.class, fr.irisa.espresso.sme.Extraction.class),
			ExtractionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(IfThenElseEditPart.class, fr.irisa.espresso.sme.IfThenElse.class),
			IfThenElseEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(MergeEditPart.class, fr.irisa.espresso.sme.Merge.class),
			MergeEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(RepetitionOperatorEditPart.class,
				fr.irisa.espresso.sme.RepetitionOperator.class), RepetitionOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SequentialDefinitionEditPart.class,
				fr.irisa.espresso.sme.SequentialDefinition.class), SequentialDefinitionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(TranspositionEditPart.class, fr.irisa.espresso.sme.Transposition.class),
			TranspositionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(TupleOperatorEditPart.class, fr.irisa.espresso.sme.TupleOperator.class),
			TupleOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(BooleanOperatorIEditPart.class,
				fr.irisa.espresso.sme.BooleanOperatorI.class), BooleanOperatorIEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(BooleanExpressionEditPart.class,
				fr.irisa.espresso.sme.BooleanExpression.class), BooleanExpressionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(AndEditPart.class, fr.irisa.espresso.sme.And.class), AndEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ComparisonOperatorEditPart.class,
				fr.irisa.espresso.sme.ComparisonOperator.class), ComparisonOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(NotEditPart.class, fr.irisa.espresso.sme.Not.class), NotEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(OrEditPart.class, fr.irisa.espresso.sme.Or.class), OrEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(XorEditPart.class, fr.irisa.espresso.sme.Xor.class), XorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ArithmeticOperatorEditPart.class,
				fr.irisa.espresso.sme.ArithmeticOperator.class), ArithmeticOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ArrayProductEditPart.class, fr.irisa.espresso.sme.ArrayProduct.class),
			ArrayProductEditPart.class);
		Platform.getAdapterManager()
				.registerAdapters(
					new EditPart2ModelAdapterFactory(ComplexOperatorEditPart.class,
						fr.irisa.espresso.sme.ComplexOperator.class), ComplexOperatorEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(NumericExpressionEditPart.class,
				fr.irisa.espresso.sme.NumericExpression.class), NumericExpressionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(UnaryMinusEditPart.class, fr.irisa.espresso.sme.UnaryMinus.class),
			UnaryMinusEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(CaseConnectionEditPart.class, fr.irisa.espresso.sme.CaseConnection.class),
			CaseConnectionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ConditionEditPart.class, fr.irisa.espresso.sme.Condition.class),
			ConditionEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ConstantAffectationEditPart.class,
				fr.irisa.espresso.sme.ConstantAffectation.class), ConstantAffectationEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(DataflowConnectionEditPart.class,
				fr.irisa.espresso.sme.DataflowConnection.class), DataflowConnectionEditPart.class);
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getId()
	 * @generated
	 */
	public String getId() {
		return new String("fr.irisa.espresso.sme.diagram.dataflow");
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getName()
	 * @generated
	 */
	public String getName() {
		return new String("Dataflow Diagram");
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getEditPartFactory()
	 * @generated
	 */
	public EditPartFactory getEditPartFactory() {
		if (editPartFactory == null) {
			editPartFactory = new DF_EditPartFactory();
		}
		
		return editPartFactory;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getPaletteManager()
	 * @generated
	 */
	public IPaletteManager getPaletteManager() {
		if (paletteManager == null) {
			paletteManager = new DF_PaletteManager(getCreationUtils());
		}
		
		return paletteManager;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getCreationUtils()
	 * @generated
	 */
	public ICreationUtils getCreationUtils() {
		if (creationUtils == null) {
			creationUtils = new DF_CreationUtils(getDiagramGraphConf());
		}
		
		return creationUtils;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getDiagramGraphConf()
	 * @generated
	 */
	public DiagramGraphConf getDiagramGraphConf() {
		if (diagramGraphConf == null) {
			URL url = DF_Plugin.getDefault().getBundle().getResource(
				"fr/irisa/espresso/sme/diagram/dataflow/diagram.graphconf");
			if (url != null) {
				URI fileURI = URI.createURI(url.toString());
				ResourceSet resourceSet = new ResourceSetImpl();
				Resource resource = resourceSet.getResource(fileURI, true);
				if (resource != null && resource.getContents().get(0) instanceof DiagramGraphConf) {
					diagramGraphConf = (DiagramGraphConf) resource.getContents().get(0);
				}
			}
			else {
				new MissingGraphConfFileException(
					"The *.diagramgraphconf file can not be retrieved. Check if the path is correct in the Configuration class of your diagram.");
			}
		}
		
		return diagramGraphConf;
	}
	
}
