/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ProjectScope;
import org.eclipse.core.runtime.Platform;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.preferences.ScopedPreferenceStore;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.EdgeObjectOffset;
import org.topcased.modeler.di.model.EdgeObjectUV;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.AbstractCreationUtils;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Input;
import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.IterationInit;
import fr.irisa.espresso.sme.MultipleInputsOperator;
import fr.irisa.espresso.sme.Output;
import fr.irisa.espresso.sme.OutputInstance;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.ParameterInstance;
import fr.irisa.espresso.sme.SignalOperator;
import fr.irisa.espresso.sme.SmeFactory;
import fr.irisa.espresso.sme.diagram.dataflow.preferences.DF_DiagramPreferenceConstants;
import fr.irisa.espresso.sme.util.SmeSwitch;

/*
 * This utility class allows to create a GraphElement associated with a Model Object
 * 
 * @generated
 */
public class DF_CreationUtils extends AbstractCreationUtils {
	
	/**
	 * Constructor
	 * 
	 * @param diagramConf
	 *        the Diagram Graphical Configuration
	 * @generated
	 */
	public DF_CreationUtils(DiagramGraphConf diagramConf) {
		super(diagramConf);
	}
	
	/**
	 * @generated NOT
	 */
	private class GraphicSmeSwitch extends SmeSwitch<Object> {
		/**
		 * The presentation of the graphical element
		 * 
		 * @generated
		 */
		private String	presentation;
		
		/**
		 * Constructor
		 * 
		 * @param presentation
		 *        the presentation of the graphical element
		 * @generated
		 */
		public GraphicSmeSwitch(String presentation) {
			this.presentation = presentation;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseBasicIterate(fr.irisa.espresso.sme.BasicIterate)
		 * @generated
		 */
		public Object caseBasicIterate(fr.irisa.espresso.sme.BasicIterate object) {
			if ("default".equals(presentation)) {
				return createGraphElementBasicIterate(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIterate(fr.irisa.espresso.sme.Iterate)
		 * @generated
		 */
		public Object caseIterate(fr.irisa.espresso.sme.Iterate object) {
			if ("default".equals(presentation)) {
				return createGraphElementIterate(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAutomaton(fr.irisa.espresso.sme.Automaton)
		 * @generated
		 */
		public Object caseAutomaton(fr.irisa.espresso.sme.Automaton object) {
			if ("default".equals(presentation)) {
				return createGraphElementAutomaton(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelDeclaration(fr.irisa.espresso.sme.ModelDeclaration)
		 * @generated
		 */
		public Object caseModelDeclaration(fr.irisa.espresso.sme.ModelDeclaration object) {
			if ("default".equals(presentation)) {
				return createGraphElementModelDeclaration(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelRef(fr.irisa.espresso.sme.ModelRef)
		 * @generated
		 */
		public Object caseModelRef(fr.irisa.espresso.sme.ModelRef object) {
			if ("default".equals(presentation)) {
				return createGraphElementModelRef(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelInstance(fr.irisa.espresso.sme.ModelInstance)
		 * @generated
		 */
		public Object caseModelInstance(fr.irisa.espresso.sme.ModelInstance object) {
			if ("default".equals(presentation)) {
				return createGraphElementModelInstance(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSubProcess(fr.irisa.espresso.sme.SubProcess)
		 * @generated
		 */
		public Object caseSubProcess(fr.irisa.espresso.sme.SubProcess object) {
			if ("default".equals(presentation)) {
				return createGraphElementSubProcess(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseEnumerationType(fr.irisa.espresso.sme.EnumerationType)
		 * @generated
		 */
		public Object caseEnumerationType(fr.irisa.espresso.sme.EnumerationType object) {
			if ("default".equals(presentation)) {
				return createGraphElementEnumerationType(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelType(fr.irisa.espresso.sme.ModelType)
		 * @generated
		 */
		public Object caseModelType(fr.irisa.espresso.sme.ModelType object) {
			if ("default".equals(presentation)) {
				return createGraphElementModelType(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSubstitutionType(fr.irisa.espresso.sme.SubstitutionType)
		 * @generated
		 */
		public Object caseSubstitutionType(fr.irisa.espresso.sme.SubstitutionType object) {
			if ("default".equals(presentation)) {
				return createGraphElementSubstitutionType(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseTupleType(fr.irisa.espresso.sme.TupleType)
		 * @generated
		 */
		public Object caseTupleType(fr.irisa.espresso.sme.TupleType object) {
			if ("default".equals(presentation)) {
				return createGraphElementTupleType(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConstantRef(fr.irisa.espresso.sme.ConstantRef)
		 * @generated
		 */
		public Object caseConstantRef(fr.irisa.espresso.sme.ConstantRef object) {
			if ("default".equals(presentation)) {
				return createGraphElementConstantRef(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConstantValue(fr.irisa.espresso.sme.ConstantValue)
		 * @generated
		 */
		public Object caseConstantValue(fr.irisa.espresso.sme.ConstantValue object) {
			if ("default".equals(presentation)) {
				return createGraphElementConstantValue(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIndex(fr.irisa.espresso.sme.Index)
		 * @generated
		 */
		public Object caseIndex(fr.irisa.espresso.sme.Index object) {
			if ("default".equals(presentation)) {
				return createGraphElementIndex(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIndex(fr.irisa.espresso.sme.Index)
		 * @generated NOT
		 */
		public Object caseIterator(fr.irisa.espresso.sme.Iterator object) {
			if ("default".equals(presentation)) {
				return createGraphElementIterator(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseLiteral(fr.irisa.espresso.sme.Literal)
		 * @generated
		 */
		public Object caseLiteral(fr.irisa.espresso.sme.Literal object) {
			if ("default".equals(presentation)) {
				return createGraphElementLiteral(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseEnumerationValue(fr.irisa.espresso.sme.EnumerationValue)
		 * @generated
		 */
		public Object caseEnumerationValue(fr.irisa.espresso.sme.EnumerationValue object) {
			if ("default".equals(presentation)) {
				return createGraphElementEnumerationValue(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseLocal(fr.irisa.espresso.sme.Local)
		 * @generated
		 */
		public Object caseLocal(fr.irisa.espresso.sme.Local object) {
			if ("default".equals(presentation)) {
				return createGraphElementLocal(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSignalRef(fr.irisa.espresso.sme.SignalRef)
		 * @generated
		 */
		public Object caseSignalRef(fr.irisa.espresso.sme.SignalRef object) {
			if ("default".equals(presentation)) {
				return createGraphElementSignalRef(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArithmeticOperator(fr.irisa.espresso.sme.ArithmeticOperator)
		 * @generated
		 */
		public Object caseArithmeticOperator(fr.irisa.espresso.sme.ArithmeticOperator object) {
			if ("default".equals(presentation)) {
				return createGraphElementArithmeticOperator(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseComplexOperator(fr.irisa.espresso.sme.ComplexOperator)
		 * @generated
		 */
		public Object caseComplexOperator(fr.irisa.espresso.sme.ComplexOperator object) {
			if ("default".equals(presentation)) {
				return createGraphElementComplexOperator(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseNumericExpression(fr.irisa.espresso.sme.NumericExpression)
		 * @generated
		 */
		public Object caseNumericExpression(fr.irisa.espresso.sme.NumericExpression object) {
			if ("default".equals(presentation)) {
				return createGraphElementNumericExpression(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseUnaryMinus(fr.irisa.espresso.sme.UnaryMinus)
		 * @generated
		 */
		public Object caseUnaryMinus(fr.irisa.espresso.sme.UnaryMinus object) {
			if ("default".equals(presentation)) {
				return createGraphElementUnaryMinus(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseComparisonOperator(fr.irisa.espresso.sme.ComparisonOperator)
		 * @generated
		 */
		public Object caseComparisonOperator(fr.irisa.espresso.sme.ComparisonOperator object) {
			if ("default".equals(presentation)) {
				return createGraphElementComparisonOperator(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseBooleanExpression(fr.irisa.espresso.sme.BooleanExpression)
		 * @generated
		 */
		public Object caseBooleanExpression(fr.irisa.espresso.sme.BooleanExpression object) {
			if ("default".equals(presentation)) {
				return createGraphElementBooleanExpression(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAnd(fr.irisa.espresso.sme.And)
		 * @generated
		 */
		public Object caseAnd(fr.irisa.espresso.sme.And object) {
			if ("default".equals(presentation)) {
				return createGraphElementAnd(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseNot(fr.irisa.espresso.sme.Not)
		 * @generated
		 */
		public Object caseNot(fr.irisa.espresso.sme.Not object) {
			if ("default".equals(presentation)) {
				return createGraphElementNot(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOr(fr.irisa.espresso.sme.Or)
		 * @generated
		 */
		public Object caseOr(fr.irisa.espresso.sme.Or object) {
			if ("default".equals(presentation)) {
				return createGraphElementOr(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseXor(fr.irisa.espresso.sme.Xor)
		 * @generated
		 */
		public Object caseXor(fr.irisa.espresso.sme.Xor object) {
			if ("default".equals(presentation)) {
				return createGraphElementXor(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArrayRecovery(fr.irisa.espresso.sme.ArrayRecovery)
		 * @generated
		 */
		public Object caseArrayRecovery(fr.irisa.espresso.sme.ArrayRecovery object) {
			if ("default".equals(presentation)) {
				return createGraphElementArrayRecovery(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCartesianProduct(fr.irisa.espresso.sme.CartesianProduct)
		 * @generated
		 */
		public Object caseCartesianProduct(fr.irisa.espresso.sme.CartesianProduct object) {
			if ("default".equals(presentation)) {
				return createGraphElementCartesianProduct(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConcatenation(fr.irisa.espresso.sme.Concatenation)
		 * @generated
		 */
		public Object caseConcatenation(fr.irisa.espresso.sme.Concatenation object) {
			if ("default".equals(presentation)) {
				return createGraphElementConcatenation(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseTransposition(fr.irisa.espresso.sme.Transposition)
		 * @generated
		 */
		public Object caseTransposition(fr.irisa.espresso.sme.Transposition object) {
			if ("default".equals(presentation)) {
				return createGraphElementTransposition(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArrayProduct(fr.irisa.espresso.sme.ArrayProduct)
		 * @generated
		 */
		public Object caseArrayProduct(fr.irisa.espresso.sme.ArrayProduct object) {
			if ("default".equals(presentation)) {
				return createGraphElementArrayProduct(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseRepetitionOperator(fr.irisa.espresso.sme.RepetitionOperator)
		 * @generated
		 */
		public Object caseRepetitionOperator(fr.irisa.espresso.sme.RepetitionOperator object) {
			if ("default".equals(presentation)) {
				return createGraphElementRepetitionOperator(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArrayRestructuration(fr.irisa.espresso.sme.ArrayRestructuration)
		 * @generated
		 */
		public Object caseArrayRestructuration(fr.irisa.espresso.sme.ArrayRestructuration object) {
			if ("default".equals(presentation)) {
				return createGraphElementArrayRestructuration(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSequentialDefinition(fr.irisa.espresso.sme.SequentialDefinition)
		 * @generated
		 */
		public Object caseSequentialDefinition(fr.irisa.espresso.sme.SequentialDefinition object) {
			if ("default".equals(presentation)) {
				return createGraphElementSequentialDefinition(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArrayEnumeration(fr.irisa.espresso.sme.ArrayEnumeration)
		 * @generated
		 */
		public Object caseArrayEnumeration(fr.irisa.espresso.sme.ArrayEnumeration object) {
			if ("default".equals(presentation)) {
				return createGraphElementArrayEnumeration(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCounter(fr.irisa.espresso.sme.Counter)
		 * @generated
		 */
		public Object caseCounter(fr.irisa.espresso.sme.Counter object) {
			if ("default".equals(presentation)) {
				return createGraphElementCounter(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseDelay(fr.irisa.espresso.sme.Delay)
		 * @generated
		 */
		public Object caseDelay(fr.irisa.espresso.sme.Delay object) {
			if ("default".equals(presentation)) {
				return createGraphElementDelay(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseExtraction(fr.irisa.espresso.sme.Extraction)
		 * @generated
		 */
		public Object caseExtraction(fr.irisa.espresso.sme.Extraction object) {
			if ("default".equals(presentation)) {
				return createGraphElementExtraction(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIfThenElse(fr.irisa.espresso.sme.IfThenElse)
		 * @generated
		 */
		public Object caseIfThenElse(fr.irisa.espresso.sme.IfThenElse object) {
			if ("default".equals(presentation)) {
				return createGraphElementIfThenElse(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCell(fr.irisa.espresso.sme.Cell)
		 * @generated
		 */
		public Object caseCell(fr.irisa.espresso.sme.Cell object) {
			if ("default".equals(presentation)) {
				return createGraphElementCell(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseMerge(fr.irisa.espresso.sme.Merge)
		 * @generated
		 */
		public Object caseMerge(fr.irisa.espresso.sme.Merge object) {
			if ("default".equals(presentation)) {
				return createGraphElementMerge(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseWindow(fr.irisa.espresso.sme.Window)
		 * @generated
		 */
		public Object caseWindow(fr.irisa.espresso.sme.Window object) {
			if ("default".equals(presentation)) {
				return createGraphElementWindow(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSwitch(fr.irisa.espresso.sme.Switch)
		 * @generated
		 */
		public Object caseSwitch(fr.irisa.espresso.sme.Switch object) {
			if ("default".equals(presentation)) {
				return createGraphElementSwitch(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseTupleOperator(fr.irisa.espresso.sme.TupleOperator)
		 * @generated
		 */
		public Object caseTupleOperator(fr.irisa.espresso.sme.TupleOperator object) {
			if ("default".equals(presentation)) {
				return createGraphElementTupleOperator(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseVariable(fr.irisa.espresso.sme.Variable)
		 * @generated
		 */
		public Object caseVariable(fr.irisa.espresso.sme.Variable object) {
			if ("default".equals(presentation)) {
				return createGraphElementVariable(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCaseConnection(fr.irisa.espresso.sme.CaseConnection)
		 * @generated
		 */
		public Object caseCaseConnection(fr.irisa.espresso.sme.CaseConnection object) {
			if ("default".equals(presentation)) {
				return createGraphElementCaseConnection(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCondition(fr.irisa.espresso.sme.Condition)
		 * @generated
		 */
		public Object caseCondition(fr.irisa.espresso.sme.Condition object) {
			if ("default".equals(presentation)) {
				return createGraphElementCondition(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConstantAffectation(fr.irisa.espresso.sme.ConstantAffectation)
		 * @generated
		 */
		public Object caseConstantAffectation(fr.irisa.espresso.sme.ConstantAffectation object) {
			if ("default".equals(presentation)) {
				return createGraphElementConstantAffectation(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseDataflowConnection(fr.irisa.espresso.sme.DataflowConnection)
		 * @generated
		 */
		public Object caseDataflowConnection(fr.irisa.espresso.sme.DataflowConnection object) {
			if ("default".equals(presentation)) {
				return createGraphElementDataflowConnection(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAssertion(fr.irisa.espresso.sme.Assertion)
		 * @generated
		 */
		public Object caseAssertion(fr.irisa.espresso.sme.Assertion object) {
			if ("default".equals(presentation)) {
				return createGraphElementAssertion(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseUseModule(fr.irisa.espresso.sme.UseModule)
		 * @generated
		 */
		public Object caseUseModule(fr.irisa.espresso.sme.UseModule object) {
			if ("default".equals(presentation)) {
				return createGraphElementUseModule(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInputPort(fr.irisa.espresso.sme.InputPort)
		 * @generated NOT
		 */
		public Object caseInputPort(fr.irisa.espresso.sme.InputPort object) {
			if ("default".equals(presentation)) {
				return createGraphElementInputPort(object, presentation);
			}
			return null;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOutputPort(fr.irisa.espresso.sme.OutputPort)
		 * @generated NOT
		 */
		public Object caseOutputPort(fr.irisa.espresso.sme.OutputPort object) {
			if ("default".equals(presentation)) {
				return createGraphElementOutputPort(object, presentation);
			}
			return null;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInput(fr.irisa.espresso.sme.Input)
		 * @generated NOT
		 */
		public Object caseInput(fr.irisa.espresso.sme.Input object) {
			if ("default".equals(presentation)) {
				return createGraphElementInput(object, presentation);
			}
			return null;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOutput(fr.irisa.espresso.sme.Output)
		 * @generated NOT
		 */
		public Object caseOutput(fr.irisa.espresso.sme.Output object) {
			if ("default".equals(presentation)) {
				return createGraphElementOutput(object, presentation);
			}
			return null;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInputInstance(fr.irisa.espresso.sme.InputInstance)
		 * @generated NOT
		 */
		public Object caseInputInstance(fr.irisa.espresso.sme.InputInstance object) {
			if ("default".equals(presentation)) {
				return createGraphElementInputInstance(object, presentation);
			}
			return null;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOutputInstance(fr.irisa.espresso.sme.OutputInstance)
		 * @generated NOT
		 */
		public Object caseOutputInstance(fr.irisa.espresso.sme.OutputInstance object) {
			if ("default".equals(presentation)) {
				return createGraphElementOutputInstance(object, presentation);
			}
			return null;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseParameter(fr.irisa.espresso.sme.Parameter)
		 * @generated NOT
		 */
		public Object caseParameter(fr.irisa.espresso.sme.Parameter object) {
			if ("default".equals(presentation)) {
				return createGraphElementParameter(object, presentation);
			}
			return null;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseParameterInstance(fr.irisa.espresso.sme.ParameterInstance)
		 * @generated NOT
		 */
		public Object caseParameterInstance(fr.irisa.espresso.sme.ParameterInstance object) {
			if ("default".equals(presentation)) {
				return createGraphElementParameterInstance(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object) {
			return null;
		}
	}
	
	/**
	 * @see org.topcased.modeler.editor.ICreationUtils#createGraphElement(org.eclipse.emf.ecore.EObject,
	 *      java.lang.String)
	 * @generated
	 */
	public GraphElement createGraphElement(EObject obj, String presentation) {
		Object graphElt = null;
		
		if ("http://www.irisa.fr/espresso/sme.ecore".equals(obj.eClass().getEPackage().getNsURI())) {
			graphElt = new GraphicSmeSwitch(presentation).doSwitch(obj);
		}
		
		return (GraphElement) graphElt;
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementBasicIterate(fr.irisa.espresso.sme.BasicIterate element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementIterate(fr.irisa.espresso.sme.Iterate element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementAutomaton(fr.irisa.espresso.sme.Automaton element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementModelDeclaration(fr.irisa.espresso.sme.ModelDeclaration element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementModelRef(fr.irisa.espresso.sme.ModelRef element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementModelInstance(fr.irisa.espresso.sme.ModelInstance element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementSubProcess(fr.irisa.espresso.sme.SubProcess element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementEnumerationType(fr.irisa.espresso.sme.EnumerationType element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementModelType(fr.irisa.espresso.sme.ModelType element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementSubstitutionType(fr.irisa.espresso.sme.SubstitutionType element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementTupleType(fr.irisa.espresso.sme.TupleType element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementConstantRef(fr.irisa.espresso.sme.ConstantRef element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementConstantValue(fr.irisa.espresso.sme.ConstantValue element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementIndex(fr.irisa.espresso.sme.Index element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementIterator(fr.irisa.espresso.sme.Iterator element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementLiteral(fr.irisa.espresso.sme.Literal element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementEnumerationValue(fr.irisa.espresso.sme.EnumerationValue element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementLocal(fr.irisa.espresso.sme.Local element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementSignalRef(fr.irisa.espresso.sme.SignalRef element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementArithmeticOperator(fr.irisa.espresso.sme.ArithmeticOperator element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementComplexOperator(fr.irisa.espresso.sme.ComplexOperator element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementNumericExpression(fr.irisa.espresso.sme.NumericExpression element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementUnaryMinus(fr.irisa.espresso.sme.UnaryMinus element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementComparisonOperator(fr.irisa.espresso.sme.ComparisonOperator element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementBooleanExpression(fr.irisa.espresso.sme.BooleanExpression element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementAnd(fr.irisa.espresso.sme.And element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementNot(fr.irisa.espresso.sme.Not element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementOr(fr.irisa.espresso.sme.Or element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementXor(fr.irisa.espresso.sme.Xor element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementArrayRecovery(fr.irisa.espresso.sme.ArrayRecovery element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementCartesianProduct(fr.irisa.espresso.sme.CartesianProduct element,
			String presentation) {
		GraphNode graphNode = (GraphNode) createGraphElementSignalOperator(element, presentation);
		
		int nbIn = element.getInputPorts().size(), nbOut = element.getOutputPorts().size();
		if (nbIn > nbOut) {
			for (int i = nbOut; i < nbIn; i++) {
				OutputPort out = SmeFactory.eINSTANCE.createOutputPort();
				element.getOuts().add(out);
				out.setOperatorParent(element);
			}
		}
		else if (nbIn < nbOut) {
			for (int i = nbIn; i < nbOut; i++) {
				InputPort in = SmeFactory.eINSTANCE.createInputPort();
				element.getIns().add(in);
				in.setOperatorParent(element);
				graphNode.getContained().add(createGraphNode(in));
			}
		}
		
		// Add OutputPorts
		for (OutputPort port : element.getOutputPorts()) {
			if (getGraphicalRepresentation(graphNode, port) == null)
				graphNode.getContained().add(createGraphNode(port));
		}
		
		return graphNode;
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementConcatenation(fr.irisa.espresso.sme.Concatenation element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementTransposition(fr.irisa.espresso.sme.Transposition element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementArrayProduct(fr.irisa.espresso.sme.ArrayProduct element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementRepetitionOperator(fr.irisa.espresso.sme.RepetitionOperator element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementArrayRestructuration(fr.irisa.espresso.sme.ArrayRestructuration element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementSequentialDefinition(fr.irisa.espresso.sme.SequentialDefinition element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementArrayEnumeration(fr.irisa.espresso.sme.ArrayEnumeration element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementCounter(fr.irisa.espresso.sme.Counter element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementDelay(fr.irisa.espresso.sme.Delay element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementExtraction(fr.irisa.espresso.sme.Extraction element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementIfThenElse(fr.irisa.espresso.sme.IfThenElse element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementCell(fr.irisa.espresso.sme.Cell element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementMerge(fr.irisa.espresso.sme.Merge element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementWindow(fr.irisa.espresso.sme.Window element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementSwitch(fr.irisa.espresso.sme.Switch element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementTupleOperator(fr.irisa.espresso.sme.TupleOperator element,
			String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementVariable(fr.irisa.espresso.sme.Variable element, String presentation) {
		return createGraphElementSignalOperator(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementCaseConnection(fr.irisa.espresso.sme.CaseConnection element,
			String presentation) {
		GraphEdge graphEdge = createGraphEdge(element, presentation);
		EdgeObjectOffset caselabelEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
		caselabelEdgeObjectOffset.setId(DF_EdgeObjectConstants.CASELABEL_EDGE_OBJECT_ID);
		caselabelEdgeObjectOffset.setOffset(new Dimension(0, 0));
		caselabelEdgeObjectOffset.setVisible(getPreferenceStore().getBoolean(
			DF_DiagramPreferenceConstants.CASECONNECTION_CASELABEL_EDGE_OBJECT_DEFAULT_VISIBILITY));
		graphEdge.getContained().add(caselabelEdgeObjectOffset);
		return graphEdge;
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementCondition(fr.irisa.espresso.sme.Condition element, String presentation) {
		GraphEdge graphEdge = createGraphEdge(element, presentation);
		EdgeObjectUV srcfieldEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
		srcfieldEdgeObjectUV.setId(DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID);
		srcfieldEdgeObjectUV.setUDistance(0);
		srcfieldEdgeObjectUV.setVDistance(0);
		srcfieldEdgeObjectUV.setVisible(getPreferenceStore().getBoolean(
			DF_DiagramPreferenceConstants.CONDITION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY));
		graphEdge.getContained().add(srcfieldEdgeObjectUV);
		EdgeObjectOffset conditionkindEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
		conditionkindEdgeObjectOffset.setId(DF_EdgeObjectConstants.CONDITIONKIND_EDGE_OBJECT_ID);
		conditionkindEdgeObjectOffset.setOffset(new Dimension(0, 0));
		conditionkindEdgeObjectOffset.setVisible(getPreferenceStore().getBoolean(
			DF_DiagramPreferenceConstants.CONDITION_CONDITIONKIND_EDGE_OBJECT_DEFAULT_VISIBILITY));
		graphEdge.getContained().add(conditionkindEdgeObjectOffset);
		return graphEdge;
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementConstantAffectation(fr.irisa.espresso.sme.ConstantAffectation element,
			String presentation) {
		GraphEdge graphEdge = createGraphEdge(element, presentation);
		EdgeObjectUV srcfieldEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
		srcfieldEdgeObjectUV.setId(DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID);
		srcfieldEdgeObjectUV.setUDistance(0);
		srcfieldEdgeObjectUV.setVDistance(0);
		srcfieldEdgeObjectUV.setVisible(getPreferenceStore().getBoolean(
			DF_DiagramPreferenceConstants.CONSTANTAFFECTATION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY));
		graphEdge.getContained().add(srcfieldEdgeObjectUV);
		return graphEdge;
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementDataflowConnection(fr.irisa.espresso.sme.DataflowConnection element,
			String presentation) {
		GraphEdge graphEdge = createGraphEdge(element, presentation);
		EdgeObjectUV srcfieldEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
		srcfieldEdgeObjectUV.setId(DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID);
		srcfieldEdgeObjectUV.setUDistance(0);
		srcfieldEdgeObjectUV.setVDistance(0);
		srcfieldEdgeObjectUV.setVisible(getPreferenceStore().getBoolean(
			DF_DiagramPreferenceConstants.DATAFLOWCONNECTION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY));
		graphEdge.getContained().add(srcfieldEdgeObjectUV);
		EdgeObjectUV dstfieldEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
		dstfieldEdgeObjectUV.setId(DF_EdgeObjectConstants.DSTFIELD_EDGE_OBJECT_ID);
		dstfieldEdgeObjectUV.setUDistance(0);
		dstfieldEdgeObjectUV.setVDistance(0);
		dstfieldEdgeObjectUV.setVisible(getPreferenceStore().getBoolean(
			DF_DiagramPreferenceConstants.DATAFLOWCONNECTION_DSTFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY));
		graphEdge.getContained().add(dstfieldEdgeObjectUV);
		EdgeObjectOffset casttypeEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
		casttypeEdgeObjectOffset.setId(DF_EdgeObjectConstants.CASTTYPE_EDGE_OBJECT_ID);
		casttypeEdgeObjectOffset.setOffset(new Dimension(0, 0));
		casttypeEdgeObjectOffset.setVisible(getPreferenceStore().getBoolean(
			DF_DiagramPreferenceConstants.DATAFLOWCONNECTION_CASTTYPE_EDGE_OBJECT_DEFAULT_VISIBILITY));
		graphEdge.getContained().add(casttypeEdgeObjectOffset);
		return graphEdge;
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementAssertion(fr.irisa.espresso.sme.Assertion element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementUseModule(fr.irisa.espresso.sme.UseModule element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementIterationInit(IterationInit element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementInputPort(InputPort element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementOutputPort(OutputPort element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementInput(Input element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementOutput(Output element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementParameter(Parameter element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementInputInstance(InputInstance element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementOutputInstance(OutputInstance element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * 
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated NOT
	 */
	protected GraphElement createGraphElementParameterInstance(ParameterInstance element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 * @param presentation
	 * @return
	 * @generated NOT
	 */
	private GraphElement createGraphElementSignalOperator(SignalOperator element, String presentation) {
		GraphNode graphNode = createGraphNode(element, presentation);
		
		if (!(element instanceof MultipleInputsOperator && element.getInputPorts().size() != 0))
			fr.irisa.espresso.sme.util.Utils.createPorts(element);
		
		// Add InputPorts
		for (InputPort port : element.getInputPorts()) {
			if (getGraphicalRepresentation(graphNode, port) == null)
				graphNode.getContained().add(createGraphNode(port));
		}
		
		return graphNode;
	}
	
	/**
	 * Get the graphNode associated with the model object in the container
	 * 
	 * @param container
	 *        the graphNode in which the search of the graphNode is done
	 * @param modelObject
	 *        the model object to display
	 * @return the GraphNode or null if it does not exist
	 * @generated NOT
	 */
	private GraphNode getGraphicalRepresentation(GraphNode container, EObject modelObject) {
		for (DiagramElement iter : container.getContained()) {
			GraphElement elt = (GraphElement) iter;
			if (Utils.getElement(elt) == modelObject && elt instanceof GraphNode)
				return (GraphNode) elt;
		}
		return null;
	}
	
	/**
	 * Create the ModelObject with its initial children
	 * 
	 * @param obj
	 *        the model object
	 * @return the model object with its children
	 * @generated
	 */
	public EObject createModelObject(EObject obj) {
		return obj;
	}
	
	/**
	 * Get the preference store associated with the current editor.
	 * 
	 * @return IPreferenceStore
	 * @generated
	 */
	private IPreferenceStore getPreferenceStore() {
		IEditorInput editorInput = DF_Plugin.getActivePage().getActiveEditor().getEditorInput();
		if (editorInput instanceof IFileEditorInput) {
			IProject project = ((IFileEditorInput) editorInput).getFile().getProject();
			Preferences root = Platform.getPreferencesService().getRootNode();
			try {
				if (root.node(ProjectScope.SCOPE).node(project.getName()).nodeExists(DF_Plugin.getId())) {
					return new ScopedPreferenceStore(new ProjectScope(project), DF_Plugin.getId());
				}
			}
			catch (BackingStoreException e) {
				e.printStackTrace();
			}
		}
		return DF_Plugin.getDefault().getPreferenceStore();
	}
}
