/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.editor.ModelerEditPartFactory;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.diagram.dataflow.edit.AndEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArithmeticOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArrayEnumerationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArrayProductEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArrayRecoveryEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ArrayRestructurationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.AssertionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.AutomatonEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.BasicIterateEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.BooleanExpressionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.BooleanOperatorIEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.CartesianProductEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.CaseConnectionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.CellEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ComparisonOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ComplexOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConcatenationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConditionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConstantAffectationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConstantExprEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConstantRefEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ConstantValueEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.CounterEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.DF_DiagramEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.DataflowConnectionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.DelayEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.EnumerationTypeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.EnumerationValueEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ExtractionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.IfThenElseEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.IndexEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.InputEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.InputInstanceEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.InputPortEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.IterateEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.IteratorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.LiteralEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.LocalEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.MergeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ModelDeclarationEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ModelInstanceEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ModelRefEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ModelTypeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.NotEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.NumericExpressionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.OrEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.OutputEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.OutputInstanceEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.OutputPortEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ParameterEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.ParameterInstanceEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.RepetitionOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SequentialDefinitionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SignalExprEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SignalOrRefEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SignalRefEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SubProcessEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SubstitutionTypeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.SwitchEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.TranspositionEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.TupleOperatorEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.TupleTypeEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.UnaryMinusEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.UseModuleEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.VariableEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.WindowEditPart;
import fr.irisa.espresso.sme.diagram.dataflow.edit.XorEditPart;
import fr.irisa.espresso.sme.util.SmeSwitch;

/*
 * Part Factory : associates a model object to its controller. <br>
 * 
 * @generated
 */
public class DF_EditPartFactory extends ModelerEditPartFactory {
	/**
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
	 * @generated
	 */
	public EditPart createEditPart(EditPart context, Object model) {
		if (model instanceof Diagram) {
			return new DF_DiagramEditPart((Diagram) model);
		}
		else if (model instanceof GraphNode) {
			final GraphNode node = (GraphNode) model;
			EObject element = Utils.getElement(node);
			if (element != null) {
				if ("http://www.irisa.fr/espresso/sme.ecore".equals(element.eClass().getEPackage().getNsURI())) {
					return (EditPart) new NodeSmeSwitch(node).doSwitch(element);
				}
			}
			
			if (node.getSemanticModel() instanceof SimpleSemanticModelElement) {
				// Manage the Element that are not associated with a model object
			}
		}
		else if (model instanceof GraphEdge) {
			final GraphEdge edge = (GraphEdge) model;
			EObject element = Utils.getElement(edge);
			if (element != null) {
				if ("http://www.irisa.fr/espresso/sme.ecore".equals(element.eClass().getEPackage().getNsURI())) {
					return (EditPart) new EdgeSmeSwitch(edge).doSwitch(element);
				}
			}
			
			if (edge.getSemanticModel() instanceof SimpleSemanticModelElement) {
				// Manage the Element that are not associated with a model object
			}
		}
		return super.createEditPart(context, model);
	}
	
	/**
	 * @generated
	 */
	private class NodeSmeSwitch extends SmeSwitch<Object> {
		/**
		 * The graphical node
		 * 
		 * @generated
		 */
		private GraphNode	node;
		
		/**
		 * Constructor
		 * 
		 * @param node
		 *        the graphical node
		 * @generated
		 */
		public NodeSmeSwitch(GraphNode node) {
			this.node = node;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSubProcess(fr.irisa.espresso.sme.SubProcess)
		 * @generated
		 */
		public Object caseSubProcess(fr.irisa.espresso.sme.SubProcess object) {
			return new SubProcessEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseBasicIterate(fr.irisa.espresso.sme.BasicIterate)
		 * @generated
		 */
		public Object caseBasicIterate(fr.irisa.espresso.sme.BasicIterate object) {
			return new BasicIterateEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIterate(fr.irisa.espresso.sme.Iterate)
		 * @generated
		 */
		public Object caseIterate(fr.irisa.espresso.sme.Iterate object) {
			return new IterateEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelDeclaration(fr.irisa.espresso.sme.ModelDeclaration)
		 * @generated
		 */
		public Object caseModelDeclaration(fr.irisa.espresso.sme.ModelDeclaration object) {
			return new ModelDeclarationEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelRef(fr.irisa.espresso.sme.ModelRef)
		 * @generated
		 */
		public Object caseModelRef(fr.irisa.espresso.sme.ModelRef object) {
			return new ModelRefEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAutomaton(fr.irisa.espresso.sme.Automaton)
		 * @generated
		 */
		public Object caseAutomaton(fr.irisa.espresso.sme.Automaton object) {
			return new AutomatonEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelInstance(fr.irisa.espresso.sme.ModelInstance)
		 * @generated
		 */
		public Object caseModelInstance(fr.irisa.espresso.sme.ModelInstance object) {
			return new ModelInstanceEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseEnumerationType(fr.irisa.espresso.sme.EnumerationType)
		 * @generated
		 */
		public Object caseEnumerationType(fr.irisa.espresso.sme.EnumerationType object) {
			return new EnumerationTypeEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSubstitutionType(fr.irisa.espresso.sme.SubstitutionType)
		 * @generated
		 */
		public Object caseSubstitutionType(fr.irisa.espresso.sme.SubstitutionType object) {
			return new SubstitutionTypeEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseTupleType(fr.irisa.espresso.sme.TupleType)
		 * @generated
		 */
		public Object caseTupleType(fr.irisa.espresso.sme.TupleType object) {
			return new TupleTypeEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseModelType(fr.irisa.espresso.sme.ModelType)
		 * @generated
		 */
		public Object caseModelType(fr.irisa.espresso.sme.ModelType object) {
			return new ModelTypeEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseUseModule(fr.irisa.espresso.sme.UseModule)
		 * @generated
		 */
		public Object caseUseModule(fr.irisa.espresso.sme.UseModule object) {
			return new UseModuleEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInputPort(fr.irisa.espresso.sme.InputPort)
		 * @generated
		 */
		public Object caseInputPort(fr.irisa.espresso.sme.InputPort object) {
			return new InputPortEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAssertion(fr.irisa.espresso.sme.Assertion)
		 * @generated
		 */
		public Object caseAssertion(fr.irisa.espresso.sme.Assertion object) {
			return new AssertionEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCartesianProduct(fr.irisa.espresso.sme.CartesianProduct)
		 * @generated
		 */
		public Object caseCartesianProduct(fr.irisa.espresso.sme.CartesianProduct object) {
			return new CartesianProductEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSwitch(fr.irisa.espresso.sme.Switch)
		 * @generated
		 */
		public Object caseSwitch(fr.irisa.espresso.sme.Switch object) {
			return new SwitchEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSignalExpr(fr.irisa.espresso.sme.SignalExpr)
		 * @generated
		 */
		public Object caseSignalExpr(fr.irisa.espresso.sme.SignalExpr object) {
			return new SignalExprEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSignalOrRef(fr.irisa.espresso.sme.SignalOrRef)
		 * @generated
		 */
		public Object caseSignalOrRef(fr.irisa.espresso.sme.SignalOrRef object) {
			return new SignalOrRefEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSignalRef(fr.irisa.espresso.sme.SignalRef)
		 * @generated
		 */
		public Object caseSignalRef(fr.irisa.espresso.sme.SignalRef object) {
			return new SignalRefEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInput(fr.irisa.espresso.sme.Input)
		 * @generated
		 */
		public Object caseInput(fr.irisa.espresso.sme.Input object) {
			return new InputEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOutput(fr.irisa.espresso.sme.Output)
		 * @generated
		 */
		public Object caseOutput(fr.irisa.espresso.sme.Output object) {
			return new OutputEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseLocal(fr.irisa.espresso.sme.Local)
		 * @generated
		 */
		public Object caseLocal(fr.irisa.espresso.sme.Local object) {
			return new LocalEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseInputInstance(fr.irisa.espresso.sme.InputInstance)
		 * @generated
		 */
		public Object caseInputInstance(fr.irisa.espresso.sme.InputInstance object) {
			return new InputInstanceEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOutputInstance(fr.irisa.espresso.sme.OutputInstance)
		 * @generated
		 */
		public Object caseOutputInstance(fr.irisa.espresso.sme.OutputInstance object) {
			return new OutputInstanceEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCell(fr.irisa.espresso.sme.Cell)
		 * @generated
		 */
		public Object caseCell(fr.irisa.espresso.sme.Cell object) {
			return new CellEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCounter(fr.irisa.espresso.sme.Counter)
		 * @generated
		 */
		public Object caseCounter(fr.irisa.espresso.sme.Counter object) {
			return new CounterEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseDelay(fr.irisa.espresso.sme.Delay)
		 * @generated
		 */
		public Object caseDelay(fr.irisa.espresso.sme.Delay object) {
			return new DelayEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseVariable(fr.irisa.espresso.sme.Variable)
		 * @generated
		 */
		public Object caseVariable(fr.irisa.espresso.sme.Variable object) {
			return new VariableEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseWindow(fr.irisa.espresso.sme.Window)
		 * @generated
		 */
		public Object caseWindow(fr.irisa.espresso.sme.Window object) {
			return new WindowEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConstantExpr(fr.irisa.espresso.sme.ConstantExpr)
		 * @generated
		 */
		public Object caseConstantExpr(fr.irisa.espresso.sme.ConstantExpr object) {
			return new ConstantExprEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOutputPort(fr.irisa.espresso.sme.OutputPort)
		 * @generated
		 */
		public Object caseOutputPort(fr.irisa.espresso.sme.OutputPort object) {
			return new OutputPortEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConstantRef(fr.irisa.espresso.sme.ConstantRef)
		 * @generated
		 */
		public Object caseConstantRef(fr.irisa.espresso.sme.ConstantRef object) {
			return new ConstantRefEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseParameter(fr.irisa.espresso.sme.Parameter)
		 * @generated
		 */
		public Object caseParameter(fr.irisa.espresso.sme.Parameter object) {
			return new ParameterEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConstantValue(fr.irisa.espresso.sme.ConstantValue)
		 * @generated
		 */
		public Object caseConstantValue(fr.irisa.espresso.sme.ConstantValue object) {
			return new ConstantValueEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseLiteral(fr.irisa.espresso.sme.Literal)
		 * @generated
		 */
		public Object caseLiteral(fr.irisa.espresso.sme.Literal object) {
			return new LiteralEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseEnumerationValue(fr.irisa.espresso.sme.EnumerationValue)
		 * @generated
		 */
		public Object caseEnumerationValue(fr.irisa.espresso.sme.EnumerationValue object) {
			return new EnumerationValueEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseParameterInstance(fr.irisa.espresso.sme.ParameterInstance)
		 * @generated
		 */
		public Object caseParameterInstance(fr.irisa.espresso.sme.ParameterInstance object) {
			return new ParameterInstanceEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIndex(fr.irisa.espresso.sme.Index)
		 * @generated
		 */
		public Object caseIndex(fr.irisa.espresso.sme.Index object) {
			return new IndexEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIterator(fr.irisa.espresso.sme.Iterator)
		 * @generated
		 */
		public Object caseIterator(fr.irisa.espresso.sme.Iterator object) {
			return new IteratorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArrayEnumeration(fr.irisa.espresso.sme.ArrayEnumeration)
		 * @generated
		 */
		public Object caseArrayEnumeration(fr.irisa.espresso.sme.ArrayEnumeration object) {
			return new ArrayEnumerationEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArrayRecovery(fr.irisa.espresso.sme.ArrayRecovery)
		 * @generated
		 */
		public Object caseArrayRecovery(fr.irisa.espresso.sme.ArrayRecovery object) {
			return new ArrayRecoveryEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArrayRestructuration(fr.irisa.espresso.sme.ArrayRestructuration)
		 * @generated
		 */
		public Object caseArrayRestructuration(fr.irisa.espresso.sme.ArrayRestructuration object) {
			return new ArrayRestructurationEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConcatenation(fr.irisa.espresso.sme.Concatenation)
		 * @generated
		 */
		public Object caseConcatenation(fr.irisa.espresso.sme.Concatenation object) {
			return new ConcatenationEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseExtraction(fr.irisa.espresso.sme.Extraction)
		 * @generated
		 */
		public Object caseExtraction(fr.irisa.espresso.sme.Extraction object) {
			return new ExtractionEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseIfThenElse(fr.irisa.espresso.sme.IfThenElse)
		 * @generated
		 */
		public Object caseIfThenElse(fr.irisa.espresso.sme.IfThenElse object) {
			return new IfThenElseEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseMerge(fr.irisa.espresso.sme.Merge)
		 * @generated
		 */
		public Object caseMerge(fr.irisa.espresso.sme.Merge object) {
			return new MergeEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseRepetitionOperator(fr.irisa.espresso.sme.RepetitionOperator)
		 * @generated
		 */
		public Object caseRepetitionOperator(fr.irisa.espresso.sme.RepetitionOperator object) {
			return new RepetitionOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSequentialDefinition(fr.irisa.espresso.sme.SequentialDefinition)
		 * @generated
		 */
		public Object caseSequentialDefinition(fr.irisa.espresso.sme.SequentialDefinition object) {
			return new SequentialDefinitionEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseTransposition(fr.irisa.espresso.sme.Transposition)
		 * @generated
		 */
		public Object caseTransposition(fr.irisa.espresso.sme.Transposition object) {
			return new TranspositionEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseTupleOperator(fr.irisa.espresso.sme.TupleOperator)
		 * @generated
		 */
		public Object caseTupleOperator(fr.irisa.espresso.sme.TupleOperator object) {
			return new TupleOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseBooleanOperatorI(fr.irisa.espresso.sme.BooleanOperatorI)
		 * @generated
		 */
		public Object caseBooleanOperatorI(fr.irisa.espresso.sme.BooleanOperatorI object) {
			return new BooleanOperatorIEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseBooleanExpression(fr.irisa.espresso.sme.BooleanExpression)
		 * @generated
		 */
		public Object caseBooleanExpression(fr.irisa.espresso.sme.BooleanExpression object) {
			return new BooleanExpressionEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseAnd(fr.irisa.espresso.sme.And)
		 * @generated
		 */
		public Object caseAnd(fr.irisa.espresso.sme.And object) {
			return new AndEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseComparisonOperator(fr.irisa.espresso.sme.ComparisonOperator)
		 * @generated
		 */
		public Object caseComparisonOperator(fr.irisa.espresso.sme.ComparisonOperator object) {
			return new ComparisonOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseNot(fr.irisa.espresso.sme.Not)
		 * @generated
		 */
		public Object caseNot(fr.irisa.espresso.sme.Not object) {
			return new NotEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseOr(fr.irisa.espresso.sme.Or)
		 * @generated
		 */
		public Object caseOr(fr.irisa.espresso.sme.Or object) {
			return new OrEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseXor(fr.irisa.espresso.sme.Xor)
		 * @generated
		 */
		public Object caseXor(fr.irisa.espresso.sme.Xor object) {
			return new XorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArithmeticOperator(fr.irisa.espresso.sme.ArithmeticOperator)
		 * @generated
		 */
		public Object caseArithmeticOperator(fr.irisa.espresso.sme.ArithmeticOperator object) {
			return new ArithmeticOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseArrayProduct(fr.irisa.espresso.sme.ArrayProduct)
		 * @generated
		 */
		public Object caseArrayProduct(fr.irisa.espresso.sme.ArrayProduct object) {
			return new ArrayProductEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseComplexOperator(fr.irisa.espresso.sme.ComplexOperator)
		 * @generated
		 */
		public Object caseComplexOperator(fr.irisa.espresso.sme.ComplexOperator object) {
			return new ComplexOperatorEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseNumericExpression(fr.irisa.espresso.sme.NumericExpression)
		 * @generated
		 */
		public Object caseNumericExpression(fr.irisa.espresso.sme.NumericExpression object) {
			return new NumericExpressionEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseUnaryMinus(fr.irisa.espresso.sme.UnaryMinus)
		 * @generated
		 */
		public Object caseUnaryMinus(fr.irisa.espresso.sme.UnaryMinus object) {
			return new UnaryMinusEditPart(node);
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object) {
			return new EMFGraphNodeEditPart(node);
		}
	}
	
	/**
	 * @generated
	 */
	private class EdgeSmeSwitch extends SmeSwitch<Object> {
		/**
		 * The graphical edge
		 * 
		 * @generated
		 */
		private GraphEdge	edge;
		
		/**
		 * Constructor
		 * 
		 * @param edge
		 *        the graphical edge
		 * @generated
		 */
		public EdgeSmeSwitch(GraphEdge edge) {
			this.edge = edge;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCaseConnection(fr.irisa.espresso.sme.CaseConnection)
		 * @generated
		 */
		public Object caseCaseConnection(fr.irisa.espresso.sme.CaseConnection object) {
			return new CaseConnectionEditPart(edge);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseCondition(fr.irisa.espresso.sme.Condition)
		 * @generated
		 */
		public Object caseCondition(fr.irisa.espresso.sme.Condition object) {
			return new ConditionEditPart(edge);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseConstantAffectation(fr.irisa.espresso.sme.ConstantAffectation)
		 * @generated
		 */
		public Object caseConstantAffectation(fr.irisa.espresso.sme.ConstantAffectation object) {
			return new ConstantAffectationEditPart(edge);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseDataflowConnection(fr.irisa.espresso.sme.DataflowConnection)
		 * @generated
		 */
		public Object caseDataflowConnection(fr.irisa.espresso.sme.DataflowConnection object) {
			return new DataflowConnectionEditPart(edge);
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object) {
			return new EMFGraphEdgeEditPart(edge);
		}
	}
	
}
