/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteStack;
import org.eclipse.gef.requests.CreationFactory;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

import fr.irisa.espresso.sme.ArithmeticOpKind;
import fr.irisa.espresso.sme.ArithmeticOperator;
import fr.irisa.espresso.sme.ComparisonOpKind;
import fr.irisa.espresso.sme.ComparisonOperator;
import fr.irisa.espresso.sme.Counter;
import fr.irisa.espresso.sme.CounterKind;
import fr.irisa.espresso.sme.ModelDeclaration;
import fr.irisa.espresso.sme.ModelKind;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.common.SmeImageRegistry;

/*
 * Generated Palette Manager
 * 
 * @generated
 */
public class DF_PaletteManager extends ModelerPaletteManager {
	// declare all the palette categories of the diagram
	/**
	 * @generated
	 */
	private PaletteDrawer	containersDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	identifiersDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	operatorsDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	linksDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	miscellaneousDrawer;
	
	/**
	 * @generated
	 */
	private ICreationUtils	creationUtils;
	
	/**
	 * The Constructor
	 * 
	 * @param utils
	 *        the creation utils for the tools of the palette
	 * @generated
	 */
	public DF_PaletteManager(ICreationUtils utils) {
		super();
		this.creationUtils = utils;
	}
	
	/**
	 * Creates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void createCategories() {
		createContainersDrawer();
		createIdentifiersDrawer();
		createOperatorsDrawer();
		createLinksDrawer();
		createMiscellaneousDrawer();
	}
	
	/**
	 * Updates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void updateCategories() {
		// deletion of the existing categories and creation of the updated categories
		
		getRoot().remove(containersDrawer);
		createContainersDrawer();
		
		getRoot().remove(identifiersDrawer);
		createIdentifiersDrawer();
		
		getRoot().remove(operatorsDrawer);
		createOperatorsDrawer();
		
		getRoot().remove(linksDrawer);
		createLinksDrawer();
		
		getRoot().remove(miscellaneousDrawer);
		createMiscellaneousDrawer();
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createContainersDrawer() {
		containersDrawer = new PaletteDrawer("Containers", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		ModelerCreationToolEntry objectTool;
		PaletteStack stack;
		
		stack = new PaletteStack("Loop construction", "Loop Construction Selection", SmeImageRegistry
				.getImageDescriptor("BASICITERATE"));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getBasicIterate(), "default");
		objectTool = new ModelerCreationToolEntry("Basic Iteration", "Basic Iteration", factory, SmeImageRegistry
				.getImageDescriptor("BASICITERATE"), SmeImageRegistry.getImageDescriptor("BASICITERATE_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getIterate(), "default");
		objectTool = new ModelerCreationToolEntry("Iteration", "Iteration", factory, SmeImageRegistry
				.getImageDescriptor("ITERATE"), SmeImageRegistry.getImageDescriptor("ITERATE_LARGE"));
		stack.add(objectTool);
		entries.add(stack);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getAutomaton(), "default");
		entries.add(new ModelerCreationToolEntry("Mode automaton", "Mode automaton", factory, SmeImageRegistry
				.getImageDescriptor("AUTOMATON"), SmeImageRegistry.getImageDescriptor("AUTOMATON_LARGE")));
		
		stack = new PaletteStack("Model Declaration", "Model Declaration Selection", SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelDeclaration()) {
			public EObject getNewModelObject() {
				ModelDeclaration md = (ModelDeclaration) super.getNewModelObject();
				md.setModelType(ModelKind.ACTION_LITERAL);
				return md;
			}
		};
		objectTool = new ModelerCreationToolEntry("Action", "Action", factory, SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"), SmeImageRegistry.getImageDescriptor("MODELDECLARATION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelDeclaration()) {
			public EObject getNewModelObject() {
				ModelDeclaration md = (ModelDeclaration) super.getNewModelObject();
				md.setModelType(ModelKind.FUNCTION_LITERAL);
				return md;
			}
		};
		objectTool = new ModelerCreationToolEntry("Function", "Function", factory, SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"), SmeImageRegistry.getImageDescriptor("MODELDECLARATION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelDeclaration()) {
			public EObject getNewModelObject() {
				ModelDeclaration md = (ModelDeclaration) super.getNewModelObject();
				md.setModelType(ModelKind.NODE_LITERAL);
				return md;
			}
		};
		objectTool = new ModelerCreationToolEntry("Node", "Node", factory, SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"), SmeImageRegistry.getImageDescriptor("MODELDECLARATION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelDeclaration()) {
			public EObject getNewModelObject() {
				ModelDeclaration md = (ModelDeclaration) super.getNewModelObject();
				md.setModelType(ModelKind.PROCESS_LITERAL);
				return md;
			}
		};
		objectTool = new ModelerCreationToolEntry("Process", "Process", factory, SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"), SmeImageRegistry.getImageDescriptor("MODELDECLARATION_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		entries.add(stack);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelRef(), "default");
		entries.add(new ModelerCreationToolEntry("Model reference", "Model reference", factory, SmeImageRegistry
				.getImageDescriptor("MODELREF"), SmeImageRegistry.getImageDescriptor("MODELREF_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelInstance(), "default");
		entries.add(new ModelerCreationToolEntry("Model instance", "Model instance", factory, SmeImageRegistry
				.getImageDescriptor("MODELINSTANCE"), SmeImageRegistry.getImageDescriptor("MODELINSTANCE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getSubProcess(), "default");
		entries.add(new ModelerCreationToolEntry("SubProcess", "SubProcess", factory, SmeImageRegistry
				.getImageDescriptor("SUBPROCESS"), SmeImageRegistry.getImageDescriptor("SUBPROCESS_LARGE")));
		
		stack = new PaletteStack("Type Definition", "Type Definition Selection", null);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getEnumerationType(), "default");
		objectTool = new ModelerCreationToolEntry("Enumeration", "Enumeration", factory, SmeImageRegistry
				.getImageDescriptor("ENUMERATIONTYPE"), SmeImageRegistry.getImageDescriptor("ENUMERATIONTYPE_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelType(), "default");
		stack.add(new ModelerCreationToolEntry("Model type", "Model type", factory, SmeImageRegistry
				.getImageDescriptor("MODELTYPE"), SmeImageRegistry.getImageDescriptor("MODELTYPE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getSubstitutionType(), "default");
		stack
				.add(new ModelerCreationToolEntry("Substitution type", "Substitution type", factory, SmeImageRegistry
						.getImageDescriptor("SUBSTITUTIONTYPE"), SmeImageRegistry
						.getImageDescriptor("SUBSTITUTIONTYPE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getTupleType(), "default");
		stack.add(new ModelerCreationToolEntry("Tuple kind type", "Tuple kind type", factory, SmeImageRegistry
				.getImageDescriptor("TUPLETYPE"), SmeImageRegistry.getImageDescriptor("TUPLETYPE_LARGE")));
		entries.add(stack);
		
		containersDrawer.addAll(entries);
		getRoot().add(containersDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createIdentifiersDrawer() {
		identifiersDrawer = new PaletteDrawer("Identifiers", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getConstantRef(), "default");
		entries.add(new ModelerCreationToolEntry("Constant reference", "Constant reference", factory, SmeImageRegistry
				.getImageDescriptor("CONSTANTREF"), SmeImageRegistry.getImageDescriptor("CONSTANTREF_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getConstantValue(), "default");
		entries.add(new ModelerCreationToolEntry("Constant value", "Constant value", factory, SmeImageRegistry
				.getImageDescriptor("CONSTANTVALUE"), SmeImageRegistry.getImageDescriptor("CONSTANTVALUE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getIndex(), "default");
		entries.add(new ModelerCreationToolEntry("Index", "Index", factory, SmeImageRegistry
				.getImageDescriptor("INDEX"), SmeImageRegistry.getImageDescriptor("INDEX_LARGE")));
		
		PaletteStack stack = new PaletteStack("Literal Construction", "Literal Construction Selection",
			SmeImageRegistry.getImageDescriptor("LITERAL"));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getLiteral(), "default");
		ModelerCreationToolEntry objectTool = new ModelerCreationToolEntry("Literal", "Literal", factory,
			SmeImageRegistry.getImageDescriptor("LITERAL"), SmeImageRegistry.getImageDescriptor("LITERAL_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getEnumerationValue(), "default");
		stack
				.add(new ModelerCreationToolEntry("Enumeration Value", "Enumeration Value", factory, SmeImageRegistry
						.getImageDescriptor("ENUMERATIONVALUE"), SmeImageRegistry
						.getImageDescriptor("ENUMERATIONVALUE_LARGE")));
		entries.add(stack);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getLocal(), "default");
		entries.add(new ModelerCreationToolEntry("Local signal", "Local signal", factory, SmeImageRegistry
				.getImageDescriptor("LOCAL"), SmeImageRegistry.getImageDescriptor("LOCAL_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getSignalRef(), "default");
		entries.add(new ModelerCreationToolEntry("Signal reference", "Signal reference", factory, SmeImageRegistry
				.getImageDescriptor("SIGNALREF"), SmeImageRegistry.getImageDescriptor("SIGNALREF_LARGE")));
		
		identifiersDrawer.addAll(entries);
		getRoot().add(identifiersDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createOperatorsDrawer() {
		operatorsDrawer = new PaletteDrawer("Operators", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		ModelerCreationToolEntry objectTool;
		PaletteStack stack;
		
		/** Arithmetic operators */
		stack = new PaletteStack("Arithmetic Operator", "Arithmetic Operator Selection", SmeImageRegistry
				.getImageDescriptor("ARITHMETICOPERATOR"));
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArithmeticOperator()) {
			public EObject getNewModelObject() {
				ArithmeticOperator ao = (ArithmeticOperator) super.getNewModelObject();
				ao.setOperator(ArithmeticOpKind.ADDITION_LITERAL);
				return ao;
			}
		};
		objectTool = new ModelerCreationToolEntry("Addition", "Addition", factory, SmeImageRegistry
				.getImageDescriptor("ADDITION"), SmeImageRegistry.getImageDescriptor("ADDITION_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArithmeticOperator()) {
			public EObject getNewModelObject() {
				ArithmeticOperator ao = (ArithmeticOperator) super.getNewModelObject();
				ao.setOperator(ArithmeticOpKind.SUBSTRACTION_LITERAL);
				return ao;
			}
		};
		stack.add(new ModelerCreationToolEntry("Substraction", "Substraction", factory, SmeImageRegistry
				.getImageDescriptor("SUBSTRACTION"), SmeImageRegistry.getImageDescriptor("SUBSTRACTION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArithmeticOperator()) {
			public EObject getNewModelObject() {
				ArithmeticOperator ao = (ArithmeticOperator) super.getNewModelObject();
				ao.setOperator(ArithmeticOpKind.MULTIPLICATION_LITERAL);
				return ao;
			}
		};
		stack.add(new ModelerCreationToolEntry("Multiplication", "Multiplication", factory, SmeImageRegistry
				.getImageDescriptor("MULTIPLICATION"), SmeImageRegistry.getImageDescriptor("MULTIPLICATION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArithmeticOperator()) {
			public EObject getNewModelObject() {
				ArithmeticOperator ao = (ArithmeticOperator) super.getNewModelObject();
				ao.setOperator(ArithmeticOpKind.DIVISION_LITERAL);
				return ao;
			}
		};
		stack.add(new ModelerCreationToolEntry("Division", "Division", factory, SmeImageRegistry
				.getImageDescriptor("DIVISION"), SmeImageRegistry.getImageDescriptor("DIVISION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArithmeticOperator()) {
			public EObject getNewModelObject() {
				ArithmeticOperator ao = (ArithmeticOperator) super.getNewModelObject();
				ao.setOperator(ArithmeticOpKind.MODULO_LITERAL);
				return ao;
			}
		};
		stack.add(new ModelerCreationToolEntry("Modulo", "Modulo", factory, SmeImageRegistry
				.getImageDescriptor("MODULO"), SmeImageRegistry.getImageDescriptor("MODULO_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArithmeticOperator()) {
			public EObject getNewModelObject() {
				ArithmeticOperator ao = (ArithmeticOperator) super.getNewModelObject();
				ao.setOperator(ArithmeticOpKind.POWER_LITERAL);
				return ao;
			}
		};
		stack.add(new ModelerCreationToolEntry("Power", "Power", factory, SmeImageRegistry.getImageDescriptor("POWER"),
			SmeImageRegistry.getImageDescriptor("POWER_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComplexOperator(), "default");
		stack.add(new ModelerCreationToolEntry("Complex operator", "Complex operator", factory, SmeImageRegistry
				.getImageDescriptor("COMPLEXOPERATOR"), SmeImageRegistry.getImageDescriptor("COMPLEXOPERATOR_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getNumericExpression(), "default");
		stack.add(new ModelerCreationToolEntry("Composite numeric expression", "Composite numeric expression", factory,
			SmeImageRegistry.getImageDescriptor("NUMERICEXPRESSION"), SmeImageRegistry
					.getImageDescriptor("NUMERICEXPRESSION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getUnaryMinus(), "default");
		stack.add(new ModelerCreationToolEntry("Unary Minus", "Unary Minus", factory, SmeImageRegistry
				.getImageDescriptor("UNARYMINUS"), SmeImageRegistry.getImageDescriptor("UNARYMINUS_LARGE")));
		
		entries.add(stack);
		
		/** Boolean operators */
		stack = new PaletteStack("Boolean Operator", "Boolean Operator Selection", SmeImageRegistry
				.getImageDescriptor("BOOLEANOPERATOR"));
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComparisonOperator()) {
			public EObject getNewModelObject() {
				ComparisonOperator co = (ComparisonOperator) super.getNewModelObject();
				co.setOperator(ComparisonOpKind.EQUAL_LITERAL);
				return co;
			}
		};
		objectTool = new ModelerCreationToolEntry("Equal", "Equal", factory, SmeImageRegistry
				.getImageDescriptor("EQUAL"), SmeImageRegistry.getImageDescriptor("EQUAL_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComparisonOperator()) {
			public EObject getNewModelObject() {
				ComparisonOperator co = (ComparisonOperator) super.getNewModelObject();
				co.setOperator(ComparisonOpKind.EQUAL_ANY_LITERAL);
				return co;
			}
		};
		stack.add(new ModelerCreationToolEntry("Equal Any", "Equal Any", factory, SmeImageRegistry
				.getImageDescriptor("EQUAL_ANY"), SmeImageRegistry.getImageDescriptor("EQUAL_ANY_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComparisonOperator()) {
			public EObject getNewModelObject() {
				ComparisonOperator co = (ComparisonOperator) super.getNewModelObject();
				co.setOperator(ComparisonOpKind.NOT_EQUAL_LITERAL);
				return co;
			}
		};
		stack.add(new ModelerCreationToolEntry("Not Equal", "Not Equal", factory, SmeImageRegistry
				.getImageDescriptor("NOT_EQUAL"), SmeImageRegistry.getImageDescriptor("NOT_EQUAL_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComparisonOperator()) {
			public EObject getNewModelObject() {
				ComparisonOperator co = (ComparisonOperator) super.getNewModelObject();
				co.setOperator(ComparisonOpKind.SMALLER_LITERAL);
				return co;
			}
		};
		stack.add(new ModelerCreationToolEntry("Smaller", "Smaller", factory, SmeImageRegistry
				.getImageDescriptor("SMALLER"), SmeImageRegistry.getImageDescriptor("SMALLER_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComparisonOperator()) {
			public EObject getNewModelObject() {
				ComparisonOperator co = (ComparisonOperator) super.getNewModelObject();
				co.setOperator(ComparisonOpKind.SMALLER_OR_EQUAL_LITERAL);
				return co;
			}
		};
		stack
				.add(new ModelerCreationToolEntry("Smaller or Equal", "Smaller or Equal", factory, SmeImageRegistry
						.getImageDescriptor("SMALLER_OR_EQUAL"), SmeImageRegistry
						.getImageDescriptor("SMALLER_OR_EQUAL_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComparisonOperator()) {
			public EObject getNewModelObject() {
				ComparisonOperator co = (ComparisonOperator) super.getNewModelObject();
				co.setOperator(ComparisonOpKind.SMALLER_OR_EQUAL_ANY_LITERAL);
				return co;
			}
		};
		stack.add(new ModelerCreationToolEntry("Smaller or Equal Any", "Smaller or Equal Any", factory,
			SmeImageRegistry.getImageDescriptor("SMALLER_OR_EQUAL_ANY"), SmeImageRegistry
					.getImageDescriptor("SMALLER_OR_EQUAL_ANY_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComparisonOperator()) {
			public EObject getNewModelObject() {
				ComparisonOperator co = (ComparisonOperator) super.getNewModelObject();
				co.setOperator(ComparisonOpKind.GREATER_LITERAL);
				return co;
			}
		};
		stack.add(new ModelerCreationToolEntry("Greater", "Greater", factory, SmeImageRegistry
				.getImageDescriptor("GREATER"), SmeImageRegistry.getImageDescriptor("GREATER_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getComparisonOperator()) {
			public EObject getNewModelObject() {
				ComparisonOperator co = (ComparisonOperator) super.getNewModelObject();
				co.setOperator(ComparisonOpKind.GREATER_OR_EQUAL_LITERAL);
				return co;
			}
		};
		stack
				.add(new ModelerCreationToolEntry("Greater or Equal", "Greater or Equal", factory, SmeImageRegistry
						.getImageDescriptor("GREATER_OR_EQUAL"), SmeImageRegistry
						.getImageDescriptor("GREATER_OR_EQUAL_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getBooleanExpression(), "default");
		stack.add(new ModelerCreationToolEntry("Composite boolean expression", "Composite boolean expression", factory,
			SmeImageRegistry.getImageDescriptor("BOOLEANEXPRESSION"), SmeImageRegistry
					.getImageDescriptor("BOOLEANEXPRESSION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getAnd(), "default");
		stack.add(new ModelerCreationToolEntry("Logical gate AND", "Logical gate AND", factory, SmeImageRegistry
				.getImageDescriptor("AND"), SmeImageRegistry.getImageDescriptor("AND_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getNot(), "default");
		stack.add(new ModelerCreationToolEntry("Logical gate NOT", "Logical gate NOT", factory, SmeImageRegistry
				.getImageDescriptor("NOT"), SmeImageRegistry.getImageDescriptor("NOT_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getOr(), "default");
		stack.add(new ModelerCreationToolEntry("Logical gate OR", "Logical gate OR", factory, SmeImageRegistry
				.getImageDescriptor("OR"), SmeImageRegistry.getImageDescriptor("OR_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getXor(), "default");
		stack.add(new ModelerCreationToolEntry("Logical gate XOR", "Logical gate XOR", factory, SmeImageRegistry
				.getImageDescriptor("XOR"), SmeImageRegistry.getImageDescriptor("XOR_LARGE")));
		entries.add(stack);
		
		/** Array Operators */
		stack = new PaletteStack("Array Operator", "Array Operator Selection", null);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArrayRecovery(), "default");
		stack.add(new ModelerCreationToolEntry("Array Recovery", "Array Recovery", factory, SmeImageRegistry
				.getImageDescriptor("ARRAYRECOVERY"), SmeImageRegistry.getImageDescriptor("ARRAYRECOVERY_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getCartesianProduct(), "default");
		stack
				.add(new ModelerCreationToolEntry("Cartesian product", "Cartesian product", factory, SmeImageRegistry
						.getImageDescriptor("CARTESIANPRODUCT"), SmeImageRegistry
						.getImageDescriptor("CARTESIANPRODUCT_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getConcatenation(), "default");
		stack.add(new ModelerCreationToolEntry("Concatenation", "Concatenation", factory, SmeImageRegistry
				.getImageDescriptor("CONCATENATION"), SmeImageRegistry.getImageDescriptor("CONCATENATION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getTransposition(), "default");
		stack.add(new ModelerCreationToolEntry("Matrix transposition", "Matrix transposition", factory,
			SmeImageRegistry.getImageDescriptor("TRANSPOSITION"), SmeImageRegistry
					.getImageDescriptor("TRANSPOSITION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArrayProduct(), "default");
		stack.add(new ModelerCreationToolEntry("Product of arrays", "Product of arrays", factory, SmeImageRegistry
				.getImageDescriptor("ARRAYPRODUCT"), SmeImageRegistry.getImageDescriptor("ARRAYPRODUCT_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getRepetitionOperator(),
			"default");
		stack.add(new ModelerCreationToolEntry("Repetition", "Repetition", factory, SmeImageRegistry
				.getImageDescriptor("REPETITIONOPERATOR"), SmeImageRegistry
				.getImageDescriptor("REPETITIONOPERATOR_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArrayRestructuration(),
			"default");
		stack.add(new ModelerCreationToolEntry("Restructuration", "Restructuration", factory, SmeImageRegistry
				.getImageDescriptor("ARRAYRESTRUCTURATION"), SmeImageRegistry
				.getImageDescriptor("ARRAYRESTRUCTURATION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getSequentialDefinition(),
			"default");
		stack.add(new ModelerCreationToolEntry("Sequential definition", "Sequential definition", factory,
			SmeImageRegistry.getImageDescriptor("SEQUENTIALDEFINITION"), SmeImageRegistry
					.getImageDescriptor("SEQUENTIALDEFINITION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getArrayEnumeration(), "default");
		stack
				.add(new ModelerCreationToolEntry("Value enumeration", "Value enumeration", factory, SmeImageRegistry
						.getImageDescriptor("ARRAYENUMERATION"), SmeImageRegistry
						.getImageDescriptor("ARRAYENUMERATION_LARGE")));
		entries.add(stack);
		
		/** Specific operators */
		stack = new PaletteStack("Signal Specific Operator", "Signal Specific Operator Selection", null);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getCounter()) {
			public EObject getNewModelObject() {
				Counter c = (Counter) super.getNewModelObject();
				c.setCounterKind(CounterKind.FROM);
				return c;
			}
		};
		stack.add(new ModelerCreationToolEntry("Count from", "Count from", factory, SmeImageRegistry
				.getImageDescriptor("COUNTER_FROM"), SmeImageRegistry.getImageDescriptor("COUNTER_FROM_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getCounter()) {
			public EObject getNewModelObject() {
				Counter c = (Counter) super.getNewModelObject();
				c.setCounterKind(CounterKind.AFTER);
				return c;
			}
		};
		stack.add(new ModelerCreationToolEntry("Count after", "Count after", factory, SmeImageRegistry
				.getImageDescriptor("COUNTER_AFTER"), SmeImageRegistry.getImageDescriptor("COUNTER_AFTER_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getCounter()) {
			public EObject getNewModelObject() {
				Counter c = (Counter) super.getNewModelObject();
				c.setCounterKind(CounterKind.COUNT);
				return c;
			}
		};
		stack.add(new ModelerCreationToolEntry("Count Modulo", "Count Modulo", factory, SmeImageRegistry
				.getImageDescriptor("COUNTER_COUNT"), SmeImageRegistry.getImageDescriptor("COUNTER_COUNT_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getDelay(), "default");
		objectTool = new ModelerCreationToolEntry("Delay", "Delay", factory, SmeImageRegistry
				.getImageDescriptor("DELAY"), SmeImageRegistry.getImageDescriptor("DELAY_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getExtraction(), "default");
		stack.add(new ModelerCreationToolEntry("Down sampling", "Down sampling", factory, SmeImageRegistry
				.getImageDescriptor("EXTRACTION"), SmeImageRegistry.getImageDescriptor("EXTRACTION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getIfThenElse(), "default");
		stack.add(new ModelerCreationToolEntry("IfThenElse", "IfThenElse", factory, SmeImageRegistry
				.getImageDescriptor("IFTHENELSE"), SmeImageRegistry.getImageDescriptor("IFTHENELSE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getCell(), "default");
		stack.add(new ModelerCreationToolEntry("Memorization", "Memorization", factory, SmeImageRegistry
				.getImageDescriptor("CELL"), SmeImageRegistry.getImageDescriptor("CELL_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getMerge(), "default");
		stack.add(new ModelerCreationToolEntry("Merging", "Merging", factory, SmeImageRegistry
				.getImageDescriptor("MERGE"), SmeImageRegistry.getImageDescriptor("MERGE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getWindow(), "default");
		stack.add(new ModelerCreationToolEntry("Sliding window", "Sliding window", factory, SmeImageRegistry
				.getImageDescriptor("WINDOW"), SmeImageRegistry.getImageDescriptor("WINDOW_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getSwitch(), "default");
		stack.add(new ModelerCreationToolEntry("Swtich case", "Swtich case", factory, SmeImageRegistry
				.getImageDescriptor("SWITCH"), SmeImageRegistry.getImageDescriptor("SWITCH_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getTupleOperator(), "default");
		stack.add(new ModelerCreationToolEntry("Tuple operator", "Tuple operator", factory, SmeImageRegistry
				.getImageDescriptor("TUPLEOPERATOR"), SmeImageRegistry.getImageDescriptor("TUPLEOPERATOR_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getVariable(), "default");
		stack.add(new ModelerCreationToolEntry("Variable clock signal", "Variable clock signal", factory,
			SmeImageRegistry.getImageDescriptor("VARIABLE"), SmeImageRegistry.getImageDescriptor("VARIABLE_LARGE")));
		entries.add(stack);
		
		operatorsDrawer.addAll(entries);
		getRoot().add(operatorsDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createLinksDrawer() {
		linksDrawer = new PaletteDrawer("Links", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getCaseConnection(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("Case connector", "Case connector", factory,
			SmeImageRegistry.getImageDescriptor("CASECONNECTION"), SmeImageRegistry
					.getImageDescriptor("CASECONNECTION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getCondition(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("Condition connector", "Condition connector", factory,
			SmeImageRegistry.getImageDescriptor("CONDITION"), SmeImageRegistry.getImageDescriptor("CONDITION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getConstantAffectation(),
			"default");
		entries.add(new ModelerConnectionCreationToolEntry("Constant affectation", "Constant affectation", factory,
			SmeImageRegistry.getImageDescriptor("CONSTANTAFFECTATION"), SmeImageRegistry
					.getImageDescriptor("CONSTANTAFFECTATION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getDataflowConnection(),
			"default");
		entries.add(new ModelerConnectionCreationToolEntry("Dataflow Connector", "Dataflow Connector", factory,
			SmeImageRegistry.getImageDescriptor("DATAFLOWCONNECTION"), SmeImageRegistry
					.getImageDescriptor("DATAFLOWCONNECTION_LARGE")));
		
		linksDrawer.addAll(entries);
		getRoot().add(linksDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createMiscellaneousDrawer() {
		miscellaneousDrawer = new PaletteDrawer("Miscellaneous", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getAssertion(), "default");
		entries.add(new ModelerCreationToolEntry("Assertion", "Assertion", factory, SmeImageRegistry
				.getImageDescriptor("ASSERTION"), SmeImageRegistry.getImageDescriptor("ASSERTION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getUseModule(), "default");
		entries.add(new ModelerCreationToolEntry("Import modules", "Import modules", factory, SmeImageRegistry
				.getImageDescriptor("USEMODULE"), SmeImageRegistry.getImageDescriptor("USEMODULE_LARGE")));
		
		miscellaneousDrawer.addAll(entries);
		getRoot().add(miscellaneousDrawer);
	}
	
}
