/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.commands;

import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionnedExpr;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.ModelConnection;
import fr.irisa.espresso.sme.SignalExpr;
import fr.irisa.espresso.sme.SignalOrRef;

/*
 * SignalOrRef restore connection command
 * 
 * @generated
 */
public class SignalOrRefRestoreConnectionCommand extends SignalExprRestoreConnectionCommand {
	/**
	 * @param part
	 *        the EditPart that is restored
	 * @generated
	 */
	public SignalOrRefRestoreConnectionCommand(EditPart part) {
		super(part);
	}
	
	/**
	 * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
	 * @generated NOT
	 */
	protected void initializeCommands() {
		super.initializeCommands();
		
		GraphElement graphElementSrc = getGraphElement();
		EObject eObjectSrc = Utils.getElement(graphElementSrc);
		
		if (eObjectSrc instanceof SignalOrRef) {
			for (GraphElement graphElementTgt : getAllGraphElements()) {
				boolean autoRef = graphElementTgt.equals(graphElementSrc);
				
				EObject eObjectTgt = Utils.getElement(graphElementTgt);
				
				if (eObjectTgt instanceof ConditionnedExpr) {
					if (!autoRef) {
						// if the graphElementSrc is the source of the edge or if it is the target and that the
						// SourceTargetCouple is reversible
						createConditionFromSignalOrRefToConditionnedExpr_Dst_Condition(graphElementSrc, graphElementTgt);
					}
				}
				
				if (eObjectTgt instanceof SignalExpr) {
					if (!autoRef) {
						// if graphElementSrc is the target of the edge or if it is the source and that the
						// SourceTargetCouple is reversible
						createDataflowConnectionFromSignalExprToSignalOrRef_Dst_DataflowConnections(graphElementTgt,
							graphElementSrc);
					}
				}
			}
		}
	}
	
	/**
	 * @param srcElt
	 *        the source element
	 * @param targetElt
	 *        the target element
	 * @generated NOT
	 */
	private void createConditionFromSignalOrRefToConditionnedExpr_Dst_Condition(GraphElement srcElt,
			GraphElement targetElt) {
		SignalOrRef sourceObject = (SignalOrRef) Utils.getElement(srcElt);
		ConditionnedExpr targetObject = (ConditionnedExpr) Utils.getElement(targetElt);
		
		EList<ModelConnection> edgeObjectList = ((fr.irisa.espresso.sme.Model) Utils.getDiagramModelObject(srcElt))
				.getConnections();
		for (ModelConnection obj : edgeObjectList) {
			if (obj instanceof Condition) {
				Condition edgeObject = (Condition) obj;
				if (targetObject.equals(edgeObject.getDst()) && sourceObject.equals(edgeObject.getSrc())
						&& sourceObject.getSrc_Conditions().contains(edgeObject)
						&& targetObject.getDst_Condition().contains(edgeObject)) {
					// check if the relation does not exists yet
					List<GraphEdge> existing = getExistingEdges(srcElt, targetElt, Condition.class);
					if (!isAlreadyPresent(existing, edgeObject)) {
						ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
						// restore the link with its default presentation
						GraphElement edge = factory.createGraphElement(edgeObject);
						if (edge instanceof GraphEdge) {
							ConditionEdgeCreationCommand cmd = new ConditionEdgeCreationCommand(getEditDomain(),
								(GraphEdge) edge, srcElt, false);
							cmd.setTarget(targetElt);
							add(cmd);
						}
					}
				}
			}
		}
	}
	
	/**
	 * @param srcElt
	 *        the source element
	 * @param targetElt
	 *        the target element
	 * @generated NOT
	 */
	private void createDataflowConnectionFromSignalExprToSignalOrRef_Dst_DataflowConnections(GraphElement srcElt,
			GraphElement targetElt) {
		SignalExpr sourceObject = (SignalExpr) Utils.getElement(srcElt);
		SignalOrRef targetObject = (SignalOrRef) Utils.getElement(targetElt);
		
		EList<ModelConnection> edgeObjectList = ((fr.irisa.espresso.sme.Model) Utils.getDiagramModelObject(srcElt))
				.getConnections();
		for (ModelConnection obj : edgeObjectList) {
			if (obj instanceof DataflowConnection) {
				DataflowConnection edgeObject = (DataflowConnection) obj;
				if (targetObject.equals(edgeObject.getDst()) && sourceObject.equals(edgeObject.getSrc())
						&& sourceObject.getSrc_DataflowConnections().contains(edgeObject)
						&& targetObject.getDst_DataflowConnections().contains(edgeObject)) {
					// check if the relation does not exists yet
					List<GraphEdge> existing = getExistingEdges(srcElt, targetElt, DataflowConnection.class);
					if (!isAlreadyPresent(existing, edgeObject)) {
						ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
						// restore the link with its default presentation
						GraphElement edge = factory.createGraphElement(edgeObject);
						if (edge instanceof GraphEdge) {
							DataflowConnectionEdgeCreationCommand cmd = new DataflowConnectionEdgeCreationCommand(
								getEditDomain(), (GraphEdge) edge, srcElt, false);
							cmd.setTarget(targetElt);
							add(cmd);
						}
					}
				}
			}
		}
	}
}
