/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.commands.AbstractRestoreConnectionCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ModelConnection;
import fr.irisa.espresso.sme.ParameterInstance;
import fr.irisa.espresso.sme.TupleType;

/*
 * TupleType restore connection command
 * 
 * @generated
 */
public class TupleTypeRestoreConnectionCommand extends AbstractRestoreConnectionCommand {
	/**
	 * @param part
	 *        the EditPart that is restored
	 * @generated
	 */
	public TupleTypeRestoreConnectionCommand(EditPart part) {
		super(part);
	}
	
	/**
	 * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
	 * @generated NOT
	 */
	protected void initializeCommands() {
		GraphElement elt = getGraphElement();
		EObject eltObject = Utils.getElement(elt);
		
		if (eltObject instanceof TupleType) {
			Iterator<EObject> itDiagContents = getModeler().getActiveDiagram().eAllContents();
			while (itDiagContents.hasNext()) {
				Object obj = itDiagContents.next();
				if (obj instanceof GraphElement
						&& DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null) {
					boolean autoRef = obj.equals(elt);
					GraphElement elt2 = (GraphElement) obj;
					EObject eltObject2 = Utils.getElement(elt2);
					
					if (eltObject2 instanceof ParameterInstance) {
						if (!autoRef) {
							// if the elt is the source of the edge or if it is the target and that the
							// SourceTargetCouple is reversible
							createConstantAffectationFromTupleTypeToParameterInstance_Dst_ConstantAffectation(elt, elt2);
						}
					}
					
				}
			}
		}
	}
	
	/**
	 * @param srcElt
	 *        the source element
	 * @param targetElt
	 *        the target element
	 * @generated NOT
	 */
	private void createConstantAffectationFromTupleTypeToParameterInstance_Dst_ConstantAffectation(GraphElement srcElt,
			GraphElement targetElt) {
		TupleType sourceObject = (TupleType) Utils.getElement(srcElt);
		ParameterInstance targetObject = (ParameterInstance) Utils.getElement(targetElt);
		
		EList<ModelConnection> edgeObjectList = ((fr.irisa.espresso.sme.Model) Utils.getDiagramModelObject(srcElt))
				.getConnections();
		for (ModelConnection obj : edgeObjectList) {
			if (obj instanceof ConstantAffectation) {
				ConstantAffectation edgeObject = (ConstantAffectation) obj;
				if (targetObject.equals(edgeObject.getDst()) && sourceObject.equals(edgeObject.getSrc())
						&& sourceObject.getSrc_ConstantAffectations().contains(edgeObject)
						&& edgeObject.equals(targetObject.getDst_ConstantAffectation())) {
					// check if the relation does not exists yet
					List<GraphEdge> existing = getExistingEdges(srcElt, targetElt, ConstantAffectation.class);
					if (!isAlreadyPresent(existing, edgeObject)) {
						ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
						// restore the link with its default presentation
						GraphElement edge = factory.createGraphElement(edgeObject);
						if (edge instanceof GraphEdge) {
							ConstantAffectationEdgeCreationCommand cmd = new ConstantAffectationEdgeCreationCommand(
								getEditDomain(), (GraphEdge) edge, srcElt, false);
							cmd.setTarget(targetElt);
							add(cmd);
						}
					}
				}
			}
		}
	}
	
}
