/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.SignalOperator;
import fr.irisa.espresso.sme.common.figures.SignalOperatorFigure;
import fr.irisa.espresso.sme.common.policies.MultipleInputsLayoutEditPolicy;
import fr.irisa.espresso.sme.diagram.dataflow.commands.CartesianProductRestoreConnectionCommand;
import fr.irisa.espresso.sme.diagram.dataflow.preferences.DF_DiagramPreferenceConstants;

/*
 * The CartesianProduct object controller
 * 
 * @generated
 */
public class CartesianProductEditPart extends EMFGraphNodeEditPart {
	/**
	 * Constructor
	 * 
	 * @param obj
	 *        the graph node
	 * @generated
	 */
	public CartesianProductEditPart(GraphNode obj) {
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy() {
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request) {
				return new CartesianProductRestoreConnectionCommand(getHost());
			}
		});
		
		ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
		resizableEditPolicy.setResizeDirections(PositionConstants.NORTH_SOUTH);
		installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);
		
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new MultipleInputsLayoutEditPolicy());
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated NOT
	 */
	protected IFigure createFigure() {
		return new SignalOperatorFigure("CARTESIANPRODUCT_LARGE");
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultBackgroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultBackgroundColor() {
		String backgroundColor = getPreferenceStore().getString(
			DF_DiagramPreferenceConstants.CARTESIANPRODUCT_DEFAULT_BACKGROUND_COLOR);
		if (backgroundColor.length() != 0) {
			return Utils.getColor(backgroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultForegroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String foregroundColor = getPreferenceStore().getString(
			DF_DiagramPreferenceConstants.CARTESIANPRODUCT_DEFAULT_FOREGROUND_COLOR);
		if (foregroundColor.length() != 0) {
			return Utils.getColor(foregroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultFont()
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(
			DF_DiagramPreferenceConstants.CARTESIANPRODUCT_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshConstraints()
	 * @generated NOT
	 */
	protected void refreshConstraints() {
		super.refreshConstraints();
		
		updatePorts();
	}
	
	/**
	 * Update the number of InputPorts
	 * 
	 * @generated NOT
	 */
	private void updatePorts() {
		int nbIns = ((SignalOperator) getEObject()).getInputPorts().size();
		int nbOuts = ((SignalOperator) getEObject()).getOutputPorts().size();
		
		SignalOperatorFigure fig = (SignalOperatorFigure) getFigure();
		if (fig.getNbInputPorts() != nbIns || fig.getNbOutputPorts() != nbOuts) {
			fig.setNbInputPorts(nbIns);
			fig.setNbOutputPorts(nbOuts);
			getGraphNode().setSize(fig.getPreferredSize());
			fig.repaint();
		}
		
		if (nbIns > 0 || nbOuts > 0) {
			// place the ports
			Dimension dim;
			if (getGraphNode().getSize() == null)
				dim = fig.getPreferredSize(-1, -1);
			else
				dim = getGraphNode().getSize();
			
			int nIn = 0, nOut = 0;
			int space = dim.height / Math.max(nbIns, nbOuts);
			int init = (space - SignalOperatorFigure.PORT_SIZE.height) / 2;
			int outX = dim.width - SignalOperatorFigure.PORT_SIZE.width;
			
			for (Object obj : getModelChildren()) {
				GraphElement elt = (GraphElement) obj;
				if (Utils.getElement(elt) instanceof InputPort) {
					elt.setPosition(new Point(0, init + (nIn * space)));
					nIn++;
				}
				else if (Utils.getElement(elt) instanceof OutputPort) {
					elt.setPosition(new Point(outX, init + (nOut * space)));
					nOut++;
				}
			}
		}
	}
}
