/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.edit;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.figures.PortAnchor;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Constant;
import fr.irisa.espresso.sme.ConstantRef;
import fr.irisa.espresso.sme.ConstantValue;
import fr.irisa.espresso.sme.common.ConstantKind;
import fr.irisa.espresso.sme.common.figures.ImageWithLabel;
import fr.irisa.espresso.sme.diagram.dataflow.DF_EditPolicyConstants;
import fr.irisa.espresso.sme.diagram.dataflow.commands.ConstantRefRestoreConnectionCommand;
import fr.irisa.espresso.sme.diagram.dataflow.figures.ConstantRefFigure;
import fr.irisa.espresso.sme.diagram.dataflow.policies.ConditionEdgeCreationEditPolicy;
import fr.irisa.espresso.sme.diagram.dataflow.preferences.DF_DiagramPreferenceConstants;

/*
 * The ConstantRef object controller
 * 
 * @generated
 */
public class ConstantRefEditPart extends ConstantExprEditPart {
	/**
	 * Constructor
	 * 
	 * @param obj
	 *        the graph node
	 * @generated
	 */
	public ConstantRefEditPart(GraphNode obj) {
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(DF_EditPolicyConstants.CONDITION_EDITPOLICY, new ConditionEdgeCreationEditPolicy());
		
		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy() {
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request) {
				return new ConstantRefRestoreConnectionCommand(getHost());
			}
		});
		
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated NOT
	 */
	protected IFigure createFigure() {
		IFigure fig = new ConstantRefFigure();
		ConstantRef cr = (ConstantRef) Utils.getElement((GraphElement) this.getModel());
		Label lbl = new Label(cr.toString(true));
		lbl.setParent(fig);
		fig.setToolTip(lbl);
		return fig;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultBackgroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultBackgroundColor() {
		String backgroundColor = getPreferenceStore().getString(
			DF_DiagramPreferenceConstants.CONSTANTREF_DEFAULT_BACKGROUND_COLOR);
		if (backgroundColor.length() != 0) {
			return Utils.getColor(backgroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultForegroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String foregroundColor = getPreferenceStore().getString(
			DF_DiagramPreferenceConstants.CONSTANTREF_DEFAULT_FOREGROUND_COLOR);
		if (foregroundColor.length() != 0) {
			return Utils.getColor(foregroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultFont()
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(DF_DiagramPreferenceConstants.CONSTANTREF_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
	 * @generated NOT
	 */
	protected void handleModelChanged(Notification msg) {
		if (msg.getNotifier() instanceof ConstantRef) {
			IFigure fig = getFigure();
			Label lbl = (Label) fig.getToolTip();
			ConstantRef cr = (ConstantRef) Utils.getElement((GraphElement) this.getModel());
			lbl.setText(cr.toString(true));
		}
		super.handleModelChanged(msg);
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
	 * @generated NOT
	 */
	protected void refreshVisuals() {
		super.refreshVisuals();
		
		updateConstantRefKind();
	}
	
	/**
	 * Update the kind of constant referenced by the ConstantRef object
	 * 
	 * @generated NOT
	 */
	protected void updateConstantRefKind() {
		Constant cst = ((ConstantRef) getEObject()).getConstant();
		ConstantKind kind;
		if (cst == null)
			kind = ConstantKind.NONE_LITERAL;
		else if (cst instanceof ConstantValue)
			kind = ConstantKind.CONSTANTVALUE_LITERAL;
		else
			// ConstantValue
			kind = ConstantKind.PARAMETER_LITERAL;
		
		ConstantRefFigure fig = (ConstantRefFigure) getFigure();
		if (fig.getConstantRefKind() != kind) {
			fig.setConstantRefKind(kind);
			fig.repaint();
		}
	}
	
	// -----------------------
	// GESTION DES ANCHORS
	// -----------------------
	
	/**
	 * Returns a new Anchor for the Figure
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 * @generated NOT
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return getPortConnectionAnchor();
	}
	
	/**
	 * Returns a new Anchor for the Figure
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return getPortConnectionAnchor();
	}
	
	/**
	 * Returns a new Anchor for the Figure
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 * @generated NOT
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return getPortConnectionAnchor();
	}
	
	/**
	 * Returns a new Anchor for the Figure
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return getPortConnectionAnchor();
	}
	
	/**
	 * This method return the Figure that correspond to the graphical representation of the port
	 * 
	 * @return the Figure that represent the port
	 * @generated NOT
	 */
	protected IFigure getPortFigure() {
		IFigure fig = getFigure();
		if (fig instanceof ImageWithLabel)
			return ((ImageWithLabel) fig).getFigure();
		else
			return fig;
	}
	
	/**
	 * Return the connection Anchor depending on the container of the Port
	 * 
	 * @return the ConnectionAnchor
	 * @generated NOT
	 */
	protected ConnectionAnchor getPortConnectionAnchor() {
		return new PortAnchor(getPortFigure(), true);
	}
}
