/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.edit;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerColorConstants;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.diagram.dataflow.DF_EdgeObjectConstants;
import fr.irisa.espresso.sme.diagram.dataflow.figures.DataflowConnectionFigure;
import fr.irisa.espresso.sme.diagram.dataflow.policies.DataflowConnectionEdgeObjectUVEditPolicy;
import fr.irisa.espresso.sme.diagram.dataflow.preferences.DF_DiagramPreferenceConstants;

/*
 * DataflowConnection controller
 * 
 * @generated
 */
public class DataflowConnectionEditPart extends EMFGraphEdgeEditPart {
	
	/**
	 * Constructor
	 * 
	 * @param model
	 *        the graph object
	 * @generated
	 */
	public DataflowConnectionEditPart(GraphEdge model) {
		super(model);
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 * @generated
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY,
			new DataflowConnectionEdgeObjectUVEditPolicy());
		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());
		
	}
	
	/**
	 * @return the Figure
	 * @generated NOT
	 */
	protected IFigure createFigure() {
		DataflowConnectionFigure connection = new DataflowConnectionFigure();
		
		// If the destination object is an InputPort, we don't add the decoration
		DataflowConnection dc = (DataflowConnection) Utils.getElement((GraphElement) getModel());
		if (!(dc.getDestination() instanceof InputPort))
			createTargetDecoration(connection);
		
		return connection;
	}
	
	/**
	 * @param connection
	 *        the PolylineConnection
	 * @generated NOT
	 */
	private void createTargetDecoration(PolylineConnection connection) {
		PolygonDecoration decoration = new PolygonDecoration();
		decoration.setScale(14, 6);
		decoration.setBackgroundColor(ModelerColorConstants.black);
		connection.setTargetDecoration(decoration);
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
	 * @generated
	 */
	public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject) {
		if (DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID.equals(edgeObject.getId())) {
			return ((DataflowConnectionFigure) getFigure()).getSrcFieldEdgeObjectFigure();
		}
		if (DF_EdgeObjectConstants.DSTFIELD_EDGE_OBJECT_ID.equals(edgeObject.getId())) {
			return ((DataflowConnectionFigure) getFigure()).getDstFieldEdgeObjectFigure();
		}
		if (DF_EdgeObjectConstants.CASTTYPE_EDGE_OBJECT_ID.equals(edgeObject.getId())) {
			return ((DataflowConnectionFigure) getFigure()).getCastTypeEdgeObjectFigure();
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
	 * @generated
	 */
	protected void refreshEdgeObjects() {
		super.refreshEdgeObjects();
		updateSrcFieldLabel();
		updateDstFieldLabel();
		updateCastTypeLabel();
	}
	
	/**
	 * Update the srcField Label
	 * 
	 * @generated
	 */
	private void updateSrcFieldLabel() {
		((Label) ((DataflowConnectionFigure) getFigure()).getSrcFieldEdgeObjectFigure())
				.setText(((DataflowConnection) getEObject()).getSrcField());
	}
	/**
	 * Update the dstField Label
	 * 
	 * @generated
	 */
	private void updateDstFieldLabel() {
		((Label) ((DataflowConnectionFigure) getFigure()).getDstFieldEdgeObjectFigure())
				.setText(((DataflowConnection) getEObject()).getDstField());
	}
	/**
	 * Update the castType Label
	 * 
	 * @generated NOT
	 */
	private void updateCastTypeLabel() {
		String castType = ((DataflowConnection) getEObject()).getCastType().getLiteral();
		if (castType != null)
			((Label) ((DataflowConnectionFigure) getFigure()).getCastTypeEdgeObjectFigure()).setText(castType
					.toString());
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultRouter()
	 * 
	 * @generated
	 */
	protected String getPreferenceDefaultRouter() {
		return getPreferenceStore().getString(DF_DiagramPreferenceConstants.DATAFLOWCONNECTION_EDGE_DEFAULT_ROUTER);
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultForegroundColor()
	 * 
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String preferenceForeground = getPreferenceStore().getString(
			DF_DiagramPreferenceConstants.DATAFLOWCONNECTION_EDGE_DEFAULT_FOREGROUND_COLOR);
		if (preferenceForeground.length() != 0) {
			return Utils.getColor(preferenceForeground);
		}
		return null;
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultFont()
	 * 
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(
			DF_DiagramPreferenceConstants.DATAFLOWCONNECTION_EDGE_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
	}
	/**
	 * @see org.topcased.modeler.edit.EMFGraphEdgeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
	 * @generated NOT
	 */
	protected void handleModelChanged(Notification msg) {
		if (msg.getNotifier() instanceof DataflowConnection) {
			switch (msg.getFeatureID(DataflowConnection.class)) {
				case SmePackage.DATAFLOW_CONNECTION__USE_SRC_CLOCK:
				case SmePackage.DATAFLOW_CONNECTION__IS_DEFAULT_VALUE:
					refreshVisuals();
					break;
			}
		}
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshColors()
	 * @generated NOT
	 */
	protected void refreshColors() {
		super.refreshColors();
		
		DataflowConnectionFigure fig = (DataflowConnectionFigure) this.getFigure();
		DataflowConnection def = (DataflowConnection) Utils.getElement((GraphElement) this.getModel());
		fig.setLineStyle(Graphics.LINE_SOLID);
		
		if (def.isUseSrcClock())
			fig.setForegroundColor(ModelerColorConstants.blue);
		else
			fig.setForegroundColor(ModelerColorConstants.black);
	}
}
