/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.edit;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.figures.PortAnchor;
import org.topcased.modeler.requests.RestoreConnectionsRequest;

import fr.irisa.espresso.sme.common.figures.ImageWithLabel;
import fr.irisa.espresso.sme.diagram.dataflow.DF_EditPolicyConstants;
import fr.irisa.espresso.sme.diagram.dataflow.commands.SignalOrRefRestoreConnectionCommand;
import fr.irisa.espresso.sme.diagram.dataflow.policies.ConditionEdgeCreationEditPolicy;
import fr.irisa.espresso.sme.diagram.dataflow.policies.DataflowConnectionEdgeCreationEditPolicy;

/*
 * The SignalOrRef object controller
 * 
 * @generated
 */
public class SignalOrRefEditPart extends SignalExprEditPart {
	/**
	 * Constructor
	 * 
	 * @param obj
	 *        the graph node
	 * @generated
	 */
	public SignalOrRefEditPart(GraphNode obj) {
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(DF_EditPolicyConstants.CONDITION_EDITPOLICY, new ConditionEdgeCreationEditPolicy());
		
		installEditPolicy(DF_EditPolicyConstants.DATAFLOWCONNECTION_EDITPOLICY,
			new DataflowConnectionEdgeCreationEditPolicy());
		
		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy() {
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request) {
				return new SignalOrRefRestoreConnectionCommand(getHost());
			}
		});
		
	}
	
	// -----------------------
	// GESTION DES ANCHORS
	// -----------------------
	
	/**
	 * Returns a new Anchor for the Figure
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 * @generated NOT
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return getPortConnectionAnchor();
	}
	
	/**
	 * Returns a new Anchor for the Figure
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return getPortConnectionAnchor();
	}
	
	/**
	 * Returns a new Anchor for the Figure
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 * @generated NOT
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return getPortConnectionAnchor();
	}
	
	/**
	 * Returns a new Anchor for the Figure
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return getPortConnectionAnchor();
	}
	
	/**
	 * This method return the Figure that correspond to the graphical representation of the port
	 * 
	 * @return the Figure that represent the port
	 * @generated NOT
	 */
	protected IFigure getPortFigure() {
		IFigure fig = getFigure();
		if (fig instanceof ImageWithLabel)
			return ((ImageWithLabel) fig).getFigure();
		else
			return fig;
	}
	
	/**
	 * Return the connection Anchor depending on the container of the Port
	 * 
	 * @return the ConnectionAnchor
	 * @generated NOT
	 */
	protected ConnectionAnchor getPortConnectionAnchor() {
		return new PortAnchor(getPortFigure(), true);
	}
}
