/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;
import org.topcased.modeler.ModelerColorConstants;

import fr.irisa.espresso.sme.common.SignalKind;
import fr.irisa.espresso.sme.common.figures.ImageFigure;
import fr.irisa.espresso.sme.common.figures.ImageWithLabel;

/*
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class SignalRefFigure extends ImageWithLabel {
	private final static String	REF_IMG		= "SIGNALREF_LARGE";
	
	private final static String	INPUT_IMG	= "INPUT_LARGE";
	
	private final static String	OUTPUT_IMG	= "OUTPUT_LARGE";
	
	private final static String	LOCAL_IMG	= "LOCAL_LARGE";
	
	private SignalKind			kind		= SignalKind.NONE_LITERAL;
	
	/**
	 * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalRefFigure() {
		super(REF_IMG);
	}
	
	/**
	 * @return the current kind of object referenced by the SignalRef
	 */
	public SignalKind getSignalRefKind() {
		return kind;
	}
	
	/**
	 * Creates the contentPane object.
	 * 
	 * @return the container figure
	 */
	protected IFigure createBody() {
		return new ImageFigure(REF_IMG) {
			/**
			 * Refresh the figure
			 * 
			 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
			 */
			protected void paintFigure(Graphics graphics) {
				super.paintFigure(graphics);
				
				graphics.pushState();
				
				Rectangle rect = getBounds();
				Color color = this.getForegroundColor();
				this.setForegroundColor(ModelerColorConstants.black);
				graphics.setLineWidth(2);
				graphics.drawRectangle(rect.x, rect.y, rect.width, rect.height);
				this.setForegroundColor(color);
				
				graphics.popState();
			}
			
			/**
			 * 
			 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
			 * @generated
			 */
			public Dimension getPreferredSize(int wHint, int hHint) {
				Dimension dim = super.getPreferredSize(wHint, hHint);
				dim.width += 4;
				dim.height += 4;
				return dim;
			}
		};
	}
	
	/**
	 * @param kind
	 *        The kind of the object referenced by the SignalRef
	 */
	public void setSignalRefKind(SignalKind kind) {
		this.kind = kind;
		
		switch (this.kind.getValue()) {
			case SignalKind.NONE:
				this.setImage(REF_IMG);
				break;
			case SignalKind.INPUT:
				this.setImage(INPUT_IMG);
				break;
			case SignalKind.OUTPUT:
				this.setImage(OUTPUT_IMG);
				break;
			case SignalKind.LOCAL:
				this.setImage(LOCAL_IMG);
				break;
		}
	}
}
