/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.Iterate;
import fr.irisa.espresso.sme.IterationInit;
import fr.irisa.espresso.sme.common.SmeRequestConstants;
import fr.irisa.espresso.sme.common.commands.AddNewPortCommand;
import fr.irisa.espresso.sme.common.commands.ChangeAspectCommand;

/*
 * @generated
 */
public class IterateLayoutEditPolicy extends org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy {
	/**
	 * Default contructor.
	 * 
	 * @generated
	 */
	public IterateLayoutEditPolicy() {
		super();
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject,
	 *      org.eclipse.emf.ecore.EObject)
	 * @generated
	 */
	protected boolean isValid(EObject child, EObject parent) {
		if (child instanceof IterationInit) {
			return true;
		}
		if (child instanceof InputPort) {
			return true;
		}
		return false;
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#getCommand(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public Command getCommand(Request request) {
		if (SmeRequestConstants.REQ_ADD_NEW_PORT.equals(request.getType())) {
			Modeler modeler = ((ModelerGraphicalViewer) getHost().getViewer()).getModelerEditor();
			if (getHost().getModel() instanceof GraphNode) {
				AddNewPortCommand command = new AddNewPortCommand((GraphNode) getHost().getModel(), modeler);
				return command;
			}
			return null;
		}
		else if (SmeRequestConstants.REQ_GO_TO_DATAFLOW_ASPECT.equals(request.getType())) {
			ModelerGraphicalViewer modeler = (ModelerGraphicalViewer) getHost().getViewer();
			GraphElement node = (GraphElement) getHost().getModel();
			ChangeAspectCommand command = new ChangeAspectCommand(node, modeler,
				"fr.irisa.espresso.sme.diagram.dataflow");
			return command;
		}
		else if (SmeRequestConstants.REQ_GO_TO_CLOCKRELATION_ASPECT.equals(request.getType())) {
			ModelerGraphicalViewer modeler = (ModelerGraphicalViewer) getHost().getViewer();
			GraphElement node = (GraphElement) getHost().getModel();
			ChangeAspectCommand command = new ChangeAspectCommand(node, modeler,
				"fr.irisa.espresso.sme.diagram.clockrelations");
			return command;
		}
		else if (SmeRequestConstants.REQ_SHOW_ITERATION_INIT.equals(request.getType())) {
			ModelerGraphicalViewer modeler = (ModelerGraphicalViewer) getHost().getViewer();
			GraphElement node = (GraphElement) getHost().getModel();
			EObject obj = Utils.getElement(node);
			if (obj instanceof Iterate) {
				ChangeAspectCommand command = new ChangeAspectCommand(((Iterate) obj).getIterationInit(), modeler,
					"fr.irisa.espresso.sme.diagram.dataflow");
				return command;
			}
			return null;
		}
		return super.getCommand(request);
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#understandsRequest(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public boolean understandsRequest(Request req) {
		EObject obj = Utils.getElement((GraphElement) getHost().getModel());
		if (SmeRequestConstants.REQ_ADD_NEW_PORT.equals(req.getType()))
			return (obj instanceof Iterate);
		else if (SmeRequestConstants.REQ_GO_TO_DATAFLOW_ASPECT.equals(req.getType()))
			return (obj instanceof Iterate);
		else if (SmeRequestConstants.REQ_GO_TO_CLOCKRELATION_ASPECT.equals(req.getType()))
			return (obj instanceof Iterate);
		else if (SmeRequestConstants.REQ_SHOW_ITERATION_INIT.equals(req.getType()))
			return (obj instanceof Iterate);
		return super.understandsRequest(req);
	}
}
