/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.dataflow.preferences;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.topcased.modeler.preferences.AbstractEdgePreferencePage;

import fr.irisa.espresso.sme.diagram.dataflow.DF_EdgeObjectConstants;
import fr.irisa.espresso.sme.modeler.SmePlugin;

/*
 * This class represents a preference page that is contributed to the Preferences dialog. This page is used to modify
 * preferences only. They are stored in the preference store that belongs to the main plug-in class. That way,
 * preferences can be accessed directly via the preference store.
 * 
 * @generated
 */
public class ConditionPreferencePage extends AbstractEdgePreferencePage {
	/**
	 * @see org.topcased.modeler.preferences.AbstractEdgePreferencePage#getEdgeFont()
	 * @generated
	 */
	protected String getEdgeFont() {
		return DF_DiagramPreferenceConstants.CONDITION_EDGE_DEFAULT_FONT;
	}
	
	/**
	 * @see org.topcased.modeler.preferences.AbstractEdgePreferencePage#getEdgeForegroundColor()
	 * @generated
	 */
	protected String getEdgeForegroundColor() {
		return DF_DiagramPreferenceConstants.CONDITION_EDGE_DEFAULT_FOREGROUND_COLOR;
	}
	
	/**
	 * @see org.topcased.modeler.preferences.AbstractEdgePreferencePage#getEdgeRouter()
	 * @generated
	 */
	protected String getEdgeRouter() {
		return DF_DiagramPreferenceConstants.CONDITION_EDGE_DEFAULT_ROUTER;
	}
	
	/**
	 * @see org.topcased.modeler.preferences.AbstractEdgePreferencePage#getHiddenElements()
	 * @generated
	 */
	protected List<String> getHiddenElements() {
		List<String> choiceOfValues = new ArrayList<String>();
		IPreferenceStore ps = getPreferenceStore();
		if (!ps.getBoolean(DF_DiagramPreferenceConstants.CONDITION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY)) {
			choiceOfValues.add(DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID);
		}
		if (!ps.getBoolean(DF_DiagramPreferenceConstants.CONDITION_CONDITIONKIND_EDGE_OBJECT_DEFAULT_VISIBILITY)) {
			choiceOfValues.add(DF_EdgeObjectConstants.CONDITIONKIND_EDGE_OBJECT_ID);
		}
		return choiceOfValues;
	}
	
	/**
	 * @see org.topcased.modeler.preferences.AbstractEdgePreferencePage#getDefaultHiddenElements()
	 * @generated
	 */
	protected List<String> getDefaultHiddenElements() {
		List<String> choiceOfValues = new ArrayList<String>();
		IPreferenceStore ps = getPreferenceStore();
		if (!ps.getDefaultBoolean(DF_DiagramPreferenceConstants.CONDITION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY)) {
			choiceOfValues.add(DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID);
		}
		if (!ps.getDefaultBoolean(DF_DiagramPreferenceConstants.CONDITION_CONDITIONKIND_EDGE_OBJECT_DEFAULT_VISIBILITY)) {
			choiceOfValues.add(DF_EdgeObjectConstants.CONDITIONKIND_EDGE_OBJECT_ID);
		}
		return choiceOfValues;
	}
	
	/**
	 * @see org.topcased.modeler.preferences.AbstractEdgePreferencePage#getVisibleElements()
	 * @generated
	 */
	protected List<String> getVisibleElements() {
		List<String> choiceOfValues = new ArrayList<String>();
		IPreferenceStore ps = getPreferenceStore();
		if (ps.getBoolean(DF_DiagramPreferenceConstants.CONDITION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY)) {
			choiceOfValues.add(DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID);
		}
		if (ps.getBoolean(DF_DiagramPreferenceConstants.CONDITION_CONDITIONKIND_EDGE_OBJECT_DEFAULT_VISIBILITY)) {
			choiceOfValues.add(DF_EdgeObjectConstants.CONDITIONKIND_EDGE_OBJECT_ID);
		}
		return choiceOfValues;
	}
	
	/**
	 * @see org.topcased.modeler.preferences.AbstractEdgePreferencePage#getDefaultVisibleElements()
	 * @generated
	 */
	protected List<String> getDefaultVisibleElements() {
		List<String> choiceOfValues = new ArrayList<String>();
		IPreferenceStore ps = getPreferenceStore();
		if (ps.getDefaultBoolean(DF_DiagramPreferenceConstants.CONDITION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY)) {
			choiceOfValues.add(DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID);
		}
		if (ps.getDefaultBoolean(DF_DiagramPreferenceConstants.CONDITION_CONDITIONKIND_EDGE_OBJECT_DEFAULT_VISIBILITY)) {
			choiceOfValues.add(DF_EdgeObjectConstants.CONDITIONKIND_EDGE_OBJECT_ID);
		}
		return choiceOfValues;
	}
	
	/**
	 * @see org.topcased.modeler.preferences.AbstractEdgePreferencePage#storeEdgeObjectVisibility(java.util.List)
	 * @generated
	 */
	protected void storeEdgeObjectVisibility(List visibleElement) {
		IPreferenceStore ps = getPreferenceStore();
		if (visibleElement.contains(DF_EdgeObjectConstants.SRCFIELD_EDGE_OBJECT_ID)) {
			ps.setValue(DF_DiagramPreferenceConstants.CONDITION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY, true);
		}
		else {
			ps.setValue(DF_DiagramPreferenceConstants.CONDITION_SRCFIELD_EDGE_OBJECT_DEFAULT_VISIBILITY, false);
		}
		if (visibleElement.contains(DF_EdgeObjectConstants.CONDITIONKIND_EDGE_OBJECT_ID)) {
			ps.setValue(DF_DiagramPreferenceConstants.CONDITION_CONDITIONKIND_EDGE_OBJECT_DEFAULT_VISIBILITY, true);
		}
		else {
			ps.setValue(DF_DiagramPreferenceConstants.CONDITION_CONDITIONKIND_EDGE_OBJECT_DEFAULT_VISIBILITY, false);
		}
	}
	
	/**
	 * @see org.topcased.facilities.preferences.AbstractTopcasedPreferencePage#getBundleId()
	 * @generated
	 */
	protected String getBundleId() {
		return SmePlugin.getId();
	}
	
}
