/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.interface_;

import java.net.URL;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.editor.IConfiguration;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.IPaletteManager;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.graphconf.exceptions.MissingGraphConfFileException;

import fr.irisa.espresso.sme.diagram.interface_.edit.InputEditPart;
import fr.irisa.espresso.sme.diagram.interface_.edit.OutputEditPart;
import fr.irisa.espresso.sme.diagram.interface_.edit.ParameterEditPart;
import fr.irisa.espresso.sme.diagram.interface_.edit.PragmaEditPart;
import fr.irisa.espresso.sme.diagram.interface_.edit.SpecificationsEditPart;

/*
 * A diagram configuration : manages Palette, EditPartFactory for this diagram.
 * 
 * @generated
 */
public class ID_Configuration implements IConfiguration {
	/**
	 * @generated
	 */
	private ID_PaletteManager	paletteManager;
	
	/**
	 * @generated
	 */
	private ID_EditPartFactory	editPartFactory;
	
	/**
	 * @generated
	 */
	private ID_CreationUtils	creationUtils;
	
	/**
	 * The DiagramGraphConf that contains graphical informations on the configuration
	 * 
	 * @generated
	 */
	private DiagramGraphConf	diagramGraphConf;
	
	/**
	 * Constructor. Initialize Adapter factories.
	 * 
	 * @generated
	 */
	public ID_Configuration() {
		registerAdapters();
	}
	
	/**
	 * Registers the Adapter Factories for all the EditParts
	 * 
	 * @generated
	 */
	private void registerAdapters() {
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(InputEditPart.class, fr.irisa.espresso.sme.Input.class),
			InputEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(OutputEditPart.class, fr.irisa.espresso.sme.Output.class),
			OutputEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(ParameterEditPart.class, fr.irisa.espresso.sme.Parameter.class),
			ParameterEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(SpecificationsEditPart.class, fr.irisa.espresso.sme.Specifications.class),
			SpecificationsEditPart.class);
		Platform.getAdapterManager().registerAdapters(
			new EditPart2ModelAdapterFactory(PragmaEditPart.class, fr.irisa.espresso.sme.Pragma.class),
			PragmaEditPart.class);
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getId()
	 * @generated
	 */
	public String getId() {
		return new String("fr.irisa.espresso.sme.diagram.interface_");
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getName()
	 * @generated
	 */
	public String getName() {
		return new String("Interface Definition Diagram");
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getEditPartFactory()
	 * @generated
	 */
	public EditPartFactory getEditPartFactory() {
		if (editPartFactory == null) {
			editPartFactory = new ID_EditPartFactory();
		}
		
		return editPartFactory;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getPaletteManager()
	 * @generated
	 */
	public IPaletteManager getPaletteManager() {
		if (paletteManager == null) {
			paletteManager = new ID_PaletteManager(getCreationUtils());
		}
		
		return paletteManager;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getCreationUtils()
	 * @generated
	 */
	public ICreationUtils getCreationUtils() {
		if (creationUtils == null) {
			creationUtils = new ID_CreationUtils(getDiagramGraphConf());
		}
		
		return creationUtils;
	}
	
	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getDiagramGraphConf()
	 * @generated
	 */
	public DiagramGraphConf getDiagramGraphConf() {
		if (diagramGraphConf == null) {
			URL url = ID_Plugin.getDefault().getBundle().getResource(
				"fr/irisa/espresso/sme/diagram/interface_/diagram.graphconf");
			if (url != null) {
				URI fileURI = URI.createURI(url.toString());
				ResourceSet resourceSet = new ResourceSetImpl();
				Resource resource = resourceSet.getResource(fileURI, true);
				if (resource != null && resource.getContents().get(0) instanceof DiagramGraphConf) {
					diagramGraphConf = (DiagramGraphConf) resource.getContents().get(0);
				}
			}
			else {
				new MissingGraphConfFileException(
					"The *.diagramgraphconf file can not be retrieved. Check if the path is correct in the Configuration class of your diagram.");
			}
		}
		
		return diagramGraphConf;
	}
	
}
