/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.interface_;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.requests.CreationFactory;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.common.SmeImageRegistry;

/*
 * Generated Palette Manager
 * 
 * @generated
 */
public class ID_PaletteManager extends ModelerPaletteManager {
	// declare all the palette categories of the diagram
	/**
	 * @generated
	 */
	private PaletteDrawer	elementsDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	miscellanousDrawer;
	
	/**
	 * @generated
	 */
	private ICreationUtils	creationUtils;
	
	/**
	 * The Constructor
	 * 
	 * @param utils
	 *        the creation utils for the tools of the palette
	 * @generated
	 */
	public ID_PaletteManager(ICreationUtils utils) {
		super();
		this.creationUtils = utils;
	}
	
	/**
	 * Creates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void createCategories() {
		createElementsDrawer();
		createMiscellanousDrawer();
	}
	
	/**
	 * Updates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void updateCategories() {
		// deletion of the existing categories and creation of the updated categories
		
		getRoot().remove(elementsDrawer);
		createElementsDrawer();
		
		getRoot().remove(miscellanousDrawer);
		createMiscellanousDrawer();
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createElementsDrawer() {
		elementsDrawer = new PaletteDrawer("Elements", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getInput(), "default");
		entries.add(new ModelerCreationToolEntry("Input signal", "Input signal", factory, SmeImageRegistry
				.getImageDescriptor("INPUT"), SmeImageRegistry.getImageDescriptor("INPUT_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getParameter(), "default");
		entries.add(new ModelerCreationToolEntry("Constant input parameter", "Constant input parameter", factory,
			SmeImageRegistry.getImageDescriptor("PARAMETER"), SmeImageRegistry.getImageDescriptor("PARAMETER_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getOutput(), "default");
		entries.add(new ModelerCreationToolEntry("Output signal", "Output signal", factory, SmeImageRegistry
				.getImageDescriptor("OUTPUT"), SmeImageRegistry.getImageDescriptor("OUTPUT_LARGE")));
		
		elementsDrawer.addAll(entries);
		getRoot().add(elementsDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createMiscellanousDrawer() {
		miscellanousDrawer = new PaletteDrawer("Miscellanous", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getPragma(), "default");
		entries.add(new ModelerCreationToolEntry("Pragma", "Pragma", factory, SmeImageRegistry
				.getImageDescriptor("PRAGMA"), SmeImageRegistry.getImageDescriptor("PRAGMA_LARGE")));
		
		miscellanousDrawer.addAll(entries);
		getRoot().add(miscellanousDrawer);
	}
	
}
