/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.library;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteStack;
import org.eclipse.gef.requests.CreationFactory;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

import fr.irisa.espresso.sme.ModelDeclaration;
import fr.irisa.espresso.sme.ModelKind;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.common.SmeImageRegistry;

/*
 * Generated Palette Manager
 * 
 * @generated
 */
public class LIB_PaletteManager extends ModelerPaletteManager {
	// declare all the palette categories of the diagram
	/**
	 * @generated
	 */
	private PaletteDrawer	elementsDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	miscellaneousDrawer;
	
	/**
	 * @generated
	 */
	private ICreationUtils	creationUtils;
	
	/**
	 * The Constructor
	 * 
	 * @param utils
	 *        the creation utils for the tools of the palette
	 * @generated
	 */
	public LIB_PaletteManager(ICreationUtils utils) {
		super();
		this.creationUtils = utils;
	}
	
	/**
	 * Creates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void createCategories() {
		createElementsDrawer();
		createMiscellaneousDrawer();
	}
	
	/**
	 * Updates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void updateCategories() {
		// deletion of the existing categories and creation of the updated categories
		
		getRoot().remove(elementsDrawer);
		createElementsDrawer();
		
		getRoot().remove(miscellaneousDrawer);
		createMiscellaneousDrawer();
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createElementsDrawer() {
		elementsDrawer = new PaletteDrawer("Elements", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		ModelerCreationToolEntry objectTool;
		PaletteStack stack;
		
		stack = new PaletteStack("Model Declaration", "Model Declaration Selection", SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelDeclaration()) {
			public EObject getNewModelObject() {
				ModelDeclaration md = (ModelDeclaration) super.getNewModelObject();
				md.setModelType(ModelKind.ACTION_LITERAL);
				return md;
			}
		};
		objectTool = new ModelerCreationToolEntry("Action", "Action", factory, SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"), SmeImageRegistry.getImageDescriptor("MODELDECLARATION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelDeclaration()) {
			public EObject getNewModelObject() {
				ModelDeclaration md = (ModelDeclaration) super.getNewModelObject();
				md.setModelType(ModelKind.FUNCTION_LITERAL);
				return md;
			}
		};
		objectTool = new ModelerCreationToolEntry("Function", "Function", factory, SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"), SmeImageRegistry.getImageDescriptor("MODELDECLARATION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelDeclaration()) {
			public EObject getNewModelObject() {
				ModelDeclaration md = (ModelDeclaration) super.getNewModelObject();
				md.setModelType(ModelKind.NODE_LITERAL);
				return md;
			}
		};
		objectTool = new ModelerCreationToolEntry("Node", "Node", factory, SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"), SmeImageRegistry.getImageDescriptor("MODELDECLARATION_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelDeclaration()) {
			public EObject getNewModelObject() {
				ModelDeclaration md = (ModelDeclaration) super.getNewModelObject();
				md.setModelType(ModelKind.PROCESS_LITERAL);
				return md;
			}
		};
		objectTool = new ModelerCreationToolEntry("Process", "Process", factory, SmeImageRegistry
				.getImageDescriptor("MODELDECLARATION"), SmeImageRegistry.getImageDescriptor("MODELDECLARATION_LARGE"));
		stack.add(objectTool);
		stack.setActiveEntry(objectTool);
		entries.add(stack);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getConstantValue(), "default");
		entries.add(new ModelerCreationToolEntry("Constant declaration", "Constant declaration", factory,
			SmeImageRegistry.getImageDescriptor("CONSTANTVALUE"), SmeImageRegistry
					.getImageDescriptor("CONSTANTVALUE_LARGE")));
		
		stack = new PaletteStack("Type Definition", "Type Definition Selection", null);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getEnumerationType(), "default");
		objectTool = new ModelerCreationToolEntry("Enumeration", "Enumeration", factory, SmeImageRegistry
				.getImageDescriptor("ENUMERATIONTYPE"), SmeImageRegistry.getImageDescriptor("ENUMERATIONTYPE_LARGE"));
		stack.add(objectTool);
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getModelType(), "default");
		stack.add(new ModelerCreationToolEntry("Model type", "Model type", factory, SmeImageRegistry
				.getImageDescriptor("MODELTYPE"), SmeImageRegistry.getImageDescriptor("MODELTYPE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getSubstitutionType(), "default");
		stack
				.add(new ModelerCreationToolEntry("Generic type", "Generic type", factory, SmeImageRegistry
						.getImageDescriptor("SUBSTITUTIONTYPE"), SmeImageRegistry
						.getImageDescriptor("SUBSTITUTIONTYPE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getTupleType(), "default");
		stack.add(new ModelerCreationToolEntry("Tuple kind type", "Tuple kind type", factory, SmeImageRegistry
				.getImageDescriptor("TUPLETYPE"), SmeImageRegistry.getImageDescriptor("TUPLETYPE_LARGE")));
		entries.add(stack);
		
		elementsDrawer.addAll(entries);
		getRoot().add(elementsDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createMiscellaneousDrawer() {
		miscellaneousDrawer = new PaletteDrawer("Miscellaneous", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getPragma(), "default");
		entries.add(new ModelerCreationToolEntry("Pragma", "Pragma", factory, SmeImageRegistry
				.getImageDescriptor("PRAGMA"), SmeImageRegistry.getImageDescriptor("PRAGMA_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getUseModule(), "default");
		entries.add(new ModelerCreationToolEntry("Import Modules", "Import Modules", factory, SmeImageRegistry
				.getImageDescriptor("USEMODULE"), SmeImageRegistry.getImageDescriptor("USEMODULE_LARGE")));
		
		miscellaneousDrawer.addAll(entries);
		getRoot().add(miscellaneousDrawer);
	}
	
}
