/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.modeautomaton;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.requests.CreationFactory;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.common.SmeImageRegistry;

/*
 * Generated Palette Manager
 * 
 * @generated
 */
public class MA_PaletteManager extends ModelerPaletteManager {
	// declare all the palette categories of the diagram
	/**
	 * @generated
	 */
	private PaletteDrawer	modesDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	transitionsDrawer;
	/**
	 * @generated
	 */
	private PaletteDrawer	miscellaneousDrawer;
	
	/**
	 * @generated
	 */
	private ICreationUtils	creationUtils;
	
	/**
	 * The Constructor
	 * 
	 * @param utils
	 *        the creation utils for the tools of the palette
	 * @generated
	 */
	public MA_PaletteManager(ICreationUtils utils) {
		super();
		this.creationUtils = utils;
	}
	
	/**
	 * Creates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void createCategories() {
		createModesDrawer();
		createTransitionsDrawer();
		createMiscellaneousDrawer();
	}
	
	/**
	 * Updates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void updateCategories() {
		// deletion of the existing categories and creation of the updated categories
		
		getRoot().remove(modesDrawer);
		createModesDrawer();
		
		getRoot().remove(transitionsDrawer);
		createTransitionsDrawer();
		
		getRoot().remove(miscellaneousDrawer);
		createMiscellaneousDrawer();
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createModesDrawer() {
		modesDrawer = new PaletteDrawer("Modes", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getAutomaton(), "default");
		entries.add(new ModelerCreationToolEntry("Automaton", "Automaton", factory, SmeImageRegistry
				.getImageDescriptor("AUTOMATON"), SmeImageRegistry.getImageDescriptor("AUTOMATON_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getAndState(), "default");
		entries.add(new ModelerCreationToolEntry("And state", "And state", factory, SmeImageRegistry
				.getImageDescriptor("ANDSTATE"), SmeImageRegistry.getImageDescriptor("ANDSTATE_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getState(), "default");
		entries.add(new ModelerCreationToolEntry("Final state", "Final state", factory, SmeImageRegistry
				.getImageDescriptor("STATE"), SmeImageRegistry.getImageDescriptor("STATE_LARGE")));
		
		modesDrawer.addAll(entries);
		getRoot().add(modesDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createTransitionsDrawer() {
		transitionsDrawer = new PaletteDrawer("Transitions", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getWeakTransition(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("Weak transition", "Weak transition", factory,
			SmeImageRegistry.getImageDescriptor("WEAKTRANSITION"), SmeImageRegistry
					.getImageDescriptor("WEAKTRANSITION_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getStrongTransition(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("Strong transition", "Strong transition", factory,
			SmeImageRegistry.getImageDescriptor("STRONGTRANSITION"), SmeImageRegistry
					.getImageDescriptor("STRONGTRANSITION_LARGE")));
		
		transitionsDrawer.addAll(entries);
		getRoot().add(transitionsDrawer);
	}
	
	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated NOT
	 */
	private void createMiscellaneousDrawer() {
		miscellaneousDrawer = new PaletteDrawer("Miscellaneous", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();
		
		CreationFactory factory;
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getHistory(), "default");
		entries.add(new ModelerCreationToolEntry("History", "History", factory, SmeImageRegistry
				.getImageDescriptor("HISTORY"), SmeImageRegistry.getImageDescriptor("HISTORY_LARGE")));
		
		factory = new GraphElementCreationFactory(creationUtils, SmePackage.eINSTANCE.getLocal(), "default");
		entries.add(new ModelerCreationToolEntry("Local signal", "Local signal", factory, SmeImageRegistry
				.getImageDescriptor("LOCAL"), SmeImageRegistry.getImageDescriptor("LOCAL_LARGE")));
		
		miscellaneousDrawer.addAll(entries);
		getRoot().add(miscellaneousDrawer);
	}
	
}
