/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.modeautomaton.edit;

import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;

import fr.irisa.espresso.sme.common.SmeEditPolicyConstants;
import fr.irisa.espresso.sme.common.policies.IdentifierEditPolicy;
import fr.irisa.espresso.sme.diagram.modeautomaton.MA_EditPolicyConstants;
import fr.irisa.espresso.sme.diagram.modeautomaton.commands.StateBaseRestoreConnectionCommand;
import fr.irisa.espresso.sme.diagram.modeautomaton.policies.SetInitialStateEditPolicy;
import fr.irisa.espresso.sme.diagram.modeautomaton.policies.StrongTransitionEdgeCreationEditPolicy;
import fr.irisa.espresso.sme.diagram.modeautomaton.policies.WeakTransitionEdgeCreationEditPolicy;

/*
 * The StateBase object controller
 * 
 * @generated
 */
public class StateBaseEditPart extends EMFGraphNodeEditPart {
	/**
	 * Constructor
	 * 
	 * @param obj
	 *        the graph node
	 * @generated
	 */
	public StateBaseEditPart(GraphNode obj) {
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(SmeEditPolicyConstants.IDENTIFIER_EDITPOLICY, new IdentifierEditPolicy());
		
		installEditPolicy(MA_EditPolicyConstants.WEAKTRANSITION_EDITPOLICY, new WeakTransitionEdgeCreationEditPolicy());
		
		installEditPolicy(MA_EditPolicyConstants.STRONGTRANSITION_EDITPOLICY,
			new StrongTransitionEdgeCreationEditPolicy());
		
		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy() {
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request) {
				return new StateBaseRestoreConnectionCommand(getHost());
			}
		});
		
		installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());
		
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
		
		// Add policy to manage the initial state in an Automaton
		installEditPolicy(MA_EditPolicyConstants.INITIAL_STATE_EDITPOLICY, new SetInitialStateEditPolicy());
	}
}
