/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.modeautomaton.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.State;
import fr.irisa.espresso.sme.common.commands.ChangeAspectCommand;
import fr.irisa.espresso.sme.common.figures.StateFigure;
import fr.irisa.espresso.sme.common.policies.CustomEditPolicy;
import fr.irisa.espresso.sme.diagram.modeautomaton.preferences.MA_DiagramPreferenceConstants;

/*
 * The State object controller
 * 
 * @generated
 */
public class StateEditPart extends StateBaseEditPart {
	/**
	 * Constructor
	 * 
	 * @param obj
	 *        the graph node
	 * @generated
	 */
	public StateEditPart(GraphNode obj) {
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		/* Add menu to go in State Aspect */
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new CustomEditPolicy());
		
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated NOT
	 */
	protected IFigure createFigure() {
		IFigure fig = new StateFigure("STATE_LARGE");
		State s = (State) Utils.getElement((GraphElement) this.getModel());
		Label lbl = new Label(s.toString(true, true));
		lbl.setParent(fig);
		fig.setToolTip(lbl);
		return fig;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultBackgroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultBackgroundColor() {
		String backgroundColor = getPreferenceStore().getString(
			MA_DiagramPreferenceConstants.STATE_DEFAULT_BACKGROUND_COLOR);
		if (backgroundColor.length() != 0) {
			return Utils.getColor(backgroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultForegroundColor()
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String foregroundColor = getPreferenceStore().getString(
			MA_DiagramPreferenceConstants.STATE_DEFAULT_FOREGROUND_COLOR);
		if (foregroundColor.length() != 0) {
			return Utils.getColor(foregroundColor);
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getPreferenceDefaultFont()
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(MA_DiagramPreferenceConstants.STATE_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
	 * @generated NOT
	 */
	protected void refreshConstraints() {
		super.refreshConstraints();
		
		if (getFigure() instanceof StateFigure) {
			State st = (State) getEObject();
			if (st.getParent() instanceof Automaton) {
				Automaton p = (Automaton) st.getParent();
				((StateFigure) getFigure()).setInitState(st.equals(p.getInitState()));
			}
			else
				((StateFigure) getFigure()).setInitState(false);
		}
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
	 * @generated NOT
	 */
	protected void handleModelChanged(Notification msg) {
		if (msg.getNotifier() instanceof State) {
			IFigure fig = getFigure();
			Label lbl = (Label) fig.getToolTip();
			State s = (State) Utils.getElement((GraphElement) this.getModel());
			lbl.setText(s.toString(true, true));
		}
		super.handleModelChanged(msg);
	}
	
	/**
	 * When double-click on an State component, go directly to its Dataflow Aspect
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#performRequest(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public void performRequest(Request req) {
		if (req.getType() == RequestConstants.REQ_OPEN) {
			if (getViewer() instanceof ModelerGraphicalViewer) {
				ModelerGraphicalViewer viewer = (ModelerGraphicalViewer) getViewer();
				Command command = new ChangeAspectCommand((GraphNode) getModel(), viewer,
					"fr.irisa.espresso.sme.diagram.dataflow");
				if (command.canExecute())
					((CommandStack) viewer.getModelerEditor().getAdapter(CommandStack.class)).execute(command);
			}
		}
		else
			super.performRequest(req);
	}
}
