/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.modeautomaton.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.LabelCellEditorLocator;
import org.topcased.modeler.edit.ModelerLabelDirectEditManager;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.StrongTransition;
import fr.irisa.espresso.sme.diagram.modeautomaton.MA_EdgeObjectConstants;
import fr.irisa.espresso.sme.diagram.modeautomaton.MA_EditPolicyConstants;
import fr.irisa.espresso.sme.diagram.modeautomaton.figures.StrongTransitionFigure;
import fr.irisa.espresso.sme.diagram.modeautomaton.policies.StrongTransitionEdgeObjectUVEditPolicy;
import fr.irisa.espresso.sme.diagram.modeautomaton.policies.TransitionEditPolicy;
import fr.irisa.espresso.sme.diagram.modeautomaton.preferences.MA_DiagramPreferenceConstants;

/*
 * StrongTransition controller
 * 
 * @generated
 */
public class StrongTransitionEditPart extends EMFGraphEdgeEditPart {
	
	/**
	 * The DirectEditManager
	 * 
	 * @generated
	 */
	private DirectEditManager	manager;
	
	/**
	 * Constructor
	 * 
	 * @param model
	 *        the graph object
	 * @generated
	 */
	public StrongTransitionEditPart(GraphEdge model) {
		super(model);
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 * @generated NOT
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		
		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY,
			new StrongTransitionEdgeObjectUVEditPolicy());
		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
		
		// Edit the guard through the popup menu
		installEditPolicy(MA_EditPolicyConstants.TRANSITION_EDITPOLICY, new TransitionEditPolicy());
	}
	
	/**
	 * @return the Figure
	 * @generated
	 */
	protected IFigure createFigure() {
		StrongTransitionFigure connection = new StrongTransitionFigure();
		
		createTargetDecoration(connection);
		
		return connection;
	}
	
	/**
	 * @param connection
	 *        the PolylineConnection
	 * @generated
	 */
	private void createTargetDecoration(PolylineConnection connection) {
		
		PolylineDecoration decoration = new PolylineDecoration();
		decoration.setScale(10, 5);
		connection.setTargetDecoration(decoration);
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
	 * @generated
	 */
	public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject) {
		if (MA_EdgeObjectConstants.PRIORITY_EDGE_OBJECT_ID.equals(edgeObject.getId())) {
			return ((StrongTransitionFigure) getFigure()).getPriorityEdgeObjectFigure();
		}
		if (MA_EdgeObjectConstants.GUARD_EDGE_OBJECT_ID.equals(edgeObject.getId())) {
			return ((StrongTransitionFigure) getFigure()).getGuardEdgeObjectFigure();
		}
		return null;
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
	 * @generated
	 */
	protected void refreshEdgeObjects() {
		super.refreshEdgeObjects();
		updatePriorityLabel();
		updateGuardLabel();
	}
	
	/**
	 * Update the priority Label
	 * 
	 * @generated NOT
	 */
	private void updatePriorityLabel() {
		((Label) ((StrongTransitionFigure) getFigure()).getPriorityEdgeObjectFigure()).setText(""
				+ ((StrongTransition) getEObject()).getPriority());
	}
	
	/**
	 * Update the guard Label
	 * 
	 * @generated NOT
	 */
	private void updateGuardLabel() {
		String guard = ((StrongTransition) getEObject()).getGuard();
		((Label) ((StrongTransitionFigure) getFigure()).getGuardEdgeObjectFigure()).setText("["
				+ (guard == null ? "true" : guard) + "]");
	}
	
	// ---------------------------------------
	// Ajout des fonctions pour le DirectEdit
	// ---------------------------------------
	
	/**
	 * Handle selection
	 * 
	 * @param value
	 *        The value of the selection
	 * @generated
	 */
	public void setSelected(int value) {
		super.setSelected(value);
		((EditableLabel) ((StrongTransitionFigure) getFigure()).getGuardEdgeObjectFigure()).setSelected(value != 0);
	}
	
	/**
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 * @generated
	 */
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT) {
			if (request instanceof DirectEditRequest
					&& !(directEditHitTest(((DirectEditRequest) request).getLocation().getCopy()))) {
				return;
			}
			performDirectEdit();
		}
	}
	
	/**
	 * @param requestLoc
	 *        the location of the cursor
	 * @return true if the cursor is over the EditableLabel
	 * @generated
	 */
	private boolean directEditHitTest(Point requestLoc) {
		Label guardName = (Label) ((StrongTransitionFigure) getFigure()).getGuardEdgeObjectFigure();
		if (guardName != null) {
			guardName.translateToRelative(requestLoc);
			return guardName.containsPoint(requestLoc);
		}
		return false;
	}
	
	/**
	 * @generated
	 */
	private void performDirectEdit() {
		if (manager == null) {
			Label guardName = (Label) ((StrongTransitionFigure) getFigure()).getGuardEdgeObjectFigure();
			manager = new ModelerLabelDirectEditManager(this, TextCellEditor.class, new LabelCellEditorLocator(
				guardName), guardName);
		}
		manager.show();
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultRouter()
	 * 
	 * @generated
	 */
	protected String getPreferenceDefaultRouter() {
		return getPreferenceStore().getString(MA_DiagramPreferenceConstants.STRONGTRANSITION_EDGE_DEFAULT_ROUTER);
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultForegroundColor()
	 * 
	 * @generated
	 */
	protected Color getPreferenceDefaultForegroundColor() {
		String preferenceForeground = getPreferenceStore().getString(
			MA_DiagramPreferenceConstants.STRONGTRANSITION_EDGE_DEFAULT_FOREGROUND_COLOR);
		if (preferenceForeground.length() != 0) {
			return Utils.getColor(preferenceForeground);
		}
		return null;
		
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getPreferenceDefaultFont()
	 * 
	 * @generated
	 */
	protected Font getPreferenceDefaultFont() {
		String preferenceFont = getPreferenceStore().getString(
			MA_DiagramPreferenceConstants.STRONGTRANSITION_EDGE_DEFAULT_FONT);
		if (preferenceFont.length() != 0) {
			return Utils.getFont(new FontData(preferenceFont));
		}
		return null;
	}
}
