/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.modeautomaton.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.AbstractEditPolicy;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.StateBase;
import fr.irisa.espresso.sme.common.SmeRequestConstants;
import fr.irisa.espresso.sme.common.commands.SetInitStateCommand;

/**
 * The EditPolicy that handle the Set Initial State command for StateBase model object. Created 2 mar. 2007
 * 
 * @author Christian Brunette
 */
public class SetInitialStateEditPolicy extends AbstractEditPolicy {
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#getCommand(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public Command getCommand(Request request) {
		if (SmeRequestConstants.REQ_SET_INIT_STATE.equals(request.getType())) {
			EObject model = Utils.getElement((GraphElement) getHost().getModel());
			if (model instanceof StateBase) {
				SetInitStateCommand command = new SetInitStateCommand((StateBase) model);
				return command;
			}
		}
		return null;
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#understandsRequest(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public boolean understandsRequest(Request req) {
		if (SmeRequestConstants.REQ_SET_INIT_STATE.equals(req.getType())) {
			EObject model = Utils.getElement((GraphElement) getHost().getModel());
			if (model instanceof StateBase)
				return ((StateBase) model).getParent() instanceof Automaton;
		}
		return false;
	}
	
}
