/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.modeautomaton.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy;
import org.topcased.modeler.utils.SourceTargetData;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.History;
import fr.irisa.espresso.sme.StateBase;
import fr.irisa.espresso.sme.StrongTransition;
import fr.irisa.espresso.sme.diagram.modeautomaton.commands.StrongTransitionEdgeCreationCommand;

/*
 * StrongTransition edge creation
 * 
 * @generated
 */
public class StrongTransitionEdgeCreationEditPolicy extends AbstractEdgeCreationEditPolicy {
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#createCommand(org.eclipse.gef.EditDomain,
	 *      org.topcased.modeler.di.model.GraphEdge, org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected CreateTypedEdgeCommand createCommand(EditDomain domain, GraphEdge edge, GraphElement source) {
		return new StrongTransitionEdgeCreationCommand(domain, edge, source);
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkEdge(org.topcased.modeler.di.model.GraphEdge)
	 * @generated
	 */
	protected boolean checkEdge(GraphEdge edge) {
		return Utils.getElement(edge) instanceof StrongTransition;
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkSource(org.topcased.modeler.di.model.GraphElement)
	 * @generated NOT
	 */
	protected boolean checkSource(GraphElement source) {
		EObject object = Utils.getElement(source);
		// Check that the source is of any kind of state and that its direct parent is an Automaton
		return (object instanceof StateBase) && ((StateBase) object).getParent() instanceof Automaton;
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkTargetForSource(org.topcased.modeler.di.model.GraphElement,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated NOT
	 */
	protected boolean checkTargetForSource(GraphElement source, GraphElement target) {
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);
		
		if (sourceObject instanceof StateBase && targetObject instanceof History) {
			if (((StateBase) sourceObject).getParent().equals(((History) targetObject).getParentState().getParent())) {
				if (!sourceObject.equals(targetObject))
					return true;
			}
		}
		
		if (sourceObject instanceof StateBase && targetObject instanceof StateBase) {
			if (((StateBase) sourceObject).getParent().equals(((StateBase) targetObject).getParent())) {
				if (!sourceObject.equals(targetObject))
					return true;
			}
		}
		return false;
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkCommand(org.eclipse.gef.commands.Command)
	 * @generated
	 */
	protected boolean checkCommand(Command command) {
		return command instanceof StrongTransitionEdgeCreationCommand;
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getSourceTargetData(org.topcased.modeler.di.model.GraphElement,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected SourceTargetData getSourceTargetData(GraphElement source, GraphElement target) {
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);
		
		if (sourceObject instanceof fr.irisa.espresso.sme.StateBase
				&& targetObject instanceof fr.irisa.espresso.sme.History) {
			return new SourceTargetData(false, false, SourceTargetData.DIAGRAM, "fr.irisa.espresso.sme.Automaton",
				"transitions", "src", "dst", "src_StrongTransitions", null, "dst_StrongTransitions", null);
		}
		if (sourceObject instanceof fr.irisa.espresso.sme.StateBase
				&& targetObject instanceof fr.irisa.espresso.sme.StateBase) {
			return new SourceTargetData(false, false, SourceTargetData.DIAGRAM, "fr.irisa.espresso.sme.Automaton",
				"transitions", "src", "dst", "src_StrongTransitions", null, "dst_StrongTransitions", null);
		}
		return null;
	}
	
}
