/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.tupletype;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ProjectScope;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.preferences.ScopedPreferenceStore;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.editor.AbstractCreationUtils;
import org.topcased.modeler.graphconf.DiagramGraphConf;

import fr.irisa.espresso.sme.util.SmeSwitch;

/*
 * This utility class allows to create a GraphElement associated with a Model Object
 * 
 * @generated
 */
public class TT_CreationUtils extends AbstractCreationUtils {
	
	/**
	 * Constructor
	 * 
	 * @param diagramConf
	 *        the Diagram Graphical Configuration
	 * @generated
	 */
	public TT_CreationUtils(DiagramGraphConf diagramConf) {
		super(diagramConf);
	}
	
	/**
	 * @generated
	 */
	private class GraphicSmeSwitch extends SmeSwitch<Object> {
		/**
		 * The presentation of the graphical element
		 * 
		 * @generated
		 */
		private String	presentation;
		
		/**
		 * Constructor
		 * 
		 * @param presentation
		 *        the presentation of the graphical element
		 * @generated
		 */
		public GraphicSmeSwitch(String presentation) {
			this.presentation = presentation;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseLocal(fr.irisa.espresso.sme.Local)
		 * @generated
		 */
		public Object caseLocal(fr.irisa.espresso.sme.Local object) {
			if ("default".equals(presentation)) {
				return createGraphElementLocal(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSpecifications(fr.irisa.espresso.sme.Specifications)
		 * @generated
		 */
		public Object caseSpecifications(fr.irisa.espresso.sme.Specifications object) {
			if ("default".equals(presentation)) {
				return createGraphElementSpecifications(object, presentation);
			}
			return null;
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object) {
			return null;
		}
	}
	
	/**
	 * @see org.topcased.modeler.editor.ICreationUtils#createGraphElement(org.eclipse.emf.ecore.EObject,
	 *      java.lang.String)
	 * @generated
	 */
	public GraphElement createGraphElement(EObject obj, String presentation) {
		Object graphElt = null;
		
		if ("http://www.irisa.fr/espresso/sme.ecore".equals(obj.eClass().getEPackage().getNsURI())) {
			graphElt = new GraphicSmeSwitch(presentation).doSwitch(obj);
		}
		
		return (GraphElement) graphElt;
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementLocal(fr.irisa.espresso.sme.Local element, String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * @param element
	 *        the model element
	 * @param presentation
	 *        the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementSpecifications(fr.irisa.espresso.sme.Specifications element,
			String presentation) {
		return createGraphNode(element, presentation);
	}
	
	/**
	 * Create the ModelObject with its initial children
	 * 
	 * @param obj
	 *        the model object
	 * @return the model object with its children
	 * @generated
	 */
	public EObject createModelObject(EObject obj) {
		return obj;
	}
	
	/**
	 * Get the preference store associated with the current editor.
	 * 
	 * @return IPreferenceStore
	 * @generated
	 */
	private IPreferenceStore getPreferenceStore() {
		IEditorInput editorInput = TT_Plugin.getActivePage().getActiveEditor().getEditorInput();
		if (editorInput instanceof IFileEditorInput) {
			IProject project = ((IFileEditorInput) editorInput).getFile().getProject();
			Preferences root = Platform.getPreferencesService().getRootNode();
			try {
				if (root.node(ProjectScope.SCOPE).node(project.getName()).nodeExists(TT_Plugin.getId())) {
					return new ScopedPreferenceStore(new ProjectScope(project), TT_Plugin.getId());
				}
			}
			catch (BackingStoreException e) {
				e.printStackTrace();
			}
		}
		return TT_Plugin.getDefault().getPreferenceStore();
	}
}
