/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.diagram.tupletype;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.editor.ModelerEditPartFactory;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.diagram.tupletype.edit.LocalEditPart;
import fr.irisa.espresso.sme.diagram.tupletype.edit.SpecificationsEditPart;
import fr.irisa.espresso.sme.diagram.tupletype.edit.TT_DiagramEditPart;
import fr.irisa.espresso.sme.util.SmeSwitch;

/*
 * Part Factory : associates a model object to its controller. <br>
 * 
 * @generated
 */
public class TT_EditPartFactory extends ModelerEditPartFactory {
	/**
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
	 * @generated
	 */
	public EditPart createEditPart(EditPart context, Object model) {
		if (model instanceof Diagram) {
			return new TT_DiagramEditPart((Diagram) model);
		}
		else if (model instanceof GraphNode) {
			final GraphNode node = (GraphNode) model;
			EObject element = Utils.getElement(node);
			if (element != null) {
				if ("http://www.irisa.fr/espresso/sme.ecore".equals(element.eClass().getEPackage().getNsURI())) {
					return (EditPart) new NodeSmeSwitch(node).doSwitch(element);
				}
			}
			
			if (node.getSemanticModel() instanceof SimpleSemanticModelElement) {
				// Manage the Element that are not associated with a model object
			}
		}
		else if (model instanceof GraphEdge) {
			final GraphEdge edge = (GraphEdge) model;
			EObject element = Utils.getElement(edge);
			if (element != null) {
				if ("http://www.irisa.fr/espresso/sme.ecore".equals(element.eClass().getEPackage().getNsURI())) {
					return (EditPart) new EdgeSmeSwitch(edge).doSwitch(element);
				}
			}
			
			if (edge.getSemanticModel() instanceof SimpleSemanticModelElement) {
				// Manage the Element that are not associated with a model object
			}
		}
		return super.createEditPart(context, model);
	}
	
	/**
	 * @generated
	 */
	private class NodeSmeSwitch extends SmeSwitch<Object> {
		/**
		 * The graphical node
		 * 
		 * @generated
		 */
		private GraphNode	node;
		
		/**
		 * Constructor
		 * 
		 * @param node
		 *        the graphical node
		 * @generated
		 */
		public NodeSmeSwitch(GraphNode node) {
			this.node = node;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseLocal(fr.irisa.espresso.sme.Local)
		 * @generated
		 */
		public Object caseLocal(fr.irisa.espresso.sme.Local object) {
			return new LocalEditPart(node);
		}
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#caseSpecifications(fr.irisa.espresso.sme.Specifications)
		 * @generated
		 */
		public Object caseSpecifications(fr.irisa.espresso.sme.Specifications object) {
			return new SpecificationsEditPart(node);
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object) {
			return new EMFGraphNodeEditPart(node);
		}
	}
	
	/**
	 * @generated
	 */
	private class EdgeSmeSwitch extends SmeSwitch<Object> {
		/**
		 * The graphical edge
		 * 
		 * @generated
		 */
		private GraphEdge	edge;
		
		/**
		 * Constructor
		 * 
		 * @param edge
		 *        the graphical edge
		 * @generated
		 */
		public EdgeSmeSwitch(GraphEdge edge) {
			this.edge = edge;
		}
		
		/**
		 * @see fr.irisa.espresso.sme.util.SmeSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object) {
			return new EMFGraphEdgeEditPart(edge);
		}
	}
	
}
