/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

/**
 * A representation of the literals of the enumeration '<em><b>Signal Kind</b></em>', and utility methods for working
 * with them.
 * 
 * Created 3 ao�t 06
 */
public enum SignalKind implements Enumerator {
	NONE_LITERAL(0, "NONE", "NONE"), INPUT_LITERAL(1, "INPUT", "INPUT"), OUTPUT_LITERAL(2, "OUTPUT", "OUTPUT"),
	LOCAL_LITERAL(3, "LOCAL", "LOCAL");
	
	/**
	 * 
	 */
	public static final int					NONE			= 0;
	
	/**
	 * 
	 */
	public static final int					INPUT			= 1;
	
	/**
	 * 
	 */
	public static final int					OUTPUT			= 2;
	
	/**
	 * 
	 */
	public static final int					LOCAL			= 3;
	
	/**
	 * An array of all the '<em><b>Signal Kind</b></em>' enumerators.
	 * 
	 */
	private static final SignalKind[]		VALUES_ARRAY	= new SignalKind[] { NONE_LITERAL, INPUT_LITERAL,
			OUTPUT_LITERAL, LOCAL_LITERAL,					};
	
	/**
	 * A public read-only list of all the '<em><b>Signal Kind</b></em>' enumerators.
	 * 
	 */
	public static final List<SignalKind>	VALUES			= Collections.unmodifiableList(Arrays.asList(VALUES_ARRAY));
	
	/**
	 * Returns the '<em><b>Signal Kind</b></em>' literal with the specified literal value.
	 * 
	 */
	public static SignalKind get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			SignalKind result = VALUES_ARRAY[i];
			if (result.toString().equals(literal))
				return result;
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Signal Kind</b></em>' literal with the specified name.
	 */
	public static SignalKind getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			SignalKind result = VALUES_ARRAY[i];
			if (result.getName().equals(name))
				return result;
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Signal Kind</b></em>' literal with the specified integer value.
	 * 
	 */
	public static SignalKind get(int value) {
		switch (value) {
			case NONE:
				return NONE_LITERAL;
			case INPUT:
				return INPUT_LITERAL;
			case OUTPUT:
				return OUTPUT_LITERAL;
			case LOCAL:
				return LOCAL_LITERAL;
			default:
				return NONE_LITERAL;
		}
	}
	
	/**
	 * 
	 */
	private final int		value;
	
	/**
	 * 
	 */
	private final String	name;
	
	/**
	 * 
	 */
	private final String	literal;
	
	/**
	 * Only this class can construct instances.
	 */
	private SignalKind(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}
	
	/**
	 * 
	 */
	public int getValue() {
		return value;
	}
	
	/**
	 * 
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * 
	 */
	public String getLiteral() {
		return literal;
	}
	
	/**
	 * Returns the literal value of the enumerator, which is its string representation.
	 */
	@Override
	public String toString() {
		return literal;
	}
	
}
