/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Tool;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.topcased.modeler.commands.CreateDiagramCommand;
import org.topcased.modeler.commands.CreateDiagramsCommand;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.diagrams.model.util.DiagramsUtils;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.extensions.DiagramDescriptor;
import org.topcased.modeler.extensions.DiagramsManager;
import org.topcased.modeler.utils.Utils;

/**
 * 
 * Creation : 08 june 2006
 */
public class ChangeAspectCommand extends Command {
	private EObject					obj;
	
	private Diagram					initDiagram;
	
	private ModelerGraphicalViewer	viewer;
	
	private String					diagramId;
	
	/**
	 * Constructor
	 * 
	 * @param element
	 * @param viewer
	 * @param diagramId
	 */
	public ChangeAspectCommand(GraphElement element, ModelerGraphicalViewer viewer, String diagramId) {
		super("Change Aspect");
		obj = Utils.getElement(element);
		this.viewer = viewer;
		this.diagramId = diagramId;
	}
	
	/**
	 * Constructor
	 * 
	 * @param element
	 * @param viewer
	 * @param diagramId
	 */
	public ChangeAspectCommand(EObject element, ModelerGraphicalViewer viewer, String diagramId) {
		super("Change Aspect");
		obj = element;
		this.viewer = viewer;
		this.diagramId = diagramId;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return (obj != null) && (viewer != null) && (diagramId != null);
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		initDiagram = viewer.getModelerEditor().getActiveDiagram();
		redo();
	}
	
	/**
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		Modeler editor = viewer.getModelerEditor();
		
		// save the active tool
		Tool activeTool = viewer.getEditDomain().getActiveTool();
		
		// deactivate the active tool
		viewer.getEditDomain().setActiveTool(null);
		viewer.getEditDomain().getPaletteViewer().setActiveTool(null);
		
		// We search if a diagram already exists and let the user choose if at least two exist.
		List<Diagram> existingDiagramList = DiagramsUtils.findAllExistingDiagram(editor.getDiagrams(), obj);
		
		boolean diagFound = false;
		for (Iterator<Diagram> it = existingDiagramList.iterator(); diagFound == false && it.hasNext();) {
			Diagram diag = it.next();
			if (diag.getSemanticModel().getPresentation().equals(diagramId)) {
				diagFound = true;
				editor.setActiveDiagram(diag);
			}
		}
		
		// If the diagram does not exist yet, we create it
		if (diagFound == false) {
			// Retrieve the DiagramDescriptors that can be created into this EObject
			DiagramDescriptor[] diagramDescriptors = DiagramsManager.getInstance().getDiagrams();
			for (int i = 0; (diagFound == false) && (i < diagramDescriptors.length); i++) {
				if (diagramDescriptors[i].canCreateDiagramOn(obj, editor.getId())
						&& diagramDescriptors[i].getId().equals(diagramId)) {
					diagFound = true;
					// The CreateDiagramsCommand should be first executed before the
					// CreateDiagramCommand : indeed, the CreateDiagramCommand need the
					// container Diagrams where the Diagram should be added in the case when
					// it must be created.
					((CommandStack) editor.getAdapter(CommandStack.class)).execute(new CreateDiagramsCommand(editor,
						obj));
					((CommandStack) editor.getAdapter(CommandStack.class)).execute(new CreateDiagramCommand(editor,
						obj, diagramDescriptors[i], true));
				}
			}
			if (!diagFound)
				// restore the active tool
				viewer.getEditDomain().setActiveTool(activeTool);
		}
	}
	
	/**
	 * Remove the last added InputPort (and OutputPort)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		viewer.getModelerEditor().setActiveDiagram(initDiagram);
	}
}
