/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.commands;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CompoundCommand;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.OutputInstance;
import fr.irisa.espresso.sme.ParameterInstance;

/**
 * This command is used to create the ports of a ModelInstance
 * 
 * Creation : 26 july 2006
 */
public class CreatePortInstanceNodeCommand extends CompoundCommand {
	/** The graphNode that should be updated */
	private GraphNode		container;
	
	/** The EditDomain associated */
	private EditDomain		editDomain;
	
	private ICreationUtils	creationUtils;
	
	private boolean			withParameters;
	
	/**
	 * Constructor
	 * 
	 * @param domain
	 *        the EditDomain
	 * @param container
	 *        the editPart in which the new PortInstance will be added
	 * @param creationUtils
	 *        the creationUtils factory
	 * @param in
	 *        the ParameterInstance to add
	 */
	public CreatePortInstanceNodeCommand(EditDomain domain, GraphNode container, ICreationUtils creationUtils,
			boolean withParameters) {
		super("Create Port Instance GraphElement Command");
		
		this.editDomain = domain;
		this.container = container;
		this.creationUtils = creationUtils;
		this.withParameters = withParameters;
		
		initialize();
	}
	
	private void initialize() {
		EObject eObject = Utils.getElement(container);
		
		int nbPar = 0, nbIn = 0, nbOut = 0;
		// Count the instance nodes already added: AVOID the problem of an unapplicable command
		for (DiagramElement elt : container.getContained()) {
			if (elt instanceof GraphElement) {
				EObject port = Utils.getElement((GraphElement) elt);
				if (port instanceof InputInstance)
					nbIn++;
				else if (port instanceof OutputInstance)
					nbOut++;
				else if (port instanceof ParameterInstance)
					nbPar++;
			}
		}
		
		if (eObject instanceof ModelInstance) {
			ModelInstance mi = (ModelInstance) eObject;
			if (withParameters && !mi.getParameters().isEmpty() && nbPar < mi.getParameters().size())
				add(new CreatePortInstanceCommand(editDomain, container, creationUtils, mi.getParameters()));
			if (!mi.getInputs().isEmpty() && nbIn < mi.getInputs().size())
				add(new CreatePortInstanceCommand(editDomain, container, creationUtils, mi.getInputs()));
			if (!mi.getOutputs().isEmpty() && nbOut < mi.getOutputs().size())
				add(new CreatePortInstanceCommand(editDomain, container, creationUtils, mi.getOutputs()));
			add(new PositionInstanceCommand(container, withParameters));
		}
	}
}
