/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.commands;

import java.util.Map;

import org.eclipse.gef.commands.Command;

import fr.irisa.espresso.sme.Assertion;
import fr.irisa.espresso.sme.BooleanExpression;
import fr.irisa.espresso.sme.NumericExpression;

/**
 * Created 25 oct. 06
 * 
 */
public class EditExpressionCommand extends Command {
	private BooleanExpression	be;
	
	private NumericExpression	ne;
	
	private Assertion			assertion;
	
	private String				oldExpression;
	
	private String				newExpression;
	
	/**
	 * Constructor
	 * 
	 * @param be
	 *        the BooleanExpression model Object
	 * @param params
	 *        a Map that contains the new expression String
	 */
	public EditExpressionCommand(BooleanExpression be, Map<String, String> params) {
		super("Edit Boolean Expression");
		this.be = be;
		newExpression = params.get("editExpression");
	}
	
	/**
	 * Constructor
	 * 
	 * @param be
	 *        the NumericExpression model Object
	 * @param params
	 *        a Map that contains the new expression String
	 */
	public EditExpressionCommand(NumericExpression ne, Map<String, String> params) {
		super("Edit Numeric Expression");
		this.ne = ne;
		newExpression = params.get("editExpression");
	}
	
	/**
	 * Constructor
	 * 
	 * @param assertion
	 *        the Assertion model Object
	 * @param params
	 *        a Map that contains the new expression String
	 */
	public EditExpressionCommand(Assertion assertion, Map<String, String> params) {
		super("Edit Assertion Expression");
		this.assertion = assertion;
		newExpression = params.get("editExpression");
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return (be != null || ne != null || assertion != null) && newExpression != null;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		if (ne != null)
			oldExpression = ne.getExpression();
		else if (be != null)
			oldExpression = be.getExpression();
		else if (assertion != null)
			oldExpression = assertion.getExpression();
		redo();
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		if (ne != null)
			ne.setExpression(newExpression);
		else if (be != null)
			be.setExpression(newExpression);
		else if (assertion != null)
			assertion.setExpression(newExpression);
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		if (ne != null)
			ne.setExpression(oldExpression);
		else if (be != null)
			be.setExpression(oldExpression);
		else if (assertion != null)
			assertion.setExpression(oldExpression);
	}
	
}
