/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.commands;

import java.util.Map;

import org.eclipse.gef.commands.Command;

import fr.irisa.espresso.sme.Transition;

/**
 * Created 25 oct. 06
 * 
 */
public class EditGuardCommand extends Command {
	private Transition	transition;
	
	private String		oldExpression;
	
	private String		newExpression;
	
	/**
	 * Constructor
	 * 
	 * @param tr
	 *        the Transition model Object
	 * @param params
	 *        a Map that contains the new expression String
	 */
	public EditGuardCommand(Transition tr, Map<String, String> params) {
		super("Edit Guard");
		this.transition = tr;
		newExpression = params.get("editGuard");
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return (transition != null) && newExpression != null;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		if (transition != null)
			oldExpression = transition.getGuard();
		redo();
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		if (transition != null)
			transition.setGuard(newExpression);
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		if (transition != null)
			transition.setGuard(oldExpression);
	}
	
}
