/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.commands;

import org.eclipse.gef.commands.Command;

import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.StateBase;

/**
 * Creation : 08 june 2006
 */
public class SetInitStateCommand extends Command {
	private StateBase	newInitState;
	
	private StateBase	oldInitState;
	
	/**
	 * Constructor
	 * 
	 * @param element
	 */
	public SetInitStateCommand(StateBase element) {
		super("Set the node as the initial state");
		this.newInitState = element;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#canExecute()
	 */
	public boolean canExecute() {
		return newInitState != null && newInitState.getParent() instanceof Automaton;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		redo();
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		Automaton parent = (Automaton) newInitState.getParent();
		oldInitState = parent.getInitState();
		parent.setInitState(newInitState);
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		Automaton parent = (Automaton) newInitState.getParent();
		parent.setInitState(oldInitState);
	}
}
