/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;

/**
 * Created 25 oct. 06
 * 
 */
public class EditStringDialog extends Dialog implements ITopcasedDialogConstants {
	// new string value
	private String		newString;
	
	// SWT Objects
	private Composite	dialog;
	
	private Text		stringText;
	
	private String		shellText, labelText;
	
	/**
	 * Create the Dialog window that handles the string editing
	 * 
	 * @param initialComment
	 *        the initial comment
	 * @param parentShell
	 *        the parent Shell
	 */
	public EditStringDialog(String initialString, Shell parentShell, String shellText, String labelText) {
		super(parentShell);
		setBlockOnOpen(true);
		setShellStyle(getShellStyle() | SWT.RESIZE);
		if (initialString == null)
			this.newString = "";
		else
			this.newString = initialString;
		this.shellText = shellText;
		this.labelText = labelText;
	}
	
	/**
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell) {
		newShell.setText(shellText);
		newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);
		
		super.configureShell(newShell);
	}
	
	/**
	 * Create the Dialog area
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		// Dialog
		dialog = (Composite) super.createDialogArea(parent);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.widthHint = DEFAULT_DIALOG_WIDTH;
		data.heightHint = DEFAULT_DIALOG_HEIGHT;
		dialog.setLayoutData(data);
		
		// Edit comment
		Label label = new Label(dialog, SWT.WRAP);
		label.setText(labelText);
		label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		stringText = new Text(dialog, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		data = new GridData(GridData.FILL_BOTH);
		stringText.setLayoutData(data);
		stringText.setText(newString);
		
		return dialog;
	}
	
	/**
	 * Store the comment
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		newString = stringText.getText();
		
		super.okPressed();
	}
	
	/**
	 * Returns the new string value
	 * 
	 * @return the string representing the new string value
	 */
	public String getNewStringValue() {
		if (newString != null)
			return newString;
		
		return "";
	}
}
