/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;

import fr.irisa.espresso.sme.common.SmeImageRegistry;

/**
 * Created 22 mars 2006
 * 
 */
public class ImageFigure extends Figure {
	/** the image itself */
	protected Image	img;
	
	/**
	 * Constructor
	 * 
	 * @param key
	 *        the key to access to the image
	 */
	public ImageFigure(String key) {
		super();
		if (key != null && "".equals(key) == false)
			img = SmeImageRegistry.getImage(key);
	}
	
	/**
	 * Refresh the figure
	 * 
	 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
	 */
	protected void paintFigure(Graphics graphics) {
		super.paintFigure(graphics);
		
		graphics.pushState();
		
		drawImage(graphics);
		
		graphics.popState();
	}
	
	/**
	 * Draw the relevant clock relation operator image at the center of the figure
	 * 
	 */
	protected void drawImage(Graphics graphics) {
		org.eclipse.draw2d.geometry.Rectangle rectDst = getBounds();
		Rectangle rectSrc = img.getBounds();
		
		graphics.drawImage(img, rectSrc.x, rectSrc.y, rectSrc.width, rectSrc.height, rectDst.x, rectDst.y,
			rectDst.width, rectDst.height);
	}
	
	/**
	 * @param key
	 *        the key to access to the image
	 */
	public void setImage(String key) {
		img = SmeImageRegistry.getImage(key);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 * @generated NOT
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		return new Dimension(img);
	}
	
	/**
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		return new Dimension(img);
	}
}
