/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.topcased.draw2d.figures.PortFigure;

import fr.irisa.espresso.sme.common.SmeImageRegistry;

/**
 * Created 29 sept. 06
 * 
 */
public class ImagePortFigure extends PortFigure {
	private final static int	IMG_DIMENSION	= 10;
	
	/** the image itself */
	private Image				img;
	
	/**
	 * Constructor
	 * 
	 * @param key
	 *        the key to access to the image
	 */
	public ImagePortFigure(String key) {
		super();
		img = SmeImageRegistry.getImage(key);
	}
	
	/**
	 * @see org.topcased.draw2d.figures.PortFigure#fillShape(org.eclipse.draw2d.Graphics)
	 */
	protected void fillShape(Graphics graphics) {
		graphics.pushState();
		
		org.eclipse.draw2d.geometry.Rectangle rectDst = getBounds();
		Rectangle rectSrc = img.getBounds();
		
		graphics.drawImage(img, rectSrc.x, rectSrc.y, rectSrc.width, rectSrc.height, rectDst.x, rectDst.y,
			IMG_DIMENSION, IMG_DIMENSION);
		
		graphics.popState();
	}
	
	/**
	 * @see org.eclipse.draw2d.Figure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		return new Dimension(IMG_DIMENSION, IMG_DIMENSION);
	}
}
