/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ToolbarLayout;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;

/**
 * The figure to display an Image with a Label at the Bottom.
 * 
 * Created 21 March 2006
 * 
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 */
public class ImageWithLabel extends Figure implements ILabelFigure, IContainerFigure {
	/** The image that represent the model */
	private IFigure		_figure;
	
	/** A name to associate with the model */
	private ILabel		_label;
	
	/** the key to access the image */
	protected String	_key;
	
	/**
	 * The Constructor
	 * 
	 * @param key
	 *        the key to access the image
	 */
	public ImageWithLabel(String key) {
		_key = key;
		drawFigure();
	}
	
	public ImageWithLabel(IFigure body) {
		_key = null;
		_figure = body;
		drawFigure();
	}
	
	/**
	 * Initialize the figure
	 */
	protected void drawFigure() {
		ToolbarLayout layout = new ToolbarLayout();
		layout.setVertical(ToolbarLayout.HORIZONTAL);
		layout.setStretchMinorAxis(false);
		layout.setMinorAlignment(ToolbarLayout.ALIGN_CENTER);
		setLayoutManager(layout);
		
		if (getFigure() != null)
			add(getFigure());
		if (getLabel() != null)
			add(getLabel());
	}
	
	/**
	 * Creates the header figure
	 * 
	 * @return the header figure
	 */
	protected ILabel createLabel() {
		return new EditableLabel();
	}
	
	/**
	 * Creates the contentPane object.
	 * 
	 * @return the container figure
	 */
	protected IFigure createBody() {
		return new ImageFigure(_key);
	}
	
	/**
	 * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
	 */
	public ILabel getLabel() {
		if (_label == null)
			_label = createLabel();
		return _label;
	}
	
	/**
	 * Set the label of the figure
	 * 
	 * @param label
	 */
	public void setLabel(ILabel label) {
		if (label == _label)
			return;
		if (_label != null)
			remove(_label);
		_label = label;
		if (_label != null)
			add(_label);
	}
	
	/**
	 * @return the graphical part of the figure
	 */
	public IFigure getFigure() {
		if (_figure == null)
			_figure = createBody();
		return _figure;
	}
	
	/**
	 * set the graphical part of the figure
	 */
	public void setFigure(IFigure figure) {
		if (figure == _figure)
			return;
		if (_figure != null)
			remove(_figure);
		_figure = figure;
		if (_figure != null)
			add(_figure);
		if (_label != null) {
			remove(_label);
			add(_label);
		}
	}
	
	/**
	 * @param key
	 *        the key to access to the image
	 */
	public void setImage(String key) {
		_key = key;
		if (_figure instanceof ImageFigure)
			((ImageFigure) _figure).setImage(_key);
	}
	
	/**
	 * @see org.topcased.draw2d.figures.IContainerFigure#getContentPane()
	 */
	public IFigure getContentPane() {
		return getFigure();
	}
}
