/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.layout.BorderAttachedLayout;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class ModelInstanceContainerFigure extends ImageFigure implements IContainerFigure {
	private int	nbInputs	= 0;
	
	private int	nbOutputs	= 0;
	
	/**
	 * Constructor
	 * 
	 * @param key
	 *        the key specified in the images.properties file to access to the image file representing the operator
	 */
	public ModelInstanceContainerFigure() {
		super("MODELINSTANCE_LARGE");
		
		setLayoutManager(new BorderAttachedLayout());
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
	 * @generated NOT
	 */
	protected void paintFigure(Graphics graphics) {
		super.paintFigure(graphics);
		
		graphics.pushState();
		
		graphics.drawImage(img, new Rectangle(img.getBounds()), getBounds());
		
		graphics.popState();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @return itself
	 */
	public IFigure getContentPane() {
		return this;
	}
	
	/**
	 * Get the number of InputPorts
	 * 
	 * @return nbInDataPorts
	 */
	public int getNbInputs() {
		return nbInputs;
	}
	
	/**
	 * Set the number of Inputs
	 * 
	 * @param newValue
	 *        the new value
	 */
	public void setNbInputs(int newValue) {
		nbInputs = newValue;
	}
	
	/**
	 * Get the number of Outputs
	 * 
	 * @return nbOutputs
	 */
	public int getNbOutputs() {
		return nbOutputs;
	}
	
	/**
	 * Set the number of Outputs
	 * 
	 * @param newValue
	 *        the new value
	 */
	public void setNbOutputs(int newValue) {
		nbOutputs = newValue;
	}
	
	/**
	 * @see IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		Dimension dim = super.getPreferredSize(wHint, hHint);
		dim.width = Math.max(dim.width, 120);
		dim.height = Math.max(70, Math.max(dim.height, 20 * Math.max(nbInputs, nbOutputs)));
		return dim;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.common.figures.ImageFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		return new Dimension(120, Math.max(70, 20 * Math.max(nbInputs, nbOutputs)));
	}
}
