/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.topcased.draw2d.figures.IContainerFigure;

/**
 * 
 */
public class SignalOperatorFigure extends ImageFigure implements IContainerFigure {
	public final static Dimension	PORT_SIZE		= new Dimension(10, 10);
	
	private int						nbInputPorts	= 0;
	
	private int						nbOutputPorts	= 0;
	
	/**
	 * Constructor
	 * 
	 * @param key
	 *        the key specified in the images.properties file to access to the image file representing the operator
	 */
	public SignalOperatorFigure(String key) {
		super(key);
		setLayoutManager(new XYLayout());
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
	 * @generated NOT
	 */
	protected void paintFigure(Graphics graphics) {
		super.paintFigure(graphics);
		
		graphics.pushState();
		
		drawLines(graphics);
		
		graphics.popState();
	}
	
	/**
	 * Draw the lines between InputPorts and between OutputPorts
	 * 
	 * @param graphics
	 *        the graphics in which the lines can be drawed
	 */
	protected void drawLines(Graphics graphics) {
		graphics.setLineWidth(2);
		
		int space;
		
		if (nbInputPorts > 1) {
			// draw the links for all InputPorts
			space = (bounds.height - (nbInputPorts * PORT_SIZE.height)) / nbInputPorts;
			// draw the vertical line
			graphics.drawLine(bounds.getTopLeft().getTranslated(PORT_SIZE.width, space / 2), bounds.getBottomLeft()
					.getTranslated(PORT_SIZE.width, -(space / 2)));
		}
		
		if (nbOutputPorts > 1) {
			// draw the links for the OutputPorts
			space = (bounds.height - (nbOutputPorts * PORT_SIZE.height)) / nbOutputPorts;
			// draw the vertical line
			graphics.drawLine(bounds.getTopRight().getTranslated(-(PORT_SIZE.width), space / 2), bounds
					.getBottomRight().getTranslated(-(PORT_SIZE.width), -(space / 2)));
		}
		
	}
	
	/**
	 * @see fr.irisa.espresso.sme.common.figures.ImageFigure#drawImage(org.eclipse.draw2d.Graphics)
	 */
	protected void drawImage(Graphics graphics) {
		// Display the image at the center of the SignalOperator
		Dimension dim = new Dimension(img);
		
		graphics.drawImage(img, 0, 0, dim.width, dim.height, (bounds.x + PORT_SIZE.width),
			(bounds.y + (bounds.height - (dim.height)) / 2), dim.width, dim.height);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @return itself
	 */
	public IFigure getContentPane() {
		return this;
	}
	
	/**
	 * Get the number of InputPorts
	 * 
	 * @return nbInDataPorts
	 */
	public int getNbInputPorts() {
		return nbInputPorts;
	}
	
	/**
	 * Set the number of InputPorts
	 * 
	 * @param newValue
	 *        the new value
	 */
	public void setNbInputPorts(int newValue) {
		nbInputPorts = newValue;
	}
	
	/**
	 * Get the number of OutputPorts
	 * 
	 * @return nbInDataPorts
	 */
	public int getNbOutputPorts() {
		return nbOutputPorts;
	}
	
	/**
	 * Set the number of OutputPorts
	 * 
	 * @param newValue
	 *        the new value
	 */
	public void setNbOutputPorts(int newValue) {
		nbOutputPorts = newValue;
	}
	
	/**
	 * @see IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		Dimension dim;
		if (img != null)
			dim = new Dimension(img);
		else
			dim = new Dimension(0, 0);
		dim.width += ((nbInputPorts > 0) ? PORT_SIZE.width : 0) + ((nbOutputPorts > 0) ? PORT_SIZE.width : 0);
		dim.height = Math.max(dim.height, (5 + PORT_SIZE.height) * Math.max(nbInputPorts, nbOutputPorts));
		return dim;
	}
}
