/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.figures;

import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.RoundedRectangle;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;

/**
 * @generated NOT
 */
public class StateFigure extends RoundedRectangle implements ILabelFigure {
	private static final int	ROUND_CORNER	= 10;
	
	/** The imageFigure * */
	private ImageFigure			figure;
	
	/** The label of the automaton */
	private ILabel				header;
	
	/**
	 * Constructor
	 */
	public StateFigure(String key) {
		setBorder(new MarginBorder(5));
		setCornerDimensions(new Dimension(ROUND_CORNER, ROUND_CORNER));
		ToolbarLayout layout = new ToolbarLayout(true);
		layout.setSpacing(5);
		layout.setMinorAlignment(ToolbarLayout.ALIGN_CENTER);
		setLayoutManager(layout);
		
		figure = new ImageFigure(key);
		add(figure);
		
		header = createLabel();
		add(header);
	}
	
	/**
	 * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
	 */
	public ILabel getLabel() {
		return header;
	}
	
	/**
	 * Changes the label object
	 * 
	 * @param newLabel
	 *        the new object
	 */
	public void setLabel(ILabel newLabel) {
		header = newLabel;
	}
	
	/**
	 * Creates the label of the figure.<br>
	 * 
	 * @return the label of the figure
	 */
	protected ILabel createLabel() {
		EditableLabel label = new EditableLabel();
		return label;
	}
	
	/**
	 * @param isInitState
	 *        true if the state must be drawn as the initial state
	 */
	public void setInitState(boolean isInitState) {
		if (isInitState)
			this.setLineWidth(3);
		else
			this.setLineWidth(1);
	}
}
