/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.ModelerGraphicalViewer;

import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.IterationInit;
import fr.irisa.espresso.sme.common.SmeRequestConstants;
import fr.irisa.espresso.sme.common.commands.ChangeAspectCommand;

/**
 * 
 * @generated
 */
public class CustomEditPolicy extends DiagramLayoutEditPolicy {
	/**
	 * Default contructor. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CustomEditPolicy() {
		super();
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject,
	 *      org.eclipse.emf.ecore.EObject)
	 * @generated NOT
	 */
	protected boolean isValid(EObject child, EObject parent) {
		// This method is added to avoid adding automatically the InputPort or IterationInit object in an Iteration
		// Model
		return super.isValid(child, parent) && !((child instanceof InputPort) || (child instanceof IterationInit));
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#getCommand(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public Command getCommand(Request request) {
		if (SmeRequestConstants.REQ_GO_TO_INTERFACE_ASPECT.equals(request.getType())) {
			ModelerGraphicalViewer modeler = (ModelerGraphicalViewer) getHost().getViewer();
			GraphNode node = (GraphNode) getHost().getModel();
			return new ChangeAspectCommand(node, modeler, "fr.irisa.espresso.sme.diagram.interface_");
		}
		else if (SmeRequestConstants.REQ_GO_TO_DATAFLOW_ASPECT.equals(request.getType())) {
			ModelerGraphicalViewer modeler = (ModelerGraphicalViewer) getHost().getViewer();
			GraphNode node = (GraphNode) getHost().getModel();
			return new ChangeAspectCommand(node, modeler, "fr.irisa.espresso.sme.diagram.dataflow");
		}
		else if (SmeRequestConstants.REQ_GO_TO_CLOCKRELATION_ASPECT.equals(request.getType())) {
			ModelerGraphicalViewer modeler = (ModelerGraphicalViewer) getHost().getViewer();
			GraphNode node = (GraphNode) getHost().getModel();
			return new ChangeAspectCommand(node, modeler, "fr.irisa.espresso.sme.diagram.clockrelations");
		}
		return super.getCommand(request);
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#understandsRequest(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public boolean understandsRequest(Request req) {
		Modeler modeler = ((ModelerGraphicalViewer) getHost().getViewer()).getModelerEditor();
		Diagram currentDiag = modeler.getActiveDiagram();
		if (SmeRequestConstants.REQ_GO_TO_INTERFACE_ASPECT.equals(req.getType()))
			return !(currentDiag.getSemanticModel().getPresentation()
					.equals("fr.irisa.espresso.sme.diagram.interface_"));
		else if (SmeRequestConstants.REQ_GO_TO_DATAFLOW_ASPECT.equals(req.getType()))
			return !(currentDiag.getSemanticModel().getPresentation().equals("fr.irisa.espresso.sme.diagram.dataflow"));
		else if (SmeRequestConstants.REQ_GO_TO_CLOCKRELATION_ASPECT.equals(req.getType()))
			return !(currentDiag.getSemanticModel().getPresentation()
					.equals("fr.irisa.espresso.sme.diagram.clockrelations"));
		return super.understandsRequest(req);
	}
	
}
