/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.AbstractEditPolicy;
import org.topcased.modeler.ModelerRequestConstants;
import org.topcased.modeler.commands.DeleteModelCommand;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.SignalOperator;

/**
 * The EditPolicy that handle the Delete Model Object command for SME ports. It must be at least two InputPorts and two
 * OutputPorts for SignalOperators
 * 
 * Created 4 avr. 2006
 * 
 */
public class DeletePortEditPolicy extends AbstractEditPolicy {
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#getCommand(org.eclipse.gef.Request)
	 */
	public Command getCommand(Request request) {
		if (ModelerRequestConstants.REQ_DELETE_MODEL_OBJECT.equals(request.getType())) {
			EObject model = null;
			if (getHost() instanceof EMFGraphNodeEditPart) {
				GraphElement element = (GraphElement) getHost().getModel();
				model = Utils.getElement(element);
			}
			
			if (model != null) {
				if (model instanceof InputPort) {
					SignalOperator sigOp = ((InputPort) model).getOperatorParent();
					if (sigOp.getInputPorts().size() <= 2) {
						return null;
					}
				}
				else if (model instanceof OutputPort) {
					SignalOperator sigOp = ((OutputPort) model).getOperatorParent();
					if (sigOp.getOutputPorts().size() <= 2) {
						return null;
					}
				}
				
				if (getHost().getViewer().getEditDomain() instanceof MixedEditDomain) {
					MixedEditDomain domain = ((MixedEditDomain) getHost().getViewer().getEditDomain());
					Diagrams diagrams = ((Modeler) domain.getEditorPart()).getDiagrams();
					if (domain != null && diagrams != null) {
						DeleteModelCommand command = new DeleteModelCommand(domain, diagrams, model);
						return command;
					}
				}
			}
		}
		return null;
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		if (ModelerRequestConstants.REQ_DELETE_MODEL_OBJECT.equals(req.getType())) {
			return true;
		}
		return false;
	}
	
}
