/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.common.policies;

import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.AbstractEditPolicy;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Assertion;
import fr.irisa.espresso.sme.BooleanExpression;
import fr.irisa.espresso.sme.NumericExpression;
import fr.irisa.espresso.sme.common.SmeRequestConstants;
import fr.irisa.espresso.sme.common.commands.EditExpressionCommand;

/**
 * 
 */
public class ExpressionEditPolicy extends AbstractEditPolicy {
	/**
	 * Default contructor
	 * 
	 */
	public ExpressionEditPolicy() {
		super();
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#getCommand(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public Command getCommand(Request request) {
		if (SmeRequestConstants.REQ_EDIT_EXPRESSION.equals(request.getType())) {
			EObject obj = Utils.getElement((GraphElement) getHost().getModel());
			Map<String, String> params = request.getExtendedData();
			if (obj instanceof BooleanExpression)
				return new EditExpressionCommand((BooleanExpression) obj, params);
			else if (obj instanceof NumericExpression)
				return new EditExpressionCommand((NumericExpression) obj, params);
			else if (obj instanceof Assertion)
				return new EditExpressionCommand((Assertion) obj, params);
		}
		return null;
	}
	
	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#understandsRequest(org.eclipse.gef.Request)
	 * @generated NOT
	 */
	public boolean understandsRequest(Request req) {
		EObject obj = Utils.getElement((GraphElement) getHost().getModel());
		if (SmeRequestConstants.REQ_EDIT_EXPRESSION.equals(req.getType()))
			return (obj instanceof BooleanExpression || obj instanceof NumericExpression || obj instanceof Assertion);
		return false;
	}
}
