/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.modeler.actions;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.ui.IWorkbenchPart;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Iterate;
import fr.irisa.espresso.sme.MultipleInputsOperator;
import fr.irisa.espresso.sme.common.SmeActionConstants;
import fr.irisa.espresso.sme.common.SmeRequestConstants;

/**
 * created 27 june 2006
 * 
 */
public class AddNewPortAction extends WorkbenchPartAction implements ISelectionChangedListener {
	/**
	 * The selected EditPart object
	 */
	private EMFGraphNodeEditPart	template;
	
	/**
	 * @param part
	 */
	public AddNewPortAction(IWorkbenchPart part) {
		super(part);
	}
	
	/**
	 * Add a new port
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		// construct the AddNewPortRequest
		Request request = new Request(SmeRequestConstants.REQ_ADD_NEW_PORT);
		
		// get the Command
		Command actionCommand = template.getCommand(request);
		
		// execute the command
		getCommandStack().execute(actionCommand);
	}
	
	/**
	 * Determine if the action must appear in the context menu
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		if (template == null)
			return false;
		
		// construct the AddNewPortRequest
		Request request = new Request(SmeRequestConstants.REQ_ADD_NEW_PORT);
		
		// return true if the EditPart can understand the Request
		return template.understandsRequest(request);
	}
	
	/**
	 * Initializes the delete model object action
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		setId(SmeActionConstants.ADD_NEW_PORT);
		setText("Add a New Port");
	}
	
	/**
	 * Sets the selected EditPart and refreshes the enabled state of this action.
	 * 
	 * @param event
	 * 
	 * @see ISelectionChangedListener#selectionChanged(SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		ISelection s = event.getSelection();
		if (!(s instanceof IStructuredSelection))
			return;
		IStructuredSelection selection = (IStructuredSelection) s;
		template = null;
		
		if (selection != null && selection.size() == 1) {
			Object obj = selection.getFirstElement();
			if (obj instanceof EMFGraphNodeEditPart) {
				EMFGraphNodeEditPart emfGraphNodeEditPart = (EMFGraphNodeEditPart) obj;
				EObject model = Utils.getElement((GraphNode) emfGraphNodeEditPart.getModel());
				// the action is available only for MultipleInputsOperator and Iterate model objects
				if (model instanceof MultipleInputsOperator || model instanceof Iterate)
					template = emfGraphNodeEditPart;
			}
		}
		
		refresh();
	}
	
}
