/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.modeler.actions;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPart;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Assertion;
import fr.irisa.espresso.sme.BooleanExpression;
import fr.irisa.espresso.sme.NumericExpression;
import fr.irisa.espresso.sme.common.SmeActionConstants;
import fr.irisa.espresso.sme.common.SmeRequestConstants;
import fr.irisa.espresso.sme.common.dialogs.EditStringDialog;

/**
 * Created 25 oct. 06
 * 
 */
public class EditExpressionAction extends WorkbenchPartAction implements ISelectionChangedListener {
	/**
	 * The selected EditPart object
	 */
	private AbstractGraphicalEditPart	template;
	
	/**
	 * @param part
	 */
	public EditExpressionAction(IWorkbenchPart part) {
		super(part);
	}
	
	/**
	 * Edit the expression of a BooleanExpression or a NumericExpression
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		GraphElement graphElement = (GraphElement) template.getModel();
		EObject obj = Utils.getElement(graphElement);
		String currentExpression = null;
		if (obj instanceof NumericExpression)
			currentExpression = ((NumericExpression) obj).getExpression();
		else if (obj instanceof BooleanExpression)
			currentExpression = ((BooleanExpression) obj).getExpression();
		else if (obj instanceof Assertion)
			currentExpression = ((Assertion) obj).getExpression();
		
		EditStringDialog editDialog = new EditStringDialog(currentExpression, new Shell(), "Expression Editing",
			"Edit composite expression:");
		if (editDialog.open() == Window.OK) {
			// construct the EditCommentRequest
			Request request = new Request(SmeRequestConstants.REQ_EDIT_EXPRESSION);
			Map<String, String> params = new HashMap<String, String>();
			params.put("editExpression", editDialog.getNewStringValue());
			request.setExtendedData(params);
			
			// get the Command
			Command actionCommand = template.getCommand(request);
			
			// execute the command
			getCommandStack().execute(actionCommand);
		}
	}
	
	/**
	 * Determine if the action must appear in the context menu
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		if (template == null)
			return false;
		
		Request request = new Request(SmeRequestConstants.REQ_EDIT_EXPRESSION);
		
		return template.understandsRequest(request);
	}
	
	/**
	 * Initializes the edit comment action
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		setId(SmeActionConstants.EDIT_EXPRESSION);
		setText("Edit Composite Expression");
	}
	
	/**
	 * Sets the selected EditPart and refreshes the enabled state of this action.
	 * 
	 * @param event
	 * 
	 * @see ISelectionChangedListener#selectionChanged(SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		ISelection s = event.getSelection();
		if (!(s instanceof IStructuredSelection))
			return;
		IStructuredSelection selection = (IStructuredSelection) s;
		template = null;
		
		if (selection != null && selection.size() == 1) {
			Object obj = selection.getFirstElement();
			if (obj instanceof AbstractGraphicalEditPart)
				template = (AbstractGraphicalEditPart) obj;
		}
		refresh();
	}
	
}
