/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.modeler.actions;

import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.ui.IWorkbenchPart;
import org.topcased.modeler.ModelerActionConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.diagrams.model.util.DiagramsUtils;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.utils.Utils;

import fr.irisa.espresso.sme.Specifications;
import fr.irisa.espresso.sme.modeler.SmePlugin;

/**
 * Created 6 nov. 06
 */
public class OpenParentDiagramAction extends WorkbenchPartAction {
	/**
	 * The Constructor
	 * 
	 * @param part
	 *        the IWorkbenchPart
	 */
	public OpenParentDiagramAction(IWorkbenchPart part) {
		super(part);
	}
	
	/**
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		return true;
	}
	
	/**
	 * Initializes the action
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		setId(ModelerActionConstants.OPEN_PARENT_DIAGRAM);
		setText("Open a Parent Diagram");
	}
	
	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		// get the modeler
		Modeler modeler = (Modeler) getWorkbenchPart();
		
		List<Diagram> possibleDiagrams = null;
		
		// get the current diagram
		Diagram currentDiagram = modeler.getActiveDiagram();
		
		// get the model objects (the model object associated with the current
		// diagram and its parent in the model
		EObject modelObject = Utils.getElement(currentDiagram.getSemanticModel().getGraphElement());
		EObject currentModelObject = modelObject;
		
		while (modelObject.eContainer() != null && (possibleDiagrams == null || possibleDiagrams.isEmpty())) {
			EObject parentModelObject = modelObject.eContainer();
			
			// We search if a diagram already exists
			possibleDiagrams = DiagramsUtils.findAllExistingDiagram(modeler.getDiagrams(), parentModelObject);
			
			modelObject = parentModelObject;
		}
		
		if (possibleDiagrams != null && !possibleDiagrams.isEmpty()) {
			Diagram selectedDiagram = null;
			if (possibleDiagrams.size() == 1)
				selectedDiagram = (Diagram) possibleDiagrams.get(0);
			else if (possibleDiagrams.size() > 1) {
				if (currentModelObject instanceof Specifications) {
					// If the currentModelObject parent is a Specifications and there are
					// several parent diagrams, we use the interface one.
					for (Diagram diag : possibleDiagrams) {
						if (diag.getSemanticModel().getPresentation()
								.equals("fr.irisa.espresso.sme.diagram.interface_")) {
							selectedDiagram = diag;
							break;
						}
					}
				}
				else {
					String currentDiagramId = currentDiagram.getSemanticModel().getPresentation();
					// We try to go in the same kind of diagram than the current one
					for (Diagram diag : possibleDiagrams) {
						if (diag.getSemanticModel().getPresentation().equals(currentDiagramId)) {
							selectedDiagram = diag;
							break;
						}
					}
				}
				if (selectedDiagram == null) {
					// By default, we select the first one
					selectedDiagram = possibleDiagrams.get(0);
				}
			}
			if (selectedDiagram != null)
				modeler.setActiveDiagram(selectedDiagram);
			
		}
		else {
			SmePlugin.displayDialog("No parent Diagram", "There is no parent diagram", IStatus.INFO);
		}
		
	}
}
