/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.modeler.editor;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.topcased.modeler.editor.ModelerContextMenuProvider;

import fr.irisa.espresso.sme.common.SmeActionConstants;

/**
 * Provide a Context Menu for the Polychrony Graphical Editor with specific actions
 * 
 * @generated NOT
 */
public class SmeContextMenuProvider extends ModelerContextMenuProvider {
	/**
	 * Constructs a context menu for the specified EditPartViewer and the Actions registered in the ActionRegistry
	 * 
	 * @param viewer
	 *        the EditPartViewer
	 * @param registry
	 *        the ActionRegistry
	 */
	public SmeContextMenuProvider(EditPartViewer viewer, ActionRegistry registry) {
		super(viewer, registry);
	}
	
	/**
	 * Called when the menu is about to show. Construct the context menu by adding the "add new port" actions common to
	 * all editparts.
	 * 
	 * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void buildContextMenu(IMenuManager manager) {
		super.buildContextMenu(manager);
		
		IAction action = getActionRegistry().getAction(SmeActionConstants.ADD_NEW_PORT);
		if (action.isEnabled())
			manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
		
		action = getActionRegistry().getAction(SmeActionConstants.EDIT_COMMENT);
		if (action.isEnabled())
			manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
		
		action = getActionRegistry().getAction(SmeActionConstants.EDIT_EXPRESSION);
		if (action.isEnabled())
			manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
		
		action = getActionRegistry().getAction(SmeActionConstants.EDIT_GUARD);
		if (action.isEnabled())
			manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
		
		action = getActionRegistry().getAction(SmeActionConstants.SHOW_ITERATION_INIT);
		if (action.isEnabled())
			manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
		
		action = getActionRegistry().getAction(SmeActionConstants.RELOAD_MODELINSTANCE);
		if (action.isEnabled())
			manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
		
		action = getActionRegistry().getAction(SmeActionConstants.SET_INIT_STATE);
		if (action.isEnabled())
			manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
		
		MenuManager subMenu = new MenuManager("Go To Aspect...");
		boolean subMenuEnabled, res;
		
		action = getActionRegistry().getAction(SmeActionConstants.GO_TO_INTERFACE_ASPECT);
		subMenuEnabled = action.isEnabled();
		if (subMenuEnabled)
			subMenu.add(action);
		
		action = getActionRegistry().getAction(SmeActionConstants.GO_TO_DATAFLOW_ASPECT);
		res = action.isEnabled();
		subMenuEnabled = subMenuEnabled || res;
		if (res)
			subMenu.add(action);
		
		action = getActionRegistry().getAction(SmeActionConstants.GO_TO_CLOCKRELATION_ASPECT);
		res = action.isEnabled();
		subMenuEnabled = subMenuEnabled || res;
		if (res)
			subMenu.add(action);
		
		if (subMenuEnabled)
			manager.appendToGroup(GEFActionConstants.GROUP_EDIT, subMenu);
	}
}
