/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.modeler.editor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ProjectScope;
import org.eclipse.core.runtime.Platform;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.preferences.ScopedPreferenceStore;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;
import org.topcased.modeler.commands.GEFtoEMFCommandStackWrapper;
import org.topcased.modeler.documentation.EAnnotationDocPage;
import org.topcased.modeler.documentation.IDocPage;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.ModelerGraphicalViewer;

import fr.irisa.espresso.sme.common.SmeActionConstants;
import fr.irisa.espresso.sme.modeler.SmePlugin;
import fr.irisa.espresso.sme.modeler.actions.AddNewPortAction;
import fr.irisa.espresso.sme.modeler.actions.EditCommentAction;
import fr.irisa.espresso.sme.modeler.actions.EditExpressionAction;
import fr.irisa.espresso.sme.modeler.actions.EditGuardAction;
import fr.irisa.espresso.sme.modeler.actions.GoToClockRelationAspectAction;
import fr.irisa.espresso.sme.modeler.actions.GoToDataflowAspectAction;
import fr.irisa.espresso.sme.modeler.actions.GoToInterfaceAspectAction;
import fr.irisa.espresso.sme.modeler.actions.LoadModelInstanceAction;
import fr.irisa.espresso.sme.modeler.actions.OpenParentDiagramAction;
import fr.irisa.espresso.sme.modeler.actions.SetInitStateAction;
import fr.irisa.espresso.sme.modeler.actions.ShowIterationInitBehaviorAction;

/*
 * Generated Model editor
 * 
 * @generated
 */
public class SmeEditor extends Modeler {
	
	public static final String	EDITOR_ID	= "fr.irisa.espresso.sme.modeler.editor.SmeEditor";
	
	/**
	 * @see org.topcased.modeler.editor.Modeler#getAdapterFactories()
	 * @generated
	 */
	protected List getAdapterFactories() {
		List factories = new ArrayList();
		factories.add(new fr.irisa.espresso.sme.provider.SmeItemProviderAdapterFactory());
		factories.add(new fr.irisa.espresso.sme.modeler.providers.SmeModelerProviderAdapterFactory());
		
		factories.addAll(super.getAdapterFactories());
		
		return factories;
	}
	
	/**
	 * @see org.topcased.modeler.editor.Modeler#getId()
	 * @generated
	 */
	public String getId() {
		return EDITOR_ID;
	}
	
	/**
	 * @see org.topcased.modeler.editor.Modeler#getAdapter(java.lang.Class)
	 * @generated NOT
	 */
	public Object getAdapter(Class type) {
		if (type == IDocPage.class) {
			GEFtoEMFCommandStackWrapper wrapper = new GEFtoEMFCommandStackWrapper(getCommandStack());
			return new EAnnotationDocPage(wrapper);
		}
		return super.getAdapter(type);
	}
	
	/**
	 * @see org.topcased.modeler.editor.Modeler#getPreferenceStore()
	 * @generated
	 */
	public IPreferenceStore getPreferenceStore() {
		IProject project = (((IFileEditorInput) getEditorInput()).getFile()).getProject();
		
		Preferences root = Platform.getPreferencesService().getRootNode();
		try {
			if (root.node(ProjectScope.SCOPE).node(project.getName()).nodeExists(SmePlugin.getId())) {
				return new ScopedPreferenceStore(new ProjectScope(project), SmePlugin.getId());
			}
		}
		catch (BackingStoreException e) {
			e.printStackTrace();
		}
		return SmePlugin.getDefault().getPreferenceStore();
	}
	
	/**
	 * @see org.topcased.modeler.editor.Modeler#createActions()
	 * @generated NOT
	 */
	protected void createActions() {
		super.createActions();
		
		ActionRegistry registry = getActionRegistry();
		IAction action = new AddNewPortAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new EditCommentAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new EditExpressionAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new EditGuardAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new ShowIterationInitBehaviorAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new GoToInterfaceAspectAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new GoToDataflowAspectAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new GoToClockRelationAspectAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new LoadModelInstanceAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		action = new SetInitStateAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		// We modify the open parent diagram action
		// Diagram navigation Action
		action = new OpenParentDiagramAction(this);
		registry.registerAction(action);
	}
	
	/**
	 * @see org.topcased.modeler.editor.Modeler#configureGraphicalViewer()
	 * @generated NOT
	 */
	protected void configureGraphicalViewer() {
		super.configureGraphicalViewer();
		
		GraphicalViewer viewer = getGraphicalViewer();
		
		KeyHandler keyHandler = viewer.getKeyHandler();
		
		IAction action = getActionRegistry().getAction(SmeActionConstants.ADD_NEW_PORT);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.EDIT_COMMENT);
		keyHandler.put(KeyStroke.getPressed(SWT.F4, 0), action);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.EDIT_EXPRESSION);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.EDIT_GUARD);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.SHOW_ITERATION_INIT);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.GO_TO_INTERFACE_ASPECT);
		keyHandler.put(KeyStroke.getPressed(SWT.F9, 0), action);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.GO_TO_DATAFLOW_ASPECT);
		keyHandler.put(KeyStroke.getPressed(SWT.F10, 0), action);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.GO_TO_CLOCKRELATION_ASPECT);
		keyHandler.put(KeyStroke.getPressed(SWT.F11, 0), action);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.RELOAD_MODELINSTANCE);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
		
		action = getActionRegistry().getAction(SmeActionConstants.SET_INIT_STATE);
		viewer.addSelectionChangedListener((ISelectionChangedListener) action);
	}
	
	/**
	 * @see org.topcased.modeler.editor.Modeler#getContextMenuProvider(org.topcased.modeler.editor.ModelerGraphicalViewer)
	 * @generated NOT
	 */
	protected ContextMenuProvider getContextMenuProvider(ModelerGraphicalViewer viewer) {
		return new SmeContextMenuProvider(viewer, getActionRegistry());
	}
}
