/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.modeler.wizards;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.ide.IDE;
import org.topcased.modeler.extensions.Template;
import org.topcased.modeler.extensions.TemplatesManager;
import org.topcased.modeler.tools.DiagramFileInitializer;
import org.topcased.modeler.wizards.DiagramsPage;

import fr.irisa.espresso.sme.modeler.SmeImageRegistry;
import fr.irisa.espresso.sme.modeler.SmePlugin;

/*
 * Generated wizard that offers the model creation facilities.
 * 
 * @generated
 */
public class NewSmeDiagrams extends Wizard implements INewWizard {
	/**
	 * @generated
	 */
	private IStructuredSelection	selection;
	
	/**
	 * @generated
	 */
	private DiagramsPage			diagPage;
	
	/**
	 * @generated
	 */
	private IFile					createdFile;
	
	/**
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 *      org.eclipse.jface.viewers.IStructuredSelection)
	 * @generated
	 */
	public void init(IWorkbench workbench, IStructuredSelection sel) {
		createdFile = null;
		selection = sel;
		
		setDefaultPageImageDescriptor(SmeImageRegistry.getImageDescriptor("NEW_PAGE_WZD"));
		setDialogSettings(SmePlugin.getDefault().getDialogSettings());
		setWindowTitle("Create new Sme diagrams");
	}
	
	/**
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 * @generated
	 */
	public boolean performFinish() {
		boolean result = true;
		if (diagPage.isPageComplete()) {
			if (diagPage.isNewModel()) {
				result = result & createModelFile();
				result = result & createDiagramFile();
				if (createdFile != null && result) {
					// Open the newly created model
					try {
						IDE.openEditor(
							SmePlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage(),
							createdFile, true);
					}
					catch (PartInitException pie) {
						SmePlugin.log(pie);
					}
				}
			}
			else {
				createDiagramFromExistingModel();
			}
		}
		return result;
	}
	
	/**
	 * @generated
	 */
	private boolean createDiagramFromExistingModel() {
		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			/**
			 * @see org.eclipse.ui.actions.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
			 */
			protected void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException,
					InterruptedException {
				DiagramFileInitializer initializer = new DiagramFileInitializer();
				try {
					initializer.createDiagram(diagPage.getRootEObject(), diagPage.getDiagramId(), diagPage
							.isInitialized(), monitor);
				}
				catch (IOException ioe) {
					throw new InvocationTargetException(ioe);
				}
			}
		};
		
		try {
			getContainer().run(false, true, op);
			return true;
		}
		catch (InvocationTargetException ite) {
			SmePlugin.log(ite);
		}
		catch (InterruptedException ie) {
			// Wizard stopped
		}
		return false;
	}
	
	/**
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 * @generated
	 */
	public void addPages() {
		diagPage = new SmeDiagramsPage("New Polychrony Graphical Modeler Diagram", selection);
		diagPage.setTitle("Sme Diagrams");
		diagPage.setDescription("Define the model diagram informations.");
		addPage(diagPage);
	}
	
	/**
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 * @generated
	 */
	public boolean canFinish() {
		return diagPage.isPageComplete();
	}
	
	/**
	 * @return true if the model file was successfully created
	 * @generated
	 */
	private boolean createModelFile() {
		try {
			Template template = TemplatesManager.getInstance().find(diagPage.getTemplateId()).getTemplateModel();
			template.setDestination(diagPage.getSelectedIContainer());
			template.addVariable("name", diagPage.getModelName());
			
			template.generate(new NullProgressMonitor());
		}
		catch (CoreException ce) {
			SmePlugin.log(ce);
			SmePlugin.displayDialog(null, "An error occured during the template generation.", IStatus.ERROR);
			return false;
		}
		return true;
	}
	
	/**
	 * @return true if the diagram file was successfully created
	 * @generated
	 */
	private boolean createDiagramFile() {
		try {
			Template template = TemplatesManager.getInstance().find(diagPage.getTemplateId()).getTemplateDI();
			template.setDestination(diagPage.getSelectedIContainer());
			template.addVariable("name", diagPage.getModelName());
			// Bug #1395 : Add an additional variable used to encode the model file name
			template.addVariable("escapedName", URI.encodeFragment(diagPage.getModelName(), false));
			
			createdFile = (IFile) template.generate(new NullProgressMonitor());
		}
		catch (CoreException ce) {
			SmePlugin.log(ce);
			SmePlugin.displayDialog(null, "An error occured during the template generation.", IStatus.ERROR);
			return false;
		}
		return true;
	}
}
