/* 
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/

/*
  Java-C interface for Signal simulation (C code generated by Signal compiler).
*/

#include <jni.h> 
#include "SimuC.h"
#include "SimuCToJava.h"

static JNIEnv *env_appel; 
static jclass cls; 
static jobject obj_appel; 

static jmethodID mid_createFrame;
static jmethodID mid_createButton;
static jmethodID mid_createLabel;
static jmethodID mid_createPanel;
static jmethodID mid_createCanvas;
static jmethodID mid_createTextArea;
static jmethodID mid_createSlider;
static jmethodID mid_createCheckBox;
static jmethodID mid_readButton; 
static jmethodID mid_readSlider;
static jmethodID mid_readCheckBox;
static jmethodID mid_drawLine;
static jmethodID mid_clearCanvas;
static jmethodID mid_fillRect;
static jmethodID mid_drawInteger;
static jmethodID mid_drawReal;
static jmethodID mid_setTextArea;
static jmethodID mid_quitAppli;
static jmethodID mid_createBorder;
static jmethodID mid_getcolor;

/* C --> Java link.
   THE INTERFACE MUST BE COHERENT WITH THE JAVA CODE.
   For example:
      mid_createFrame=(*env_appel)->GetMethodID(env_appel,cls,"createFrame","(II)I"); 
   is the coding of the method:
    public int createFrame(int dimx, int dimy) ...
*/
static void savID(JNIEnv *env, jobject obj) 
{ 
  cls = (*env)->GetObjectClass(env,obj); 
  env_appel= env; 
  obj_appel= obj; 
  mid_createFrame=(*env_appel)->GetMethodID(env_appel,cls,"createFrame","(II)I"); 
  mid_createButton=(*env_appel)->GetMethodID(env_appel,cls,"createButton","(I)I"); 
  mid_createLabel=(*env_appel)->GetMethodID(env_appel,cls,"createLabel","(I)I"); 
  mid_createPanel=(*env_appel)->GetMethodID(env_appel,cls,"createPanel","(II)I"); 
  mid_createTextArea=(*env_appel)->GetMethodID(env_appel,cls,"createTextArea","(III)I");
  mid_createCanvas=(*env_appel)->GetMethodID(env_appel,cls,"createCanvas","(III)I"); 
  mid_createSlider=(*env_appel)->GetMethodID(env_appel,cls,"createSlider","(IIIIIII)I");
  mid_createCheckBox=(*env_appel)->GetMethodID(env_appel,cls,"createCheckBox","(IZ)I"); 
  mid_readButton=(*env_appel)->GetMethodID(env_appel,cls,"readButton","(I)Z");
  mid_readSlider=(*env_appel)->GetMethodID(env_appel,cls,"readSlider","(I)I");
  mid_readCheckBox=(*env_appel)->GetMethodID(env_appel,cls,"readCheckBox","(I)Z");
  mid_drawLine=(*env_appel)->GetMethodID(env_appel,cls,"drawLine","(IIIIII)V");
  mid_clearCanvas=(*env_appel)->GetMethodID(env_appel,cls,"clearCanvas","(IIII)V");
  mid_fillRect=(*env_appel)->GetMethodID(env_appel,cls,"fillRect","(IIIIII)V");
  mid_drawInteger=(*env_appel)->GetMethodID(env_appel,cls,"drawInteger","(IIII)V");
  mid_drawReal=(*env_appel)->GetMethodID(env_appel,cls,"drawReal","(IDII)V");
  mid_setTextArea=(*env_appel)->GetMethodID(env_appel,cls,"setTextArea","(II)V");
  mid_quitAppli=(*env_appel)->GetMethodID(env_appel,cls,"quitAppli","()V");
  mid_createBorder=(*env_appel)->GetMethodID(env_appel,cls,"createBorder","(III)V");
  mid_getcolor=(*env_appel)->GetMethodID(env_appel,cls,"getColor","(IIIZ)I");
}



/* Java --> C link.
   This method is called from Java to initialize the Signal Simultation.
   Then it saves values used for C to Java link.
   Note: initSimu must be defined by the user.
*/
JNIEXPORT void JNICALL Java_Signal_lib_Java_graphical_SimuC_n_1initSimu /*Java_SimuC_n_1initSimu*/ (JNIEnv *env, jobject obj) 
{ 
  savID(env, obj);
  initSimu();
} 

/* Java --> C link.
   This method is called from Java for each step of Signal Simultation.
   Note: iterateSimu must be defined by the user.
*/
JNIEXPORT jboolean JNICALL Java_Signal_lib_Java_graphical_SimuC_n_1iterate /*Java_SimuC_n_1iterate*/ (JNIEnv *env, jobject obj)
{
  savID(env, obj);
  return iterateSimu();
}


/* C --> Java link.
   The following functions are called by the C-Signal program for graphical i/o.
   It is the implementation of the libGraphics.SIG library provided by polychrony.
*/

extern int createFrame(int x, int y)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_createFrame, (jint)x, (jint) y));
}
 
extern int createPanel(int father,int position)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_createPanel,father, position)); 
}

extern int createButton(int num)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_createButton, num)); 
}

extern int createLabel(int num)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_createLabel, num)); 
}

extern int createCanvas(int num,int x, int y)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_createCanvas, num,x,y)); 
}

extern int createTextArea(int num ,int x, int y){
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_createTextArea, num,x,y)); 
}

extern int createSlider(int num,int min, int max, int defaut,int x, int y, int tickDisplay)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_createSlider,num,min,max,defaut,x,y,tickDisplay ));
}

extern int createCheckBox(int num, int state)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_createCheckBox,num,state));
}

extern int createBorder(int num,int x, int y)
{
  (*env_appel)->CallVoidMethod(env_appel,obj_appel, mid_createBorder,num,x,y);
  return 1;
}

extern void setTextArea(int numArea,int valeur)
{
  (*env_appel)->CallVoidMethod(env_appel,obj_appel, mid_setTextArea,numArea,valeur);
}

extern int readButton(int numButton)
{
  return ((*env_appel)->CallBooleanMethod(env_appel,obj_appel, mid_readButton, numButton)); 
}

extern int readSlider (int numSlider)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_readSlider,numSlider));
}

extern int readCheckBox (int numCheckBox)
{
  return ((*env_appel)->CallBooleanMethod(env_appel,obj_appel, mid_readCheckBox,numCheckBox));
}

extern void drawLine(int numCanvas, int x1, int y1, int y2, int vcolor)
{
  (*env_appel)->CallVoidMethod(env_appel,obj_appel, mid_drawLine,numCanvas,x1,y1,x1,y2, vcolor);
}

extern void clearCanvas(int numCanvas, int x1,int dx,int dy)
{
  (*env_appel)->CallVoidMethod(env_appel,obj_appel, mid_clearCanvas,numCanvas,x1,dx,dy);
}

extern void fillRect(int numCanvas, int x,int y,int dx,int dy, int vcolor)
{
  (*env_appel)->CallVoidMethod(env_appel,obj_appel, mid_fillRect,numCanvas,x,y,dx,dy, vcolor);
}

extern void drawInteger(int numCanvas, int val,int x,int y)
{
  (*env_appel)->CallVoidMethod(env_appel,obj_appel, mid_drawInteger,numCanvas,val,x,y);
}

extern void drawReal(int numCanvas, double val,int x,int y)
{
  (*env_appel)->CallVoidMethod(env_appel,obj_appel, mid_drawReal, numCanvas, val,x,y);
}

extern void quitAppli(void)
{
  (*env_appel)->CallVoidMethod(env_appel,obj_appel, mid_quitAppli);
}

extern int getColor (int R, int G, int B, int brightness)
{
  return ((*env_appel)->CallIntMethod(env_appel,obj_appel, mid_getcolor,R,G,B,brightness));
}
