/***Polychrony License******************************************
 *                                                             *
 * Copyright: IRISA/INRIA Rennes - ESPRESSO team.              *
 * This software is under the terms of the CeCILL-B License.   *
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html *
 *                                                             *
 ***************************************************************/
#ifndef __QUEUINGPORTMANAGER_H__
#define __QUEUINGPORTMANAGER_H__

#include "TYPES.h"

/**********************************************************************************/
/****************************  QUEUING PORT OBJECT : BEGIN *************************/
/**********************************************************************************/

/* Object representing APEX Queuing Port */
typedef class QueuingPort * ptQueuingPort;
class QueuingPort
{
   public :
   APEX_QueuingPort_type queuingport;
   QueuingPort(APEX_QueuingPort_type s);
};

/**********************************************************************************/
/****************************  QUEUING PORT OBJECT : END   ************************/
/**********************************************************************************/




/**********************************************************************************/
/*********************** QUEUING PORT MANAGER_NODE  : BEGIN   ***********************/
/**********************************************************************************/

/* Type of manager node contents */
struct QueuingPortAssoc_type
{
  Comm_ComponentID_type QueuingPort_ID; 
  ptQueuingPort ptrQueuingPort;
};

/* Manager node type definition */
typedef class QueuingPortManagerNode *ptQueuingPortManagerNode;
class QueuingPortManagerNode
{
 private :
  QueuingPortAssoc_type Contents;                             /* Current node contents */
  ptQueuingPortManagerNode next;                                     /* Next node */
  
 public :
  QueuingPortManagerNode();
  QueuingPortManagerNode(QueuingPortAssoc_type sampling_port);
  ~QueuingPortManagerNode();
  void setContents(QueuingPortAssoc_type contents);
  QueuingPortAssoc_type getContents();
  void setNext(ptQueuingPortManagerNode node);
  ptQueuingPortManagerNode getNext();
};

/**********************************************************************************/
/************************ QUEUING PORT MANAGER_NODE  : END   ***********************/
/**********************************************************************************/




/**********************************************************************************/
/************************** QUEUING PORT MANAGER : BEGIN  *************************/
/**********************************************************************************/

/* Manager type definition */
typedef class QueuingPortManager *ptQueuingPortManager;
class QueuingPortManager
{
 private :
  ptQueuingPortManagerNode first;                    /* First node in the manager */
  ptQueuingPortManagerNode last;                      /* Last node in the manager */
  int max_size;                                     /* Size limit of the manager */
  int current_size;                        /* Current number of nodes in manager */

  void setFirst(ptQueuingPortManagerNode node);
  void setLast(ptQueuingPortManagerNode node);
  ptQueuingPortManagerNode getFirst();
  ptQueuingPortManagerNode getLast();
  bool getElemNamed(Comm_ComponentName_type queuingport_Name, ptQueuingPortManagerNode *elem);
  bool getElemId(Comm_ComponentID_type queuingport_ID, ptQueuingPortManagerNode *elem);
  
 public :
  QueuingPortManager();
  QueuingPortManager(int size);
  ~QueuingPortManager();
 
 bool QueuingPortRecord(Comm_ComponentName_type queuingport_Name,
			QueuingPortSize_type queuingPort_Size,
			PortDirection_type port_direction,
			MessageSize_type message_size,
			QueuingDiscipline_type queuing_discipline,
			Comm_ComponentID_type& queuingport_ID); 
 
 /* If the manager was not full, a new sampling port node is created, initialized
    with input parameters and it is returned true. Else, it is returned false. */

 bool QueuingPortCheckName(Comm_ComponentName_type queuingport_Name, Comm_ComponentID_type& queuingport_ID); 
 /* If a blackboard named queuingport_Name exists in the manager, queuingport_ID
    is its identifier and it is returned true. Else, it is returned false */

  bool QueuingPortCheckID(Comm_ComponentID_type queuingport_ID, APEX_QueuingPort_type& queuingport);
  /* If there is an identifier queuingport_ID in the manager, board is the 
     queuingport associated and it is returned true. Else, it is returned false */

  void QueuingPortUpdate(APEX_QueuingPort_type queuingport);
  /* The sampling port from the manager, which has the same name as queuingport, is updated */
  bool Full();
  /* Returns true if the manager size limit is reached. Else, it is returned
     false */

  void printFile(Comm_ComponentID_type queuingport_ID, char *result_file);
  /* Prints queuingport_ID status in result_file */

};

/*********************************************************************************/
/********************************* QUEUING PORT MANAGER : END  ******************************/
/*********************************************************************************/


#endif // __QUEUINGPORTMANAGER_H__










