#include "TYPES.h"
#include "SamplingPortManager.h"


/*****************************************   SAMPLING PORT OBJECT ****************************************/


SamplingPort::SamplingPort(APEX_SamplingPort_type s):samplingport(s){}


/************************************** SAMPLING PORT MANAGER_NODE  **************************************/

SamplingPortManagerNode::SamplingPortManagerNode(SamplingPortAssoc_type contents):next(NULL)
{
  Contents.SamplingPort_ID = contents.SamplingPort_ID;
  Contents.ptrSamplingPort = contents.ptrSamplingPort;
 }

SamplingPortManagerNode::SamplingPortManagerNode(){}

SamplingPortManagerNode::~SamplingPortManagerNode(){}


void SamplingPortManagerNode::setContents(SamplingPortAssoc_type contents)
{
  Contents.SamplingPort_ID = contents.SamplingPort_ID;
  Contents.ptrSamplingPort = contents.ptrSamplingPort;
}

void SamplingPortManagerNode::setNext(ptSamplingPortManagerNode node)
{
  next = node;
}

SamplingPortAssoc_type SamplingPortManagerNode::getContents()
{ 
  return Contents;
}

ptSamplingPortManagerNode SamplingPortManagerNode::getNext()
{ 
  return next;
}

/*************************************** SAMPLING PORT MANAGER  *****************************************/

SamplingPortManager::SamplingPortManager(int size):first(NULL),last(NULL),max_size(size),current_size(0){}

SamplingPortManager::SamplingPortManager():first(NULL),last(NULL),max_size(MAX_NUMBER_OF_SAMPLING_PORTS),current_size(0){}

SamplingPortManager::~SamplingPortManager(){}

/**
   @return "true" if an element named name is in the manager, "false" otherwise.
   @param elem (out) the found element.
   @param name (in)
*/
bool SamplingPortManager::getElemNamed(Comm_ComponentName_type name, ptSamplingPortManagerNode *elem)
{
  ptSamplingPortManagerNode iter = first;

  while (iter != NULL)
    if (strcmp ((((iter->getContents()).ptrSamplingPort)->samplingport).SamplingPort_Name, name) ==0)
      {
	*elem = iter;
	return true;
      }
    else
      iter = iter->getNext();
  return false;
}
/**
   @return "true" if an element with samplingport_ID as identifier is in the manager, "false" otherwise.
   @param elem (out) the found element.
   @param name (in) a sampling port identifier.
*/
bool SamplingPortManager::getElemId(Comm_ComponentID_type samplingport_ID, ptSamplingPortManagerNode *elem)
{
  ptSamplingPortManagerNode iter = first;
  
  while (iter != NULL)
    if ((iter->getContents()).SamplingPort_ID == samplingport_ID)
      { 
	*elem = iter; 
	return true;
      }
    else
      iter = iter->getNext();
  return false;
}

void SamplingPortManager::setFirst(ptSamplingPortManagerNode node)
{
  first = node;
}

void SamplingPortManager::setLast(ptSamplingPortManagerNode node)
{
  last = node;
}

ptSamplingPortManagerNode SamplingPortManager::getFirst()
{
  return first;
}

ptSamplingPortManagerNode SamplingPortManager::getLast()
{
  return last;
}

bool SamplingPortManager::SamplingPortRecord(Comm_ComponentName_type samplingport_Name,
					     SamplingPortSize_type samplingPort_Size, 
					     PortDirection_type port_direction, 
					     SystemTime_type refresh_period, 
					     Comm_ComponentID_type& samplingport_ID)
{
  if (current_size < max_size)
    {
      current_size++;
      SamplingPortAssoc_type new_record;
      new_record.SamplingPort_ID = current_size;
      
      APEX_SamplingPort_type samplingport;
     
      strcpy(samplingport.SamplingPort_Name, samplingport_Name);
      samplingport.Empty = true;
      samplingport.SamplingPort_Size = samplingPort_Size;
      samplingport.Port_Direction = port_direction;
      samplingport.Refresh_Period = refresh_period;

      new_record.ptrSamplingPort = new SamplingPort(samplingport);
      samplingport_ID = current_size;

      ptSamplingPortManagerNode new_node = new SamplingPortManagerNode(new_record);
        
      if (first == NULL)
        { 
	  first = new_node; 
	  last = new_node;
	}
      else
        {
	  ptSamplingPortManagerNode iter = first;
          while (iter != last) iter = iter->getNext();
          last->setNext(new_node); last = new_node;
         };
      return true;
    }
  return false;
 }

bool SamplingPortManager::SamplingPortCheckName(Comm_ComponentName_type samplingport_Name, Comm_ComponentID_type& samplingport_ID)
{
  bool returnCode;
  ptSamplingPortManagerNode sport;

  if (!getElemNamed(samplingport_Name, &sport))
    { 
      samplingport_ID = NULL; 
      returnCode = false; 
    }
  else
    {
      samplingport_ID = (sport->getContents()).SamplingPort_ID;
      returnCode =true;
    }
  return returnCode;
 }

bool SamplingPortManager::SamplingPortCheckID(Comm_ComponentID_type samplingport_ID, APEX_SamplingPort_type& samplingport)
{
  ptSamplingPortManagerNode sport;
  
  if (!getElemId(samplingport_ID, &sport)) return false;
  samplingport = ((sport->getContents()).ptrSamplingPort)->samplingport;
  return true; 
}


void SamplingPortManager::SamplingPortUpdate(APEX_SamplingPort_type samplingport)
{
  ptSamplingPortManagerNode sport;
  APEX_SamplingPort_type elem;
  
  // WHY? Comm_ComponentName_type name; strcpy(name, samplingport.SamplingPort_Name); 
  if (getElemNamed(samplingport.SamplingPort_Name, &sport))
    {
      elem = ((sport->getContents()).ptrSamplingPort)->samplingport;
      elem.SamplingPort_Size = samplingport.SamplingPort_Size; 
      elem.Port_Direction = samplingport.Port_Direction;
      elem.Refresh_Period = samplingport.Refresh_Period;
      elem.Validity = samplingport.Validity;
      elem.Empty = samplingport.Empty;
      elem.Message_Age = samplingport.Message_Age;
      elem.Message = samplingport.Message;
    }
}

bool SamplingPortManager::Full()
{ 
  return (max_size ==current_size);
}

bool SamplingPortManager::SamplingPortCheckCreated (Comm_ComponentName_type samplingport_Name)
{
  return created;
}

void SamplingPortManager::SamplingPortSetCreated (Comm_ComponentName_type samplingport_Name)
{
  created = true;
}

/* void SamplingPortManager::printFile(Comm_ComponentID_type samplingport_ID, char *result_file){ 
   FILE *file_out;
   APEX_SamplingPort_type samplingport;
   if (this->SamplingPortCheckID(samplingport_ID, samplingport))
     { file_out = fopen(result_file,"wt");
       if (!file_out)
         {
          fprintf(stderr,"Can not open file %s\n",result_file);
          exit(1);
         };    
       fprintf(file_out, "(SamplingPort_Identifier : %i, Name : %s, Displayed_Message : %i, Empty_Indicator : %i, Waiting_Processes : %i) \n" ,samplingport_ID, samplingport.SamplingPort_Name, samplingport.Message, samplingport.Empty_Indicator, samplingport.Waiting_Processes);
       fclose(file_out);
    }
}
*/

