/***Polychrony License******************************************
 *                                                             *
 * Copyright: IRISA/INRIA Rennes - ESPRESSO team.              *
 * This software is under the terms of the CeCILL-B License.   *
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html *
 *                                                             *
 ***************************************************************/
#ifndef __BLACKBOARDMANAGER_H__
#define __BLACKBOARDMANAGER_H__



/**********************************************************************************/
/****************************   BLACKBOARD OBJECT : BEGIN *************************/
/**********************************************************************************/

/* Object representing APEX Blackboard */
typedef class Blackboard * ptBlackboard;
class Blackboard
{
   public :
   APEX_Blackboard_type board;
   Blackboard(APEX_Blackboard_type b);
};

/**********************************************************************************/
/*****************************   BLACKBOARD_OBJECT : END   ************************/
/**********************************************************************************/




/**********************************************************************************/
/*********************** BLACKBOARD MANAGER_NODE  : BEGIN   ***********************/
/**********************************************************************************/

/* Type of manager node contents */
struct BoardAssoc_type{
  Comm_ComponentID_type BlackboardID; 
  ptBlackboard ptrBlackboard;
};

/* Manager node type definition */
typedef class BlackboardManagerNode *ptBlackboardManagerNode;
class BlackboardManagerNode
{
 private :
  BoardAssoc_type contents;                             /* Current node contents */
  ptBlackboardManagerNode next;                                     /* Next node */

 public :
  BlackboardManagerNode();
  BlackboardManagerNode(BoardAssoc_type value);
  ~BlackboardManagerNode();
  void setValue(BoardAssoc_type value);
  BoardAssoc_type getValue();
  void setNext(ptBlackboardManagerNode ptvalue);
  ptBlackboardManagerNode getNext();
};

/**********************************************************************************/
/************************ BLACKBOARD  MANAGER_NODE  : END   ***********************/
/**********************************************************************************/




/**********************************************************************************/
/************************** BLACKBOARD   MANAGER : BEGIN  *************************/
/**********************************************************************************/

/* Manager type definition */
typedef class BlackboardManager *ptBlackboardManager;
class BlackboardManager
{
 private :
  ptBlackboardManagerNode first;                    /* First node in the manager */
  ptBlackboardManagerNode last;                      /* Last node in the manager */
  int max_size;                                     /* Size limit of the manager */
  int current_size;                        /* Current number of nodes in manager */

  void setFirst(ptBlackboardManagerNode ptvalue);
  void setLast(ptBlackboardManagerNode ptvalue);
  ptBlackboardManagerNode getFirst();
  ptBlackboardManagerNode getLast();

 public :
  BlackboardManager();
  BlackboardManager(int size);
  ~BlackboardManager();

  bool BlackboardRecord(Comm_ComponentName_type blackboard_Name, 
                        int message_Size, 
                        Comm_ComponentID_type& blackboardID);
  /* If the manager was not full, a new blackboard node is created, initialized
     with input parameters and it is returned true. Else, it is returned false. */

  bool BlackboardCheckName(Comm_ComponentName_type blackboard_Name, 
                           Comm_ComponentID_type& blackboardID); 
  /* If a blackboard named blackboard_Name exists in the manager, blackboardID
     is its identifier and it is returned true. Else, it is returned false */

  bool BlackboardCheckID(Comm_ComponentID_type blackboardID, 
                         APEX_Blackboard_type& board);
  /* If there is an identifier blackboardID in the manager, board is the 
     blackboard associated and it is returned true. Else, it is returned false */

  void BlackboardUpdate(APEX_Blackboard_type board);
  /* The blackboard from the manager, which has the same name as board, is 
     updated */

  bool Full();
  /* Returns true if the manager size limit is reached. Else, it is returned
     false */

  void printFile(Comm_ComponentID_type blackboardID, char *result_file);
  /* Prints blackboardID status in result_file */

};

/*********************************************************************************/
/*********************************   MANAGER : END  ******************************/
/*********************************************************************************/


#endif // __BLACKBOARDMANAGER_H__










