/***Polychrony License******************************************
 *                                                             *
 * Copyright: IRISA/INRIA Rennes - ESPRESSO team.              *
 * This software is under the terms of the CeCILL-B License.   *
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html *
 *                                                             *
 ***************************************************************/
#ifndef __EVENTMANAGER_H__
#define __EVENTMANAGER_H__




/**********************************************************************************/
/******************************   EVENT OBJECT : BEGIN ****************************/
/**********************************************************************************/

/* Object representing APEX Buffer */
typedef class Event * ptEvent;
class Event
{
   public :
   APEX_Event_type event;
   Event(APEX_Event_type e);
};


/**********************************************************************************/
/*******************************   EVENT OBJECT : END   ***************************/
/**********************************************************************************/




/**********************************************************************************/
/**************************  EVENT MANAGER NODE  : BEGIN   ************************/
/**********************************************************************************/

/* Type of manager node contents */
struct EvtAssoc_type{
  Comm_ComponentID_type EventID; 
  ptEvent ptrEvent;
};


/* Manager node type definition */
typedef class EventManagerNode *ptEventManagerNode;
class EventManagerNode
{
 private :
  EvtAssoc_type contents;                                  /* Contents of a node */
  ptEventManagerNode next;                                          /* Next node */

 public :
  EventManagerNode();
  EventManagerNode(EvtAssoc_type componentRecord);
  ~EventManagerNode();
  void setValue(EvtAssoc_type value);
  void setNext(ptEventManagerNode ptvalue);
  EvtAssoc_type getValue();
  ptEventManagerNode getNext();
};

/**********************************************************************************/
/**************************** EVENT MANAGER NODE : END   **************************/
/**********************************************************************************/



/**********************************************************************************/
/****************************** EVENT MANAGER : BEGIN  ****************************/
/**********************************************************************************/

typedef class EventManager *ptEventManager;
class EventManager
{
 private :
  ptEventManagerNode first;                          /* First node in the manager */
  ptEventManagerNode last;                            /* Last node in the manager */
  int max_size;                                      /* Size limit of the manager */
  int current_size;                     /* Current number of nodes in the manager */

  void setFirst(ptEventManagerNode ptvalue);
  void setLast(ptEventManagerNode ptvalue);
  ptEventManagerNode getFirst();
  ptEventManagerNode getLast();

 public :
  EventManager();
  EventManager(int size);
  ~EventManager();

  bool EventRecord(Comm_ComponentName_type event_Name, Comm_ComponentID_type& eventID);
   /* If the manager was not full, a new event node is created, initialized
     with input parameters and it is returned true. Else, it is returned false. */

  bool EventCheckName(Comm_ComponentName_type event_Name, Comm_ComponentID_type& eventID); 
   /* If an event named buffer_Name exists in the manager, eventID is its 
     identifier and it is returned true. Else, it is returned false */

  bool EventCheckID(Comm_ComponentID_type eventID, APEX_Event_type& evt);
   /* If there is an identifier eventID in the manager, evt is the 
     event associated and it is returned true. Else, it is returned false */

  void EventUpdate(APEX_Event_type evt);
  /* The event from the manager, which has the same name as evt, is updated */
 
  bool Full();
  /* Returns true if the manager size limit is reached. Else, it is returned 
    false */

  void printFile(Comm_ComponentID_type eventID, char *result_file);
    /* Prints eventID status in result_file */

};

/*********************************************************************************/
/****************************** EVENT MANAGER : END  *****************************/
/*********************************************************************************/


//static ptEventManager EVENT_MANAGER;

#endif // __EVENTMANAGER_H__










