/***Polychrony License******************************************
 *                                                             *
 * Copyright: IRISA/INRIA Rennes - ESPRESSO team.              *
 * This software is under the terms of the CeCILL-B License.   *
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html *
 *                                                             *
 ***************************************************************/
#ifndef __FIFORECEIVEQUEUE_H__
#define __FIFORECEIVEQUEUE_H__





/***********************************************************************************/
/*****************************   FIFO_RECEIVEQUEUE_NODE   **************************/
/***********************************************************************************/

typedef class FIFO_ReceiveQueueNode *ptFIFO_ReceiveQueueNode;
class FIFO_ReceiveQueueNode
{
  private :
   ProcessID_type contents;                                  /* Contents of a node */
   ptFIFO_ReceiveQueueNode next;                                      /* Next node */

  public :
   FIFO_ReceiveQueueNode();
   FIFO_ReceiveQueueNode(ProcessID_type Process_ID);
   ~FIFO_ReceiveQueueNode();
   void setContents(ProcessID_type Process_ID);
   void setNext(ptFIFO_ReceiveQueueNode nextNode);
   ProcessID_type getContents();
   ptFIFO_ReceiveQueueNode getNext();
};



/**********************************************************************************/
/******************************   FIFO_RECEIVEQUEUE   *****************************/
/**********************************************************************************/

typedef class FIFO_ReceiveQueue *ptFIFO_ReceiveQueue;
class FIFO_ReceiveQueue
{
  private :
   ptFIFO_ReceiveQueueNode first;                      /* First node of the QUEUE */
   ptFIFO_ReceiveQueueNode last;                        /* Last node of the QUEUE */
   int max_size;                                       /* Size limit of the QUEUE */
   int current_size;                      /* Current number of nodes in the QUEUE */

  void setFirst(ptFIFO_ReceiveQueueNode Node);
  void setLast(ptFIFO_ReceiveQueueNode Node);
  ptFIFO_ReceiveQueueNode getFirst();
  ptFIFO_ReceiveQueueNode getLast();

 public :
  FIFO_ReceiveQueue();
  FIFO_ReceiveQueue(int size);
  ~FIFO_ReceiveQueue();
  int getMax_size();
  int getCurrent_size();

  void insert(ProcessID_type Process_ID);
  /* Insert Process_ID in the QUEUE */

  void remove(ProcessID_type& Process_ID);
  /* Remove the oldest node from the QUEUE and Process_ID is this node */

  bool empty();
  /* The QUEUE is empty */

  bool full();
  /* The QUEUE size limit is reached */

  void printFile(char *result_file);
  /* prints the QUEUE in result_file */

};



#endif // __FIFORECEIVEQUEUE_H__










