/***Polychrony License******************************************
 *                                                             *
 * Copyright: IRISA/INRIA Rennes - ESPRESSO team.              *
 * This software is under the terms of the CeCILL-B License.   *
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html *
 *                                                             *
 ***************************************************************/
#ifndef __FIFOSENDQUEUE_H__
#define __FIFOSENDQUEUE_H__




/***********************************************************************************/
/******************************   FIFO_SENDQUEUE_NODE   ****************************/
/***********************************************************************************/


typedef class FIFO_SendQueueNode *ptFIFO_SendQueueNode;
class FIFO_SendQueueNode
{
  private :
   ProcMess_type contents;                                   /* Contents of a node */
   ptFIFO_SendQueueNode next;                                         /* Next node */

  public :
   FIFO_SendQueueNode();
   FIFO_SendQueueNode(const ProcMess_type& PM);
   ~FIFO_SendQueueNode();
   void setContents(ProcMess_type PM);
   void setNext(ptFIFO_SendQueueNode nextPM);
   ProcMess_type getContents();
   ptFIFO_SendQueueNode getNext();
};



/**********************************************************************************/
/**********************************   FIFO_SENDQUEUE  *****************************/
/**********************************************************************************/


typedef class FIFO_SendQueue *ptFIFO_SendQueue;
class FIFO_SendQueue
{
 private :
  ptFIFO_SendQueueNode first;                          /* First node of the QUEUE */
  ptFIFO_SendQueueNode last;                            /* Last node of the QUEUE */
  int max_size;                                        /* Size limit of the QUEUE */
  int current_size;                       /* Current number of nodes in the QUEUE */

  void setFirst(ptFIFO_SendQueueNode Node);
  void setLast(ptFIFO_SendQueueNode Node);
  ptFIFO_SendQueueNode getFirst();
  ptFIFO_SendQueueNode getLast();

 public :
  FIFO_SendQueue();
  FIFO_SendQueue(int size);
  ~FIFO_SendQueue();
  int getMax_size();
  int getCurrent_size();

  void insert(ProcessID_type Process_ID, Message_type Message);
  /* Insert (Process_ID, Message) in the QUEUE */

  ProcessID_type remove(Message_type& Message);
  /* Remove the oldest node from the QUEUE. The corresponding process ID is sent and
     and Message is the corresponding message */

  bool empty();
  /* The QUEUE is empty */

  bool full();
  /* The QUEUE size limit is reached */

  void printFile(char *result_file);
  /* prints the QUEUE in result_file */

};



#endif // __FIFOSENDQUEUE_H__










