/***Polychrony License******************************************
 *                                                             *
 * Copyright: IRISA/INRIA Rennes - ESPRESSO team.              *
 * This software is under the terms of the CeCILL-B License.   *
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html *
 *                                                             *
 ***************************************************************/
#ifndef __PRIORITYSENDQUEUE_H__
#define __PRIORITYSENDQUEUE_H__


#include <string.h> 
#include <stdlib.h>
#include <stdio.h>


/***********************************************************************************/
/*****************************   PRIORITY_SENDQUEUE_NODE  **************************/
/***********************************************************************************/


typedef class PRIORITY_SendQueueNode *ptPRIORITY_SendQueueNode;
class PRIORITY_SendQueueNode
{
  private :
   ProcMess_type contents;                                   /* Contents of a node */
   ptPRIORITY_SendQueueNode next;                                     /* Next node */

  public :
   PRIORITY_SendQueueNode();
   PRIORITY_SendQueueNode(ProcMess_type PM);
   ~PRIORITY_SendQueueNode();
   void setContents(ProcMess_type PM);
   void setNext(ptPRIORITY_SendQueueNode nextPM);
   ProcMess_type getContents();
   ptPRIORITY_SendQueueNode getNext();
};



/***********************************************************************************/
/*******************************  PRIORITY_SENDQUEUE  ******************************/
/***********************************************************************************/


typedef class PRIORITY_SendQueue *ptPRIORITY_SendQueue;
class PRIORITY_SendQueue
{
  private:
   ptPRIORITY_SendQueueNode first;            /* First node of the PRIORITY_QUEUE */
   ptPRIORITY_SendQueueNode last;              /* Last node of the PRIORITY_QUEUE */
   int max_size;                              /* Size limit of the PRIORITY_QUEUE */
   int current_size;             /* Current number of nodes in the PRIORITY_QUEUE */
   
   void setFirst(ptPRIORITY_SendQueueNode node);
   void setLast(ptPRIORITY_SendQueueNode node);
   ptPRIORITY_SendQueueNode getFirst();
   ptPRIORITY_SendQueueNode getLast();

   
  public:

   PRIORITY_SendQueue();
   PRIORITY_SendQueue(int nbproc);
   ~PRIORITY_SendQueue();

   int getCurrentSize();

   void insert(ProcessID_type Process_ID, Message_type Message);
   /* Insert (Process_ID, Message) in the QUEUE */

   ProcessID_type remove(Message_type& Message);
    /* Remove the oldest node from the QUEUE. The corresponding process ID is sent and
     and Message is the corresponding message */

   void clean();
   /* delete all nodes in the QUEUE */

   bool empty();
   /* The QUEUE is empty */

   bool full();
   /* The QUEUE size limit is reached */

   void printFile(char *result_file);
   /* prints the QUEUE in result_file */

};


#endif // __PRIORITYSENDQUEUE_H__










