/**
 * Copyright : IRISA / INRIA Rennes - ESPRESSO team This plug-in is under the terms of the CeCILL-B License.
 * http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
 * 
 */
package Signal.lib.Java.graphical;
import java.io.IOException;

/**
   This class builts a window for the simulation (with button, canvas, ...) of a Signal Program
   when C is produced by the Signal compiler and when the predefined provided libGraphics.SIG 
   Signal library is used in the Signal program.
   To make portable the simulation program, the graphical part is written in Java.
   The connexion Java/C is done through SimuC.c module defined in export directory.
*/
public class SimuC
{
    protected Simu gw;

    /* Java/C calls */
    public native void n_initSimu();
    public native boolean n_iterate();

    static {System.loadLibrary("Simu");}
    public synchronized void initSimu() { n_initSimu(); }// initialising the C/JAVA connexion
    public synchronized boolean iterateSignal() { return n_iterate(); }

    private Direction getEnumDirection(int vpos)
    {
	if (vpos == _enumDirection_.horiz) return Direction.horiz;
	return Direction.vertic;
    }

    /* C/Java calls */
    public int createFrame(int x,int y) throws IOException { return gw.createFrame(x, y); }
    public int createPanel(int x, int y) {return gw.createPanel(x, getEnumDirection(y));};
    public int createButton(int x) throws IOException {return gw.createButton(x);}
    public int createCheckBox(int x,boolean y) throws IOException {return gw.createCheckBox(x,y);}
    public int createLabel(int x) throws IOException {return gw.createButton(x);}
    public int createCanvas(int x ,int y,int z) {return gw.createCanvas(x, y, z);}
    public int createTextArea(int x ,int y,int z) {return gw.createTextArea(x, y, z);}
    public void createBorder(int x ,int y,int z) {gw.createBorder(x, y, z);}
    public int createSlider(int x1,int x2,int x3,int x4,int x5,int x6, int x7) {return gw.createSlider(x1, x2, x3, x4, x5, x6, x7);}
    public int createComboBox(int x1,int x2,int x3,int x4,int x5,int x6) {return gw.createComboBox(x1, x2, x3, x4, x5, x6);}
    public void setTextArea(int x, int y) { gw.setTextArea(x, y);}
    public boolean readButton(int x) {return gw.readButton(x);}
    public int readSlider(int x) {return gw.readSlider(x);}
    public double readComboBox(int x) {return gw.readComboBox(x);}
    public boolean readCheckBox(int x) {return gw.readCheckBox(x);}

    public void clearCanvas(int x1,int x2,int x3,int x4) { gw.clearCanvas(x1, x2, x3, x4);}
    public void drawLine(int x1,int x2,int x3,int x4,int x5, int vcolor) { gw.drawLine(x1, x2, x3, x4, x5, vcolor);}
    public void fillRect(int x1,int x2,int x3,int x4,int x5, int vcolor) { gw.fillRect(x1, x2, x3, x4, x5, vcolor);}
    public void drawInteger(int x1,int x2,int x3,int x4) { gw.drawInteger(x1, x2, x3, x4);}
    public void drawReal(int x1, double x2,int x3,int x4) { gw.drawReal(x1, x2, x3, x4);}

    public void quitAppli() { gw.quitAppli();}
    public int getColor(int R, int G, int B, boolean brightness) {return gw.getColor(R,G,B,brightness);}

    public SimuC ()
    {
	gw = new Simu();
    }
    
    public void GO ()
    {
	initSimu();
	while (true) iterateSignal();
    }

}
