
package Signal.lib.Java.graphical;

import java.awt.Color;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;

class Scanner
{
	final static char	SEPARATOR1	= '@';
	final static char	COMMA		= ',';

	private Color		VCOLOR;
	private String		VNAME;
	private String		CURRENTLINE;
	private int			LENGTH;			// length(CURRENTLINE)
	private int			pIndex;			// previous index on CURRENTLINE

	// Fon info (textArea)
	private Font		VFONT;
	private String		fontName, vstyle;
	private int			sizeFont;

	private String searchString(String fileName, int num, boolean bmess)
	{
		FileInputStream inStr;
		File inputFile = new File(fileName);

		try
		{
			inStr = new FileInputStream(inputFile);
			BufferedReader in = new BufferedReader(new InputStreamReader(inStr));
			// Chaine contenant la valeur de retour
			String retour = new String("");
			// indice du nombre de lignes parcourus
			int i = 0;
			while (((retour = in.readLine()) != null) && i < num)
				i++;
			in.close();
			// rend le numero du bouton si la ligne est vide
			if (retour == null)
				return new String("" + num);
			return retour;
		}
		catch (Exception exc)
		{
			if (bmess)
				System.out.println(" File not found: " + fileName);
			return new String("" + num);
		}
	}

	private int getIndexOf(char sep)
	{
		int i = pIndex;
		try
		{
			while (CURRENTLINE.charAt(i) != sep)
				i++;
		}
		catch (IndexOutOfBoundsException e)
		{}
		return i;
	}

	private int getvalueOf(String vstr)
	{
		int Result, i;
		char c;

		i = 0;
		Result = 0;
		try
		{
			while (true)
			{
				c = vstr.charAt(i);
				if (!Character.isWhitespace(c))
					Result = Result * 10 + Character.getNumericValue(c);
				i++;
			}
		}
		catch (IndexOutOfBoundsException e)
		{};
		return Result;
	}

	private String ExtractString(char sep)
	{
		if (pIndex >= LENGTH)
			return new String("");
		int n = pIndex;
		int i = getIndexOf(sep);
		pIndex = i + 1;
		return CURRENTLINE.substring(n, i);
	}

	private int ExtractInt(char sep)
	{
		if (pIndex >= LENGTH)
			return -1;
		int n = pIndex;
		int i = getIndexOf(sep);
		pIndex = i + 1;
		return getvalueOf(CURRENTLINE.substring(n, i));
	}

	private void makeFont()
	{
		int st;
		if (!fontName.equals(""))
		{
			if (vstyle.equals("BOLD"))
				st = Font.BOLD;
			else if (vstyle.equals("ITALIC"))
				st = Font.ITALIC;
			else if (vstyle.equals("PLAIN"))
				st = Font.PLAIN;
			else if (vstyle.equals("BOLD+ITALIC"))
				st = Font.BOLD + Font.ITALIC;
			else
				st = Font.PLAIN;

			VFONT = new Font(fontName, st, sizeFont);
		}
	}

	/*
	 * Analizing the CURRENTLINE line. It extracts a name, a color and the darkness Example : toto @ 0,255,110, 1 @ -
	 * name is "toto" - color codification : 0,255,110 - brightness/darkness : 0/1
	 */
	private boolean ScanParameters()
	{
		int RGB_R, RGB_G, RGB_B, bdark;

		VNAME = ExtractString(SEPARATOR1);
		RGB_R = ExtractInt(COMMA);
		if (RGB_R < 0)
			return false;
		RGB_G = ExtractInt(COMMA);
		if (RGB_G < 0)
			return false;
		RGB_B = ExtractInt(COMMA);
		if (RGB_B < 0)
			return false;
		VCOLOR = new Color(RGB_R, RGB_G, RGB_B);
		bdark = ExtractInt(SEPARATOR1);
		if (bdark == 1)
			VCOLOR = VCOLOR.darker();
		else
			VCOLOR = VCOLOR.brighter();
		// FONT
		fontName = ExtractString(COMMA);
		vstyle = ExtractString(COMMA); // PLAIN, ITALIC,BOLD, BOLD+ITALIC
		sizeFont = ExtractInt(SEPARATOR1);
		makeFont();
		return true;
	}
	private void initGetLine()
	{
		pIndex = 0;
		LENGTH = CURRENTLINE.length();
		fontName = "";
		vstyle = "";
		sizeFont = -20;
	}

	// -------------- PUBLIC METHODS

	public void loadPanelParameters(int index)
	{
		CURRENTLINE = searchString("panel.txt", index, true);
		initGetLine();
		VCOLOR = Color.red.darker(); // Default Color
		ScanParameters();
	}

	public void loadCheckBoxParameters(int index)
	{
		CURRENTLINE = searchString("checkbox.txt", index, true);
		initGetLine();
		VCOLOR = Color.green.darker(); // Default Color
		ScanParameters();
	}

	public void loadFrameParameters(int index)
	{
		CURRENTLINE = searchString("frame.txt", index, true);
		initGetLine();
		VCOLOR = Color.red.darker(); // Default Color
		ScanParameters();
	}

	public void loadLabelParameters(int index)
	{
		CURRENTLINE = searchString("label.txt", index, true);
		initGetLine();
		VCOLOR = Color.red.darker(); // Default Color
		ScanParameters();
	}

	public void loadButtonParameters(int index)
	{
		CURRENTLINE = searchString("button.txt", index, true);
		initGetLine();
		VCOLOR = Color.cyan.darker(); // Default Color
		ScanParameters();
	}

	public void loadSliderParameters(int index)
	{
		CURRENTLINE = searchString("slider.txt", index, false);
		initGetLine();
		VCOLOR = new Color(200, 200, 230); // Default Color
		ScanParameters();
	}

	public void loadComboBoxParameters(int index)
	{
		CURRENTLINE = searchString("combobox.txt", index, false);
		initGetLine();
		VCOLOR = new Color(200, 200, 230); // Default Color
		ScanParameters();
	}

	public void loadTextAreaParameters(int index)
	{
		CURRENTLINE = searchString("textarea.txt", index, false);
		initGetLine();
		VCOLOR = new Color(200, 200, 230); // Default Color
		VFONT = new Font("Helvetica", Font.BOLD, 20); // Default Font
		ScanParameters();
	}

	public void loadCanvasParameters(int index)
	{
		CURRENTLINE = searchString("canvas.txt", index, false);
		initGetLine();
		VCOLOR = Color.white; // Default Color
		VFONT = new Font("Serif", Font.PLAIN, 10); // Default Font
		ScanParameters();
	}

	public Color getColor()
	{
		return VCOLOR;
	}
	public String getName()
	{
		return VNAME;
	}
	public Font getFont()
	{
		return VFONT;
	}

}
