
package Signal.lib.Java.graphical;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;

import javax.swing.JPanel;

/**
 * SimuCanvas : canvas utilis pour la simulation des programmes Signal, principalement pour l'affichage de courbes
 */
public class SimuCanvas extends JPanel
{
	/**
	 * 
	 */
	private static final long	serialVersionUID	= 4369372835125351297L;

	private Graphics			gc;

	/**
	 * Constructor
	 */
	public SimuCanvas(int x, int y)
	{
		init(x, y, Color.white);
	}

	public SimuCanvas(int x, int y, Color bgColor, Font vfont)
	{
		init(x, y, bgColor);
		setFont(vfont);
	}

	/**
	 * Initialization
	 */
	private void init(int x, int y, Color bgColor)
	{
		setSize(x, y);
		setVisible(true);
		setBackground(bgColor);

		gc = getGraphics();
	}

	/**
	 * dessine un segment de droite
	 * 
	 * @param x1
	 *        abcisse du premier point
	 * @param y1
	 *        ordonne du premier point
	 * @param x2
	 *        abcisse du deuxime point
	 * @param y2
	 *        ordonne du deuxime point
	 */
	public void drawLine(int x1, int y1, int x2, int y2, Color vcolor)
	{
		gc = getGraphics();
		gc.setColor(vcolor);
		gc.drawLine(x1, y1, x2, y2);
	}

	/**
	 * dessine un rectangle plein de couleur noire
	 * 
	 * @param x
	 *        abcisse origine du rectangle,
	 * @param y
	 *        ordonne origine du rectangle,
	 * @param dx
	 *        largeur du rectangle
	 * @param dy
	 *        hauteur du rectangle
	 */
	public void fillRect(int x, int y, int dx, int dy, Color vcolor)
	{
		gc = getGraphics();
		gc.setColor(vcolor);
		if (dy == 0)
			dy = 1;
		gc.fillRect(x, y, dx, dy);
	}

	/**
	 * efface une zone rectangle; l'ordonne du point d'origine est forcemment 0.
	 * 
	 * @param x1
	 *        abcisse du point d'origne
	 * @param dx
	 *        largeur du rectangle  effacer
	 * @param dy
	 *        hauteur du rectangle  effacer
	 */
	public void clearRect(int x, int y, int dx, int dy)
	{
		gc = getGraphics();
		gc.setColor(getBackground());
		gc.fillRect(x, y, dx, dy);
	}

	/**
	 * affiche un entier sous forme de chane
	 * 
	 * @param value
	 *        valeur  afficher
	 * @param x
	 *        abcisse o sera affiche la chaine dans le canvas
	 * @param y
	 *        ordonne o sera affiche la chaine dans le canvas
	 */
	public void drawInteger(int value, int x, int y, Color vcolor)
	{
		String chaine = new String("" + value);
		gc = getGraphics();
		gc.setColor(vcolor);
		gc.drawString(chaine, x, y);
	}

	public void drawReal(double value, int x, int y, Color vcolor)
	{
		int ind;
		String chaine = new String("" + value);
		ind = chaine.lastIndexOf(".");
		if (chaine.length() >= ind + 3)
			chaine = chaine.substring(0, ind + 3);
		else
			chaine = chaine.substring(0, ind + 2);
		gc = getGraphics();
		gc.setColor(vcolor);
		gc.drawString(chaine, x, y);
	}

}
