/* 
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/
#ifndef __PRIORITYSENDQUEUE_H__
#define __PRIORITYSENDQUEUE_H__


#include <string.h> 
#include <stdlib.h>
#include <stdio.h>


/***********************************************************************************/
/*****************************   PRIORITY_SENDQUEUE_NODE  **************************/
/***********************************************************************************/


typedef class PRIORITY_SendQueueNode *ptPRIORITY_SendQueueNode;
class PRIORITY_SendQueueNode
{
  private :
   ProcMess_type contents;                                   /* Contents of a node */
   ptPRIORITY_SendQueueNode next;                                     /* Next node */

  public :
   PRIORITY_SendQueueNode();
   PRIORITY_SendQueueNode(ProcMess_type PM);
   ~PRIORITY_SendQueueNode();
   void setContents(ProcMess_type PM);
   void setNext(ptPRIORITY_SendQueueNode nextPM);
   ProcMess_type getContents();
   ptPRIORITY_SendQueueNode getNext();
};



/***********************************************************************************/
/*******************************  PRIORITY_SENDQUEUE  ******************************/
/***********************************************************************************/


typedef class PRIORITY_SendQueue *ptPRIORITY_SendQueue;
class PRIORITY_SendQueue
{
  private:
   ptPRIORITY_SendQueueNode first;            /* First node of the PRIORITY_QUEUE */
   ptPRIORITY_SendQueueNode last;              /* Last node of the PRIORITY_QUEUE */
   int max_size;                              /* Size limit of the PRIORITY_QUEUE */
   int current_size;             /* Current number of nodes in the PRIORITY_QUEUE */
   
   void setFirst(ptPRIORITY_SendQueueNode node);
   void setLast(ptPRIORITY_SendQueueNode node);
   ptPRIORITY_SendQueueNode getFirst();
   ptPRIORITY_SendQueueNode getLast();

   
  public:

   PRIORITY_SendQueue();
   PRIORITY_SendQueue(int nbproc);
   ~PRIORITY_SendQueue();

   int getCurrentSize();

   void insert(ProcessID_type Process_ID, Message_type Message);
   /* Insert (Process_ID, Message) in the QUEUE */

   ProcessID_type remove(Message_type& Message);
    /* Remove the oldest node from the QUEUE. The corresponding process ID is sent and
     and Message is the corresponding message */

   void clean();
   /* delete all nodes in the QUEUE */

   bool empty();
   /* The QUEUE is empty */

   bool full();
   /* The QUEUE size limit is reached */

   void printFile(char *result_file);
   /* prints the QUEUE in result_file */

};


#endif // __PRIORITYSENDQUEUE_H__










