#include "TYPES.h"
#include "ProcessDescriptor.h"
#include "Scheduler.h"

 

                              /*============================*/
                              /* Author: A. GAMATIE         */
                              /* Last modified 03/13/2003   */
                              /*============================*/

/**************************************************************************************/
/*******************************   class SCHEDULER (body) *****************************/
/**************************************************************************************/

 Scheduler::Scheduler()
 { for (int i=0; i<MAX_NUMBER_OF_PROCESSES; i += 1) Range[i] =NULL;
   Current_Size = 0;
   Lock_Level = 0;
   Operating_Mode = IDLE;
   Active_Process = NULL;
   Preempted_Process = NULL;
 }


 logical Scheduler::empty()
 { return(Current_Size == 0); }


 logical Scheduler::full()
 { return(Current_Size == MAX_NUMBER_OF_PROCESSES); }


 void Scheduler::clean()
 { for (int i=0; i<MAX_NUMBER_OF_PROCESSES; i += 1) Range[i] =NULL;
   Current_Size = 0;
 }

 
 int Scheduler::getCurrentNbProc()
 { return Current_Size; }


 Scheduler::~Scheduler(){}

   
 void Scheduler::ProcessSetState(ProcessID_type ProcessID, ProcessState_type state)
 {  int location = ProcessID - 1;
    if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     {
       if ((state != READY) || (!Range[location]->getSuspensionIndicator()))
         /* A process currently suspended can go back to ready state only after being
            resumed 
         */
         {
           ProcessDescriptor desc = *(Range[location]);
           ProcessState_type previous_state = desc.getProcessState();

           if (state != RUNNING)
             { if ((previous_state == WAITING) && (state == READY)) desc.setWaitIndicator(0);
                 /* The wait indicator associated with the process becomes 0 if the 
                    process was previously on waiting, and the input "state" value is READY.
                 */
	       if (Range[location] == Active_Process)
                 /* The process is no longer the active one if it was previously running,
                    instead it is preempted.                     
                 */
                 { Active_Process = NULL; 
                   Preempted_Process = Range[location];
                 }
             }
           desc.setProcessState(state);
           Range[location] = new ProcessDescriptor(desc);
           if (state == RUNNING) Active_Process = Range[location];
             /* The active process pointer is the new running process. */ 
         }     
     }
 }


 ProcessState_type Scheduler::ProcessGetState(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     { return Range[location]->getProcessState();}
 }
  

 SystemTime_type Scheduler::ProcessGetPeriod(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
    { return Range[location]->getPeriod();}
 }


 void Scheduler::ProcessSetState_many(logical process_range[MAX_NUMBER_OF_PROCESSES], ProcessState_type state)
 { for (int location = 0; location < Current_Size; location++)
    { if (process_range[location]) this->ProcessSetState(location, state);}
 }
 

 void Scheduler::ProcessSetPriority(ProcessID_type ProcessID, Priority_type Priority)  
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
    {
      ProcessDescriptor desc = *(Range[location]);
      desc.setCurrentPriority(Priority);
      Range[location] = new ProcessDescriptor(desc);
    }
 }
 

 void Scheduler::ProcessSetDeadlineTime(ProcessID_type ProcessID, SystemTime_type time)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
    {
      ProcessDescriptor desc = *(Range[location]);
      desc.setDeadlineTime(time);
      Range[location] = new ProcessDescriptor(desc);
    }
 }


 void Scheduler::ProcessSetAttributes(ProcessID_type ProcessID, ProcessAttributes_type attributes)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
    {
      ProcessDescriptor desc = *(Range[location]);
      desc.setProcessAttributes(attributes);
      Range[location] = new ProcessDescriptor(desc);
    }
 }
 

 ProcessStatus_type Scheduler::ProcessGetStatus(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
    { return (Range[location]->getProcessStatus());}
 }


 ProcessID_type Scheduler::ProcessGetActive(ProcessStatus_type& status, logical& ok)
 {
   if (Active_Process != NULL)
     { int location = 0;
       while ((location < Current_Size) && (Range[location] != Active_Process))
            {location++;}
       if (location < Current_Size) 
         { status = Active_Process->getProcessStatus();
           ok = 1;
           return (location + 1);
         }
       else
         { ok = 0;}
     }
   else {ok = 0;}
 }  

  
 ProcessID_type Scheduler::ProcessGetHighestPriority(logical& ok)
 { ProcessID_type PID;
   logical success = 0;
   Priority_type prior = LOWEST_PRIORITY - 1;
   /* Any significant priority value must be greater than LOWEST_PRIORITY */
   for (int location=0; location<Current_Size; location += 1)
     { if ( (Range[location]->getProcessState() == READY) && 
            (Range[location]->getCurrentPriority() > prior))
         {
           prior = Range[location]->getCurrentPriority();
           PID = location + 1;
           /* Record any process that satisfies the criterion */
           success = 1;
         }  
     }
   ok = success;
   if (success) return PID;
  }


 Priority_type Scheduler::ProcessGetBasicPriority(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     { return (Range[location]->getBasePriority());}
 }


 SystemTime_type Scheduler::ProcessGetTimeCapacity(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     { return (Range[location]->getTimeCapacity());}
 }


 logical Scheduler::ProcessCheckWaitIndicator(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     { return (Range[location]->getWaitIndicator());}
 }


 logical Scheduler::ProcessCheckName(ProcessName_type name, ProcessID_type& ProcessID)
 { ProcessName_type P_name; 
   logical res; int location = 0;
   if (!this->empty())
     { if (Range[location] != NULL) Range[location]->getProcessName(P_name);
       while((location < Current_Size) && strcmp (P_name, name))
          { location++; if (Range[location] != NULL) (Range[location])->getProcessName(P_name);};
       if (location < Current_Size)
         { 
           ProcessID = location + 1;
           res = 1;
         }
       else
         { res = 0;}
     }
   else
      { res = 0;}
   return res;
  }


 logical Scheduler::ProcessCheckID(ProcessID_type ProcessID, ProcessStatus_type& status)
 { logical res; int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
    { 
      status = Range[location]->getProcessStatus();
      res = 1;
    }
  else
    { res = 0;}
  return res;
 }


 void Scheduler::ProcessIncrementLockLevel(logical inc)
 { if (inc) Lock_Level += 1; }


 void Scheduler::ProcessDecrementLockLevel(logical& reschedule)
 { Lock_Level -= 1; 
   if (Lock_Level == 0) 
     { reschedule = 1;}
   else reschedule = 0;
 }


 void Scheduler::ProcessResetLockLevel(logical res)
 { if (res) Lock_Level = 0; }


 OperatingMode_type Scheduler::ProcessCheckOperatingMode()
 { return Operating_Mode; }


 logical Scheduler::ProcessCheckCreated(ProcessName_type name)
 { ProcessName_type P_name; 
   logical res; int location = 0;
   if (!this->empty())
     { if (Range[location] != NULL) Range[location]->getProcessName(P_name);
       while((location < Current_Size) && strcmp (P_name, name))
          { location++; if (Range[location] != NULL) (Range[location])->getProcessName(P_name);};
       if (location < Current_Size)
         { res = Range[location]->getCreationIndicator();}
       else
         { res = 0;}
     }
   else
      { res = 0;}
   return res;
  }


 void Scheduler::ProcessSetCreated(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
    {
      ProcessDescriptor desc = *(Range[location]);
      desc.setCreationIndicator(1);
      Range[location] = new ProcessDescriptor(desc);
    }
 }

   
 logical Scheduler::ProcessCheckSuspensionIndicator(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     return Range[location]->getSuspensionIndicator();
 }

   
 void Scheduler::ProcessSetSuspensionIndicator(ProcessID_type ProcessID, logical suspension)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     {
       ProcessDescriptor desc = *(Range[location]);
       desc.setSuspensionIndicator(suspension);
       Range[location] = new ProcessDescriptor(desc);
     }
 }


 logical Scheduler::ProcessCheckPreviousPreempted(ProcessID_type ProcessID)
 { logical OK;
   if (Preempted_Process != NULL)
     { OK = (Range[ProcessID -1] == Preempted_Process); }
   else
     { OK = 0;}
   return OK;
 }


 logical Scheduler::ProcessRecording(ProcessName_type name)
 { logical issue;
   if (!this->full())
     { ptProcessDescriptor newdesc = new ProcessDescriptor(name,0.0,0,0,0.0,0.0,SOFT);
       Range[Current_Size] = newdesc;
       Current_Size++;
       issue = 1;
     }
   else issue = 0;
   
   return issue;
 }


 ProcessID_type Scheduler::ProcessRecord(ProcessName_type name)
 { if (!this->full())
     { ptProcessDescriptor newdesc = new ProcessDescriptor(name,0.0,0,0,0.0,0.0,SOFT);
       Range[Current_Size] = newdesc;
       Current_Size++;
     };
   return 0;
 }


 logical Scheduler::ProcessCheckPreemptionStatus(logical request)
 { if (request) return (Lock_Level < 1); }


 logical Scheduler::ProcessSchedulingRequest()
 { logical success;
   if (Lock_Level < 1)
     { /* Preemption is enabled. */
       ProcessStatus_type status;
       logical ok;
       ProcessID_type active = this->ProcessGetActive(status, ok);
       if (ok) this->ProcessSetState(active, READY); 
       /* The active process becomes ready. */
 
       ProcessID_type newactive = this->ProcessGetHighestPriority(ok);
       if (ok) 
         { this->ProcessSetState(newactive, RUNNING);
            /* The new active process is the one with the highest priority. */
           if (Range[newactive -1] != Active_Process) 
             { Preempted_Process = Active_Process;
               Active_Process = Range[newactive -1];
               /* If the previous active process has not the highest priority
                  level. */
             }
         }
       success = 1;     
     }
   else success = 0;
   return success;
 }


 void Scheduler::ProcessUpdateTimeCounters(logical timedout[MAX_NUMBER_OF_PROCESSES])
 { for (int location = 0; location < Current_Size; location += 1)
      { 
        if (!(Range[location]->nullWaitingTime()))
          {
            Range[location]->decrementWaitingTime();
            if (Range[location]->nullWaitingTime())
              {
                Range[location]->setProcessState(READY);
                Range[location]->setWaitingTime(0.0);
                timedout[location] = 1;
              }
	  }
        else timedout[location] = 0;
      }
   GlobalTime++;
 }


 void Scheduler::ProcessUpdateTimeCountersProfiling(double val, logical timedout[MAX_NUMBER_OF_PROCESSES])
 { for (int location=0; location<Current_Size; location += 1)
      { 
        if (!(Range[location]->nullWaitingTime()))
          {
            Range[location]->decrementWaitingTimeProfiling(val);
            if (Range[location]->nullWaitingTime())
              {
                Range[location]->setProcessState(READY);
                Range[location]->setWaitingTime(0.0);
                timedout[location] = 1;
               }
	  }
        else timedout[location] = 0;
     }
   GlobalTime = GlobalTime + val;
 }
 

 Message_type Scheduler::ProcessRetrieveMessage(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     return Range[location]->getMessage(); 
 }
 

 void Scheduler::ProcessCopyMessage(Message_type message, ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
     Range[location]->setMessage(message);
 }


 void Scheduler::ProcessStartCounter(ProcessID_type ProcessID, SystemTime_type delay)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
      Range[location]->setWaitingTime(delay);
 }  


 void Scheduler::ProcessStopCounter(ProcessID_type ProcessID)
 { int location = ProcessID - 1;
   if ((location>=0) && 
       (location < Current_Size) && 
       (Range[location] != NULL))
      Range[location]->setWaitingTime(0.0); 
 }


 void Scheduler::ProcessStopCounters_many(logical process_range[MAX_NUMBER_OF_PROCESSES])
 {
  for (int location = 0; location < Current_Size; location++)
     { if (process_range[location]) this->ProcessStopCounter(location);};
 }


 SystemTime_type Scheduler::ProcessGetTime()
 { return GlobalTime; }


 void Scheduler::ProcessReleaseResources(ProcessID_type ProcessID){}
 

 void Scheduler::ProcessResetContext(ProcessID_type ProcessID){}
 
  
 void Scheduler::ProcessResetErrorStatus(){}
 
