/* 
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/
#ifndef __TYPES_H__
#define __TYPES_H__

#include <cstring> 
#include <cstdlib>
#include <cstdio>


/* Constants used only in CPP files */
// #define MAX_VALUE 10
// #define DEFAULT_SIZE 50  
// #define MAX_SIZE_SCHEDULER 10
// #define LOWEST_PRIORITY -99

#define MAX_RESOURCES 100
#define LOWEST_PRIORITY -100
/************************************/

#define pK_MAXSTRINGLENGTH_ 256
typedef char pK_String [pK_MAXSTRINGLENGTH_];

typedef int logical;

#define MAX_NAME_LENGTH 30
#define MAX_NUMBER_OF_PARTITIONS 20
#define MAX_NUMBER_OF_PROCESSES 63
#define MAX_NUMBER_OF_SAMPLING_PORTS 100
#define MAX_NUMBER_OF_QUEUING_PORTS 100
#define MAX_NUMBER_OF_BUFFERS 100
#define MAX_NUMBER_OF_BLACKBOARDS 100
#define MAX_NUMBER_OF_SEMAPHORES 100
#define MAX_SEMAPHORE_VALUE 32767
#define MAX_NUMBER_OF_EVENTS 63
#define ERROR_HANDLER 0
#define MAX_TIMEOUT_VALUE 100.0
#define MAX_ENTRY_POINT_ADDRESS 99999.9
#define MAX_STACK_SIZE_VALUE 100
#define MAX_PERIOD_VALUE 100.0
#define MAX_TIME_CAPACITY_VALUE 100.0
#define APERIODIC -1.0
#define INFINITE_TIME_VALUE -99.0
#define MAX_LOCK_LEVEL 16
#define MAX_PRIORITY_VALUE 63
#define MAX_SAMPLING_PORT_SIZE MAX_MESSAGE_SIZE
#define MAX_BLACKBOARD_MESSAGE_SIZE 1024
#define MAX_BLACKBOARD_SIZE MAX_BLACKBOARD_MESSAGE_SIZE
#define MAX_MESSAGE_SIZE 1024
#define MAX_QUEUING_PORT_SIZE 8*MAX_MESSAGE_SIZE
#define MAX_BUFFER_MESSAGE_SIZE 1024
#define MAX_BUFFER_SIZE 8*MAX_BUFFER_MESSAGE_SIZE


typedef int MessageSize_type;
typedef int BufferSize_type;
typedef int QueuingPortSize_type;
typedef int ProcessID_type;
typedef double MessageArea_type;

// Added L.Besnard -Sept 2007-
#define UNDEFINED_ProcessID_type  -1

struct Message_type
{
  MessageArea_type Message_Area;
  MessageSize_type Message_Size; 
};

typedef int Comm_ComponentID_type;
typedef pK_String Comm_ComponentName_type;
typedef void *QueueAddress_type;
typedef int WaitingRange_type;
typedef int MessageRange_type;
typedef enum{FIFO, PRIORITY} QueuingDiscipline_type;
typedef MessageSize_type BlackboardSize_type;


struct BlackboardStatus_type{
  logical Empty_Indicator;
  MessageSize_type Message_Size;
  WaitingRange_type Waiting_Processes; 
};


struct APEX_Blackboard_type{
  Comm_ComponentName_type Blackboard_Name;
  Message_type Message;
  MessageSize_type Message_Size;
  logical Empty_Indicator;
  WaitingRange_type Waiting_Processes;
  ProcessID_type Waiting_Queue[MAX_NUMBER_OF_PROCESSES]; 
};



struct BufferStatus_type{
  MessageRange_type Nb_Message;
  MessageRange_type Max_Message;
  MessageSize_type Message_Size;
  WaitingRange_type Waiting_Processes; 
};



struct APEX_Buffer_type{
  Comm_ComponentName_type Buffer_Name;
  Message_type Message_Range[MAX_BUFFER_SIZE];
  MessageRange_type Nb_Message;
  MessageRange_type Max_Message;
  MessageSize_type Message_Size;
  WaitingRange_type WaitingOnSend_Processes;
  WaitingRange_type WaitingOnReceive_Processes;
  QueuingDiscipline_type Queuing_Discipline;
  QueueAddress_type WaitingOnSend_Queue;
  QueueAddress_type WaitingOnReceive_Queue; 
};


//typedef logical EventStateValue_type;
typedef enum{UP, DOWN} EventStateValue_type;

   
struct EventStatus_type{
  EventStateValue_type Event_State;
  WaitingRange_type Waiting_Processes; 
};


struct APEX_Event_type{
  Comm_ComponentName_type Event_Name;
  EventStateValue_type Event_State;
  WaitingRange_type Waiting_Processes;
  ProcessID_type Waiting_Queue[MAX_NUMBER_OF_PROCESSES]; 
};


typedef int SemaphoreValue_type;


struct SemaphoreStatus_type{
  SemaphoreValue_type Current_Value;
  SemaphoreValue_type Maximum_Value;
  WaitingRange_type Waiting_Processes; 
};


struct APEX_Semaphore_type{
  Comm_ComponentName_type Semaphore_Name;
  SemaphoreValue_type Current_Value;
  SemaphoreValue_type Maximum_Value;
  WaitingRange_type Waiting_Processes;
  QueuingDiscipline_type Queuing_Discipline;
  QueueAddress_type Waiting_Queue; 
};



typedef pK_String ProcessName_type;
typedef int Priority_type;
typedef int StackSize_type;
typedef int LockLevel_type;
typedef double SystemAddress_type;
typedef enum{DORMANT, READY, RUNNING, WAITING}ProcessState_type;
typedef enum{SOFT, HARD}Deadline_type;
typedef double SystemTime_type;


struct ProcessAttributes_type{
  ProcessName_type Name;
  SystemAddress_type Entry_Point;
  StackSize_type Stack_Size;
  Priority_type Base_Priority;
  SystemTime_type Period;
  SystemTime_type Time_Capacity;
  Deadline_type Deadline; 
};


struct ProcessStatus_type{
  ProcessAttributes_type Attributes;
  Priority_type Current_Priority;
  SystemTime_type Deadline_Time;
  ProcessState_type Process_State; 
};


typedef enum{NO_ERROR, NO_ACTION, NOT_AVAILABLE, INVALID_PARAM, INVALID_CONFIG, INVALID_MODE, TIMED_OUT}ReturnCode_type;
typedef enum{IDLE, COLD_START, WARM_START, NORMAL}OperatingMode_type;
typedef int PartitionID_type;



struct PartitionStatus_type{
  PartitionID_type Identifier;
  SystemTime_type Period;
  SystemTime_type Duration;
  LockLevel_type LockLevel;
  OperatingMode_type OperatingMode; 
};


typedef enum{SOURCE, DESTINATION}PortDirection_type;
typedef int SamplingPortSize_type;



struct SamplingPortStatus_type{
  SamplingPortSize_type SamplingPort_Size;
  PortDirection_type Port_Direction;
  SystemTime_type Refresh_Period;
  logical Validity; 
};



struct APEX_SamplingPort_type{
  Comm_ComponentName_type SamplingPort_Name;
  SamplingPortSize_type SamplingPort_Size;
  PortDirection_type Port_Direction;
  SystemTime_type Refresh_Period;
  logical Validity;
  logical Empty;
  SystemTime_type Message_Age;
  Message_type Message; 
};
 


struct QueuingPortStatus_type{
  QueuingPortSize_type QueuingPort_Size;
  PortDirection_type Port_Direction;
  MessageRange_type Nb_Message;
  WaitingRange_type Waiting_Processes; 
};


struct APEX_QueuingPort_type{
  Comm_ComponentName_type QueuingPort_Name;
  QueuingPortSize_type QueuingPort_Size;
  PortDirection_type Port_Direction;
  MessageSize_type Message_Size;
  MessageRange_type Nb_Message;
  Message_type Message_Range[MAX_QUEUING_PORT_SIZE];
  QueuingDiscipline_type Queuing_Discipline;
  WaitingRange_type WaitingOnSend_Processes;
  WaitingRange_type WaitingOnReceive_Processes;
  QueueAddress_type WaitingOnSend_Queue;
  QueueAddress_type WaitingOnReceive_Queue; 
};


typedef enum{ONSUSPEND, ONWAITING}WaitingMode_type;
typedef enum{LOCKED, UNLOCKED}LockLevelStatus_type;




/* Defined and used in CPP only*/
struct ProcMess_type {
  ProcessID_type Proc;
  Message_type Mess;
};





#endif // __TYPES_H__

  
