/*
   Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - EPL License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).

   Polychrony SME Platform is free software; you can redistribute it and/or modify it under
   the terms of the EPL as published by the Eclipse Foundation.

   Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the Eclipse Public License - v 1.0 for more details.

   You should have received a copy of the Eclipse Public License - v 1.0
   along with Polychrony SME Platform; see the file COPYING. If not, consult the url
   http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse Foundation, Inc.
   102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

extern void setJNIEnv(JNIEnv *env)
{
	currentJNIEnv = env;
}

extern void printTextToConsole(char * text)
{
	if (currentJNIEnv == NULL)
	{
		fprintf(stderr, "Cannot print the text message\n");
		return;
	}

	if (text == NULL)
		return;

	consoleClass = currentJNIEnv->FindClass("fr/irisa/espresso/sme/polychrony/utils/PolychronyConsole");
	if (consoleClass == NULL)
	{
		fprintf(stderr, "The Java PolychronyConsole Class was not found\n");
		return; /* class not found */
	}
	printMethod = currentJNIEnv->GetStaticMethodID(consoleClass, "printInfo", "(Ljava/lang/String;)V");
	if (printMethod == NULL)
	{
		fprintf(stderr, "The Java printInfo method of PolychronyConsole Class was not found\n");
		return;
	}

	jstring chaine = currentJNIEnv->NewStringUTF(text);
	if (chaine == NULL)
	{
		fprintf(stderr, "Out of memory detected\n");
		return; /* OutOfMemoryError already thrown */
	}

	currentJNIEnv->CallStaticVoidMethod(consoleClass, printMethod, chaine);

	// Release memory
	currentJNIEnv->DeleteLocalRef(consoleClass);

	//currentJNIEnv->ReleaseStringUTFChars(chaine, text);
	currentJNIEnv->DeleteLocalRef(chaine);
}

extern void printNewLineToConsole()
{
	if (currentJNIEnv == NULL)
	{
		fprintf(stderr, "Cannot print the new line\n");
		return;
	}

	consoleClass = currentJNIEnv->FindClass("fr/irisa/espresso/sme/polychrony/utils/PolychronyConsole");
	if (consoleClass == NULL)
	{
		fprintf(stderr, "The Java PolychronyConsole Class was not found\n");
		return; /* class not found */
	}
	printNewLineMethod = currentJNIEnv->GetStaticMethodID(consoleClass, "printNewLine", "()V");
	if (printNewLineMethod == NULL)
	{
		fprintf(stderr, "The Java printNewLine method of PolychronyConsole Class was not found\n");
		return;
	}

	currentJNIEnv->CallStaticVoidMethod(consoleClass, printNewLineMethod);

	currentJNIEnv->DeleteLocalRef(consoleClass);
}
