/*
   Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - EPL License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).

   Polychrony SME Platform is free software; you can redistribute it and/or modify it under
   the terms of the EPL as published by the Eclipse Foundation.

   Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the Eclipse Public License - v 1.0 for more details.

   You should have received a copy of the Eclipse Public License - v 1.0
   along with Polychrony SME Platform; see the file COPYING. If not, consult the url
   http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse Foundation, Inc.
   102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
*/

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    newMachine
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_newMachine(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	vpKServices = initMachine();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    initMachine
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_initMachine(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	vpKServices->initialize(eclipsePlugin);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    resetMachine
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_resetMachine(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	vpKServices->reset();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    displayWarnings
 * Signature: (Z)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_displayWarnings(JNIEnv * env, jobject obj, jboolean enable)
{
	setJNIEnv(env);
	// 2 is the value of the Warning mode
	vpKServices->setMessengerOption(2, enable);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_finalize(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	vpKServices->finalyze();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    tree2IGraph
 * Signature: (IZIZLjava/lang/String;Z)Lfr/irisa/espresso/sme/polychrony/GraphValue;
 */
JNIEXPORT jobject JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_tree2DCG(JNIEnv * env, jobject obj, jlong tree, jboolean retTreeError, jlong paramTree, jboolean retParamTreeError, jstring dirName, jboolean genLIS)
{
	setJNIEnv(env);
	ptDCG iGraph = NULL;
	ptDCG_basic gb = NULL;
	char* directory = jstring2CString(dirName, env);
	jint ret = (jint) vpKServices->SIGNALTree2DCG((ntrees) tree, (boolean) retTreeError, (ntrees) paramTree, (boolean) retParamTreeError, directory, (boolean) genLIS, (&iGraph));

	jclass cls = env->FindClass("fr/irisa/espresso/sme/polychrony/GraphValue");
	if (env->ExceptionOccurred() || cls == NULL) return NULL; /* exception thrown */

	/* Get the method ID for the GraphValue constructor */
	jmethodID cid = env->GetMethodID(cls, "<init>", "(IJ)V");
	if (env->ExceptionOccurred() || cid == NULL) return NULL; /* exception thrown */

	/* Construct a GraphValue object */
	gb = new DCG_basic(iGraph);
	jobject result = env->NewObject(cls, cid, ret, (jlong) gb);

	/* Free local references */
	env->DeleteLocalRef(cls);
	return result;
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    signal2SME
 * Signature: (JLjava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_signal2sme__JLjava_lang_String_2(JNIEnv * env, jobject obj, jlong tree, jstring dirName)
{
	setJNIEnv(env);
	char* directory = jstring2CString(dirName, env);
	return (jint) vpKServices->signal2xmi((ntrees) tree, directory);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    signal2SME
 * Signature: (Ljava/lang/String;Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_signal2sme__Ljava_lang_String_2Ljava_lang_String_2(JNIEnv * env, jobject obj, jstring srcFile, jstring dirName)
{
	setJNIEnv(env);
	char* file = jstring2CString(srcFile, env);
	char* directory = jstring2CString(dirName, env);
	return (jint) vpKServices->signal2xmi(file, directory);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2C
 * Signature: (IZZZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_genCodeC(JNIEnv * env, jobject obj, jlong graphNode, jboolean mainFile, jboolean bodyFile, jboolean ioFile)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->genCodeC( (ptDCG_sch) graphNode, 1);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2CPP
 * Signature: (IZZZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_genCodeCPP(JNIEnv * env, jobject obj, jlong graphNode, jboolean mainFile, jboolean bodyFile, jboolean ioFile)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->genCodeCPP( (ptDCG_sch) graphNode, 1);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2CPPSeparate
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_genCodeCPPSeparate(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->genCodeCPP( (ptDCG_sch) graphNode, 3);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2CSeparate
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_genCodeCSeparate(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->genCodeC( (ptDCG_sch) graphNode, 3);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2DotCode
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_IGraph2DotCode(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return true;
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2Java
 * Signature: (IZZZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_genCodeJava(JNIEnv * env, jobject obj, jlong graphNode, jboolean mainFile, jboolean bodyFile, jboolean ioFile)
{
  setJNIEnv(env);
  return (jboolean) vpKServices->genCodeJava( (ptDCG_sch) graphNode, 1);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2JavaSeparate
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_genCodeJavaSeparate(JNIEnv * env, jobject obj, jlong graphNode)
{
  setJNIEnv(env);
  return (jboolean) vpKServices->genCodeJava( (ptDCG_sch) graphNode, 3);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2threadsJava
 * Signature: (IZZZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_genCodeJavaThreads(JNIEnv * env, jobject obj, jlong graphNode, jboolean mainFile, jboolean bodyFile, jboolean ioFile)
{
  setJNIEnv(env);
  return (jboolean) vpKServices->genCodeJava( (ptDCG_sch) graphNode, 2); 
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2Kernel
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_expressionExpand(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->expressionExpand((ptDCG_basic) graphNode);
}


/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2Signal
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_unparse(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->unparse((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2SigTree
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_produce(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jlong) vpKServices->produce((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2SigTreeFlat
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_produceFlat(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jlong) vpKServices->produceFlat((ptDCG_basic) graphNode, false);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2SynDEx
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_SynDExUnparse(JNIEnv * env, jobject obj, jlong graphNode)
{
  setJNIEnv(env);
  return (jboolean) vpKServices->SynDExUnparse((ptDCG_flat) graphNode, true);
}


/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2Z3z
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_SigaliPrintExtraction(JNIEnv * env, jobject obj, jlong graphNode)
{
  setJNIEnv(env);
  return (jboolean) vpKServices->SigaliPrintExtraction((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphAbstraction2SigTree
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_produceInterfaceAbstraction(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jlong) vpKServices->produceInterfaceAbstraction((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphAbstractionGraph
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_buildAbstraction(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	ptDCG_poly * gResult = 0;
	if(vpKServices->buildAbstraction((ptDCG_poly) graphNode, gResult)) return (jlong) gResult;
	return (jlong) ERROR_NODE;
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphAggregatingMultiplexing
 * Signature: (IZ)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_memoryOptimizations(JNIEnv * env, jobject obj, jlong graphNode, jboolean delayMultiplexing)
{
	setJNIEnv(env);
	vpKServices->memoryOptimizations((ptDCG_poly)graphNode, delayMultiplexing);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphBooleans2Events
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_transformBooleanClocks2Events(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->transformBooleanClocks2Events((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphBooleansOthersPartitionning
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_separateBooleansOthers(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	vpKServices->separateBooleansOthers((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphClockEnvBuilding
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_setIntrinsicClocks(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->setIntrinsicClocks((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphCodeGenerationReinforcing
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_dependenceReinforcing(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->dependenceReinforcing((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphConstraintsAsExceptions
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_addExceptionsForConstraints(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->addExceptionsForConstraints((ptDCG_bool) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphCycleDetection
 * Signature: (IZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_hasStaticCycle(JNIEnv * env, jobject obj, jlong graphNode, jboolean debug)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->hasStaticCycle((ptDCG_poly) graphNode, debug);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphDefaultSpliting
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_splitDefaultDefinition(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->splitDefaultDefinition((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphDelaysMemoryOptimisation
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_setDelayMemoryOverloading(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->setDelayMemoryOverloading((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphDownStreamRetiming
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_moveIn2OutDelays(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->moveIn2OutDelays((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    n_IGraphExclusiveClocks
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_computeClockExclusions(JNIEnv * env, jobject obj, jlong graphNode, jint ecLevel)
{
	setJNIEnv(env);
	vpKServices->computeClockExclusions((ptDCG_poly)graphNode, (integer) ecLevel);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphExpand
 * Signature: (IZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_expandModelReference(JNIEnv * env, jobject obj, jlong graphNode, jboolean asSubGraph)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->expandModelReference((ptDCG_basic) graphNode, asSubGraph);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphFlatten
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_breakCPG(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->breakCPG((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphGenCodeVerifs
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_checkExplicitTyping(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->checkExplicitTyping((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphHierarchyStatistics
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_printCHStatistics(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->printCHStatistics((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphInputsTrain
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_phylumClustering(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->phylumClustering((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphInterfaceCalculus
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_buildClockProcessInterface(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->buildClockProcessInterface((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphIsWithConstraints
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_hasUnsolvedClockConstraints(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->hasUnsolvedClockConstraints((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphMakeTreeDelays
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_delayTreeBuild(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->delayTreeBuild((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphPartialDefinitionsMerging
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_mergePartialDefinitions(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->mergePartialDefinitions((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphProfiling
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_applyTabularMorphism(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->applyTabularMorphism((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphProjection
 * Signature: (IZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_getIOClockSubHierarchy(JNIEnv * env, jobject obj, jlong graphNode, jboolean clockOfDependences)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->getIOClockSubHierarchy((ptDCG_poly) graphNode, clockOfDependences);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphPruning
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_nonProductiveSignalElimination(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->nonProductiveSignalElimination((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphRenamesEliminating
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_removeSignalEqualities(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->removeSignalEqualities((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSeparatedCompiling
 * Signature: (IZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_separatedCompilingPoly(JNIEnv * env, jobject obj, jlong graphNode, jboolean genCode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->separatedCompilingPoly((ptDCG_poly) graphNode/*, genCode*/);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSequentializing
 * Signature: (IZZZ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_Serializing(JNIEnv * env, jobject obj, jlong graphNode, jboolean btotalOrder, jboolean balwaysCode, jboolean bcheck, jboolean bdelayMultiplexing)
{
	setJNIEnv(env);
	ptDCG_sch gSeq = NULL;
	return (jboolean) vpKServices->Serializing((ptDCG_bool) graphNode, btotalOrder, balwaysCode, bcheck, bdelayMultiplexing, &gSeq);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSetDependences
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_setDataDependences(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->setDataDependences((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSIGNALAbstractionPrinting
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_unparseAbstraction(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->unparseAbstraction((ptDCG_bool) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSplitingStructure
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_splitDataStructure(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->splitDataStructure((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphStatesOthersPartitionning
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_separateStatesOthers(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	vpKServices->separateStatesOthers( (ptDCG_poly)graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphStateVarUtilityClock
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_computeProductiveClockStateVar(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->computeProductiveClockStateVar((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSubGraph2SigTree
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_produceSubgraph(JNIEnv * env, jobject obj, jlong graphNode, jlong subGraph)
{
	setJNIEnv(env);
	return (jlong) vpKServices->produceSubgraph((ptDCG_basic) graphNode, (ptINode) subGraph);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSubstitute
 * Signature: (II)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_substituteExpr2Var(JNIEnv * env, jobject obj, jlong graphNode, jint maxRef)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->substituteExpr2Var((ptDCG_basic) graphNode, maxRef);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphTransitiveClosure
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_computeIOTransitiveClosure(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->computeIOTransitiveClosure((ptDCG_poly) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphUnifyingNodes
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_removeSignalClones(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->removeSignalClones((ptDCG_basic) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphUtilityClock
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_computeProductiveClock(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->computeProductiveClock((ptDCG_poly) graphNode);
}


/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    Signal2Lustre
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_LustreUnparse(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->LustreUnparse((ptDCG_bool) graphNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    parse
 * Signature: (Ljava/lang/String;Ljava/lang/String;)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_parse(JNIEnv * env, jobject obj, jstring src, jstring param)
{
	setJNIEnv(env);
	char* paramfile= jstring2CString(param, env);
	char *srcfile = jstring2CString(src, env);

	int n = vpKServices->SIGNALFiles2DCG(srcfile, paramfile, ".", false, &vgraph);
	pK_messenger->gmessload(mkfgmess);
	if(n == 0) return (jlong) vgraph;
	return (jlong) ERROR_NODE;
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    printSignalLIS
 * Signature: (JJ)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_printSignalLIS
  (JNIEnv * env, jobject obj, jlong tree, jlong params)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->printSignalLis((ntrees)tree,(ntrees)params);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    setEnv
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_setEnv(JNIEnv * env, jobject obj, jstring key, jstring value)
{
	setJNIEnv(env);
	char *cKey = jstring2CString(key, env);
	char *cValue = jstring2CString(value, env);

	if(vpKServices != NULL)
	{
		vpKServices->setEnv(cKey, cValue);
	}
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    getEnv
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_getEnv(JNIEnv * env, jobject obj, jstring key)
{
  setJNIEnv(env);
  char *cKey = jstring2CString(key, env);
  char *cValue = NULL;
  
  if(vpKServices != NULL) cValue = vpKServices->getEnv(cKey);
  if(cValue == NULL) return (jstring) NULL;
  return (jstring) env->NewStringUTF(cValue);
}

// **************************

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphBooleanizing
 * Signature: (I)Z
 */
JNIEXPORT jobject JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_DCGPoly2DCGBool(JNIEnv * env, jobject obj, jlong graphNode)
{
    ptDCG_bool gBool;
    setJNIEnv(env);

    jint ret = 0;
    if ((jboolean) vpKServices->DCGPoly2DCGBool((ptDCG_poly) graphNode, &gBool) == false) {
    	ret = 1;
    }

    jclass cls = env->FindClass("fr/irisa/espresso/sme/polychrony/GraphValue");
    if (env->ExceptionOccurred() || cls == NULL) return NULL; /* exception thrown */

    /* Get the method ID for the GraphValue constructor */
    jmethodID cid = env->GetMethodID(cls, "<init>", "(IJ)V");
    if (env->ExceptionOccurred() || cid == NULL) return NULL; /* exception thrown */

    /* Construct a GraphValue object */
    jobject result = env->NewObject(cls, cid, ret, (jlong) gBool);

    return result;
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSTSFlattening
 * Signature: (I)Z
 */
JNIEXPORT jobject JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_DCGPoly2DCGFlat(JNIEnv * env, jobject obj, jlong graphNode)
{
    ptDCG_flat gf;
    setJNIEnv(env);

    jint ret = 0;
    if ((jboolean) vpKServices->DCGPoly2DCGFlat((ptDCG_poly) graphNode, &gf) == false) {
    	ret = 1;
    }

    jclass cls = env->FindClass("fr/irisa/espresso/sme/polychrony/GraphValue");
    if (env->ExceptionOccurred() || cls == NULL) return NULL; /* exception thrown */

    /* Get the method ID for the GraphValue constructor */
    jmethodID cid = env->GetMethodID(cls, "<init>", "(IJ)V");
    if (env->ExceptionOccurred() || cid == NULL) return NULL; /* exception thrown */

    /* Construct a GraphValue object */
    jobject result = env->NewObject(cls, cid, ret, (jlong) gf);

    return result;
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphClockCalculus
 * Signature: (I)I
 */
JNIEXPORT jobject JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_DCGBasic2DCGPoly(JNIEnv * env, jobject obj, jlong graphNode)
{
    setJNIEnv(env);
    ptDCG_poly gp;
    jint ret = (jint) vpKServices->DCGBasic2DCGPoly((ptDCG_basic) graphNode, 5, &gp);
    jclass cls = env->FindClass("fr/irisa/espresso/sme/polychrony/GraphValue");
    if (env->ExceptionOccurred() || cls == NULL) return NULL; /* exception thrown */

    /* Get the method ID for the GraphValue constructor */
    jmethodID cid = env->GetMethodID(cls, "<init>", "(IJ)V");
    if (env->ExceptionOccurred() || cid == NULL) return NULL; /* exception thrown */

    /* Construct a GraphValue object */
    jobject result = env->NewObject(cls, cid, ret, (jlong) gp);

    return result;
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    DCG2dotCode
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_DotUnparse(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->DotUnparse((ptDCG_basic) graphNode);
}

// ===removed===
/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraph2Vhdl
 * Signature: (I)Z

JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_IGraph2Vhdl(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->IGraph2vhdl((ptIGraph) graphNode);
}
 */

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphsbDCplus2bDCplus
 * Signature: (I)Z
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_IGraphsbDCplus2bDCplus(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->IGraphUnserializing((ptIGraph) graphNode);
}
 */

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphStatus2StateVariables
 * Signature: (I)Z
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_IGraphStatus2StateVariables(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->IGraphStatus2stateVariables((ptIGraph) graphNode);
}
 */

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphSTS2DC
 * Signature: (I)Z
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_IGraphSTS2DC(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->IGraphSTS2DC((ptIGraph) graphNode);
}

*/

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphArrayClocksFixing
 * Signature: (I)Z

JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_IGraphArrayClocksFixing(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jboolean) vpKServices->IGraphArrayClocksFixing((ptDCG_basic) graphNode);
}

*/

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKServices
 * Method:    IGraphCopy
 * Signature: (I)I

JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKServices_IGraphCopy(JNIEnv * env, jobject obj, jlong graphNode)
{
	setJNIEnv(env);
	return (jlong) vpKServices->IGraphCopy((ptIGraph) graphNode);
}
*/
