/*
   Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - EPL License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).

   Polychrony SME Platform is free software; you can redistribute it and/or modify it under
   the terms of the EPL as published by the Eclipse Foundation.

   Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the Eclipse Public License - v 1.0 for more details.

   You should have received a copy of the Eclipse Public License - v 1.0
   along with Polychrony SME Platform; see the file COPYING. If not, consult the url
   http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse Foundation, Inc.
   102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    memoryFinalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_memoryFinalize(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	ntreesMemoryFinalize ();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    memoryInitialize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_memoryInitialize(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	stringMemoryInitialize();
	ntreesMemoryInitialize ();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    atomChar
 * Signature: (IC)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_atomChar(JNIEnv * env, jobject obj, jint opCode, jchar c)
{
	return (jlong) natomchar ( (noperators) opCode, (char) c);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    atomInt
 * Signature: (IJ)J
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_atomInt(JNIEnv * env, jobject obj, jint opCode, jlong val)
{
	return (jlong) natomint ( (noperators) opCode, (long) val);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    atomString
 * Signature: (ILjava/lang/String;)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_atomString(JNIEnv * env, jobject obj, jint opCode, jstring s)
{
	return (jlong) natomstring ((noperators) opCode, jstring2str(s, env));
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    changChild
 * Signature: (III)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_changChild(JNIEnv * env, jobject obj, jlong listNode, jlong node, jint i)
{
	nchangchild ( (ntrees) listNode, (ntrees) node, (int) i);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    charOf
 * Signature: (I)C
 */
JNIEXPORT jchar JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_charOf(JNIEnv * env, jobject obj, jlong node)
{
	return (char) ncharof ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    child
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_child(JNIEnv * env, jobject obj, jint i, jlong node)
{
	return (jlong) nchild ( (int) i, (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    copyList
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_copyList(JNIEnv * env, jobject obj, jlong node)
{
	return (jlong) nrecopielist ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    destroy
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_destroy(JNIEnv * env, jobject obj, jlong listNode, jint i)
{
	return (jlong) ndestroy ( (ntrees) listNode, (int) i);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    getFather
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_getFather(JNIEnv * env, jobject obj, jlong node)
{
	return (jlong) nfather ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    getNilTree
 * Signature: ()I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_getNilTree(JNIEnv * env, jobject obj)
{
	return (jlong) ngetniltree ();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    insertLeft
 * Signature: (III)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_insertLeft(JNIEnv * env, jobject obj, jint i, jlong node, jlong listNode)
{
	ninsertleft ( (int) i, (ntrees) node, (ntrees) listNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    insertRight
 * Signature: (III)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_insertRight(JNIEnv * env, jobject obj, jint i, jlong node, jlong listNode)
{
	ninsertright ( (int) i, (ntrees) node, (ntrees) listNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    intOf
 * Signature: (J)J
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_intOf(JNIEnv * env, jobject obj, jlong node)
{
	return (jlong) nintof ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    isEmpty
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_isEmpty(JNIEnv * env, jobject obj, jlong node)
{
	return (jboolean) nisempty ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    isNilTree
 * Signature: (I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_isNilTree(JNIEnv * env, jobject obj, jlong node)
{
	return (jboolean) nisniltree ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    left
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_left(JNIEnv * env, jobject obj, jint i, jlong node)
{
	return (jlong) nleft ( (int) i, (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeBinary
 * Signature: (III)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeBinary(JNIEnv * env, jobject obj, jint opCode, jlong node1, jlong node2)
{
	return (jlong) nmakebinary ( (noperators) opCode, (ntrees) node1, (ntrees) node2);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeEmptyList
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeEmptyList(JNIEnv * env, jobject obj, jint opCode)
{
	return (jlong) nmakemptylist ( (noperators) opCode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeList
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeList(JNIEnv * env, jobject obj, jint opCode, jlong elem)
{
	return (jlong) nmakelist ( (noperators) opCode, (ntrees) elem);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeNullary
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeNullary(JNIEnv * env, jobject obj, jint opCode)
{
	return (jlong) natomstring ((noperators) opCode, nilstr);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeQuaternary
 * Signature: (IIIII)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeQuaternary(JNIEnv * env, jobject obj, jint opCode, jlong node1, jlong node2, jlong node3, jlong node4)
{
	return (jlong) nmakequaternary ( (noperators) opCode, (ntrees) node1, (ntrees) node2, (ntrees) node3, (ntrees) node4);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeTernary
 * Signature: (IIII)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeTernary(JNIEnv * env, jobject obj, jint opCode, jlong node1, jlong node2, jlong node3)
{
	return (jlong) nmaketernary ( (noperators) opCode, (ntrees) node1, (ntrees) node2, (ntrees) node3);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeUnary
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeUnary(JNIEnv * env, jobject obj, jint opCode, jlong node)
{
	return (jlong) nmakeunary ( (noperators) opCode, (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    modifCharVal
 * Signature: (IC)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_modifCharVal(JNIEnv * env, jobject obj, jlong node, jchar c)
{
	nmodifvalcar ( (ntrees) node, (char) c);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    modifIntVal
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_modifIntVal(JNIEnv * env, jobject obj, jlong node, jint val)
{
	nmodifvalint ( (ntrees) node, (int) val);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    oper
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_oper(JNIEnv * env, jobject obj, jlong node)
{
	return (jint) noper ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    posit
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_posit(JNIEnv * env, jobject obj, jlong node)
{
	return (jint) nposit ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    post
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_post(JNIEnv * env, jobject obj, jlong listNode, jlong node)
{
	return (jlong) npost ( (ntrees) listNode, (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    pre
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_pre(JNIEnv * env, jobject obj, jlong listNode, jlong node)
{
	return (jlong) npre ( (ntrees) node, (ntrees) listNode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    right
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_right(JNIEnv * env, jobject obj, jint i, jlong node)
{
	return (jlong) nright ( (int) i, (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    setOp
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_setOp(JNIEnv * env, jobject obj, jlong node, jint opCode)
{
	return (jlong) nsetop ( (ntrees) node, (noperators) opCode);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    stringOf
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_stringOf(JNIEnv * env, jobject obj, jlong node)
{
	return str2jstring(nstringof((ntrees) node), env);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    treeArity
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_treeArity(JNIEnv * env, jobject obj, jlong node)
{
	return (jint) ntreearity ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    treeDestroy
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_treeDestroy(JNIEnv * env, jobject obj, jlong node)
{
	ntreesDestroy ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    treesLoad
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_treesLoad(JNIEnv * env, jobject obj, jstring filename)
{
	return (jlong) ntrees_load (jstring2CString(filename, env));
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    treesSave
 * Signature: (Ljava/lang/String;I)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_treesSave(JNIEnv * env, jobject obj, jstring filename, jlong node)
{
	return (jboolean) ntrees_save ( jstring2CString(filename, env), (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    attrCarrier
 * Signature: (I)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_attrCarrier(JNIEnv * env, jobject obj, jlong node)
{
	return (jlong) nattrcarrier ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    attrExist
 * Signature: (II)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_attrExist(JNIEnv * env, jobject obj, jlong node, jint num)
{
	return (jboolean) nattrexist ( (ntrees) node, (int) num);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    attrInt
 * Signature: (II)I
 */
JNIEXPORT jint JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_attrInt(JNIEnv * env, jobject obj, jlong node, jint num)
{
	return (jint) nattrint ( (ntrees) node, (int) num);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    attrNtrees
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_attrNtrees(JNIEnv * env, jobject obj, jlong node, jint num)
{
	return (jlong) nattrntrees ( (ntrees) node, (int) num);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    attrNumber
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_attrNumber(JNIEnv * env, jobject obj, jlong node)
{
	return (jint) nattrnumber ( (ntrees) node);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    attrPtr
 * Signature: (II)I
 */
JNIEXPORT jlong JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_attrPtr(JNIEnv * env, jobject obj, jlong node, jint num)
{
	return (jlong) nattrptr((ntrees) node, (int) num);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    attrStr
 * Signature: (II)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_attrStr(JNIEnv * env, jobject obj, jlong node, jint num)
{
	return str2jstring(nattrstr ((ntrees) node, (int) num), env);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    destroyAttr
 * Signature: (II)Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_destroyAttr(JNIEnv * env, jobject obj, jlong node, jint num)
{
	return (jboolean) ndestroyattr ( (ntrees) node, (int) num);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeAttrInt
 * Signature: (III)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeAttrInt(JNIEnv * env, jobject obj, jlong node, jint num, jint val)
{
	nmakeattrint ( (ntrees) node, (int) num, (int) val);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeAttrNtrees
 * Signature: (III)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeAttrNtrees(JNIEnv * env, jobject obj, jlong node, jint num, jlong val)
{
	nmakeattrntrees ( (ntrees) node, (int) num, (ntrees) val);
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    makeAttrPtr
 * Signature: (III)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeAttrPtr(JNIEnv * env, jobject obj, jlong node, jint num, jlong ptr)
{
	cout << "Java_fr_irisa_espresso_sme_polychrony_PKTrees_nMakeAttrPtr : not implemented " << endl;
	// nmakeattrptr( (ntrees) node, (int) num, ptr); // ??? revoir
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    nMakeAttrStr
 * Signature: (IILjava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_makeAttrStr(JNIEnv * env, jobject obj, jlong node, jint num, jstring str)
{
	nmakeattrstr ((ntrees) node, (int) num, jstring2str(str, env));
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    globalMemoryReset
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_globalMemoryReset(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	ntrees_GlobalMemoryReset ();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    memoryReset
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_memoryReset(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	ntrees_memoryReset ();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    memoryRestoreIndex
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_memoryRestoreIndex(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	ntrees_memoryRestoreIndex ();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    memoryStoreIndex
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_memoryStoreIndex(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	ntrees_memoryStoreIndex ();
}

/*
 * Class:     fr_irisa_espresso_sme_polychrony_PKTrees
 * Method:    needGarbage
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_fr_irisa_espresso_sme_polychrony_PKTrees_needGarbage(JNIEnv * env, jobject obj)
{
	setJNIEnv(env);
	return (jboolean) ntrees_recup_necessaire ();
}
