/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

public enum ExclusiveClockLevelKind implements Enumerator {
	ONLY_SIGNAL_CLOCKS_LITERAL(0, "onlySignalClocks", "Only Signal clocks"), ACTION_P_LITERAL(1,
			"sigClocksAndExprClock", "Signal clocks & clocks referenced in expression"), ALL_CLOCKS_LITERAL(2,
			"allClocks", "All the clocks of the graph");
	
	public static final int								ONLY_SIGNAL_CLOCKS	= 0;
	public static final int								ACTION_P			= 1;
	public static final int								ALL_CLOCKS			= 2;
	
	private static final ExclusiveClockLevelKind[]		VALUES_ARRAY		= new ExclusiveClockLevelKind[] {
			ONLY_SIGNAL_CLOCKS_LITERAL, ACTION_P_LITERAL, ALL_CLOCKS_LITERAL, };
	
	public static final List<ExclusiveClockLevelKind>	VALUES				= Collections.unmodifiableList(Arrays
																					.asList(VALUES_ARRAY));
	
	public static ExclusiveClockLevelKind get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			ExclusiveClockLevelKind result = VALUES_ARRAY[i];
			if (result.toString().equals(literal)) {
				return result;
			}
		}
		return null;
	}
	
	public static ExclusiveClockLevelKind getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			ExclusiveClockLevelKind result = VALUES_ARRAY[i];
			if (result.getName().equals(name)) {
				return result;
			}
		}
		return null;
	}
	
	public static ExclusiveClockLevelKind get(int value) {
		switch (value) {
			case ONLY_SIGNAL_CLOCKS:
				return ONLY_SIGNAL_CLOCKS_LITERAL;
			case ACTION_P:
				return ACTION_P_LITERAL;
			case ALL_CLOCKS:
				return ALL_CLOCKS_LITERAL;
			default:
				return ONLY_SIGNAL_CLOCKS_LITERAL;
		}
	}
	
	private final int		value;
	
	private final String	name;
	
	private final String	literal;
	
	/**
	 * @param value
	 * @param name
	 * @param literal
	 */
	private ExclusiveClockLevelKind(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.util.Enumerator#getLiteral()
	 */
	public String getLiteral() {
		return literal;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.util.Enumerator#getName()
	 */
	public String getName() {
		return name;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.util.Enumerator#getValue()
	 */
	public int getValue() {
		return value;
	}
}
