/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony;

import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import fr.irisa.espresso.sme.polychrony.traceability.TreesInterface;

/**
 * The activator class controls the plug-in life cycle
 */
public class PkPlugin extends AbstractUIPlugin {
	
	/** The plug-in ID */
	public static final String	PLUGIN_ID			= "fr.irisa.espresso.sme.polychrony";
	
	/** The shared instance */
	private static PkPlugin		plugin;
	
	private static PKServices	POLYCHRONY_SERVICES	= null;
	
	public PkPlugin() {}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}
	
	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static PkPlugin getDefault() {
		return plugin;
	}
	
	/**
	 * @see java.lang.Object#finalize()
	 * @generated NOT
	 */
	@Override
	protected void finalize() throws Throwable {
		super.finalize();
		
		// Finalize the Polychrony Machine
		if (POLYCHRONY_SERVICES != null)
			POLYCHRONY_SERVICES.finalize();
	}
	
	/**
	 * @return the Polychrony Services API
	 */
	public static PKServices getServices() {
		if (POLYCHRONY_SERVICES == null)
			POLYCHRONY_SERVICES = PKServices.INSTANCE;
		return POLYCHRONY_SERVICES;
	}
	
	/**
	 * @return the Tree Printer Generator
	 */
	public static SignalTreePrinter getPrinter() {
		return getServices().getPrinter();
	}
	
	/**
	 * @return the Polychrony parser
	 * @generated NOT
	 * */
	public static SignalParser getParser() {
		return getServices().getParser();
	}
	
	/**
	 * @return the Trees builder
	 * @generated NOT
	 */
	public static SignalTreeBuilting getTreeBuilder() {
		return getServices().getTreeBuilder();
	}
	
	/**
	 * @return the Trees builder
	 * @generated NOT
	 */
	public static SignalTreeExploring getTreeExplorer() {
		return getServices().getTreeExplorer();
	}
	/**
	 * @return the PKTrees API
	 * @generated NOT
	 */
	public static PKTrees getTreeAPI() {
		return getServices().getTreeAPI();
	}
	
	public static SignalTreeComment getCommentAPI() {
		return getServices().getCommentAPI();
	}
	
	public static TreesInterface getTraceabilityAPI() {
		return getServices().getTraceabilityAPI();
	}
}
