/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony;

// defined from opersignal.h

/**
 * This class contains the constant values for all signal operators in the abstract syntax tree.
 * 
 */
public class SignalOperators {
	public final static int	NULLOP				= 0;		// added operator
															
	// NULLARY OPERATORS.
	public final static int	opnullary			= 1;
	
	public final static int	opnom				= 1;
	public final static int	opchar				= 2;
	public final static int	opcomplex			= 3;
	public final static int	opcstcar			= 4;
	public final static int	opcstcha			= 5;
	public final static int	opcstdp				= 6;
	public final static int	opcstent			= 7;
	public final static int	opcstreel			= 8;
	public final static int	opdcomplex			= 9;
	public final static int	opdpreal			= 10;
	public final static int	opevent				= 11;
	public final static int	opfalse				= 12;
	public final static int	ophnull				= 13;
	public final static int	opimplicit			= 14;
	public final static int	opinteger			= 15;
	public final static int	oplogical			= 16;
	public final static int	oplong				= 17;
	public final static int	opnil				= 18;
	public final static int	opreal				= 19;
	public final static int	opretun				= 20;
	public final static int	opshort				= 21;
	public final static int	opstring			= 22;
	public final static int	optabvide			= 23;
	public final static int	optrue				= 24;
	public final static int	opunsafe			= 25;
	public final static int	opmeta				= 26;
	public final static int	opcomment			= 27;
	public final static int	opnullproc			= 28;
	public final static int	opop				= 29;
	public final static int	opdeterm			= 30;
	public final static int	opsafe				= 31;
	
	// UNARY OPERATORS.
	public final static int	opunary				= 101;
	public final static int	opappeassert		= 101;
	// suppressed opcorpp 102 (value used now by oppassert)
	public final static int	opevnt				= 103;
	public final static int	opexte				= 104;
	public final static int	opvvarin			= 105;
	public final static int	oplab				= 106;
	public final static int	opnega				= 107;
	public final static int	opref				= 108;
	public final static int	opnon				= 109;
	public final static int	oppassert			= 102;
	public final static int	oppcar				= 110;
	public final static int	opsdec				= 111;
	public final static int	opstru				= 112;
	// free - public final static int opsuit= 113;
	public final static int	optran				= 114;
	// suppressed opvconp = 115;
	// suppressed opvtypp = 116;
	public final static int	opwhen1				= 117;
	
	// free 11_-126
	public final static int	opcstenum			= 127;		// added operator : access to a value of an enumerated type.
	public final static int	opindirection		= 128;		// added operator : indirection operator (used for
	// rewritting)
	public final static int	opread				= 129;		// added operator : read node (morphism)
	public final static int	opwrite				= 130;		// added operator : write node (morphism)
															
	// BINARY OPERATORS.
	public final static int	opbinary			= 201;
	public final static int	opappe				= 202;
	public final static int	opbinf				= 203;
	public final static int	opbsup				= 204;
	public final static int	opbund				= 205;
	public final static int	opcham				= 206;
	public final static int	opcomp				= 207;
	public final static int	opconc				= 208;
	public final static int	opcons				= 209;
	public final static int	opconv				= 210;
	public final static int	opcplx				= 211;
	public final static int	opdeccon			= 212;
	public final static int	opdecconp			= 213;
	public final static int	opdecsig			= 214;
	public final static int	opdefsig			= 215;
	public final static int	opdesc				= 216;
	public final static int	opdiff				= 217;
	public final static int	opdivi				= 218;
	public final static int	opegal				= 219;
	public final static int	opegeg				= 220;
	public final static int	opelem				= 221;
	public final static int	opelevec			= 222;
	public final static int	opcas				= 223;
	public final static int	opensvec			= 224;
	public final static int	opentsor			= 225;
	public final static int	opet				= 226;
	public final static int	opextr				= 227;
	public final static int	opformact			= 228;
	public final static int	opiegal				= 229;
	public final static int	opiieg				= 230;
	// free public final static int opimpo= 231;
	public final static int	opindmot			= 232;
	public final static int	opinfe				= 233;
	public final static int	opinit				= 234;
	public final static int	opinst				= 235;
	public final static int	opmele				= 236;
	public final static int	opdefegal			= 237;
	public final static int	opmmul				= 238;
	public final static int	opmod				= 239;
	public final static int	opdescmodu			= 240;
	public final static int	opdefpartdef		= 241;
	// free public final static int optypeact= 242;
	public final static int	opmodu				= 243;
	public final static int	opmoin				= 244;
	public final static int	opmult				= 245;
	public final static int	opnomba				= 246;
	public final static int	opnombd				= 247;
	public final static int	opnombm				= 248;
	public final static int	opobli				= 249;
	public final static int	opou				= 250;
	public final static int	oppconf				= 251;
	public final static int	oppetiq				= 252;
	public final static int	opparc				= 253;
	public final static int	opplus				= 254;
	// free public final static int opfrmot= 255;
	public final static int	oppuis				= 256;
	public final static int	opouex				= 257;
	public final static int	opvenum				= 258;
	public final static int	oprecu				= 259;
	public final static int	oprepe				= 260;
	public final static int	oprest				= 261;
	public final static int	opsegal				= 262;
	public final static int	opsequ				= 263;
	public final static int	opspec				= 264;
	public final static int	opdecvar			= 265;
	public final static int	opsupe				= 266;
	public final static int	optvect				= 267;
	// free public final static int opvali= 268;
	public final static int	opvar				= 269;
	public final static int	opident				= 270;		// added operator
	public final static int	opdefmul			= 273;		// added operator
	public final static int	opdecpar			= 274;		// added operator
	// free 275-308
	public final static int	opleft_tt			= 309;		// added operator
	// free 310-339
	
	public final static int	opintvff			= 340;
	public final static int	opintvfo			= 341;
	public final static int	opintvof			= 342;
	public final static int	opintvoo			= 343;
	public final static int	opdefpart			= 344;
	// free public final static int optypefunc= 345;
	// free public final static int optypenode= 346;
	// free public final static int optypeproc= 347;
	public final static int	opformfunc			= 348;
	public final static int	opformnode			= 349;
	public final static int	opformproc			= 350;
	
	// LIST OPERATORS.
	public final static int	oplists				= 401;
	
	public final static int	opchai				= 401;
	public final static int	oplcons				= 402;
	public final static int	oplcrohor			= 403;
	public final static int	opldechor			= 404;
	public final static int	opldecs				= 405;
	public final static int	opldecl				= 406;
	public final static int	opleflot			= 407;
	public final static int	oplepara			= 408;
	public final static int	oplesign			= 409;
	public final static int	oplevect			= 410;
	public final static int	oplelevec			= 411;
	// free public final static int oplentmot= 412;
	public final static int	oplequhor			= 413;
	public final static int	oplexchor			= 414;
	public final static int	oplfeed				= 415;
	public final static int	oplident			= 416;
	public final static int	opldep				= 417;
	public final static int	oplopra				= 418;
	public final static int	oplparc				= 419;
	public final static int	oplprag				= 420;
	public final static int	oplsign				= 421;
	// free public final static int oplsync= 422; replaced by oplequhor
	public final static int	opltype				= 423;
	public final static int	opltypep			= 424;
	
	public final static int	opcomments			= 425;
	public final static int	oplsuff				= 426;		// added operator
	public final static int	oplread				= 427;		// added operator
	public final static int	oplwrite			= 428;		// added operator
	public final static int	opldefpart			= 429;		// added operator
															
	public final static int	oplimpo				= 440;
	public final static int	oplmodu				= 441;
	public final static int	optenum				= 442;
	public final static int	oplcas				= 443;
	public final static int	oplattr				= 444;
	
	// TERNARY OPERATORS.
	public final static int	opternary			= 501;
	
	public final static int	opborn				= 501;
	public final static int	opcond				= 502;
	public final static int	opdepd				= 503;
	public final static int	openum				= 504;
	public final static int	opfen				= 505;
	public final static int	opfunc				= 506;
	public final static int	opfuncp				= 507;
	public final static int	opinte				= 508;
	public final static int	opmoti				= 509;
	public final static int	opprag				= 510;
	public final static int	opprocess			= 511;
	public final static int	opprocessp			= 512;
	public final static int	opret				= 513;
	public final static int	optype				= 514;
	public final static int	opnodecall			= 515;		// added operator
	public final static int	opsubgraph			= 516;		// added operator
	public final static int	opact				= 517;
	public final static int	opactp				= 518;
	public final static int	opiter				= 519;
	public final static int	optypeact			= 520;
	public final static int	optypefunc			= 521;
	public final static int	opnode				= 522;
	public final static int	opnodep				= 523;
	public final static int	opmemo				= 524;
	public final static int	opoper				= 525;
	public final static int	optypenode			= 526;
	public final static int	optypeproc			= 527;
	// free
	public final static int	oppcas				= 540;
	
	public final static int	SIG_NB_OPERATORS	= oppcas;
}
