/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony;

/**
 * This class contains methods to parse signal textual form to create an abstract syntax tree.
 */
public class SignalParser {
	/**
	 * Creates and initializes the temporary file used by the compiler
	 */
	public synchronized native void init();
	
	public SignalParser() {
		init();
	}
	
	/**
	 * @param filename
	 *        the name of the parameter file to parse (absolute path)
	 * @return a node representing the abstract tree of the parameter file or PKTrees.ERROR_NODE if there is an error.
	 */
	public synchronized native long parseParamfile(String filename);
	
	/**
	 * @param filename
	 *        the name of the SIG file to parse (absolute path)
	 * @return a node representing the abstract tree of the Signal program contained in the file or PKTrees.ERROR_NODE
	 *         if there is an error.
	 */
	public synchronized native long parseSrcfile(String filename);
	
	/**
	 * @param phylum
	 *        syntactic group of the Signal grammar
	 * @param filename
	 *        the name of the file to parse (absolute path)
	 * @return a node representing the abstract tree of the Signal expression contained in the file or
	 *         PKTrees.ERROR_NODE if there is an error.
	 * 
	 * @see fr.irisa.espresso.sme.polychrony.Phyla
	 */
	public synchronized native long sysParser(int phylum, String filename);
	
	/**
	 * It returns the abstract tree corresponding to the parsing of a text. This text is assumed to be an element of a
	 * syntactic group characterized by its phylum (aPhylum).
	 * 
	 * @param phylum
	 *        syntactic group of the Signal grammar
	 * @param expression
	 *        non-null Signal expression
	 * @return a node representing the abstract tree of the expression or PKTrees.ERROR_NODE if there is an error.
	 * 
	 * @see fr.irisa.espresso.sme.polychrony.Phyla
	 */
	public synchronized native long parseString(int phylum, String expression);
}
