/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony;

/**
 * This file contains all SignalTreeManager functions concerning the management of comments. Conversions of type between
 * C and Java
 */
public class SignalTreeComment {
	public SignalTreeComment() {};
	
	/**
	 * Add a commentTree to a node of the AST
	 * 
	 * @param node
	 *        the address of the node (ntree structure)
	 * @param commentTree
	 *        the address of the comment (ntree structure)
	 */
	public synchronized native void addComment(long node, long vattrtree);
	
	/**
	 * Add a comment to a node of the AST
	 * 
	 * @param node
	 *        the address of the node (ntree structure)
	 * @param comment
	 *        the string to add as comment
	 */
	public synchronized native void addStrComment(long node, String cstr);
	
	/**
	 * Get the comment of a node <code>t</code> of the AST
	 * 
	 * @param node
	 *        the address of the node
	 * @return the adress of the comment (ntree structure) or PKTrees.ERROR_NODE if there is no comment
	 */
	public synchronized native long getComment(long node);
	
	/**
	 * @param t
	 *        the tree
	 * @param l
	 *        the result tree
	 * @return a tree containing all the annotated nodes
	 */
	public synchronized native long getAnnotations(long t, long l);
	
	/**
	 * @param t
	 * @return true if a tree (t) is annotated with an error/warning, "false" otherwise. This annotation when it exists,
	 *         is a list of comments.
	 */
	public synchronized native long getAnnotation(long t);
	
}
