/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony;

public class SignalTreePrinter {
	public SignalTreePrinter() {
		init();
	}
	
	public synchronized native void init();
	
	/**
	 * Print the Signal Program corresponding to the given tree in a file <code> OutputDirectory/model_name.SIG </code>.
	 * Here the tree must represent a model declaration or a module.
	 * 
	 * @param tree
	 *        the tree representing the model
	 * @return true if there is no errors in the code generation, false otherwise.
	 * 
	 * @see org.topcased.sme.polychrony.SignalTreePrinter#setOutputDirectory(String)
	 */
	public synchronized native boolean printTree(long tree);
	
	/**
	 * Print the Signal Program corresponding to the given tree in a specific file. The file will be stored in the
	 * Output directory set through
	 * 
	 * @param node
	 *        the tree
	 * @param fileName
	 *        the name of the file in which the tree is generated
	 * @return true if there is no errors in the code generation, false otherwise.
	 * 
	 * @see org.topcased.sme.polychrony.SignalTreePrinter#setOutputDirectory(String)
	 */
	public synchronized native boolean printTreeFile(long node, String fileName);
	
	/**
	 * Set the Output directory in which the generated file will be stored.
	 * 
	 * @param directory
	 *        the absolute path to the directory
	 * @return true if the directory is available, false otherwise
	 */
	public synchronized native boolean setOutputDirectory(String directory);
}
