/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony.traceability;

import org.eclipse.emf.ecore.impl.EObjectImpl;

/**
 * Class used in the Traceability API to represent a SME Node, with the EMF node, a boolean indicating if there is an
 * error and the error message, if there is any.
 */
public class SMENode {
	/** The EMF Node */
	private EObjectImpl	node;
	
	/** Boolean set to true if there is an error */
	private boolean		error;
	
	/** The error message, if there is any */
	private String		errorMessage;
	
	/**
	 * Constructor
	 */
	public SMENode() {
		setNode(null);
		setError(false);
		setErrorMessage("");
	}
	
	/**
	 * the EMF node getter
	 * 
	 * @return the EMF node
	 */
	public EObjectImpl getNode() {
		return node;
	}
	
	/**
	 * the EMF node setter
	 * 
	 * @param node
	 *        the EMF node to set
	 */
	public void setNode(EObjectImpl node) {
		this.node = node;
	}
	
	/**
	 * Error boolean getter
	 * 
	 * @return true if there is an error, false otherwise.
	 */
	public boolean isError() {
		return error;
	}
	
	/**
	 * Error boolean setter
	 * 
	 * @param error
	 *        the value to set
	 */
	public void setError(boolean error) {
		this.error = error;
	}
	
	/**
	 * Error message getter
	 * 
	 * @return the error message, or "" if there isn't any
	 */
	public String getErrorMessage() {
		return errorMessage;
	}
	
	/**
	 * Error message setter
	 * 
	 * @param errorMessage
	 *        the error message to set
	 */
	public void setErrorMessage(String errorMessage) {
		this.errorMessage = errorMessage;
	}
	
	/**
	 * This method prints the informations of a SME node.
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("Node : ");
		buffer.append(getNode().toString());
		buffer.append("\n");
		buffer.append(" Has Error : ");
		buffer.append(isError());
		buffer.append("\n");
		buffer.append(" Error Message : ");
		buffer.append(getErrorMessage());
		buffer.append("\n\n");
		
		return buffer.toString();
	}
}
