/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony.utils;

import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EValidator;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.edit.ui.util.EditUIMarkerHelper;

import fr.irisa.espresso.sme.polychrony.PkPlugin;

public class PolychronyMarkerHelper extends EditUIMarkerHelper {
	public final static PolychronyMarkerHelper	INSTANCE	= new PolychronyMarkerHelper();
	
	public final static String					ID_MARKER	= EValidator.MARKER;			// "fr.irisa.espresso.sme.polychrony.markers";
																							
	@Override
	protected String getMarkerID() {
		return ID_MARKER;
	}
	
	public void createMarkers(IFile file, Diagnostic diagnostic) {
		try {
			createMarkers(file, diagnostic, null);
		}
		catch (CoreException ce) {
			PkPlugin.getDefault().getLog().log(
				new Status(IStatus.ERROR, PkPlugin.PLUGIN_ID, 0,
					"An exception has been thrown during the creation of a marker problem on " + file, ce));
		}
	}
	
	public void createMarkers(Resource resource, Diagnostic diagnostic) {
		try {
			createMarkers(getFile(resource), diagnostic, null);
		}
		catch (CoreException ce) {
			PkPlugin.getDefault().getLog()
					.log(
						new Status(IStatus.ERROR, PkPlugin.PLUGIN_ID, 0,
							"An exception has been thrown during the creation of a marker problem on "
									+ getFile(resource), ce));
		}
	}
	
	@Override
	protected String composeMessage(Diagnostic diagnostic, Diagnostic parentDiagnostic) {
		String message = diagnostic.getMessage();
		if (parentDiagnostic != null) {
			String parentMessage = parentDiagnostic.getMessage();
			if (parentMessage != null) {
				message = message != null ? parentMessage + ". " + message : parentMessage;
			}
		}
		return message;
	}
	
	@Override
	protected void adjustMarker(IMarker marker, Diagnostic diagnostic, Diagnostic parentDiagnostic)
			throws CoreException {
		List<?> data = diagnostic.getData();
		StringBuilder relatedURIs = new StringBuilder();
		boolean first = true;
		for (Object object : data) {
			if (object instanceof EObject) {
				EObject eObject = (EObject) object;
				if (first) {
					first = false;
					marker.setAttribute(EValidator.URI_ATTRIBUTE, ResourceHelper.getURI(eObject).toString());
				}
				else {
					if (relatedURIs.length() != 0) {
						relatedURIs.append(' ');
					}
					relatedURIs.append(URI.encodeFragment(ResourceHelper.getURI(eObject).toString(), false));
				}
			}
		}
		
		if (relatedURIs.length() > 0) {
			marker.setAttribute(EValidator.RELATED_URIS_ATTRIBUTE, relatedURIs.toString());
		}
		
		super.adjustMarker(marker, diagnostic, parentDiagnostic);
	}
}
