/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.polychrony.utils;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;

public class ResourceHelper {
	/**
	 * The eclipse workspace's root
	 */
	static final public IWorkspaceRoot	root	= ResourcesPlugin.getWorkspace().getRoot();
	
	/**
	 * This method is the one of the EcoreUtil class, but the returned URI is modify if the resource containing the
	 * EObject is present in the workspace
	 * 
	 * @param eObject
	 * @return
	 */
	public static URI getURI(EObject eObject) {
		// If it's a proxy, use that.
		URI proxyURI = ((InternalEObject) eObject).eProxyURI();
		if (proxyURI != null) {
			return proxyURI;
		}
		else {
			// If it is in a resource, form the URI relative to that resource.
			Resource resource = eObject.eResource();
			if (resource != null) {
				URI uri = resource.getURI();
				String path = uri.toString();
				
				if (path.startsWith(root.getLocationURI().toString())) {
					path = path.substring(root.getLocationURI().toString().length());
					uri = URI.createPlatformResourceURI(path, false);
				}
				return uri.appendFragment(resource.getURIFragment(eObject));
			}
			else {
				String id = EcoreUtil.getID(eObject);
				if (id != null) {
					return URI.createURI("#" + id);
				}
				else {
					InternalEObject internalEObject = (InternalEObject) eObject;
					List<String> uriFragmentPath = new ArrayList<String>();
					for (InternalEObject container = internalEObject.eInternalContainer(); container != null; container = internalEObject
							.eInternalContainer()) {
						uriFragmentPath.add(container.eURIFragmentSegment(internalEObject.eContainingFeature(),
							internalEObject));
						internalEObject = container;
					}
					
					StringBuffer result = new StringBuffer("#//");
					
					for (int i = uriFragmentPath.size() - 1; i >= 0; --i) {
						result.append('/');
						result.append(uriFragmentPath.get(i));
					}
					return URI.createURI(result.toString());
				}
			}
		}
	}
}
