/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.provider;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.ResourceLocator;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ItemProviderAdapter;
import org.eclipse.emf.edit.provider.ViewerNotification;

import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.SmeFactory;
import fr.irisa.espresso.sme.SmePackage;

/**
 * This is the item provider adapter for a {@link fr.irisa.espresso.sme.Automaton} object. <!-- begin-user-doc --> <!--
 * end-user-doc -->
 * 
 * @generated
 */
public class AutomatonItemProvider extends ItemProviderAdapter implements IEditingDomainItemProvider,
		IStructuredItemContentProvider, ITreeItemContentProvider, IItemLabelProvider, IItemPropertySource {
	/**
	 * This constructs an instance from a factory and a notifier. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public AutomatonItemProvider(AdapterFactory adapterFactory) {
		super(adapterFactory);
	}
	
	/**
	 * This returns the property descriptors for the adapted class. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object) {
		if (itemPropertyDescriptors == null) {
			super.getPropertyDescriptors(object);
			
			addDst_StrongTransitionsPropertyDescriptor(object);
			addDst_WeakTransitionsPropertyDescriptor(object);
			addNamePropertyDescriptor(object);
			addCommentPropertyDescriptor(object);
			addSrc_WeakTransitionsPropertyDescriptor(object);
			addSrc_StrongTransitionsPropertyDescriptor(object);
			addSrc_ClockRelationsPropertyDescriptor(object);
			addDst_ClockRelationsPropertyDescriptor(object);
			addSrc_DependencesPropertyDescriptor(object);
			addDst_DependencesPropertyDescriptor(object);
			addInitStatePropertyDescriptor(object);
		}
		return itemPropertyDescriptors;
	}
	
	/**
	 * This adds a property descriptor for the Dst Strong Transitions feature. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	protected void addDst_StrongTransitionsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_StateOrHistory_dst_StrongTransitions_feature"),
			getString("_UI_StateOrHistory_dst_StrongTransitions_description"),
			SmePackage.Literals.STATE_OR_HISTORY__DST_STRONG_TRANSITIONS, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Dst Weak Transitions feature. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	protected void addDst_WeakTransitionsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_StateOrHistory_dst_WeakTransitions_feature"),
			getString("_UI_StateOrHistory_dst_WeakTransitions_description"),
			SmePackage.Literals.STATE_OR_HISTORY__DST_WEAK_TRANSITIONS, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Name feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addNamePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_NamedElement_name_feature"),
			getString("_UI_NamedElement_name_description"), SmePackage.Literals.NAMED_ELEMENT__NAME, true, false,
			false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_BasicPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Comment feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addCommentPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_Identifier_comment_feature"), getString(
			"_UI_PropertyDescriptor_description", "_UI_Identifier_comment_feature", "_UI_Identifier_type"),
			SmePackage.Literals.IDENTIFIER__COMMENT, true, true, false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE,
			getString("_UI_BasicPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Src Weak Transitions feature. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	protected void addSrc_WeakTransitionsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_StateBase_src_WeakTransitions_feature"),
			getString("_UI_StateBase_src_WeakTransitions_description"),
			SmePackage.Literals.STATE_BASE__SRC_WEAK_TRANSITIONS, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Src Strong Transitions feature. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	protected void addSrc_StrongTransitionsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_StateBase_src_StrongTransitions_feature"),
			getString("_UI_StateBase_src_StrongTransitions_description"),
			SmePackage.Literals.STATE_BASE__SRC_STRONG_TRANSITIONS, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Src Clock Relations feature. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	protected void addSrc_ClockRelationsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_ClockedExpr_src_ClockRelations_feature"),
			getString("_UI_ClockedExpr_src_ClockRelations_description"),
			SmePackage.Literals.CLOCKED_EXPR__SRC_CLOCK_RELATIONS, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Dst Clock Relations feature. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	protected void addDst_ClockRelationsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_ClockedExpr_dst_ClockRelations_feature"),
			getString("_UI_ClockedExpr_dst_ClockRelations_description"),
			SmePackage.Literals.CLOCKED_EXPR__DST_CLOCK_RELATIONS, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Src Dependences feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addSrc_DependencesPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_DependenceInOut_src_Dependences_feature"),
			getString("_UI_DependenceInOut_src_Dependences_description"),
			SmePackage.Literals.DEPENDENCE_IN_OUT__SRC_DEPENDENCES, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Dst Dependences feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addDst_DependencesPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_DependenceInOut_dst_Dependences_feature"),
			getString("_UI_DependenceInOut_dst_Dependences_description"),
			SmePackage.Literals.DEPENDENCE_IN_OUT__DST_DEPENDENCES, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Init State feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected void addInitStatePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(new ItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_Automaton_initState_feature"),
			getString("_UI_Automaton_initState_description"), SmePackage.Literals.AUTOMATON__INIT_STATE, true, false,
			false, null, getString("_UI_BasicPropertyCategory"), null) {
			protected Collection<?> getComboBoxObjects(Object object) {
				Automaton automat = (Automaton) object;
				return automat.getStates();
			}
		});
	}
	
	/**
	 * This specifies how to implement {@link #getChildren} and is used to deduce an appropriate feature for an
	 * {@link org.eclipse.emf.edit.command.AddCommand}, {@link org.eclipse.emf.edit.command.RemoveCommand} or
	 * {@link org.eclipse.emf.edit.command.MoveCommand} in {@link #createCommand}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Collection<? extends EStructuralFeature> getChildrenFeatures(Object object) {
		if (childrenFeatures == null) {
			super.getChildrenFeatures(object);
			childrenFeatures.add(SmePackage.Literals.COMPOUND_STATE__STATES);
			childrenFeatures.add(SmePackage.Literals.COMPOUND_STATE__HISTORY);
			childrenFeatures.add(SmePackage.Literals.COMPOUND_STATE__LOCALS);
			childrenFeatures.add(SmePackage.Literals.AUTOMATON__TRANSITIONS);
		}
		return childrenFeatures;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EStructuralFeature getChildFeature(Object object, Object child) {
		// Check the type of the specified child object and return the proper feature to use for
		// adding (see {@link AddCommand}) it as a child.
		
		return super.getChildFeature(object, child);
	}
	
	/**
	 * This returns Automaton.gif. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object getImage(Object object) {
		return overlayImage(object, getResourceLocator().getImage("full/obj16/Automaton"));
	}
	
	/**
	 * This returns the label text for the adapted class. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String getText(Object object) {
		String label = ((Automaton) object).getName();
		return label == null || label.length() == 0 ? getString("_UI_Automaton_type") : getString("_UI_Automaton_type")
				+ " " + label;
	}
	
	/**
	 * This handles model notifications by calling {@link #updateChildren} to update any cached children and by creating
	 * a viewer notification, which it passes to {@link #fireNotifyChanged}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public void notifyChanged(Notification notification) {
		updateChildren(notification);
		
		switch (notification.getFeatureID(Automaton.class)) {
			case SmePackage.AUTOMATON__NAME:
			case SmePackage.AUTOMATON__COMMENT:
			case SmePackage.AUTOMATON__INIT_STATE:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), false, true));
				return;
			case SmePackage.AUTOMATON__STATES:
			case SmePackage.AUTOMATON__HISTORY:
			case SmePackage.AUTOMATON__LOCALS:
			case SmePackage.AUTOMATON__TRANSITIONS:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), true, false));
				return;
		}
		super.notifyChanged(notification);
	}
	
	/**
	 * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children that can be created
	 * under this object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object) {
		super.collectNewChildDescriptors(newChildDescriptors, object);
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.COMPOUND_STATE__STATES, SmeFactory.eINSTANCE
				.createState()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.COMPOUND_STATE__STATES, SmeFactory.eINSTANCE
				.createAutomaton()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.COMPOUND_STATE__STATES, SmeFactory.eINSTANCE
				.createAndState()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.COMPOUND_STATE__HISTORY, SmeFactory.eINSTANCE
				.createHistory()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.COMPOUND_STATE__LOCALS, SmeFactory.eINSTANCE
				.createLocal()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.AUTOMATON__TRANSITIONS, SmeFactory.eINSTANCE
				.createWeakTransition()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.AUTOMATON__TRANSITIONS, SmeFactory.eINSTANCE
				.createStrongTransition()));
	}
	
	/**
	 * Return the resource locator for this item provider's resources. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public ResourceLocator getResourceLocator() {
		return SmeEditPlugin.INSTANCE;
	}
	
}
