/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.provider;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.ResourceLocator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ItemProviderAdapter;
import org.eclipse.emf.edit.provider.ViewerNotification;

import fr.irisa.espresso.sme.CartesianProduct;
import fr.irisa.espresso.sme.Constant;
import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SmePackage;

/**
 * This is the item provider adapter for a {@link fr.irisa.espresso.sme.ConstantAffectation} object. <!-- begin-user-doc
 * --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ConstantAffectationItemProvider extends ItemProviderAdapter implements IEditingDomainItemProvider,
		IStructuredItemContentProvider, ITreeItemContentProvider, IItemLabelProvider, IItemPropertySource {
	/**
	 * This constructs an instance from a factory and a notifier. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectationItemProvider(AdapterFactory adapterFactory) {
		super(adapterFactory);
	}
	
	/**
	 * This returns the property descriptors for the adapted class. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object) {
		if (itemPropertyDescriptors == null) {
			super.getPropertyDescriptors(object);
			
			addNamePropertyDescriptor(object);
			addSrcPropertyDescriptor(object);
			addDstPropertyDescriptor(object);
			addSrcFieldPropertyDescriptor(object);
		}
		return itemPropertyDescriptors;
	}
	
	/**
	 * This adds a property descriptor for the Name feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addNamePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_NamedElement_name_feature"),
			getString("_UI_NamedElement_name_description"), SmePackage.Literals.NAMED_ELEMENT__NAME, true, false,
			false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_BasicPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Src feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected void addSrcPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(new ItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ConstantAffectation_src_feature"),
			getString("_UI_ConstantAffectation_src_description"), SmePackage.Literals.CONSTANT_AFFECTATION__SRC, true,
			false, false, null, getString("_UI_SourceTargetPropertyCategory"), null) {
			protected Collection<?> getComboBoxObjects(Object object) {
				SignalContainer parent = ((ConstantAffectation) object).getParent();
				if (parent == null)
					return Collections.EMPTY_LIST;
				
				EList<ConstantExpr> result = new BasicEList<ConstantExpr>();
				for (EObject obj : parent.eContents()) {
					if (obj instanceof ModelInstance) {
						result.addAll(((ModelInstance) obj).getVisibleConstants());
						result.addAll(((ModelInstance) obj).getOutputs());
					}
					else if (obj instanceof CartesianProduct)
						result.addAll(((CartesianProduct) obj).getOuts());
					else if (obj instanceof ConstantExpr)
						result.add((ConstantExpr) obj);
				}
				return result;
			}
		});
	}
	
	/**
	 * This adds a property descriptor for the Dst feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected void addDstPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(new ItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ConstantAffectation_dst_feature"),
			getString("_UI_ConstantAffectation_dst_description"), SmePackage.Literals.CONSTANT_AFFECTATION__DST, true,
			false, false, null, getString("_UI_SourceTargetPropertyCategory"), null) {
			protected Collection<?> getComboBoxObjects(Object object) {
				SignalContainer parent = ((ConstantAffectation) object).getParent();
				if (parent == null)
					return Collections.EMPTY_LIST;
				
				EList<Constant> result = new BasicEList<Constant>();
				for (EObject obj : parent.eContents()) {
					if (obj instanceof ModelInstance)
						result.addAll(((ModelInstance) obj).getParameters());
					else if (obj instanceof Constant)
						result.add((Constant) obj);
				}
				return result;
			}
		});
	}
	
	/**
	 * This adds a property descriptor for the Src Field feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addSrcFieldPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ConstantAffectation_srcField_feature"),
			getString("_UI_ConstantAffectation_srcField_description"),
			SmePackage.Literals.CONSTANT_AFFECTATION__SRC_FIELD, true, false, false,
			ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_SourceTargetPropertyCategory"), null));
	}
	
	/**
	 * This returns ConstantAffectation.gif. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object getImage(Object object) {
		return overlayImage(object, getResourceLocator().getImage("full/obj16/ConstantAffectation"));
	}
	
	/**
	 * This returns the label text for the adapted class. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String getText(Object object) {
		String label = ((ConstantAffectation) object).getName();
		return label == null || label.length() == 0 ? getString("_UI_ConstantAffectation_type")
				: getString("_UI_ConstantAffectation_type") + " " + label;
	}
	
	/**
	 * This handles model notifications by calling {@link #updateChildren} to update any cached children and by creating
	 * a viewer notification, which it passes to {@link #fireNotifyChanged}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public void notifyChanged(Notification notification) {
		updateChildren(notification);
		
		switch (notification.getFeatureID(ConstantAffectation.class)) {
			case SmePackage.CONSTANT_AFFECTATION__NAME:
			case SmePackage.CONSTANT_AFFECTATION__SRC:
			case SmePackage.CONSTANT_AFFECTATION__DST:
			case SmePackage.CONSTANT_AFFECTATION__SRC_FIELD:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), false, true));
				return;
		}
		super.notifyChanged(notification);
	}
	
	/**
	 * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children that can be created
	 * under this object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object) {
		super.collectNewChildDescriptors(newChildDescriptors, object);
	}
	
	/**
	 * Return the resource locator for this item provider's resources. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public ResourceLocator getResourceLocator() {
		return SmeEditPlugin.INSTANCE;
	}
	
}
