/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.provider;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.ResourceLocator;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ItemProviderAdapter;
import org.eclipse.emf.edit.provider.ViewerNotification;

import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.ModelDeclaration;
import fr.irisa.espresso.sme.ModelType;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SmeFactory;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.util.Utils;

/**
 * This is the item provider adapter for a {@link fr.irisa.espresso.sme.ModelDeclaration} object. <!-- begin-user-doc
 * --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ModelDeclarationItemProvider extends ItemProviderAdapter implements IEditingDomainItemProvider,
		IStructuredItemContentProvider, ITreeItemContentProvider, IItemLabelProvider, IItemPropertySource {
	/**
	 * This constructs an instance from a factory and a notifier. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelDeclarationItemProvider(AdapterFactory adapterFactory) {
		super(adapterFactory);
	}
	
	/**
	 * This returns the property descriptors for the adapted class. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object) {
		if (itemPropertyDescriptors == null) {
			super.getPropertyDescriptors(object);
			
			addTypePropertyDescriptor(object);
			addInputsPropertyDescriptor(object);
			addOutputsPropertyDescriptor(object);
			addParametersPropertyDescriptor(object);
			addProcessClassPropertyDescriptor(object);
			addNamePropertyDescriptor(object);
			addCommentPropertyDescriptor(object);
			addSrc_ConstantAffectationsPropertyDescriptor(object);
			addModelTypePropertyDescriptor(object);
			addVisibilityPropertyDescriptor(object);
			addIsMainPropertyDescriptor(object);
			addIsSigaliProcessPropertyDescriptor(object);
			addCompilationDirectivePropertyDescriptor(object);
			addRunOnPropertyDescriptor(object);
			addC_CODEPropertyDescriptor(object);
			addCpp_CODEPropertyDescriptor(object);
			addJava_CODEPropertyDescriptor(object);
			addProcessor_typePropertyDescriptor(object);
		}
		return itemPropertyDescriptors;
	}
	
	/**
	 * This adds a property descriptor for the Type feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected void addTypePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(new ItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_InterfaceDefinition_type_feature"),
			getString("_UI_InterfaceDefinition_type_description"), SmePackage.Literals.INTERFACE_DEFINITION__TYPE,
			true, false, true, null, getString("_UI_ExpertPropertyCategory"), null) {
			protected Collection<ModelType> getComboBoxObjects(Object object) {
				// We put all intrinsic type in the list
				List<ModelType> sortedList = new LinkedList<ModelType>();
				
				// And we add all type declaration which are in the scope of the input signal.
				SignalContainer parent = ((ModelDeclaration) object).getParent();
				if (parent != null) {
					EList<Identifier> vt = parent.getVisibleTypes();
					for (Identifier id : vt) {
						if (id instanceof ModelType)
							sortedList.add((ModelType) id);
					}
				}
				return sortedList;
			}
		});
	}
	
	/**
	 * This adds a property descriptor for the Inputs feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addInputsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_InterfaceDefinition_inputs_feature"),
			getString("_UI_InterfaceDefinition_inputs_description"), SmePackage.Literals.INTERFACE_DEFINITION__INPUTS,
			false, false, false, null, getString("_UI_ExpertPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Outputs feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addOutputsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_InterfaceDefinition_outputs_feature"),
			getString("_UI_InterfaceDefinition_outputs_description"),
			SmePackage.Literals.INTERFACE_DEFINITION__OUTPUTS, false, false, false, null,
			getString("_UI_ExpertPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Parameters feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addParametersPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_InterfaceDefinition_parameters_feature"),
			getString("_UI_InterfaceDefinition_parameters_description"),
			SmePackage.Literals.INTERFACE_DEFINITION__PARAMETERS, false, false, false, null,
			getString("_UI_ExpertPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Process Class feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addProcessClassPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_InterfaceDefinition_processClass_feature"),
			getString("_UI_InterfaceDefinition_processClass_description"),
			SmePackage.Literals.INTERFACE_DEFINITION__PROCESS_CLASS, true, false, false,
			ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_ExpertPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Name feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addNamePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_NamedElement_name_feature"),
			getString("_UI_NamedElement_name_description"), SmePackage.Literals.NAMED_ELEMENT__NAME, true, false,
			false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_BasicPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Comment feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addCommentPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_Identifier_comment_feature"), getString(
			"_UI_PropertyDescriptor_description", "_UI_Identifier_comment_feature", "_UI_Identifier_type"),
			SmePackage.Literals.IDENTIFIER__COMMENT, true, true, false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE,
			getString("_UI_BasicPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Src Constant Affectations feature. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addSrc_ConstantAffectationsPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_ConstantExpr_src_ConstantAffectations_feature"),
			getString("_UI_ConstantExpr_src_ConstantAffectations_description"),
			SmePackage.Literals.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS, false, false, false, null,
			getString("_UI_ConnectedlinksPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Model Type feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addModelTypePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ModelDeclaration_modelType_feature"),
			getString("_UI_ModelDeclaration_modelType_description"), SmePackage.Literals.MODEL_DECLARATION__MODEL_TYPE,
			true, false, false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_ExpertPropertyCategory"),
			null));
	}
	
	/**
	 * This adds a property descriptor for the Visibility feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addVisibilityPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ModelDeclaration_visibility_feature"),
			getString("_UI_ModelDeclaration_visibility_description"),
			SmePackage.Literals.MODEL_DECLARATION__VISIBILITY, true, false, false,
			ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_BasicPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Is Main feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addIsMainPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ModelDeclaration_isMain_feature"),
			getString("_UI_ModelDeclaration_isMain_description"), SmePackage.Literals.MODEL_DECLARATION__IS_MAIN, true,
			false, false, ItemPropertyDescriptor.BOOLEAN_VALUE_IMAGE, getString("_UI_PragmasPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Is Sigali Process feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addIsSigaliProcessPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_ModelDeclaration_isSigaliProcess_feature"),
			getString("_UI_ModelDeclaration_isSigaliProcess_description"),
			SmePackage.Literals.MODEL_DECLARATION__IS_SIGALI_PROCESS, true, false, false,
			ItemPropertyDescriptor.BOOLEAN_VALUE_IMAGE, getString("_UI_PragmasPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Compilation Directive feature. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	protected void addCompilationDirectivePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_ModelDeclaration_compilationDirective_feature"),
			getString("_UI_ModelDeclaration_compilationDirective_description"),
			SmePackage.Literals.MODEL_DECLARATION__COMPILATION_DIRECTIVE, true, false, false,
			ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_PragmasPropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Run On feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addRunOnPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ModelDeclaration_runOn_feature"),
			getString("_UI_PropertyDescriptor_description", "_UI_ModelDeclaration_runOn_feature",
				"_UI_ModelDeclaration_type"), SmePackage.Literals.MODEL_DECLARATION__RUN_ON, true, false, false,
			ItemPropertyDescriptor.INTEGRAL_VALUE_IMAGE, null, null));
	}
	
	/**
	 * This adds a property descriptor for the CCODE feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addC_CODEPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ModelDeclaration_c_CODE_feature"),
			getString("_UI_ModelDeclaration_c_CODE_description"), SmePackage.Literals.MODEL_DECLARATION__CCODE, true,
			true, false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_ExternalcodePropertyCategory"),
			null));
	}
	
	/**
	 * This adds a property descriptor for the Cpp CODE feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addCpp_CODEPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ModelDeclaration_cpp_CODE_feature"),
			getString("_UI_ModelDeclaration_cpp_CODE_description"), SmePackage.Literals.MODEL_DECLARATION__CPP_CODE,
			true, true, false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE,
			getString("_UI_ExternalcodePropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Java CODE feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addJava_CODEPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(), getString("_UI_ModelDeclaration_java_CODE_feature"),
			getString("_UI_ModelDeclaration_java_CODE_description"), SmePackage.Literals.MODEL_DECLARATION__JAVA_CODE,
			true, true, false, ItemPropertyDescriptor.GENERIC_VALUE_IMAGE,
			getString("_UI_ExternalcodePropertyCategory"), null));
	}
	
	/**
	 * This adds a property descriptor for the Processor type feature. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected void addProcessor_typePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add(createItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory)
				.getRootAdapterFactory(), getResourceLocator(),
			getString("_UI_ModelDeclaration_processor_type_feature"),
			getString("_UI_ModelDeclaration_processor_type_description"),
			SmePackage.Literals.MODEL_DECLARATION__PROCESSOR_TYPE, true, false, false,
			ItemPropertyDescriptor.GENERIC_VALUE_IMAGE, getString("_UI_PragmasPropertyCategory"), null));
	}
	
	/**
	 * This specifies how to implement {@link #getChildren} and is used to deduce an appropriate feature for an
	 * {@link org.eclipse.emf.edit.command.AddCommand}, {@link org.eclipse.emf.edit.command.RemoveCommand} or
	 * {@link org.eclipse.emf.edit.command.MoveCommand} in {@link #createCommand}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Collection<? extends EStructuralFeature> getChildrenFeatures(Object object) {
		if (childrenFeatures == null) {
			super.getChildrenFeatures(object);
			childrenFeatures.add(SmePackage.Literals.MODEL__ASSERTIONS);
			childrenFeatures.add(SmePackage.Literals.MODEL__AUTOMATA);
			childrenFeatures.add(SmePackage.Literals.MODEL__CONNECTIONS);
			childrenFeatures.add(SmePackage.Literals.MODEL__CONSTANT_REF);
			childrenFeatures.add(SmePackage.Literals.MODEL__CONSTANTS);
			childrenFeatures.add(SmePackage.Literals.MODEL__CONSTRAINTS);
			childrenFeatures.add(SmePackage.Literals.MODEL__INDEXES);
			childrenFeatures.add(SmePackage.Literals.MODEL__CONTAINED_INSTANCES);
			childrenFeatures.add(SmePackage.Literals.MODEL__LITERALS);
			childrenFeatures.add(SmePackage.Literals.MODEL__LOCALS);
			childrenFeatures.add(SmePackage.Literals.MODEL__OPERATORS);
			childrenFeatures.add(SmePackage.Literals.MODEL__SIGNAL_REF);
			childrenFeatures.add(SmePackage.Literals.MODEL__SUBPROCESSES);
			childrenFeatures.add(SmePackage.Literals.MODEL__TYPES);
			childrenFeatures.add(SmePackage.Literals.MODEL__USE_MODULE);
			childrenFeatures.add(SmePackage.Literals.MODEL__MODEL_REFS);
			childrenFeatures.add(SmePackage.Literals.INTERFACE_DEFINITION__INPUTS);
			childrenFeatures.add(SmePackage.Literals.INTERFACE_DEFINITION__OUTPUTS);
			childrenFeatures.add(SmePackage.Literals.INTERFACE_DEFINITION__PARAMETERS);
			childrenFeatures.add(SmePackage.Literals.INTERFACE_DEFINITION__SPEC);
			childrenFeatures.add(SmePackage.Literals.INTERFACE_DEFINITION__PRAGMAS);
			childrenFeatures.add(SmePackage.Literals.MODEL_DECLARATION__MODEL_DECLARATIONS);
		}
		return childrenFeatures;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EStructuralFeature getChildFeature(Object object, Object child) {
		// Check the type of the specified child object and return the proper feature to use for
		// adding (see {@link AddCommand}) it as a child.
		
		return super.getChildFeature(object, child);
	}
	
	/**
	 * This returns ModelDeclaration.gif. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object getImage(Object object) {
		return overlayImage(object, getResourceLocator().getImage("full/obj16/ModelDeclaration"));
	}
	
	/**
	 * This returns the label text for the adapted class. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String getText(Object object) {
		String label = ((ModelDeclaration) object).getName();
		return label == null || label.length() == 0 ? getString("_UI_ModelDeclaration_type")
				: getString("_UI_ModelDeclaration_type") + " " + label;
	}
	
	/**
	 * This handles model notifications by calling {@link #updateChildren} to update any cached children and by creating
	 * a viewer notification, which it passes to {@link #fireNotifyChanged}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public void notifyChanged(Notification notification) {
		updateChildren(notification);
		
		switch (notification.getFeatureID(ModelDeclaration.class)) {
			case SmePackage.MODEL_DECLARATION__TYPE:
			case SmePackage.MODEL_DECLARATION__PROCESS_CLASS:
			case SmePackage.MODEL_DECLARATION__NAME:
			case SmePackage.MODEL_DECLARATION__COMMENT:
			case SmePackage.MODEL_DECLARATION__MODEL_TYPE:
			case SmePackage.MODEL_DECLARATION__VISIBILITY:
			case SmePackage.MODEL_DECLARATION__IS_MAIN:
			case SmePackage.MODEL_DECLARATION__IS_SIGALI_PROCESS:
			case SmePackage.MODEL_DECLARATION__COMPILATION_DIRECTIVE:
			case SmePackage.MODEL_DECLARATION__RUN_ON:
			case SmePackage.MODEL_DECLARATION__CCODE:
			case SmePackage.MODEL_DECLARATION__CPP_CODE:
			case SmePackage.MODEL_DECLARATION__JAVA_CODE:
			case SmePackage.MODEL_DECLARATION__PROCESSOR_TYPE:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), false, true));
				return;
			case SmePackage.MODEL_DECLARATION__ASSERTIONS:
			case SmePackage.MODEL_DECLARATION__AUTOMATA:
			case SmePackage.MODEL_DECLARATION__CONNECTIONS:
			case SmePackage.MODEL_DECLARATION__CONSTANT_REF:
			case SmePackage.MODEL_DECLARATION__CONSTANTS:
			case SmePackage.MODEL_DECLARATION__CONSTRAINTS:
			case SmePackage.MODEL_DECLARATION__INDEXES:
			case SmePackage.MODEL_DECLARATION__CONTAINED_INSTANCES:
			case SmePackage.MODEL_DECLARATION__LITERALS:
			case SmePackage.MODEL_DECLARATION__LOCALS:
			case SmePackage.MODEL_DECLARATION__OPERATORS:
			case SmePackage.MODEL_DECLARATION__SIGNAL_REF:
			case SmePackage.MODEL_DECLARATION__SUBPROCESSES:
			case SmePackage.MODEL_DECLARATION__TYPES:
			case SmePackage.MODEL_DECLARATION__USE_MODULE:
			case SmePackage.MODEL_DECLARATION__MODEL_REFS:
			case SmePackage.MODEL_DECLARATION__INPUTS:
			case SmePackage.MODEL_DECLARATION__OUTPUTS:
			case SmePackage.MODEL_DECLARATION__PARAMETERS:
			case SmePackage.MODEL_DECLARATION__SPEC:
			case SmePackage.MODEL_DECLARATION__PRAGMAS:
			case SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), true, false));
				return;
		}
		super.notifyChanged(notification);
	}
	
	/**
	 * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children that can be created
	 * under this object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	@Override
	protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object) {
		super.collectNewChildDescriptors(newChildDescriptors, object);
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createAnd())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createArithmeticOperator())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createArrayEnumeration())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createArrayProduct())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createArrayRecovery())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createArrayRestructuration())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__ASSERTIONS, SmeFactory.eINSTANCE
				.createAssertion()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTRAINTS, SmeFactory.eINSTANCE
				.createAssertionClockConstraintOperator()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTRAINTS, Utils
				.createPorts(SmeFactory.eINSTANCE.createAssertionClockSpeedOperator())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__AUTOMATA, SmeFactory.eINSTANCE
				.createAutomaton()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__SUBPROCESSES, SmeFactory.eINSTANCE
				.createBasicIterate()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, SmeFactory.eINSTANCE
				.createBooleanExpression()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONNECTIONS, SmeFactory.eINSTANCE
				.createCaseConnection()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createCartesianProduct())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createCell())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTRAINTS, SmeFactory.eINSTANCE
				.createClockConstraintOperator()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONNECTIONS, SmeFactory.eINSTANCE
				.createClockRelation()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTRAINTS, Utils
				.createPorts(SmeFactory.eINSTANCE.createClockRelationOperator())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTRAINTS, Utils
				.createPorts(SmeFactory.eINSTANCE.createClockSpeedOperator())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createComparisonOperator())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createComplexOperator())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createConcatenation())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONNECTIONS, SmeFactory.eINSTANCE
				.createCondition()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONNECTIONS, SmeFactory.eINSTANCE
				.createConstantAffectation()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTANT_REF, SmeFactory.eINSTANCE
				.createConstantRef()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTANTS, SmeFactory.eINSTANCE
				.createConstantValue()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createCounter())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONNECTIONS, SmeFactory.eINSTANCE
				.createDataflowConnection()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createDelay())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONNECTIONS, SmeFactory.eINSTANCE
				.createDependence()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTRAINTS, SmeFactory.eINSTANCE
				.createDependenceOperator()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__TYPES, SmeFactory.eINSTANCE
				.createEnumerationType()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__LITERALS, SmeFactory.eINSTANCE
				.createEnumerationValue()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createExtraction())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createIfThenElse())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__INDEXES, SmeFactory.eINSTANCE
				.createIndex()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.INTERFACE_DEFINITION__INPUTS,
			SmeFactory.eINSTANCE.createInput()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__SUBPROCESSES, Utils
				.createPorts(SmeFactory.eINSTANCE.createIterate())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__LITERALS, SmeFactory.eINSTANCE
				.createLiteral()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__LOCALS, SmeFactory.eINSTANCE
				.createLocal()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createMerge())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL_DECLARATION__MODEL_DECLARATIONS,
			SmeFactory.eINSTANCE.createModelDeclaration()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONTAINED_INSTANCES,
			SmeFactory.eINSTANCE.createModelInstance()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__MODEL_REFS, SmeFactory.eINSTANCE
				.createModelRef()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__TYPES, SmeFactory.eINSTANCE
				.createModelType()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__CONSTRAINTS, SmeFactory.eINSTANCE
				.createNullClock()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createNot())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, SmeFactory.eINSTANCE
				.createNumericExpression()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createOr())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.INTERFACE_DEFINITION__OUTPUTS,
			SmeFactory.eINSTANCE.createOutput()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.INTERFACE_DEFINITION__PARAMETERS,
			SmeFactory.eINSTANCE.createParameter()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.INTERFACE_DEFINITION__PRAGMAS,
			SmeFactory.eINSTANCE.createPragma()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createRepetitionOperator())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createSequentialDefinition())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__SIGNAL_REF, SmeFactory.eINSTANCE
				.createSignalRef()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__SUBPROCESSES, SmeFactory.eINSTANCE
				.createSubProcess()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__TYPES, SmeFactory.eINSTANCE
				.createSubstitutionType()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, SmeFactory.eINSTANCE
				.createSwitch()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createTransposition())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createTupleOperator())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__TYPES, SmeFactory.eINSTANCE
				.createTupleType()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createUnaryMinus())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__USE_MODULE, SmeFactory.eINSTANCE
				.createUseModule()));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createVariable())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createWindow())));
		
		newChildDescriptors.add(createChildParameter(SmePackage.Literals.MODEL__OPERATORS, Utils
				.createPorts(SmeFactory.eINSTANCE.createXor())));
		
	}
	
	/**
	 * Return the resource locator for this item provider's resources. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public ResourceLocator getResourceLocator() {
		return SmeEditPlugin.INSTANCE;
	}
	
}
