/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.provider;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.edit.provider.ChangeNotifier;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.IChangeNotifier;
import org.eclipse.emf.edit.provider.IDisposable;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.INotifyChangedListener;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;

import fr.irisa.espresso.sme.util.SmeAdapterFactory;

/**
 * This is the factory that is used to provide the interfaces needed to support Viewers. The adapters generated by this
 * factory convert EMF adapter notifications into calls to {@link #fireNotifyChanged fireNotifyChanged}. The adapters
 * also support Eclipse property sheets. Note that most of the adapters are shared among multiple instances. <!--
 * begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class SmeItemProviderAdapterFactory extends SmeAdapterFactory implements ComposeableAdapterFactory,
		IChangeNotifier, IDisposable {
	/**
	 * This keeps track of the root adapter factory that delegates to this adapter factory. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	protected ComposedAdapterFactory	parentAdapterFactory;
	
	/**
	 * This is used to implement {@link org.eclipse.emf.edit.provider.IChangeNotifier}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	protected IChangeNotifier			changeNotifier	= new ChangeNotifier();
	
	/**
	 * This keeps track of all the supported types checked by {@link #isFactoryForType isFactoryForType}. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected Collection<Object>		supportedTypes	= new ArrayList<Object>();
	
	/**
	 * This constructs an instance. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SmeItemProviderAdapterFactory() {
		supportedTypes.add(IEditingDomainItemProvider.class);
		supportedTypes.add(IStructuredItemContentProvider.class);
		supportedTypes.add(ITreeItemContentProvider.class);
		supportedTypes.add(IItemLabelProvider.class);
		supportedTypes.add(IItemPropertySource.class);
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.BasicIterate} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected BasicIterateItemProvider	basicIterateItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.BasicIterate}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createBasicIterateAdapter() {
		if (basicIterateItemProvider == null) {
			basicIterateItemProvider = new BasicIterateItemProvider(this);
		}
		
		return basicIterateItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Iterate} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected IterateItemProvider	iterateItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Iterate}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createIterateAdapter() {
		if (iterateItemProvider == null) {
			iterateItemProvider = new IterateItemProvider(this);
		}
		
		return iterateItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.IterationInit} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected IterationInitItemProvider	iterationInitItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.IterationInit}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createIterationInitAdapter() {
		if (iterationInitItemProvider == null) {
			iterationInitItemProvider = new IterationInitItemProvider(this);
		}
		
		return iterationInitItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Specifications} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SpecificationsItemProvider	specificationsItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Specifications}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createSpecificationsAdapter() {
		if (specificationsItemProvider == null) {
			specificationsItemProvider = new SpecificationsItemProvider(this);
		}
		
		return specificationsItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.SubProcess} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SubProcessItemProvider	subProcessItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.SubProcess}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createSubProcessAdapter() {
		if (subProcessItemProvider == null) {
			subProcessItemProvider = new SubProcessItemProvider(this);
		}
		
		return subProcessItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.EnumerationType} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected EnumerationTypeItemProvider	enumerationTypeItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.EnumerationType}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createEnumerationTypeAdapter() {
		if (enumerationTypeItemProvider == null) {
			enumerationTypeItemProvider = new EnumerationTypeItemProvider(this);
		}
		
		return enumerationTypeItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ModelType} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModelTypeItemProvider	modelTypeItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ModelType}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createModelTypeAdapter() {
		if (modelTypeItemProvider == null) {
			modelTypeItemProvider = new ModelTypeItemProvider(this);
		}
		
		return modelTypeItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.SubstitutionType} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SubstitutionTypeItemProvider	substitutionTypeItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.SubstitutionType}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createSubstitutionTypeAdapter() {
		if (substitutionTypeItemProvider == null) {
			substitutionTypeItemProvider = new SubstitutionTypeItemProvider(this);
		}
		
		return substitutionTypeItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.TupleType} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected TupleTypeItemProvider	tupleTypeItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.TupleType}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createTupleTypeAdapter() {
		if (tupleTypeItemProvider == null) {
			tupleTypeItemProvider = new TupleTypeItemProvider(this);
		}
		
		return tupleTypeItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ModelDeclaration} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModelDeclarationItemProvider	modelDeclarationItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ModelDeclaration}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createModelDeclarationAdapter() {
		if (modelDeclarationItemProvider == null) {
			modelDeclarationItemProvider = new ModelDeclarationItemProvider(this);
		}
		
		return modelDeclarationItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ModelRef} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModelRefItemProvider	modelRefItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ModelRef}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createModelRefAdapter() {
		if (modelRefItemProvider == null) {
			modelRefItemProvider = new ModelRefItemProvider(this);
		}
		
		return modelRefItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ModelInstance} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModelInstanceItemProvider	modelInstanceItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ModelInstance}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createModelInstanceAdapter() {
		if (modelInstanceItemProvider == null) {
			modelInstanceItemProvider = new ModelInstanceItemProvider(this);
		}
		
		return modelInstanceItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Module} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModuleItemProvider	moduleItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Module}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createModuleAdapter() {
		if (moduleItemProvider == null) {
			moduleItemProvider = new ModuleItemProvider(this);
		}
		
		return moduleItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.UseModule} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected UseModuleItemProvider	useModuleItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.UseModule}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createUseModuleAdapter() {
		if (useModuleItemProvider == null) {
			useModuleItemProvider = new UseModuleItemProvider(this);
		}
		
		return useModuleItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ConstantRef} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ConstantRefItemProvider	constantRefItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ConstantRef}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createConstantRefAdapter() {
		if (constantRefItemProvider == null) {
			constantRefItemProvider = new ConstantRefItemProvider(this);
		}
		
		return constantRefItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ConstantValue} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ConstantValueItemProvider	constantValueItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ConstantValue}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createConstantValueAdapter() {
		if (constantValueItemProvider == null) {
			constantValueItemProvider = new ConstantValueItemProvider(this);
		}
		
		return constantValueItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Literal} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected LiteralItemProvider	literalItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Literal}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createLiteralAdapter() {
		if (literalItemProvider == null) {
			literalItemProvider = new LiteralItemProvider(this);
		}
		
		return literalItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.EnumerationValue} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected EnumerationValueItemProvider	enumerationValueItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.EnumerationValue}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createEnumerationValueAdapter() {
		if (enumerationValueItemProvider == null) {
			enumerationValueItemProvider = new EnumerationValueItemProvider(this);
		}
		
		return enumerationValueItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Parameter} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ParameterItemProvider	parameterItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Parameter}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createParameterAdapter() {
		if (parameterItemProvider == null) {
			parameterItemProvider = new ParameterItemProvider(this);
		}
		
		return parameterItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ParameterInstance} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ParameterInstanceItemProvider	parameterInstanceItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ParameterInstance}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createParameterInstanceAdapter() {
		if (parameterInstanceItemProvider == null) {
			parameterInstanceItemProvider = new ParameterInstanceItemProvider(this);
		}
		
		return parameterInstanceItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Iterator} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected IteratorItemProvider	iteratorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Iterator}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createIteratorAdapter() {
		if (iteratorItemProvider == null) {
			iteratorItemProvider = new IteratorItemProvider(this);
		}
		
		return iteratorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Index} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected IndexItemProvider	indexItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Index}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createIndexAdapter() {
		if (indexItemProvider == null) {
			indexItemProvider = new IndexItemProvider(this);
		}
		
		return indexItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.SignalRef} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SignalRefItemProvider	signalRefItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.SignalRef}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createSignalRefAdapter() {
		if (signalRefItemProvider == null) {
			signalRefItemProvider = new SignalRefItemProvider(this);
		}
		
		return signalRefItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Input} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected InputItemProvider	inputItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Input}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createInputAdapter() {
		if (inputItemProvider == null) {
			inputItemProvider = new InputItemProvider(this);
		}
		
		return inputItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.InputInstance} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected InputInstanceItemProvider	inputInstanceItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.InputInstance}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createInputInstanceAdapter() {
		if (inputInstanceItemProvider == null) {
			inputInstanceItemProvider = new InputInstanceItemProvider(this);
		}
		
		return inputInstanceItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Output} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected OutputItemProvider	outputItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Output}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createOutputAdapter() {
		if (outputItemProvider == null) {
			outputItemProvider = new OutputItemProvider(this);
		}
		
		return outputItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.OutputInstance} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected OutputInstanceItemProvider	outputInstanceItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.OutputInstance}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createOutputInstanceAdapter() {
		if (outputInstanceItemProvider == null) {
			outputInstanceItemProvider = new OutputInstanceItemProvider(this);
		}
		
		return outputInstanceItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Local} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected LocalItemProvider	localItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Local}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createLocalAdapter() {
		if (localItemProvider == null) {
			localItemProvider = new LocalItemProvider(this);
		}
		
		return localItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.CaseConnection} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected CaseConnectionItemProvider	caseConnectionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.CaseConnection}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createCaseConnectionAdapter() {
		if (caseConnectionItemProvider == null) {
			caseConnectionItemProvider = new CaseConnectionItemProvider(this);
		}
		
		return caseConnectionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ClockRelation} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ClockRelationItemProvider	clockRelationItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ClockRelation}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createClockRelationAdapter() {
		if (clockRelationItemProvider == null) {
			clockRelationItemProvider = new ClockRelationItemProvider(this);
		}
		
		return clockRelationItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Condition} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ConditionItemProvider	conditionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Condition}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createConditionAdapter() {
		if (conditionItemProvider == null) {
			conditionItemProvider = new ConditionItemProvider(this);
		}
		
		return conditionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ConstantAffectation} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ConstantAffectationItemProvider	constantAffectationItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ConstantAffectation}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createConstantAffectationAdapter() {
		if (constantAffectationItemProvider == null) {
			constantAffectationItemProvider = new ConstantAffectationItemProvider(this);
		}
		
		return constantAffectationItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Dependence} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected DependenceItemProvider	dependenceItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Dependence}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createDependenceAdapter() {
		if (dependenceItemProvider == null) {
			dependenceItemProvider = new DependenceItemProvider(this);
		}
		
		return dependenceItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.DataflowConnection} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected DataflowConnectionItemProvider	dataflowConnectionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.DataflowConnection}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createDataflowConnectionAdapter() {
		if (dataflowConnectionItemProvider == null) {
			dataflowConnectionItemProvider = new DataflowConnectionItemProvider(this);
		}
		
		return dataflowConnectionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.InputPort} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected InputPortItemProvider	inputPortItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.InputPort}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createInputPortAdapter() {
		if (inputPortItemProvider == null) {
			inputPortItemProvider = new InputPortItemProvider(this);
		}
		
		return inputPortItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.OutputPort} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected OutputPortItemProvider	outputPortItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.OutputPort}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createOutputPortAdapter() {
		if (outputPortItemProvider == null) {
			outputPortItemProvider = new OutputPortItemProvider(this);
		}
		
		return outputPortItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.And} instances. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AndItemProvider	andItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.And}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createAndAdapter() {
		if (andItemProvider == null) {
			andItemProvider = new AndItemProvider(this);
		}
		
		return andItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ArithmeticOperator} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ArithmeticOperatorItemProvider	arithmeticOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ArithmeticOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createArithmeticOperatorAdapter() {
		if (arithmeticOperatorItemProvider == null) {
			arithmeticOperatorItemProvider = new ArithmeticOperatorItemProvider(this);
		}
		
		return arithmeticOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ArrayEnumeration} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ArrayEnumerationItemProvider	arrayEnumerationItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ArrayEnumeration}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createArrayEnumerationAdapter() {
		if (arrayEnumerationItemProvider == null) {
			arrayEnumerationItemProvider = new ArrayEnumerationItemProvider(this);
		}
		
		return arrayEnumerationItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ArrayProduct} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ArrayProductItemProvider	arrayProductItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ArrayProduct}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createArrayProductAdapter() {
		if (arrayProductItemProvider == null) {
			arrayProductItemProvider = new ArrayProductItemProvider(this);
		}
		
		return arrayProductItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ArrayRecovery} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ArrayRecoveryItemProvider	arrayRecoveryItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ArrayRecovery}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createArrayRecoveryAdapter() {
		if (arrayRecoveryItemProvider == null) {
			arrayRecoveryItemProvider = new ArrayRecoveryItemProvider(this);
		}
		
		return arrayRecoveryItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ArrayRestructuration} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ArrayRestructurationItemProvider	arrayRestructurationItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ArrayRestructuration}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createArrayRestructurationAdapter() {
		if (arrayRestructurationItemProvider == null) {
			arrayRestructurationItemProvider = new ArrayRestructurationItemProvider(this);
		}
		
		return arrayRestructurationItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Assertion} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AssertionItemProvider	assertionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Assertion}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createAssertionAdapter() {
		if (assertionItemProvider == null) {
			assertionItemProvider = new AssertionItemProvider(this);
		}
		
		return assertionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.AssertionClockConstraintOperator}
	 * instances. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AssertionClockConstraintOperatorItemProvider	assertionClockConstraintOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.AssertionClockConstraintOperator}. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createAssertionClockConstraintOperatorAdapter() {
		if (assertionClockConstraintOperatorItemProvider == null) {
			assertionClockConstraintOperatorItemProvider = new AssertionClockConstraintOperatorItemProvider(this);
		}
		
		return assertionClockConstraintOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.AssertionClockSpeedOperator}
	 * instances. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AssertionClockSpeedOperatorItemProvider	assertionClockSpeedOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.AssertionClockSpeedOperator}. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createAssertionClockSpeedOperatorAdapter() {
		if (assertionClockSpeedOperatorItemProvider == null) {
			assertionClockSpeedOperatorItemProvider = new AssertionClockSpeedOperatorItemProvider(this);
		}
		
		return assertionClockSpeedOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.BooleanExpression} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected BooleanExpressionItemProvider	booleanExpressionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.BooleanExpression}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createBooleanExpressionAdapter() {
		if (booleanExpressionItemProvider == null) {
			booleanExpressionItemProvider = new BooleanExpressionItemProvider(this);
		}
		
		return booleanExpressionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.CartesianProduct} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected CartesianProductItemProvider	cartesianProductItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.CartesianProduct}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createCartesianProductAdapter() {
		if (cartesianProductItemProvider == null) {
			cartesianProductItemProvider = new CartesianProductItemProvider(this);
		}
		
		return cartesianProductItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Cell} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected CellItemProvider	cellItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Cell}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createCellAdapter() {
		if (cellItemProvider == null) {
			cellItemProvider = new CellItemProvider(this);
		}
		
		return cellItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ClockConstraintOperator} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ClockConstraintOperatorItemProvider	clockConstraintOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ClockConstraintOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createClockConstraintOperatorAdapter() {
		if (clockConstraintOperatorItemProvider == null) {
			clockConstraintOperatorItemProvider = new ClockConstraintOperatorItemProvider(this);
		}
		
		return clockConstraintOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ClockRelationOperator} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ClockRelationOperatorItemProvider	clockRelationOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ClockRelationOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createClockRelationOperatorAdapter() {
		if (clockRelationOperatorItemProvider == null) {
			clockRelationOperatorItemProvider = new ClockRelationOperatorItemProvider(this);
		}
		
		return clockRelationOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ClockSpeedOperator} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ClockSpeedOperatorItemProvider	clockSpeedOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ClockSpeedOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createClockSpeedOperatorAdapter() {
		if (clockSpeedOperatorItemProvider == null) {
			clockSpeedOperatorItemProvider = new ClockSpeedOperatorItemProvider(this);
		}
		
		return clockSpeedOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ComparisonOperator} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ComparisonOperatorItemProvider	comparisonOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ComparisonOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createComparisonOperatorAdapter() {
		if (comparisonOperatorItemProvider == null) {
			comparisonOperatorItemProvider = new ComparisonOperatorItemProvider(this);
		}
		
		return comparisonOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.ComplexOperator} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ComplexOperatorItemProvider	complexOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.ComplexOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createComplexOperatorAdapter() {
		if (complexOperatorItemProvider == null) {
			complexOperatorItemProvider = new ComplexOperatorItemProvider(this);
		}
		
		return complexOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Concatenation} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ConcatenationItemProvider	concatenationItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Concatenation}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createConcatenationAdapter() {
		if (concatenationItemProvider == null) {
			concatenationItemProvider = new ConcatenationItemProvider(this);
		}
		
		return concatenationItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Counter} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected CounterItemProvider	counterItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Counter}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createCounterAdapter() {
		if (counterItemProvider == null) {
			counterItemProvider = new CounterItemProvider(this);
		}
		
		return counterItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Delay} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected DelayItemProvider	delayItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Delay}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createDelayAdapter() {
		if (delayItemProvider == null) {
			delayItemProvider = new DelayItemProvider(this);
		}
		
		return delayItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.DependenceOperator} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected DependenceOperatorItemProvider	dependenceOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.DependenceOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createDependenceOperatorAdapter() {
		if (dependenceOperatorItemProvider == null) {
			dependenceOperatorItemProvider = new DependenceOperatorItemProvider(this);
		}
		
		return dependenceOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Extraction} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ExtractionItemProvider	extractionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Extraction}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createExtractionAdapter() {
		if (extractionItemProvider == null) {
			extractionItemProvider = new ExtractionItemProvider(this);
		}
		
		return extractionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.IfThenElse} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected IfThenElseItemProvider	ifThenElseItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.IfThenElse}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createIfThenElseAdapter() {
		if (ifThenElseItemProvider == null) {
			ifThenElseItemProvider = new IfThenElseItemProvider(this);
		}
		
		return ifThenElseItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Merge} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected MergeItemProvider	mergeItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Merge}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createMergeAdapter() {
		if (mergeItemProvider == null) {
			mergeItemProvider = new MergeItemProvider(this);
		}
		
		return mergeItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Not} instances. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected NotItemProvider	notItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Not}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createNotAdapter() {
		if (notItemProvider == null) {
			notItemProvider = new NotItemProvider(this);
		}
		
		return notItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.NullClock} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected NullClockItemProvider	nullClockItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.NullClock}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createNullClockAdapter() {
		if (nullClockItemProvider == null) {
			nullClockItemProvider = new NullClockItemProvider(this);
		}
		
		return nullClockItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.NumericExpression} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected NumericExpressionItemProvider	numericExpressionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.NumericExpression}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createNumericExpressionAdapter() {
		if (numericExpressionItemProvider == null) {
			numericExpressionItemProvider = new NumericExpressionItemProvider(this);
		}
		
		return numericExpressionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Or} instances. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected OrItemProvider	orItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Or}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createOrAdapter() {
		if (orItemProvider == null) {
			orItemProvider = new OrItemProvider(this);
		}
		
		return orItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Pragma} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected PragmaItemProvider	pragmaItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Pragma}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createPragmaAdapter() {
		if (pragmaItemProvider == null) {
			pragmaItemProvider = new PragmaItemProvider(this);
		}
		
		return pragmaItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.RepetitionOperator} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected RepetitionOperatorItemProvider	repetitionOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.RepetitionOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createRepetitionOperatorAdapter() {
		if (repetitionOperatorItemProvider == null) {
			repetitionOperatorItemProvider = new RepetitionOperatorItemProvider(this);
		}
		
		return repetitionOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.SequentialDefinition} instances.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SequentialDefinitionItemProvider	sequentialDefinitionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.SequentialDefinition}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createSequentialDefinitionAdapter() {
		if (sequentialDefinitionItemProvider == null) {
			sequentialDefinitionItemProvider = new SequentialDefinitionItemProvider(this);
		}
		
		return sequentialDefinitionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Switch} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SwitchItemProvider	switchItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Switch}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createSwitchAdapter() {
		if (switchItemProvider == null) {
			switchItemProvider = new SwitchItemProvider(this);
		}
		
		return switchItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Transposition} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected TranspositionItemProvider	transpositionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Transposition}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createTranspositionAdapter() {
		if (transpositionItemProvider == null) {
			transpositionItemProvider = new TranspositionItemProvider(this);
		}
		
		return transpositionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.TupleOperator} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected TupleOperatorItemProvider	tupleOperatorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.TupleOperator}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createTupleOperatorAdapter() {
		if (tupleOperatorItemProvider == null) {
			tupleOperatorItemProvider = new TupleOperatorItemProvider(this);
		}
		
		return tupleOperatorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.UnaryMinus} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected UnaryMinusItemProvider	unaryMinusItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.UnaryMinus}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createUnaryMinusAdapter() {
		if (unaryMinusItemProvider == null) {
			unaryMinusItemProvider = new UnaryMinusItemProvider(this);
		}
		
		return unaryMinusItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Variable} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected VariableItemProvider	variableItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Variable}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createVariableAdapter() {
		if (variableItemProvider == null) {
			variableItemProvider = new VariableItemProvider(this);
		}
		
		return variableItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Window} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected WindowItemProvider	windowItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Window}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createWindowAdapter() {
		if (windowItemProvider == null) {
			windowItemProvider = new WindowItemProvider(this);
		}
		
		return windowItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Xor} instances. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected XorItemProvider	xorItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Xor}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createXorAdapter() {
		if (xorItemProvider == null) {
			xorItemProvider = new XorItemProvider(this);
		}
		
		return xorItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.History} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected HistoryItemProvider	historyItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.History}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createHistoryAdapter() {
		if (historyItemProvider == null) {
			historyItemProvider = new HistoryItemProvider(this);
		}
		
		return historyItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.State} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected StateItemProvider	stateItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.State}. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createStateAdapter() {
		if (stateItemProvider == null) {
			stateItemProvider = new StateItemProvider(this);
		}
		
		return stateItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.Automaton} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AutomatonItemProvider	automatonItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.Automaton}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createAutomatonAdapter() {
		if (automatonItemProvider == null) {
			automatonItemProvider = new AutomatonItemProvider(this);
		}
		
		return automatonItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.AndState} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AndStateItemProvider	andStateItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.AndState}. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createAndStateAdapter() {
		if (andStateItemProvider == null) {
			andStateItemProvider = new AndStateItemProvider(this);
		}
		
		return andStateItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.WeakTransition} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected WeakTransitionItemProvider	weakTransitionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.WeakTransition}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createWeakTransitionAdapter() {
		if (weakTransitionItemProvider == null) {
			weakTransitionItemProvider = new WeakTransitionItemProvider(this);
		}
		
		return weakTransitionItemProvider;
	}
	
	/**
	 * This keeps track of the one adapter used for all {@link fr.irisa.espresso.sme.StrongTransition} instances. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected StrongTransitionItemProvider	strongTransitionItemProvider;
	
	/**
	 * This creates an adapter for a {@link fr.irisa.espresso.sme.StrongTransition}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter createStrongTransitionAdapter() {
		if (strongTransitionItemProvider == null) {
			strongTransitionItemProvider = new StrongTransitionItemProvider(this);
		}
		
		return strongTransitionItemProvider;
	}
	
	/**
	 * This returns the root adapter factory that contains this factory. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ComposeableAdapterFactory getRootAdapterFactory() {
		return parentAdapterFactory == null ? this : parentAdapterFactory.getRootAdapterFactory();
	}
	
	/**
	 * This sets the composed adapter factory that contains this factory. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setParentAdapterFactory(ComposedAdapterFactory parentAdapterFactory) {
		this.parentAdapterFactory = parentAdapterFactory;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object type) {
		return supportedTypes.contains(type) || super.isFactoryForType(type);
	}
	
	/**
	 * This implementation substitutes the factory itself as the key for the adapter. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Adapter adapt(Notifier notifier, Object type) {
		return super.adapt(notifier, this);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object adapt(Object object, Object type) {
		if (isFactoryForType(type)) {
			Object adapter = super.adapt(object, type);
			if (!(type instanceof Class) || (((Class<?>) type).isInstance(adapter))) {
				return adapter;
			}
		}
		
		return null;
	}
	
	/**
	 * This adds a listener. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void addListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.addListener(notifyChangedListener);
	}
	
	/**
	 * This removes a listener. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void removeListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.removeListener(notifyChangedListener);
	}
	
	/**
	 * This delegates to {@link #changeNotifier} and to {@link #parentAdapterFactory}. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public void fireNotifyChanged(Notification notification) {
		changeNotifier.fireNotifyChanged(notification);
		
		if (parentAdapterFactory != null) {
			parentAdapterFactory.fireNotifyChanged(notification);
		}
	}
	
	/**
	 * This disposes all of the item providers created by this factory. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void dispose() {
		if (basicIterateItemProvider != null)
			basicIterateItemProvider.dispose();
		if (iterateItemProvider != null)
			iterateItemProvider.dispose();
		if (iterationInitItemProvider != null)
			iterationInitItemProvider.dispose();
		if (specificationsItemProvider != null)
			specificationsItemProvider.dispose();
		if (subProcessItemProvider != null)
			subProcessItemProvider.dispose();
		if (enumerationTypeItemProvider != null)
			enumerationTypeItemProvider.dispose();
		if (modelTypeItemProvider != null)
			modelTypeItemProvider.dispose();
		if (substitutionTypeItemProvider != null)
			substitutionTypeItemProvider.dispose();
		if (tupleTypeItemProvider != null)
			tupleTypeItemProvider.dispose();
		if (modelDeclarationItemProvider != null)
			modelDeclarationItemProvider.dispose();
		if (modelRefItemProvider != null)
			modelRefItemProvider.dispose();
		if (modelInstanceItemProvider != null)
			modelInstanceItemProvider.dispose();
		if (moduleItemProvider != null)
			moduleItemProvider.dispose();
		if (useModuleItemProvider != null)
			useModuleItemProvider.dispose();
		if (constantRefItemProvider != null)
			constantRefItemProvider.dispose();
		if (constantValueItemProvider != null)
			constantValueItemProvider.dispose();
		if (literalItemProvider != null)
			literalItemProvider.dispose();
		if (enumerationValueItemProvider != null)
			enumerationValueItemProvider.dispose();
		if (parameterItemProvider != null)
			parameterItemProvider.dispose();
		if (parameterInstanceItemProvider != null)
			parameterInstanceItemProvider.dispose();
		if (iteratorItemProvider != null)
			iteratorItemProvider.dispose();
		if (indexItemProvider != null)
			indexItemProvider.dispose();
		if (signalRefItemProvider != null)
			signalRefItemProvider.dispose();
		if (inputItemProvider != null)
			inputItemProvider.dispose();
		if (inputInstanceItemProvider != null)
			inputInstanceItemProvider.dispose();
		if (outputItemProvider != null)
			outputItemProvider.dispose();
		if (outputInstanceItemProvider != null)
			outputInstanceItemProvider.dispose();
		if (localItemProvider != null)
			localItemProvider.dispose();
		if (caseConnectionItemProvider != null)
			caseConnectionItemProvider.dispose();
		if (clockRelationItemProvider != null)
			clockRelationItemProvider.dispose();
		if (conditionItemProvider != null)
			conditionItemProvider.dispose();
		if (constantAffectationItemProvider != null)
			constantAffectationItemProvider.dispose();
		if (dataflowConnectionItemProvider != null)
			dataflowConnectionItemProvider.dispose();
		if (dependenceItemProvider != null)
			dependenceItemProvider.dispose();
		if (inputPortItemProvider != null)
			inputPortItemProvider.dispose();
		if (outputPortItemProvider != null)
			outputPortItemProvider.dispose();
		if (andItemProvider != null)
			andItemProvider.dispose();
		if (arithmeticOperatorItemProvider != null)
			arithmeticOperatorItemProvider.dispose();
		if (arrayEnumerationItemProvider != null)
			arrayEnumerationItemProvider.dispose();
		if (arrayProductItemProvider != null)
			arrayProductItemProvider.dispose();
		if (arrayRecoveryItemProvider != null)
			arrayRecoveryItemProvider.dispose();
		if (arrayRestructurationItemProvider != null)
			arrayRestructurationItemProvider.dispose();
		if (assertionItemProvider != null)
			assertionItemProvider.dispose();
		if (assertionClockConstraintOperatorItemProvider != null)
			assertionClockConstraintOperatorItemProvider.dispose();
		if (assertionClockSpeedOperatorItemProvider != null)
			assertionClockSpeedOperatorItemProvider.dispose();
		if (booleanExpressionItemProvider != null)
			booleanExpressionItemProvider.dispose();
		if (cartesianProductItemProvider != null)
			cartesianProductItemProvider.dispose();
		if (cellItemProvider != null)
			cellItemProvider.dispose();
		if (clockConstraintOperatorItemProvider != null)
			clockConstraintOperatorItemProvider.dispose();
		if (clockRelationOperatorItemProvider != null)
			clockRelationOperatorItemProvider.dispose();
		if (clockSpeedOperatorItemProvider != null)
			clockSpeedOperatorItemProvider.dispose();
		if (comparisonOperatorItemProvider != null)
			comparisonOperatorItemProvider.dispose();
		if (complexOperatorItemProvider != null)
			complexOperatorItemProvider.dispose();
		if (concatenationItemProvider != null)
			concatenationItemProvider.dispose();
		if (counterItemProvider != null)
			counterItemProvider.dispose();
		if (delayItemProvider != null)
			delayItemProvider.dispose();
		if (dependenceOperatorItemProvider != null)
			dependenceOperatorItemProvider.dispose();
		if (extractionItemProvider != null)
			extractionItemProvider.dispose();
		if (ifThenElseItemProvider != null)
			ifThenElseItemProvider.dispose();
		if (mergeItemProvider != null)
			mergeItemProvider.dispose();
		if (notItemProvider != null)
			notItemProvider.dispose();
		if (nullClockItemProvider != null)
			nullClockItemProvider.dispose();
		if (numericExpressionItemProvider != null)
			numericExpressionItemProvider.dispose();
		if (orItemProvider != null)
			orItemProvider.dispose();
		if (pragmaItemProvider != null)
			pragmaItemProvider.dispose();
		if (repetitionOperatorItemProvider != null)
			repetitionOperatorItemProvider.dispose();
		if (sequentialDefinitionItemProvider != null)
			sequentialDefinitionItemProvider.dispose();
		if (switchItemProvider != null)
			switchItemProvider.dispose();
		if (transpositionItemProvider != null)
			transpositionItemProvider.dispose();
		if (tupleOperatorItemProvider != null)
			tupleOperatorItemProvider.dispose();
		if (unaryMinusItemProvider != null)
			unaryMinusItemProvider.dispose();
		if (variableItemProvider != null)
			variableItemProvider.dispose();
		if (windowItemProvider != null)
			windowItemProvider.dispose();
		if (xorItemProvider != null)
			xorItemProvider.dispose();
		if (historyItemProvider != null)
			historyItemProvider.dispose();
		if (stateItemProvider != null)
			stateItemProvider.dispose();
		if (automatonItemProvider != null)
			automatonItemProvider.dispose();
		if (andStateItemProvider != null)
			andStateItemProvider.dispose();
		if (weakTransitionItemProvider != null)
			weakTransitionItemProvider.dispose();
		if (strongTransitionItemProvider != null)
			strongTransitionItemProvider.dispose();
	}
	
}
