/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.actions;

import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.edit.ui.action.CommandActionHandler;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.commands.UpdatePortInstanceListsCommand;

/**
 * Created 27 juin 07
 * 
 */
public class LoadModelInstanceAction extends CommandActionHandler {
	protected ModelInstance	selectedObject;
	
	/**
	 * Constructor
	 */
	public LoadModelInstanceAction() {
		super(null, "Load Model Instance Inputs/Outputs");
		setDescription("Reload the Model Instance to load the corresponding Input, Output, and Parameter Instances");
		selectedObject = null;
	}
	
	public void run() {
		if (selectedObject != null && domain != null) {
			command = new UpdatePortInstanceListsCommand(selectedObject);
			super.run();
		}
	}
	
	/**
	 * @see org.eclipse.emf.edit.ui.action.CommandActionHandler#updateSelection(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	@Override
	public boolean updateSelection(IStructuredSelection selection) {
		if (selection == null || selection.size() != 1)
			return false;
		if (selection.getFirstElement() instanceof ModelInstance) {
			selectedObject = (ModelInstance) selection.getFirstElement();
			setEnabled(true);
		}
		else {
			selectedObject = null;
			setEnabled(false);
		}
		return true;
	}
	
	/**
	 * @param workbenchPart
	 */
	public void setActiveWorkbenchPart(IWorkbenchPart workbenchPart) {
		if (workbenchPart instanceof IEditingDomainProvider) {
			domain = ((IEditingDomainProvider) workbenchPart).getEditingDomain();
		}
	}
}
