/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.commands;

import org.eclipse.emf.common.command.AbstractCommand;

import fr.irisa.espresso.sme.Input;
import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.Output;
import fr.irisa.espresso.sme.OutputInstance;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.ParameterInstance;
import fr.irisa.espresso.sme.SmeFactory;

/**
 * This command is used to update the model of a ModelInstance
 * 
 * Created 28 sept. 06
 * 
 */
public class UpdatePortInstanceListsCommand extends AbstractCommand {
	/** The graph node that should be updated */
	private ModelInstance	instance;
	
	/** Instance Counter * */
	private static int		counter	= 0;
	
	/**
	 * Constructor
	 * 
	 * @param domain
	 * @param instance
	 */
	public UpdatePortInstanceListsCommand(ModelInstance instance) {
		super("Update port Instance lists");
		
		this.instance = instance;
	}
	
	/**
	 * @see org.eclipse.emf.common.command.Command#execute()
	 */
	public void execute() {
		redo();
		
	}
	
	/**
	 * @see org.eclipse.emf.common.command.Command#redo()
	 */
	public void redo() {
		InterfaceDefinition id = instance.getReferredInterface();
		if (id == null) {
			instance.getInputs().clear();
			instance.getOutputs().clear();
			instance.getParameters().clear();
		}
		else {
			for (Parameter par : id.getParameters()) {
				boolean res = true;
				for (ParameterInstance pi : instance.getParameters()) {
					if (pi.getParameter().equals(par)) {
						res = false;
						break;
					}
				}
				if (res) {
					ParameterInstance ref = SmeFactory.eINSTANCE.createParameterInstance();
					ref.setName("_" + par.getName() + "_" + counter++);
					ref.setParameter(par);
					ref.setInstanceParent(instance);
				}
			}
			for (Input in : id.getInputs()) {
				boolean res = true;
				for (InputInstance ii : instance.getInputs()) {
					if (ii.getInput().equals(in)) {
						res = false;
						break;
					}
				}
				if (res) {
					InputInstance ref = SmeFactory.eINSTANCE.createInputInstance();
					ref.setName("_" + in.getName() + "_" + counter++);
					ref.setInput(in);
					ref.setInstanceParent(instance);
				}
			}
			for (Output out : id.getOutputs()) {
				boolean res = true;
				for (OutputInstance oi : instance.getOutputs()) {
					if (oi.getOutput().equals(out)) {
						res = false;
						break;
					}
				}
				if (res) {
					OutputInstance ref = SmeFactory.eINSTANCE.createOutputInstance();
					ref.setName("_" + out.getName() + "_" + counter++);
					ref.setOutput(out);
					ref.setInstanceParent(instance);
				}
			}
		}
	}
	
	/**
	 * @see org.eclipse.emf.common.command.AbstractCommand#canUndo()
	 */
	@Override
	public boolean canUndo() {
		return false;
	}
	
	/**
	 * @see org.eclipse.emf.common.command.AbstractCommand#prepare()
	 */
	@Override
	protected boolean prepare() {
		return true;
	}
}
