/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

/**
 * <!-- begin-user-doc --> A representation of the literals of the enumeration '<em><b>Case Kind</b></em>', and utility
 * methods for working with them. <!-- end-user-doc --> <!-- begin-model-doc --> This enumeration contains the different
 * kinds of expression that can be used as choices in a switch expression. <!-- end-model-doc -->
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getCaseKind()
 * @model annotation="http://www.topcased.org/uuid uuid='_2I15Me5WEduz9o2pAin6-g'"
 * @generated
 */
public enum CaseKind implements Enumerator {
	/**
	 * The '<em><b>Else Case</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #ELSE_CASE
	 * @generated
	 * @ordered
	 */
	ELSE_CASE_LITERAL(0, "ElseCase", "Else Case"),

	/**
	 * The '<em><b>Enumeration</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #ENUMERATION
	 * @generated
	 * @ordered
	 */
	ENUMERATION_LITERAL(1, "Enumeration", "Enumeration"),

	/**
	 * The '<em><b>Closed Interval</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #CLOSED_INTERVAL
	 * @generated
	 * @ordered
	 */
	CLOSED_INTERVAL_LITERAL(2, "ClosedInterval", "Closed Interval"),

	/**
	 * The '<em><b>Right Half Opened Interval</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #RIGHT_HALF_OPENED_INTERVAL
	 * @generated
	 * @ordered
	 */
	RIGHT_HALF_OPENED_INTERVAL_LITERAL(3, "RightHalfOpenedInterval", "Right Half Opened Interval"),

	/**
	 * The '<em><b>Left Half Opened Interval</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #LEFT_HALF_OPENED_INTERVAL
	 * @generated
	 * @ordered
	 */
	LEFT_HALF_OPENED_INTERVAL_LITERAL(4, "LeftHalfOpenedInterval", "Left Half Opened Interval"),

	/**
	 * The '<em><b>Opened Interval</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #OPENED_INTERVAL
	 * @generated
	 * @ordered
	 */
	OPENED_INTERVAL_LITERAL(5, "OpenedInterval", "Opened Interval");
	
	/**
	 * The '<em><b>Else Case</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Else Case</b></em>' literal object isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> The default expression of a switch ("case") expression. <!--
	 * end-model-doc -->
	 * 
	 * @see #ELSE_CASE_LITERAL
	 * @model name="ElseCase" literal="Else Case"
	 *        annotation="http://www.topcased.org/uuid uuid='_2I_DIO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				ELSE_CASE					= 0;
	
	/**
	 * The '<em><b>Enumeration</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Enumeration</b></em>' literal object isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> With this case kind, the case consists in several value (integer
	 * or enumerated value) separated by commas.
	 * 
	 * <!-- end-model-doc -->
	 * 
	 * @see #ENUMERATION_LITERAL
	 * @model name="Enumeration" annotation="http://www.topcased.org/uuid uuid='_2I_DIe5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				ENUMERATION					= 1;
	
	/**
	 * The '<em><b>Closed Interval</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Closed Interval</b></em>' literal object isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> With this case kind, it means that user has to define an interval
	 * of value whose bounds (lower and upper) are both included. The intervals are usable only if the values of the
	 * type of the signal tested are totally ordered The bounds of the interval are optional (one of the two must be
	 * present): if the lower bound is absent, the interval represents all the values smaller than the upper bound, if
	 * the upper bound is absent, the interval represents all the values greater than the lower bound. <!--
	 * end-model-doc -->
	 * 
	 * @see #CLOSED_INTERVAL_LITERAL
	 * @model name="ClosedInterval" literal="Closed Interval"
	 *        annotation="http://www.topcased.org/uuid uuid='_2I_DIu5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				CLOSED_INTERVAL				= 2;
	
	/**
	 * The '<em><b>Right Half Opened Interval</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Right Half Opened Interval</b></em>' literal object isn't clear, there really should be
	 * more of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> With this case kind, it means that user has to define an interval
	 * of value whose lower bound are included and not its upper. The intervals are usable only if the values of the
	 * type of the signal tested are totally ordered The bounds of the interval are optional (one of the two must be
	 * present): if the lower bound is absent, the interval represents all the values strictly smaller than the upper
	 * bound, if the upper bound is absent, the interval represents all the values greater than the lower bound. <!--
	 * end-model-doc -->
	 * 
	 * @see #RIGHT_HALF_OPENED_INTERVAL_LITERAL
	 * @model name="RightHalfOpenedInterval" literal="Right Half Opened Interval"
	 *        annotation="http://www.topcased.org/uuid uuid='_2JI0IO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				RIGHT_HALF_OPENED_INTERVAL	= 3;
	
	/**
	 * The '<em><b>Left Half Opened Interval</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Left Half Opened Interval</b></em>' literal object isn't clear, there really should be
	 * more of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> With this case kind, it means that user has to define an interval
	 * of value whose upper bound are included and not its lower. The intervals are usable only if the values of the
	 * type of the signal tested are totally ordered The bounds of the interval are optional (one of the two must be
	 * present): if the lower bound is absent, the interval represents all the values smaller than the upper bound, if
	 * the upper bound is absent, the interval represents all the values strictly greater than the lower bound. <!--
	 * end-model-doc -->
	 * 
	 * @see #LEFT_HALF_OPENED_INTERVAL_LITERAL
	 * @model name="LeftHalfOpenedInterval" literal="Left Half Opened Interval"
	 *        annotation="http://www.topcased.org/uuid uuid='_2JI0Ie5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				LEFT_HALF_OPENED_INTERVAL	= 4;
	
	/**
	 * The '<em><b>Opened Interval</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Opened Interval</b></em>' literal object isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> With this case kind, it means that user has to define an interval
	 * of value whose bounds (lower and upper) are both excluded. The intervals are usable only if the values of the
	 * type of the signal tested are totally ordered The bounds of the interval are optional (one of the two must be
	 * present): if the lower bound is absent, the interval represents all the values strictly smaller than the upper
	 * bound, if the upper bound is absent, the interval represents all the values strictly greater than the lower
	 * bound. <!-- end-model-doc -->
	 * 
	 * @see #OPENED_INTERVAL_LITERAL
	 * @model name="OpenedInterval" literal="Opened Interval"
	 *        annotation="http://www.topcased.org/uuid uuid='_2JSlIO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				OPENED_INTERVAL				= 5;
	
	/**
	 * An array of all the '<em><b>Case Kind</b></em>' enumerators. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private static final CaseKind[]		VALUES_ARRAY				= new CaseKind[] { ELSE_CASE_LITERAL,
			ENUMERATION_LITERAL, CLOSED_INTERVAL_LITERAL, RIGHT_HALF_OPENED_INTERVAL_LITERAL,
			LEFT_HALF_OPENED_INTERVAL_LITERAL, OPENED_INTERVAL_LITERAL, };
	
	/**
	 * A public read-only list of all the '<em><b>Case Kind</b></em>' enumerators. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static final List<CaseKind>	VALUES						= Collections.unmodifiableList(Arrays
																			.asList(VALUES_ARRAY));
	
	/**
	 * Returns the '<em><b>Case Kind</b></em>' literal with the specified literal value. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static CaseKind get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			CaseKind result = VALUES_ARRAY[i];
			if (result.toString().equals(literal)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Case Kind</b></em>' literal with the specified name. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static CaseKind getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			CaseKind result = VALUES_ARRAY[i];
			if (result.getName().equals(name)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Case Kind</b></em>' literal with the specified integer value. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static CaseKind get(int value) {
		switch (value) {
			case ELSE_CASE:
				return ELSE_CASE_LITERAL;
			case ENUMERATION:
				return ENUMERATION_LITERAL;
			case CLOSED_INTERVAL:
				return CLOSED_INTERVAL_LITERAL;
			case RIGHT_HALF_OPENED_INTERVAL:
				return RIGHT_HALF_OPENED_INTERVAL_LITERAL;
			case LEFT_HALF_OPENED_INTERVAL:
				return LEFT_HALF_OPENED_INTERVAL_LITERAL;
			case OPENED_INTERVAL:
				return OPENED_INTERVAL_LITERAL;
		}
		return null;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final int		value;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	name;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	literal;
	
	/**
	 * Only this class can construct instances. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private CaseKind(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public int getValue() {
		return value;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getLiteral() {
		return literal;
	}
	
	/**
	 * Returns the literal value of the enumerator, which is its string representation. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		return literal;
	}
	
} // CaseKind
