/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

/**
 * <!-- begin-user-doc --> A representation of the literals of the enumeration '<em><b>Clock Constraint Op Kind</b></em>
 * ', and utility methods for working with them. <!-- end-user-doc --> <!-- begin-model-doc --> This enumeration
 * contains the different kind of clock constraint operators.
 * 
 * <!-- end-model-doc -->
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getClockConstraintOpKind()
 * @model annotation="http://www.topcased.org/uuid uuid='_2KVG8O5WEduz9o2pAin6-g'"
 * @generated
 */
public enum ClockConstraintOpKind implements Enumerator {
	/**
	 * The '<em><b>Synchronized</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #SYNCHRONIZED
	 * @generated
	 * @ordered
	 */
	SYNCHRONIZED_LITERAL(0, "Synchronized", "Synchronized"),

	/**
	 * The '<em><b>Excluded</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #EXCLUDED
	 * @generated
	 * @ordered
	 */
	EXCLUDED_LITERAL(1, "Excluded", "Excluded"),

	/**
	 * The '<em><b>Identity</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #IDENTITY
	 * @generated
	 * @ordered
	 */
	IDENTITY_LITERAL(2, "Identity", "Identity");
	
	/**
	 * The '<em><b>Synchronized</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Synchronized</b></em>' literal object isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @see #SYNCHRONIZED_LITERAL
	 * @model name="Synchronized" annotation="http://www.topcased.org/uuid uuid='_2KVG8e5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int							SYNCHRONIZED	= 0;
	
	/**
	 * The '<em><b>Excluded</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Excluded</b></em>' literal object isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @see #EXCLUDED_LITERAL
	 * @model name="Excluded" annotation="http://www.topcased.org/uuid uuid='_2KVG8u5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int							EXCLUDED		= 1;
	
	/**
	 * The '<em><b>Identity</b></em>' literal value. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of '<em><b>Identity</b></em>' literal object isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @see #IDENTITY_LITERAL
	 * @model name="Identity" annotation="http://www.topcased.org/uuid uuid='_2KVG8-5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int							IDENTITY		= 2;
	
	/**
	 * An array of all the '<em><b>Clock Constraint Op Kind</b></em>' enumerators. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	private static final ClockConstraintOpKind[]	VALUES_ARRAY	= new ClockConstraintOpKind[] {
			SYNCHRONIZED_LITERAL, EXCLUDED_LITERAL, IDENTITY_LITERAL, };
	
	/**
	 * A public read-only list of all the '<em><b>Clock Constraint Op Kind</b></em>' enumerators. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static final List<ClockConstraintOpKind>	VALUES			= Collections.unmodifiableList(Arrays
																			.asList(VALUES_ARRAY));
	
	/**
	 * Returns the '<em><b>Clock Constraint Op Kind</b></em>' literal with the specified literal value. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static ClockConstraintOpKind get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			ClockConstraintOpKind result = VALUES_ARRAY[i];
			if (result.toString().equals(literal)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Clock Constraint Op Kind</b></em>' literal with the specified name. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static ClockConstraintOpKind getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			ClockConstraintOpKind result = VALUES_ARRAY[i];
			if (result.getName().equals(name)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Clock Constraint Op Kind</b></em>' literal with the specified integer value. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static ClockConstraintOpKind get(int value) {
		switch (value) {
			case SYNCHRONIZED:
				return SYNCHRONIZED_LITERAL;
			case EXCLUDED:
				return EXCLUDED_LITERAL;
			case IDENTITY:
				return IDENTITY_LITERAL;
		}
		return null;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final int		value;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	name;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	literal;
	
	/**
	 * Only this class can construct instances. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private ClockConstraintOpKind(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public int getValue() {
		return value;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getLiteral() {
		return literal;
	}
	
	/**
	 * Returns the literal value of the enumerator, which is its string representation. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		return literal;
	}
	
} // ClockConstraintOpKind
