/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

/**
 * <!-- begin-user-doc --> A representation of the literals of the enumeration '
 * <em><b>Compilation Directive Kind</b></em>', and utility methods for working with them. <!-- end-user-doc -->
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getCompilationDirectiveKind()
 * @model annotation="http://www.topcased.org/uuid uuid='_2EXo4O5WEduz9o2pAin6-g'"
 * @generated
 */
public enum CompilationDirectiveKind implements Enumerator {
	/**
	 * The '<em><b>NONE</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #NONE
	 * @generated
	 * @ordered
	 */
	NONE_LITERAL(0, "NONE", "NONE"),

	/**
	 * The '<em><b>Unexpanded</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #UNEXPANDED
	 * @generated
	 * @ordered
	 */
	UNEXPANDED_LITERAL(1, "unexpanded", "unexpanded"),

	/**
	 * The '<em><b>Separate</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #SEPARATE
	 * @generated
	 * @ordered
	 */
	SEPARATE_LITERAL(2, "separate", "separate"),

	/**
	 * The '<em><b>Black Box</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #BLACK_BOX
	 * @generated
	 * @ordered
	 */
	BLACK_BOX_LITERAL(3, "Black_Box", "Black_Box"),

	/**
	 * The '<em><b>Grey Box</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #GREY_BOX
	 * @generated
	 * @ordered
	 */
	GREY_BOX_LITERAL(4, "Grey_Box", "Grey_Box"),

	/**
	 * The '<em><b>Delay Cluster</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #DELAY_CLUSTER
	 * @generated
	 * @ordered
	 */
	DELAY_CLUSTER_LITERAL(5, "DelayCluster", "DelayCluster");
	
	/**
	 * The '<em><b>NONE</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc -->
	 * No compilation directive. <!-- end-model-doc -->
	 * 
	 * @see #NONE_LITERAL
	 * @model annotation="http://www.topcased.org/uuid uuid='_2IPcQO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int								NONE			= 0;
	
	/**
	 * The '<em><b>Unexpanded</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!--
	 * begin-model-doc --> Means that the ModelDeclaration is not expanded when it is called. Each one of its instances
	 * is compiled in its calling context and has an associated object (graph) in the internal representation. <!--
	 * end-model-doc -->
	 * 
	 * @see #UNEXPANDED_LITERAL
	 * @model name="unexpanded" annotation="http://www.topcased.org/uuid uuid='_2IPcQe5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int								UNEXPANDED		= 1;
	
	/**
	 * The '<em><b>Separate</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc
	 * --> Means that the model is not expanded when it is called. It is compiled separately and has an associated
	 * object (graph) in the internal representation. <!-- end-model-doc -->
	 * 
	 * @see #SEPARATE_LITERAL
	 * @model name="separate" annotation="http://www.topcased.org/uuid uuid='_2IZNQO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int								SEPARATE		= 2;
	
	/**
	 * The '<em><b>Black Box</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc
	 * --> Represents the �black box� abstraction of a model (may be the result of a compilation). Only the interface of
	 * the model, including its external graph, is represented: its body is empty. <!-- end-model-doc -->
	 * 
	 * @see #BLACK_BOX_LITERAL
	 * @model name="Black_Box" annotation="http://www.topcased.org/uuid uuid='_2IZNQe5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int								BLACK_BOX		= 3;
	
	/**
	 * The '<em><b>Grey Box</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc
	 * --> Represents a �grey box� abstraction of a model. It contains an external graph that represents clock and
	 * dependence relations of the interface, but also a restructuring of the model into clusters together with a
	 * representation of the scheduling of these clusters (clock and dependence relations between these clusters). Each
	 * cluster is represented as a �black box� abstraction which is such that any input of the cluster precedes any of
	 * its outputs. <!-- end-model-doc -->
	 * 
	 * @see #GREY_BOX_LITERAL
	 * @model name="Grey_Box" annotation="http://www.topcased.org/uuid uuid='_2IZNQu5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int								GREY_BOX		= 4;
	
	/**
	 * The '<em><b>Delay Cluster</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!--
	 * begin-model-doc --> May qualify one of the clusters of a �grey box� abstraction when code generation is expected
	 * from this abstraction: in that case, one of the clusters, the �delay cluster� (represented, like the other ones,
	 * by its �black box� abstraction), groups together the delay operations of the model and is preceded by each one of
	 * the other clusters (in the generated code, memories will be updated at the end of one instant). <!--
	 * end-model-doc -->
	 * 
	 * @see #DELAY_CLUSTER_LITERAL
	 * @model name="DelayCluster" annotation="http://www.topcased.org/uuid uuid='_2Ii-QO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int								DELAY_CLUSTER	= 5;
	
	/**
	 * An array of all the '<em><b>Compilation Directive Kind</b></em>' enumerators. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	private static final CompilationDirectiveKind[]		VALUES_ARRAY	= new CompilationDirectiveKind[] {
			NONE_LITERAL, UNEXPANDED_LITERAL, SEPARATE_LITERAL, BLACK_BOX_LITERAL, GREY_BOX_LITERAL,
			DELAY_CLUSTER_LITERAL,										};
	
	/**
	 * A public read-only list of all the '<em><b>Compilation Directive Kind</b></em>' enumerators. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static final List<CompilationDirectiveKind>	VALUES			= Collections.unmodifiableList(Arrays
																				.asList(VALUES_ARRAY));
	
	/**
	 * Returns the '<em><b>Compilation Directive Kind</b></em>' literal with the specified literal value. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static CompilationDirectiveKind get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			CompilationDirectiveKind result = VALUES_ARRAY[i];
			if (result.toString().equals(literal)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Compilation Directive Kind</b></em>' literal with the specified name. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static CompilationDirectiveKind getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			CompilationDirectiveKind result = VALUES_ARRAY[i];
			if (result.getName().equals(name)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Compilation Directive Kind</b></em>' literal with the specified integer value. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static CompilationDirectiveKind get(int value) {
		switch (value) {
			case NONE:
				return NONE_LITERAL;
			case UNEXPANDED:
				return UNEXPANDED_LITERAL;
			case SEPARATE:
				return SEPARATE_LITERAL;
			case BLACK_BOX:
				return BLACK_BOX_LITERAL;
			case GREY_BOX:
				return GREY_BOX_LITERAL;
			case DELAY_CLUSTER:
				return DELAY_CLUSTER_LITERAL;
		}
		return null;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final int		value;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	name;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	literal;
	
	/**
	 * Only this class can construct instances. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private CompilationDirectiveKind(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public int getValue() {
		return value;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getLiteral() {
		return literal;
	}
	
	/**
	 * Returns the literal value of the enumerator, which is its string representation. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		return literal;
	}
	
} // CompilationDirectiveKind
