/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

/**
 * <!-- begin-user-doc --> A representation of the literals of the enumeration '<em><b>Counter Kind</b></em>', and
 * utility methods for working with them. <!-- end-user-doc -->
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getCounterKind()
 * @model annotation="http://www.topcased.org/uuid uuid='_2LEt0e5WEduz9o2pAin6-g'"
 * @generated
 */
public enum CounterKind implements Enumerator {
	/**
	 * The '<em><b>After</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #AFTER_VALUE
	 * @generated
	 * @ordered
	 */
	AFTER(0, "after", "after"),

	/**
	 * The '<em><b>From</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #FROM_VALUE
	 * @generated
	 * @ordered
	 */
	FROM(1, "from", "from"),

	/**
	 * The '<em><b>Count</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #COUNT_VALUE
	 * @generated
	 * @ordered
	 */
	COUNT(2, "count", "count");
	
	/**
	 * The '<em><b>After</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc -->
	 * This operator counts the number of occurences of the first input expression which follows the last occurence of
	 * the second output expression. If the first expression is simultaneous with the second, the occurence is not
	 * counted. <!-- end-model-doc -->
	 * 
	 * @see #AFTER
	 * @model name="after" annotation="http://www.topcased.org/uuid uuid='_2LEt0u5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int					AFTER_VALUE		= 0;
	
	/**
	 * The '<em><b>From</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc -->
	 * This operator works as "after", but, if an occurence of the first expression is simultenous with an occurence of
	 * the second expression, this occurence is counted. <!-- end-model-doc -->
	 * 
	 * @see #FROM
	 * @model name="from" annotation="http://www.topcased.org/uuid uuid='_2LEt0-5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int					FROM_VALUE		= 1;
	
	/**
	 * The '<em><b>Count</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc -->
	 * This operator counts the number of occurence of the first input expression modulo the constant integer expression
	 * given in the second input expression. <!-- end-model-doc -->
	 * 
	 * @see #COUNT
	 * @model name="count" annotation="http://www.topcased.org/uuid uuid='_2LOe0O5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int					COUNT_VALUE		= 2;
	
	/**
	 * An array of all the '<em><b>Counter Kind</b></em>' enumerators. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private static final CounterKind[]		VALUES_ARRAY	= new CounterKind[] { AFTER, FROM, COUNT, };
	
	/**
	 * A public read-only list of all the '<em><b>Counter Kind</b></em>' enumerators. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static final List<CounterKind>	VALUES			= Collections.unmodifiableList(Arrays.asList(VALUES_ARRAY));
	
	/**
	 * Returns the '<em><b>Counter Kind</b></em>' literal with the specified literal value. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static CounterKind get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			CounterKind result = VALUES_ARRAY[i];
			if (result.toString().equals(literal)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Counter Kind</b></em>' literal with the specified name. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static CounterKind getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			CounterKind result = VALUES_ARRAY[i];
			if (result.getName().equals(name)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Counter Kind</b></em>' literal with the specified integer value. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static CounterKind get(int value) {
		switch (value) {
			case AFTER_VALUE:
				return AFTER;
			case FROM_VALUE:
				return FROM;
			case COUNT_VALUE:
				return COUNT;
		}
		return null;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final int		value;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	name;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	literal;
	
	/**
	 * Only this class can construct instances. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private CounterKind(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public int getValue() {
		return value;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getLiteral() {
		return literal;
	}
	
	/**
	 * Returns the literal value of the enumerator, which is its string representation. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		return literal;
	}
	
} // CounterKind
