/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import org.eclipse.emf.common.util.EList;

/**
 * <!-- begin-user-doc --> A representation of the model object '<em><b>Dependence In Out</b></em>'. <!-- end-user-doc
 * -->
 * 
 * <!-- begin-model-doc --> DependenceInOut is an abstract meta-class that represents elements that can be used as the
 * source or target of a Dependence Connection. So it represents, initially, all identifiers and the DependenceOperator.
 * <!-- end-model-doc -->
 * 
 * <p>
 * The following features are supported:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.DependenceInOut#getSrc_Dependences <em>Src Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.DependenceInOut#getDst_Dependences <em>Dst Dependences</em>}</li>
 * </ul>
 * </p>
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getDependenceInOut()
 * @model interface="true" abstract="true" annotation="http://www.topcased.org/uuid uuid='_2LOe4O5WEduz9o2pAin6-g'"
 * @generated
 */
public interface DependenceInOut extends ConnectableElement {
	/**
	 * Returns the value of the '<em><b>Src Dependences</b></em>' reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Dependence}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Dependence#getSrc <em>Src</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Src Dependences</em>' reference list isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> This elements references all outgoing Dependence relations from
	 * the current element. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Src Dependences</em>' reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getDependenceInOut_Src_Dependences()
	 * @see fr.irisa.espresso.sme.Dependence#getSrc
	 * @model opposite="src" annotation="http://www.topcased.org/uuid uuid='_2LOe4e5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Dependence> getSrc_Dependences();
	
	/**
	 * Returns the value of the '<em><b>Dst Dependences</b></em>' reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Dependence}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Dependence#getDst <em>Dst</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Dst Dependences</em>' reference list isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> This elements references all incoming Dependence relations to the
	 * current element. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Dst Dependences</em>' reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getDependenceInOut_Dst_Dependences()
	 * @see fr.irisa.espresso.sme.Dependence#getDst
	 * @model opposite="dst" annotation="http://www.topcased.org/uuid uuid='_2LOe4u5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Dependence> getDst_Dependences();
	
} // DependenceInOut
