/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import org.eclipse.emf.common.util.EList;

/**
 * <!-- begin-user-doc --> A representation of the model object '<em><b>Interface Definition</b></em>'. <!--
 * end-user-doc -->
 * 
 * <!-- begin-model-doc --> This abstract meta-class represents an interface of a process (i.e. the declaration of Input
 * and Output signal, and constant Parameter). This is mainly used by ModelDeclaration (process, function...) and also
 * by Model Type declarations. <!-- end-model-doc -->
 * 
 * <p>
 * The following features are supported:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.InterfaceDefinition#getType <em>Type</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.InterfaceDefinition#getInputs <em>Inputs</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.InterfaceDefinition#getOutputs <em>Outputs</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.InterfaceDefinition#getParameters <em>Parameters</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.InterfaceDefinition#getSpec <em>Spec</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.InterfaceDefinition#getPragmas <em>Pragmas</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.InterfaceDefinition#getProcessClass <em>Process Class</em>}</li>
 * </ul>
 * </p>
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getInterfaceDefinition()
 * @model interface="true" abstract="true" annotation="http://www.topcased.org/uuid uuid='_2LOe6e5WEduz9o2pAin6-g'"
 * @generated
 */
public interface InterfaceDefinition extends SignalContainer {
	/**
	 * Returns the value of the '<em><b>Type</b></em>' reference. <!-- begin-user-doc --> <!-- end-user-doc --> <!--
	 * begin-model-doc --> If the type is specified, no other input/output/parameter can be added. <!-- end-model-doc
	 * -->
	 * 
	 * @return the value of the '<em>Type</em>' reference.
	 * @see #setType(ModelType)
	 * @see fr.irisa.espresso.sme.SmePackage#getInterfaceDefinition_Type()
	 * @model annotation="http://www.topcased.org/uuid uuid='_2LOe6u5WEduz9o2pAin6-g'"
	 * @generated
	 */
	ModelType getType();
	
	/**
	 * Sets the value of the '{@link fr.irisa.espresso.sme.InterfaceDefinition#getType <em>Type</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param value
	 *        the new value of the '<em>Type</em>' reference.
	 * @see #getType()
	 * @generated
	 */
	void setType(ModelType value);
	
	/**
	 * Returns the value of the '<em><b>Inputs</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Input}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Input#getInterfaceParent <em>Interface Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Inputs</em>' containment reference list isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> An ordered list of all Input signals <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Inputs</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getInterfaceDefinition_Inputs()
	 * @see fr.irisa.espresso.sme.Input#getInterfaceParent
	 * @model opposite="interfaceParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe6-5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Input> getInputs();
	
	/**
	 * Returns the value of the '<em><b>Outputs</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Output}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Output#getInterfaceParent <em>Interface Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Outputs</em>' containment reference list isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> An ordered list of all Output signals <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Outputs</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getInterfaceDefinition_Outputs()
	 * @see fr.irisa.espresso.sme.Output#getInterfaceParent
	 * @model opposite="interfaceParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe7O5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Output> getOutputs();
	
	/**
	 * Returns the value of the '<em><b>Parameters</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Parameter}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Parameter#getInterfaceParent <em>Interface Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Parameters</em>' containment reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> An ordered list of all Parameter constants. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Parameters</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getInterfaceDefinition_Parameters()
	 * @see fr.irisa.espresso.sme.Parameter#getInterfaceParent
	 * @model opposite="interfaceParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe7e5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Parameter> getParameters();
	
	/**
	 * Returns the value of the '<em><b>Spec</b></em>' containment reference. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Specifications#getInterfaceParent <em>Interface Parent</em>}'. <!-- begin-user-doc
	 * -->
	 * <p>
	 * If the meaning of the '<em>Spec</em>' containment reference isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> The Specification area which has to used to specify clock
	 * relations between the different input and output signals of the interface. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Spec</em>' containment reference.
	 * @see #setSpec(Specifications)
	 * @see fr.irisa.espresso.sme.SmePackage#getInterfaceDefinition_Spec()
	 * @see fr.irisa.espresso.sme.Specifications#getInterfaceParent
	 * @model opposite="interfaceParent" containment="true" required="true" ordered="false"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe7u5WEduz9o2pAin6-g'"
	 * @generated
	 */
	Specifications getSpec();
	
	/**
	 * Sets the value of the '{@link fr.irisa.espresso.sme.InterfaceDefinition#getSpec <em>Spec</em>}' containment
	 * reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param value
	 *        the new value of the '<em>Spec</em>' containment reference.
	 * @see #getSpec()
	 * @generated
	 */
	void setSpec(Specifications value);
	
	/**
	 * Returns the value of the '<em><b>Pragmas</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Pragma}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Pragma#getInterfaceParent <em>Interface Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Pragmas</em>' containment reference list isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of pragmas to apply to this interface <!-- end-model-doc
	 * -->
	 * 
	 * @return the value of the '<em>Pragmas</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getInterfaceDefinition_Pragmas()
	 * @see fr.irisa.espresso.sme.Pragma#getInterfaceParent
	 * @model opposite="interfaceParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe7-5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Pragma> getPragmas();
	
	/**
	 * Returns the value of the '<em><b>Process Class</b></em>' attribute. The default value is <code>"unsafe"</code>.
	 * The literals are from the enumeration {@link fr.irisa.espresso.sme.ProcessAttributeKind}. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Process Class</em>' attribute isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> Define the class of the process represented by this interface.
	 * <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Process Class</em>' attribute.
	 * @see fr.irisa.espresso.sme.ProcessAttributeKind
	 * @see #setProcessClass(ProcessAttributeKind)
	 * @see fr.irisa.espresso.sme.SmePackage#getInterfaceDefinition_ProcessClass()
	 * @model default="unsafe" ordered="false" annotation="http://www.topcased.org/uuid uuid='_2LOe8O5WEduz9o2pAin6-g'"
	 * @generated
	 */
	ProcessAttributeKind getProcessClass();
	
	/**
	 * Sets the value of the '{@link fr.irisa.espresso.sme.InterfaceDefinition#getProcessClass <em>Process Class</em>}'
	 * attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param value
	 *        the new value of the '<em>Process Class</em>' attribute.
	 * @see fr.irisa.espresso.sme.ProcessAttributeKind
	 * @see #getProcessClass()
	 * @generated
	 */
	void setProcessClass(ProcessAttributeKind value);
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc --> This method returns a string describing
	 * the interface definition.
	 * 
	 * @param asCode
	 *        if it is true, the generated string will be formatted as in code generation otherwise, it calls the
	 *        classical toString method.
	 * @param withComments
	 *        if it is true, the comment will be added to the String. <!-- end-model-doc -->
	 * @model required="true" asCodeRequired="true" withCommentsRequired="true"
	 * @generated
	 */
	String toString(boolean asCode, boolean withComments);
	
} // InterfaceDefinition
