/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import org.eclipse.emf.common.util.EList;

/**
 * <!-- begin-user-doc --> A representation of the model object '<em><b>Model</b></em>'. <!-- end-user-doc -->
 * 
 * <!-- begin-model-doc --> This abstract meta-class represents a process with no interface. In each kind of model, it
 * is possible to define dataflow equation and/or clock and dependence relations <!-- end-model-doc -->
 * 
 * <p>
 * The following features are supported:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.Model#getAssertions <em>Assertions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getAutomata <em>Automata</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getConnections <em>Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getConstantRef <em>Constant Ref</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getConstants <em>Constants</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getConstraints <em>Constraints</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getIndexes <em>Indexes</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getContainedInstances <em>Contained Instances</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getLiterals <em>Literals</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getLocals <em>Locals</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getOperators <em>Operators</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getSignalRef <em>Signal Ref</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getSubprocesses <em>Subprocesses</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getTypes <em>Types</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getUseModule <em>Use Module</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.Model#getModelRefs <em>Model Refs</em>}</li>
 * </ul>
 * </p>
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getModel()
 * @model abstract="true" annotation="http://www.topcased.org/uuid uuid='_2LOe8e5WEduz9o2pAin6-g'"
 * @generated
 */
public interface Model extends SignalContainer {
	/**
	 * Returns the value of the '<em><b>Assertions</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Assertion}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Assertion#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Assertions</em>' containment reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of assertions on boolean expression. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Assertions</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Assertions()
	 * @see fr.irisa.espresso.sme.Assertion#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe8u5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Assertion> getAssertions();
	
	/**
	 * Returns the value of the '<em><b>Automata</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Automaton}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Automaton#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Automata</em>' containment reference list isn't clear, there really should be more of
	 * a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of mode automata <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Automata</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Automata()
	 * @see fr.irisa.espresso.sme.Automaton#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe8-5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Automaton> getAutomata();
	
	/**
	 * Returns the value of the '<em><b>Connections</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.ModelConnection}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.ModelConnection#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Connections</em>' containment reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of all connections that can be added in a model <!--
	 * end-model-doc -->
	 * 
	 * @return the value of the '<em>Connections</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Connections()
	 * @see fr.irisa.espresso.sme.ModelConnection#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe9O5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<ModelConnection> getConnections();
	
	/**
	 * Returns the value of the '<em><b>Constant Ref</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.ConstantRef}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.ConstantRef#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Constant Ref</em>' containment reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of references to constant identifier declared at another
	 * place. The reference must be in the scope of the constant identifier. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Constant Ref</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_ConstantRef()
	 * @see fr.irisa.espresso.sme.ConstantRef#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe9e5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<ConstantRef> getConstantRef();
	
	/**
	 * Returns the value of the '<em><b>Constants</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.ConstantValue}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.ConstantValue#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Constants</em>' containment reference list isn't clear, there really should be more of
	 * a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of named constant value declarations <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Constants</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Constants()
	 * @see fr.irisa.espresso.sme.ConstantValue#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe9u5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<ConstantValue> getConstants();
	
	/**
	 * Returns the value of the '<em><b>Constraints</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.ControlflowOperator}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.ControlflowOperator#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Constraints</em>' containment reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of control flow operator: clock constraints, dependence...
	 * <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Constraints</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Constraints()
	 * @see fr.irisa.espresso.sme.ControlflowOperator#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe9-5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<ControlflowOperator> getConstraints();
	
	/**
	 * Returns the value of the '<em><b>Indexes</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Index}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Index#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Indexes</em>' containment reference list isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of index declarations. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Indexes</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Indexes()
	 * @see fr.irisa.espresso.sme.Index#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe-O5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Index> getIndexes();
	
	/**
	 * Returns the value of the '<em><b>Contained Instances</b></em>' containment reference list. The list contents are
	 * of type {@link fr.irisa.espresso.sme.ModelInstance}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.ModelInstance#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Contained Instances</em>' containment reference list isn't clear, there really should
	 * be more of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of model (process) instantiations. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Contained Instances</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_ContainedInstances()
	 * @see fr.irisa.espresso.sme.ModelInstance#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe-e5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<ModelInstance> getContainedInstances();
	
	/**
	 * Returns the value of the '<em><b>Literals</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Literal}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Literal#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Literals</em>' containment reference list isn't clear, there really should be more of
	 * a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of unnamed constant value <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Literals</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Literals()
	 * @see fr.irisa.espresso.sme.Literal#getModelParent
	 * @model opposite="modelParent" containment="true"
	 * @generated
	 */
	EList<Literal> getLiterals();
	
	/**
	 * Returns the value of the '<em><b>Locals</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.Local}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.Local#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Locals</em>' containment reference list isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of local signal declarations <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Locals</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Locals()
	 * @see fr.irisa.espresso.sme.Local#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe-u5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<Local> getLocals();
	
	/**
	 * Returns the value of the '<em><b>Operators</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.DataflowOperator}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.DataflowOperator#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Operators</em>' containment reference list isn't clear, there really should be more of
	 * a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of data flow operator: boolean, arithmetic, merging,
	 * down-sampling... <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Operators</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Operators()
	 * @see fr.irisa.espresso.sme.DataflowOperator#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe--5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<DataflowOperator> getOperators();
	
	/**
	 * Returns the value of the '<em><b>Signal Ref</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.SignalRef}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.SignalRef#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Signal Ref</em>' containment reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of references to signal identifier declared at another
	 * place. The reference must be in the scope of the signal identifier. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Signal Ref</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_SignalRef()
	 * @see fr.irisa.espresso.sme.SignalRef#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe_O5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<SignalRef> getSignalRef();
	
	/**
	 * Returns the value of the '<em><b>Subprocesses</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.SubProcess}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.SubProcess#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Subprocesses</em>' containment reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of nested model (without interface). <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Subprocesses</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Subprocesses()
	 * @see fr.irisa.espresso.sme.SubProcess#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe_e5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<SubProcess> getSubprocesses();
	
	/**
	 * Returns the value of the '<em><b>Types</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.TypeDeclaration}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.TypeDeclaration#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Types</em>' containment reference list isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of type declarations <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Types</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_Types()
	 * @see fr.irisa.espresso.sme.TypeDeclaration#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe_u5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<TypeDeclaration> getTypes();
	
	/**
	 * Returns the value of the '<em><b>Use Module</b></em>' containment reference. It is bidirectional and its opposite
	 * is '{@link fr.irisa.espresso.sme.UseModule#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Use Module</em>' containment reference isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> This block is used to reference all modules included by the
	 * current model. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Use Module</em>' containment reference.
	 * @see #setUseModule(UseModule)
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_UseModule()
	 * @see fr.irisa.espresso.sme.UseModule#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOe_-5WEduz9o2pAin6-g'"
	 * @generated
	 */
	UseModule getUseModule();
	
	/**
	 * Sets the value of the '{@link fr.irisa.espresso.sme.Model#getUseModule <em>Use Module</em>}' containment
	 * reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param value
	 *        the new value of the '<em>Use Module</em>' containment reference.
	 * @see #getUseModule()
	 * @generated
	 */
	void setUseModule(UseModule value);
	
	/**
	 * Returns the value of the '<em><b>Model Refs</b></em>' containment reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.ModelRef}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.ModelRef#getModelParent <em>Model Parent</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Model Refs</em>' containment reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc --> <!-- begin-model-doc --> A list of references to model declaration declared at another
	 * place. The reference must be in the scope of the model declaration. <!-- end-model-doc -->
	 * 
	 * @return the value of the '<em>Model Refs</em>' containment reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getModel_ModelRefs()
	 * @see fr.irisa.espresso.sme.ModelRef#getModelParent
	 * @model opposite="modelParent" containment="true"
	 *        annotation="http://www.topcased.org/uuid uuid='_X5lIABW_EdyjKrSFpqO4YA'"
	 * @generated
	 */
	EList<ModelRef> getModelRefs();
	
} // Model
