/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;

/**
 * <!-- begin-user-doc --> A representation of the literals of the enumeration '<em><b>Model Kind</b></em>', and utility
 * methods for working with them. <!-- end-user-doc --> <!-- begin-model-doc --> This enumeration represents and
 * contains the different classes of process models. <!-- end-model-doc -->
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getModelKind()
 * @model annotation="http://www.topcased.org/uuid uuid='_2B1SQe5WEduz9o2pAin6-g'"
 * @generated
 */
public enum ModelKind implements Enumerator {
	/**
	 * The '<em><b>Action</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #ACTION
	 * @generated
	 * @ordered
	 */
	ACTION_LITERAL(517, "action", "action"),

	/**
	 * The '<em><b>Function</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #FUNCTION
	 * @generated
	 * @ordered
	 */
	FUNCTION_LITERAL(506, "function", "function"),

	/**
	 * The '<em><b>Node</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #NODE
	 * @generated
	 * @ordered
	 */
	NODE_LITERAL(522, "node", "node"),

	/**
	 * The '<em><b>Process</b></em>' literal object. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #PROCESS
	 * @generated
	 * @ordered
	 */
	PROCESS_LITERAL(511, "process", "process");
	
	/**
	 * The '<em><b>Action</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc
	 * --> Actions are processes that are called (activated) at a specific clock. An action (described by a model of
	 * action) has to respect some relations regarding its clocks and dependences: Its tick (which is a clock greater
	 * than the upper bound of the clocks of all the signals designated in the action) is necessarily accessible from
	 * the outside of the action. This tick is the clock designated by the label under the scope of which the action
	 * call is. For the dependence relation, each input of an action precedes each output of that action at the product
	 * (intersection) of their clocks. <!-- end-model-doc -->
	 * 
	 * @see #ACTION_LITERAL
	 * @model name="action" annotation="http://www.topcased.org/uuid uuid='_2B_DQO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				ACTION			= 517;
	
	/**
	 * The '<em><b>Function</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc
	 * --> A function is a particular case of node and has to respect all the relations respected by a node regarding
	 * its clocks and dependences. In addition, all the inputs and outputs of a function must have the same clock. A
	 * function is constant on time and does not produce any side effect. In particular, it cannot contain delay
	 * operators (or other operators derived from delay), that define some memory. <!-- end-model-doc -->
	 * 
	 * @see #FUNCTION_LITERAL
	 * @model name="function" annotation="http://www.topcased.org/uuid uuid='_2CI0QO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				FUNCTION		= 506;
	
	/**
	 * The '<em><b>Node</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc -->
	 * Nodes are essentially endochronous processes. Roughly speaking, an endochronous process knows when it has to read
	 * its inputs, thus it is autonomous when run in a given environment. It may be shown that if the clock relations
	 * associated with a process can be organized as a tree of clocks, the root of the tree representing the most
	 * frequent clock (which is the single greatest clock) of the system, then this process is endochronous. Besides the
	 * property that it is endochronous, a node (described by a model of node) has to respect some relations regarding
	 * its clocks and dependences: Its tick (which is in that case a clock equal to the upper bound of the clocks of all
	 * the signals designated in the node) is necessarily the clock of an input or output of the node. In addition, the
	 * clock of any input or output signal of the node should be considered as explicitly defined (without clock
	 * calculus synthesis). For those input or output signals the clock of which is not explicitly defined, their clock
	 * is fixed to the tick. Thus, there is no synthesis from the clock calculus for the clocks of inputs and outputs,
	 * but only verification. However, there is possibly such a synthesis for the clocks of internal signals. For the
	 * dependence relation, each input of a node precedes each output of that node at the product (intersection) of
	 * their clocks. <!-- end-model-doc -->
	 * 
	 * @see #NODE_LITERAL
	 * @model name="node" annotation="http://www.topcased.org/uuid uuid='_2CbvMO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				NODE			= 522;
	
	/**
	 * The '<em><b>Process</b></em>' literal value. <!-- begin-user-doc --> <!-- end-user-doc --> <!-- begin-model-doc
	 * --> A process (described by a model of process) belongs to the most general class of processes. There are no
	 * required particular relations regarding clocks as well as dependences. It is the job of the compilation (clock
	 * calculus, dependence calculus) to synthesize these relations. <!-- end-model-doc -->
	 * 
	 * @see #PROCESS_LITERAL
	 * @model name="process" annotation="http://www.topcased.org/uuid uuid='_2CuqIO5WEduz9o2pAin6-g'"
	 * @generated
	 * @ordered
	 */
	public static final int				PROCESS			= 511;
	
	/**
	 * An array of all the '<em><b>Model Kind</b></em>' enumerators. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private static final ModelKind[]	VALUES_ARRAY	= new ModelKind[] { ACTION_LITERAL, FUNCTION_LITERAL,
			NODE_LITERAL, PROCESS_LITERAL,				};
	
	/**
	 * A public read-only list of all the '<em><b>Model Kind</b></em>' enumerators. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static final List<ModelKind>	VALUES			= Collections.unmodifiableList(Arrays.asList(VALUES_ARRAY));
	
	/**
	 * Returns the '<em><b>Model Kind</b></em>' literal with the specified literal value. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static ModelKind get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			ModelKind result = VALUES_ARRAY[i];
			if (result.toString().equals(literal)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Model Kind</b></em>' literal with the specified name. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static ModelKind getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			ModelKind result = VALUES_ARRAY[i];
			if (result.getName().equals(name)) {
				return result;
			}
		}
		return null;
	}
	
	/**
	 * Returns the '<em><b>Model Kind</b></em>' literal with the specified integer value. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	public static ModelKind get(int value) {
		switch (value) {
			case ACTION:
				return ACTION_LITERAL;
			case FUNCTION:
				return FUNCTION_LITERAL;
			case NODE:
				return NODE_LITERAL;
			case PROCESS:
				return PROCESS_LITERAL;
		}
		return null;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final int		value;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	name;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private final String	literal;
	
	/**
	 * Only this class can construct instances. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	private ModelKind(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public int getValue() {
		return value;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getLiteral() {
		return literal;
	}
	
	/**
	 * Returns the literal value of the enumerator, which is its string representation. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		return literal;
	}
	
} // ModelKind
