/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme;

import org.eclipse.emf.common.util.EList;

import fr.irisa.espresso.sme.polychrony.DeclarationValue;

/**
 * <!-- begin-user-doc --> A representation of the model object '<em><b>State Base</b></em>'. <!-- end-user-doc -->
 * 
 * <p>
 * The following features are supported:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.StateBase#getStateParent <em>State Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.StateBase#getSrc_WeakTransitions <em>Src Weak Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.StateBase#getSrc_StrongTransitions <em>Src Strong Transitions</em>}</li>
 * </ul>
 * </p>
 * 
 * @see fr.irisa.espresso.sme.SmePackage#getStateBase()
 * @model interface="true" abstract="true" annotation="http://www.topcased.org/uuid uuid='_2LOf_-5WEduz9o2pAin6-g'"
 * @generated
 */
public interface StateBase extends StateOrHistory, Identifier {
	/**
	 * Returns the value of the '<em><b>State Parent</b></em>' container reference. It is bidirectional and its opposite
	 * is '{@link fr.irisa.espresso.sme.CompoundState#getStates <em>States</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>State Parent</em>' container reference isn't clear, there really should be more of a
	 * description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @return the value of the '<em>State Parent</em>' container reference.
	 * @see #setStateParent(CompoundState)
	 * @see fr.irisa.espresso.sme.SmePackage#getStateBase_StateParent()
	 * @see fr.irisa.espresso.sme.CompoundState#getStates
	 * @model opposite="states" resolveProxies="false" transient="false" ordered="false"
	 *        annotation="http://www.topcased.org/uuid uuid='_2LOgAO5WEduz9o2pAin6-g'"
	 * @generated
	 */
	CompoundState getStateParent();
	
	/**
	 * Sets the value of the '{@link fr.irisa.espresso.sme.StateBase#getStateParent <em>State Parent</em>}' container
	 * reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param value
	 *        the new value of the '<em>State Parent</em>' container reference.
	 * @see #getStateParent()
	 * @generated
	 */
	void setStateParent(CompoundState value);
	
	/**
	 * Returns the value of the '<em><b>Src Weak Transitions</b></em>' reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.WeakTransition}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.WeakTransition#getSrc <em>Src</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Src Weak Transitions</em>' reference list isn't clear, there really should be more of
	 * a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @return the value of the '<em>Src Weak Transitions</em>' reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getStateBase_Src_WeakTransitions()
	 * @see fr.irisa.espresso.sme.WeakTransition#getSrc
	 * @model opposite="src" annotation="http://www.topcased.org/uuid uuid='_2LOgAe5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<WeakTransition> getSrc_WeakTransitions();
	
	/**
	 * Returns the value of the '<em><b>Src Strong Transitions</b></em>' reference list. The list contents are of type
	 * {@link fr.irisa.espresso.sme.StrongTransition}. It is bidirectional and its opposite is '
	 * {@link fr.irisa.espresso.sme.StrongTransition#getSrc <em>Src</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Src Strong Transitions</em>' reference list isn't clear, there really should be more
	 * of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @return the value of the '<em>Src Strong Transitions</em>' reference list.
	 * @see fr.irisa.espresso.sme.SmePackage#getStateBase_Src_StrongTransitions()
	 * @see fr.irisa.espresso.sme.StrongTransition#getSrc
	 * @model opposite="src" annotation="http://www.topcased.org/uuid uuid='_2LOgAu5WEduz9o2pAin6-g'"
	 * @generated
	 */
	EList<StrongTransition> getSrc_StrongTransitions();
	
	/**
	 * This method creates the AST of the StateBase element and add it to the bodyTree.
	 * 
	 * @param body
	 *        node referring to a list in which to add the specification of the StateBase element
	 * @param rootDeclaration
	 *        corresponds to the AST of the declaration of the root Automaton.
	 * @param reinitSignalName
	 *        designate the name of the reinitialization signal. If the value is null, it means that the StateBase
	 *        element is the root of the Automaton.
	 * @return a reference of the AST representing the Automaton.
	 */
	long buildAutomaton(long body, DeclarationValue rootDeclaration, String reinitSignalName);
	
} // StateBase
