/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.AndState;
import fr.irisa.espresso.sme.CompoundState;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.History;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.Local;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.StateBase;
import fr.irisa.espresso.sme.StrongTransition;
import fr.irisa.espresso.sme.WeakTransition;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.DeclarationValue;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.SignalTreeBuilting;
import fr.irisa.espresso.sme.polychrony.SignalTreeExploring;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>And State</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getDst_StrongTransitions <em>Dst Strong Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getDst_WeakTransitions <em>Dst Weak Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getStateParent <em>State Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getSrc_WeakTransitions <em>Src Weak Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getSrc_StrongTransitions <em>Src Strong Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getStates <em>States</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getHistory <em>History</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AndStateImpl#getLocals <em>Locals</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class AndStateImpl extends EObjectImpl implements AndState {
	/**
	 * The cached value of the '{@link #getDst_StrongTransitions() <em>Dst Strong Transitions</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_StrongTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<StrongTransition>	dst_StrongTransitions;
	
	/**
	 * The cached value of the '{@link #getDst_WeakTransitions() <em>Dst Weak Transitions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_WeakTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<WeakTransition>		dst_WeakTransitions;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String		NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String					name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String		COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String					comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc_WeakTransitions() <em>Src Weak Transitions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_WeakTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<WeakTransition>		src_WeakTransitions;
	
	/**
	 * The cached value of the '{@link #getSrc_StrongTransitions() <em>Src Strong Transitions</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_StrongTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<StrongTransition>	src_StrongTransitions;
	
	/**
	 * The cached value of the '{@link #getStates() <em>States</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getStates()
	 * @generated
	 * @ordered
	 */
	protected EList<StateBase>			states;
	
	/**
	 * The cached value of the '{@link #getHistory() <em>History</em>}' containment reference. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getHistory()
	 * @generated
	 * @ordered
	 */
	protected History					history;
	
	/**
	 * The cached value of the '{@link #getLocals() <em>Locals</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getLocals()
	 * @generated
	 * @ordered
	 */
	protected EList<Local>				locals;
	
	/**
	 * This string is used to build the name of the different signals used to control the automaton.
	 * 
	 * @generated NOT
	 */
	private String						prefix				= null;
	/** @generated NOT */
	PKTrees								API					= PkPlugin.getTreeAPI();
	/**
	 * This integer is used to count the number of AndState and the number is used in the prefix
	 * 
	 * @generated NOT
	 */
	private static int					countAndState		= 0;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AndStateImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.AND_STATE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<StrongTransition> getDst_StrongTransitions() {
		if (dst_StrongTransitions == null) {
			dst_StrongTransitions = new EObjectWithInverseResolvingEList<StrongTransition>(StrongTransition.class,
				this, SmePackage.AND_STATE__DST_STRONG_TRANSITIONS, SmePackage.STRONG_TRANSITION__DST);
		}
		return dst_StrongTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<WeakTransition> getDst_WeakTransitions() {
		if (dst_WeakTransitions == null) {
			dst_WeakTransitions = new EObjectWithInverseResolvingEList<WeakTransition>(WeakTransition.class, this,
				SmePackage.AND_STATE__DST_WEAK_TRANSITIONS, SmePackage.WEAK_TRANSITION__DST);
		}
		return dst_WeakTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AND_STATE__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AND_STATE__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CompoundState getStateParent() {
		if (eContainerFeatureID != SmePackage.AND_STATE__STATE_PARENT)
			return null;
		return (CompoundState) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetStateParent(CompoundState newStateParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newStateParent, SmePackage.AND_STATE__STATE_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setStateParent(CompoundState newStateParent) {
		if (newStateParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.AND_STATE__STATE_PARENT && newStateParent != null)) {
			if (EcoreUtil.isAncestor(this, newStateParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newStateParent != null)
				msgs = ((InternalEObject) newStateParent).eInverseAdd(this, SmePackage.COMPOUND_STATE__STATES,
					CompoundState.class, msgs);
			msgs = basicSetStateParent(newStateParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AND_STATE__STATE_PARENT, newStateParent,
				newStateParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<WeakTransition> getSrc_WeakTransitions() {
		if (src_WeakTransitions == null) {
			src_WeakTransitions = new EObjectWithInverseResolvingEList<WeakTransition>(WeakTransition.class, this,
				SmePackage.AND_STATE__SRC_WEAK_TRANSITIONS, SmePackage.WEAK_TRANSITION__SRC);
		}
		return src_WeakTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<StrongTransition> getSrc_StrongTransitions() {
		if (src_StrongTransitions == null) {
			src_StrongTransitions = new EObjectWithInverseResolvingEList<StrongTransition>(StrongTransition.class,
				this, SmePackage.AND_STATE__SRC_STRONG_TRANSITIONS, SmePackage.STRONG_TRANSITION__SRC);
		}
		return src_StrongTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<StateBase> getStates() {
		if (states == null) {
			states = new EObjectContainmentWithInverseEList<StateBase>(StateBase.class, this,
				SmePackage.AND_STATE__STATES, SmePackage.STATE_BASE__STATE_PARENT);
		}
		return states;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public History getHistory() {
		return history;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetHistory(History newHistory, NotificationChain msgs) {
		History oldHistory = history;
		history = newHistory;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.AND_STATE__HISTORY, oldHistory, newHistory);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setHistory(History newHistory) {
		if (newHistory != history) {
			NotificationChain msgs = null;
			if (history != null)
				msgs = ((InternalEObject) history).eInverseRemove(this, SmePackage.HISTORY__PARENT_STATE,
					History.class, msgs);
			if (newHistory != null)
				msgs = ((InternalEObject) newHistory).eInverseAdd(this, SmePackage.HISTORY__PARENT_STATE,
					History.class, msgs);
			msgs = basicSetHistory(newHistory, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AND_STATE__HISTORY, newHistory, newHistory));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Local> getLocals() {
		if (locals == null) {
			locals = new EObjectContainmentWithInverseEList<Local>(Local.class, this, SmePackage.AND_STATE__LOCALS,
				SmePackage.LOCAL__STATE_PARENT);
		}
		return locals;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<ConstantOrRef> getVisibleConstants() {
		if (getStateParent() != null)
			return getStateParent().getVisibleConstants();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		if (getStateParent() != null)
			return getStateParent().getVisibleModelProcesses();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		EList<SignalOrRef> list = new BasicEList<SignalOrRef>(this.getLocals());
		
		if (getStateParent() != null)
			list.addAll(getStateParent().getVisibleSignals());
		
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<Identifier> getVisibleTypes() {
		if (getStateParent() != null)
			return getStateParent().getVisibleTypes();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.AND_STATE__DST_STRONG_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_StrongTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.AND_STATE__DST_WEAK_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_WeakTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.AND_STATE__STATE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetStateParent((CompoundState) otherEnd, msgs);
			case SmePackage.AND_STATE__SRC_WEAK_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_WeakTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.AND_STATE__SRC_STRONG_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_StrongTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.AND_STATE__STATES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getStates()).basicAdd(otherEnd, msgs);
			case SmePackage.AND_STATE__HISTORY:
				if (history != null)
					msgs = ((InternalEObject) history).eInverseRemove(this, EOPPOSITE_FEATURE_BASE
							- SmePackage.AND_STATE__HISTORY, null, msgs);
				return basicSetHistory((History) otherEnd, msgs);
			case SmePackage.AND_STATE__LOCALS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getLocals()).basicAdd(otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.AND_STATE__DST_STRONG_TRANSITIONS:
				return ((InternalEList<?>) getDst_StrongTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.AND_STATE__DST_WEAK_TRANSITIONS:
				return ((InternalEList<?>) getDst_WeakTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.AND_STATE__STATE_PARENT:
				return basicSetStateParent(null, msgs);
			case SmePackage.AND_STATE__SRC_WEAK_TRANSITIONS:
				return ((InternalEList<?>) getSrc_WeakTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.AND_STATE__SRC_STRONG_TRANSITIONS:
				return ((InternalEList<?>) getSrc_StrongTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.AND_STATE__STATES:
				return ((InternalEList<?>) getStates()).basicRemove(otherEnd, msgs);
			case SmePackage.AND_STATE__HISTORY:
				return basicSetHistory(null, msgs);
			case SmePackage.AND_STATE__LOCALS:
				return ((InternalEList<?>) getLocals()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.AND_STATE__STATE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.COMPOUND_STATE__STATES,
					CompoundState.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.AND_STATE__DST_STRONG_TRANSITIONS:
				return getDst_StrongTransitions();
			case SmePackage.AND_STATE__DST_WEAK_TRANSITIONS:
				return getDst_WeakTransitions();
			case SmePackage.AND_STATE__NAME:
				return getName();
			case SmePackage.AND_STATE__COMMENT:
				return getComment();
			case SmePackage.AND_STATE__STATE_PARENT:
				return getStateParent();
			case SmePackage.AND_STATE__SRC_WEAK_TRANSITIONS:
				return getSrc_WeakTransitions();
			case SmePackage.AND_STATE__SRC_STRONG_TRANSITIONS:
				return getSrc_StrongTransitions();
			case SmePackage.AND_STATE__STATES:
				return getStates();
			case SmePackage.AND_STATE__HISTORY:
				return getHistory();
			case SmePackage.AND_STATE__LOCALS:
				return getLocals();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.AND_STATE__DST_STRONG_TRANSITIONS:
				getDst_StrongTransitions().clear();
				getDst_StrongTransitions().addAll((Collection<? extends StrongTransition>) newValue);
				return;
			case SmePackage.AND_STATE__DST_WEAK_TRANSITIONS:
				getDst_WeakTransitions().clear();
				getDst_WeakTransitions().addAll((Collection<? extends WeakTransition>) newValue);
				return;
			case SmePackage.AND_STATE__NAME:
				setName((String) newValue);
				return;
			case SmePackage.AND_STATE__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.AND_STATE__STATE_PARENT:
				setStateParent((CompoundState) newValue);
				return;
			case SmePackage.AND_STATE__SRC_WEAK_TRANSITIONS:
				getSrc_WeakTransitions().clear();
				getSrc_WeakTransitions().addAll((Collection<? extends WeakTransition>) newValue);
				return;
			case SmePackage.AND_STATE__SRC_STRONG_TRANSITIONS:
				getSrc_StrongTransitions().clear();
				getSrc_StrongTransitions().addAll((Collection<? extends StrongTransition>) newValue);
				return;
			case SmePackage.AND_STATE__STATES:
				getStates().clear();
				getStates().addAll((Collection<? extends StateBase>) newValue);
				return;
			case SmePackage.AND_STATE__HISTORY:
				setHistory((History) newValue);
				return;
			case SmePackage.AND_STATE__LOCALS:
				getLocals().clear();
				getLocals().addAll((Collection<? extends Local>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.AND_STATE__DST_STRONG_TRANSITIONS:
				getDst_StrongTransitions().clear();
				return;
			case SmePackage.AND_STATE__DST_WEAK_TRANSITIONS:
				getDst_WeakTransitions().clear();
				return;
			case SmePackage.AND_STATE__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.AND_STATE__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.AND_STATE__STATE_PARENT:
				setStateParent((CompoundState) null);
				return;
			case SmePackage.AND_STATE__SRC_WEAK_TRANSITIONS:
				getSrc_WeakTransitions().clear();
				return;
			case SmePackage.AND_STATE__SRC_STRONG_TRANSITIONS:
				getSrc_StrongTransitions().clear();
				return;
			case SmePackage.AND_STATE__STATES:
				getStates().clear();
				return;
			case SmePackage.AND_STATE__HISTORY:
				setHistory((History) null);
				return;
			case SmePackage.AND_STATE__LOCALS:
				getLocals().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.AND_STATE__DST_STRONG_TRANSITIONS:
				return dst_StrongTransitions != null && !dst_StrongTransitions.isEmpty();
			case SmePackage.AND_STATE__DST_WEAK_TRANSITIONS:
				return dst_WeakTransitions != null && !dst_WeakTransitions.isEmpty();
			case SmePackage.AND_STATE__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.AND_STATE__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.AND_STATE__STATE_PARENT:
				return getStateParent() != null;
			case SmePackage.AND_STATE__SRC_WEAK_TRANSITIONS:
				return src_WeakTransitions != null && !src_WeakTransitions.isEmpty();
			case SmePackage.AND_STATE__SRC_STRONG_TRANSITIONS:
				return src_StrongTransitions != null && !src_StrongTransitions.isEmpty();
			case SmePackage.AND_STATE__STATES:
				return states != null && !states.isEmpty();
			case SmePackage.AND_STATE__HISTORY:
				return history != null;
			case SmePackage.AND_STATE__LOCALS:
				return locals != null && !locals.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.AND_STATE__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.AND_STATE__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == SignalContainer.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.AND_STATE__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.AND_STATE__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == SignalContainer.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * 
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			result.append(name);
			result.append("::(| ... |)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getParent()
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getStateParent();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * 
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		// This method must not be called - use buildAutomaton
		throw new UnsupportedOperationException();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE)
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
			else
				result = idNode;
		}
		else
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME + name, this);
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
	
	/**
	 * @return the name of the associated object (process) in Signal.
	 * @generated NOT
	 */
	private long getProcessName() {
		return API.string2ident("AndState_" + getName());
	}
	
	/**
	 * @see fr.irisa.espresso.sme.StateBase#buildAutomaton(fr.irisa.espresso.sme.polychrony.DeclarationValue,
	 *      java.lang.String)
	 * @generated NOT
	 */
	public long buildAutomaton(long bodyTree, DeclarationValue rootDeclaration, String reinitSignalName) {
		long mp, declTree, body;
		
		Utils.TRACE("buildAutomaton : AndStateImpl BEGIN: " + getName() + " start");
		
		SignalTreeBuilting TBAPI = PkPlugin.getTreeBuilder(); // Signal Tree API
		SignalTreeExploring TEAPI = PkPlugin.getTreeExplorer(); // Signal Tree explorer API
		
		mp = TBAPI.n_mkSkeletonModel(getProcessName(), SignalOperators.opprocess);
		/*
		 * long theTick = PkPlugin.getParser().parseString(Phyla.ph_dec_s, "event" + TICKNAME);
		 * TBAPI.n_addDeclarationToInterface(mp, theTick, 2);
		 */

		if (getStates().size() < 2)
			PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.AND_STATE_SUB_STATES_NUMBER, this);
		
		declTree = API.makeEmptyList(SignalOperators.opldecl);
		long labels = API.makeEmptyList(SignalOperators.oplident);
		body = API.makeEmptyList(SignalOperators.oplfeed);
		
		// Creation of a composition between the different sub-states
		for (StateBase sb : states) {
			try { // We get the result of the automaton building a model
				long subState = sb.buildAutomaton(PKTrees.ERROR_NODE, rootDeclaration, reinitSignalName);
				declTree = API.post(declTree, subState);
				long def = TBAPI.mkReferenceToModel(TEAPI.getModelName(subState));
				long idNode = sb.getAST(null);
				if (idNode != PKTrees.ERROR_NODE) {
					def = API.makeBinary(SignalOperators.oppetiq, idNode, def);
					labels = API.post(labels, TBAPI.copy(idNode, false));
				}
				body = API.post(body, def);
			}
			catch (CyclicDefinitionException cde) {
				// Normally not thrown
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		if (!API.isEmpty(labels))
			declTree = API.post(declTree, API.makeUnary(SignalOperators.oplab, labels));
		// If the composition list is empty, we return a null process
		if (API.isEmpty(body))
			body = API.makeNullary(SignalOperators.opnullproc);
		// Get the declarations of the local signals and of the label for each
		// process
		declTree = Utils.makeDeclarationList(getLocals(), declTree);
		declTree = Utils.makeDeclarationList(getStates(), declTree);
		
		TBAPI.addExpressionOfProcessToaModel(mp, body);
		if (!API.isEmpty(declTree))
			TBAPI.setLocalDeclarations(mp, declTree);
		Utils.TRACE("AndStateImpl END : " + getName() + " done");
		return mp;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.irisa.espresso.sme.CompoundState#getPrefix()
	 */
	public String getPrefix() {
		if (prefix == null) {
			String tmp = getName();
			if (tmp != null)
				tmp = tmp.replaceAll(" ", "_");
			prefix = "__" + tmp + "_AS_" + countAndState++;
		}
		return prefix;
	}
	
} // AndStateImpl
