/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EClass;

import fr.irisa.espresso.sme.AssertionClockSpeedOperator;
import fr.irisa.espresso.sme.ClockSpeedOpKind;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Assertion Clock Speed Operator</b></em>'. <!--
 * end-user-doc -->
 * <p>
 * </p>
 * 
 * @generated
 */
public class AssertionClockSpeedOperatorImpl extends ClockSpeedOperatorImpl implements AssertionClockSpeedOperator {
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AssertionClockSpeedOperatorImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.ASSERTION_CLOCK_SPEED_OPERATOR;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		Set<SignalElement> path = new HashSet<SignalElement>();
		long result;
		PKTrees API = PkPlugin.getTreeAPI();
		
		if (operator.equals(ClockSpeedOpKind.SLOWER_LITERAL))
			result = API.makeEmptyList(SignalOperators.oplcrohor);
		else
			// ClockSpeedOpKind.FASTER_LITERAL
			result = API.makeEmptyList(SignalOperators.opldechor);
		
		int size = 0;
		for (InputPort in : getIns()) {
			try {
				long node = in.getAST(path);
				if (node != PKTrees.ERROR_NODE) {
					result = API.post(result, node);
					size++;
				}
			}
			catch (CyclicDefinitionException cde) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		
		if (size > 1)
			bodyTree = API.post(bodyTree, API.makeUnary(SignalOperators.oppassert, API.makeUnary(
				SignalOperators.oplfeed, result)));
		
		return bodyTree;
	}
	
	/**
	 * This method is not supported: returns an UnsupportedOperationException
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * No declaration for this operator. declTree is returned unmodified
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
	
} // AssertionClockSpeedOperatorImpl
