/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ClockedExpr;
import fr.irisa.espresso.sme.CompoundState;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.DependenceInOut;
import fr.irisa.espresso.sme.History;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.Local;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.StateBase;
import fr.irisa.espresso.sme.StateOrHistory;
import fr.irisa.espresso.sme.StrongTransition;
import fr.irisa.espresso.sme.Transition;
import fr.irisa.espresso.sme.WeakTransition;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.DeclarationValue;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.SignalTreeBuilting;
import fr.irisa.espresso.sme.polychrony.SignalTreeExploring;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Automaton</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getDst_StrongTransitions <em>Dst Strong Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getDst_WeakTransitions <em>Dst Weak Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getStateParent <em>State Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getSrc_WeakTransitions <em>Src Weak Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getSrc_StrongTransitions <em>Src Strong Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getStates <em>States</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getHistory <em>History</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getLocals <em>Locals</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getSrc_ClockRelations <em>Src Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getDst_ClockRelations <em>Dst Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getSrc_Dependences <em>Src Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getDst_Dependences <em>Dst Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getInitState <em>Init State</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.AutomatonImpl#getTransitions <em>Transitions</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class AutomatonImpl extends EObjectImpl implements Automaton {
	/**
	 * The cached value of the '{@link #getDst_StrongTransitions() <em>Dst Strong Transitions</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_StrongTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<StrongTransition>	dst_StrongTransitions;
	
	/**
	 * The cached value of the '{@link #getDst_WeakTransitions() <em>Dst Weak Transitions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_WeakTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<WeakTransition>		dst_WeakTransitions;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String		NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String					name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String		COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String					comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc_WeakTransitions() <em>Src Weak Transitions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_WeakTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<WeakTransition>		src_WeakTransitions;
	
	/**
	 * The cached value of the '{@link #getSrc_StrongTransitions() <em>Src Strong Transitions</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_StrongTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<StrongTransition>	src_StrongTransitions;
	
	/**
	 * The cached value of the '{@link #getStates() <em>States</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getStates()
	 * @generated
	 * @ordered
	 */
	protected EList<StateBase>			states;
	
	/**
	 * The cached value of the '{@link #getHistory() <em>History</em>}' containment reference. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getHistory()
	 * @generated
	 * @ordered
	 */
	protected History					history;
	
	/**
	 * The cached value of the '{@link #getLocals() <em>Locals</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getLocals()
	 * @generated
	 * @ordered
	 */
	protected EList<Local>				locals;
	
	/**
	 * The cached value of the '{@link #getSrc_ClockRelations() <em>Src Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		src_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getDst_ClockRelations() <em>Dst Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		dst_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getSrc_Dependences() <em>Src Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>			src_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_Dependences() <em>Dst Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>			dst_Dependences;
	
	/**
	 * The cached value of the '{@link #getInitState() <em>Init State</em>}' reference. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getInitState()
	 * @generated
	 * @ordered
	 */
	protected StateBase					initState;
	
	/**
	 * The cached value of the '{@link #getTransitions() <em>Transitions</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<Transition>			transitions;
	
	/*
	 * Some constant strings used to suffix the prefix to build signal names
	 */
	private final static String			CURRENT				= "_cUrReNtStAtE";
	private final static String			NEXT				= "_nExTsTaTe";
	private final static String			PNEXT				= "_zNeXtStAtE";
	private final static String			PREVIOUS			= "_pReViOuSsTaTe";
	private final static String			REINIT				= "_rEiNiT";
	private final static String			NEXTHISTORY			= "_nExTuSeHiStOrY";
	private final static String			HISTORY				= "_uSeHisToRy";
	private final static String			TYPE				= "_nAmEsTaTeTyPe";
	private final static String			DELAY				= "_dELaY";
	private final static String			TICKNAME			= "_trigger";					// before _Tick.
	/* @generated NOT */
	final PKTrees						API					= PkPlugin.getTreeAPI();		// Tree API low level
	/* @generated NOT */
	final SignalTreeBuilting			TBAPI				= PkPlugin.getTreeBuilder();	// Signal Tree API
	/* @generated NOT */
	final SignalTreeExploring			TEAPI				= PkPlugin.getTreeExplorer();	// Signal Tree explorer API
																							
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected AutomatonImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.AUTOMATON;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<StrongTransition> getDst_StrongTransitions() {
		if (dst_StrongTransitions == null) {
			dst_StrongTransitions = new EObjectWithInverseResolvingEList<StrongTransition>(StrongTransition.class,
				this, SmePackage.AUTOMATON__DST_STRONG_TRANSITIONS, SmePackage.STRONG_TRANSITION__DST);
		}
		return dst_StrongTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<WeakTransition> getDst_WeakTransitions() {
		if (dst_WeakTransitions == null) {
			dst_WeakTransitions = new EObjectWithInverseResolvingEList<WeakTransition>(WeakTransition.class, this,
				SmePackage.AUTOMATON__DST_WEAK_TRANSITIONS, SmePackage.WEAK_TRANSITION__DST);
		}
		return dst_WeakTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AUTOMATON__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AUTOMATON__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CompoundState getStateParent() {
		if (eContainerFeatureID != SmePackage.AUTOMATON__STATE_PARENT)
			return null;
		return (CompoundState) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetStateParent(CompoundState newStateParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newStateParent, SmePackage.AUTOMATON__STATE_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setStateParent(CompoundState newStateParent) {
		if (newStateParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.AUTOMATON__STATE_PARENT && newStateParent != null)) {
			if (EcoreUtil.isAncestor(this, newStateParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newStateParent != null)
				msgs = ((InternalEObject) newStateParent).eInverseAdd(this, SmePackage.COMPOUND_STATE__STATES,
					CompoundState.class, msgs);
			msgs = basicSetStateParent(newStateParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AUTOMATON__STATE_PARENT, newStateParent,
				newStateParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<WeakTransition> getSrc_WeakTransitions() {
		if (src_WeakTransitions == null) {
			src_WeakTransitions = new EObjectWithInverseResolvingEList<WeakTransition>(WeakTransition.class, this,
				SmePackage.AUTOMATON__SRC_WEAK_TRANSITIONS, SmePackage.WEAK_TRANSITION__SRC);
		}
		return src_WeakTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<StrongTransition> getSrc_StrongTransitions() {
		if (src_StrongTransitions == null) {
			src_StrongTransitions = new EObjectWithInverseResolvingEList<StrongTransition>(StrongTransition.class,
				this, SmePackage.AUTOMATON__SRC_STRONG_TRANSITIONS, SmePackage.STRONG_TRANSITION__SRC);
		}
		return src_StrongTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<StateBase> getStates() {
		if (states == null) {
			states = new EObjectContainmentWithInverseEList<StateBase>(StateBase.class, this,
				SmePackage.AUTOMATON__STATES, SmePackage.STATE_BASE__STATE_PARENT);
		}
		return states;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public History getHistory() {
		return history;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetHistory(History newHistory, NotificationChain msgs) {
		History oldHistory = history;
		history = newHistory;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.AUTOMATON__HISTORY, oldHistory, newHistory);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setHistory(History newHistory) {
		if (newHistory != history) {
			NotificationChain msgs = null;
			if (history != null)
				msgs = ((InternalEObject) history).eInverseRemove(this, SmePackage.HISTORY__PARENT_STATE,
					History.class, msgs);
			if (newHistory != null)
				msgs = ((InternalEObject) newHistory).eInverseAdd(this, SmePackage.HISTORY__PARENT_STATE,
					History.class, msgs);
			msgs = basicSetHistory(newHistory, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AUTOMATON__HISTORY, newHistory, newHistory));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Local> getLocals() {
		if (locals == null) {
			locals = new EObjectContainmentWithInverseEList<Local>(Local.class, this, SmePackage.AUTOMATON__LOCALS,
				SmePackage.LOCAL__STATE_PARENT);
		}
		return locals;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getSrc_ClockRelations() {
		if (src_ClockRelations == null) {
			src_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__SRC);
		}
		return src_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getDst_ClockRelations() {
		if (dst_ClockRelations == null) {
			dst_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.AUTOMATON__DST_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__DST);
		}
		return dst_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getSrc_Dependences() {
		if (src_Dependences == null) {
			src_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.AUTOMATON__SRC_DEPENDENCES, SmePackage.DEPENDENCE__SRC);
		}
		return src_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getDst_Dependences() {
		if (dst_Dependences == null) {
			dst_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.AUTOMATON__DST_DEPENDENCES, SmePackage.DEPENDENCE__DST);
		}
		return dst_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.AUTOMATON__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.AUTOMATON__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.AUTOMATON__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__AUTOMATA, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AUTOMATON__MODEL_PARENT, newModelParent,
				newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public StateBase getInitState() {
		if (initState != null && initState.eIsProxy()) {
			InternalEObject oldInitState = (InternalEObject) initState;
			initState = (StateBase) eResolveProxy(oldInitState);
			if (initState != oldInitState) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.AUTOMATON__INIT_STATE,
						oldInitState, initState));
			}
		}
		return initState;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public StateBase basicGetInitState() {
		return initState;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setInitState(StateBase newInitState) {
		StateBase oldInitState = initState;
		initState = newInitState;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.AUTOMATON__INIT_STATE, oldInitState,
				initState));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Transition> getTransitions() {
		if (transitions == null)
			transitions = new EObjectContainmentWithInverseEList<Transition>(Transition.class, this,
				SmePackage.AUTOMATON__TRANSITIONS, SmePackage.TRANSITION__AUTOMATON_PARENT);
		return transitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<ConstantOrRef> getVisibleConstants() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleConstants();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleModelProcesses();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		EList<SignalOrRef> list = new BasicEList<SignalOrRef>(this.getLocals());
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleSignals());
		
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<Identifier> getVisibleTypes() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleTypes();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		SignalContainer parent = getModelParent();
		if (parent != null)
			return parent;
		else
			return getStateParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.AUTOMATON__DST_STRONG_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_StrongTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.AUTOMATON__DST_WEAK_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_WeakTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.AUTOMATON__STATE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetStateParent((CompoundState) otherEnd, msgs);
			case SmePackage.AUTOMATON__SRC_WEAK_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_WeakTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.AUTOMATON__SRC_STRONG_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_StrongTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.AUTOMATON__STATES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getStates()).basicAdd(otherEnd, msgs);
			case SmePackage.AUTOMATON__HISTORY:
				if (history != null)
					msgs = ((InternalEObject) history).eInverseRemove(this, EOPPOSITE_FEATURE_BASE
							- SmePackage.AUTOMATON__HISTORY, null, msgs);
				return basicSetHistory((History) otherEnd, msgs);
			case SmePackage.AUTOMATON__LOCALS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getLocals()).basicAdd(otherEnd, msgs);
			case SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.AUTOMATON__DST_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.AUTOMATON__SRC_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.AUTOMATON__DST_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.AUTOMATON__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
			case SmePackage.AUTOMATON__TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getTransitions()).basicAdd(otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.AUTOMATON__DST_STRONG_TRANSITIONS:
				return ((InternalEList<?>) getDst_StrongTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__DST_WEAK_TRANSITIONS:
				return ((InternalEList<?>) getDst_WeakTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__STATE_PARENT:
				return basicSetStateParent(null, msgs);
			case SmePackage.AUTOMATON__SRC_WEAK_TRANSITIONS:
				return ((InternalEList<?>) getSrc_WeakTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__SRC_STRONG_TRANSITIONS:
				return ((InternalEList<?>) getSrc_StrongTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__STATES:
				return ((InternalEList<?>) getStates()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__HISTORY:
				return basicSetHistory(null, msgs);
			case SmePackage.AUTOMATON__LOCALS:
				return ((InternalEList<?>) getLocals()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS:
				return ((InternalEList<?>) getSrc_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__DST_CLOCK_RELATIONS:
				return ((InternalEList<?>) getDst_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__SRC_DEPENDENCES:
				return ((InternalEList<?>) getSrc_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__DST_DEPENDENCES:
				return ((InternalEList<?>) getDst_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.AUTOMATON__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
			case SmePackage.AUTOMATON__TRANSITIONS:
				return ((InternalEList<?>) getTransitions()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.AUTOMATON__STATE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.COMPOUND_STATE__STATES,
					CompoundState.class, msgs);
			case SmePackage.AUTOMATON__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__AUTOMATA, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.AUTOMATON__DST_STRONG_TRANSITIONS:
				return getDst_StrongTransitions();
			case SmePackage.AUTOMATON__DST_WEAK_TRANSITIONS:
				return getDst_WeakTransitions();
			case SmePackage.AUTOMATON__NAME:
				return getName();
			case SmePackage.AUTOMATON__COMMENT:
				return getComment();
			case SmePackage.AUTOMATON__STATE_PARENT:
				return getStateParent();
			case SmePackage.AUTOMATON__SRC_WEAK_TRANSITIONS:
				return getSrc_WeakTransitions();
			case SmePackage.AUTOMATON__SRC_STRONG_TRANSITIONS:
				return getSrc_StrongTransitions();
			case SmePackage.AUTOMATON__STATES:
				return getStates();
			case SmePackage.AUTOMATON__HISTORY:
				return getHistory();
			case SmePackage.AUTOMATON__LOCALS:
				return getLocals();
			case SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS:
				return getSrc_ClockRelations();
			case SmePackage.AUTOMATON__DST_CLOCK_RELATIONS:
				return getDst_ClockRelations();
			case SmePackage.AUTOMATON__SRC_DEPENDENCES:
				return getSrc_Dependences();
			case SmePackage.AUTOMATON__DST_DEPENDENCES:
				return getDst_Dependences();
			case SmePackage.AUTOMATON__MODEL_PARENT:
				return getModelParent();
			case SmePackage.AUTOMATON__INIT_STATE:
				if (resolve)
					return getInitState();
				return basicGetInitState();
			case SmePackage.AUTOMATON__TRANSITIONS:
				return getTransitions();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.AUTOMATON__DST_STRONG_TRANSITIONS:
				getDst_StrongTransitions().clear();
				getDst_StrongTransitions().addAll((Collection<? extends StrongTransition>) newValue);
				return;
			case SmePackage.AUTOMATON__DST_WEAK_TRANSITIONS:
				getDst_WeakTransitions().clear();
				getDst_WeakTransitions().addAll((Collection<? extends WeakTransition>) newValue);
				return;
			case SmePackage.AUTOMATON__NAME:
				setName((String) newValue);
				return;
			case SmePackage.AUTOMATON__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.AUTOMATON__STATE_PARENT:
				setStateParent((CompoundState) newValue);
				return;
			case SmePackage.AUTOMATON__SRC_WEAK_TRANSITIONS:
				getSrc_WeakTransitions().clear();
				getSrc_WeakTransitions().addAll((Collection<? extends WeakTransition>) newValue);
				return;
			case SmePackage.AUTOMATON__SRC_STRONG_TRANSITIONS:
				getSrc_StrongTransitions().clear();
				getSrc_StrongTransitions().addAll((Collection<? extends StrongTransition>) newValue);
				return;
			case SmePackage.AUTOMATON__STATES:
				getStates().clear();
				getStates().addAll((Collection<? extends StateBase>) newValue);
				return;
			case SmePackage.AUTOMATON__HISTORY:
				setHistory((History) newValue);
				return;
			case SmePackage.AUTOMATON__LOCALS:
				getLocals().clear();
				getLocals().addAll((Collection<? extends Local>) newValue);
				return;
			case SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				getSrc_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.AUTOMATON__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				getDst_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.AUTOMATON__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				getSrc_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.AUTOMATON__DST_DEPENDENCES:
				getDst_Dependences().clear();
				getDst_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.AUTOMATON__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.AUTOMATON__INIT_STATE:
				setInitState((StateBase) newValue);
				return;
			case SmePackage.AUTOMATON__TRANSITIONS:
				getTransitions().clear();
				getTransitions().addAll((Collection<? extends Transition>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.AUTOMATON__DST_STRONG_TRANSITIONS:
				getDst_StrongTransitions().clear();
				return;
			case SmePackage.AUTOMATON__DST_WEAK_TRANSITIONS:
				getDst_WeakTransitions().clear();
				return;
			case SmePackage.AUTOMATON__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.AUTOMATON__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.AUTOMATON__STATE_PARENT:
				setStateParent((CompoundState) null);
				return;
			case SmePackage.AUTOMATON__SRC_WEAK_TRANSITIONS:
				getSrc_WeakTransitions().clear();
				return;
			case SmePackage.AUTOMATON__SRC_STRONG_TRANSITIONS:
				getSrc_StrongTransitions().clear();
				return;
			case SmePackage.AUTOMATON__STATES:
				getStates().clear();
				return;
			case SmePackage.AUTOMATON__HISTORY:
				setHistory((History) null);
				return;
			case SmePackage.AUTOMATON__LOCALS:
				getLocals().clear();
				return;
			case SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				return;
			case SmePackage.AUTOMATON__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				return;
			case SmePackage.AUTOMATON__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				return;
			case SmePackage.AUTOMATON__DST_DEPENDENCES:
				getDst_Dependences().clear();
				return;
			case SmePackage.AUTOMATON__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.AUTOMATON__INIT_STATE:
				setInitState((StateBase) null);
				return;
			case SmePackage.AUTOMATON__TRANSITIONS:
				getTransitions().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.AUTOMATON__DST_STRONG_TRANSITIONS:
				return dst_StrongTransitions != null && !dst_StrongTransitions.isEmpty();
			case SmePackage.AUTOMATON__DST_WEAK_TRANSITIONS:
				return dst_WeakTransitions != null && !dst_WeakTransitions.isEmpty();
			case SmePackage.AUTOMATON__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.AUTOMATON__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.AUTOMATON__STATE_PARENT:
				return getStateParent() != null;
			case SmePackage.AUTOMATON__SRC_WEAK_TRANSITIONS:
				return src_WeakTransitions != null && !src_WeakTransitions.isEmpty();
			case SmePackage.AUTOMATON__SRC_STRONG_TRANSITIONS:
				return src_StrongTransitions != null && !src_StrongTransitions.isEmpty();
			case SmePackage.AUTOMATON__STATES:
				return states != null && !states.isEmpty();
			case SmePackage.AUTOMATON__HISTORY:
				return history != null;
			case SmePackage.AUTOMATON__LOCALS:
				return locals != null && !locals.isEmpty();
			case SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS:
				return src_ClockRelations != null && !src_ClockRelations.isEmpty();
			case SmePackage.AUTOMATON__DST_CLOCK_RELATIONS:
				return dst_ClockRelations != null && !dst_ClockRelations.isEmpty();
			case SmePackage.AUTOMATON__SRC_DEPENDENCES:
				return src_Dependences != null && !src_Dependences.isEmpty();
			case SmePackage.AUTOMATON__DST_DEPENDENCES:
				return dst_Dependences != null && !dst_Dependences.isEmpty();
			case SmePackage.AUTOMATON__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.AUTOMATON__INIT_STATE:
				return initState != null;
			case SmePackage.AUTOMATON__TRANSITIONS:
				return transitions != null && !transitions.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.AUTOMATON__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.AUTOMATON__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == SignalContainer.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ClockedExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS:
					return SmePackage.CLOCKED_EXPR__SRC_CLOCK_RELATIONS;
				case SmePackage.AUTOMATON__DST_CLOCK_RELATIONS:
					return SmePackage.CLOCKED_EXPR__DST_CLOCK_RELATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (derivedFeatureID) {
				case SmePackage.AUTOMATON__SRC_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES;
				case SmePackage.AUTOMATON__DST_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.AUTOMATON__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.AUTOMATON__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == SignalContainer.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ClockedExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CLOCKED_EXPR__SRC_CLOCK_RELATIONS:
					return SmePackage.AUTOMATON__SRC_CLOCK_RELATIONS;
				case SmePackage.CLOCKED_EXPR__DST_CLOCK_RELATIONS:
					return SmePackage.AUTOMATON__DST_CLOCK_RELATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (baseFeatureID) {
				case SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES:
					return SmePackage.AUTOMATON__SRC_DEPENDENCES;
				case SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES:
					return SmePackage.AUTOMATON__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			result.append(name);
			result.append("::(| ... |)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		// This method is only called by an upper Model
		// We add here the Automaton as a SubProcess labelled with its name
		// And eventually clock relation and dependence equations.
		Utils.TRACE("<<<<<<<<<<<<<<<  Automaton : buildEquationsAST BEGIN : " + getName());
		
		if (getParent() instanceof Model) {
			for (ClockRelation cr : getDst_ClockRelations())
				bodyTree = cr.buildEquationsAST(bodyTree);
			for (Dependence dep : getDst_Dependences())
				bodyTree = dep.buildEquationsAST(bodyTree);
			
			Utils.TRACE("---------------------- Automaton : buildEquationsAST BEGIN : " + getName());
			
			long def = TBAPI.mkReferenceToModel(getProcessName());
			long idNode = getAST(null);
			long body = API.makeBinary(SignalOperators.oppetiq, idNode, def);
			bodyTree = API.post(bodyTree, body);
			Utils.TRACE("---------------------- Automaton : buildEquationsAST END : " + getName());
		}
		else
			throw new UnsupportedOperationException();
		
		Utils.TRACE(">>>>>>>>>>>>>>>> Automaton : buildEquationsAST BEGIN : " + getName());
		
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE)
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
			else
				result = idNode;
		}
		else
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * Add the declaration of the automaton to the declaration list. If the automaton is declared in a model, then the
	 * method adds the AST of a label declaration.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		// If the parent is not an Automaton, we declare the label for the current Automaton
		if (getParent() instanceof Model) // if (!(getParent() instanceof Automaton))
		
		{
			Utils.TRACE("getDeclarationAST :AutomatonImpl  BEGIN " + getName());
			long idNode = getAST(null);
			if (idNode != PKTrees.ERROR_NODE)
				declTree = API.post(declTree, API.makeUnary(SignalOperators.oplab, API.makeList(
					SignalOperators.oplident, idNode)));
			long mp = buildAutomaton(PKTrees.ERROR_NODE /* bodyTree */, new DeclarationValue(API
					.makeEmptyList(SignalOperators.opldecl)), null);
			declTree = API.post(declTree, mp);
			Utils.TRACE("getDeclarationAST :AutomatonImpl  END " + getName());
		}
		return declTree;
	}
	
	/**
	 * It returns the expression : f default e (or e is e is the nil tree).
	 * 
	 * @generated NOT
	 */
	private long mkdefault(long nextEq, long node) {
		if (nextEq == PKTrees.ERROR_NODE)
			return node;
		return API.makeBinary(SignalOperators.opmele, nextEq, node);
	}
	
	/**
	 * It adds the definition "vsig := f default e" to an (assumed) body tree (body).
	 * 
	 * @generated NOT
	 */
	private long finalizeAndAddDefinition(String vsig, long currentEq, long defCurrent, long body) {
		return API.post(body, TBAPI.mkEquation(false, API.string2ident(vsig), mkdefault(currentEq, defCurrent)));
	}
	
	/**
	 * Build the reinitialization equation for a state (state). The expressions are added in the body.
	 * 
	 * @return the reinit signal, or null if some pb has been detected.
	 * @generated NOT
	 */
	private String mkAutomatonReinit(StateBase state, String reinitSignalName, long body, long declTree) {
		String reinitSignal, reinitSignalDelay;
		final String current = getPrefix() + CURRENT;
		final String previous = getPrefix() + PREVIOUS; // previous state
		final String history = getPrefix() + HISTORY; // does the current state use the history ?
		
		// Build the reinitialization equation for this state
		// and call the building for states contained in this Compound State.
		reinitSignal = ((CompoundState) state).getPrefix() + REINIT;
		
		reinitSignalDelay = reinitSignal + DELAY;
		String tmp = reinitSignalDelay + " := " + reinitSignal + "$ init false";
		long reinitDelayNode = PkPlugin.getParser().parseString(Phyla.ph_e_proc, tmp);
		body = API.post(body, reinitDelayNode);
		
		String reinitCond = "true when (not " + history + ") when (" + current + "=#" + state.getName() + ") when not "
				+ reinitSignalDelay + " default false when (" + previous + "=#" + state.getName() + ") when ("
				+ reinitSignalDelay + " )";
		if (reinitSignalName != null)
			reinitCond = reinitSignal + " := true when " + reinitSignalName + " default " + reinitCond + " when (not "
					+ reinitSignalName + ")";
		else
			reinitCond = reinitSignal + " := " + reinitCond;
		
		long reinitNode = PkPlugin.getParser().parseString(Phyla.ph_e_proc, reinitCond);
		if (reinitNode == PKTrees.ERROR_NODE) {
			// It has to be correct normally
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.AUTOMATON_REINIT_CONDITION + name, this);
			reinitSignal = null;
		}
		else {
			body = API.post(body, reinitNode);
			long ls = API.makeList(SignalOperators.oplsign, API.string2ident(reinitSignal));
			ls = API.post(ls, API.string2ident(reinitSignalDelay));
			declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecsig, API
					.makeNullary(SignalOperators.oplogical), ls));
		}
		return reinitSignal;
	}
	
	/**
	 * @generated NOT
	 * @return a reference to the value (node) of a enumerated type.
	 */
	private long mkRefState(long node) {
		return API.makeBinary(SignalOperators.opvenum, TBAPI.mkAtomNilTree(), node);
	}
	
	/**
	 * @generated NOT
	 * @return the definition of the next state signal using the set of transition (weakTr)
	 */
	private long mkDefNextState(EList<Transition> weakTr, long nextEq) {
		try {
			for (Transition tr : weakTr) {
				StateOrHistory dst = (StateOrHistory) tr.getDestination();
				long node = dst.getAST(null);
				if (node != PKTrees.ERROR_NODE) {
					node = API.makeBinary(SignalOperators.opextr, mkRefState(node), tr.getAST(null));
					nextEq = mkdefault(nextEq, node);
				}
			}
		}
		catch (CyclicDefinitionException cde) {
			// Normally not thrown
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
					.getElement());
		}
		return nextEq;
	}
	
	/**
	 * @generated NOT
	 * @return the definition of the next state signal using the set of transition (weakTr)
	 */
	private long mkDefNextHistory(EList<Transition> weakTr, long nextHistoryEq) {
		try {
			for (Transition tr : weakTr) {
				StateOrHistory dst = (StateOrHistory) tr.getDestination();
				long node = dst.getAST(null);
				if (node != PKTrees.ERROR_NODE) {
					long bNode = API.makeNullary((dst instanceof History) ? SignalOperators.optrue
							: SignalOperators.opfalse);
					bNode = API.makeBinary(SignalOperators.opextr, bNode, tr.getAST(null));
					nextHistoryEq = mkdefault(nextHistoryEq, bNode);
				}
			}
		}
		catch (CyclicDefinitionException cde) {
			// Normally not thrown
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
					.getElement());
		}
		return nextHistoryEq;
	}
	
	/**
	 * @generated NOT
	 * @return the definition of the current state signal from the set of transition (Tr)
	 */
	private long mkDefCurrentState(EList<Transition> Transitions, long currentEq) {
		try {
			for (Transition tr : Transitions) {
				StateOrHistory dst = (StateOrHistory) tr.getDestination();
				long node = dst.getAST(null);
				if (node != PKTrees.ERROR_NODE) {
					// Build the current state equation
					// Add a "#" to use the value of the enumerated type
					node = API.makeBinary(SignalOperators.opextr, mkRefState(node), tr.getAST(null));
					currentEq = mkdefault(currentEq, node);
				}
			}
		}
		catch (CyclicDefinitionException cde) {
			// Normally not thrown
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
					.getElement());
		}
		return currentEq;
	}
	
	/**
	 * @generated NOT
	 * @return the definition of the xxxx
	 */
	private long mkDefHistory(EList<Transition> Transitions, long historyEq) {
		try {
			for (Transition tr : Transitions) {
				StateOrHistory dst = (StateOrHistory) tr.getDestination();
				long node = dst.getAST(null);
				if (node != PKTrees.ERROR_NODE) {
					long bNode = API.makeNullary((dst instanceof History) ? SignalOperators.optrue
							: SignalOperators.opfalse);
					bNode = API.makeBinary(SignalOperators.opextr, bNode, tr.getAST(null));
					historyEq = mkdefault(historyEq, bNode);
				}
			}
		}
		catch (CyclicDefinitionException cde) {
			// Normally not thrown
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
					.getElement());
		}
		return historyEq;
	}
	
	/**
	 * @ return the name of the Signal process associated with and automata.
	 * 
	 * @generated NOT
	 */
	private long getProcessName() {
		return API.string2ident("Automaton_" + getName());
	}
	
	/**
	 * @ return the model associated with and automata.
	 * 
	 * @see fr.irisa.espresso.sme.StateBase#buildAutomaton(fr.irisa.espresso.sme.polychrony.DeclarationValue,
	 *      java.lang.String) REV add a call to min_clock on every signal declared in a Compound state
	 * @generated NOT
	 */
	public long buildAutomaton(long tttt, DeclarationValue rootDeclaration, String reinitSignalName) {
		
		// Create some constant strings representing the different signals of control of the Automaton
		final String type = getPrefix() + TYPE; // name of the enumerated type for the current automaton
		final String current = getPrefix() + CURRENT; // current state
		final String next = getPrefix() + NEXT; // next state
		final String pnext = getPrefix() + PNEXT; // previous value of next state
		final String previous = getPrefix() + PREVIOUS; // previous state
		final String nextHistory = getPrefix() + NEXTHISTORY; // does the next state use the history ?
		final String history = getPrefix() + HISTORY; // does the current state use the history ?
		boolean isCorrect = true;
		
		// Automaton building
		long typeDeclare = API.makeEmptyList(SignalOperators.optenum); // Declaration of the enumerated type for the
		// current Automaton
		
		long currentEq = PKTrees.ERROR_NODE; // Equation for the current state signal
		long historyEq = PKTrees.ERROR_NODE; // Equation for the history signal
		long nextEq = PKTrees.ERROR_NODE; // Equation for the next state signal
		long nextHistoryEq = PKTrees.ERROR_NODE; // Equation for the next history signal
		
		boolean hasCompoundState = false; // specify if the current Automaton contains CompoundState
		long switchEq = API.makeEmptyList(SignalOperators.oplcas); // Equation of the switch case for each Automaton.
		Set<String> stateNames = new HashSet<String>(); // Set to check if the name of each contained state are
		// different.
		
		Utils.TRACE("buildAutomaton :AutomatonImpl  " + getName());
		long mp = TBAPI.n_mkSkeletonModel(getProcessName(), SignalOperators.opprocess);
		/*
		 * long theTick = PkPlugin.getParser().parseString(Phyla.ph_dec_s, "event" + TICKNAME);
		 * TBAPI.n_addDeclarationToInterface(mp, theTick, 2);
		 */

		if (getStates().isEmpty()) {
			PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.AUTOMATON_NO_SUB_STATE + name, this);
			return mp;
		}
		
		long body = API.makeEmptyList(SignalOperators.oplfeed); // Local composition list for the body of the Automaton
		long declTree; // Local declaration tree
		String reinitSignal;
		
		// if (getParent() instanceof Model) declTree = rootDeclaration.tree; else
		
		declTree = API.makeEmptyList(SignalOperators.opldecl);
		
		// Get the initial state
		StateBase initialState;
		if (getInitState() == null) {
			PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.AUTOMATON_NO_SUB_STATE + name, this);
			initialState = states.get(0);
		}
		else
			initialState = initState;
		
		if (reinitSignalName != null) {
			try {
				long initNode = initialState.getAST(null);
				if (initNode == PKTrees.ERROR_NODE)
					isCorrect = false;
				else
					currentEq = API.makeBinary(SignalOperators.opextr, mkRefState(initNode), API
							.string2ident(reinitSignalName));
			}
			catch (CyclicDefinitionException cde) {
				isCorrect = false;
				// Normally not thrown
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		
		// Now we build the different equation by iterating on each contained state
		for (StateBase state : states) {
			// Check if there is no several state with the same name. If it is the case, we cannot use them
			if (!stateNames.add(state.getName())) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.AUTOMATON_STATES_NAMES + name + " / "
						+ state.getName(), state);
				isCorrect = false;
				continue;
			}
			
			try {
				long idNode = state.getAST(null);
				if (idNode == PKTrees.ERROR_NODE) {
					isCorrect = false;
					continue;
				}
				typeDeclare = API.post(typeDeclare, idNode); // adding the name of the state to the enumerated type.
				
				if (isCorrect) {
					hasCompoundState = (state instanceof CompoundState);
					if (hasCompoundState)
						reinitSignal = mkAutomatonReinit(state, reinitSignalName, body, declTree);
					else
						reinitSignal = null;
					
					// vt is a model of process : Declaring it and adding a reference to this model.
					long vt = state.buildAutomaton(PKTrees.ERROR_NODE, rootDeclaration, reinitSignal);
					declTree = API.post(declTree, vt);
					
					long def = TBAPI.mkReferenceToModel(TEAPI.getModelName(vt));
					def = API.makeList(SignalOperators.oplfeed, def);
					
					// Adding a case element.
					idNode = API.makeList(SignalOperators.oplesign, API.makeBinary(SignalOperators.opvenum, TBAPI
							.mkAtomNilTree(), state.getAST(null)));
					long vt1 = API.makeBinary(SignalOperators.opcas, idNode, def);
					switchEq = API.post(switchEq, vt1);
					
					// We process all weak transitions whose source is state to compute the nextEq and the nextHistoryEq
					// First, we order the weak transition according to their priority
					// Is it necessary to have a priority in the SME automata ? (lbesnard)
					EList<Transition> weakTr = Utils.orderTransitions(state.getSrc_WeakTransitions());
					nextEq = mkDefNextState(weakTr, nextEq); // Equation for the next state signal
					nextHistoryEq = mkDefNextHistory(weakTr, nextHistoryEq); // Equation for the next history signal
					
					// We process all strong transitions whose source is state to compute the currentEq and the
					// historyEq.
					// First, we order the strong transition according to their priority
					EList<Transition> strongTr = Utils.orderTransitions(state.getSrc_StrongTransitions());
					currentEq = mkDefCurrentState(strongTr, currentEq);
					historyEq = mkDefHistory(strongTr, historyEq);
				}
			}
			catch (CyclicDefinitionException cde) {
				// Normally not thrown
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		
		// Add the switch construction
		switchEq = API.makeTernary(SignalOperators.oppcas, API.string2ident(current), switchEq, TBAPI.mkAtomNilTree());
		body = API.post(body, switchEq);
		
		// We add the equations of each transition contained by the Automaton
		body = Utils.makeEquationsList(getTransitions(), body);
		
		// Add the local declarations
		declTree = Utils.makeDeclarationList(getTransitions(), declTree);
		declTree = Utils.makeDeclarationList(getLocals(), declTree);
		
		if (isCorrect) {
			// Add the enumeration type declaration to the root Automaton declarations
			typeDeclare = API.makeList(SignalOperators.opltype, API.makeTernary(SignalOperators.optype, API
					.string2ident(type), typeDeclare, TBAPI.mkAtomNilTree()));
			declTree = API.post(declTree, typeDeclare);
			
			// Add the declaration of the current state signal at the top level of the Automaton
			declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecsig, API.string2ident(type), API
					.makeList(SignalOperators.oplsign, API.string2ident(current))));
			
			// Add the declaration of the next state, previous state and previous next state to this level
			long listNode = API.makeList(SignalOperators.oplsign, API.string2ident(next));
			listNode = API.post(listNode, API.string2ident(previous));
			listNode = API.post(listNode, API.string2ident(pnext));
			declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecsig, API.string2ident(type), listNode));
			
			// Terminate the building and add the current state definition equation ; it is synchronized to TICKNAME
			body = finalizeAndAddDefinition(current, currentEq, API.string2ident(pnext), body);
			listNode = API.makeList(SignalOperators.oplequhor, API.string2ident(current));
			listNode = API.post(listNode, API.string2ident(TICKNAME));
			API.post(body, listNode);
			// Terminate the building and add the next state definition equation
			body = finalizeAndAddDefinition(next, nextEq, API.string2ident(current), body);
			try {
				// Build and add the previous state signal definition equation
				long previousEq = initialState.getAST(null);
				if (previousEq != PKTrees.ERROR_NODE) // ////// lbesnard : voir dupplication paragraphe suivant.
				{
					// Normally, this test is always true, this operation already passed
					// Here we build the expression #<enum value>
					previousEq = mkRefState(previousEq);
					// Here we build the delay expression
					previousEq = API.makeTernary(SignalOperators.opret, API.string2ident(current), API
							.makeNullary(SignalOperators.opretun), previousEq);
					// Here we build the definition equation
					previousEq = API.makeBinary(SignalOperators.opdefsig, API.string2ident(previous), previousEq);
					// Add the equation to the composition list
					body = API.post(body, previousEq);
				}
				// Build and add the definition equation of the signal whose
				// value is the previous value of the next state
				long pnextEq = initialState.getAST(null);
				if (pnextEq != PKTrees.ERROR_NODE) {
					// Normally, this test is always true, this operation already passed
					// Here we build the expression #<enum value>
					pnextEq = mkRefState(pnextEq);
					// Here we build the delay expression
					pnextEq = API.makeTernary(SignalOperators.opret, API.string2ident(next), API
							.makeNullary(SignalOperators.opretun), pnextEq);
					// Here we build the definition equation
					pnextEq = API.makeBinary(SignalOperators.opdefsig, API.string2ident(pnext), pnextEq);
					// Add the equation to the composition list
					body = API.post(body, pnextEq);
				}
			}
			catch (CyclicDefinitionException cde) {
				// Normally not thrown
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
			
			// If there is/are one or more compound state, we add the equation concerning the management of history
			if (hasCompoundState) {
				// Declaration of history, nextHistory signals.
				listNode = API.makeList(SignalOperators.oplsign, API.string2ident(history));
				listNode = API.post(listNode, API.string2ident(nextHistory));
				declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecsig, API
						.makeNullary(SignalOperators.oplogical), listNode));
				
				// Here, we terminate the building and add the equation of the
				// definition of the history signal
				long defHistory = API.makeTernary(SignalOperators.opret, API.string2ident(nextHistory), API
						.makeNullary(SignalOperators.opretun), API.makeNullary(SignalOperators.optrue));
				body = finalizeAndAddDefinition(history, historyEq, defHistory, body);
				// Here, we terminate the building and add the equation of the definition of the nextHistory signal
				long defNextHistory = API.makeNullary(SignalOperators.optrue);
				body = finalizeAndAddDefinition(nextHistory, nextHistoryEq, defNextHistory, body);
			}
		}
		
		// if (API.isEmpty(body)) body = API.makeNullary(SignalOperators.opnullproc);
		TBAPI.setBody(mp, body);
		TBAPI.setLocalDeclarations(mp, declTree);
		return mp;
	}
	
	private long getTreeId(String str) {
		return PkPlugin.getParser().parseString(Phyla.ph_ident, str);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.irisa.espresso.sme.Automaton#getCurrentStateNameAST()
	 */
	public long getCurrentStateNameAST() {
		return getTreeId(getPrefix() + CURRENT);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.irisa.espresso.sme.Automaton#getCurrentStateNameAST()
	 */
	public long getzNextStateNameAST() {
		return getTreeId(getPrefix() + PNEXT);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.irisa.espresso.sme.CompoundState#getPrefix()
	 */
	public String getPrefix() {
		return "";
		/*
		 * if (prefix == null) { String tmp = getName(); if (tmp != null) tmp = tmp.replaceAll(" ", "_"); prefix = "__"
		 * + tmp + "_A_" + countAutomata++; } return prefix;
		 */
	}
} // AutomatonImpl
