/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EDataTypeUniqueEList;

import fr.irisa.espresso.sme.CaseConnection;
import fr.irisa.espresso.sme.CaseKind;
import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.SubProcess;
import fr.irisa.espresso.sme.Switch;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Case Connection</b></em>'. <!-- end-user-doc
 * -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.CaseConnectionImpl#getCaseKind <em>Case Kind</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CaseConnectionImpl#getEnumeration <em>Enumeration</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CaseConnectionImpl#getLowerBound <em>Lower Bound</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CaseConnectionImpl#getUpperBound <em>Upper Bound</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CaseConnectionImpl#getCaseLabel <em>Case Label</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CaseConnectionImpl#getSrc <em>Src</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CaseConnectionImpl#getDst <em>Dst</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class CaseConnectionImpl extends ModelConnectionImpl implements CaseConnection {
	/**
	 * The default value of the '{@link #getCaseKind() <em>Case Kind</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getCaseKind()
	 * @generated
	 * @ordered
	 */
	protected static final CaseKind	CASE_KIND_EDEFAULT		= CaseKind.ELSE_CASE_LITERAL;
	
	/**
	 * The cached value of the '{@link #getCaseKind() <em>Case Kind</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getCaseKind()
	 * @generated
	 * @ordered
	 */
	protected CaseKind				caseKind				= CASE_KIND_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getEnumeration() <em>Enumeration</em>}' attribute list. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getEnumeration()
	 * @generated
	 * @ordered
	 */
	protected EList<String>			enumeration;
	
	/**
	 * The default value of the '{@link #getLowerBound() <em>Lower Bound</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getLowerBound()
	 * @generated
	 * @ordered
	 */
	protected static final String	LOWER_BOUND_EDEFAULT	= "";
	
	/**
	 * The cached value of the '{@link #getLowerBound() <em>Lower Bound</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getLowerBound()
	 * @generated
	 * @ordered
	 */
	protected String				lowerBound				= LOWER_BOUND_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getUpperBound() <em>Upper Bound</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getUpperBound()
	 * @generated
	 * @ordered
	 */
	protected static final String	UPPER_BOUND_EDEFAULT	= "";
	
	/**
	 * The cached value of the '{@link #getUpperBound() <em>Upper Bound</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getUpperBound()
	 * @generated
	 * @ordered
	 */
	protected String				upperBound				= UPPER_BOUND_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getCaseLabel() <em>Case Label</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getCaseLabel()
	 * @generated
	 * @ordered
	 */
	protected static final String	CASE_LABEL_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getSrc() <em>Src</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc()
	 * @generated
	 * @ordered
	 */
	protected Switch				src;
	
	/**
	 * The cached value of the '{@link #getDst() <em>Dst</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst()
	 * @generated
	 * @ordered
	 */
	protected SubProcess			dst;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected CaseConnectionImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.CASE_CONNECTION;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CaseKind getCaseKind() {
		return caseKind;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setCaseKind(CaseKind newCaseKind) {
		CaseKind oldCaseKind = caseKind;
		caseKind = newCaseKind == null ? CASE_KIND_EDEFAULT : newCaseKind;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CASE_CONNECTION__CASE_KIND, oldCaseKind,
				caseKind));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getEnumeration() {
		if (enumeration == null) {
			enumeration = new EDataTypeUniqueEList<String>(String.class, this, SmePackage.CASE_CONNECTION__ENUMERATION);
		}
		return enumeration;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getLowerBound() {
		return lowerBound;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setLowerBound(String newLowerBound) {
		String oldLowerBound = lowerBound;
		lowerBound = newLowerBound;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CASE_CONNECTION__LOWER_BOUND,
				oldLowerBound, lowerBound));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getUpperBound() {
		return upperBound;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setUpperBound(String newUpperBound) {
		String oldUpperBound = upperBound;
		upperBound = newUpperBound;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CASE_CONNECTION__UPPER_BOUND,
				oldUpperBound, upperBound));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public String getCaseLabel() {
		String tmp = "_NO_SIG_";
		
		Switch op = getSrc();
		if (op != null && op.getSignal() != null)
			tmp = op.getSignal().getName();
		tmp += " in ";
		
		int type = caseKind.getValue();
		switch (type) {
			case CaseKind.ENUMERATION:
				tmp += "{";
				Iterator<String> it = this.getEnumeration().iterator();
				boolean res = it.hasNext();
				while (res) {
					tmp += it.next();
					res = it.hasNext();
					if (res)
						tmp += ", ";
				}
				tmp += "}";
				break;
			case CaseKind.CLOSED_INTERVAL:
				tmp += "[" + this.getLowerBound() + ", " + this.getUpperBound() + "]";
				break;
			case CaseKind.LEFT_HALF_OPENED_INTERVAL:
				tmp += "(" + this.getLowerBound() + ", " + this.getUpperBound() + "]";
				break;
			case CaseKind.RIGHT_HALF_OPENED_INTERVAL:
				tmp += "[" + this.getLowerBound() + ", " + this.getUpperBound() + ")";
				break;
			case CaseKind.OPENED_INTERVAL:
				tmp += "(" + this.getLowerBound() + ", " + this.getUpperBound() + ")";
				break;
			default:
				tmp = "else";
		}
		return tmp;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Switch getSrc() {
		if (src != null && src.eIsProxy()) {
			InternalEObject oldSrc = (InternalEObject) src;
			src = (Switch) eResolveProxy(oldSrc);
			if (src != oldSrc) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.CASE_CONNECTION__SRC, oldSrc,
						src));
			}
		}
		return src;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Switch basicGetSrc() {
		return src;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSrc(Switch newSrc, NotificationChain msgs) {
		Switch oldSrc = src;
		src = newSrc;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.CASE_CONNECTION__SRC, oldSrc, newSrc);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSrc(Switch newSrc) {
		if (newSrc != src) {
			NotificationChain msgs = null;
			if (src != null)
				msgs = ((InternalEObject) src).eInverseRemove(this, SmePackage.SWITCH__SRC_CASE_CONNECTIONS,
					Switch.class, msgs);
			if (newSrc != null)
				msgs = ((InternalEObject) newSrc).eInverseAdd(this, SmePackage.SWITCH__SRC_CASE_CONNECTIONS,
					Switch.class, msgs);
			msgs = basicSetSrc(newSrc, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CASE_CONNECTION__SRC, newSrc, newSrc));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SubProcess getDst() {
		if (dst != null && dst.eIsProxy()) {
			InternalEObject oldDst = (InternalEObject) dst;
			dst = (SubProcess) eResolveProxy(oldDst);
			if (dst != oldDst) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.CASE_CONNECTION__DST, oldDst,
						dst));
			}
		}
		return dst;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SubProcess basicGetDst() {
		return dst;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst(SubProcess newDst, NotificationChain msgs) {
		SubProcess oldDst = dst;
		dst = newDst;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.CASE_CONNECTION__DST, oldDst, newDst);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst(SubProcess newDst) {
		if (newDst != dst) {
			NotificationChain msgs = null;
			if (dst != null)
				msgs = ((InternalEObject) dst).eInverseRemove(this, SmePackage.SUB_PROCESS__DST_CASE_CONNECTION,
					SubProcess.class, msgs);
			if (newDst != null)
				msgs = ((InternalEObject) newDst).eInverseAdd(this, SmePackage.SUB_PROCESS__DST_CASE_CONNECTION,
					SubProcess.class, msgs);
			msgs = basicSetDst(newDst, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CASE_CONNECTION__DST, newDst, newDst));
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#getDestination()
	 * @generated NOT
	 */
	public ConnectableElement getDestination() {
		return getDst();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#getSource()
	 * @generated NOT
	 */
	public ConnectableElement getSource() {
		return getSrc();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CASE_CONNECTION__SRC:
				if (src != null)
					msgs = ((InternalEObject) src).eInverseRemove(this, SmePackage.SWITCH__SRC_CASE_CONNECTIONS,
						Switch.class, msgs);
				return basicSetSrc((Switch) otherEnd, msgs);
			case SmePackage.CASE_CONNECTION__DST:
				if (dst != null)
					msgs = ((InternalEObject) dst).eInverseRemove(this, SmePackage.SUB_PROCESS__DST_CASE_CONNECTION,
						SubProcess.class, msgs);
				return basicSetDst((SubProcess) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CASE_CONNECTION__SRC:
				return basicSetSrc(null, msgs);
			case SmePackage.CASE_CONNECTION__DST:
				return basicSetDst(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.CASE_CONNECTION__CASE_KIND:
				return getCaseKind();
			case SmePackage.CASE_CONNECTION__ENUMERATION:
				return getEnumeration();
			case SmePackage.CASE_CONNECTION__LOWER_BOUND:
				return getLowerBound();
			case SmePackage.CASE_CONNECTION__UPPER_BOUND:
				return getUpperBound();
			case SmePackage.CASE_CONNECTION__CASE_LABEL:
				return getCaseLabel();
			case SmePackage.CASE_CONNECTION__SRC:
				if (resolve)
					return getSrc();
				return basicGetSrc();
			case SmePackage.CASE_CONNECTION__DST:
				if (resolve)
					return getDst();
				return basicGetDst();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.CASE_CONNECTION__CASE_KIND:
				setCaseKind((CaseKind) newValue);
				return;
			case SmePackage.CASE_CONNECTION__ENUMERATION:
				getEnumeration().clear();
				getEnumeration().addAll((Collection<? extends String>) newValue);
				return;
			case SmePackage.CASE_CONNECTION__LOWER_BOUND:
				setLowerBound((String) newValue);
				return;
			case SmePackage.CASE_CONNECTION__UPPER_BOUND:
				setUpperBound((String) newValue);
				return;
			case SmePackage.CASE_CONNECTION__SRC:
				setSrc((Switch) newValue);
				return;
			case SmePackage.CASE_CONNECTION__DST:
				setDst((SubProcess) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.CASE_CONNECTION__CASE_KIND:
				setCaseKind(CASE_KIND_EDEFAULT);
				return;
			case SmePackage.CASE_CONNECTION__ENUMERATION:
				getEnumeration().clear();
				return;
			case SmePackage.CASE_CONNECTION__LOWER_BOUND:
				setLowerBound(LOWER_BOUND_EDEFAULT);
				return;
			case SmePackage.CASE_CONNECTION__UPPER_BOUND:
				setUpperBound(UPPER_BOUND_EDEFAULT);
				return;
			case SmePackage.CASE_CONNECTION__SRC:
				setSrc((Switch) null);
				return;
			case SmePackage.CASE_CONNECTION__DST:
				setDst((SubProcess) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.CASE_CONNECTION__CASE_KIND:
				return caseKind != CASE_KIND_EDEFAULT;
			case SmePackage.CASE_CONNECTION__ENUMERATION:
				return enumeration != null && !enumeration.isEmpty();
			case SmePackage.CASE_CONNECTION__LOWER_BOUND:
				return LOWER_BOUND_EDEFAULT == null ? lowerBound != null : !LOWER_BOUND_EDEFAULT.equals(lowerBound);
			case SmePackage.CASE_CONNECTION__UPPER_BOUND:
				return UPPER_BOUND_EDEFAULT == null ? upperBound != null : !UPPER_BOUND_EDEFAULT.equals(upperBound);
			case SmePackage.CASE_CONNECTION__CASE_LABEL:
				return CASE_LABEL_EDEFAULT == null ? getCaseLabel() != null : !CASE_LABEL_EDEFAULT
						.equals(getCaseLabel());
			case SmePackage.CASE_CONNECTION__SRC:
				return src != null;
			case SmePackage.CASE_CONNECTION__DST:
				return dst != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (caseKind: ");
		result.append(caseKind);
		result.append(", enumeration: ");
		result.append(enumeration);
		result.append(", lowerBound: ");
		result.append(lowerBound);
		result.append(", upperBound: ");
		result.append(upperBound);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long dstNode = getDst().buildEquationsAST(PKTrees.ERROR_NODE);
		if (dstNode == PKTrees.ERROR_NODE) {
			PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, PKTrees.ERROR_NODE);
			return PKTrees.ERROR_NODE;
		}
		
		if (caseKind.equals(CaseKind.ELSE_CASE_LITERAL)) {
			PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, dstNode);
			return dstNode;
		}
		
		long result;
		
		PKTrees API = PkPlugin.getTreeAPI();
		
		if (caseKind.equals(CaseKind.ENUMERATION_LITERAL)) {
			result = API.makeEmptyList(SignalOperators.oplesign);
			for (String st : getEnumeration()) {
				long node = PkPlugin.getParser().parseString(Phyla.ph_o_e_sign, st);
				if (node == PKTrees.ERROR_NODE)
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.ENUM_VALUE + st, this);
				else
					result = API.post(result, node);
			}
			if (API.isEmpty(result))
				result = PKTrees.ERROR_NODE;
			else
				result = API.makeBinary(SignalOperators.opcas, result, dstNode);
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.CASE_CONNECTION_INTERVALS, this);
			
			// It is an interval
			boolean allOk = true;
			long binf;
			if (lowerBound == LOWER_BOUND_EDEFAULT)
				binf = API.makeNullary(SignalOperators.opnil);
			else {
				binf = PkPlugin.getParser().parseString(Phyla.ph_o_e_sign, lowerBound);
				if (binf == PKTrees.ERROR_NODE) {
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CASE_CONNECTION_LOWER_BOUND + name, this);
					allOk = false;
				}
			}
			long bsup;
			if (upperBound == UPPER_BOUND_EDEFAULT)
				bsup = API.makeNullary(SignalOperators.opnil);
			else {
				bsup = PkPlugin.getParser().parseString(Phyla.ph_o_e_sign, upperBound);
				if (bsup == PKTrees.ERROR_NODE) {
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CASE_CONNECTION_UPPER_BOUND + name, this);
					allOk = false;
				}
			}
			if (allOk) {
				int opCode;
				if (caseKind.equals(CaseKind.CLOSED_INTERVAL_LITERAL))
					opCode = SignalOperators.opintvff;
				else if (caseKind.equals(CaseKind.LEFT_HALF_OPENED_INTERVAL_LITERAL))
					opCode = SignalOperators.opintvof;
				else if (caseKind.equals(CaseKind.RIGHT_HALF_OPENED_INTERVAL_LITERAL))
					opCode = SignalOperators.opintvfo;
				else
					opCode = SignalOperators.opintvoo;
				
				result = API.makeBinary(SignalOperators.opcas, API.makeBinary(opCode, binf, bsup), dstNode);
			}
			else
				result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
} // CaseConnectionImpl
