/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EDataTypeEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.Cell;
import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionExpr;
import fr.irisa.espresso.sme.ConditionnedExpr;
import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.DataflowOperator;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalExpr;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Cell</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.CellImpl#getDst_Condition <em>Dst Condition</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CellImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CellImpl#getSrc_Conditions <em>Src Conditions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CellImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.CellImpl#getInitialValue <em>Initial Value</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class CellImpl extends SingleInputOperatorImpl implements Cell {
	/**
	 * The cached value of the '{@link #getDst_Condition() <em>Dst Condition</em>}' reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_Condition()
	 * @generated
	 * @ordered
	 */
	protected EList<Condition>			dst_Condition;
	
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>	src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_Conditions() <em>Src Conditions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Conditions()
	 * @generated
	 * @ordered
	 */
	protected EList<Condition>			src_Conditions;
	
	/**
	 * The cached value of the '{@link #getInitialValue() <em>Initial Value</em>}' attribute list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getInitialValue()
	 * @generated
	 * @ordered
	 */
	protected EList<String>				initialValue;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected CellImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.CELL;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Condition> getDst_Condition() {
		if (dst_Condition == null) {
			dst_Condition = new EObjectWithInverseResolvingEList<Condition>(Condition.class, this,
				SmePackage.CELL__DST_CONDITION, SmePackage.CONDITION__DST);
		}
		return dst_Condition;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Condition> getSrc_Conditions() {
		if (src_Conditions == null) {
			src_Conditions = new EObjectWithInverseResolvingEList<Condition>(Condition.class, this,
				SmePackage.CELL__SRC_CONDITIONS, SmePackage.CONDITION__SRC);
		}
		return src_Conditions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.CELL__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.CELL__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.CELL__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__OPERATORS, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CELL__MODEL_PARENT, newModelParent,
				newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getInitialValue() {
		if (initialValue == null) {
			initialValue = new EDataTypeEList<String>(String.class, this, SmePackage.CELL__INITIAL_VALUE);
		}
		return initialValue;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CELL__DST_CONDITION:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_Condition())
						.basicAdd(otherEnd, msgs);
			case SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.CELL__SRC_CONDITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Conditions()).basicAdd(otherEnd,
					msgs);
			case SmePackage.CELL__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CELL__DST_CONDITION:
				return ((InternalEList<?>) getDst_Condition()).basicRemove(otherEnd, msgs);
			case SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.CELL__SRC_CONDITIONS:
				return ((InternalEList<?>) getSrc_Conditions()).basicRemove(otherEnd, msgs);
			case SmePackage.CELL__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.CELL__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__OPERATORS, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.CELL__DST_CONDITION:
				return getDst_Condition();
			case SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.CELL__SRC_CONDITIONS:
				return getSrc_Conditions();
			case SmePackage.CELL__MODEL_PARENT:
				return getModelParent();
			case SmePackage.CELL__INITIAL_VALUE:
				return getInitialValue();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.CELL__DST_CONDITION:
				getDst_Condition().clear();
				getDst_Condition().addAll((Collection<? extends Condition>) newValue);
				return;
			case SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.CELL__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				getSrc_Conditions().addAll((Collection<? extends Condition>) newValue);
				return;
			case SmePackage.CELL__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.CELL__INITIAL_VALUE:
				getInitialValue().clear();
				getInitialValue().addAll((Collection<? extends String>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.CELL__DST_CONDITION:
				getDst_Condition().clear();
				return;
			case SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.CELL__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				return;
			case SmePackage.CELL__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.CELL__INITIAL_VALUE:
				getInitialValue().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.CELL__DST_CONDITION:
				return dst_Condition != null && !dst_Condition.isEmpty();
			case SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.CELL__SRC_CONDITIONS:
				return src_Conditions != null && !src_Conditions.isEmpty();
			case SmePackage.CELL__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.CELL__INITIAL_VALUE:
				return initialValue != null && !initialValue.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ConditionnedExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.CELL__DST_CONDITION:
					return SmePackage.CONDITIONNED_EXPR__DST_CONDITION;
				default:
					return -1;
			}
		}
		if (baseClass == SignalExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS:
					return SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.CELL__SRC_CONDITIONS:
					return SmePackage.CONDITION_EXPR__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowOperator.class) {
			switch (derivedFeatureID) {
				case SmePackage.CELL__MODEL_PARENT:
					return SmePackage.DATAFLOW_OPERATOR__MODEL_PARENT;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ConditionnedExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONDITIONNED_EXPR__DST_CONDITION:
					return SmePackage.CELL__DST_CONDITION;
				default:
					return -1;
			}
		}
		if (baseClass == SignalExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS:
					return SmePackage.CELL__SRC_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONDITION_EXPR__SRC_CONDITIONS:
					return SmePackage.CELL__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowOperator.class) {
			switch (baseFeatureID) {
				case SmePackage.DATAFLOW_OPERATOR__MODEL_PARENT:
					return SmePackage.CELL__MODEL_PARENT;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (initialValue: ");
		result.append(initialValue);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getModelParent();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) throws CyclicDefinitionException {
		if (path.contains(this))
			throw new CyclicDefinitionException(this);
		path.add(this);
		
		long condNode = PKTrees.ERROR_NODE;
		if (getDst_Condition().isEmpty()) {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CELL_NO_TARGET + name, this);
		}
		else {
			for (Condition cd : dst_Condition) {
				if (condNode == PKTrees.ERROR_NODE)
					condNode = cd.getAST(path);
				else {
					long tmp = cd.getAST(path);
					if (tmp != PKTrees.ERROR_NODE)
						condNode = PkPlugin.getTreeAPI().makeBinary(SignalOperators.opextr, condNode, tmp);
				}
			}
		}
		
		PKTrees API = PkPlugin.getTreeAPI();
		
		long initNode;
		int size = getInitialValue().size();
		if (size > 0) {
			if (size > 1) {
				boolean allOk = true;
				initNode = API.makeEmptyList(SignalOperators.oplevect);
				// Value represents an array
				for (String st : initialValue) {
					long tmp = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
					if (tmp == PKTrees.ERROR_NODE) {
						PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CELL_INITIAL_VALUE + st, this);
						allOk = false;
					}
					else
						initNode = API.post(initNode, tmp);
				}
				if (!allOk)
					initNode = API.makeNullary(SignalOperators.opnil);
			}
			else {
				// Value is a simple value
				initNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, initialValue.get(0));
				if (initNode == PKTrees.ERROR_NODE) {
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CELL_INITIAL_VALUE + name, this);
					initNode = API.makeNullary(SignalOperators.opnil);
				}
			}
		}
		else
			initNode = API.makeNullary(SignalOperators.opnil);
		
		long result = PKTrees.ERROR_NODE;
		long node = in.getAST(path);
		if (node != PKTrees.ERROR_NODE && condNode != PKTrees.ERROR_NODE)
			result = API.makeTernary(SignalOperators.opmemo, node, condNode, initNode);
		
		path.remove(this);
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
} // CellImpl
