/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ClockConstraintOpKind;
import fr.irisa.espresso.sme.ClockConstraintOperator;
import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ControlflowOperator;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Clock Constraint Operator</b></em>'. <!--
 * end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ClockConstraintOperatorImpl#getSrc_ClockRelations <em>Src Clock Relations</em>}
 * </li>
 * <li>{@link fr.irisa.espresso.sme.impl.ClockConstraintOperatorImpl#getDst_ClockRelations <em>Dst Clock Relations</em>}
 * </li>
 * <li>{@link fr.irisa.espresso.sme.impl.ClockConstraintOperatorImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ClockConstraintOperatorImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ClockConstraintOperatorImpl#getOperator <em>Operator</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class ClockConstraintOperatorImpl extends EObjectImpl implements ClockConstraintOperator {
	/**
	 * The cached value of the '{@link #getSrc_ClockRelations() <em>Src Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>					src_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getDst_ClockRelations() <em>Dst Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>					dst_ClockRelations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String					NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String								name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getOperator() <em>Operator</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getOperator()
	 * @generated
	 * @ordered
	 */
	protected static final ClockConstraintOpKind	OPERATOR_EDEFAULT	= ClockConstraintOpKind.SYNCHRONIZED_LITERAL;
	
	/**
	 * The cached value of the '{@link #getOperator() <em>Operator</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getOperator()
	 * @generated
	 * @ordered
	 */
	protected ClockConstraintOpKind					operator			= OPERATOR_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ClockConstraintOperatorImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.CLOCK_CONSTRAINT_OPERATOR;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getSrc_ClockRelations() {
		if (src_ClockRelations == null) {
			src_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.CLOCK_CONSTRAINT_OPERATOR__SRC_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__SRC);
		}
		return src_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getDst_ClockRelations() {
		if (dst_ClockRelations == null) {
			dst_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.CLOCK_CONSTRAINT_OPERATOR__DST_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__DST);
		}
		return dst_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CLOCK_CONSTRAINT_OPERATOR__NAME, oldName,
				name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT,
			msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__CONSTRAINTS, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT,
				newModelParent, newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ClockConstraintOpKind getOperator() {
		return operator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setOperator(ClockConstraintOpKind newOperator) {
		ClockConstraintOpKind oldOperator = operator;
		operator = newOperator == null ? OPERATOR_EDEFAULT : newOperator;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CLOCK_CONSTRAINT_OPERATOR__OPERATOR,
				oldOperator, operator));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return this.getModelParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__SRC_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__DST_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__SRC_CLOCK_RELATIONS:
				return ((InternalEList<?>) getSrc_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__DST_CLOCK_RELATIONS:
				return ((InternalEList<?>) getDst_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__CONSTRAINTS, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__SRC_CLOCK_RELATIONS:
				return getSrc_ClockRelations();
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__DST_CLOCK_RELATIONS:
				return getDst_ClockRelations();
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__NAME:
				return getName();
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT:
				return getModelParent();
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__OPERATOR:
				return getOperator();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				getSrc_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				getDst_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__NAME:
				setName((String) newValue);
				return;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__OPERATOR:
				setOperator((ClockConstraintOpKind) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				return;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				return;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__OPERATOR:
				setOperator(OPERATOR_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__SRC_CLOCK_RELATIONS:
				return src_ClockRelations != null && !src_ClockRelations.isEmpty();
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__DST_CLOCK_RELATIONS:
				return dst_ClockRelations != null && !dst_ClockRelations.isEmpty();
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR__OPERATOR:
				return operator != OPERATOR_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.CLOCK_CONSTRAINT_OPERATOR__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == ControlflowOperator.class) {
			switch (derivedFeatureID) {
				case SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT:
					return SmePackage.CONTROLFLOW_OPERATOR__MODEL_PARENT;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.CLOCK_CONSTRAINT_OPERATOR__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == ControlflowOperator.class) {
			switch (baseFeatureID) {
				case SmePackage.CONTROLFLOW_OPERATOR__MODEL_PARENT:
					return SmePackage.CLOCK_CONSTRAINT_OPERATOR__MODEL_PARENT;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", operator: ");
		result.append(operator);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		if (getSrc_ClockRelations().size() != 0) {
			for (ClockRelation cr : src_ClockRelations)
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CLOCK_CONSTRAINTS_SOURCE + name, cr);
		}
		
		if (getDst_ClockRelations().size() > 1) {
			PKTrees API = PkPlugin.getTreeAPI();
			Set<SignalElement> path = new HashSet<SignalElement>();
			long result = API.makeEmptyList(SignalOperators.oplequhor);
			
			int size = 0;
			for (ClockRelation cr : dst_ClockRelations) {
				try {
					long node = cr.getAST(path);
					if (node != PKTrees.ERROR_NODE) {
						result = API.pre(result, node);
						size++;
					}
				}
				catch (CyclicDefinitionException cde) {
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
							.getElement());
				}
			}
			
			if (size > 1) {
				if (operator.equals(ClockConstraintOpKind.SYNCHRONIZED_LITERAL))
					bodyTree = API.post(bodyTree, result);
				else if (operator.equals(ClockConstraintOpKind.EXCLUDED_LITERAL)) {
					result = API.setOp(result, SignalOperators.oplexchor);
					bodyTree = API.post(bodyTree, result);
				}
				else {
					// Identity equations
					long first = API.child(1, result);
					for (int i = 2; i <= size; i++) {
						// We create an identity equation between the first and the i-th elements
						// And it is added to the list of equation
						bodyTree = API.post(bodyTree, API.makeBinary(SignalOperators.opdefegal, PkPlugin
								.getTreeBuilder().copy(first, true), API.child(i, result)));
					}
				}
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CLOCK_CONSTRAINTS_TARGET, this);
		}
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) throws CyclicDefinitionException {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
} // ClockConstraintOperatorImpl
