/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionExpr;
import fr.irisa.espresso.sme.ConditionnedExpr;
import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.ConstantRef;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SignalRef;
import fr.irisa.espresso.sme.SignalStateKind;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Condition</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ConditionImpl#getConditionKind <em>Condition Kind</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConditionImpl#getSrc <em>Src</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConditionImpl#getDst <em>Dst</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConditionImpl#getSrcField <em>Src Field</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class ConditionImpl extends ModelConnectionImpl implements Condition {
	/**
	 * The default value of the '{@link #getConditionKind() <em>Condition Kind</em>}' attribute. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getConditionKind()
	 * @generated
	 * @ordered
	 */
	protected static final SignalStateKind	CONDITION_KIND_EDEFAULT	= SignalStateKind.PRESENT_LITERAL;
	
	/**
	 * The cached value of the '{@link #getConditionKind() <em>Condition Kind</em>}' attribute. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getConditionKind()
	 * @generated
	 * @ordered
	 */
	protected SignalStateKind				conditionKind			= CONDITION_KIND_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc() <em>Src</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc()
	 * @generated
	 * @ordered
	 */
	protected ConditionExpr					src;
	
	/**
	 * The cached value of the '{@link #getDst() <em>Dst</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst()
	 * @generated
	 * @ordered
	 */
	protected ConditionnedExpr				dst;
	
	/**
	 * The default value of the '{@link #getSrcField() <em>Src Field</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSrcField()
	 * @generated
	 * @ordered
	 */
	protected static final String			SRC_FIELD_EDEFAULT		= "";
	
	/**
	 * The cached value of the '{@link #getSrcField() <em>Src Field</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSrcField()
	 * @generated
	 * @ordered
	 */
	protected String						srcField				= SRC_FIELD_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ConditionImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.CONDITION;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalStateKind getConditionKind() {
		return conditionKind;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setConditionKind(SignalStateKind newConditionKind) {
		SignalStateKind oldConditionKind = conditionKind;
		conditionKind = newConditionKind == null ? CONDITION_KIND_EDEFAULT : newConditionKind;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONDITION__CONDITION_KIND,
				oldConditionKind, conditionKind));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConditionExpr getSrc() {
		if (src != null && src.eIsProxy()) {
			InternalEObject oldSrc = (InternalEObject) src;
			src = (ConditionExpr) eResolveProxy(oldSrc);
			if (src != oldSrc) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.CONDITION__SRC, oldSrc, src));
			}
		}
		return src;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConditionExpr basicGetSrc() {
		return src;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSrc(ConditionExpr newSrc, NotificationChain msgs) {
		ConditionExpr oldSrc = src;
		src = newSrc;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET, SmePackage.CONDITION__SRC,
				oldSrc, newSrc);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSrc(ConditionExpr newSrc) {
		if (newSrc != src) {
			NotificationChain msgs = null;
			if (src != null)
				msgs = ((InternalEObject) src).eInverseRemove(this, SmePackage.CONDITION_EXPR__SRC_CONDITIONS,
					ConditionExpr.class, msgs);
			if (newSrc != null)
				msgs = ((InternalEObject) newSrc).eInverseAdd(this, SmePackage.CONDITION_EXPR__SRC_CONDITIONS,
					ConditionExpr.class, msgs);
			msgs = basicSetSrc(newSrc, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONDITION__SRC, newSrc, newSrc));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConditionnedExpr getDst() {
		if (dst != null && dst.eIsProxy()) {
			InternalEObject oldDst = (InternalEObject) dst;
			dst = (ConditionnedExpr) eResolveProxy(oldDst);
			if (dst != oldDst) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.CONDITION__DST, oldDst, dst));
			}
		}
		return dst;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConditionnedExpr basicGetDst() {
		return dst;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst(ConditionnedExpr newDst, NotificationChain msgs) {
		ConditionnedExpr oldDst = dst;
		dst = newDst;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET, SmePackage.CONDITION__DST,
				oldDst, newDst);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst(ConditionnedExpr newDst) {
		if (newDst != dst) {
			NotificationChain msgs = null;
			if (dst != null)
				msgs = ((InternalEObject) dst).eInverseRemove(this, SmePackage.CONDITIONNED_EXPR__DST_CONDITION,
					ConditionnedExpr.class, msgs);
			if (newDst != null)
				msgs = ((InternalEObject) newDst).eInverseAdd(this, SmePackage.CONDITIONNED_EXPR__DST_CONDITION,
					ConditionnedExpr.class, msgs);
			msgs = basicSetDst(newDst, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONDITION__DST, newDst, newDst));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getSrcField() {
		return srcField;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSrcField(String newSrcField) {
		String oldSrcField = srcField;
		srcField = newSrcField;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONDITION__SRC_FIELD, oldSrcField,
				srcField));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CONDITION__SRC:
				if (src != null)
					msgs = ((InternalEObject) src).eInverseRemove(this, SmePackage.CONDITION_EXPR__SRC_CONDITIONS,
						ConditionExpr.class, msgs);
				return basicSetSrc((ConditionExpr) otherEnd, msgs);
			case SmePackage.CONDITION__DST:
				if (dst != null)
					msgs = ((InternalEObject) dst).eInverseRemove(this, SmePackage.CONDITIONNED_EXPR__DST_CONDITION,
						ConditionnedExpr.class, msgs);
				return basicSetDst((ConditionnedExpr) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CONDITION__SRC:
				return basicSetSrc(null, msgs);
			case SmePackage.CONDITION__DST:
				return basicSetDst(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.CONDITION__CONDITION_KIND:
				return getConditionKind();
			case SmePackage.CONDITION__SRC:
				if (resolve)
					return getSrc();
				return basicGetSrc();
			case SmePackage.CONDITION__DST:
				if (resolve)
					return getDst();
				return basicGetDst();
			case SmePackage.CONDITION__SRC_FIELD:
				return getSrcField();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.CONDITION__CONDITION_KIND:
				setConditionKind((SignalStateKind) newValue);
				return;
			case SmePackage.CONDITION__SRC:
				setSrc((ConditionExpr) newValue);
				return;
			case SmePackage.CONDITION__DST:
				setDst((ConditionnedExpr) newValue);
				return;
			case SmePackage.CONDITION__SRC_FIELD:
				setSrcField((String) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.CONDITION__CONDITION_KIND:
				setConditionKind(CONDITION_KIND_EDEFAULT);
				return;
			case SmePackage.CONDITION__SRC:
				setSrc((ConditionExpr) null);
				return;
			case SmePackage.CONDITION__DST:
				setDst((ConditionnedExpr) null);
				return;
			case SmePackage.CONDITION__SRC_FIELD:
				setSrcField(SRC_FIELD_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.CONDITION__CONDITION_KIND:
				return conditionKind != CONDITION_KIND_EDEFAULT;
			case SmePackage.CONDITION__SRC:
				return src != null;
			case SmePackage.CONDITION__DST:
				return dst != null;
			case SmePackage.CONDITION__SRC_FIELD:
				return SRC_FIELD_EDEFAULT == null ? srcField != null : !SRC_FIELD_EDEFAULT.equals(srcField);
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (conditionKind: ");
		result.append(conditionKind);
		result.append(", srcField: ");
		result.append(srcField);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#getDestination()
	 * @generated NOT
	 */
	@Override
	public ConnectableElement getDestination() {
		return getDst();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#getSource()
	 * @generated NOT
	 */
	@Override
	public ConnectableElement getSource() {
		return this.getSrc();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) throws CyclicDefinitionException {
		long srcNode;
		
		if ((src instanceof SignalOrRef || src instanceof ConstantOrRef) && srcField != null
				&& "".equals(srcField) == false) {
			String tmp;
			// Check on Identifier and not on Signal or Constant because it can be an InputInstance, OutputInstance, or
			// ParameterInstance
			if (src instanceof Identifier)
				tmp = ((Identifier) src).getName();
			else if (src instanceof SignalRef)
				tmp = ((SignalRef) src).getSigName();
			else if (src instanceof ConstantRef)
				tmp = ((ConstantRef) src).getCstName();
			else
				tmp = null;
			
			tmp = Utils.getSignalString(tmp, srcField);
			
			// Bug fix #5766: replace Phyla.ph_l_e_sign by Phyla.ph_e_sign
			srcNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, tmp);
			if (srcNode == PKTrees.ERROR_NODE)
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.SOURCE_FIELD, this);
		}
		else {
			srcNode = src.getAST(path);
			if (srcField != null && "".equals(srcField) == false)
				PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.SOURCE_FIELD, this);
		}
		
		if (srcNode == PKTrees.ERROR_NODE)
			return PKTrees.ERROR_NODE;
		
		// Analyze the ConditionKind and create the corresponding node
		if (conditionKind.equals(SignalStateKind.FALSE_LITERAL))
			srcNode = PkPlugin.getTreeAPI().makeUnary(SignalOperators.opnon, srcNode);
		else if (conditionKind.equals(SignalStateKind.PRESENT_LITERAL))
			srcNode = PkPlugin.getTreeAPI().makeUnary(SignalOperators.opevnt, srcNode);
		
		return srcNode;
	}
	
} // ConditionImpl
