/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EDataTypeEList;
import org.eclipse.emf.ecore.util.EDataTypeUniqueEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionExpr;
import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.ConstantValue;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.Module;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalIntrinsicType;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.VisibilityKind;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Constant Value</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}
 * </li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getSrc_ConstantAffectations <em>Src Constant Affectations
 * </em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getDst_ConstantAffectation <em>Dst Constant Affectation</em>}
 * </li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getSrc_Conditions <em>Src Conditions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getModuleParent <em>Module Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getArrayDimensions <em>Array Dimensions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getType <em>Type</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getValue <em>Value</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ConstantValueImpl#getVisibility <em>Visibility</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class ConstantValueImpl extends EObjectImpl implements ConstantValue {
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>		src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String			NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String						name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String			COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String						comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getDst_ConstantAffectation() <em>Dst Constant Affectation</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ConstantAffectation()
	 * @generated
	 * @ordered
	 */
	protected ConstantAffectation			dst_ConstantAffectation;
	
	/**
	 * The cached value of the '{@link #getSrc_Conditions() <em>Src Conditions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Conditions()
	 * @generated
	 * @ordered
	 */
	protected EList<Condition>				src_Conditions;
	
	/**
	 * The cached value of the '{@link #getArrayDimensions() <em>Array Dimensions</em>}' attribute list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getArrayDimensions()
	 * @generated
	 * @ordered
	 */
	protected EList<String>					arrayDimensions;
	
	/**
	 * The default value of the '{@link #getType() <em>Type</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getType()
	 * @generated
	 * @ordered
	 */
	protected static final String			TYPE_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getType() <em>Type</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getType()
	 * @generated
	 * @ordered
	 */
	protected String						type				= TYPE_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getValue() <em>Value</em>}' attribute list. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getValue()
	 * @generated
	 * @ordered
	 */
	protected EList<String>					value;
	
	/**
	 * The default value of the '{@link #getVisibility() <em>Visibility</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getVisibility()
	 * @generated
	 * @ordered
	 */
	protected static final VisibilityKind	VISIBILITY_EDEFAULT	= VisibilityKind.PUBLIC_LITERAL;
	
	/**
	 * The cached value of the '{@link #getVisibility() <em>Visibility</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getVisibility()
	 * @generated
	 * @ordered
	 */
	protected VisibilityKind				visibility			= VISIBILITY_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ConstantValueImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.CONSTANT_VALUE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.CONSTANT_VALUE__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONSTANT_VALUE__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONSTANT_VALUE__COMMENT, oldComment,
				comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation getDst_ConstantAffectation() {
		if (dst_ConstantAffectation != null && dst_ConstantAffectation.eIsProxy()) {
			InternalEObject oldDst_ConstantAffectation = (InternalEObject) dst_ConstantAffectation;
			dst_ConstantAffectation = (ConstantAffectation) eResolveProxy(oldDst_ConstantAffectation);
			if (dst_ConstantAffectation != oldDst_ConstantAffectation) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE,
						SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation,
						dst_ConstantAffectation));
			}
		}
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation basicGetDst_ConstantAffectation() {
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation,
			NotificationChain msgs) {
		ConstantAffectation oldDst_ConstantAffectation = dst_ConstantAffectation;
		dst_ConstantAffectation = newDst_ConstantAffectation;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation,
				newDst_ConstantAffectation);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation) {
		if (newDst_ConstantAffectation != dst_ConstantAffectation) {
			NotificationChain msgs = null;
			if (dst_ConstantAffectation != null)
				msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			if (newDst_ConstantAffectation != null)
				msgs = ((InternalEObject) newDst_ConstantAffectation).eInverseAdd(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			msgs = basicSetDst_ConstantAffectation(newDst_ConstantAffectation, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION,
				newDst_ConstantAffectation, newDst_ConstantAffectation));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Condition> getSrc_Conditions() {
		if (src_Conditions == null) {
			src_Conditions = new EObjectWithInverseResolvingEList<Condition>(Condition.class, this,
				SmePackage.CONSTANT_VALUE__SRC_CONDITIONS, SmePackage.CONDITION__SRC);
		}
		return src_Conditions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.CONSTANT_VALUE__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.CONSTANT_VALUE__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.CONSTANT_VALUE__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__CONSTANTS, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONSTANT_VALUE__MODEL_PARENT,
				newModelParent, newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Module getModuleParent() {
		if (eContainerFeatureID != SmePackage.CONSTANT_VALUE__MODULE_PARENT)
			return null;
		return (Module) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModuleParent(Module newModuleParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModuleParent, SmePackage.CONSTANT_VALUE__MODULE_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModuleParent(Module newModuleParent) {
		if (newModuleParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.CONSTANT_VALUE__MODULE_PARENT && newModuleParent != null)) {
			if (EcoreUtil.isAncestor(this, newModuleParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModuleParent != null)
				msgs = ((InternalEObject) newModuleParent).eInverseAdd(this, SmePackage.MODULE__CONSTANTS,
					Module.class, msgs);
			msgs = basicSetModuleParent(newModuleParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONSTANT_VALUE__MODULE_PARENT,
				newModuleParent, newModuleParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getArrayDimensions() {
		if (arrayDimensions == null) {
			arrayDimensions = new EDataTypeUniqueEList<String>(String.class, this,
				SmePackage.CONSTANT_VALUE__ARRAY_DIMENSIONS);
		}
		return arrayDimensions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void setType(String newType) {
		String oldType = type;
		if (newType == "")
			type = TYPE_EDEFAULT;
		else
			type = newType;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONSTANT_VALUE__TYPE, oldType, type));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getValue() {
		if (value == null) {
			value = new EDataTypeEList<String>(String.class, this, SmePackage.CONSTANT_VALUE__VALUE);
		}
		return value;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public VisibilityKind getVisibility() {
		return visibility;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setVisibility(VisibilityKind newVisibility) {
		VisibilityKind oldVisibility = visibility;
		visibility = newVisibility == null ? VISIBILITY_EDEFAULT : newVisibility;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.CONSTANT_VALUE__VISIBILITY, oldVisibility,
				visibility));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CONSTANT_VALUE__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
			case SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION:
				if (dst_ConstantAffectation != null)
					msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
						SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
				return basicSetDst_ConstantAffectation((ConstantAffectation) otherEnd, msgs);
			case SmePackage.CONSTANT_VALUE__SRC_CONDITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Conditions()).basicAdd(otherEnd,
					msgs);
			case SmePackage.CONSTANT_VALUE__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
			case SmePackage.CONSTANT_VALUE__MODULE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModuleParent((Module) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.CONSTANT_VALUE__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
			case SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION:
				return basicSetDst_ConstantAffectation(null, msgs);
			case SmePackage.CONSTANT_VALUE__SRC_CONDITIONS:
				return ((InternalEList<?>) getSrc_Conditions()).basicRemove(otherEnd, msgs);
			case SmePackage.CONSTANT_VALUE__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
			case SmePackage.CONSTANT_VALUE__MODULE_PARENT:
				return basicSetModuleParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.CONSTANT_VALUE__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__CONSTANTS, Model.class, msgs);
			case SmePackage.CONSTANT_VALUE__MODULE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODULE__CONSTANTS, Module.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.CONSTANT_VALUE__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.CONSTANT_VALUE__NAME:
				return getName();
			case SmePackage.CONSTANT_VALUE__COMMENT:
				return getComment();
			case SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION:
				if (resolve)
					return getDst_ConstantAffectation();
				return basicGetDst_ConstantAffectation();
			case SmePackage.CONSTANT_VALUE__SRC_CONDITIONS:
				return getSrc_Conditions();
			case SmePackage.CONSTANT_VALUE__MODEL_PARENT:
				return getModelParent();
			case SmePackage.CONSTANT_VALUE__MODULE_PARENT:
				return getModuleParent();
			case SmePackage.CONSTANT_VALUE__ARRAY_DIMENSIONS:
				return getArrayDimensions();
			case SmePackage.CONSTANT_VALUE__TYPE:
				return getType();
			case SmePackage.CONSTANT_VALUE__VALUE:
				return getValue();
			case SmePackage.CONSTANT_VALUE__VISIBILITY:
				return getVisibility();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.CONSTANT_VALUE__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__NAME:
				setName((String) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				getSrc_Conditions().addAll((Collection<? extends Condition>) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__MODULE_PARENT:
				setModuleParent((Module) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				getArrayDimensions().addAll((Collection<? extends String>) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__TYPE:
				setType((String) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__VALUE:
				getValue().clear();
				getValue().addAll((Collection<? extends String>) newValue);
				return;
			case SmePackage.CONSTANT_VALUE__VISIBILITY:
				setVisibility((VisibilityKind) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.CONSTANT_VALUE__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.CONSTANT_VALUE__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.CONSTANT_VALUE__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) null);
				return;
			case SmePackage.CONSTANT_VALUE__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				return;
			case SmePackage.CONSTANT_VALUE__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.CONSTANT_VALUE__MODULE_PARENT:
				setModuleParent((Module) null);
				return;
			case SmePackage.CONSTANT_VALUE__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				return;
			case SmePackage.CONSTANT_VALUE__TYPE:
				setType(TYPE_EDEFAULT);
				return;
			case SmePackage.CONSTANT_VALUE__VALUE:
				getValue().clear();
				return;
			case SmePackage.CONSTANT_VALUE__VISIBILITY:
				setVisibility(VISIBILITY_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.CONSTANT_VALUE__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.CONSTANT_VALUE__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.CONSTANT_VALUE__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.CONSTANT_VALUE__DST_CONSTANT_AFFECTATION:
				return dst_ConstantAffectation != null;
			case SmePackage.CONSTANT_VALUE__SRC_CONDITIONS:
				return src_Conditions != null && !src_Conditions.isEmpty();
			case SmePackage.CONSTANT_VALUE__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.CONSTANT_VALUE__MODULE_PARENT:
				return getModuleParent() != null;
			case SmePackage.CONSTANT_VALUE__ARRAY_DIMENSIONS:
				return arrayDimensions != null && !arrayDimensions.isEmpty();
			case SmePackage.CONSTANT_VALUE__TYPE:
				return TYPE_EDEFAULT == null ? type != null : !TYPE_EDEFAULT.equals(type);
			case SmePackage.CONSTANT_VALUE__VALUE:
				return value != null && !value.isEmpty();
			case SmePackage.CONSTANT_VALUE__VISIBILITY:
				return visibility != VISIBILITY_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.CONSTANT_VALUE__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.CONSTANT_VALUE__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.CONSTANT_VALUE__SRC_CONDITIONS:
					return SmePackage.CONDITION_EXPR__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_VALUE__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.CONSTANT_VALUE__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.CONSTANT_VALUE__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONDITION_EXPR__SRC_CONDITIONS:
					return SmePackage.CONSTANT_VALUE__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(", arrayDimensions: ");
		result.append(arrayDimensions);
		result.append(", type: ");
		result.append(type);
		result.append(", value: ");
		result.append(value);
		result.append(", visibility: ");
		result.append(visibility);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
				result.append("private ");
			result.append("constant");
			if (type != null) {
				result.append(" " + type);
				if (getArrayDimensions().size() > 0) {
					result.append("[");
					Iterator<String> it = arrayDimensions.iterator();
					result.append(it.next());
					while (it.hasNext()) {
						result.append(", ");
						result.append(it.next());
					}
					result.append("]");
				}
			}
			result.append(" " + name);
			if (getValue().size() > 0) {
				result.append(" = ");
				if (value.size() > 1)
					result.append("[");
				Iterator<String> it = value.iterator();
				result.append(it.next());
				while (it.hasNext()) {
					result.append(", ");
					result.append(it.next());
				}
				if (value.size() > 1)
					result.append("]");
			}
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		SignalContainer parent = getModuleParent();
		if (parent != null)
			return parent;
		else
			return getModelParent();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * 
	 * Returns an UnsupportedOperationException. A constant declaration doest not define equations in the body of its
	 * parent.
	 * 
	 * @see fr.irisa.espresso.sme.Identifier#buildEquationsAST(int)
	 * 
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConstantImpl#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			
			long typeNode;
			if (TYPE_EDEFAULT == type || type.equals(SignalIntrinsicType.UNDEFINED_LITERAL.getLiteral())) {
				// Implicit type
				typeNode = API.makeNullary(SignalOperators.opimplicit);
				PolychronyDiagnostic
						.add(Diagnostic.WARNING, WarningMessages.CONSTANT_VALUE_UNDEFINED_TYPE + name, this);
			}
			else {
				if (type.equals(SignalIntrinsicType.BOOLEAN_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.oplogical);
				else if (type.equals(SignalIntrinsicType.CHAR_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opchar);
				else if (type.equals(SignalIntrinsicType.COMPLEX_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opcomplex);
				else if (type.equals(SignalIntrinsicType.DCOMPLEX_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opdcomplex);
				else if (type.equals(SignalIntrinsicType.DREAL_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opdpreal);
				else if (type.equals(SignalIntrinsicType.EVENT_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opevent);
				else if (type.equals(SignalIntrinsicType.INTEGER_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opinteger);
				else if (type.equals(SignalIntrinsicType.LONG_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.oplong);
				else if (type.equals(SignalIntrinsicType.REAL_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opreal);
				else if (type.equals(SignalIntrinsicType.SHORT_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opshort);
				else if (type.equals(SignalIntrinsicType.STRING_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opstring);
				else
					typeNode = API.atomString(SignalOperators.opnom, type);
				
				// Add the specification of arrayDimensions. Parse the textual expression
				if (getArrayDimensions().size() > 0) {
					long arrayNode = API.makeEmptyList(SignalOperators.oplesign);
					boolean allOk = true;
					for (String st : arrayDimensions) {
						long exprNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
						if (exprNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR,
								ErrorMessages.CONSTANT_VALUE_ARRAY_DIMENSION_VALUE + st, this);
							allOk = false;
						}
						else
							arrayNode = API.post(arrayNode, exprNode);
					}
					if (allOk)
						typeNode = API.makeBinary(SignalOperators.optvect, arrayNode, typeNode);
				}
			}
			
			long valueNode;
			if (getDst_ConstantAffectation() != null) {
				try {
					valueNode = dst_ConstantAffectation.getAST(new HashSet<SignalElement>());
					if (valueNode != PKTrees.ERROR_NODE)
						idNode = API.makeBinary(SignalOperators.opcons, idNode, valueNode);
				}
				catch (CyclicDefinitionException cde) {
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
							.getElement());
				}
			}
			else {
				int size = getValue().size();
				if (size > 0) {
					if (size > 1) {
						valueNode = API.makeEmptyList(SignalOperators.oplevect);
						boolean allOk = true;
						// Value represents an array
						for (String st : value) {
							long exprNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
							if (exprNode == PKTrees.ERROR_NODE) {
								PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CONSTANT_VALUE_VALUE + st,
									this);
								allOk = false;
							}
							else
								valueNode = API.post(valueNode, exprNode);
						}
						if (allOk)
							idNode = API.makeBinary(SignalOperators.opcons, idNode, valueNode);
					}
					else {
						// Value is a simple value
						valueNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, value.get(0));
						if (valueNode == PKTrees.ERROR_NODE)
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CONSTANT_VALUE_VALUE + name, this);
						else
							idNode = API.makeBinary(SignalOperators.opcons, idNode, valueNode);
					}
				}
			}
			
			if (getParent() instanceof Module && visibility.equals(VisibilityKind.PRIVATE_LITERAL))
				declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecconp, typeNode, API.makeList(
					SignalOperators.oplsign, idNode)));
			else
				declTree = API.post(declTree, API.makeBinary(SignalOperators.opdeccon, typeNode, API.makeList(
					SignalOperators.oplsign, idNode)));
		}
		return declTree;
	}
} // ConstantValueImpl
