/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.ConstantRef;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.DataflowConnector;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalExpr;
import fr.irisa.espresso.sme.SignalIntrinsicType;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SignalRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Dataflow Connection</b></em>'. <!--
 * end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.DataflowConnectionImpl#isIsDefaultValue <em>Is Default Value</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DataflowConnectionImpl#getCastType <em>Cast Type</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DataflowConnectionImpl#isUseSrcClock <em>Use Src Clock</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DataflowConnectionImpl#isUseLastIterationValue <em>Use Last Iteration Value
 * </em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DataflowConnectionImpl#getSrc <em>Src</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DataflowConnectionImpl#getDst <em>Dst</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DataflowConnectionImpl#getSrcField <em>Src Field</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DataflowConnectionImpl#getDstField <em>Dst Field</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class DataflowConnectionImpl extends ModelConnectionImpl implements DataflowConnection {
	/**
	 * The default value of the '{@link #isIsDefaultValue() <em>Is Default Value</em>}' attribute. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #isIsDefaultValue()
	 * @generated
	 * @ordered
	 */
	protected static final boolean				IS_DEFAULT_VALUE_EDEFAULT			= false;
	
	/**
	 * The cached value of the '{@link #isIsDefaultValue() <em>Is Default Value</em>}' attribute. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #isIsDefaultValue()
	 * @generated
	 * @ordered
	 */
	protected boolean							isDefaultValue						= IS_DEFAULT_VALUE_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getCastType() <em>Cast Type</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getCastType()
	 * @generated
	 * @ordered
	 */
	protected static final SignalIntrinsicType	CAST_TYPE_EDEFAULT					= SignalIntrinsicType.UNDEFINED_LITERAL;
	
	/**
	 * The cached value of the '{@link #getCastType() <em>Cast Type</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getCastType()
	 * @generated
	 * @ordered
	 */
	protected SignalIntrinsicType				castType							= CAST_TYPE_EDEFAULT;
	
	/**
	 * The default value of the '{@link #isUseSrcClock() <em>Use Src Clock</em>}' attribute. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #isUseSrcClock()
	 * @generated
	 * @ordered
	 */
	protected static final boolean				USE_SRC_CLOCK_EDEFAULT				= false;
	
	/**
	 * The cached value of the '{@link #isUseSrcClock() <em>Use Src Clock</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #isUseSrcClock()
	 * @generated
	 * @ordered
	 */
	protected boolean							useSrcClock							= USE_SRC_CLOCK_EDEFAULT;
	
	/**
	 * The default value of the '{@link #isUseLastIterationValue() <em>Use Last Iteration Value</em>}' attribute. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #isUseLastIterationValue()
	 * @generated
	 * @ordered
	 */
	protected static final boolean				USE_LAST_ITERATION_VALUE_EDEFAULT	= false;
	
	/**
	 * The cached value of the '{@link #isUseLastIterationValue() <em>Use Last Iteration Value</em>}' attribute. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #isUseLastIterationValue()
	 * @generated
	 * @ordered
	 */
	protected boolean							useLastIterationValue				= USE_LAST_ITERATION_VALUE_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc() <em>Src</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc()
	 * @generated
	 * @ordered
	 */
	protected SignalExpr						src;
	
	/**
	 * The cached value of the '{@link #getDst() <em>Dst</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst()
	 * @generated
	 * @ordered
	 */
	protected DataflowConnector					dst;
	
	/**
	 * The default value of the '{@link #getSrcField() <em>Src Field</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSrcField()
	 * @generated
	 * @ordered
	 */
	protected static final String				SRC_FIELD_EDEFAULT					= "";
	
	/**
	 * The cached value of the '{@link #getSrcField() <em>Src Field</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSrcField()
	 * @generated
	 * @ordered
	 */
	protected String							srcField							= SRC_FIELD_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getDstField() <em>Dst Field</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getDstField()
	 * @generated
	 * @ordered
	 */
	protected static final String				DST_FIELD_EDEFAULT					= "";
	
	/**
	 * The cached value of the '{@link #getDstField() <em>Dst Field</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getDstField()
	 * @generated
	 * @ordered
	 */
	protected String							dstField							= DST_FIELD_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected DataflowConnectionImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.DATAFLOW_CONNECTION;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public boolean isIsDefaultValue() {
		return isDefaultValue;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setIsDefaultValue(boolean newIsDefaultValue) {
		boolean oldIsDefaultValue = isDefaultValue;
		isDefaultValue = newIsDefaultValue;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DATAFLOW_CONNECTION__IS_DEFAULT_VALUE,
				oldIsDefaultValue, isDefaultValue));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalIntrinsicType getCastType() {
		return castType;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setCastType(SignalIntrinsicType newCastType) {
		SignalIntrinsicType oldCastType = castType;
		castType = newCastType == null ? CAST_TYPE_EDEFAULT : newCastType;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DATAFLOW_CONNECTION__CAST_TYPE,
				oldCastType, castType));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public boolean isUseSrcClock() {
		return useSrcClock;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setUseSrcClock(boolean newUseSrcClock) {
		boolean oldUseSrcClock = useSrcClock;
		useSrcClock = newUseSrcClock;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DATAFLOW_CONNECTION__USE_SRC_CLOCK,
				oldUseSrcClock, useSrcClock));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public boolean isUseLastIterationValue() {
		return useLastIterationValue;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setUseLastIterationValue(boolean newUseLastIterationValue) {
		boolean oldUseLastIterationValue = useLastIterationValue;
		useLastIterationValue = newUseLastIterationValue;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET,
				SmePackage.DATAFLOW_CONNECTION__USE_LAST_ITERATION_VALUE, oldUseLastIterationValue,
				useLastIterationValue));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalExpr getSrc() {
		if (src != null && src.eIsProxy()) {
			InternalEObject oldSrc = (InternalEObject) src;
			src = (SignalExpr) eResolveProxy(oldSrc);
			if (src != oldSrc) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.DATAFLOW_CONNECTION__SRC,
						oldSrc, src));
			}
		}
		return src;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalExpr basicGetSrc() {
		return src;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSrc(SignalExpr newSrc, NotificationChain msgs) {
		SignalExpr oldSrc = src;
		src = newSrc;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.DATAFLOW_CONNECTION__SRC, oldSrc, newSrc);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSrc(SignalExpr newSrc) {
		if (newSrc != src) {
			NotificationChain msgs = null;
			if (src != null)
				msgs = ((InternalEObject) src).eInverseRemove(this, SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS,
					SignalExpr.class, msgs);
			if (newSrc != null)
				msgs = ((InternalEObject) newSrc).eInverseAdd(this, SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS,
					SignalExpr.class, msgs);
			msgs = basicSetSrc(newSrc, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DATAFLOW_CONNECTION__SRC, newSrc, newSrc));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public DataflowConnector getDst() {
		if (dst != null && dst.eIsProxy()) {
			InternalEObject oldDst = (InternalEObject) dst;
			dst = (DataflowConnector) eResolveProxy(oldDst);
			if (dst != oldDst) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.DATAFLOW_CONNECTION__DST,
						oldDst, dst));
			}
		}
		return dst;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public DataflowConnector basicGetDst() {
		return dst;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst(DataflowConnector newDst, NotificationChain msgs) {
		DataflowConnector oldDst = dst;
		dst = newDst;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.DATAFLOW_CONNECTION__DST, oldDst, newDst);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst(DataflowConnector newDst) {
		if (newDst != dst) {
			NotificationChain msgs = null;
			if (dst != null)
				msgs = ((InternalEObject) dst).eInverseRemove(this,
					SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS, DataflowConnector.class, msgs);
			if (newDst != null)
				msgs = ((InternalEObject) newDst).eInverseAdd(this,
					SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS, DataflowConnector.class, msgs);
			msgs = basicSetDst(newDst, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DATAFLOW_CONNECTION__DST, newDst, newDst));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getSrcField() {
		return srcField;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSrcField(String newSrcField) {
		String oldSrcField = srcField;
		srcField = newSrcField;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DATAFLOW_CONNECTION__SRC_FIELD,
				oldSrcField, srcField));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getDstField() {
		return dstField;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDstField(String newDstField) {
		String oldDstField = dstField;
		dstField = newDstField;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DATAFLOW_CONNECTION__DST_FIELD,
				oldDstField, dstField));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.DATAFLOW_CONNECTION__SRC:
				if (src != null)
					msgs = ((InternalEObject) src).eInverseRemove(this,
						SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS, SignalExpr.class, msgs);
				return basicSetSrc((SignalExpr) otherEnd, msgs);
			case SmePackage.DATAFLOW_CONNECTION__DST:
				if (dst != null)
					msgs = ((InternalEObject) dst).eInverseRemove(this,
						SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS, DataflowConnector.class, msgs);
				return basicSetDst((DataflowConnector) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.DATAFLOW_CONNECTION__SRC:
				return basicSetSrc(null, msgs);
			case SmePackage.DATAFLOW_CONNECTION__DST:
				return basicSetDst(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.DATAFLOW_CONNECTION__IS_DEFAULT_VALUE:
				return isIsDefaultValue() ? Boolean.TRUE : Boolean.FALSE;
			case SmePackage.DATAFLOW_CONNECTION__CAST_TYPE:
				return getCastType();
			case SmePackage.DATAFLOW_CONNECTION__USE_SRC_CLOCK:
				return isUseSrcClock() ? Boolean.TRUE : Boolean.FALSE;
			case SmePackage.DATAFLOW_CONNECTION__USE_LAST_ITERATION_VALUE:
				return isUseLastIterationValue() ? Boolean.TRUE : Boolean.FALSE;
			case SmePackage.DATAFLOW_CONNECTION__SRC:
				if (resolve)
					return getSrc();
				return basicGetSrc();
			case SmePackage.DATAFLOW_CONNECTION__DST:
				if (resolve)
					return getDst();
				return basicGetDst();
			case SmePackage.DATAFLOW_CONNECTION__SRC_FIELD:
				return getSrcField();
			case SmePackage.DATAFLOW_CONNECTION__DST_FIELD:
				return getDstField();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.DATAFLOW_CONNECTION__IS_DEFAULT_VALUE:
				setIsDefaultValue(((Boolean) newValue).booleanValue());
				return;
			case SmePackage.DATAFLOW_CONNECTION__CAST_TYPE:
				setCastType((SignalIntrinsicType) newValue);
				return;
			case SmePackage.DATAFLOW_CONNECTION__USE_SRC_CLOCK:
				setUseSrcClock(((Boolean) newValue).booleanValue());
				return;
			case SmePackage.DATAFLOW_CONNECTION__USE_LAST_ITERATION_VALUE:
				setUseLastIterationValue(((Boolean) newValue).booleanValue());
				return;
			case SmePackage.DATAFLOW_CONNECTION__SRC:
				setSrc((SignalExpr) newValue);
				return;
			case SmePackage.DATAFLOW_CONNECTION__DST:
				setDst((DataflowConnector) newValue);
				return;
			case SmePackage.DATAFLOW_CONNECTION__SRC_FIELD:
				setSrcField((String) newValue);
				return;
			case SmePackage.DATAFLOW_CONNECTION__DST_FIELD:
				setDstField((String) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.DATAFLOW_CONNECTION__IS_DEFAULT_VALUE:
				setIsDefaultValue(IS_DEFAULT_VALUE_EDEFAULT);
				return;
			case SmePackage.DATAFLOW_CONNECTION__CAST_TYPE:
				setCastType(CAST_TYPE_EDEFAULT);
				return;
			case SmePackage.DATAFLOW_CONNECTION__USE_SRC_CLOCK:
				setUseSrcClock(USE_SRC_CLOCK_EDEFAULT);
				return;
			case SmePackage.DATAFLOW_CONNECTION__USE_LAST_ITERATION_VALUE:
				setUseLastIterationValue(USE_LAST_ITERATION_VALUE_EDEFAULT);
				return;
			case SmePackage.DATAFLOW_CONNECTION__SRC:
				setSrc((SignalExpr) null);
				return;
			case SmePackage.DATAFLOW_CONNECTION__DST:
				setDst((DataflowConnector) null);
				return;
			case SmePackage.DATAFLOW_CONNECTION__SRC_FIELD:
				setSrcField(SRC_FIELD_EDEFAULT);
				return;
			case SmePackage.DATAFLOW_CONNECTION__DST_FIELD:
				setDstField(DST_FIELD_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.DATAFLOW_CONNECTION__IS_DEFAULT_VALUE:
				return isDefaultValue != IS_DEFAULT_VALUE_EDEFAULT;
			case SmePackage.DATAFLOW_CONNECTION__CAST_TYPE:
				return castType != CAST_TYPE_EDEFAULT;
			case SmePackage.DATAFLOW_CONNECTION__USE_SRC_CLOCK:
				return useSrcClock != USE_SRC_CLOCK_EDEFAULT;
			case SmePackage.DATAFLOW_CONNECTION__USE_LAST_ITERATION_VALUE:
				return useLastIterationValue != USE_LAST_ITERATION_VALUE_EDEFAULT;
			case SmePackage.DATAFLOW_CONNECTION__SRC:
				return src != null;
			case SmePackage.DATAFLOW_CONNECTION__DST:
				return dst != null;
			case SmePackage.DATAFLOW_CONNECTION__SRC_FIELD:
				return SRC_FIELD_EDEFAULT == null ? srcField != null : !SRC_FIELD_EDEFAULT.equals(srcField);
			case SmePackage.DATAFLOW_CONNECTION__DST_FIELD:
				return DST_FIELD_EDEFAULT == null ? dstField != null : !DST_FIELD_EDEFAULT.equals(dstField);
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (isDefaultValue: ");
		result.append(isDefaultValue);
		result.append(", castType: ");
		result.append(castType);
		result.append(", useSrcClock: ");
		result.append(useSrcClock);
		result.append(", useLastIterationValue: ");
		result.append(useLastIterationValue);
		result.append(", srcField: ");
		result.append(srcField);
		result.append(", dstField: ");
		result.append(dstField);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#getDestination()
	 * @generated NOT
	 */
	@Override
	public ConnectableElement getDestination() {
		return getDst();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#getSource()
	 * @generated NOT
	 */
	@Override
	public ConnectableElement getSource() {
		return getSrc();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#buildEquationsAST(int)
	 * @generated NOT
	 */
	@Override
	public long buildEquationsAST(long bodyTree) {
		DataflowConnector dst = getDst();
		// If the destination corresponds to a Signal or a SignalRef, it creates a definition or partial definition
		// equation for the targeted signal
		// else the destination is an Input Port of an operator, so this method do not have to be called for this
		// purpose (the good method is getAST()).
		if (dst instanceof SignalOrRef) {
			int opCode;
			if (((SignalOrRef) dst).isPartiallyDefined())
				opCode = isDefaultValue ? SignalOperators.opdefpartdef : SignalOperators.opdefpart;
			else
				opCode = SignalOperators.opdefsig;
			
			try {
				long dstNode;
				
				String tmp;
				// Check on Identifier and not on Signal because it can be an InputInstance or OutputInstance
				if (dst instanceof Identifier)
					tmp = ((Identifier) dst).getName();
				else
					tmp = ((SignalRef) dst).getSigName();
				
				if (tmp == null || "".equals(tmp))
					return PKTrees.ERROR_NODE;
				
				// Add destination field information for tuple type identifiers
				if (dstField != null && "".equals(dstField) == false) {
					tmp = Utils.getSignalString(tmp, dstField);
					
					dstNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, tmp);
					if (dstNode == PKTrees.ERROR_NODE) {
						PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.DESTINATION_FIELD, this);
						return bodyTree;
					}
				}
				else
					dstNode = PkPlugin.getTreeAPI().atomString(SignalOperators.opnom, tmp);
				
				long srcNode = getAST(new HashSet<SignalElement>());
				if (srcNode != PKTrees.ERROR_NODE)
					bodyTree = PkPlugin.getTreeAPI().post(bodyTree,
						PkPlugin.getTreeAPI().makeBinary(opCode, dstNode, srcNode));
			}
			catch (CyclicDefinitionException cde) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) throws CyclicDefinitionException {
		long srcNode;
		
		if ((getSrc() instanceof SignalOrRef || getSrc() instanceof ConstantOrRef) && srcField != null
				&& "".equals(srcField) == false) {
			String tmp;
			// Check on Identifier and not on Signal because it can be an InputInstance or OutputInstance
			if (src instanceof Identifier)
				tmp = ((Identifier) src).getName();
			else if (src instanceof SignalRef)
				tmp = ((SignalRef) src).getSigName();
			else if (src instanceof ConstantRef)
				tmp = ((ConstantRef) src).getCstName();
			else
				tmp = null;
			
			if (tmp == null || "".equals(tmp)) {
				return PKTrees.ERROR_NODE;
			}
			if (isUseLastIterationValue()) {
				tmp += "[?]";
			}
			tmp = Utils.getSignalString(tmp, srcField);
			
			srcNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, tmp);
			if (srcNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.SOURCE_FIELD, this);
			}
		}
		else {
			if (srcField != null && "".equals(srcField) == false)
				PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.SOURCE_FIELD, this);
			srcNode = src.getAST(path);
		}
		
		if (srcNode == PKTrees.ERROR_NODE) {
			return PKTrees.ERROR_NODE;
		}
		
		if (!(getDst() instanceof SignalOrRef)) {
			if (isIsDefaultValue())
				PolychronyDiagnostic
						.add(Diagnostic.WARNING, WarningMessages.DATAFLOW_CONNECTION_IS_DEFAULT_VALUE, this);
		}
		
		PKTrees API = PkPlugin.getTreeAPI();
		
		// if (isUseLastIterationValue())
		// srcNode = API.makeUnary(SignalOperators.opsdec, srcNode);
		
		if (castType != CAST_TYPE_EDEFAULT) {
			long typeNode;
			if (castType.equals(SignalIntrinsicType.BOOLEAN_LITERAL))
				typeNode = API.makeNullary(SignalOperators.oplogical);
			else if (castType.equals(SignalIntrinsicType.CHAR_LITERAL))
				typeNode = API.makeNullary(SignalOperators.opchar);
			else if (castType.equals(SignalIntrinsicType.COMPLEX_LITERAL))
				typeNode = API.makeNullary(SignalOperators.opcomplex);
			else if (castType.equals(SignalIntrinsicType.DCOMPLEX_LITERAL))
				typeNode = API.makeNullary(SignalOperators.opdcomplex);
			else if (castType.equals(SignalIntrinsicType.DREAL_LITERAL))
				typeNode = API.makeNullary(SignalOperators.opdpreal);
			else if (castType.equals(SignalIntrinsicType.EVENT_LITERAL))
				typeNode = API.makeNullary(SignalOperators.opevent);
			else if (castType.equals(SignalIntrinsicType.INTEGER_LITERAL))
				typeNode = API.makeNullary(SignalOperators.opinteger);
			else if (castType.equals(SignalIntrinsicType.LONG_LITERAL))
				typeNode = API.makeNullary(SignalOperators.oplong);
			else if (castType.equals(SignalIntrinsicType.REAL_LITERAL))
				typeNode = API.makeNullary(SignalOperators.opreal);
			else if (castType.equals(SignalIntrinsicType.SHORT_LITERAL))
				typeNode = API.makeNullary(SignalOperators.opshort);
			else
				// it is a string
				typeNode = API.makeNullary(SignalOperators.opstring);
			
			srcNode = API.makeBinary(SignalOperators.opconv, typeNode, srcNode);
		}
		
		if (isUseSrcClock())
			srcNode = API.makeUnary(SignalOperators.opevnt, srcNode);
		return srcNode;
	}
} // DataflowConnectionImpl
