/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionnedExpr;
import fr.irisa.espresso.sme.ControlflowOperator;
import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.DependenceOperator;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Dependence Operator</b></em>'. <!--
 * end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.DependenceOperatorImpl#getSrc_Dependences <em>Src Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DependenceOperatorImpl#getDst_Dependences <em>Dst Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DependenceOperatorImpl#getDst_Condition <em>Dst Condition</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DependenceOperatorImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.DependenceOperatorImpl#getModelParent <em>Model Parent</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class DependenceOperatorImpl extends EObjectImpl implements DependenceOperator {
	/**
	 * The cached value of the '{@link #getSrc_Dependences() <em>Src Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>		src_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_Dependences() <em>Dst Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>		dst_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_Condition() <em>Dst Condition</em>}' reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_Condition()
	 * @generated
	 * @ordered
	 */
	protected EList<Condition>		dst_Condition;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String	NAME_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String				name			= NAME_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected DependenceOperatorImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.DEPENDENCE_OPERATOR;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getSrc_Dependences() {
		if (src_Dependences == null) {
			src_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.DEPENDENCE_OPERATOR__SRC_DEPENDENCES, SmePackage.DEPENDENCE__SRC);
		}
		return src_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getDst_Dependences() {
		if (dst_Dependences == null) {
			dst_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.DEPENDENCE_OPERATOR__DST_DEPENDENCES, SmePackage.DEPENDENCE__DST);
		}
		return dst_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Condition> getDst_Condition() {
		if (dst_Condition == null) {
			dst_Condition = new EObjectWithInverseResolvingEList<Condition>(Condition.class, this,
				SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION, SmePackage.CONDITION__DST);
		}
		return dst_Condition;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DEPENDENCE_OPERATOR__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__CONSTRAINTS, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT,
				newModelParent, newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getModelParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.DEPENDENCE_OPERATOR__SRC_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.DEPENDENCE_OPERATOR__DST_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_Condition())
						.basicAdd(otherEnd, msgs);
			case SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.DEPENDENCE_OPERATOR__SRC_DEPENDENCES:
				return ((InternalEList<?>) getSrc_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.DEPENDENCE_OPERATOR__DST_DEPENDENCES:
				return ((InternalEList<?>) getDst_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION:
				return ((InternalEList<?>) getDst_Condition()).basicRemove(otherEnd, msgs);
			case SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__CONSTRAINTS, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.DEPENDENCE_OPERATOR__SRC_DEPENDENCES:
				return getSrc_Dependences();
			case SmePackage.DEPENDENCE_OPERATOR__DST_DEPENDENCES:
				return getDst_Dependences();
			case SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION:
				return getDst_Condition();
			case SmePackage.DEPENDENCE_OPERATOR__NAME:
				return getName();
			case SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT:
				return getModelParent();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.DEPENDENCE_OPERATOR__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				getSrc_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.DEPENDENCE_OPERATOR__DST_DEPENDENCES:
				getDst_Dependences().clear();
				getDst_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION:
				getDst_Condition().clear();
				getDst_Condition().addAll((Collection<? extends Condition>) newValue);
				return;
			case SmePackage.DEPENDENCE_OPERATOR__NAME:
				setName((String) newValue);
				return;
			case SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.DEPENDENCE_OPERATOR__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				return;
			case SmePackage.DEPENDENCE_OPERATOR__DST_DEPENDENCES:
				getDst_Dependences().clear();
				return;
			case SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION:
				getDst_Condition().clear();
				return;
			case SmePackage.DEPENDENCE_OPERATOR__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT:
				setModelParent((Model) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.DEPENDENCE_OPERATOR__SRC_DEPENDENCES:
				return src_Dependences != null && !src_Dependences.isEmpty();
			case SmePackage.DEPENDENCE_OPERATOR__DST_DEPENDENCES:
				return dst_Dependences != null && !dst_Dependences.isEmpty();
			case SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION:
				return dst_Condition != null && !dst_Condition.isEmpty();
			case SmePackage.DEPENDENCE_OPERATOR__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT:
				return getModelParent() != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConditionnedExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION:
					return SmePackage.CONDITIONNED_EXPR__DST_CONDITION;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.DEPENDENCE_OPERATOR__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == ControlflowOperator.class) {
			switch (derivedFeatureID) {
				case SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT:
					return SmePackage.CONTROLFLOW_OPERATOR__MODEL_PARENT;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConditionnedExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONDITIONNED_EXPR__DST_CONDITION:
					return SmePackage.DEPENDENCE_OPERATOR__DST_CONDITION;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.DEPENDENCE_OPERATOR__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == ControlflowOperator.class) {
			switch (baseFeatureID) {
				case SmePackage.CONTROLFLOW_OPERATOR__MODEL_PARENT:
					return SmePackage.DEPENDENCE_OPERATOR__MODEL_PARENT;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		Set<SignalElement> path = new HashSet<SignalElement>();
		PKTrees API = PkPlugin.getTreeAPI();
		
		long srcNode = API.makeEmptyList(SignalOperators.oplesign);
		boolean dstEmpty = true;
		for (Dependence dep : getDst_Dependences()) {
			try {
				long node = dep.getAST(path);
				if (node != PKTrees.ERROR_NODE) {
					srcNode = API.post(srcNode, node);
					dstEmpty = false;
				}
			}
			catch (CyclicDefinitionException cde) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
			path.clear();
		}
		
		long dstNode = API.makeEmptyList(SignalOperators.oplesign);
		boolean srcEmpty = true;
		for (Dependence dep : getSrc_Dependences()) {
			long node = dep.getDstAST();
			if (node != PKTrees.ERROR_NODE) {
				dstNode = API.post(dstNode, node);
				srcEmpty = false;
			}
		}
		
		if (srcEmpty || dstEmpty)
			return bodyTree;
		
		if (!getDst_Condition().isEmpty()) {
			long condNode = PKTrees.ERROR_NODE;
			try {
				for (Condition cd : dst_Condition) {
					if (condNode == PKTrees.ERROR_NODE)
						condNode = cd.getAST(path);
					else {
						long tmp = cd.getAST(path);
						if (tmp != PKTrees.ERROR_NODE)
							condNode = PkPlugin.getTreeAPI().makeBinary(SignalOperators.opextr, condNode, tmp);
					}
				}
				
				if (condNode != PKTrees.ERROR_NODE) {
					bodyTree = API.post(bodyTree, API.makeTernary(SignalOperators.opdepd, srcNode, dstNode, condNode));
				}
			}
			catch (CyclicDefinitionException cde) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		else {
			bodyTree = API.post(bodyTree, API.pre(API.makeList(SignalOperators.opldep, dstNode), srcNode));
		}
		
		return bodyTree;
	}
	
	/**
	 * This method is not supported. Call the buildEquationAST method.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * Nothing to declare. The declaration tree is returned without being modified.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
	
} // DependenceOperatorImpl
