/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.EDataTypeEList;
import org.eclipse.emf.ecore.util.EDataTypeUniqueEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.EnumerationType;
import fr.irisa.espresso.sme.Module;
import fr.irisa.espresso.sme.ParameterType;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalType;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.VisibilityKind;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Enumeration Type</b></em>'. <!-- end-user-doc
 * -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.EnumerationTypeImpl#getSrc_ConstantAffectations <em>Src Constant Affectations
 * </em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.EnumerationTypeImpl#getEnumValues <em>Enum Values</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.EnumerationTypeImpl#getArrayDimensions <em>Array Dimensions</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class EnumerationTypeImpl extends TypeDeclarationImpl implements EnumerationType {
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The cached value of the '{@link #getArrayDimensions() <em>Array Dimensions</em>}' attribute list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getArrayDimensions()
	 * @generated
	 * @ordered
	 */
	protected EList<String>					arrayDimensions;
	
	/**
	 * The cached value of the '{@link #getEnumValues() <em>Enum Values</em>}' attribute list. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getEnumValues()
	 * @generated
	 * @ordered
	 */
	protected EList<String>					enumValues;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected EnumerationTypeImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.ENUMERATION_TYPE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getEnumValues() {
		if (enumValues == null) {
			enumValues = new EDataTypeUniqueEList<String>(String.class, this, SmePackage.ENUMERATION_TYPE__ENUM_VALUES);
		}
		return enumValues;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getArrayDimensions() {
		if (arrayDimensions == null) {
			arrayDimensions = new EDataTypeEList<String>(String.class, this,
				SmePackage.ENUMERATION_TYPE__ARRAY_DIMENSIONS);
		}
		return arrayDimensions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.ENUMERATION_TYPE__ARRAY_DIMENSIONS:
				return getArrayDimensions();
			case SmePackage.ENUMERATION_TYPE__ENUM_VALUES:
				return getEnumValues();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.ENUMERATION_TYPE__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				getArrayDimensions().addAll((Collection<? extends String>) newValue);
				return;
			case SmePackage.ENUMERATION_TYPE__ENUM_VALUES:
				getEnumValues().clear();
				getEnumValues().addAll((Collection<? extends String>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.ENUMERATION_TYPE__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				return;
			case SmePackage.ENUMERATION_TYPE__ENUM_VALUES:
				getEnumValues().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.ENUMERATION_TYPE__ARRAY_DIMENSIONS:
				return arrayDimensions != null && !arrayDimensions.isEmpty();
			case SmePackage.ENUMERATION_TYPE__ENUM_VALUES:
				return enumValues != null && !enumValues.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ParameterType.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == SignalType.class) {
			switch (derivedFeatureID) {
				case SmePackage.ENUMERATION_TYPE__ARRAY_DIMENSIONS:
					return SmePackage.SIGNAL_TYPE__ARRAY_DIMENSIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ParameterType.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.ENUMERATION_TYPE__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == SignalType.class) {
			switch (baseFeatureID) {
				case SmePackage.SIGNAL_TYPE__ARRAY_DIMENSIONS:
					return SmePackage.ENUMERATION_TYPE__ARRAY_DIMENSIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (arrayDimensions: ");
		result.append(arrayDimensions);
		result.append(", enumValues: ");
		result.append(enumValues);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
				result.append("private ");
			result.append("type ");
			result.append(name);
			result.append(" = ");
			if (getArrayDimensions().size() > 0) {
				result.append("[");
				Iterator<String> it = arrayDimensions.iterator();
				result.append(it.next());
				while (it.hasNext()) {
					result.append(", ");
					result.append(it.next());
				}
				result.append("]");
			}
			result.append(" enum(");
			for (String st : getEnumValues()) {
				result.append(st);
				result.append(",\n   ");
			}
			result.append(")");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * 
	 * An enumeration declaration does not define equations in the body of its parent.
	 * 
	 * @see fr.irisa.espresso.sme.Identifier#buildEquationsAST(int)
	 * 
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long decl;
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			
			if (!getEnumValues().isEmpty()) {
				// Generate the tree concerning the enumeration values
				long enumNode = API.makeEmptyList(SignalOperators.optenum);
				
				for (String st : getEnumValues()) {
					if (st == null || "".equals(st))
						PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.ENUMERATION_TYPE_NO_VALUE, this);
					else {
						long node = PkPlugin.getParser().parseString(Phyla.ph_ident, st);
						if (node == PKTrees.ERROR_NODE)
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.ENUM_VALUE + st, this);
						else
							enumNode = API.post(enumNode, node);
					}
				}
				
				// Add the specification of arrayDimensions, if any. Parse the textual expression
				if (!getArrayDimensions().isEmpty()) {
					long arrayNode = API.makeEmptyList(SignalOperators.oplesign);
					boolean allOk = true;
					for (String st : arrayDimensions) {
						long exprNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
						if (exprNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.ARRAY_DIMENSION_VALUE + st, this);
							allOk = false;
						}
						else
							arrayNode = API.post(arrayNode, exprNode);
					}
					if (allOk)
						enumNode = API.makeBinary(SignalOperators.optvect, arrayNode, enumNode);
				}
				
				decl = API
						.makeTernary(SignalOperators.optype, idNode, enumNode, API.makeNullary(SignalOperators.opnil));
			}
			else {
				PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.ENUMERATION_TYPE_UNDEFINED_ENUM, this);
				decl = idNode;
			}
			
			if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
				decl = API.makeList(SignalOperators.opltypep, decl);
			else
				decl = API.makeList(SignalOperators.opltype, decl);
			
			declTree = API.post(declTree, decl);
		}
		return declTree;
	}
	
} // EnumerationTypeImpl
