/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.irisa.espresso.sme.EnumerationValue;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Enumeration Value</b></em>'. <!-- end-user-doc
 * -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.EnumerationValueImpl#getTypeValue <em>Type Value</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class EnumerationValueImpl extends LiteralImpl implements EnumerationValue {
	/**
	 * The default value of the '{@link #getTypeValue() <em>Type Value</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getTypeValue()
	 * @generated
	 * @ordered
	 */
	protected static final String	TYPE_VALUE_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getTypeValue() <em>Type Value</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getTypeValue()
	 * @generated
	 * @ordered
	 */
	protected String				typeValue			= TYPE_VALUE_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected EnumerationValueImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.ENUMERATION_VALUE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getTypeValue() {
		return typeValue;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setTypeValue(String newTypeValue) {
		String oldTypeValue = typeValue;
		typeValue = newTypeValue;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.ENUMERATION_VALUE__TYPE_VALUE,
				oldTypeValue, typeValue));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.ENUMERATION_VALUE__TYPE_VALUE:
				return getTypeValue();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.ENUMERATION_VALUE__TYPE_VALUE:
				setTypeValue((String) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.ENUMERATION_VALUE__TYPE_VALUE:
				setTypeValue(TYPE_VALUE_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.ENUMERATION_VALUE__TYPE_VALUE:
				return TYPE_VALUE_EDEFAULT == null ? typeValue != null : !TYPE_VALUE_EDEFAULT.equals(typeValue);
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (typeValue: ");
		result.append(typeValue);
		result.append(')');
		return result.toString();
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 */
	public long getAST(Set<SignalElement> path) {
		PKTrees API = PkPlugin.getTreeAPI();
		// By default, the value of the type is not needed, so it will be a nil
		long typeNode = API.makeNullary(SignalOperators.opnil);
		if (typeValue != null && "".equals(typeValue) == false) {
			long tmp = PkPlugin.getParser().parseString(Phyla.ph_ident, typeValue);
			if (tmp == PKTrees.ERROR_NODE)
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.ENUMERATION_VALUE_INCORRECT_NAME, this);
			else
				typeNode = tmp;
		}
		long valueNode = PkPlugin.getParser().parseString(Phyla.ph_ident, value);
		if (valueNode == PKTrees.ERROR_NODE)
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.LITERAL_VALUE, this);
		else
			valueNode = API.makeBinary(SignalOperators.opvenum, typeNode, valueNode);
		
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, valueNode);
		return valueNode;
	}
	
} // EnumerationValueImpl
