/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.Index;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Index</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getSrc_ConstantAffectations <em>Src Constant Affectations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getDst_ConstantAffectation <em>Dst Constant Affectation</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getFrom <em>From</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getTo <em>To</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IndexImpl#getStep <em>Step</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class IndexImpl extends EObjectImpl implements Index {
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>		src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String			NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String						name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String			COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String						comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getDst_ConstantAffectation() <em>Dst Constant Affectation</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ConstantAffectation()
	 * @generated
	 * @ordered
	 */
	protected ConstantAffectation			dst_ConstantAffectation;
	
	/**
	 * The default value of the '{@link #getFrom() <em>From</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getFrom()
	 * @generated
	 * @ordered
	 */
	protected static final String			FROM_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getFrom() <em>From</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getFrom()
	 * @generated
	 * @ordered
	 */
	protected String						from				= FROM_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getTo() <em>To</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getTo()
	 * @generated
	 * @ordered
	 */
	protected static final String			TO_EDEFAULT			= null;
	
	/**
	 * The cached value of the '{@link #getTo() <em>To</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getTo()
	 * @generated
	 * @ordered
	 */
	protected String						to					= TO_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getStep() <em>Step</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getStep()
	 * @generated
	 * @ordered
	 */
	protected static final String			STEP_EDEFAULT		= "1";
	
	/**
	 * The cached value of the '{@link #getStep() <em>Step</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getStep()
	 * @generated
	 * @ordered
	 */
	protected String						step				= STEP_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected IndexImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.INDEX;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.INDEX__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INDEX__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INDEX__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation getDst_ConstantAffectation() {
		if (dst_ConstantAffectation != null && dst_ConstantAffectation.eIsProxy()) {
			InternalEObject oldDst_ConstantAffectation = (InternalEObject) dst_ConstantAffectation;
			dst_ConstantAffectation = (ConstantAffectation) eResolveProxy(oldDst_ConstantAffectation);
			if (dst_ConstantAffectation != oldDst_ConstantAffectation) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE,
						SmePackage.INDEX__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation, dst_ConstantAffectation));
			}
		}
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation basicGetDst_ConstantAffectation() {
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation,
			NotificationChain msgs) {
		ConstantAffectation oldDst_ConstantAffectation = dst_ConstantAffectation;
		dst_ConstantAffectation = newDst_ConstantAffectation;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.INDEX__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation, newDst_ConstantAffectation);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation) {
		if (newDst_ConstantAffectation != dst_ConstantAffectation) {
			NotificationChain msgs = null;
			if (dst_ConstantAffectation != null)
				msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			if (newDst_ConstantAffectation != null)
				msgs = ((InternalEObject) newDst_ConstantAffectation).eInverseAdd(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			msgs = basicSetDst_ConstantAffectation(newDst_ConstantAffectation, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INDEX__DST_CONSTANT_AFFECTATION,
				newDst_ConstantAffectation, newDst_ConstantAffectation));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.INDEX__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.INDEX__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.INDEX__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__INDEXES, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INDEX__MODEL_PARENT, newModelParent,
				newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getFrom() {
		return from;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setFrom(String newFrom) {
		String oldFrom = from;
		from = newFrom;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INDEX__FROM, oldFrom, from));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getTo() {
		return to;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setTo(String newTo) {
		String oldTo = to;
		to = newTo;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INDEX__TO, oldTo, to));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getStep() {
		return step;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setStep(String newStep) {
		String oldStep = step;
		step = newStep;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INDEX__STEP, oldStep, step));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getModelParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.INDEX__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
			case SmePackage.INDEX__DST_CONSTANT_AFFECTATION:
				if (dst_ConstantAffectation != null)
					msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
						SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
				return basicSetDst_ConstantAffectation((ConstantAffectation) otherEnd, msgs);
			case SmePackage.INDEX__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.INDEX__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
			case SmePackage.INDEX__DST_CONSTANT_AFFECTATION:
				return basicSetDst_ConstantAffectation(null, msgs);
			case SmePackage.INDEX__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.INDEX__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__INDEXES, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.INDEX__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.INDEX__NAME:
				return getName();
			case SmePackage.INDEX__COMMENT:
				return getComment();
			case SmePackage.INDEX__DST_CONSTANT_AFFECTATION:
				if (resolve)
					return getDst_ConstantAffectation();
				return basicGetDst_ConstantAffectation();
			case SmePackage.INDEX__MODEL_PARENT:
				return getModelParent();
			case SmePackage.INDEX__FROM:
				return getFrom();
			case SmePackage.INDEX__TO:
				return getTo();
			case SmePackage.INDEX__STEP:
				return getStep();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.INDEX__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.INDEX__NAME:
				setName((String) newValue);
				return;
			case SmePackage.INDEX__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.INDEX__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) newValue);
				return;
			case SmePackage.INDEX__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.INDEX__FROM:
				setFrom((String) newValue);
				return;
			case SmePackage.INDEX__TO:
				setTo((String) newValue);
				return;
			case SmePackage.INDEX__STEP:
				setStep((String) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.INDEX__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.INDEX__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.INDEX__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.INDEX__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) null);
				return;
			case SmePackage.INDEX__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.INDEX__FROM:
				setFrom(FROM_EDEFAULT);
				return;
			case SmePackage.INDEX__TO:
				setTo(TO_EDEFAULT);
				return;
			case SmePackage.INDEX__STEP:
				setStep(STEP_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.INDEX__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.INDEX__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.INDEX__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.INDEX__DST_CONSTANT_AFFECTATION:
				return dst_ConstantAffectation != null;
			case SmePackage.INDEX__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.INDEX__FROM:
				return FROM_EDEFAULT == null ? from != null : !FROM_EDEFAULT.equals(from);
			case SmePackage.INDEX__TO:
				return TO_EDEFAULT == null ? to != null : !TO_EDEFAULT.equals(to);
			case SmePackage.INDEX__STEP:
				return STEP_EDEFAULT == null ? step != null : !STEP_EDEFAULT.equals(step);
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.INDEX__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.INDEX__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.INDEX__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.INDEX__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.INDEX__COMMENT;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(", from: ");
		result.append(from);
		result.append(", to: ");
		result.append(to);
		result.append(", step: ");
		result.append(step);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			result.append(name);
			result.append(" := ");
			if (!(from == null || "".equals(from)))
				result.append(from);
			else
				result.append("0");
			result.append(" .. ");
			result.append(to);
			result.append(" step ");
			if (!(step == null || "".equals(step)))
				result.append(step);
			else
				result.append("1");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic
						.add(
							Diagnostic.ERROR,
							"The name of the Index '"
									+ name
									+ "' is not correct. Please, check that it is not a keyword of the language or a name with spaces.",
							this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, "The name of the Index must be set.", this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#buildEquationsAST(int)
	 * 
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		PKTrees API = PkPlugin.getTreeAPI();
		
		if (!(from == null || "".equals(from) || to == null || "".equals(to))) {
			long fromNode = PkPlugin.getParser().parseString(Phyla.ph_o_e_sign, from);
			if (fromNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INDEX_FROM, this);
				return bodyTree;
			}
			
			long toNode = PkPlugin.getParser().parseString(Phyla.ph_o_e_sign, to);
			if (toNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INDEX_TO, this);
				return bodyTree;
			}
			
			long stepNode;
			if (step == null || "".equals(step)) {
				stepNode = PkPlugin.getParser().parseString(Phyla.ph_o_e_sign, step);
				if (stepNode == PKTrees.ERROR_NODE) {
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INDEX_STEP, this);
					return bodyTree;
				}
			}
			else
				stepNode = API.makeNullary(SignalOperators.opnil);
			
			long idNode = getAST(null);
			if (idNode != PKTrees.ERROR_NODE) {
				long indexNode = API.makeTernary(SignalOperators.openum, fromNode, toNode, stepNode);
				long equationNode = API.makeBinary(SignalOperators.opdefsig, idNode, indexNode);
				bodyTree = API.post(bodyTree, equationNode);
			}
		}
		else
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INDEX_FROM_TO, this);
		return bodyTree;
	}
	
	/**
	 * Returns the AST of the declaration of an index, which is an array of integer value
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		// The index has to be declared as an integer array to avoid type error when the index is used
		long idNode = getAST(null);
		
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecsig, API
					.makeNullary(SignalOperators.opimplicit), API.makeList(SignalOperators.oplsign, idNode)));
			
		}
		return declTree;
	}
} // IndexImpl
