/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ClockedExpr;
import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionExpr;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.DataflowConnector;
import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.DependenceInOut;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.Input;
import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalIntrinsicType;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Input Instance</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}
 * </li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getSrc_ClockRelations <em>Src Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getDst_ClockRelations <em>Dst Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getSrc_Dependences <em>Src Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getDst_Dependences <em>Dst Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getDst_DataflowConnections <em>Dst Dataflow Connections</em>}
 * </li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getSrc_Conditions <em>Src Conditions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getInput <em>Input</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputInstanceImpl#getInstanceParent <em>Instance Parent</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class InputInstanceImpl extends EObjectImpl implements InputInstance {
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>	src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_ClockRelations() <em>Src Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		src_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getDst_ClockRelations() <em>Dst Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		dst_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getSrc_Dependences() <em>Src Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>			src_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_Dependences() <em>Dst Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>			dst_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_DataflowConnections() <em>Dst Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>	dst_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_Conditions() <em>Src Conditions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Conditions()
	 * @generated
	 * @ordered
	 */
	protected EList<Condition>			src_Conditions;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String		NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String					name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String		COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String					comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getInput() <em>Input</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getInput()
	 * @generated
	 * @ordered
	 */
	protected Input						input;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected InputInstanceImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.INPUT_INSTANCE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.INPUT_INSTANCE__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getSrc_ClockRelations() {
		if (src_ClockRelations == null) {
			src_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__SRC);
		}
		return src_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getDst_ClockRelations() {
		if (dst_ClockRelations == null) {
			dst_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__DST);
		}
		return dst_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getSrc_Dependences() {
		if (src_Dependences == null) {
			src_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES, SmePackage.DEPENDENCE__SRC);
		}
		return src_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getDst_Dependences() {
		if (dst_Dependences == null) {
			dst_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.INPUT_INSTANCE__DST_DEPENDENCES, SmePackage.DEPENDENCE__DST);
		}
		return dst_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getDst_DataflowConnections() {
		if (dst_DataflowConnections == null) {
			dst_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__DST);
		}
		return dst_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Condition> getSrc_Conditions() {
		if (src_Conditions == null) {
			src_Conditions = new EObjectWithInverseResolvingEList<Condition>(Condition.class, this,
				SmePackage.INPUT_INSTANCE__SRC_CONDITIONS, SmePackage.CONDITION__SRC);
		}
		return src_Conditions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INPUT_INSTANCE__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INPUT_INSTANCE__COMMENT, oldComment,
				comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Input getInput() {
		if (input != null && input.eIsProxy()) {
			InternalEObject oldInput = (InternalEObject) input;
			input = (Input) eResolveProxy(oldInput);
			if (input != oldInput) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.INPUT_INSTANCE__INPUT,
						oldInput, input));
			}
		}
		return input;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Input basicGetInput() {
		return input;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setInput(Input newInput) {
		Input oldInput = input;
		input = newInput;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INPUT_INSTANCE__INPUT, oldInput, input));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelInstance getInstanceParent() {
		if (eContainerFeatureID != SmePackage.INPUT_INSTANCE__INSTANCE_PARENT)
			return null;
		return (ModelInstance) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetInstanceParent(ModelInstance newInstanceParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newInstanceParent, SmePackage.INPUT_INSTANCE__INSTANCE_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setInstanceParent(ModelInstance newInstanceParent) {
		if (newInstanceParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.INPUT_INSTANCE__INSTANCE_PARENT && newInstanceParent != null)) {
			if (EcoreUtil.isAncestor(this, newInstanceParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newInstanceParent != null)
				msgs = ((InternalEObject) newInstanceParent).eInverseAdd(this, SmePackage.MODEL_INSTANCE__INPUTS,
					ModelInstance.class, msgs);
			msgs = basicSetInstanceParent(newInstanceParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INPUT_INSTANCE__INSTANCE_PARENT,
				newInstanceParent, newInstanceParent));
	}
	
	/**
	 * This method returns true if the current signal uses only partial definition
	 * 
	 * @generated NOT
	 */
	public boolean isPartiallyDefined() {
		// Only local signal with status equal to shared or statevar can be partially defined
		return false;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.INPUT_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.INPUT_INSTANCE__DST_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__SRC_CONDITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Conditions()).basicAdd(otherEnd,
					msgs);
			case SmePackage.INPUT_INSTANCE__INSTANCE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetInstanceParent((ModelInstance) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.INPUT_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS:
				return ((InternalEList<?>) getSrc_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS:
				return ((InternalEList<?>) getDst_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES:
				return ((InternalEList<?>) getSrc_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__DST_DEPENDENCES:
				return ((InternalEList<?>) getDst_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getDst_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__SRC_CONDITIONS:
				return ((InternalEList<?>) getSrc_Conditions()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_INSTANCE__INSTANCE_PARENT:
				return basicSetInstanceParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.INPUT_INSTANCE__INSTANCE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL_INSTANCE__INPUTS,
					ModelInstance.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.INPUT_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS:
				return getSrc_ClockRelations();
			case SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS:
				return getDst_ClockRelations();
			case SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES:
				return getSrc_Dependences();
			case SmePackage.INPUT_INSTANCE__DST_DEPENDENCES:
				return getDst_Dependences();
			case SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS:
				return getDst_DataflowConnections();
			case SmePackage.INPUT_INSTANCE__SRC_CONDITIONS:
				return getSrc_Conditions();
			case SmePackage.INPUT_INSTANCE__NAME:
				return getName();
			case SmePackage.INPUT_INSTANCE__COMMENT:
				return getComment();
			case SmePackage.INPUT_INSTANCE__INPUT:
				if (resolve)
					return getInput();
				return basicGetInput();
			case SmePackage.INPUT_INSTANCE__INSTANCE_PARENT:
				return getInstanceParent();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.INPUT_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				getSrc_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				getDst_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				getSrc_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__DST_DEPENDENCES:
				getDst_Dependences().clear();
				getDst_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS:
				getDst_DataflowConnections().clear();
				getDst_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				getSrc_Conditions().addAll((Collection<? extends Condition>) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__NAME:
				setName((String) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__INPUT:
				setInput((Input) newValue);
				return;
			case SmePackage.INPUT_INSTANCE__INSTANCE_PARENT:
				setInstanceParent((ModelInstance) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.INPUT_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				return;
			case SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				return;
			case SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				return;
			case SmePackage.INPUT_INSTANCE__DST_DEPENDENCES:
				getDst_Dependences().clear();
				return;
			case SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS:
				getDst_DataflowConnections().clear();
				return;
			case SmePackage.INPUT_INSTANCE__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				return;
			case SmePackage.INPUT_INSTANCE__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.INPUT_INSTANCE__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.INPUT_INSTANCE__INPUT:
				setInput((Input) null);
				return;
			case SmePackage.INPUT_INSTANCE__INSTANCE_PARENT:
				setInstanceParent((ModelInstance) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.INPUT_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS:
				return src_ClockRelations != null && !src_ClockRelations.isEmpty();
			case SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS:
				return dst_ClockRelations != null && !dst_ClockRelations.isEmpty();
			case SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES:
				return src_Dependences != null && !src_Dependences.isEmpty();
			case SmePackage.INPUT_INSTANCE__DST_DEPENDENCES:
				return dst_Dependences != null && !dst_Dependences.isEmpty();
			case SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS:
				return dst_DataflowConnections != null && !dst_DataflowConnections.isEmpty();
			case SmePackage.INPUT_INSTANCE__SRC_CONDITIONS:
				return src_Conditions != null && !src_Conditions.isEmpty();
			case SmePackage.INPUT_INSTANCE__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.INPUT_INSTANCE__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.INPUT_INSTANCE__INPUT:
				return input != null;
			case SmePackage.INPUT_INSTANCE__INSTANCE_PARENT:
				return getInstanceParent() != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ClockedExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS:
					return SmePackage.CLOCKED_EXPR__SRC_CLOCK_RELATIONS;
				case SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS:
					return SmePackage.CLOCKED_EXPR__DST_CLOCK_RELATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (derivedFeatureID) {
				case SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES;
				case SmePackage.INPUT_INSTANCE__DST_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowConnector.class) {
			switch (derivedFeatureID) {
				case SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS:
					return SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.INPUT_INSTANCE__SRC_CONDITIONS:
					return SmePackage.CONDITION_EXPR__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.INPUT_INSTANCE__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.INPUT_INSTANCE__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ClockedExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CLOCKED_EXPR__SRC_CLOCK_RELATIONS:
					return SmePackage.INPUT_INSTANCE__SRC_CLOCK_RELATIONS;
				case SmePackage.CLOCKED_EXPR__DST_CLOCK_RELATIONS:
					return SmePackage.INPUT_INSTANCE__DST_CLOCK_RELATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (baseFeatureID) {
				case SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES:
					return SmePackage.INPUT_INSTANCE__SRC_DEPENDENCES;
				case SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES:
					return SmePackage.INPUT_INSTANCE__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowConnector.class) {
			switch (baseFeatureID) {
				case SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS:
					return SmePackage.INPUT_INSTANCE__DST_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONDITION_EXPR__SRC_CONDITIONS:
					return SmePackage.INPUT_INSTANCE__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.INPUT_INSTANCE__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.INPUT_INSTANCE__COMMENT;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (getInput() != null)
				result.append(input.toString(asCode, true));
			else
				result.append("refers(null)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getInstanceParent();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		for (DataflowConnection dc : getDst_DataflowConnections())
			bodyTree = dc.buildEquationsAST(bodyTree);
		for (ClockRelation cr : getDst_ClockRelations())
			bodyTree = cr.buildEquationsAST(bodyTree);
		for (Dependence dep : getDst_Dependences())
			bodyTree = dep.buildEquationsAST(bodyTree);
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			
			long typeNode;
			
			// Get Input information
			if (getInput() != null) {
				String type = input.getType();
				if (null == type || type.equals(SignalIntrinsicType.UNDEFINED_LITERAL.getLiteral())) {
					// Implicit type ==> we do not use arrayDimensions
					typeNode = API.makeNullary(SignalOperators.opimplicit);
				}
				else {
					boolean isParam = false;
					if (type.equals(SignalIntrinsicType.BOOLEAN_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.oplogical);
					else if (type.equals(SignalIntrinsicType.CHAR_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opchar);
					else if (type.equals(SignalIntrinsicType.COMPLEX_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opcomplex);
					else if (type.equals(SignalIntrinsicType.DCOMPLEX_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opdcomplex);
					else if (type.equals(SignalIntrinsicType.DREAL_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opdpreal);
					else if (type.equals(SignalIntrinsicType.EVENT_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opevent);
					else if (type.equals(SignalIntrinsicType.INTEGER_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opinteger);
					else if (type.equals(SignalIntrinsicType.LONG_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.oplong);
					else if (type.equals(SignalIntrinsicType.REAL_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opreal);
					else if (type.equals(SignalIntrinsicType.SHORT_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opshort);
					else if (type.equals(SignalIntrinsicType.STRING_LITERAL.getLiteral()))
						typeNode = API.makeNullary(SignalOperators.opstring);
					else {
						for (Parameter p : input.getInterfaceParent().getParameters()) {
							if (p.getName().equals(type)) {
								isParam = true;
								break;
							}
						}
						// If the type is the name of a parameter of the same interface, we cannot use its name to type
						// the input instance outside of the scope of this parameter
						if (isParam)
							typeNode = API.makeNullary(SignalOperators.opimplicit);
						else
							typeNode = API.atomString(SignalOperators.opnom, type);
					}
					
					// Add the specification of arrayDimensions. Parse the textual expression
					if (!input.getArrayDimensions().isEmpty() && !isParam) {
						long arrayNode = API.makeEmptyList(SignalOperators.oplesign);
						boolean allOK = true;
						for (String st : input.getArrayDimensions()) {
							long exprNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
							if (exprNode == PKTrees.ERROR_NODE)
								allOK = false;
							else
								arrayNode = API.post(arrayNode, exprNode);
						}
						if (allOK)
							typeNode = API.makeBinary(SignalOperators.optvect, arrayNode, typeNode);
					}
				}
			}
			else
				typeNode = API.makeNullary(SignalOperators.opimplicit);
			
			declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecsig, typeNode, API.makeList(
				SignalOperators.oplsign, idNode)));
		}
		return declTree;
	}
} // InputInstanceImpl
