/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ControlflowOperator;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.DataflowConnector;
import fr.irisa.espresso.sme.DataflowOperator;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.Iterate;
import fr.irisa.espresso.sme.OperatorPort;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOperator;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Input Port</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.InputPortImpl#getSrc_ClockRelations <em>Src Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputPortImpl#getDst_ClockRelations <em>Dst Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputPortImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputPortImpl#getDst_DataflowConnections <em>Dst Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.InputPortImpl#getOperatorParent <em>Operator Parent</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class InputPortImpl extends EObjectImpl implements InputPort {
	/**
	 * The cached value of the '{@link #getSrc_ClockRelations() <em>Src Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		src_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getDst_ClockRelations() <em>Dst Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		dst_ClockRelations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String		NAME_EDEFAULT	= "";
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String					name			= NAME_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getDst_DataflowConnections() <em>Dst Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>	dst_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getOperatorParent() <em>Operator Parent</em>}' reference. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getOperatorParent()
	 * @generated
	 * @ordered
	 */
	protected SignalOperator			operatorParent;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected InputPortImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.INPUT_PORT;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getSrc_ClockRelations() {
		if (src_ClockRelations == null) {
			src_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.INPUT_PORT__SRC_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__SRC);
		}
		return src_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getDst_ClockRelations() {
		if (dst_ClockRelations == null) {
			dst_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.INPUT_PORT__DST_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__DST);
		}
		return dst_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public String getName() {
		if (getOperatorParent() != null)
			name = operatorParent.getPortName(this);
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void setName(String newName) {}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getDst_DataflowConnections() {
		if (dst_DataflowConnections == null) {
			dst_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__DST);
		}
		return dst_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalOperator getOperatorParent() {
		if (operatorParent != null && operatorParent.eIsProxy()) {
			InternalEObject oldOperatorParent = (InternalEObject) operatorParent;
			operatorParent = (SignalOperator) eResolveProxy(oldOperatorParent);
			if (operatorParent != oldOperatorParent) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.INPUT_PORT__OPERATOR_PARENT,
						oldOperatorParent, operatorParent));
			}
		}
		return operatorParent;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalOperator basicGetOperatorParent() {
		return operatorParent;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setOperatorParent(SignalOperator newOperatorParent) {
		SignalOperator oldOperatorParent = operatorParent;
		operatorParent = newOperatorParent;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.INPUT_PORT__OPERATOR_PARENT,
				oldOperatorParent, operatorParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getOperatorParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.INPUT_PORT__SRC_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.INPUT_PORT__DST_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_DataflowConnections()).basicAdd(
					otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.INPUT_PORT__SRC_CLOCK_RELATIONS:
				return ((InternalEList<?>) getSrc_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_PORT__DST_CLOCK_RELATIONS:
				return ((InternalEList<?>) getDst_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getDst_DataflowConnections()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.INPUT_PORT__SRC_CLOCK_RELATIONS:
				return getSrc_ClockRelations();
			case SmePackage.INPUT_PORT__DST_CLOCK_RELATIONS:
				return getDst_ClockRelations();
			case SmePackage.INPUT_PORT__NAME:
				return getName();
			case SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS:
				return getDst_DataflowConnections();
			case SmePackage.INPUT_PORT__OPERATOR_PARENT:
				if (resolve)
					return getOperatorParent();
				return basicGetOperatorParent();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.INPUT_PORT__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				getSrc_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.INPUT_PORT__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				getDst_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.INPUT_PORT__NAME:
				setName((String) newValue);
				return;
			case SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS:
				getDst_DataflowConnections().clear();
				getDst_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.INPUT_PORT__OPERATOR_PARENT:
				setOperatorParent((SignalOperator) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.INPUT_PORT__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				return;
			case SmePackage.INPUT_PORT__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				return;
			case SmePackage.INPUT_PORT__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS:
				getDst_DataflowConnections().clear();
				return;
			case SmePackage.INPUT_PORT__OPERATOR_PARENT:
				setOperatorParent((SignalOperator) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.INPUT_PORT__SRC_CLOCK_RELATIONS:
				return src_ClockRelations != null && !src_ClockRelations.isEmpty();
			case SmePackage.INPUT_PORT__DST_CLOCK_RELATIONS:
				return dst_ClockRelations != null && !dst_ClockRelations.isEmpty();
			case SmePackage.INPUT_PORT__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS:
				return dst_DataflowConnections != null && !dst_DataflowConnections.isEmpty();
			case SmePackage.INPUT_PORT__OPERATOR_PARENT:
				return operatorParent != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == OperatorPort.class) {
			switch (derivedFeatureID) {
				case SmePackage.INPUT_PORT__NAME:
					return SmePackage.OPERATOR_PORT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowConnector.class) {
			switch (derivedFeatureID) {
				case SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS:
					return SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == OperatorPort.class) {
			switch (baseFeatureID) {
				case SmePackage.OPERATOR_PORT__NAME:
					return SmePackage.INPUT_PORT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowConnector.class) {
			switch (baseFeatureID) {
				case SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS:
					return SmePackage.INPUT_PORT__DST_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) throws CyclicDefinitionException {
		long result;
		
		for (ClockRelation cr : getSrc_ClockRelations()) {
			PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.INPUT_PORT_SOURCE, cr);
		}
		if (this.getOperatorParent() instanceof DataflowOperator || operatorParent instanceof Iterate) {
			for (ClockRelation cr : getDst_ClockRelations()) {
				PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.INPUT_PORT_TARGET, cr);
			}
			if (this.getDst_DataflowConnections().size() != 1) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INPUT_PORT_DATAFLOW_CONNECTIONS_NUMBER, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				DataflowConnection c = dst_DataflowConnections.get(0);
				result = c.getAST(path);
			}
		}
		else if (operatorParent instanceof ControlflowOperator) {
			for (DataflowConnection df : getDst_DataflowConnections()) {
				PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.INPUT_PORT_DATAFLOW_TARGET, df);
			}
			if (this.getDst_ClockRelations().size() != 1) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INPUT_PORT_CLOCK_RELATION_LINKS_NUMBER, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				ClockRelation c = dst_ClockRelations.get(0);
				result = c.getAST(path);
			}
		}
		else
			throw new UnsupportedOperationException();
		
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
	
} // InputPortImpl
