/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.BasicIterate;
import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.Iterator;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Iterator</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.IteratorImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IteratorImpl#getSrc_ConstantAffectations <em>Src Constant Affectations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IteratorImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IteratorImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IteratorImpl#getDst_ConstantAffectation <em>Dst Constant Affectation</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.IteratorImpl#getArrayParent <em>Array Parent</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class IteratorImpl extends EObjectImpl implements Iterator {
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>		src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String			NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String						name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String			COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String						comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getDst_ConstantAffectation() <em>Dst Constant Affectation</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ConstantAffectation()
	 * @generated
	 * @ordered
	 */
	protected ConstantAffectation			dst_ConstantAffectation;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected IteratorImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.ITERATOR;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.ITERATOR__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.ITERATOR__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.ITERATOR__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation getDst_ConstantAffectation() {
		if (dst_ConstantAffectation != null && dst_ConstantAffectation.eIsProxy()) {
			InternalEObject oldDst_ConstantAffectation = (InternalEObject) dst_ConstantAffectation;
			dst_ConstantAffectation = (ConstantAffectation) eResolveProxy(oldDst_ConstantAffectation);
			if (dst_ConstantAffectation != oldDst_ConstantAffectation) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE,
						SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation,
						dst_ConstantAffectation));
			}
		}
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation basicGetDst_ConstantAffectation() {
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation,
			NotificationChain msgs) {
		ConstantAffectation oldDst_ConstantAffectation = dst_ConstantAffectation;
		dst_ConstantAffectation = newDst_ConstantAffectation;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation, newDst_ConstantAffectation);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation) {
		if (newDst_ConstantAffectation != dst_ConstantAffectation) {
			NotificationChain msgs = null;
			if (dst_ConstantAffectation != null)
				msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			if (newDst_ConstantAffectation != null)
				msgs = ((InternalEObject) newDst_ConstantAffectation).eInverseAdd(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			msgs = basicSetDst_ConstantAffectation(newDst_ConstantAffectation, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION,
				newDst_ConstantAffectation, newDst_ConstantAffectation));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public BasicIterate getArrayParent() {
		if (eContainerFeatureID != SmePackage.ITERATOR__ARRAY_PARENT)
			return null;
		return (BasicIterate) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetArrayParent(BasicIterate newArrayParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newArrayParent, SmePackage.ITERATOR__ARRAY_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setArrayParent(BasicIterate newArrayParent) {
		if (newArrayParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.ITERATOR__ARRAY_PARENT && newArrayParent != null)) {
			if (EcoreUtil.isAncestor(this, newArrayParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newArrayParent != null)
				msgs = ((InternalEObject) newArrayParent).eInverseAdd(this, SmePackage.BASIC_ITERATE__ITERATOR,
					BasicIterate.class, msgs);
			msgs = basicSetArrayParent(newArrayParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.ITERATOR__ARRAY_PARENT, newArrayParent,
				newArrayParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getArrayParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.ITERATOR__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
			case SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION:
				if (dst_ConstantAffectation != null)
					msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
						SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
				return basicSetDst_ConstantAffectation((ConstantAffectation) otherEnd, msgs);
			case SmePackage.ITERATOR__ARRAY_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetArrayParent((BasicIterate) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.ITERATOR__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
			case SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION:
				return basicSetDst_ConstantAffectation(null, msgs);
			case SmePackage.ITERATOR__ARRAY_PARENT:
				return basicSetArrayParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.ITERATOR__ARRAY_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.BASIC_ITERATE__ITERATOR,
					BasicIterate.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.ITERATOR__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.ITERATOR__NAME:
				return getName();
			case SmePackage.ITERATOR__COMMENT:
				return getComment();
			case SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION:
				if (resolve)
					return getDst_ConstantAffectation();
				return basicGetDst_ConstantAffectation();
			case SmePackage.ITERATOR__ARRAY_PARENT:
				return getArrayParent();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.ITERATOR__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.ITERATOR__NAME:
				setName((String) newValue);
				return;
			case SmePackage.ITERATOR__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) newValue);
				return;
			case SmePackage.ITERATOR__ARRAY_PARENT:
				setArrayParent((BasicIterate) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.ITERATOR__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.ITERATOR__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.ITERATOR__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) null);
				return;
			case SmePackage.ITERATOR__ARRAY_PARENT:
				setArrayParent((BasicIterate) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.ITERATOR__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.ITERATOR__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.ITERATOR__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.ITERATOR__DST_CONSTANT_AFFECTATION:
				return dst_ConstantAffectation != null;
			case SmePackage.ITERATOR__ARRAY_PARENT:
				return getArrayParent() != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.ITERATOR__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.ITERATOR__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.ITERATOR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.ITERATOR__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.ITERATOR__COMMENT;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/*
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * 
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			result.append("integer " + name);
			return result.toString();
		}
		else
			return toString();
	}
	
	/*
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * 
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/*
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * 
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/*
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * 
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
	
} // IteratorImpl
