/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.CompilationDirectiveKind;
import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.ConstantValue;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.Index;
import fr.irisa.espresso.sme.Input;
import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.Local;
import fr.irisa.espresso.sme.ModelDeclaration;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.ModelKind;
import fr.irisa.espresso.sme.ModelType;
import fr.irisa.espresso.sme.Module;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.Output;
import fr.irisa.espresso.sme.OutputInstance;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.ParameterInstance;
import fr.irisa.espresso.sme.Pragma;
import fr.irisa.espresso.sme.ProcessAttributeKind;
import fr.irisa.espresso.sme.RootModel;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SmeFactory;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.Specifications;
import fr.irisa.espresso.sme.SubProcess;
import fr.irisa.espresso.sme.TypeDeclaration;
import fr.irisa.espresso.sme.VisibilityKind;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Model Declaration</b></em>'. <!-- end-user-doc
 * -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getType <em>Type</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getInputs <em>Inputs</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getOutputs <em>Outputs</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getParameters <em>Parameters</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getSpec <em>Spec</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getPragmas <em>Pragmas</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getProcessClass <em>Process Class</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getSrc_ConstantAffectations <em>Src Constant Affectations
 * </em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getModuleParent <em>Module Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getSpecParent <em>Spec Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getModelType <em>Model Type</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getVisibility <em>Visibility</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#isIsMain <em>Is Main</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#isIsSigaliProcess <em>Is Sigali Process</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getCompilationDirective <em>Compilation Directive</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getRunOn <em>Run On</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getC_CODE <em>CCODE</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getCpp_CODE <em>Cpp CODE</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getJava_CODE <em>Java CODE</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getProcessor_type <em>Processor type</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelDeclarationImpl#getModelDeclarations <em>Model Declarations</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class ModelDeclarationImpl extends ModelImpl implements ModelDeclaration {
	/**
	 * The cached value of the '{@link #getType() <em>Type</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getType()
	 * @generated
	 * @ordered
	 */
	protected ModelType								type;
	
	/**
	 * The cached value of the '{@link #getInputs() <em>Inputs</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getInputs()
	 * @generated
	 * @ordered
	 */
	protected EList<Input>							inputs;
	
	/**
	 * The cached value of the '{@link #getOutputs() <em>Outputs</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getOutputs()
	 * @generated
	 * @ordered
	 */
	protected EList<Output>							outputs;
	
	/**
	 * The cached value of the '{@link #getParameters() <em>Parameters</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getParameters()
	 * @generated
	 * @ordered
	 */
	protected EList<Parameter>						parameters;
	
	/**
	 * The cached value of the '{@link #getSpec() <em>Spec</em>}' containment reference. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSpec()
	 * @generated
	 * @ordered
	 */
	protected Specifications						spec;
	
	/**
	 * The cached value of the '{@link #getPragmas() <em>Pragmas</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getPragmas()
	 * @generated
	 * @ordered
	 */
	protected EList<Pragma>							pragmas;
	
	/**
	 * The default value of the '{@link #getProcessClass() <em>Process Class</em>}' attribute. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getProcessClass()
	 * @generated
	 * @ordered
	 */
	protected static final ProcessAttributeKind		PROCESS_CLASS_EDEFAULT			= ProcessAttributeKind.UNSAFE_LITERAL;
	
	/**
	 * The cached value of the '{@link #getProcessClass() <em>Process Class</em>}' attribute. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getProcessClass()
	 * @generated
	 * @ordered
	 */
	protected ProcessAttributeKind					processClass					= PROCESS_CLASS_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String					NAME_EDEFAULT					= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String								name							= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String					COMMENT_EDEFAULT				= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String								comment							= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>			src_ConstantAffectations;
	
	/**
	 * The default value of the '{@link #getModelType() <em>Model Type</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getModelType()
	 * @generated
	 * @ordered
	 */
	protected static final ModelKind				MODEL_TYPE_EDEFAULT				= ModelKind.PROCESS_LITERAL;
	
	/**
	 * The cached value of the '{@link #getModelType() <em>Model Type</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getModelType()
	 * @generated
	 * @ordered
	 */
	protected ModelKind								modelType						= MODEL_TYPE_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getVisibility() <em>Visibility</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getVisibility()
	 * @generated
	 * @ordered
	 */
	protected static final VisibilityKind			VISIBILITY_EDEFAULT				= VisibilityKind.PUBLIC_LITERAL;
	
	/**
	 * The cached value of the '{@link #getVisibility() <em>Visibility</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getVisibility()
	 * @generated
	 * @ordered
	 */
	protected VisibilityKind						visibility						= VISIBILITY_EDEFAULT;
	
	/**
	 * The default value of the '{@link #isIsMain() <em>Is Main</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #isIsMain()
	 * @generated
	 * @ordered
	 */
	protected static final boolean					IS_MAIN_EDEFAULT				= false;
	
	/**
	 * The cached value of the '{@link #isIsMain() <em>Is Main</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #isIsMain()
	 * @generated
	 * @ordered
	 */
	protected boolean								isMain							= IS_MAIN_EDEFAULT;
	
	/**
	 * The default value of the '{@link #isIsSigaliProcess() <em>Is Sigali Process</em>}' attribute. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #isIsSigaliProcess()
	 * @generated
	 * @ordered
	 */
	protected static final boolean					IS_SIGALI_PROCESS_EDEFAULT		= false;
	
	/**
	 * The cached value of the '{@link #isIsSigaliProcess() <em>Is Sigali Process</em>}' attribute. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #isIsSigaliProcess()
	 * @generated
	 * @ordered
	 */
	protected boolean								isSigaliProcess					= IS_SIGALI_PROCESS_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getCompilationDirective() <em>Compilation Directive</em>}' attribute. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getCompilationDirective()
	 * @generated
	 * @ordered
	 */
	protected static final CompilationDirectiveKind	COMPILATION_DIRECTIVE_EDEFAULT	= CompilationDirectiveKind.NONE_LITERAL;
	
	/**
	 * The cached value of the '{@link #getCompilationDirective() <em>Compilation Directive</em>}' attribute. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getCompilationDirective()
	 * @generated
	 * @ordered
	 */
	protected CompilationDirectiveKind				compilationDirective			= COMPILATION_DIRECTIVE_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getRunOn() <em>Run On</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getRunOn()
	 * @generated
	 * @ordered
	 */
	protected static final int						RUN_ON_EDEFAULT					= -1;
	
	/**
	 * The cached value of the '{@link #getRunOn() <em>Run On</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getRunOn()
	 * @generated
	 * @ordered
	 */
	protected int									runOn							= RUN_ON_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getC_CODE() <em>CCODE</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getC_CODE()
	 * @generated
	 * @ordered
	 */
	protected static final String					CCODE_EDEFAULT					= null;
	
	/**
	 * The cached value of the '{@link #getC_CODE() <em>CCODE</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getC_CODE()
	 * @generated
	 * @ordered
	 */
	protected String								c_CODE							= CCODE_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getCpp_CODE() <em>Cpp CODE</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getCpp_CODE()
	 * @generated
	 * @ordered
	 */
	protected static final String					CPP_CODE_EDEFAULT				= null;
	
	/**
	 * The cached value of the '{@link #getCpp_CODE() <em>Cpp CODE</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getCpp_CODE()
	 * @generated
	 * @ordered
	 */
	protected String								cpp_CODE						= CPP_CODE_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getJava_CODE() <em>Java CODE</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getJava_CODE()
	 * @generated
	 * @ordered
	 */
	protected static final String					JAVA_CODE_EDEFAULT				= null;
	
	/**
	 * The cached value of the '{@link #getJava_CODE() <em>Java CODE</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getJava_CODE()
	 * @generated
	 * @ordered
	 */
	protected String								java_CODE						= JAVA_CODE_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getProcessor_type() <em>Processor type</em>}' attribute. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getProcessor_type()
	 * @generated
	 * @ordered
	 */
	protected static final String					PROCESSOR_TYPE_EDEFAULT			= null;
	
	/**
	 * The cached value of the '{@link #getProcessor_type() <em>Processor type</em>}' attribute. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getProcessor_type()
	 * @generated
	 * @ordered
	 */
	protected String								processor_type					= PROCESSOR_TYPE_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getModelDeclarations() <em>Model Declarations</em>}' containment reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getModelDeclarations()
	 * @generated
	 * @ordered
	 */
	protected EList<ModelDeclaration>				modelDeclarations;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModelDeclarationImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.MODEL_DECLARATION;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelType getType() {
		if (type != null && type.eIsProxy()) {
			InternalEObject oldType = (InternalEObject) type;
			type = (ModelType) eResolveProxy(oldType);
			if (type != oldType) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.MODEL_DECLARATION__TYPE,
						oldType, type));
			}
		}
		return type;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelType basicGetType() {
		return type;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setType(ModelType newType) {
		ModelType oldType = type;
		type = newType;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__TYPE, oldType, type));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Input> getInputs() {
		if (inputs == null) {
			inputs = new EObjectContainmentWithInverseEList<Input>(Input.class, this,
				SmePackage.MODEL_DECLARATION__INPUTS, SmePackage.INPUT__INTERFACE_PARENT);
		}
		return inputs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Output> getOutputs() {
		if (outputs == null) {
			outputs = new EObjectContainmentWithInverseEList<Output>(Output.class, this,
				SmePackage.MODEL_DECLARATION__OUTPUTS, SmePackage.OUTPUT__INTERFACE_PARENT);
		}
		return outputs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Parameter> getParameters() {
		if (parameters == null) {
			parameters = new EObjectContainmentWithInverseEList<Parameter>(Parameter.class, this,
				SmePackage.MODEL_DECLARATION__PARAMETERS, SmePackage.PARAMETER__INTERFACE_PARENT);
		}
		return parameters;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public Specifications getSpec() {
		if (spec == null) {
			spec = SmeFactory.eINSTANCE.createSpecifications();
			spec.setInterfaceParent(this);
		}
		return spec;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSpec(Specifications newSpec, NotificationChain msgs) {
		Specifications oldSpec = spec;
		spec = newSpec;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.MODEL_DECLARATION__SPEC, oldSpec, newSpec);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSpec(Specifications newSpec) {
		if (newSpec != spec) {
			NotificationChain msgs = null;
			if (spec != null)
				msgs = ((InternalEObject) spec).eInverseRemove(this, SmePackage.SPECIFICATIONS__INTERFACE_PARENT,
					Specifications.class, msgs);
			if (newSpec != null)
				msgs = ((InternalEObject) newSpec).eInverseAdd(this, SmePackage.SPECIFICATIONS__INTERFACE_PARENT,
					Specifications.class, msgs);
			msgs = basicSetSpec(newSpec, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__SPEC, newSpec, newSpec));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Pragma> getPragmas() {
		if (pragmas == null) {
			pragmas = new EObjectContainmentWithInverseEList<Pragma>(Pragma.class, this,
				SmePackage.MODEL_DECLARATION__PRAGMAS, SmePackage.PRAGMA__INTERFACE_PARENT);
		}
		return pragmas;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ProcessAttributeKind getProcessClass() {
		return processClass;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void setProcessClass(ProcessAttributeKind newProcessClass) {
		ProcessAttributeKind oldProcessClass = processClass;
		if (newProcessClass == null) {
			/* A function model is mandatory safe */
			if (modelType == ModelKind.FUNCTION_LITERAL)
				processClass = ProcessAttributeKind.SAFE_LITERAL;
			else if (modelType == ModelKind.NODE_LITERAL) /* And a node is not unsafe */
				processClass = ProcessAttributeKind.DETERMINISTIC_LITERAL;
			else
				processClass = PROCESS_CLASS_EDEFAULT;
		}
		else {
			/* A function model is mandatory safe */
			if (modelType == ModelKind.FUNCTION_LITERAL)
				processClass = ProcessAttributeKind.SAFE_LITERAL;
			else if (modelType == ModelKind.NODE_LITERAL && newProcessClass == ProcessAttributeKind.UNSAFE_LITERAL) {
				// And a node is not unsafe
				processClass = ProcessAttributeKind.DETERMINISTIC_LITERAL;
			}
			else
				processClass = newProcessClass;
		}
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__PROCESS_CLASS,
				oldProcessClass, processClass));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__COMMENT, oldComment,
				comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelDeclaration getModelParent() {
		if (eContainerFeatureID != SmePackage.MODEL_DECLARATION__MODEL_PARENT)
			return null;
		return (ModelDeclaration) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(ModelDeclaration newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.MODEL_DECLARATION__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(ModelDeclaration newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.MODEL_DECLARATION__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this,
					SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS, ModelDeclaration.class, msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__MODEL_PARENT,
				newModelParent, newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Module getModuleParent() {
		if (eContainerFeatureID != SmePackage.MODEL_DECLARATION__MODULE_PARENT)
			return null;
		return (Module) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModuleParent(Module newModuleParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModuleParent, SmePackage.MODEL_DECLARATION__MODULE_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModuleParent(Module newModuleParent) {
		if (newModuleParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.MODEL_DECLARATION__MODULE_PARENT && newModuleParent != null)) {
			if (EcoreUtil.isAncestor(this, newModuleParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModuleParent != null)
				msgs = ((InternalEObject) newModuleParent).eInverseAdd(this, SmePackage.MODULE__MODEL_DECLARATIONS,
					Module.class, msgs);
			msgs = basicSetModuleParent(newModuleParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__MODULE_PARENT,
				newModuleParent, newModuleParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Specifications getSpecParent() {
		if (eContainerFeatureID != SmePackage.MODEL_DECLARATION__SPEC_PARENT)
			return null;
		return (Specifications) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSpecParent(Specifications newSpecParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newSpecParent, SmePackage.MODEL_DECLARATION__SPEC_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSpecParent(Specifications newSpecParent) {
		if (newSpecParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.MODEL_DECLARATION__SPEC_PARENT && newSpecParent != null)) {
			if (EcoreUtil.isAncestor(this, newSpecParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newSpecParent != null)
				msgs = ((InternalEObject) newSpecParent).eInverseAdd(this,
					SmePackage.SPECIFICATIONS__MODEL_DECLARATIONS, Specifications.class, msgs);
			msgs = basicSetSpecParent(newSpecParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__SPEC_PARENT,
				newSpecParent, newSpecParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelKind getModelType() {
		return modelType;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void setModelType(ModelKind newModelType) {
		ModelKind oldModelType = modelType;
		modelType = newModelType == null ? MODEL_TYPE_EDEFAULT : newModelType;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__MODEL_TYPE,
				oldModelType, modelType));
		
		/* A function model is mandatory safe */
		if (modelType == ModelKind.FUNCTION_LITERAL)
			this.setProcessClass(ProcessAttributeKind.SAFE_LITERAL);
		else if (modelType == ModelKind.NODE_LITERAL) /* And a node is not unsafe */
			this.setProcessClass(ProcessAttributeKind.DETERMINISTIC_LITERAL);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public VisibilityKind getVisibility() {
		return visibility;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setVisibility(VisibilityKind newVisibility) {
		VisibilityKind oldVisibility = visibility;
		visibility = newVisibility == null ? VISIBILITY_EDEFAULT : newVisibility;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__VISIBILITY,
				oldVisibility, visibility));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public boolean isIsMain() {
		return isMain;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setIsMain(boolean newIsMain) {
		boolean oldIsMain = isMain;
		isMain = newIsMain;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__IS_MAIN, oldIsMain,
				isMain));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public boolean isIsSigaliProcess() {
		return isSigaliProcess;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setIsSigaliProcess(boolean newIsSigaliProcess) {
		boolean oldIsSigaliProcess = isSigaliProcess;
		isSigaliProcess = newIsSigaliProcess;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__IS_SIGALI_PROCESS,
				oldIsSigaliProcess, isSigaliProcess));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CompilationDirectiveKind getCompilationDirective() {
		return compilationDirective;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setCompilationDirective(CompilationDirectiveKind newCompilationDirective) {
		CompilationDirectiveKind oldCompilationDirective = compilationDirective;
		compilationDirective = newCompilationDirective == null ? COMPILATION_DIRECTIVE_EDEFAULT
				: newCompilationDirective;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__COMPILATION_DIRECTIVE,
				oldCompilationDirective, compilationDirective));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public int getRunOn() {
		return runOn;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setRunOn(int newRunOn) {
		int oldRunOn = runOn;
		runOn = newRunOn;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__RUN_ON, oldRunOn, runOn));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getC_CODE() {
		return c_CODE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setC_CODE(String newC_CODE) {
		String oldC_CODE = c_CODE;
		c_CODE = newC_CODE;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__CCODE, oldC_CODE,
				c_CODE));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getCpp_CODE() {
		return cpp_CODE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setCpp_CODE(String newCpp_CODE) {
		String oldCpp_CODE = cpp_CODE;
		cpp_CODE = newCpp_CODE;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__CPP_CODE, oldCpp_CODE,
				cpp_CODE));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getJava_CODE() {
		return java_CODE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setJava_CODE(String newJava_CODE) {
		String oldJava_CODE = java_CODE;
		java_CODE = newJava_CODE;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__JAVA_CODE,
				oldJava_CODE, java_CODE));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getProcessor_type() {
		return processor_type;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setProcessor_type(String newProcessor_type) {
		String oldProcessor_type = processor_type;
		processor_type = newProcessor_type;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_DECLARATION__PROCESSOR_TYPE,
				oldProcessor_type, processor_type));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ModelDeclaration> getModelDeclarations() {
		if (modelDeclarations == null) {
			modelDeclarations = new EObjectContainmentWithInverseEList<ModelDeclaration>(ModelDeclaration.class, this,
				SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS, SmePackage.MODEL_DECLARATION__MODEL_PARENT);
		}
		return modelDeclarations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.MODEL_DECLARATION__INPUTS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getInputs()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__OUTPUTS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getOutputs()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__PARAMETERS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getParameters()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__SPEC:
				if (spec != null)
					msgs = ((InternalEObject) spec).eInverseRemove(this, EOPPOSITE_FEATURE_BASE
							- SmePackage.MODEL_DECLARATION__SPEC, null, msgs);
				return basicSetSpec((Specifications) otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__PRAGMAS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getPragmas()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((ModelDeclaration) otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__MODULE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModuleParent((Module) otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__SPEC_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetSpecParent((Specifications) otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getModelDeclarations()).basicAdd(otherEnd,
					msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.MODEL_DECLARATION__INPUTS:
				return ((InternalEList<?>) getInputs()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__OUTPUTS:
				return ((InternalEList<?>) getOutputs()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__PARAMETERS:
				return ((InternalEList<?>) getParameters()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__SPEC:
				return basicSetSpec(null, msgs);
			case SmePackage.MODEL_DECLARATION__PRAGMAS:
				return ((InternalEList<?>) getPragmas()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_DECLARATION__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
			case SmePackage.MODEL_DECLARATION__MODULE_PARENT:
				return basicSetModuleParent(null, msgs);
			case SmePackage.MODEL_DECLARATION__SPEC_PARENT:
				return basicSetSpecParent(null, msgs);
			case SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS:
				return ((InternalEList<?>) getModelDeclarations()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.MODEL_DECLARATION__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS,
					ModelDeclaration.class, msgs);
			case SmePackage.MODEL_DECLARATION__MODULE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODULE__MODEL_DECLARATIONS, Module.class,
					msgs);
			case SmePackage.MODEL_DECLARATION__SPEC_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.SPECIFICATIONS__MODEL_DECLARATIONS,
					Specifications.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.MODEL_DECLARATION__TYPE:
				if (resolve)
					return getType();
				return basicGetType();
			case SmePackage.MODEL_DECLARATION__INPUTS:
				return getInputs();
			case SmePackage.MODEL_DECLARATION__OUTPUTS:
				return getOutputs();
			case SmePackage.MODEL_DECLARATION__PARAMETERS:
				return getParameters();
			case SmePackage.MODEL_DECLARATION__SPEC:
				return getSpec();
			case SmePackage.MODEL_DECLARATION__PRAGMAS:
				return getPragmas();
			case SmePackage.MODEL_DECLARATION__PROCESS_CLASS:
				return getProcessClass();
			case SmePackage.MODEL_DECLARATION__NAME:
				return getName();
			case SmePackage.MODEL_DECLARATION__COMMENT:
				return getComment();
			case SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.MODEL_DECLARATION__MODEL_PARENT:
				return getModelParent();
			case SmePackage.MODEL_DECLARATION__MODULE_PARENT:
				return getModuleParent();
			case SmePackage.MODEL_DECLARATION__SPEC_PARENT:
				return getSpecParent();
			case SmePackage.MODEL_DECLARATION__MODEL_TYPE:
				return getModelType();
			case SmePackage.MODEL_DECLARATION__VISIBILITY:
				return getVisibility();
			case SmePackage.MODEL_DECLARATION__IS_MAIN:
				return isIsMain() ? Boolean.TRUE : Boolean.FALSE;
			case SmePackage.MODEL_DECLARATION__IS_SIGALI_PROCESS:
				return isIsSigaliProcess() ? Boolean.TRUE : Boolean.FALSE;
			case SmePackage.MODEL_DECLARATION__COMPILATION_DIRECTIVE:
				return getCompilationDirective();
			case SmePackage.MODEL_DECLARATION__RUN_ON:
				return new Integer(getRunOn());
			case SmePackage.MODEL_DECLARATION__CCODE:
				return getC_CODE();
			case SmePackage.MODEL_DECLARATION__CPP_CODE:
				return getCpp_CODE();
			case SmePackage.MODEL_DECLARATION__JAVA_CODE:
				return getJava_CODE();
			case SmePackage.MODEL_DECLARATION__PROCESSOR_TYPE:
				return getProcessor_type();
			case SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS:
				return getModelDeclarations();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.MODEL_DECLARATION__TYPE:
				setType((ModelType) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__INPUTS:
				getInputs().clear();
				getInputs().addAll((Collection<? extends Input>) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__OUTPUTS:
				getOutputs().clear();
				getOutputs().addAll((Collection<? extends Output>) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__PARAMETERS:
				getParameters().clear();
				getParameters().addAll((Collection<? extends Parameter>) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__SPEC:
				setSpec((Specifications) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__PRAGMAS:
				getPragmas().clear();
				getPragmas().addAll((Collection<? extends Pragma>) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__PROCESS_CLASS:
				setProcessClass((ProcessAttributeKind) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__NAME:
				setName((String) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__MODEL_PARENT:
				setModelParent((ModelDeclaration) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__MODULE_PARENT:
				setModuleParent((Module) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__SPEC_PARENT:
				setSpecParent((Specifications) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__MODEL_TYPE:
				setModelType((ModelKind) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__VISIBILITY:
				setVisibility((VisibilityKind) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__IS_MAIN:
				setIsMain(((Boolean) newValue).booleanValue());
				return;
			case SmePackage.MODEL_DECLARATION__IS_SIGALI_PROCESS:
				setIsSigaliProcess(((Boolean) newValue).booleanValue());
				return;
			case SmePackage.MODEL_DECLARATION__COMPILATION_DIRECTIVE:
				setCompilationDirective((CompilationDirectiveKind) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__RUN_ON:
				setRunOn(((Integer) newValue).intValue());
				return;
			case SmePackage.MODEL_DECLARATION__CCODE:
				setC_CODE((String) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__CPP_CODE:
				setCpp_CODE((String) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__JAVA_CODE:
				setJava_CODE((String) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__PROCESSOR_TYPE:
				setProcessor_type((String) newValue);
				return;
			case SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS:
				getModelDeclarations().clear();
				getModelDeclarations().addAll((Collection<? extends ModelDeclaration>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.MODEL_DECLARATION__TYPE:
				setType((ModelType) null);
				return;
			case SmePackage.MODEL_DECLARATION__INPUTS:
				getInputs().clear();
				return;
			case SmePackage.MODEL_DECLARATION__OUTPUTS:
				getOutputs().clear();
				return;
			case SmePackage.MODEL_DECLARATION__PARAMETERS:
				getParameters().clear();
				return;
			case SmePackage.MODEL_DECLARATION__SPEC:
				setSpec((Specifications) null);
				return;
			case SmePackage.MODEL_DECLARATION__PRAGMAS:
				getPragmas().clear();
				return;
			case SmePackage.MODEL_DECLARATION__PROCESS_CLASS:
				setProcessClass(PROCESS_CLASS_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.MODEL_DECLARATION__MODEL_PARENT:
				setModelParent((ModelDeclaration) null);
				return;
			case SmePackage.MODEL_DECLARATION__MODULE_PARENT:
				setModuleParent((Module) null);
				return;
			case SmePackage.MODEL_DECLARATION__SPEC_PARENT:
				setSpecParent((Specifications) null);
				return;
			case SmePackage.MODEL_DECLARATION__MODEL_TYPE:
				setModelType(MODEL_TYPE_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__VISIBILITY:
				setVisibility(VISIBILITY_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__IS_MAIN:
				setIsMain(IS_MAIN_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__IS_SIGALI_PROCESS:
				setIsSigaliProcess(IS_SIGALI_PROCESS_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__COMPILATION_DIRECTIVE:
				setCompilationDirective(COMPILATION_DIRECTIVE_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__RUN_ON:
				setRunOn(RUN_ON_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__CCODE:
				setC_CODE(CCODE_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__CPP_CODE:
				setCpp_CODE(CPP_CODE_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__JAVA_CODE:
				setJava_CODE(JAVA_CODE_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__PROCESSOR_TYPE:
				setProcessor_type(PROCESSOR_TYPE_EDEFAULT);
				return;
			case SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS:
				getModelDeclarations().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.MODEL_DECLARATION__TYPE:
				return type != null;
			case SmePackage.MODEL_DECLARATION__INPUTS:
				return inputs != null && !inputs.isEmpty();
			case SmePackage.MODEL_DECLARATION__OUTPUTS:
				return outputs != null && !outputs.isEmpty();
			case SmePackage.MODEL_DECLARATION__PARAMETERS:
				return parameters != null && !parameters.isEmpty();
			case SmePackage.MODEL_DECLARATION__SPEC:
				return spec != null;
			case SmePackage.MODEL_DECLARATION__PRAGMAS:
				return pragmas != null && !pragmas.isEmpty();
			case SmePackage.MODEL_DECLARATION__PROCESS_CLASS:
				return processClass != PROCESS_CLASS_EDEFAULT;
			case SmePackage.MODEL_DECLARATION__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.MODEL_DECLARATION__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.MODEL_DECLARATION__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.MODEL_DECLARATION__MODULE_PARENT:
				return getModuleParent() != null;
			case SmePackage.MODEL_DECLARATION__SPEC_PARENT:
				return getSpecParent() != null;
			case SmePackage.MODEL_DECLARATION__MODEL_TYPE:
				return modelType != MODEL_TYPE_EDEFAULT;
			case SmePackage.MODEL_DECLARATION__VISIBILITY:
				return visibility != VISIBILITY_EDEFAULT;
			case SmePackage.MODEL_DECLARATION__IS_MAIN:
				return isMain != IS_MAIN_EDEFAULT;
			case SmePackage.MODEL_DECLARATION__IS_SIGALI_PROCESS:
				return isSigaliProcess != IS_SIGALI_PROCESS_EDEFAULT;
			case SmePackage.MODEL_DECLARATION__COMPILATION_DIRECTIVE:
				return compilationDirective != COMPILATION_DIRECTIVE_EDEFAULT;
			case SmePackage.MODEL_DECLARATION__RUN_ON:
				return runOn != RUN_ON_EDEFAULT;
			case SmePackage.MODEL_DECLARATION__CCODE:
				return CCODE_EDEFAULT == null ? c_CODE != null : !CCODE_EDEFAULT.equals(c_CODE);
			case SmePackage.MODEL_DECLARATION__CPP_CODE:
				return CPP_CODE_EDEFAULT == null ? cpp_CODE != null : !CPP_CODE_EDEFAULT.equals(cpp_CODE);
			case SmePackage.MODEL_DECLARATION__JAVA_CODE:
				return JAVA_CODE_EDEFAULT == null ? java_CODE != null : !JAVA_CODE_EDEFAULT.equals(java_CODE);
			case SmePackage.MODEL_DECLARATION__PROCESSOR_TYPE:
				return PROCESSOR_TYPE_EDEFAULT == null ? processor_type != null : !PROCESSOR_TYPE_EDEFAULT
						.equals(processor_type);
			case SmePackage.MODEL_DECLARATION__MODEL_DECLARATIONS:
				return modelDeclarations != null && !modelDeclarations.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == InterfaceDefinition.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_DECLARATION__TYPE:
					return SmePackage.INTERFACE_DEFINITION__TYPE;
				case SmePackage.MODEL_DECLARATION__INPUTS:
					return SmePackage.INTERFACE_DEFINITION__INPUTS;
				case SmePackage.MODEL_DECLARATION__OUTPUTS:
					return SmePackage.INTERFACE_DEFINITION__OUTPUTS;
				case SmePackage.MODEL_DECLARATION__PARAMETERS:
					return SmePackage.INTERFACE_DEFINITION__PARAMETERS;
				case SmePackage.MODEL_DECLARATION__SPEC:
					return SmePackage.INTERFACE_DEFINITION__SPEC;
				case SmePackage.MODEL_DECLARATION__PRAGMAS:
					return SmePackage.INTERFACE_DEFINITION__PRAGMAS;
				case SmePackage.MODEL_DECLARATION__PROCESS_CLASS:
					return SmePackage.INTERFACE_DEFINITION__PROCESS_CLASS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_DECLARATION__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == ConnectableElement.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_DECLARATION__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == RootModel.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == InterfaceDefinition.class) {
			switch (baseFeatureID) {
				case SmePackage.INTERFACE_DEFINITION__TYPE:
					return SmePackage.MODEL_DECLARATION__TYPE;
				case SmePackage.INTERFACE_DEFINITION__INPUTS:
					return SmePackage.MODEL_DECLARATION__INPUTS;
				case SmePackage.INTERFACE_DEFINITION__OUTPUTS:
					return SmePackage.MODEL_DECLARATION__OUTPUTS;
				case SmePackage.INTERFACE_DEFINITION__PARAMETERS:
					return SmePackage.MODEL_DECLARATION__PARAMETERS;
				case SmePackage.INTERFACE_DEFINITION__SPEC:
					return SmePackage.MODEL_DECLARATION__SPEC;
				case SmePackage.INTERFACE_DEFINITION__PRAGMAS:
					return SmePackage.MODEL_DECLARATION__PRAGMAS;
				case SmePackage.INTERFACE_DEFINITION__PROCESS_CLASS:
					return SmePackage.MODEL_DECLARATION__PROCESS_CLASS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.MODEL_DECLARATION__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == ConnectableElement.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.MODEL_DECLARATION__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.MODEL_DECLARATION__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == RootModel.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (processClass: ");
		result.append(processClass);
		result.append(", name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(", modelType: ");
		result.append(modelType);
		result.append(", visibility: ");
		result.append(visibility);
		result.append(", isMain: ");
		result.append(isMain);
		result.append(", isSigaliProcess: ");
		result.append(isSigaliProcess);
		result.append(", compilationDirective: ");
		result.append(compilationDirective);
		result.append(", runOn: ");
		result.append(runOn);
		result.append(", c_CODE: ");
		result.append(c_CODE);
		result.append(", cpp_CODE: ");
		result.append(cpp_CODE);
		result.append(", java_CODE: ");
		result.append(java_CODE);
		result.append(", processor_type: ");
		result.append(processor_type);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (visibility.equals(VisibilityKind.PRIVATE_LITERAL))
				result.append("private ");
			result.append(modelType);
			result.append(" ");
			result.append(name);
			result.append(" = ");
			result.append("{");
			for (Parameter par : getParameters()) {
				result.append(par.toString(asCode, false));
				result.append(";\n   ");
			}
			result.append("}\n   (? ");
			for (Input in : getInputs()) {
				result.append(in.toString(asCode, false));
				result.append(";\n   ");
			}
			result.append("! ");
			for (Output out : getOutputs()) {
				result.append(out.toString(asCode, false));
				result.append(";\n   ");
			}
			result.append(")\n   ");
			result.append(processClass);
			result.append("\n   spec (| ... |)");
			return result.toString();
		}
		else
			return this.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleConstants()
	 * @generated NOT
	 */
	@Override
	public EList<ConstantOrRef> getVisibleConstants() {
		EList<ConstantOrRef> list = super.getVisibleConstants();;
		list.addAll(this.getParameters());
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleConstants());
		
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleModelProcesses()
	 * @generated NOT
	 */
	@Override
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		EList<InterfaceDefinition> list = super.getVisibleModelProcesses();
		list.addAll(this.getModelDeclarations());
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleModelProcesses());
		
		return list;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleSignals()
	 * @generated NOT
	 */
	@Override
	public EList<SignalOrRef> getVisibleSignals() {
		EList<SignalOrRef> list = super.getVisibleSignals();
		list.addAll(this.getInputs());
		list.addAll(this.getOutputs());
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleSignals());
		
		return list;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleTypes()
	 * @generated NOT
	 */
	@Override
	public EList<Identifier> getVisibleTypes() {
		EList<Identifier> list = super.getVisibleTypes();
		
		for (Parameter p : getParameters()) {
			if (p.getType() == "type")
				list.add(p);
		}
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleTypes());
		
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getParent()
	 * @generated NOT
	 */
	@Override
	public SignalContainer getParent() {
		SignalContainer parent = getModelParent();
		if (parent != null)
			return parent;
		parent = getModuleParent();
		if (parent != null)
			return parent;
		else
			return getSpecParent();
	}
	
	/**
	 * 
	 * A model declaration does not define equations in the body of its parent. So bodyTree is not modified.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * Returns the AST of the identifier of the ModelDeclaration
	 * 
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		PKTrees API = PkPlugin.getTreeAPI();
		
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			// Analyze the attribute to obtain the relevant opCode
			int opCode;
			if (this.getVisibility().equals(VisibilityKind.PRIVATE_LITERAL) && this.getParent() instanceof Module) {
				if (modelType.equals(ModelKind.PROCESS_LITERAL))
					opCode = SignalOperators.opprocessp;
				else if (modelType.equals(ModelKind.ACTION_LITERAL))
					opCode = SignalOperators.opactp;
				else if (modelType.equals(ModelKind.FUNCTION_LITERAL))
					opCode = SignalOperators.opfuncp;
				else
					opCode = SignalOperators.opnodep;
			}
			else {
				if (modelType.equals(ModelKind.PROCESS_LITERAL))
					opCode = SignalOperators.opprocess;
				else if (modelType.equals(ModelKind.ACTION_LITERAL))
					opCode = SignalOperators.opact;
				else if (modelType.equals(ModelKind.FUNCTION_LITERAL))
					opCode = SignalOperators.opfunc;
				else
					opCode = SignalOperators.opnode;
			}
			
			long decl = makeInterfaceSignalTree();
			if (decl != PKTrees.ERROR_NODE) {
				long body = super.buildEquationsAST(PKTrees.ERROR_NODE);
				if (API.oper(body) == SignalOperators.opnullproc)
					body = API.makeNullary(SignalOperators.opnil);
				
				decl = API.makeTernary(opCode, idNode, decl, API.makeBinary(SignalOperators.opdesc,
					makePragmasSignalTree(), body));
				
				// If the model is declared at the top level
				if (declTree == PKTrees.ERROR_NODE)
					declTree = decl;
				else
					declTree = API.post(declTree, decl);
			}
		}
		return declTree;
	}
	/**
	 * @return
	 * @generated NOT
	 */
	private long makeInterfaceSignalTree() {
		PKTrees API = PkPlugin.getTreeAPI();
		
		long inteNode;
		if (type == null) {
			long parList;
			if (getParameters().size() == 0)
				parList = API.makeNullary(SignalOperators.opnil);
			else {
				parList = API.makeEmptyList(SignalOperators.opldecs);
				parList = Utils.makeDeclarationList(parameters, parList);
			}
			long inList;
			if (getInputs().size() == 0)
				inList = API.makeNullary(SignalOperators.opnil);
			else {
				inList = API.makeEmptyList(SignalOperators.opldecs);
				inList = Utils.makeDeclarationList(inputs, inList);
			}
			long outList;
			if (getOutputs().size() == 0)
				outList = API.makeNullary(SignalOperators.opnil);
			else {
				outList = API.makeEmptyList(SignalOperators.opldecs);
				outList = Utils.makeDeclarationList(outputs, outList);
			}
			
			// Generate interface using input/output/parameter
			inteNode = API.makeTernary(SignalOperators.opinte, parList, API.makeBinary(SignalOperators.opentsor,
				inList, outList), makeExternalGraphSignalTree());
		}
		else {
			// Use the identifier of the type as interface
			try {
				inteNode = type.getAST(null);
			}
			catch (CyclicDefinitionException cde) {
				// It cannot be thrown normally
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
				inteNode = PKTrees.ERROR_NODE;
			}
		}
		return inteNode;
	}
	
	/**
	 * @return
	 * @generated NOT
	 */
	private long makeExternalGraphSignalTree() {
		PKTrees API = PkPlugin.getTreeAPI();
		long attr;
		if (processClass.equals(ProcessAttributeKind.UNSAFE_LITERAL))
			attr = API.makeNullary(SignalOperators.opunsafe);
		else if (processClass.equals(ProcessAttributeKind.SAFE_LITERAL))
			attr = API.makeNullary(SignalOperators.opsafe);
		else
			attr = API.makeNullary(SignalOperators.opdeterm);
		
		long specNode;
		if (spec != null) {
			// PKTrees.ERROR_NODE, because, it is an independent part, as ModelDeclaration at the top level
			specNode = spec.buildEquationsAST(PKTrees.ERROR_NODE);
		}
		else
			specNode = API.makeNullary(SignalOperators.opnil);
		
		return API.makeBinary(SignalOperators.opspec, API.makeList(SignalOperators.oplattr, attr), specNode);
	}
	
	/**
	 * @return
	 * @generated NOT
	 */
	private long makePragmasSignalTree() {
		PKTrees API = PkPlugin.getTreeAPI();
		
		long result = API.makeEmptyList(SignalOperators.oplprag);
		if (isMain) {
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"Main"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce("true")));
		}
		
		if (isSigaliProcess) {
			
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"Sigali"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce("true")));
			
		}
		
		if (compilationDirective.equals(CompilationDirectiveKind.BLACK_BOX_LITERAL)) {
			
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"BlackBox"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce("true")));
			
		}
		else if (compilationDirective.equals(CompilationDirectiveKind.GREY_BOX_LITERAL)) {
			
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"GreyBox"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce("true")));
			
		}
		else if (compilationDirective.equals(CompilationDirectiveKind.UNEXPANDED_LITERAL)) {
			
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"Unexpanded"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce("true")));
			
		}
		else if (compilationDirective.equals(CompilationDirectiveKind.SEPARATE_LITERAL)) {
			
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"Separate"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce("true")));
			
		}
		else if (compilationDirective.equals(CompilationDirectiveKind.DELAY_CLUSTER_LITERAL)) {
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"DelayCluster"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce("true")));
		}
		
		if (runOn != RUN_ON_EDEFAULT && runOn > 0) {
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"RunOn"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce("" + runOn)));
		}
		
		if (c_CODE != null && "".equals(c_CODE) == false) {
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"C_Code"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce(c_CODE)));
		}
		
		if (cpp_CODE != null && "".equals(cpp_CODE) == false) {
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"CPP_Code"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce(cpp_CODE)));
			
		}
		
		if (java_CODE != null && "".equals(java_CODE) == false) {
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"Java_Code"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce(java_CODE)));
		}
		
		if (processor_type != null && "".equals(processor_type) == false) {
			result = API.post(result, API.makeTernary(SignalOperators.opprag, API.atomString(SignalOperators.opnom,
				"ProcessorType"), API.makeNullary(SignalOperators.opnil), API.makePragmaEnonce(processor_type)));
		}
		
		// Add all comment pragmas: first the Model Declaration one and then all identifiers contain directly (first
		// level) by the Model Declaration
		result = Utils.getCommentAST(this, result);
		for (Input in : getInputs())
			result = Utils.getCommentAST(in, result);
		for (Output out : getOutputs())
			result = Utils.getCommentAST(out, result);
		for (Parameter par : getParameters())
			result = Utils.getCommentAST(par, result);
		for (Automaton aut : getAutomata())
			result = Utils.getCommentAST(aut, result);
		for (ConstantValue cst : getConstants())
			result = Utils.getCommentAST(cst, result);
		for (Index ind : getIndexes())
			result = Utils.getCommentAST(ind, result);
		for (ModelInstance mi : getContainedInstances()) {
			result = Utils.getCommentAST(mi, result);
			for (InputInstance ii : mi.getInputs())
				result = Utils.getCommentAST(ii, result);
			for (OutputInstance oi : mi.getOutputs())
				result = Utils.getCommentAST(oi, result);
			for (ParameterInstance pi : mi.getParameters())
				result = Utils.getCommentAST(pi, result);
		}
		for (Local loc : getLocals())
			result = Utils.getCommentAST(loc, result);
		for (SubProcess sub : getSubprocesses())
			result = Utils.getCommentAST(sub, result);
		for (TypeDeclaration type : getTypes())
			result = Utils.getCommentAST(type, result);
		
		for (Pragma p : getPragmas())
			result = p.buildEquationsAST(result);
		
		if (API.isEmpty(result))
			result = API.makeNullary(SignalOperators.opnil);
		
		return result;
	}
	/**
	 * @return the AST of the Model Declaration Block.
	 * 
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#makeDeclarations(int)
	 * @generated NOT
	 */
	protected long makeDeclarations(long declTree) {
		declTree = super.makeDeclarations(declTree);
		declTree = Utils.makeDeclarationList(getModelDeclarations(), declTree);
		return declTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#makeEquations()
	 * @generated NOT
	 */
	protected long makeEquations(long bodyTree) {
		bodyTree = super.makeEquations(bodyTree);
		bodyTree = Utils.makeEquationsList(getOutputs(), bodyTree);
		return bodyTree;
	}
	
} // ModelDeclarationImpl
