/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.Assertion;
import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.CartesianProduct;
import fr.irisa.espresso.sme.ClockConstraintOperator;
import fr.irisa.espresso.sme.ClockSpeedOperator;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.ConstantRef;
import fr.irisa.espresso.sme.ConstantValue;
import fr.irisa.espresso.sme.ControlflowOperator;
import fr.irisa.espresso.sme.DataflowOperator;
import fr.irisa.espresso.sme.DependenceOperator;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.Index;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.Literal;
import fr.irisa.espresso.sme.Local;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.ModelConnection;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.ModelRef;
import fr.irisa.espresso.sme.ModelType;
import fr.irisa.espresso.sme.Module;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SignalRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.SubProcess;
import fr.irisa.espresso.sme.Switch;
import fr.irisa.espresso.sme.TypeDeclaration;
import fr.irisa.espresso.sme.UseModule;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Model</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getAssertions <em>Assertions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getAutomata <em>Automata</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getConnections <em>Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getConstantRef <em>Constant Ref</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getConstants <em>Constants</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getConstraints <em>Constraints</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getIndexes <em>Indexes</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getContainedInstances <em>Contained Instances</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getLiterals <em>Literals</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getLocals <em>Locals</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getOperators <em>Operators</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getSignalRef <em>Signal Ref</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getSubprocesses <em>Subprocesses</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getTypes <em>Types</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getUseModule <em>Use Module</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelImpl#getModelRefs <em>Model Refs</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public abstract class ModelImpl extends EObjectImpl implements Model {
	/**
	 * The cached value of the '{@link #getAssertions() <em>Assertions</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getAssertions()
	 * @generated
	 * @ordered
	 */
	protected EList<Assertion>				assertions;
	
	/**
	 * The cached value of the '{@link #getAutomata() <em>Automata</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getAutomata()
	 * @generated
	 * @ordered
	 */
	protected EList<Automaton>				automata;
	
	/**
	 * The cached value of the '{@link #getConnections() <em>Connections</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<ModelConnection>		connections;
	
	/**
	 * The cached value of the '{@link #getConstantRef() <em>Constant Ref</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getConstantRef()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantRef>			constantRef;
	
	/**
	 * The cached value of the '{@link #getConstants() <em>Constants</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getConstants()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantValue>			constants;
	
	/**
	 * The cached value of the '{@link #getConstraints() <em>Constraints</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getConstraints()
	 * @generated
	 * @ordered
	 */
	protected EList<ControlflowOperator>	constraints;
	
	/**
	 * The cached value of the '{@link #getIndexes() <em>Indexes</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getIndexes()
	 * @generated
	 * @ordered
	 */
	protected EList<Index>					indexes;
	
	/**
	 * The cached value of the '{@link #getContainedInstances() <em>Contained Instances</em>}' containment reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getContainedInstances()
	 * @generated
	 * @ordered
	 */
	protected EList<ModelInstance>			containedInstances;
	
	/**
	 * The cached value of the '{@link #getLiterals() <em>Literals</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getLiterals()
	 * @generated
	 * @ordered
	 */
	protected EList<Literal>				literals;
	
	/**
	 * The cached value of the '{@link #getLocals() <em>Locals</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getLocals()
	 * @generated
	 * @ordered
	 */
	protected EList<Local>					locals;
	
	/**
	 * The cached value of the '{@link #getOperators() <em>Operators</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getOperators()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowOperator>		operators;
	
	/**
	 * The cached value of the '{@link #getSignalRef() <em>Signal Ref</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSignalRef()
	 * @generated
	 * @ordered
	 */
	protected EList<SignalRef>				signalRef;
	
	/**
	 * The cached value of the '{@link #getSubprocesses() <em>Subprocesses</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSubprocesses()
	 * @generated
	 * @ordered
	 */
	protected EList<SubProcess>				subprocesses;
	
	/**
	 * The cached value of the '{@link #getTypes() <em>Types</em>}' containment reference list. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getTypes()
	 * @generated
	 * @ordered
	 */
	protected EList<TypeDeclaration>		types;
	
	/**
	 * The cached value of the '{@link #getUseModule() <em>Use Module</em>}' containment reference. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getUseModule()
	 * @generated
	 * @ordered
	 */
	protected UseModule						useModule;
	
	/**
	 * The cached value of the '{@link #getModelRefs() <em>Model Refs</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getModelRefs()
	 * @generated
	 * @ordered
	 */
	protected EList<ModelRef>				modelRefs;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModelImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.MODEL;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Assertion> getAssertions() {
		if (assertions == null) {
			assertions = new EObjectContainmentWithInverseEList<Assertion>(Assertion.class, this,
				SmePackage.MODEL__ASSERTIONS, SmePackage.ASSERTION__MODEL_PARENT);
		}
		return assertions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Automaton> getAutomata() {
		if (automata == null) {
			automata = new EObjectContainmentWithInverseEList<Automaton>(Automaton.class, this,
				SmePackage.MODEL__AUTOMATA, SmePackage.AUTOMATON__MODEL_PARENT);
		}
		return automata;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ModelConnection> getConnections() {
		if (connections == null) {
			connections = new EObjectContainmentWithInverseEList<ModelConnection>(ModelConnection.class, this,
				SmePackage.MODEL__CONNECTIONS, SmePackage.MODEL_CONNECTION__MODEL_PARENT);
		}
		return connections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantRef> getConstantRef() {
		if (constantRef == null) {
			constantRef = new EObjectContainmentWithInverseEList<ConstantRef>(ConstantRef.class, this,
				SmePackage.MODEL__CONSTANT_REF, SmePackage.CONSTANT_REF__MODEL_PARENT);
		}
		return constantRef;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantValue> getConstants() {
		if (constants == null) {
			constants = new EObjectContainmentWithInverseEList<ConstantValue>(ConstantValue.class, this,
				SmePackage.MODEL__CONSTANTS, SmePackage.CONSTANT_VALUE__MODEL_PARENT);
		}
		return constants;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ControlflowOperator> getConstraints() {
		if (constraints == null) {
			constraints = new EObjectContainmentWithInverseEList<ControlflowOperator>(ControlflowOperator.class, this,
				SmePackage.MODEL__CONSTRAINTS, SmePackage.CONTROLFLOW_OPERATOR__MODEL_PARENT);
		}
		return constraints;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Index> getIndexes() {
		if (indexes == null) {
			indexes = new EObjectContainmentWithInverseEList<Index>(Index.class, this, SmePackage.MODEL__INDEXES,
				SmePackage.INDEX__MODEL_PARENT);
		}
		return indexes;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ModelInstance> getContainedInstances() {
		if (containedInstances == null) {
			containedInstances = new EObjectContainmentWithInverseEList<ModelInstance>(ModelInstance.class, this,
				SmePackage.MODEL__CONTAINED_INSTANCES, SmePackage.MODEL_INSTANCE__MODEL_PARENT);
		}
		return containedInstances;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Literal> getLiterals() {
		if (literals == null) {
			literals = new EObjectContainmentWithInverseEList<Literal>(Literal.class, this, SmePackage.MODEL__LITERALS,
				SmePackage.LITERAL__MODEL_PARENT);
		}
		return literals;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Local> getLocals() {
		if (locals == null) {
			locals = new EObjectContainmentWithInverseEList<Local>(Local.class, this, SmePackage.MODEL__LOCALS,
				SmePackage.LOCAL__MODEL_PARENT);
		}
		for (DataflowOperator dfo : getOperators()) {
			if (dfo instanceof CartesianProduct) {
				for (OutputPort op : ((CartesianProduct) dfo).getOuts()) {
					boolean toWrite = true;
					for (int index = 0; index < locals.size(); index++) {
						if ((locals.get(index) instanceof LocalImpl)
								&& ((LocalImpl) locals.get(index)).getName().equals(op.getName())) {
							toWrite = false;
						}
					}
					if (toWrite) {
						LocalImpl loc = new LocalImpl();
						loc.setName(op.getName());
						locals.add(loc);
					}
				}
			}
		}
		return locals;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowOperator> getOperators() {
		if (operators == null) {
			operators = new EObjectContainmentWithInverseEList<DataflowOperator>(DataflowOperator.class, this,
				SmePackage.MODEL__OPERATORS, SmePackage.DATAFLOW_OPERATOR__MODEL_PARENT);
		}
		return operators;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<SignalRef> getSignalRef() {
		if (signalRef == null) {
			signalRef = new EObjectContainmentWithInverseEList<SignalRef>(SignalRef.class, this,
				SmePackage.MODEL__SIGNAL_REF, SmePackage.SIGNAL_REF__MODEL_PARENT);
		}
		return signalRef;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<SubProcess> getSubprocesses() {
		if (subprocesses == null) {
			subprocesses = new EObjectContainmentWithInverseEList<SubProcess>(SubProcess.class, this,
				SmePackage.MODEL__SUBPROCESSES, SmePackage.SUB_PROCESS__MODEL_PARENT);
		}
		return subprocesses;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<TypeDeclaration> getTypes() {
		if (types == null) {
			types = new EObjectContainmentWithInverseEList<TypeDeclaration>(TypeDeclaration.class, this,
				SmePackage.MODEL__TYPES, SmePackage.TYPE_DECLARATION__MODEL_PARENT);
		}
		return types;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public UseModule getUseModule() {
		return useModule;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetUseModule(UseModule newUseModule, NotificationChain msgs) {
		UseModule oldUseModule = useModule;
		useModule = newUseModule;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.MODEL__USE_MODULE, oldUseModule, newUseModule);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setUseModule(UseModule newUseModule) {
		if (newUseModule != useModule) {
			NotificationChain msgs = null;
			if (useModule != null)
				msgs = ((InternalEObject) useModule).eInverseRemove(this, SmePackage.USE_MODULE__MODEL_PARENT,
					UseModule.class, msgs);
			if (newUseModule != null)
				msgs = ((InternalEObject) newUseModule).eInverseAdd(this, SmePackage.USE_MODULE__MODEL_PARENT,
					UseModule.class, msgs);
			msgs = basicSetUseModule(newUseModule, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL__USE_MODULE, newUseModule,
				newUseModule));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ModelRef> getModelRefs() {
		if (modelRefs == null) {
			modelRefs = new EObjectContainmentWithInverseEList<ModelRef>(ModelRef.class, this,
				SmePackage.MODEL__MODEL_REFS, SmePackage.MODEL_REF__MODEL_PARENT);
		}
		return modelRefs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<ConstantOrRef> getVisibleConstants() {
		EList<ConstantOrRef> list = new BasicEList<ConstantOrRef>(this.getConstants());
		
		if (useModule != null) {
			for (Module mod : useModule.getModules())
				list.addAll(mod.getExportedConstants());
		}
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		EList<InterfaceDefinition> list = new BasicEList<InterfaceDefinition>();
		for (TypeDeclaration type : getTypes()) {
			if (type instanceof ModelType)
				list.add((ModelType) type);
		}
		
		if (useModule != null) {
			for (Module mod : useModule.getModules()) {
				list.addAll(mod.getExportedModels());
				for (TypeDeclaration type : mod.getExportedTypes()) {
					if (type instanceof ModelType)
						list.add((ModelType) type);
				}
			}
		}
		
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public abstract String getName();
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		// We add local signals + the instance ports of all ModelInstances.
		EList<SignalOrRef> list = new BasicEList<SignalOrRef>(this.getLocals());
		for (ModelInstance mi : getContainedInstances())
			list.addAll(mi.getVisibleSignals());
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<Identifier> getVisibleTypes() {
		EList<Identifier> list = new BasicEList<Identifier>(this.getTypes());
		
		if (useModule != null) {
			for (Module mod : useModule.getModules())
				list.addAll(mod.getExportedTypes());
		}
		
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public abstract SignalContainer getParent();
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.MODEL__ASSERTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getAssertions()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__AUTOMATA:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getAutomata()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getConnections()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__CONSTANT_REF:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getConstantRef()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__CONSTANTS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getConstants()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__CONSTRAINTS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getConstraints()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__INDEXES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getIndexes()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__CONTAINED_INSTANCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getContainedInstances()).basicAdd(otherEnd,
					msgs);
			case SmePackage.MODEL__LITERALS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getLiterals()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__LOCALS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getLocals()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__OPERATORS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getOperators()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__SIGNAL_REF:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSignalRef()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__SUBPROCESSES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSubprocesses()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__TYPES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getTypes()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL__USE_MODULE:
				if (useModule != null)
					msgs = ((InternalEObject) useModule).eInverseRemove(this, EOPPOSITE_FEATURE_BASE
							- SmePackage.MODEL__USE_MODULE, null, msgs);
				return basicSetUseModule((UseModule) otherEnd, msgs);
			case SmePackage.MODEL__MODEL_REFS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getModelRefs()).basicAdd(otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.MODEL__ASSERTIONS:
				return ((InternalEList<?>) getAssertions()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__AUTOMATA:
				return ((InternalEList<?>) getAutomata()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__CONNECTIONS:
				return ((InternalEList<?>) getConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__CONSTANT_REF:
				return ((InternalEList<?>) getConstantRef()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__CONSTANTS:
				return ((InternalEList<?>) getConstants()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__CONSTRAINTS:
				return ((InternalEList<?>) getConstraints()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__INDEXES:
				return ((InternalEList<?>) getIndexes()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__CONTAINED_INSTANCES:
				return ((InternalEList<?>) getContainedInstances()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__LITERALS:
				return ((InternalEList<?>) getLiterals()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__LOCALS:
				return ((InternalEList<?>) getLocals()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__OPERATORS:
				return ((InternalEList<?>) getOperators()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__SIGNAL_REF:
				return ((InternalEList<?>) getSignalRef()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__SUBPROCESSES:
				return ((InternalEList<?>) getSubprocesses()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__TYPES:
				return ((InternalEList<?>) getTypes()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL__USE_MODULE:
				return basicSetUseModule(null, msgs);
			case SmePackage.MODEL__MODEL_REFS:
				return ((InternalEList<?>) getModelRefs()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.MODEL__ASSERTIONS:
				return getAssertions();
			case SmePackage.MODEL__AUTOMATA:
				return getAutomata();
			case SmePackage.MODEL__CONNECTIONS:
				return getConnections();
			case SmePackage.MODEL__CONSTANT_REF:
				return getConstantRef();
			case SmePackage.MODEL__CONSTANTS:
				return getConstants();
			case SmePackage.MODEL__CONSTRAINTS:
				return getConstraints();
			case SmePackage.MODEL__INDEXES:
				return getIndexes();
			case SmePackage.MODEL__CONTAINED_INSTANCES:
				return getContainedInstances();
			case SmePackage.MODEL__LITERALS:
				return getLiterals();
			case SmePackage.MODEL__LOCALS:
				return getLocals();
			case SmePackage.MODEL__OPERATORS:
				return getOperators();
			case SmePackage.MODEL__SIGNAL_REF:
				return getSignalRef();
			case SmePackage.MODEL__SUBPROCESSES:
				return getSubprocesses();
			case SmePackage.MODEL__TYPES:
				return getTypes();
			case SmePackage.MODEL__USE_MODULE:
				return getUseModule();
			case SmePackage.MODEL__MODEL_REFS:
				return getModelRefs();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.MODEL__ASSERTIONS:
				getAssertions().clear();
				getAssertions().addAll((Collection<? extends Assertion>) newValue);
				return;
			case SmePackage.MODEL__AUTOMATA:
				getAutomata().clear();
				getAutomata().addAll((Collection<? extends Automaton>) newValue);
				return;
			case SmePackage.MODEL__CONNECTIONS:
				getConnections().clear();
				getConnections().addAll((Collection<? extends ModelConnection>) newValue);
				return;
			case SmePackage.MODEL__CONSTANT_REF:
				getConstantRef().clear();
				getConstantRef().addAll((Collection<? extends ConstantRef>) newValue);
				return;
			case SmePackage.MODEL__CONSTANTS:
				getConstants().clear();
				getConstants().addAll((Collection<? extends ConstantValue>) newValue);
				return;
			case SmePackage.MODEL__CONSTRAINTS:
				getConstraints().clear();
				getConstraints().addAll((Collection<? extends ControlflowOperator>) newValue);
				return;
			case SmePackage.MODEL__INDEXES:
				getIndexes().clear();
				getIndexes().addAll((Collection<? extends Index>) newValue);
				return;
			case SmePackage.MODEL__CONTAINED_INSTANCES:
				getContainedInstances().clear();
				getContainedInstances().addAll((Collection<? extends ModelInstance>) newValue);
				return;
			case SmePackage.MODEL__LITERALS:
				getLiterals().clear();
				getLiterals().addAll((Collection<? extends Literal>) newValue);
				return;
			case SmePackage.MODEL__LOCALS:
				getLocals().clear();
				getLocals().addAll((Collection<? extends Local>) newValue);
				return;
			case SmePackage.MODEL__OPERATORS:
				getOperators().clear();
				getOperators().addAll((Collection<? extends DataflowOperator>) newValue);
				return;
			case SmePackage.MODEL__SIGNAL_REF:
				getSignalRef().clear();
				getSignalRef().addAll((Collection<? extends SignalRef>) newValue);
				return;
			case SmePackage.MODEL__SUBPROCESSES:
				getSubprocesses().clear();
				getSubprocesses().addAll((Collection<? extends SubProcess>) newValue);
				return;
			case SmePackage.MODEL__TYPES:
				getTypes().clear();
				getTypes().addAll((Collection<? extends TypeDeclaration>) newValue);
				return;
			case SmePackage.MODEL__USE_MODULE:
				setUseModule((UseModule) newValue);
				return;
			case SmePackage.MODEL__MODEL_REFS:
				getModelRefs().clear();
				getModelRefs().addAll((Collection<? extends ModelRef>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.MODEL__ASSERTIONS:
				getAssertions().clear();
				return;
			case SmePackage.MODEL__AUTOMATA:
				getAutomata().clear();
				return;
			case SmePackage.MODEL__CONNECTIONS:
				getConnections().clear();
				return;
			case SmePackage.MODEL__CONSTANT_REF:
				getConstantRef().clear();
				return;
			case SmePackage.MODEL__CONSTANTS:
				getConstants().clear();
				return;
			case SmePackage.MODEL__CONSTRAINTS:
				getConstraints().clear();
				return;
			case SmePackage.MODEL__INDEXES:
				getIndexes().clear();
				return;
			case SmePackage.MODEL__CONTAINED_INSTANCES:
				getContainedInstances().clear();
				return;
			case SmePackage.MODEL__LITERALS:
				getLiterals().clear();
				return;
			case SmePackage.MODEL__LOCALS:
				getLocals().clear();
				return;
			case SmePackage.MODEL__OPERATORS:
				getOperators().clear();
				return;
			case SmePackage.MODEL__SIGNAL_REF:
				getSignalRef().clear();
				return;
			case SmePackage.MODEL__SUBPROCESSES:
				getSubprocesses().clear();
				return;
			case SmePackage.MODEL__TYPES:
				getTypes().clear();
				return;
			case SmePackage.MODEL__USE_MODULE:
				setUseModule((UseModule) null);
				return;
			case SmePackage.MODEL__MODEL_REFS:
				getModelRefs().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.MODEL__ASSERTIONS:
				return assertions != null && !assertions.isEmpty();
			case SmePackage.MODEL__AUTOMATA:
				return automata != null && !automata.isEmpty();
			case SmePackage.MODEL__CONNECTIONS:
				return connections != null && !connections.isEmpty();
			case SmePackage.MODEL__CONSTANT_REF:
				return constantRef != null && !constantRef.isEmpty();
			case SmePackage.MODEL__CONSTANTS:
				return constants != null && !constants.isEmpty();
			case SmePackage.MODEL__CONSTRAINTS:
				return constraints != null && !constraints.isEmpty();
			case SmePackage.MODEL__INDEXES:
				return indexes != null && !indexes.isEmpty();
			case SmePackage.MODEL__CONTAINED_INSTANCES:
				return containedInstances != null && !containedInstances.isEmpty();
			case SmePackage.MODEL__LITERALS:
				return literals != null && !literals.isEmpty();
			case SmePackage.MODEL__LOCALS:
				return locals != null && !locals.isEmpty();
			case SmePackage.MODEL__OPERATORS:
				return operators != null && !operators.isEmpty();
			case SmePackage.MODEL__SIGNAL_REF:
				return signalRef != null && !signalRef.isEmpty();
			case SmePackage.MODEL__SUBPROCESSES:
				return subprocesses != null && !subprocesses.isEmpty();
			case SmePackage.MODEL__TYPES:
				return types != null && !types.isEmpty();
			case SmePackage.MODEL__USE_MODULE:
				return useModule != null;
			case SmePackage.MODEL__MODEL_REFS:
				return modelRefs != null && !modelRefs.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		PKTrees API = PkPlugin.getTreeAPI();
		bodyTree = makeEquations(API.makeEmptyList(SignalOperators.oplfeed));
		if (API.isEmpty(bodyTree))
			bodyTree = API.makeNullary(SignalOperators.opnullproc);
		
		long declTree = makeDeclarations(API.makeEmptyList(SignalOperators.opldecl));
		if (!API.isEmpty(declTree)) // If there are declarations, we create a confined processus
			bodyTree = API.makeBinary(SignalOperators.oppconf, bodyTree, declTree);
		return bodyTree;
	}
	
	/**
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public abstract long getDeclarationAST(long declTree);
	
	/**
	 * @return the AST of the Model Declaration Block.
	 * @generated NOT
	 */
	protected long makeDeclarations(long declTree) {
		declTree = Utils.makeDeclarationList(getTypes(), declTree);
		declTree = Utils.makeDeclarationList(getConstants(), declTree);
		declTree = Utils.makeDeclarationList(getLocals(), declTree);
		declTree = Utils.makeDeclarationList(getAutomata(), declTree);
		declTree = Utils.makeDeclarationList(getContainedInstances(), declTree);
		declTree = Utils.makeDeclarationList(getSubprocesses(), declTree);
		if (useModule != null)
			declTree = useModule.getDeclarationAST(declTree);
		declTree = Utils.makeDeclarationList(getIndexes(), declTree);
		return declTree;
	}
	
	/**
	 * @param l
	 * @return
	 * @generated NOT
	 */
	protected long makeEquations(long l) {
		l = Utils.makeEquationsList(getContainedInstances(), l);
		l = Utils.makeEquationsList(getSubprocesses(), l);
		l = Utils.makeEquationsList(getAutomata(), l);
		l = Utils.makeEquationsList(getAssertions(), l);
		l = Utils.makeEquationsList(getLocals(), l);
		l = Utils.makeEquationsList(getSignalRef(), l);
		l = Utils.makeEquationsList(getIndexes(), l);
		for (ControlflowOperator co : getConstraints()) {
			if (co instanceof ClockConstraintOperator)
				l = ((ClockConstraintOperator) co).buildEquationsAST(l);
			else if (co instanceof ClockSpeedOperator)
				l = ((ClockSpeedOperator) co).buildEquationsAST(l);
			else if (co instanceof DependenceOperator)
				l = ((DependenceOperator) co).buildEquationsAST(l);
		}
		for (DataflowOperator dfo : getOperators()) {
			if (dfo instanceof CartesianProduct)
				l = ((CartesianProduct) dfo).buildEquationsAST(l);
			else if (dfo instanceof Switch)
				l = ((Switch) dfo).buildEquationsAST(l);
		}
		return l;
	}
	
} // ModelImpl
