/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.DependenceInOut;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.ModelType;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.OutputInstance;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.ParameterInstance;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Model Instance</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getSrc_ClockRelations <em>Src Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getDst_ClockRelations <em>Dst Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getSrc_Dependences <em>Src Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getDst_Dependences <em>Dst Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getReferredInterface <em>Referred Interface</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getReferredParameter <em>Referred Parameter</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getInputs <em>Inputs</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getOutputs <em>Outputs</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelInstanceImpl#getParameters <em>Parameters</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class ModelInstanceImpl extends EObjectImpl implements ModelInstance {
	/**
	 * The cached value of the '{@link #getSrc_ClockRelations() <em>Src Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		src_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getDst_ClockRelations() <em>Dst Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		dst_ClockRelations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String		NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String					name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String		COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String					comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc_Dependences() <em>Src Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>			src_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_Dependences() <em>Dst Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>			dst_Dependences;
	
	/**
	 * The cached value of the '{@link #getReferredInterface() <em>Referred Interface</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getReferredInterface()
	 * @generated
	 * @ordered
	 */
	protected InterfaceDefinition		referredInterface;
	
	/**
	 * The cached value of the '{@link #getReferredParameter() <em>Referred Parameter</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getReferredParameter()
	 * @generated
	 * @ordered
	 */
	protected Parameter					referredParameter;
	
	/**
	 * The cached value of the '{@link #getInputs() <em>Inputs</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getInputs()
	 * @generated
	 * @ordered
	 */
	protected EList<InputInstance>		inputs;
	
	/**
	 * The cached value of the '{@link #getOutputs() <em>Outputs</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getOutputs()
	 * @generated
	 * @ordered
	 */
	protected EList<OutputInstance>		outputs;
	
	/**
	 * The cached value of the '{@link #getParameters() <em>Parameters</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getParameters()
	 * @generated
	 * @ordered
	 */
	protected EList<ParameterInstance>	parameters;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModelInstanceImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.MODEL_INSTANCE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getSrc_ClockRelations() {
		if (src_ClockRelations == null) {
			src_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.MODEL_INSTANCE__SRC_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__SRC);
		}
		return src_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getDst_ClockRelations() {
		if (dst_ClockRelations == null) {
			dst_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.MODEL_INSTANCE__DST_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__DST);
		}
		return dst_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_INSTANCE__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_INSTANCE__COMMENT, oldComment,
				comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getSrc_Dependences() {
		if (src_Dependences == null) {
			src_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES, SmePackage.DEPENDENCE__SRC);
		}
		return src_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getDst_Dependences() {
		if (dst_Dependences == null) {
			dst_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.MODEL_INSTANCE__DST_DEPENDENCES, SmePackage.DEPENDENCE__DST);
		}
		return dst_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.MODEL_INSTANCE__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.MODEL_INSTANCE__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.MODEL_INSTANCE__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__CONTAINED_INSTANCES,
					Model.class, msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_INSTANCE__MODEL_PARENT,
				newModelParent, newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public InterfaceDefinition getReferredInterface() {
		if (referredInterface != null && referredInterface.eIsProxy()) {
			InternalEObject oldReferredInterface = (InternalEObject) referredInterface;
			referredInterface = (InterfaceDefinition) eResolveProxy(oldReferredInterface);
			if (referredInterface != oldReferredInterface) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE,
						SmePackage.MODEL_INSTANCE__REFERRED_INTERFACE, oldReferredInterface, referredInterface));
			}
		}
		return referredInterface;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public InterfaceDefinition basicGetReferredInterface() {
		return referredInterface;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setReferredInterface(InterfaceDefinition newReferredInterface) {
		InterfaceDefinition oldReferredInterface = referredInterface;
		referredInterface = newReferredInterface;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_INSTANCE__REFERRED_INTERFACE,
				oldReferredInterface, referredInterface));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Parameter getReferredParameter() {
		if (referredParameter != null && referredParameter.eIsProxy()) {
			InternalEObject oldReferredParameter = (InternalEObject) referredParameter;
			referredParameter = (Parameter) eResolveProxy(oldReferredParameter);
			if (referredParameter != oldReferredParameter) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE,
						SmePackage.MODEL_INSTANCE__REFERRED_PARAMETER, oldReferredParameter, referredParameter));
			}
		}
		return referredParameter;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Parameter basicGetReferredParameter() {
		return referredParameter;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setReferredParameter(Parameter newReferredParameter) {
		Parameter oldReferredParameter = referredParameter;
		referredParameter = newReferredParameter;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_INSTANCE__REFERRED_PARAMETER,
				oldReferredParameter, referredParameter));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<InputInstance> getInputs() {
		if (inputs == null) {
			inputs = new EObjectContainmentWithInverseEList<InputInstance>(InputInstance.class, this,
				SmePackage.MODEL_INSTANCE__INPUTS, SmePackage.INPUT_INSTANCE__INSTANCE_PARENT);
		}
		return inputs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<OutputInstance> getOutputs() {
		if (outputs == null) {
			outputs = new EObjectContainmentWithInverseEList<OutputInstance>(OutputInstance.class, this,
				SmePackage.MODEL_INSTANCE__OUTPUTS, SmePackage.OUTPUT_INSTANCE__INSTANCE_PARENT);
		}
		return outputs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ParameterInstance> getParameters() {
		if (parameters == null) {
			parameters = new EObjectContainmentWithInverseEList<ParameterInstance>(ParameterInstance.class, this,
				SmePackage.MODEL_INSTANCE__PARAMETERS, SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT);
		}
		return parameters;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<ConstantOrRef> getVisibleConstants() {
		return new BasicEList<ConstantOrRef>(getParameters());
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		if (getParent() != null)
			return getParent().getVisibleModelProcesses();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		EList<SignalOrRef> list = new BasicEList<SignalOrRef>(this.getInputs());
		list.addAll(this.getOutputs());
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<Identifier> getVisibleTypes() {
		return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getModelParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.MODEL_INSTANCE__SRC_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.MODEL_INSTANCE__DST_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.MODEL_INSTANCE__DST_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.MODEL_INSTANCE__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__INPUTS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getInputs()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__OUTPUTS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getOutputs()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__PARAMETERS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getParameters()).basicAdd(otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.MODEL_INSTANCE__SRC_CLOCK_RELATIONS:
				return ((InternalEList<?>) getSrc_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__DST_CLOCK_RELATIONS:
				return ((InternalEList<?>) getDst_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES:
				return ((InternalEList<?>) getSrc_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__DST_DEPENDENCES:
				return ((InternalEList<?>) getDst_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
			case SmePackage.MODEL_INSTANCE__INPUTS:
				return ((InternalEList<?>) getInputs()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__OUTPUTS:
				return ((InternalEList<?>) getOutputs()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_INSTANCE__PARAMETERS:
				return ((InternalEList<?>) getParameters()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.MODEL_INSTANCE__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__CONTAINED_INSTANCES, Model.class,
					msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.MODEL_INSTANCE__SRC_CLOCK_RELATIONS:
				return getSrc_ClockRelations();
			case SmePackage.MODEL_INSTANCE__DST_CLOCK_RELATIONS:
				return getDst_ClockRelations();
			case SmePackage.MODEL_INSTANCE__NAME:
				return getName();
			case SmePackage.MODEL_INSTANCE__COMMENT:
				return getComment();
			case SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES:
				return getSrc_Dependences();
			case SmePackage.MODEL_INSTANCE__DST_DEPENDENCES:
				return getDst_Dependences();
			case SmePackage.MODEL_INSTANCE__MODEL_PARENT:
				return getModelParent();
			case SmePackage.MODEL_INSTANCE__REFERRED_INTERFACE:
				if (resolve)
					return getReferredInterface();
				return basicGetReferredInterface();
			case SmePackage.MODEL_INSTANCE__REFERRED_PARAMETER:
				if (resolve)
					return getReferredParameter();
				return basicGetReferredParameter();
			case SmePackage.MODEL_INSTANCE__INPUTS:
				return getInputs();
			case SmePackage.MODEL_INSTANCE__OUTPUTS:
				return getOutputs();
			case SmePackage.MODEL_INSTANCE__PARAMETERS:
				return getParameters();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.MODEL_INSTANCE__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				getSrc_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				getDst_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__NAME:
				setName((String) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				getSrc_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__DST_DEPENDENCES:
				getDst_Dependences().clear();
				getDst_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__REFERRED_INTERFACE:
				setReferredInterface((InterfaceDefinition) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__REFERRED_PARAMETER:
				setReferredParameter((Parameter) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__INPUTS:
				getInputs().clear();
				getInputs().addAll((Collection<? extends InputInstance>) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__OUTPUTS:
				getOutputs().clear();
				getOutputs().addAll((Collection<? extends OutputInstance>) newValue);
				return;
			case SmePackage.MODEL_INSTANCE__PARAMETERS:
				getParameters().clear();
				getParameters().addAll((Collection<? extends ParameterInstance>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.MODEL_INSTANCE__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				return;
			case SmePackage.MODEL_INSTANCE__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				return;
			case SmePackage.MODEL_INSTANCE__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.MODEL_INSTANCE__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				return;
			case SmePackage.MODEL_INSTANCE__DST_DEPENDENCES:
				getDst_Dependences().clear();
				return;
			case SmePackage.MODEL_INSTANCE__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.MODEL_INSTANCE__REFERRED_INTERFACE:
				setReferredInterface((InterfaceDefinition) null);
				return;
			case SmePackage.MODEL_INSTANCE__REFERRED_PARAMETER:
				setReferredParameter((Parameter) null);
				return;
			case SmePackage.MODEL_INSTANCE__INPUTS:
				getInputs().clear();
				return;
			case SmePackage.MODEL_INSTANCE__OUTPUTS:
				getOutputs().clear();
				return;
			case SmePackage.MODEL_INSTANCE__PARAMETERS:
				getParameters().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.MODEL_INSTANCE__SRC_CLOCK_RELATIONS:
				return src_ClockRelations != null && !src_ClockRelations.isEmpty();
			case SmePackage.MODEL_INSTANCE__DST_CLOCK_RELATIONS:
				return dst_ClockRelations != null && !dst_ClockRelations.isEmpty();
			case SmePackage.MODEL_INSTANCE__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.MODEL_INSTANCE__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES:
				return src_Dependences != null && !src_Dependences.isEmpty();
			case SmePackage.MODEL_INSTANCE__DST_DEPENDENCES:
				return dst_Dependences != null && !dst_Dependences.isEmpty();
			case SmePackage.MODEL_INSTANCE__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.MODEL_INSTANCE__REFERRED_INTERFACE:
				return referredInterface != null;
			case SmePackage.MODEL_INSTANCE__REFERRED_PARAMETER:
				return referredParameter != null;
			case SmePackage.MODEL_INSTANCE__INPUTS:
				return inputs != null && !inputs.isEmpty();
			case SmePackage.MODEL_INSTANCE__OUTPUTS:
				return outputs != null && !outputs.isEmpty();
			case SmePackage.MODEL_INSTANCE__PARAMETERS:
				return parameters != null && !parameters.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_INSTANCE__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_INSTANCE__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES;
				case SmePackage.MODEL_INSTANCE__DST_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		if (baseClass == SignalContainer.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.MODEL_INSTANCE__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.MODEL_INSTANCE__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (baseFeatureID) {
				case SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES:
					return SmePackage.MODEL_INSTANCE__SRC_DEPENDENCES;
				case SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES:
					return SmePackage.MODEL_INSTANCE__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		if (baseClass == SignalContainer.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			result.append("refers: \n");
			InterfaceDefinition id = this.getReferredInterface();
			if (id != null)
				result.append(id.toString(asCode, true));
			else
				result.append("(null)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#buildEquationsAST(int)
	 * 
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		// Build the clock and dependence relation, whose target is the ModelInstance
		for (ClockRelation cr : getDst_ClockRelations())
			bodyTree = cr.buildEquationsAST(bodyTree);
		for (Dependence dep : getDst_Dependences())
			bodyTree = dep.buildEquationsAST(bodyTree);
		
		PKTrees API = PkPlugin.getTreeAPI();
		
		InterfaceDefinition id = this.getReferredInterface();
		if (id != null) {
			// Resolution of Bug #7124 by removing this line: bodyTree = API.makeEmptyList(SignalOperators.oplfeed);
			bodyTree = Utils.makeEquationsList(getParameters(), bodyTree);
			bodyTree = Utils.makeEquationsList(getInputs(), bodyTree);
			bodyTree = Utils.makeEquationsList(getOutputs(), bodyTree);
			
			boolean eqOK = true;
			if (parameters.size() != id.getParameters().size()) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.MODEL_INSTANCE_PARAMETERS_NUMBER, this);
				eqOK = false;
			}
			if (inputs.size() != id.getInputs().size()) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.MODEL_INSTANCE_INPUT_SIGNALS_NUMBER, this);
				eqOK = false;
			}
			if (getOutputs().size() != id.getOutputs().size()) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.MODEL_INSTANCE_OUTPUT_SIGNALS_NUMBER, this);
				eqOK = false;
			}
			try {
				long callNode = PKTrees.ERROR_NODE;
				Parameter par = this.getReferredParameter();
				if (!(id instanceof ModelType)) {
					if (par != null)
						PolychronyDiagnostic.add(Diagnostic.WARNING,
							WarningMessages.MODEL_INSTANCE_PARAMETER_ATTRIBUTE, this);
					
					// Put null for the Set of SignalElement because we only get its name
					callNode = id.getAST(null);
				}
				else if (par != null) // id is a ModelType
				{
					// Put null for the Set of SignalElement because we only get its name
					callNode = par.getAST(null);
				}
				else {
					PolychronyDiagnostic
							.add(Diagnostic.ERROR, ErrorMessages.MODEL_INSTANCE_NO_REFERRED_PARAMETER, this);
					eqOK = false;
					callNode = PKTrees.ERROR_NODE;
				}
				
				if (eqOK && callNode != PKTrees.ERROR_NODE) {
					long paramList = API.makeEmptyList(SignalOperators.oplepara);
					for (ParameterInstance pi : parameters) {
						// Put null for the Set of SignalElement because we only get its name
						long node = pi.getAST(null);
						if (node != PKTrees.ERROR_NODE)
							paramList = API.post(paramList, node);
					}
					if (API.isEmpty(paramList))
						paramList = API.makeNullary(SignalOperators.opnil);
					
					long inList = API.makeEmptyList(SignalOperators.oplesign);
					for (InputInstance ii : inputs) {
						// Put null for the Set of SignalElement because we only get its name
						long node = ii.getAST(null);
						if (node != PKTrees.ERROR_NODE)
							inList = API.post(inList, node);
					}
					if (API.isEmpty(inList))
						inList = API.makeNullary(SignalOperators.opnil);
					
					callNode = API.makeBinary(SignalOperators.opappe, API.makeBinary(SignalOperators.opinst, callNode,
						paramList), inList);
					
					long outList = API.makeEmptyList(SignalOperators.opleflot);
					int outArity = 0;
					for (OutputInstance oi : outputs) {
						// Put null for the Set of SignalElement because we only get its name
						long node = oi.getAST(null);
						if (node != PKTrees.ERROR_NODE) {
							outArity++;
							outList = API.post(outList, node);
						}
					}
					if (outArity != 0)
						callNode = API.makeBinary(SignalOperators.opdefsig, outList, callNode);
					
					// Add the label to the instantiation
					long idNode = API.atomString(SignalOperators.opnom, name);
					if (idNode != PKTrees.ERROR_NODE)
						callNode = API.makeBinary(SignalOperators.oppetiq, idNode, callNode);
					
					// Add the equation to the created subProcess
					bodyTree = API.post(bodyTree, callNode);
				}
			}
			catch (CyclicDefinitionException cde) {
				// It is impossible to obtain this exception. All analyzed elements are terminal.
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.MODEL_INSTANCE_NO_REFERRED_INTERFACE + name, this);
		}
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			long decl = API.makeUnary(SignalOperators.oplab, API.makeList(SignalOperators.oplident, idNode));
			declTree = API.post(declTree, decl);
		}
		
		declTree = Utils.makeDeclarationList(getParameters(), declTree);
		declTree = Utils.makeDeclarationList(getInputs(), declTree);
		declTree = Utils.makeDeclarationList(getOutputs(), declTree);
		
		return declTree;
	}
	
} // ModelInstanceImpl
