/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.Input;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.ModelKind;
import fr.irisa.espresso.sme.ModelType;
import fr.irisa.espresso.sme.Module;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.Output;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.ParameterType;
import fr.irisa.espresso.sme.Pragma;
import fr.irisa.espresso.sme.ProcessAttributeKind;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SmeFactory;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.Specifications;
import fr.irisa.espresso.sme.TypeDeclaration;
import fr.irisa.espresso.sme.VisibilityKind;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Model Type</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getType <em>Type</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getInputs <em>Inputs</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getOutputs <em>Outputs</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getParameters <em>Parameters</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getSpec <em>Spec</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getPragmas <em>Pragmas</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getProcessClass <em>Process Class</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getModuleParent <em>Module Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getVisibility <em>Visibility</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ModelTypeImpl#getModelType <em>Model Type</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class ModelTypeImpl extends EObjectImpl implements ModelType {
	/**
	 * The cached value of the '{@link #getType() <em>Type</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getType()
	 * @generated
	 * @ordered
	 */
	protected ModelType							type;
	
	/**
	 * The cached value of the '{@link #getInputs() <em>Inputs</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getInputs()
	 * @generated
	 * @ordered
	 */
	protected EList<Input>						inputs;
	
	/**
	 * The cached value of the '{@link #getOutputs() <em>Outputs</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getOutputs()
	 * @generated
	 * @ordered
	 */
	protected EList<Output>						outputs;
	
	/**
	 * The cached value of the '{@link #getParameters() <em>Parameters</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getParameters()
	 * @generated
	 * @ordered
	 */
	protected EList<Parameter>					parameters;
	
	/**
	 * The cached value of the '{@link #getSpec() <em>Spec</em>}' containment reference. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSpec()
	 * @generated
	 * @ordered
	 */
	protected Specifications					spec;
	
	/**
	 * The cached value of the '{@link #getPragmas() <em>Pragmas</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getPragmas()
	 * @generated
	 * @ordered
	 */
	protected EList<Pragma>						pragmas;
	
	/**
	 * The default value of the '{@link #getProcessClass() <em>Process Class</em>}' attribute. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getProcessClass()
	 * @generated
	 * @ordered
	 */
	protected static final ProcessAttributeKind	PROCESS_CLASS_EDEFAULT	= ProcessAttributeKind.UNSAFE_LITERAL;
	
	/**
	 * The cached value of the '{@link #getProcessClass() <em>Process Class</em>}' attribute. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @see #getProcessClass()
	 * @generated
	 * @ordered
	 */
	protected ProcessAttributeKind				processClass			= PROCESS_CLASS_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String				NAME_EDEFAULT			= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String							name					= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String				COMMENT_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String							comment					= COMMENT_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getVisibility() <em>Visibility</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getVisibility()
	 * @generated
	 * @ordered
	 */
	protected static final VisibilityKind		VISIBILITY_EDEFAULT		= VisibilityKind.PUBLIC_LITERAL;
	
	/**
	 * The cached value of the '{@link #getVisibility() <em>Visibility</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getVisibility()
	 * @generated
	 * @ordered
	 */
	protected VisibilityKind					visibility				= VISIBILITY_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getModelType() <em>Model Type</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getModelType()
	 * @generated
	 * @ordered
	 */
	protected static final ModelKind			MODEL_TYPE_EDEFAULT		= ModelKind.PROCESS_LITERAL;
	
	/**
	 * The cached value of the '{@link #getModelType() <em>Model Type</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getModelType()
	 * @generated
	 * @ordered
	 */
	protected ModelKind							modelType				= MODEL_TYPE_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ModelTypeImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.MODEL_TYPE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelType getType() {
		if (type != null && type.eIsProxy()) {
			InternalEObject oldType = (InternalEObject) type;
			type = (ModelType) eResolveProxy(oldType);
			if (type != oldType) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.MODEL_TYPE__TYPE, oldType,
						type));
			}
		}
		return type;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelType basicGetType() {
		return type;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setType(ModelType newType) {
		ModelType oldType = type;
		type = newType;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__TYPE, oldType, type));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Input> getInputs() {
		if (inputs == null) {
			inputs = new EObjectContainmentWithInverseEList<Input>(Input.class, this, SmePackage.MODEL_TYPE__INPUTS,
				SmePackage.INPUT__INTERFACE_PARENT);
		}
		return inputs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Output> getOutputs() {
		if (outputs == null) {
			outputs = new EObjectContainmentWithInverseEList<Output>(Output.class, this,
				SmePackage.MODEL_TYPE__OUTPUTS, SmePackage.OUTPUT__INTERFACE_PARENT);
		}
		return outputs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Parameter> getParameters() {
		if (parameters == null) {
			parameters = new EObjectContainmentWithInverseEList<Parameter>(Parameter.class, this,
				SmePackage.MODEL_TYPE__PARAMETERS, SmePackage.PARAMETER__INTERFACE_PARENT);
		}
		return parameters;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public Specifications getSpec() {
		if (spec == null) {
			spec = SmeFactory.eINSTANCE.createSpecifications();
			spec.setInterfaceParent(this);
		}
		return spec;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSpec(Specifications newSpec, NotificationChain msgs) {
		Specifications oldSpec = spec;
		spec = newSpec;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__SPEC,
				oldSpec, newSpec);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSpec(Specifications newSpec) {
		if (newSpec != spec) {
			NotificationChain msgs = null;
			if (spec != null)
				msgs = ((InternalEObject) spec).eInverseRemove(this, SmePackage.SPECIFICATIONS__INTERFACE_PARENT,
					Specifications.class, msgs);
			if (newSpec != null)
				msgs = ((InternalEObject) newSpec).eInverseAdd(this, SmePackage.SPECIFICATIONS__INTERFACE_PARENT,
					Specifications.class, msgs);
			msgs = basicSetSpec(newSpec, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__SPEC, newSpec, newSpec));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Pragma> getPragmas() {
		if (pragmas == null) {
			pragmas = new EObjectContainmentWithInverseEList<Pragma>(Pragma.class, this,
				SmePackage.MODEL_TYPE__PRAGMAS, SmePackage.PRAGMA__INTERFACE_PARENT);
		}
		return pragmas;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ProcessAttributeKind getProcessClass() {
		return processClass;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setProcessClass(ProcessAttributeKind newProcessClass) {
		ProcessAttributeKind oldProcessClass = processClass;
		processClass = newProcessClass == null ? PROCESS_CLASS_EDEFAULT : newProcessClass;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__PROCESS_CLASS,
				oldProcessClass, processClass));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Module getModuleParent() {
		if (eContainerFeatureID != SmePackage.MODEL_TYPE__MODULE_PARENT)
			return null;
		return (Module) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModuleParent(Module newModuleParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModuleParent, SmePackage.MODEL_TYPE__MODULE_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModuleParent(Module newModuleParent) {
		if (newModuleParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.MODEL_TYPE__MODULE_PARENT && newModuleParent != null)) {
			if (EcoreUtil.isAncestor(this, newModuleParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModuleParent != null)
				msgs = ((InternalEObject) newModuleParent).eInverseAdd(this, SmePackage.MODULE__TYPES, Module.class,
					msgs);
			msgs = basicSetModuleParent(newModuleParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__MODULE_PARENT,
				newModuleParent, newModuleParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.MODEL_TYPE__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.MODEL_TYPE__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.MODEL_TYPE__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__TYPES, Model.class, msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__MODEL_PARENT, newModelParent,
				newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public VisibilityKind getVisibility() {
		return visibility;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setVisibility(VisibilityKind newVisibility) {
		VisibilityKind oldVisibility = visibility;
		visibility = newVisibility == null ? VISIBILITY_EDEFAULT : newVisibility;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__VISIBILITY, oldVisibility,
				visibility));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelKind getModelType() {
		return modelType;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelType(ModelKind newModelType) {
		ModelKind oldModelType = modelType;
		modelType = newModelType == null ? MODEL_TYPE_EDEFAULT : newModelType;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.MODEL_TYPE__MODEL_TYPE, oldModelType,
				modelType));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<ConstantOrRef> getVisibleConstants() {
		EList<ConstantOrRef> list = new BasicEList<ConstantOrRef>(this.getParameters());
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleConstants());
		
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		// Signal Compiler forbids the use of other ModelProcesses than those declared
		// in the Specifications part of the ModelType
		return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		// Signal Compiler forbids the use of other Signals than those declared
		// in the Interface part and in the Specifications part of the ModelType
		EList<SignalOrRef> list = new BasicEList<SignalOrRef>(this.getInputs());
		list.addAll(this.getOutputs());
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<Identifier> getVisibleTypes() {
		EList<Identifier> list = new BasicEList<Identifier>();
		// We add the Parameters which are of type "type"
		for (Parameter p : getParameters()) {
			if (p.getType() == "type")
				list.add(p);
		}
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleTypes());
		
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		SignalContainer parent = getModelParent();
		if (parent != null)
			return parent;
		else
			return getModuleParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.MODEL_TYPE__INPUTS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getInputs()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_TYPE__OUTPUTS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getOutputs()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_TYPE__PARAMETERS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getParameters()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_TYPE__SPEC:
				if (spec != null)
					msgs = ((InternalEObject) spec).eInverseRemove(this, EOPPOSITE_FEATURE_BASE
							- SmePackage.MODEL_TYPE__SPEC, null, msgs);
				return basicSetSpec((Specifications) otherEnd, msgs);
			case SmePackage.MODEL_TYPE__PRAGMAS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getPragmas()).basicAdd(otherEnd, msgs);
			case SmePackage.MODEL_TYPE__MODULE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModuleParent((Module) otherEnd, msgs);
			case SmePackage.MODEL_TYPE__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.MODEL_TYPE__INPUTS:
				return ((InternalEList<?>) getInputs()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_TYPE__OUTPUTS:
				return ((InternalEList<?>) getOutputs()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_TYPE__PARAMETERS:
				return ((InternalEList<?>) getParameters()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_TYPE__SPEC:
				return basicSetSpec(null, msgs);
			case SmePackage.MODEL_TYPE__PRAGMAS:
				return ((InternalEList<?>) getPragmas()).basicRemove(otherEnd, msgs);
			case SmePackage.MODEL_TYPE__MODULE_PARENT:
				return basicSetModuleParent(null, msgs);
			case SmePackage.MODEL_TYPE__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.MODEL_TYPE__MODULE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODULE__TYPES, Module.class, msgs);
			case SmePackage.MODEL_TYPE__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__TYPES, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.MODEL_TYPE__TYPE:
				if (resolve)
					return getType();
				return basicGetType();
			case SmePackage.MODEL_TYPE__INPUTS:
				return getInputs();
			case SmePackage.MODEL_TYPE__OUTPUTS:
				return getOutputs();
			case SmePackage.MODEL_TYPE__PARAMETERS:
				return getParameters();
			case SmePackage.MODEL_TYPE__SPEC:
				return getSpec();
			case SmePackage.MODEL_TYPE__PRAGMAS:
				return getPragmas();
			case SmePackage.MODEL_TYPE__PROCESS_CLASS:
				return getProcessClass();
			case SmePackage.MODEL_TYPE__NAME:
				return getName();
			case SmePackage.MODEL_TYPE__COMMENT:
				return getComment();
			case SmePackage.MODEL_TYPE__MODULE_PARENT:
				return getModuleParent();
			case SmePackage.MODEL_TYPE__MODEL_PARENT:
				return getModelParent();
			case SmePackage.MODEL_TYPE__VISIBILITY:
				return getVisibility();
			case SmePackage.MODEL_TYPE__MODEL_TYPE:
				return getModelType();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.MODEL_TYPE__TYPE:
				setType((ModelType) newValue);
				return;
			case SmePackage.MODEL_TYPE__INPUTS:
				getInputs().clear();
				getInputs().addAll((Collection<? extends Input>) newValue);
				return;
			case SmePackage.MODEL_TYPE__OUTPUTS:
				getOutputs().clear();
				getOutputs().addAll((Collection<? extends Output>) newValue);
				return;
			case SmePackage.MODEL_TYPE__PARAMETERS:
				getParameters().clear();
				getParameters().addAll((Collection<? extends Parameter>) newValue);
				return;
			case SmePackage.MODEL_TYPE__SPEC:
				setSpec((Specifications) newValue);
				return;
			case SmePackage.MODEL_TYPE__PRAGMAS:
				getPragmas().clear();
				getPragmas().addAll((Collection<? extends Pragma>) newValue);
				return;
			case SmePackage.MODEL_TYPE__PROCESS_CLASS:
				setProcessClass((ProcessAttributeKind) newValue);
				return;
			case SmePackage.MODEL_TYPE__NAME:
				setName((String) newValue);
				return;
			case SmePackage.MODEL_TYPE__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.MODEL_TYPE__MODULE_PARENT:
				setModuleParent((Module) newValue);
				return;
			case SmePackage.MODEL_TYPE__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.MODEL_TYPE__VISIBILITY:
				setVisibility((VisibilityKind) newValue);
				return;
			case SmePackage.MODEL_TYPE__MODEL_TYPE:
				setModelType((ModelKind) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.MODEL_TYPE__TYPE:
				setType((ModelType) null);
				return;
			case SmePackage.MODEL_TYPE__INPUTS:
				getInputs().clear();
				return;
			case SmePackage.MODEL_TYPE__OUTPUTS:
				getOutputs().clear();
				return;
			case SmePackage.MODEL_TYPE__PARAMETERS:
				getParameters().clear();
				return;
			case SmePackage.MODEL_TYPE__SPEC:
				setSpec((Specifications) null);
				return;
			case SmePackage.MODEL_TYPE__PRAGMAS:
				getPragmas().clear();
				return;
			case SmePackage.MODEL_TYPE__PROCESS_CLASS:
				setProcessClass(PROCESS_CLASS_EDEFAULT);
				return;
			case SmePackage.MODEL_TYPE__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.MODEL_TYPE__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.MODEL_TYPE__MODULE_PARENT:
				setModuleParent((Module) null);
				return;
			case SmePackage.MODEL_TYPE__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.MODEL_TYPE__VISIBILITY:
				setVisibility(VISIBILITY_EDEFAULT);
				return;
			case SmePackage.MODEL_TYPE__MODEL_TYPE:
				setModelType(MODEL_TYPE_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.MODEL_TYPE__TYPE:
				return type != null;
			case SmePackage.MODEL_TYPE__INPUTS:
				return inputs != null && !inputs.isEmpty();
			case SmePackage.MODEL_TYPE__OUTPUTS:
				return outputs != null && !outputs.isEmpty();
			case SmePackage.MODEL_TYPE__PARAMETERS:
				return parameters != null && !parameters.isEmpty();
			case SmePackage.MODEL_TYPE__SPEC:
				return spec != null;
			case SmePackage.MODEL_TYPE__PRAGMAS:
				return pragmas != null && !pragmas.isEmpty();
			case SmePackage.MODEL_TYPE__PROCESS_CLASS:
				return processClass != PROCESS_CLASS_EDEFAULT;
			case SmePackage.MODEL_TYPE__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.MODEL_TYPE__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.MODEL_TYPE__MODULE_PARENT:
				return getModuleParent() != null;
			case SmePackage.MODEL_TYPE__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.MODEL_TYPE__VISIBILITY:
				return visibility != VISIBILITY_EDEFAULT;
			case SmePackage.MODEL_TYPE__MODEL_TYPE:
				return modelType != MODEL_TYPE_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_TYPE__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == ConnectableElement.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_TYPE__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == TypeDeclaration.class) {
			switch (derivedFeatureID) {
				case SmePackage.MODEL_TYPE__MODULE_PARENT:
					return SmePackage.TYPE_DECLARATION__MODULE_PARENT;
				case SmePackage.MODEL_TYPE__MODEL_PARENT:
					return SmePackage.TYPE_DECLARATION__MODEL_PARENT;
				case SmePackage.MODEL_TYPE__VISIBILITY:
					return SmePackage.TYPE_DECLARATION__VISIBILITY;
				default:
					return -1;
			}
		}
		if (baseClass == ParameterType.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.MODEL_TYPE__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == ConnectableElement.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.MODEL_TYPE__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == TypeDeclaration.class) {
			switch (baseFeatureID) {
				case SmePackage.TYPE_DECLARATION__MODULE_PARENT:
					return SmePackage.MODEL_TYPE__MODULE_PARENT;
				case SmePackage.TYPE_DECLARATION__MODEL_PARENT:
					return SmePackage.MODEL_TYPE__MODEL_PARENT;
				case SmePackage.TYPE_DECLARATION__VISIBILITY:
					return SmePackage.MODEL_TYPE__VISIBILITY;
				default:
					return -1;
			}
		}
		if (baseClass == ParameterType.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (processClass: ");
		result.append(processClass);
		result.append(", name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(", visibility: ");
		result.append(visibility);
		result.append(", modelType: ");
		result.append(modelType);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
				result.append("private ");
			result.append("type ");
			result.append(modelType);
			result.append(" ");
			result.append(name);
			result.append(" = ");
			result.append("{");
			for (Parameter p : getParameters()) {
				result.append(p.toString(asCode, false));
				result.append(";\n   ");
			}
			result.append("}\n   (! ");
			for (Input in : getInputs()) {
				result.append(in.toString(asCode, false));
				result.append(";\n   ");
			}
			result.append("? ");
			for (Output out : getOutputs()) {
				result.append(out.toString(asCode, false));
				result.append(";\n   ");
			}
			result.append(")\n spec (| ... |)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * 
	 * A model type declaration does not define equations in the body of its parent.
	 * 
	 * @see fr.irisa.espresso.sme.Identifier#buildEquationsAST(int)
	 * 
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			
			// Analyze the attribute to obtain the relevant opCode
			int opCode;
			if (modelType.equals(ModelKind.PROCESS_LITERAL))
				opCode = SignalOperators.optypeproc;
			else if (modelType.equals(ModelKind.ACTION_LITERAL))
				opCode = SignalOperators.optypeact;
			else if (modelType.equals(ModelKind.FUNCTION_LITERAL))
				opCode = SignalOperators.optypefunc;
			else
				opCode = SignalOperators.optypenode;
			
			long inteNode;
			if (type == null) {
				long parList = Utils.makeDeclarationList(parameters, API.makeEmptyList(SignalOperators.opldecs));
				if (API.isEmpty(parList))
					parList = API.makeNullary(SignalOperators.opnil);
				
				long inList = Utils.makeDeclarationList(inputs, API.makeEmptyList(SignalOperators.opldecs));
				if (API.isEmpty(inList))
					inList = API.makeNullary(SignalOperators.opnil);
				
				long outList = Utils.makeDeclarationList(outputs, API.makeEmptyList(SignalOperators.opldecs));
				if (API.isEmpty(outList))
					outList = API.makeNullary(SignalOperators.opnil);
				
				// Generate interface using input/output/parameter
				inteNode = API.makeTernary(SignalOperators.opinte, parList, API.makeBinary(SignalOperators.opentsor,
					inList, outList), makeExternalGraphSignalTree());
			}
			else {
				// Use the identifier of the type as interface
				try {
					inteNode = type.getAST(null);
				}
				catch (CyclicDefinitionException cde) {
					// It cannot be thrown normally
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
							.getElement());
					inteNode = PKTrees.ERROR_NODE;
				}
			}
			
			if (inteNode != PKTrees.ERROR_NODE) {
				long decl = API.makeTernary(opCode, idNode, inteNode, makePragmasSignalTree());
				if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
					decl = API.makeList(SignalOperators.opltypep, decl);
				else
					decl = API.makeList(SignalOperators.opltype, decl);
				
				declTree = API.post(declTree, decl);
			}
		}
		return declTree;
	}
	
	/**
	 * @return
	 * @generated NOT
	 */
	private long makePragmasSignalTree() {
		PKTrees API = PkPlugin.getTreeAPI();
		// Add all comment pragmas: first the Model Type one and then all input, output and parameter ones
		long result = Utils.getCommentAST(this, API.makeEmptyList(SignalOperators.oplprag));
		for (Input in : getInputs())
			result = Utils.getCommentAST(in, result);
		for (Output out : getOutputs())
			result = Utils.getCommentAST(out, result);
		for (Parameter par : getParameters())
			result = Utils.getCommentAST(par, result);
		
		for (Pragma p : getPragmas())
			result = p.buildEquationsAST(result);
		
		if (API.isEmpty(result))
			result = API.makeNullary(SignalOperators.opnil);
		
		return result;
	}
	
	/**
	 * @return
	 * @generated NOT
	 */
	private long makeExternalGraphSignalTree() {
		PKTrees API = PkPlugin.getTreeAPI();
		long attr;
		if (processClass.equals(ProcessAttributeKind.UNSAFE_LITERAL))
			attr = API.makeNullary(SignalOperators.opunsafe);
		else if (processClass.equals(ProcessAttributeKind.SAFE_LITERAL))
			attr = API.makeNullary(SignalOperators.opsafe);
		else
			attr = API.makeNullary(SignalOperators.opdeterm);
		
		long specNode;
		if (spec != null) {
			// PKTrees.ERROR_NODE, because, it is an independent part, as ModelDeclaration at the top level
			specNode = spec.buildEquationsAST(PKTrees.ERROR_NODE);
		}
		else
			specNode = API.makeNullary(SignalOperators.opnil);
		
		return API.makeBinary(SignalOperators.opspec, API.makeList(SignalOperators.oplattr, attr), specNode);
	}
	
} // ModelTypeImpl
