/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalExpr;
import fr.irisa.espresso.sme.SignalOperator;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Output Port</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.OutputPortImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.OutputPortImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.OutputPortImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.OutputPortImpl#getSrc_ConstantAffectations <em>Src Constant Affectations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.OutputPortImpl#getOperatorParent <em>Operator Parent</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class OutputPortImpl extends EObjectImpl implements OutputPort {
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String			NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String						name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String			COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String						comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>		src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The cached value of the '{@link #getOperatorParent() <em>Operator Parent</em>}' reference. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getOperatorParent()
	 * @generated
	 * @ordered
	 */
	protected SignalOperator				operatorParent;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected OutputPortImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.OUTPUT_PORT;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.OUTPUT_PORT__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.OUTPUT_PORT__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalOperator getOperatorParent() {
		if (operatorParent != null && operatorParent.eIsProxy()) {
			InternalEObject oldOperatorParent = (InternalEObject) operatorParent;
			operatorParent = (SignalOperator) eResolveProxy(oldOperatorParent);
			if (operatorParent != oldOperatorParent) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.OUTPUT_PORT__OPERATOR_PARENT,
						oldOperatorParent, operatorParent));
			}
		}
		return operatorParent;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalOperator basicGetOperatorParent() {
		return operatorParent;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setOperatorParent(SignalOperator newOperatorParent) {
		SignalOperator oldOperatorParent = operatorParent;
		operatorParent = newOperatorParent;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.OUTPUT_PORT__OPERATOR_PARENT,
				oldOperatorParent, operatorParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getOperatorParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.OUTPUT_PORT__NAME:
				return getName();
			case SmePackage.OUTPUT_PORT__COMMENT:
				return getComment();
			case SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.OUTPUT_PORT__OPERATOR_PARENT:
				if (resolve)
					return getOperatorParent();
				return basicGetOperatorParent();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.OUTPUT_PORT__NAME:
				setName((String) newValue);
				return;
			case SmePackage.OUTPUT_PORT__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.OUTPUT_PORT__OPERATOR_PARENT:
				setOperatorParent((SignalOperator) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.OUTPUT_PORT__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.OUTPUT_PORT__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.OUTPUT_PORT__OPERATOR_PARENT:
				setOperatorParent((SignalOperator) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.OUTPUT_PORT__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.OUTPUT_PORT__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.OUTPUT_PORT__OPERATOR_PARENT:
				return operatorParent != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == SignalExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS:
					return SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == SignalExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS:
					return SmePackage.OUTPUT_PORT__SRC_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.OUTPUT_PORT__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			result.append(" " + name);
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * No definition equations can be built from an OutputPort, except the cartesianProduct.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecsig, API
					.makeNullary(SignalOperators.opimplicit), API.makeList(SignalOperators.oplsign, idNode)));
		}
		return declTree;
	}
	
} // OutputPortImpl
