/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EDataTypeUniqueEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionExpr;
import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalIntrinsicType;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Parameter</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getSrc_ConstantAffectations <em>Src Constant Affectations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getDst_ConstantAffectation <em>Dst Constant Affectation</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getSrc_Conditions <em>Src Conditions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getInterfaceParent <em>Interface Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getArrayDimensions <em>Array Dimensions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterImpl#getType <em>Type</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class ParameterImpl extends EObjectImpl implements Parameter {
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>		src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String			NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String						name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String			COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String						comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getDst_ConstantAffectation() <em>Dst Constant Affectation</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ConstantAffectation()
	 * @generated
	 * @ordered
	 */
	protected ConstantAffectation			dst_ConstantAffectation;
	
	/**
	 * The cached value of the '{@link #getSrc_Conditions() <em>Src Conditions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Conditions()
	 * @generated
	 * @ordered
	 */
	protected EList<Condition>				src_Conditions;
	
	/**
	 * The cached value of the '{@link #getArrayDimensions() <em>Array Dimensions</em>}' attribute list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getArrayDimensions()
	 * @generated
	 * @ordered
	 */
	protected EList<String>					arrayDimensions;
	
	/**
	 * The default value of the '{@link #getType() <em>Type</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getType()
	 * @generated
	 * @ordered
	 */
	protected static final String			TYPE_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getType() <em>Type</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getType()
	 * @generated
	 * @ordered
	 */
	protected String						type				= TYPE_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ParameterImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.PARAMETER;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.PARAMETER__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation getDst_ConstantAffectation() {
		if (dst_ConstantAffectation != null && dst_ConstantAffectation.eIsProxy()) {
			InternalEObject oldDst_ConstantAffectation = (InternalEObject) dst_ConstantAffectation;
			dst_ConstantAffectation = (ConstantAffectation) eResolveProxy(oldDst_ConstantAffectation);
			if (dst_ConstantAffectation != oldDst_ConstantAffectation) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE,
						SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation,
						dst_ConstantAffectation));
			}
		}
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation basicGetDst_ConstantAffectation() {
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation,
			NotificationChain msgs) {
		ConstantAffectation oldDst_ConstantAffectation = dst_ConstantAffectation;
		dst_ConstantAffectation = newDst_ConstantAffectation;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation, newDst_ConstantAffectation);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation) {
		if (newDst_ConstantAffectation != dst_ConstantAffectation) {
			NotificationChain msgs = null;
			if (dst_ConstantAffectation != null)
				msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			if (newDst_ConstantAffectation != null)
				msgs = ((InternalEObject) newDst_ConstantAffectation).eInverseAdd(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			msgs = basicSetDst_ConstantAffectation(newDst_ConstantAffectation, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION,
				newDst_ConstantAffectation, newDst_ConstantAffectation));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Condition> getSrc_Conditions() {
		if (src_Conditions == null) {
			src_Conditions = new EObjectWithInverseResolvingEList<Condition>(Condition.class, this,
				SmePackage.PARAMETER__SRC_CONDITIONS, SmePackage.CONDITION__SRC);
		}
		return src_Conditions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public InterfaceDefinition getInterfaceParent() {
		if (eContainerFeatureID != SmePackage.PARAMETER__INTERFACE_PARENT)
			return null;
		return (InterfaceDefinition) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetInterfaceParent(InterfaceDefinition newInterfaceParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newInterfaceParent, SmePackage.PARAMETER__INTERFACE_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setInterfaceParent(InterfaceDefinition newInterfaceParent) {
		if (newInterfaceParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.PARAMETER__INTERFACE_PARENT && newInterfaceParent != null)) {
			if (EcoreUtil.isAncestor(this, newInterfaceParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newInterfaceParent != null)
				msgs = ((InternalEObject) newInterfaceParent).eInverseAdd(this,
					SmePackage.INTERFACE_DEFINITION__PARAMETERS, InterfaceDefinition.class, msgs);
			msgs = basicSetInterfaceParent(newInterfaceParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER__INTERFACE_PARENT,
				newInterfaceParent, newInterfaceParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getArrayDimensions() {
		if (arrayDimensions == null) {
			arrayDimensions = new EDataTypeUniqueEList<String>(String.class, this,
				SmePackage.PARAMETER__ARRAY_DIMENSIONS);
		}
		return arrayDimensions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void setType(String newType) {
		String oldType = type;
		if (newType == "")
			type = TYPE_EDEFAULT;
		else
			type = newType;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER__TYPE, oldType, type));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getInterfaceParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.PARAMETER__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
			case SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION:
				if (dst_ConstantAffectation != null)
					msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
						SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
				return basicSetDst_ConstantAffectation((ConstantAffectation) otherEnd, msgs);
			case SmePackage.PARAMETER__SRC_CONDITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Conditions()).basicAdd(otherEnd,
					msgs);
			case SmePackage.PARAMETER__INTERFACE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetInterfaceParent((InterfaceDefinition) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.PARAMETER__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
			case SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION:
				return basicSetDst_ConstantAffectation(null, msgs);
			case SmePackage.PARAMETER__SRC_CONDITIONS:
				return ((InternalEList<?>) getSrc_Conditions()).basicRemove(otherEnd, msgs);
			case SmePackage.PARAMETER__INTERFACE_PARENT:
				return basicSetInterfaceParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.PARAMETER__INTERFACE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.INTERFACE_DEFINITION__PARAMETERS,
					InterfaceDefinition.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.PARAMETER__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.PARAMETER__NAME:
				return getName();
			case SmePackage.PARAMETER__COMMENT:
				return getComment();
			case SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION:
				if (resolve)
					return getDst_ConstantAffectation();
				return basicGetDst_ConstantAffectation();
			case SmePackage.PARAMETER__SRC_CONDITIONS:
				return getSrc_Conditions();
			case SmePackage.PARAMETER__INTERFACE_PARENT:
				return getInterfaceParent();
			case SmePackage.PARAMETER__ARRAY_DIMENSIONS:
				return getArrayDimensions();
			case SmePackage.PARAMETER__TYPE:
				return getType();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.PARAMETER__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.PARAMETER__NAME:
				setName((String) newValue);
				return;
			case SmePackage.PARAMETER__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) newValue);
				return;
			case SmePackage.PARAMETER__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				getSrc_Conditions().addAll((Collection<? extends Condition>) newValue);
				return;
			case SmePackage.PARAMETER__INTERFACE_PARENT:
				setInterfaceParent((InterfaceDefinition) newValue);
				return;
			case SmePackage.PARAMETER__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				getArrayDimensions().addAll((Collection<? extends String>) newValue);
				return;
			case SmePackage.PARAMETER__TYPE:
				setType((String) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.PARAMETER__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.PARAMETER__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.PARAMETER__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) null);
				return;
			case SmePackage.PARAMETER__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				return;
			case SmePackage.PARAMETER__INTERFACE_PARENT:
				setInterfaceParent((InterfaceDefinition) null);
				return;
			case SmePackage.PARAMETER__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				return;
			case SmePackage.PARAMETER__TYPE:
				setType(TYPE_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.PARAMETER__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.PARAMETER__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.PARAMETER__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.PARAMETER__DST_CONSTANT_AFFECTATION:
				return dst_ConstantAffectation != null;
			case SmePackage.PARAMETER__SRC_CONDITIONS:
				return src_Conditions != null && !src_Conditions.isEmpty();
			case SmePackage.PARAMETER__INTERFACE_PARENT:
				return getInterfaceParent() != null;
			case SmePackage.PARAMETER__ARRAY_DIMENSIONS:
				return arrayDimensions != null && !arrayDimensions.isEmpty();
			case SmePackage.PARAMETER__TYPE:
				return TYPE_EDEFAULT == null ? type != null : !TYPE_EDEFAULT.equals(type);
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.PARAMETER__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.PARAMETER__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.PARAMETER__SRC_CONDITIONS:
					return SmePackage.CONDITION_EXPR__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.PARAMETER__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.PARAMETER__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.PARAMETER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONDITION_EXPR__SRC_CONDITIONS:
					return SmePackage.PARAMETER__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(", arrayDimensions: ");
		result.append(arrayDimensions);
		result.append(", type: ");
		result.append(type);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (type != null) {
				result.append(type);
				if (getArrayDimensions().size() > 0) {
					result.append("[");
					Iterator<String> it = arrayDimensions.iterator();
					result.append(it.next());
					while (it.hasNext()) {
						result.append(", ");
						result.append(it.next());
					}
					result.append("]");
				}
			}
			result.append(" " + name);
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * No equation are built from a Parameter, because it is define outside the model in which it is declared . So it is
	 * only defined for a ModelInstance.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConstantImpl#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			
			long typeNode;
			if (TYPE_EDEFAULT == type || type.equals(SignalIntrinsicType.UNDEFINED_LITERAL.getLiteral())) {
				// Implicit type ==> we do not use arrayDimensions
				typeNode = API.makeNullary(SignalOperators.opimplicit);
				PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.PARAMETER_NO_TYPE + name, this);
			}
			else if (type.equals("type")) // Return directly the type
				return API.post(declTree, API.makeUnary(SignalOperators.opltype, API.makeList(SignalOperators.oplsign,
					idNode)));
			else {
				if (type.equals(SignalIntrinsicType.BOOLEAN_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.oplogical);
				else if (type.equals(SignalIntrinsicType.CHAR_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opchar);
				else if (type.equals(SignalIntrinsicType.COMPLEX_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opcomplex);
				else if (type.equals(SignalIntrinsicType.DCOMPLEX_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opdcomplex);
				else if (type.equals(SignalIntrinsicType.DREAL_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opdpreal);
				else if (type.equals(SignalIntrinsicType.EVENT_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opevent);
				else if (type.equals(SignalIntrinsicType.INTEGER_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opinteger);
				else if (type.equals(SignalIntrinsicType.LONG_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.oplong);
				else if (type.equals(SignalIntrinsicType.REAL_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opreal);
				else if (type.equals(SignalIntrinsicType.SHORT_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opshort);
				else if (type.equals(SignalIntrinsicType.STRING_LITERAL.getLiteral()))
					typeNode = API.makeNullary(SignalOperators.opstring);
				else {
					if (type.startsWith("process ")) {
						typeNode = PkPlugin.getParser().parseString(Phyla.ph_ident, type.substring(8));
						if (typeNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.PARAMETER_TYPE, this);
							typeNode = API.makeNullary(SignalOperators.opimplicit);
						}
						else {
							typeNode = API.makeBinary(SignalOperators.opformproc, typeNode, idNode);
							return API.post(declTree, typeNode);
						}
					}
					else if (type.startsWith("function ")) {
						typeNode = PkPlugin.getParser().parseString(Phyla.ph_ident, type.substring(9));
						if (typeNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.PARAMETER_TYPE, this);
							typeNode = API.makeNullary(SignalOperators.opimplicit);
						}
						else {
							typeNode = API.makeBinary(SignalOperators.opformfunc, typeNode, idNode);
							return API.post(declTree, typeNode);
						}
					}
					else if (type.startsWith("action ")) {
						typeNode = PkPlugin.getParser().parseString(Phyla.ph_ident, type.substring(7));
						if (typeNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.PARAMETER_TYPE, this);
							typeNode = API.makeNullary(SignalOperators.opimplicit);
						}
						else {
							typeNode = API.makeBinary(SignalOperators.opformact, typeNode, idNode);
							return API.post(declTree, typeNode);
						}
					}
					else if (type.startsWith("node ")) {
						typeNode = PkPlugin.getParser().parseString(Phyla.ph_ident, type.substring(5));
						if (typeNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.PARAMETER_TYPE, this);
							typeNode = API.makeNullary(SignalOperators.opimplicit);
						}
						else {
							typeNode = API.makeBinary(SignalOperators.opformnode, typeNode, idNode);
							return API.post(declTree, typeNode);
						}
					}
					else {
						typeNode = PkPlugin.getParser().parseString(Phyla.ph_ident, type);
						if (typeNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.PARAMETER_TYPE, this);
							typeNode = API.makeNullary(SignalOperators.opimplicit);
						}
					}
				}
				
				// Add the specification of arrayDimensions. Parse the textual expression
				if (getArrayDimensions().size() > 0) {
					long arrayNode = API.makeEmptyList(SignalOperators.oplesign);
					boolean allOK = true;
					for (String st : arrayDimensions) {
						long exprNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
						if (exprNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.ARRAY_DIMENSION_VALUE + st, this);
							allOK = false;
						}
						else
							arrayNode = API.post(arrayNode, exprNode);
					}
					if (allOK)
						typeNode = API.makeBinary(SignalOperators.optvect, arrayNode, typeNode);
				}
			}
			declTree = API.post(declTree, API.makeBinary(SignalOperators.opdecsig, typeNode, API.makeList(
				SignalOperators.oplsign, idNode)));
		}
		return declTree;
	}
	
} // ParameterImpl
