/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionExpr;
import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.ParameterInstance;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Parameter Instance</b></em>'. <!-- end-user-doc
 * -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterInstanceImpl#getSrc_DataflowConnections <em>Src Dataflow Connections
 * </em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterInstanceImpl#getSrc_ConstantAffectations <em>Src Constant Affectations
 * </em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterInstanceImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterInstanceImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterInstanceImpl#getDst_ConstantAffectation <em>Dst Constant Affectation
 * </em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterInstanceImpl#getParameter <em>Parameter</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.ParameterInstanceImpl#getInstanceParent <em>Instance Parent</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class ParameterInstanceImpl extends EObjectImpl implements ParameterInstance {
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>		src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String			NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String						name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String			COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String						comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getDst_ConstantAffectation() <em>Dst Constant Affectation</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ConstantAffectation()
	 * @generated
	 * @ordered
	 */
	protected ConstantAffectation			dst_ConstantAffectation;
	
	/**
	 * The cached value of the '{@link #getSrc_Conditions() <em>Src Conditions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Conditions()
	 * @generated
	 * @ordered
	 */
	protected EList<Condition>				src_Conditions;
	
	/**
	 * The cached value of the '{@link #getParameter() <em>Parameter</em>}' reference. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getParameter()
	 * @generated
	 * @ordered
	 */
	protected Parameter						parameter;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected ParameterInstanceImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.PARAMETER_INSTANCE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.PARAMETER_INSTANCE__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER_INSTANCE__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER_INSTANCE__COMMENT, oldComment,
				comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation getDst_ConstantAffectation() {
		if (dst_ConstantAffectation != null && dst_ConstantAffectation.eIsProxy()) {
			InternalEObject oldDst_ConstantAffectation = (InternalEObject) dst_ConstantAffectation;
			dst_ConstantAffectation = (ConstantAffectation) eResolveProxy(oldDst_ConstantAffectation);
			if (dst_ConstantAffectation != oldDst_ConstantAffectation) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE,
						SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation,
						dst_ConstantAffectation));
			}
		}
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation basicGetDst_ConstantAffectation() {
		return dst_ConstantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation,
			NotificationChain msgs) {
		ConstantAffectation oldDst_ConstantAffectation = dst_ConstantAffectation;
		dst_ConstantAffectation = newDst_ConstantAffectation;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION, oldDst_ConstantAffectation,
				newDst_ConstantAffectation);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst_ConstantAffectation(ConstantAffectation newDst_ConstantAffectation) {
		if (newDst_ConstantAffectation != dst_ConstantAffectation) {
			NotificationChain msgs = null;
			if (dst_ConstantAffectation != null)
				msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			if (newDst_ConstantAffectation != null)
				msgs = ((InternalEObject) newDst_ConstantAffectation).eInverseAdd(this,
					SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
			msgs = basicSetDst_ConstantAffectation(newDst_ConstantAffectation, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET,
				SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION, newDst_ConstantAffectation,
				newDst_ConstantAffectation));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Condition> getSrc_Conditions() {
		if (src_Conditions == null) {
			src_Conditions = new EObjectWithInverseResolvingEList<Condition>(Condition.class, this,
				SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS, SmePackage.CONDITION__SRC);
		}
		return src_Conditions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Parameter getParameter() {
		if (parameter != null && parameter.eIsProxy()) {
			InternalEObject oldParameter = (InternalEObject) parameter;
			parameter = (Parameter) eResolveProxy(oldParameter);
			if (parameter != oldParameter) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.PARAMETER_INSTANCE__PARAMETER,
						oldParameter, parameter));
			}
		}
		return parameter;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Parameter basicGetParameter() {
		return parameter;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setParameter(Parameter newParameter) {
		Parameter oldParameter = parameter;
		parameter = newParameter;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER_INSTANCE__PARAMETER,
				oldParameter, parameter));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelInstance getInstanceParent() {
		if (eContainerFeatureID != SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT)
			return null;
		return (ModelInstance) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetInstanceParent(ModelInstance newInstanceParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newInstanceParent, SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT,
			msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setInstanceParent(ModelInstance newInstanceParent) {
		if (newInstanceParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT && newInstanceParent != null)) {
			if (EcoreUtil.isAncestor(this, newInstanceParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newInstanceParent != null)
				msgs = ((InternalEObject) newInstanceParent).eInverseAdd(this, SmePackage.MODEL_INSTANCE__PARAMETERS,
					ModelInstance.class, msgs);
			msgs = basicSetInstanceParent(newInstanceParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT,
				newInstanceParent, newInstanceParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getInstanceParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.PARAMETER_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
			case SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION:
				if (dst_ConstantAffectation != null)
					msgs = ((InternalEObject) dst_ConstantAffectation).eInverseRemove(this,
						SmePackage.CONSTANT_AFFECTATION__DST, ConstantAffectation.class, msgs);
				return basicSetDst_ConstantAffectation((ConstantAffectation) otherEnd, msgs);
			case SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Conditions()).basicAdd(otherEnd,
					msgs);
			case SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetInstanceParent((ModelInstance) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.PARAMETER_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
			case SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION:
				return basicSetDst_ConstantAffectation(null, msgs);
			case SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS:
				return ((InternalEList<?>) getSrc_Conditions()).basicRemove(otherEnd, msgs);
			case SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT:
				return basicSetInstanceParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL_INSTANCE__PARAMETERS,
					ModelInstance.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.PARAMETER_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.PARAMETER_INSTANCE__NAME:
				return getName();
			case SmePackage.PARAMETER_INSTANCE__COMMENT:
				return getComment();
			case SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION:
				if (resolve)
					return getDst_ConstantAffectation();
				return basicGetDst_ConstantAffectation();
			case SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS:
				return getSrc_Conditions();
			case SmePackage.PARAMETER_INSTANCE__PARAMETER:
				if (resolve)
					return getParameter();
				return basicGetParameter();
			case SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT:
				return getInstanceParent();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.PARAMETER_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.PARAMETER_INSTANCE__NAME:
				setName((String) newValue);
				return;
			case SmePackage.PARAMETER_INSTANCE__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) newValue);
				return;
			case SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				getSrc_Conditions().addAll((Collection<? extends Condition>) newValue);
				return;
			case SmePackage.PARAMETER_INSTANCE__PARAMETER:
				setParameter((Parameter) newValue);
				return;
			case SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT:
				setInstanceParent((ModelInstance) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.PARAMETER_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.PARAMETER_INSTANCE__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.PARAMETER_INSTANCE__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION:
				setDst_ConstantAffectation((ConstantAffectation) null);
				return;
			case SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				return;
			case SmePackage.PARAMETER_INSTANCE__PARAMETER:
				setParameter((Parameter) null);
				return;
			case SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT:
				setInstanceParent((ModelInstance) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.PARAMETER_INSTANCE__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.PARAMETER_INSTANCE__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.PARAMETER_INSTANCE__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.PARAMETER_INSTANCE__DST_CONSTANT_AFFECTATION:
				return dst_ConstantAffectation != null;
			case SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS:
				return src_Conditions != null && !src_Conditions.isEmpty();
			case SmePackage.PARAMETER_INSTANCE__PARAMETER:
				return parameter != null;
			case SmePackage.PARAMETER_INSTANCE__INSTANCE_PARENT:
				return getInstanceParent() != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.PARAMETER_INSTANCE__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.PARAMETER_INSTANCE__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS:
					return SmePackage.CONDITION_EXPR__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.PARAMETER_INSTANCE__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.PARAMETER_INSTANCE__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.PARAMETER_INSTANCE__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONDITION_EXPR__SRC_CONDITIONS:
					return SmePackage.PARAMETER_INSTANCE__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (getParameter() != null)
				result.append(parameter.toString(asCode, true));
			else
				result.append("refers(null)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			long typeNode = API.makeNullary(SignalOperators.opimplicit);
			
			if (getDst_ConstantAffectation() != null) {
				try {
					long valueNode = dst_ConstantAffectation.getAST(new HashSet<SignalElement>());
					if (valueNode != PKTrees.ERROR_NODE)
						idNode = API.makeBinary(SignalOperators.opcons, idNode, valueNode);
				}
				catch (CyclicDefinitionException cde) {
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
							.getElement());
				}
			}
			else {
				PolychronyDiagnostic
						.add(Diagnostic.ERROR, ErrorMessages.PARAMETER_INSTANCE_INCORRECT_LINK + name, this);
			}
			
			declTree = API.post(declTree, API.makeBinary(SignalOperators.opdeccon, typeNode, API.makeList(
				SignalOperators.oplsign, idNode)));
		}
		return declTree;
	}
	
} // ParameterInstanceImpl
