/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ClockedExpr;
import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConditionExpr;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.DataflowConnector;
import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.DependenceInOut;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.Signal;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Signal Ref</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getSrc_ClockRelations <em>Src Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getDst_ClockRelations <em>Dst Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getSrc_Dependences <em>Src Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getDst_Dependences <em>Dst Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getDst_DataflowConnections <em>Dst Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getSrc_Conditions <em>Src Conditions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getSignal <em>Signal</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getSigName <em>Sig Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SignalRefImpl#getModelParent <em>Model Parent</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class SignalRefImpl extends EObjectImpl implements SignalRef {
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>	src_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_ClockRelations() <em>Src Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		src_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getDst_ClockRelations() <em>Dst Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>		dst_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getSrc_Dependences() <em>Src Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>			src_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_Dependences() <em>Dst Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>			dst_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_DataflowConnections() <em>Dst Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>	dst_DataflowConnections;
	
	/**
	 * The cached value of the '{@link #getSrc_Conditions() <em>Src Conditions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Conditions()
	 * @generated
	 * @ordered
	 */
	protected EList<Condition>			src_Conditions;
	
	/**
	 * The cached value of the '{@link #getSignal() <em>Signal</em>}' reference. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSignal()
	 * @generated
	 * @ordered
	 */
	protected Signal					signal;
	
	/**
	 * The default value of the '{@link #getSigName() <em>Sig Name</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSigName()
	 * @generated
	 * @ordered
	 */
	protected static final String		SIG_NAME_EDEFAULT	= null;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SignalRefImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.SIGNAL_REF;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.SIGNAL_REF__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getSrc_ClockRelations() {
		if (src_ClockRelations == null) {
			src_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__SRC);
		}
		return src_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getDst_ClockRelations() {
		if (dst_ClockRelations == null) {
			dst_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__DST);
		}
		return dst_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getSrc_Dependences() {
		if (src_Dependences == null) {
			src_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.SIGNAL_REF__SRC_DEPENDENCES, SmePackage.DEPENDENCE__SRC);
		}
		return src_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getDst_Dependences() {
		if (dst_Dependences == null) {
			dst_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.SIGNAL_REF__DST_DEPENDENCES, SmePackage.DEPENDENCE__DST);
		}
		return dst_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getDst_DataflowConnections() {
		if (dst_DataflowConnections == null) {
			dst_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__DST);
		}
		return dst_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Condition> getSrc_Conditions() {
		if (src_Conditions == null) {
			src_Conditions = new EObjectWithInverseResolvingEList<Condition>(Condition.class, this,
				SmePackage.SIGNAL_REF__SRC_CONDITIONS, SmePackage.CONDITION__SRC);
		}
		return src_Conditions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Signal getSignal() {
		if (signal != null && signal.eIsProxy()) {
			InternalEObject oldSignal = (InternalEObject) signal;
			signal = (Signal) eResolveProxy(oldSignal);
			if (signal != oldSignal) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.SIGNAL_REF__SIGNAL, oldSignal,
						signal));
			}
		}
		return signal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Signal basicGetSignal() {
		return signal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSignal(Signal newSignal) {
		Signal oldSignal = signal;
		signal = newSignal;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SIGNAL_REF__SIGNAL, oldSignal, signal));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public String getSigName() {
		if (signal != null)
			return signal.getName();
		return SIG_NAME_EDEFAULT;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void setSigName(String newSigName) {}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.SIGNAL_REF__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.SIGNAL_REF__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.SIGNAL_REF__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__SIGNAL_REF, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SIGNAL_REF__MODEL_PARENT, newModelParent,
				newModelParent));
	}
	
	/**
	 * This method returns true if the current signal uses only partial definition
	 * 
	 * @generated NOT
	 */
	public boolean isPartiallyDefined() {
		// Only local signal with status equal to shared or statevar can be partially defined
		if (signal == null)
			return false;
		else
			return signal.isPartiallyDefined();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getModelParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.SIGNAL_REF__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SIGNAL_REF__SRC_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SIGNAL_REF__DST_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.SIGNAL_REF__SRC_CONDITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Conditions()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SIGNAL_REF__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.SIGNAL_REF__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS:
				return ((InternalEList<?>) getSrc_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS:
				return ((InternalEList<?>) getDst_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.SIGNAL_REF__SRC_DEPENDENCES:
				return ((InternalEList<?>) getSrc_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.SIGNAL_REF__DST_DEPENDENCES:
				return ((InternalEList<?>) getDst_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getDst_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.SIGNAL_REF__SRC_CONDITIONS:
				return ((InternalEList<?>) getSrc_Conditions()).basicRemove(otherEnd, msgs);
			case SmePackage.SIGNAL_REF__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.SIGNAL_REF__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__SIGNAL_REF, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.SIGNAL_REF__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS:
				return getSrc_ClockRelations();
			case SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS:
				return getDst_ClockRelations();
			case SmePackage.SIGNAL_REF__SRC_DEPENDENCES:
				return getSrc_Dependences();
			case SmePackage.SIGNAL_REF__DST_DEPENDENCES:
				return getDst_Dependences();
			case SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS:
				return getDst_DataflowConnections();
			case SmePackage.SIGNAL_REF__SRC_CONDITIONS:
				return getSrc_Conditions();
			case SmePackage.SIGNAL_REF__SIGNAL:
				if (resolve)
					return getSignal();
				return basicGetSignal();
			case SmePackage.SIGNAL_REF__SIG_NAME:
				return getSigName();
			case SmePackage.SIGNAL_REF__MODEL_PARENT:
				return getModelParent();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.SIGNAL_REF__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				getSrc_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				getDst_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.SIGNAL_REF__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				getSrc_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.SIGNAL_REF__DST_DEPENDENCES:
				getDst_Dependences().clear();
				getDst_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS:
				getDst_DataflowConnections().clear();
				getDst_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.SIGNAL_REF__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				getSrc_Conditions().addAll((Collection<? extends Condition>) newValue);
				return;
			case SmePackage.SIGNAL_REF__SIGNAL:
				setSignal((Signal) newValue);
				return;
			case SmePackage.SIGNAL_REF__SIG_NAME:
				setSigName((String) newValue);
				return;
			case SmePackage.SIGNAL_REF__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.SIGNAL_REF__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				return;
			case SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				return;
			case SmePackage.SIGNAL_REF__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				return;
			case SmePackage.SIGNAL_REF__DST_DEPENDENCES:
				getDst_Dependences().clear();
				return;
			case SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS:
				getDst_DataflowConnections().clear();
				return;
			case SmePackage.SIGNAL_REF__SRC_CONDITIONS:
				getSrc_Conditions().clear();
				return;
			case SmePackage.SIGNAL_REF__SIGNAL:
				setSignal((Signal) null);
				return;
			case SmePackage.SIGNAL_REF__SIG_NAME:
				setSigName(SIG_NAME_EDEFAULT);
				return;
			case SmePackage.SIGNAL_REF__MODEL_PARENT:
				setModelParent((Model) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.SIGNAL_REF__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS:
				return src_ClockRelations != null && !src_ClockRelations.isEmpty();
			case SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS:
				return dst_ClockRelations != null && !dst_ClockRelations.isEmpty();
			case SmePackage.SIGNAL_REF__SRC_DEPENDENCES:
				return src_Dependences != null && !src_Dependences.isEmpty();
			case SmePackage.SIGNAL_REF__DST_DEPENDENCES:
				return dst_Dependences != null && !dst_Dependences.isEmpty();
			case SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS:
				return dst_DataflowConnections != null && !dst_DataflowConnections.isEmpty();
			case SmePackage.SIGNAL_REF__SRC_CONDITIONS:
				return src_Conditions != null && !src_Conditions.isEmpty();
			case SmePackage.SIGNAL_REF__SIGNAL:
				return signal != null;
			case SmePackage.SIGNAL_REF__SIG_NAME:
				return SIG_NAME_EDEFAULT == null ? getSigName() != null : !SIG_NAME_EDEFAULT.equals(getSigName());
			case SmePackage.SIGNAL_REF__MODEL_PARENT:
				return getModelParent() != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ClockedExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS:
					return SmePackage.CLOCKED_EXPR__SRC_CLOCK_RELATIONS;
				case SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS:
					return SmePackage.CLOCKED_EXPR__DST_CLOCK_RELATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (derivedFeatureID) {
				case SmePackage.SIGNAL_REF__SRC_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES;
				case SmePackage.SIGNAL_REF__DST_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowConnector.class) {
			switch (derivedFeatureID) {
				case SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS:
					return SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.SIGNAL_REF__SRC_CONDITIONS:
					return SmePackage.CONDITION_EXPR__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ClockedExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CLOCKED_EXPR__SRC_CLOCK_RELATIONS:
					return SmePackage.SIGNAL_REF__SRC_CLOCK_RELATIONS;
				case SmePackage.CLOCKED_EXPR__DST_CLOCK_RELATIONS:
					return SmePackage.SIGNAL_REF__DST_CLOCK_RELATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (baseFeatureID) {
				case SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES:
					return SmePackage.SIGNAL_REF__SRC_DEPENDENCES;
				case SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES:
					return SmePackage.SIGNAL_REF__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowConnector.class) {
			switch (baseFeatureID) {
				case SmePackage.DATAFLOW_CONNECTOR__DST_DATAFLOW_CONNECTIONS:
					return SmePackage.SIGNAL_REF__DST_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == ConditionExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONDITION_EXPR__SRC_CONDITIONS:
					return SmePackage.SIGNAL_REF__SRC_CONDITIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * @see fr.irisa.espresso.sme.ConstantRef#toString(boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (getSignal() != null)
				result.append(signal.toString(asCode, true));
			else
				result.append("refers(null)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		for (DataflowConnection dc : getDst_DataflowConnections())
			bodyTree = dc.buildEquationsAST(bodyTree);
		for (ClockRelation cr : getDst_ClockRelations())
			bodyTree = cr.buildEquationsAST(bodyTree);
		for (Dependence dep : getDst_Dependences())
			bodyTree = dep.buildEquationsAST(bodyTree);
		if (dst_DataflowConnections.isEmpty() && dst_ClockRelations.isEmpty() && dst_Dependences.isEmpty()
				&& getSrc_ClockRelations().isEmpty() && getSrc_Conditions().isEmpty()
				&& getSrc_DataflowConnections().isEmpty() && getSrc_Dependences().isEmpty())
			PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.SIGNAL_REF_NOT_NEEDED + getSigName(), this);
		return bodyTree;
	}
	
	/**
	 * Returns the abstract syntax tree of the SignalRef name, thus the name of the referenced signal.
	 * 
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) throws CyclicDefinitionException {
		long result = PKTrees.ERROR_NODE;
		if (signal == null) {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.SIGNAL_REF_INCORRECT_REFERENCE, this);
			result = PKTrees.ERROR_NODE;
		}
		else {
			result = signal.getAST(path);
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * No modification on the declaration list.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
	
} // SignalRefImpl
