/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Set;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.impl.EFactoryImpl;
import org.eclipse.emf.ecore.plugin.EcorePlugin;

import fr.irisa.espresso.sme.And;
import fr.irisa.espresso.sme.AndState;
import fr.irisa.espresso.sme.ArithmeticOpKind;
import fr.irisa.espresso.sme.ArithmeticOperator;
import fr.irisa.espresso.sme.ArrayEnumeration;
import fr.irisa.espresso.sme.ArrayProduct;
import fr.irisa.espresso.sme.ArrayRecovery;
import fr.irisa.espresso.sme.ArrayRestructuration;
import fr.irisa.espresso.sme.Assertion;
import fr.irisa.espresso.sme.AssertionClockConstraintOperator;
import fr.irisa.espresso.sme.AssertionClockSpeedOperator;
import fr.irisa.espresso.sme.Automaton;
import fr.irisa.espresso.sme.BasicIterate;
import fr.irisa.espresso.sme.BooleanExpression;
import fr.irisa.espresso.sme.CartesianProduct;
import fr.irisa.espresso.sme.CaseConnection;
import fr.irisa.espresso.sme.CaseKind;
import fr.irisa.espresso.sme.Cell;
import fr.irisa.espresso.sme.ClockConstraintOpKind;
import fr.irisa.espresso.sme.ClockConstraintOperator;
import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ClockRelationOpKind;
import fr.irisa.espresso.sme.ClockRelationOperator;
import fr.irisa.espresso.sme.ClockSpeedOpKind;
import fr.irisa.espresso.sme.ClockSpeedOperator;
import fr.irisa.espresso.sme.ComparisonOpKind;
import fr.irisa.espresso.sme.ComparisonOperator;
import fr.irisa.espresso.sme.CompilationDirectiveKind;
import fr.irisa.espresso.sme.ComplexOperator;
import fr.irisa.espresso.sme.Concatenation;
import fr.irisa.espresso.sme.Condition;
import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantRef;
import fr.irisa.espresso.sme.ConstantValue;
import fr.irisa.espresso.sme.Counter;
import fr.irisa.espresso.sme.CounterKind;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.Delay;
import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.DependenceOperator;
import fr.irisa.espresso.sme.EnumerationType;
import fr.irisa.espresso.sme.EnumerationValue;
import fr.irisa.espresso.sme.Extraction;
import fr.irisa.espresso.sme.History;
import fr.irisa.espresso.sme.IfThenElse;
import fr.irisa.espresso.sme.Index;
import fr.irisa.espresso.sme.Input;
import fr.irisa.espresso.sme.InputInstance;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.Iterate;
import fr.irisa.espresso.sme.IterationInit;
import fr.irisa.espresso.sme.Iterator;
import fr.irisa.espresso.sme.Literal;
import fr.irisa.espresso.sme.Local;
import fr.irisa.espresso.sme.Merge;
import fr.irisa.espresso.sme.ModelDeclaration;
import fr.irisa.espresso.sme.ModelInstance;
import fr.irisa.espresso.sme.ModelKind;
import fr.irisa.espresso.sme.ModelRef;
import fr.irisa.espresso.sme.ModelType;
import fr.irisa.espresso.sme.Module;
import fr.irisa.espresso.sme.Not;
import fr.irisa.espresso.sme.NullClock;
import fr.irisa.espresso.sme.NumericExpression;
import fr.irisa.espresso.sme.Or;
import fr.irisa.espresso.sme.Output;
import fr.irisa.espresso.sme.OutputInstance;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.Parameter;
import fr.irisa.espresso.sme.ParameterInstance;
import fr.irisa.espresso.sme.Pragma;
import fr.irisa.espresso.sme.ProcessAttributeKind;
import fr.irisa.espresso.sme.RepetitionOperator;
import fr.irisa.espresso.sme.SequentialDefinition;
import fr.irisa.espresso.sme.SignalIntrinsicType;
import fr.irisa.espresso.sme.SignalRef;
import fr.irisa.espresso.sme.SignalStateKind;
import fr.irisa.espresso.sme.SmeFactory;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.Specifications;
import fr.irisa.espresso.sme.State;
import fr.irisa.espresso.sme.StatusKind;
import fr.irisa.espresso.sme.StrongTransition;
import fr.irisa.espresso.sme.SubProcess;
import fr.irisa.espresso.sme.SubstitutionType;
import fr.irisa.espresso.sme.Switch;
import fr.irisa.espresso.sme.Transposition;
import fr.irisa.espresso.sme.TupleOperator;
import fr.irisa.espresso.sme.TupleType;
import fr.irisa.espresso.sme.TypeKind;
import fr.irisa.espresso.sme.UnaryMinus;
import fr.irisa.espresso.sme.UseModule;
import fr.irisa.espresso.sme.Variable;
import fr.irisa.espresso.sme.VisibilityKind;
import fr.irisa.espresso.sme.WeakTransition;
import fr.irisa.espresso.sme.Window;
import fr.irisa.espresso.sme.Xor;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;

/**
 * <!-- begin-user-doc --> An implementation of the model <b>Factory</b>. <!-- end-user-doc -->
 * 
 * @generated
 */
public class SmeFactoryImpl extends EFactoryImpl implements SmeFactory {
	/**
	 * Creates the default factory implementation. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static SmeFactory init() {
		try {
			SmeFactory theSmeFactory = (SmeFactory) EPackage.Registry.INSTANCE
					.getEFactory("http://www.irisa.fr/espresso/sme.ecore");
			if (theSmeFactory != null) {
				return theSmeFactory;
			}
		}
		catch (Exception exception) {
			EcorePlugin.INSTANCE.log(exception);
		}
		return new SmeFactoryImpl();
	}
	
	/**
	 * Creates an instance of the factory. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SmeFactoryImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public EObject create(EClass eClass) {
		switch (eClass.getClassifierID()) {
			case SmePackage.BASIC_ITERATE:
				return createBasicIterate();
			case SmePackage.ITERATE:
				return createIterate();
			case SmePackage.ITERATION_INIT:
				return createIterationInit();
			case SmePackage.SPECIFICATIONS:
				return createSpecifications();
			case SmePackage.SUB_PROCESS:
				return createSubProcess();
			case SmePackage.ENUMERATION_TYPE:
				return createEnumerationType();
			case SmePackage.MODEL_TYPE:
				return createModelType();
			case SmePackage.SUBSTITUTION_TYPE:
				return createSubstitutionType();
			case SmePackage.TUPLE_TYPE:
				return createTupleType();
			case SmePackage.MODEL_DECLARATION:
				return createModelDeclaration();
			case SmePackage.MODEL_REF:
				return createModelRef();
			case SmePackage.MODEL_INSTANCE:
				return createModelInstance();
			case SmePackage.MODULE:
				return createModule();
			case SmePackage.USE_MODULE:
				return createUseModule();
			case SmePackage.CONSTANT_REF:
				return createConstantRef();
			case SmePackage.CONSTANT_VALUE:
				return createConstantValue();
			case SmePackage.LITERAL:
				return createLiteral();
			case SmePackage.ENUMERATION_VALUE:
				return createEnumerationValue();
			case SmePackage.PARAMETER:
				return createParameter();
			case SmePackage.PARAMETER_INSTANCE:
				return createParameterInstance();
			case SmePackage.ITERATOR:
				return createIterator();
			case SmePackage.INDEX:
				return createIndex();
			case SmePackage.SIGNAL_REF:
				return createSignalRef();
			case SmePackage.INPUT:
				return createInput();
			case SmePackage.INPUT_INSTANCE:
				return createInputInstance();
			case SmePackage.OUTPUT:
				return createOutput();
			case SmePackage.OUTPUT_INSTANCE:
				return createOutputInstance();
			case SmePackage.LOCAL:
				return createLocal();
			case SmePackage.CASE_CONNECTION:
				return createCaseConnection();
			case SmePackage.CLOCK_RELATION:
				return createClockRelation();
			case SmePackage.CONDITION:
				return createCondition();
			case SmePackage.CONSTANT_AFFECTATION:
				return createConstantAffectation();
			case SmePackage.DATAFLOW_CONNECTION:
				return createDataflowConnection();
			case SmePackage.DEPENDENCE:
				return createDependence();
			case SmePackage.INPUT_PORT:
				return createInputPort();
			case SmePackage.OUTPUT_PORT:
				return createOutputPort();
			case SmePackage.AND:
				return createAnd();
			case SmePackage.ARITHMETIC_OPERATOR:
				return createArithmeticOperator();
			case SmePackage.ARRAY_ENUMERATION:
				return createArrayEnumeration();
			case SmePackage.ARRAY_PRODUCT:
				return createArrayProduct();
			case SmePackage.ARRAY_RECOVERY:
				return createArrayRecovery();
			case SmePackage.ARRAY_RESTRUCTURATION:
				return createArrayRestructuration();
			case SmePackage.ASSERTION:
				return createAssertion();
			case SmePackage.ASSERTION_CLOCK_CONSTRAINT_OPERATOR:
				return createAssertionClockConstraintOperator();
			case SmePackage.ASSERTION_CLOCK_SPEED_OPERATOR:
				return createAssertionClockSpeedOperator();
			case SmePackage.BOOLEAN_EXPRESSION:
				return createBooleanExpression();
			case SmePackage.CARTESIAN_PRODUCT:
				return createCartesianProduct();
			case SmePackage.CELL:
				return createCell();
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR:
				return createClockConstraintOperator();
			case SmePackage.CLOCK_RELATION_OPERATOR:
				return createClockRelationOperator();
			case SmePackage.CLOCK_SPEED_OPERATOR:
				return createClockSpeedOperator();
			case SmePackage.COMPARISON_OPERATOR:
				return createComparisonOperator();
			case SmePackage.COMPLEX_OPERATOR:
				return createComplexOperator();
			case SmePackage.CONCATENATION:
				return createConcatenation();
			case SmePackage.COUNTER:
				return createCounter();
			case SmePackage.DELAY:
				return createDelay();
			case SmePackage.DEPENDENCE_OPERATOR:
				return createDependenceOperator();
			case SmePackage.EXTRACTION:
				return createExtraction();
			case SmePackage.IF_THEN_ELSE:
				return createIfThenElse();
			case SmePackage.MERGE:
				return createMerge();
			case SmePackage.NOT:
				return createNot();
			case SmePackage.NULL_CLOCK:
				return createNullClock();
			case SmePackage.NUMERIC_EXPRESSION:
				return createNumericExpression();
			case SmePackage.OR:
				return createOr();
			case SmePackage.PRAGMA:
				return createPragma();
			case SmePackage.REPETITION_OPERATOR:
				return createRepetitionOperator();
			case SmePackage.SEQUENTIAL_DEFINITION:
				return createSequentialDefinition();
			case SmePackage.SWITCH:
				return createSwitch();
			case SmePackage.TRANSPOSITION:
				return createTransposition();
			case SmePackage.TUPLE_OPERATOR:
				return createTupleOperator();
			case SmePackage.UNARY_MINUS:
				return createUnaryMinus();
			case SmePackage.VARIABLE:
				return createVariable();
			case SmePackage.WINDOW:
				return createWindow();
			case SmePackage.XOR:
				return createXor();
			case SmePackage.HISTORY:
				return createHistory();
			case SmePackage.STATE:
				return createState();
			case SmePackage.AUTOMATON:
				return createAutomaton();
			case SmePackage.AND_STATE:
				return createAndState();
			case SmePackage.WEAK_TRANSITION:
				return createWeakTransition();
			case SmePackage.STRONG_TRANSITION:
				return createStrongTransition();
			default:
				throw new IllegalArgumentException("The class '" + eClass.getName() + "' is not a valid classifier");
		}
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object createFromString(EDataType eDataType, String initialValue) {
		switch (eDataType.getClassifierID()) {
			case SmePackage.SIGNAL_INTRINSIC_TYPE:
				return createSignalIntrinsicTypeFromString(eDataType, initialValue);
			case SmePackage.MODEL_KIND:
				return createModelKindFromString(eDataType, initialValue);
			case SmePackage.PROCESS_ATTRIBUTE_KIND:
				return createProcessAttributeKindFromString(eDataType, initialValue);
			case SmePackage.COMPILATION_DIRECTIVE_KIND:
				return createCompilationDirectiveKindFromString(eDataType, initialValue);
			case SmePackage.SIGNAL_STATE_KIND:
				return createSignalStateKindFromString(eDataType, initialValue);
			case SmePackage.CASE_KIND:
				return createCaseKindFromString(eDataType, initialValue);
			case SmePackage.TYPE_KIND:
				return createTypeKindFromString(eDataType, initialValue);
			case SmePackage.ARITHMETIC_OP_KIND:
				return createArithmeticOpKindFromString(eDataType, initialValue);
			case SmePackage.COMPARISON_OP_KIND:
				return createComparisonOpKindFromString(eDataType, initialValue);
			case SmePackage.CLOCK_CONSTRAINT_OP_KIND:
				return createClockConstraintOpKindFromString(eDataType, initialValue);
			case SmePackage.CLOCK_RELATION_OP_KIND:
				return createClockRelationOpKindFromString(eDataType, initialValue);
			case SmePackage.CLOCK_SPEED_OP_KIND:
				return createClockSpeedOpKindFromString(eDataType, initialValue);
			case SmePackage.VISIBILITY_KIND:
				return createVisibilityKindFromString(eDataType, initialValue);
			case SmePackage.COUNTER_KIND:
				return createCounterKindFromString(eDataType, initialValue);
			case SmePackage.STATUS_KIND:
				return createStatusKindFromString(eDataType, initialValue);
			case SmePackage.CYCLIC_DEFINITION_EXCEPTION:
				return createCyclicDefinitionExceptionFromString(eDataType, initialValue);
			case SmePackage.SET:
				return createSetFromString(eDataType, initialValue);
			default:
				throw new IllegalArgumentException("The datatype '" + eDataType.getName()
						+ "' is not a valid classifier");
		}
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String convertToString(EDataType eDataType, Object instanceValue) {
		switch (eDataType.getClassifierID()) {
			case SmePackage.SIGNAL_INTRINSIC_TYPE:
				return convertSignalIntrinsicTypeToString(eDataType, instanceValue);
			case SmePackage.MODEL_KIND:
				return convertModelKindToString(eDataType, instanceValue);
			case SmePackage.PROCESS_ATTRIBUTE_KIND:
				return convertProcessAttributeKindToString(eDataType, instanceValue);
			case SmePackage.COMPILATION_DIRECTIVE_KIND:
				return convertCompilationDirectiveKindToString(eDataType, instanceValue);
			case SmePackage.SIGNAL_STATE_KIND:
				return convertSignalStateKindToString(eDataType, instanceValue);
			case SmePackage.CASE_KIND:
				return convertCaseKindToString(eDataType, instanceValue);
			case SmePackage.TYPE_KIND:
				return convertTypeKindToString(eDataType, instanceValue);
			case SmePackage.ARITHMETIC_OP_KIND:
				return convertArithmeticOpKindToString(eDataType, instanceValue);
			case SmePackage.COMPARISON_OP_KIND:
				return convertComparisonOpKindToString(eDataType, instanceValue);
			case SmePackage.CLOCK_CONSTRAINT_OP_KIND:
				return convertClockConstraintOpKindToString(eDataType, instanceValue);
			case SmePackage.CLOCK_RELATION_OP_KIND:
				return convertClockRelationOpKindToString(eDataType, instanceValue);
			case SmePackage.CLOCK_SPEED_OP_KIND:
				return convertClockSpeedOpKindToString(eDataType, instanceValue);
			case SmePackage.VISIBILITY_KIND:
				return convertVisibilityKindToString(eDataType, instanceValue);
			case SmePackage.COUNTER_KIND:
				return convertCounterKindToString(eDataType, instanceValue);
			case SmePackage.STATUS_KIND:
				return convertStatusKindToString(eDataType, instanceValue);
			case SmePackage.CYCLIC_DEFINITION_EXCEPTION:
				return convertCyclicDefinitionExceptionToString(eDataType, instanceValue);
			case SmePackage.SET:
				return convertSetToString(eDataType, instanceValue);
			default:
				throw new IllegalArgumentException("The datatype '" + eDataType.getName()
						+ "' is not a valid classifier");
		}
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public BasicIterate createBasicIterate() {
		BasicIterateImpl basicIterate = new BasicIterateImpl();
		return basicIterate;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Iterate createIterate() {
		IterateImpl iterate = new IterateImpl();
		return iterate;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public IterationInit createIterationInit() {
		IterationInitImpl iterationInit = new IterationInitImpl();
		return iterationInit;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Specifications createSpecifications() {
		SpecificationsImpl specifications = new SpecificationsImpl();
		return specifications;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SubProcess createSubProcess() {
		SubProcessImpl subProcess = new SubProcessImpl();
		return subProcess;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EnumerationType createEnumerationType() {
		EnumerationTypeImpl enumerationType = new EnumerationTypeImpl();
		return enumerationType;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelType createModelType() {
		ModelTypeImpl modelType = new ModelTypeImpl();
		return modelType;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SubstitutionType createSubstitutionType() {
		SubstitutionTypeImpl substitutionType = new SubstitutionTypeImpl();
		return substitutionType;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public TupleType createTupleType() {
		TupleTypeImpl tupleType = new TupleTypeImpl();
		return tupleType;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelDeclaration createModelDeclaration() {
		ModelDeclarationImpl modelDeclaration = new ModelDeclarationImpl();
		return modelDeclaration;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelRef createModelRef() {
		ModelRefImpl modelRef = new ModelRefImpl();
		return modelRef;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelInstance createModelInstance() {
		ModelInstanceImpl modelInstance = new ModelInstanceImpl();
		return modelInstance;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Module createModule() {
		ModuleImpl module = new ModuleImpl();
		return module;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public UseModule createUseModule() {
		UseModuleImpl useModule = new UseModuleImpl();
		return useModule;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantRef createConstantRef() {
		ConstantRefImpl constantRef = new ConstantRefImpl();
		return constantRef;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantValue createConstantValue() {
		ConstantValueImpl constantValue = new ConstantValueImpl();
		return constantValue;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Literal createLiteral() {
		LiteralImpl literal = new LiteralImpl();
		return literal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EnumerationValue createEnumerationValue() {
		EnumerationValueImpl enumerationValue = new EnumerationValueImpl();
		return enumerationValue;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Parameter createParameter() {
		ParameterImpl parameter = new ParameterImpl();
		return parameter;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ParameterInstance createParameterInstance() {
		ParameterInstanceImpl parameterInstance = new ParameterInstanceImpl();
		return parameterInstance;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Iterator createIterator() {
		IteratorImpl iterator = new IteratorImpl();
		return iterator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Index createIndex() {
		IndexImpl index = new IndexImpl();
		return index;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalRef createSignalRef() {
		SignalRefImpl signalRef = new SignalRefImpl();
		return signalRef;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Input createInput() {
		InputImpl input = new InputImpl();
		return input;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public InputInstance createInputInstance() {
		InputInstanceImpl inputInstance = new InputInstanceImpl();
		return inputInstance;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Output createOutput() {
		OutputImpl output = new OutputImpl();
		return output;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public OutputInstance createOutputInstance() {
		OutputInstanceImpl outputInstance = new OutputInstanceImpl();
		return outputInstance;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Local createLocal() {
		LocalImpl local = new LocalImpl();
		return local;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CaseConnection createCaseConnection() {
		CaseConnectionImpl caseConnection = new CaseConnectionImpl();
		return caseConnection;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ClockRelation createClockRelation() {
		ClockRelationImpl clockRelation = new ClockRelationImpl();
		return clockRelation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Condition createCondition() {
		ConditionImpl condition = new ConditionImpl();
		return condition;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ConstantAffectation createConstantAffectation() {
		ConstantAffectationImpl constantAffectation = new ConstantAffectationImpl();
		return constantAffectation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Dependence createDependence() {
		DependenceImpl dependence = new DependenceImpl();
		return dependence;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public DataflowConnection createDataflowConnection() {
		DataflowConnectionImpl dataflowConnection = new DataflowConnectionImpl();
		return dataflowConnection;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public InputPort createInputPort() {
		InputPortImpl inputPort = new InputPortImpl();
		return inputPort;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public OutputPort createOutputPort() {
		OutputPortImpl outputPort = new OutputPortImpl();
		return outputPort;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public And createAnd() {
		AndImpl and = new AndImpl();
		return and;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ArithmeticOperator createArithmeticOperator() {
		ArithmeticOperatorImpl arithmeticOperator = new ArithmeticOperatorImpl();
		return arithmeticOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ArrayEnumeration createArrayEnumeration() {
		ArrayEnumerationImpl arrayEnumeration = new ArrayEnumerationImpl();
		return arrayEnumeration;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ArrayProduct createArrayProduct() {
		ArrayProductImpl arrayProduct = new ArrayProductImpl();
		return arrayProduct;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ArrayRecovery createArrayRecovery() {
		ArrayRecoveryImpl arrayRecovery = new ArrayRecoveryImpl();
		return arrayRecovery;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ArrayRestructuration createArrayRestructuration() {
		ArrayRestructurationImpl arrayRestructuration = new ArrayRestructurationImpl();
		return arrayRestructuration;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Assertion createAssertion() {
		AssertionImpl assertion = new AssertionImpl();
		return assertion;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public AssertionClockConstraintOperator createAssertionClockConstraintOperator() {
		AssertionClockConstraintOperatorImpl assertionClockConstraintOperator = new AssertionClockConstraintOperatorImpl();
		return assertionClockConstraintOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public AssertionClockSpeedOperator createAssertionClockSpeedOperator() {
		AssertionClockSpeedOperatorImpl assertionClockSpeedOperator = new AssertionClockSpeedOperatorImpl();
		return assertionClockSpeedOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public BooleanExpression createBooleanExpression() {
		BooleanExpressionImpl booleanExpression = new BooleanExpressionImpl();
		return booleanExpression;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CartesianProduct createCartesianProduct() {
		CartesianProductImpl cartesianProduct = new CartesianProductImpl();
		return cartesianProduct;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Cell createCell() {
		CellImpl cell = new CellImpl();
		return cell;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ClockConstraintOperator createClockConstraintOperator() {
		ClockConstraintOperatorImpl clockConstraintOperator = new ClockConstraintOperatorImpl();
		return clockConstraintOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ClockRelationOperator createClockRelationOperator() {
		ClockRelationOperatorImpl clockRelationOperator = new ClockRelationOperatorImpl();
		return clockRelationOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ClockSpeedOperator createClockSpeedOperator() {
		ClockSpeedOperatorImpl clockSpeedOperator = new ClockSpeedOperatorImpl();
		return clockSpeedOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ComparisonOperator createComparisonOperator() {
		ComparisonOperatorImpl comparisonOperator = new ComparisonOperatorImpl();
		return comparisonOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ComplexOperator createComplexOperator() {
		ComplexOperatorImpl complexOperator = new ComplexOperatorImpl();
		return complexOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Concatenation createConcatenation() {
		ConcatenationImpl concatenation = new ConcatenationImpl();
		return concatenation;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Counter createCounter() {
		CounterImpl counter = new CounterImpl();
		return counter;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Delay createDelay() {
		DelayImpl delay = new DelayImpl();
		return delay;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public DependenceOperator createDependenceOperator() {
		DependenceOperatorImpl dependenceOperator = new DependenceOperatorImpl();
		return dependenceOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Extraction createExtraction() {
		ExtractionImpl extraction = new ExtractionImpl();
		return extraction;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public IfThenElse createIfThenElse() {
		IfThenElseImpl ifThenElse = new IfThenElseImpl();
		return ifThenElse;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Merge createMerge() {
		MergeImpl merge = new MergeImpl();
		return merge;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Not createNot() {
		NotImpl not = new NotImpl();
		return not;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NullClock createNullClock() {
		NullClockImpl nullClock = new NullClockImpl();
		return nullClock;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NumericExpression createNumericExpression() {
		NumericExpressionImpl numericExpression = new NumericExpressionImpl();
		return numericExpression;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Or createOr() {
		OrImpl or = new OrImpl();
		return or;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Pragma createPragma() {
		PragmaImpl pragma = new PragmaImpl();
		return pragma;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public RepetitionOperator createRepetitionOperator() {
		RepetitionOperatorImpl repetitionOperator = new RepetitionOperatorImpl();
		return repetitionOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SequentialDefinition createSequentialDefinition() {
		SequentialDefinitionImpl sequentialDefinition = new SequentialDefinitionImpl();
		return sequentialDefinition;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Switch createSwitch() {
		SwitchImpl switch_ = new SwitchImpl();
		return switch_;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Transposition createTransposition() {
		TranspositionImpl transposition = new TranspositionImpl();
		return transposition;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public TupleOperator createTupleOperator() {
		TupleOperatorImpl tupleOperator = new TupleOperatorImpl();
		return tupleOperator;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public UnaryMinus createUnaryMinus() {
		UnaryMinusImpl unaryMinus = new UnaryMinusImpl();
		return unaryMinus;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Variable createVariable() {
		VariableImpl variable = new VariableImpl();
		return variable;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Window createWindow() {
		WindowImpl window = new WindowImpl();
		return window;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Xor createXor() {
		XorImpl xor = new XorImpl();
		return xor;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public History createHistory() {
		HistoryImpl history = new HistoryImpl();
		return history;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public State createState() {
		StateImpl state = new StateImpl();
		return state;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Automaton createAutomaton() {
		AutomatonImpl automaton = new AutomatonImpl();
		return automaton;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public AndState createAndState() {
		AndStateImpl andState = new AndStateImpl();
		return andState;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public WeakTransition createWeakTransition() {
		WeakTransitionImpl weakTransition = new WeakTransitionImpl();
		return weakTransition;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public StrongTransition createStrongTransition() {
		StrongTransitionImpl strongTransition = new StrongTransitionImpl();
		return strongTransition;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalIntrinsicType createSignalIntrinsicTypeFromString(EDataType eDataType, String initialValue) {
		SignalIntrinsicType result = SignalIntrinsicType.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertSignalIntrinsicTypeToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ModelKind createModelKindFromString(EDataType eDataType, String initialValue) {
		ModelKind result = ModelKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertModelKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ProcessAttributeKind createProcessAttributeKindFromString(EDataType eDataType, String initialValue) {
		ProcessAttributeKind result = ProcessAttributeKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertProcessAttributeKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CompilationDirectiveKind createCompilationDirectiveKindFromString(EDataType eDataType, String initialValue) {
		CompilationDirectiveKind result = CompilationDirectiveKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertCompilationDirectiveKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SignalStateKind createSignalStateKindFromString(EDataType eDataType, String initialValue) {
		SignalStateKind result = SignalStateKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertSignalStateKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CaseKind createCaseKindFromString(EDataType eDataType, String initialValue) {
		CaseKind result = CaseKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertCaseKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public TypeKind createTypeKindFromString(EDataType eDataType, String initialValue) {
		TypeKind result = TypeKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertTypeKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ArithmeticOpKind createArithmeticOpKindFromString(EDataType eDataType, String initialValue) {
		ArithmeticOpKind result = ArithmeticOpKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertArithmeticOpKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ComparisonOpKind createComparisonOpKindFromString(EDataType eDataType, String initialValue) {
		ComparisonOpKind result = ComparisonOpKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertComparisonOpKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ClockConstraintOpKind createClockConstraintOpKindFromString(EDataType eDataType, String initialValue) {
		ClockConstraintOpKind result = ClockConstraintOpKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertClockConstraintOpKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ClockRelationOpKind createClockRelationOpKindFromString(EDataType eDataType, String initialValue) {
		ClockRelationOpKind result = ClockRelationOpKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertClockRelationOpKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public ClockSpeedOpKind createClockSpeedOpKindFromString(EDataType eDataType, String initialValue) {
		ClockSpeedOpKind result = ClockSpeedOpKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertClockSpeedOpKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public VisibilityKind createVisibilityKindFromString(EDataType eDataType, String initialValue) {
		VisibilityKind result = VisibilityKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertVisibilityKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CounterKind createCounterKindFromString(EDataType eDataType, String initialValue) {
		CounterKind result = CounterKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertCounterKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public StatusKind createStatusKindFromString(EDataType eDataType, String initialValue) {
		StatusKind result = StatusKind.get(initialValue);
		if (result == null)
			throw new IllegalArgumentException("The value '" + initialValue + "' is not a valid enumerator of '"
					+ eDataType.getName() + "'");
		return result;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertStatusKindToString(EDataType eDataType, Object instanceValue) {
		return instanceValue == null ? null : instanceValue.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CyclicDefinitionException createCyclicDefinitionExceptionFromString(EDataType eDataType, String initialValue) {
		return (CyclicDefinitionException) super.createFromString(eDataType, initialValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertCyclicDefinitionExceptionToString(EDataType eDataType, Object instanceValue) {
		return super.convertToString(eDataType, instanceValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Set<?> createSetFromString(EDataType eDataType, String initialValue) {
		return (Set<?>) super.createFromString(initialValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String convertSetToString(EDataType eDataType, Object instanceValue) {
		return super.convertToString(instanceValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SmePackage getSmePackage() {
		return (SmePackage) getEPackage();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @deprecated
	 * @generated
	 */
	@Deprecated
	public static SmePackage getPackage() {
		return SmePackage.eINSTANCE;
	}
	
} // SmeFactoryImpl
