/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.CompoundState;
import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.State;
import fr.irisa.espresso.sme.StateBase;
import fr.irisa.espresso.sme.StateOrHistory;
import fr.irisa.espresso.sme.StrongTransition;
import fr.irisa.espresso.sme.WeakTransition;
import fr.irisa.espresso.sme.polychrony.DeclarationValue;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.SignalTreeBuilting;
import fr.irisa.espresso.sme.polychrony.SignalTreeExploring;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;
import fr.irisa.espresso.sme.util.Utils;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>State</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.StateImpl#getDst_StrongTransitions <em>Dst Strong Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.StateImpl#getDst_WeakTransitions <em>Dst Weak Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.StateImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.StateImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.StateImpl#getStateParent <em>State Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.StateImpl#getSrc_WeakTransitions <em>Src Weak Transitions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.StateImpl#getSrc_StrongTransitions <em>Src Strong Transitions</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class StateImpl extends ModelImpl implements State {
	/**
	 * The cached value of the '{@link #getDst_StrongTransitions() <em>Dst Strong Transitions</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_StrongTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<StrongTransition>	dst_StrongTransitions;
	
	/**
	 * The cached value of the '{@link #getDst_WeakTransitions() <em>Dst Weak Transitions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_WeakTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<WeakTransition>		dst_WeakTransitions;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String		NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String					name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String		COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String					comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc_WeakTransitions() <em>Src Weak Transitions</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_WeakTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<WeakTransition>		src_WeakTransitions;
	
	/**
	 * The cached value of the '{@link #getSrc_StrongTransitions() <em>Src Strong Transitions</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_StrongTransitions()
	 * @generated
	 * @ordered
	 */
	protected EList<StrongTransition>	src_StrongTransitions;
	
	// Tree APIs
	final PKTrees						API					= PkPlugin.getTreeAPI();		// Tree API low level
	final SignalTreeBuilting			TBAPI				= PkPlugin.getTreeBuilder();	// Signal Tree Builder API
	final SignalTreeExploring			TEAPI				= PkPlugin.getTreeExplorer();	// Signal Tree explorer API
																							
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected StateImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.STATE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<StrongTransition> getDst_StrongTransitions() {
		if (dst_StrongTransitions == null) {
			dst_StrongTransitions = new EObjectWithInverseResolvingEList<StrongTransition>(StrongTransition.class,
				this, SmePackage.STATE__DST_STRONG_TRANSITIONS, SmePackage.STRONG_TRANSITION__DST);
		}
		return dst_StrongTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<WeakTransition> getDst_WeakTransitions() {
		if (dst_WeakTransitions == null) {
			dst_WeakTransitions = new EObjectWithInverseResolvingEList<WeakTransition>(WeakTransition.class, this,
				SmePackage.STATE__DST_WEAK_TRANSITIONS, SmePackage.WEAK_TRANSITION__DST);
		}
		return dst_WeakTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.STATE__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.STATE__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CompoundState getStateParent() {
		if (eContainerFeatureID != SmePackage.STATE__STATE_PARENT)
			return null;
		return (CompoundState) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetStateParent(CompoundState newStateParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newStateParent, SmePackage.STATE__STATE_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setStateParent(CompoundState newStateParent) {
		if (newStateParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.STATE__STATE_PARENT && newStateParent != null)) {
			if (EcoreUtil.isAncestor(this, newStateParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newStateParent != null)
				msgs = ((InternalEObject) newStateParent).eInverseAdd(this, SmePackage.COMPOUND_STATE__STATES,
					CompoundState.class, msgs);
			msgs = basicSetStateParent(newStateParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.STATE__STATE_PARENT, newStateParent,
				newStateParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<WeakTransition> getSrc_WeakTransitions() {
		if (src_WeakTransitions == null) {
			src_WeakTransitions = new EObjectWithInverseResolvingEList<WeakTransition>(WeakTransition.class, this,
				SmePackage.STATE__SRC_WEAK_TRANSITIONS, SmePackage.WEAK_TRANSITION__SRC);
		}
		return src_WeakTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<StrongTransition> getSrc_StrongTransitions() {
		if (src_StrongTransitions == null) {
			src_StrongTransitions = new EObjectWithInverseResolvingEList<StrongTransition>(StrongTransition.class,
				this, SmePackage.STATE__SRC_STRONG_TRANSITIONS, SmePackage.STRONG_TRANSITION__SRC);
		}
		return src_StrongTransitions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.STATE__DST_STRONG_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_StrongTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.STATE__DST_WEAK_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_WeakTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.STATE__STATE_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetStateParent((CompoundState) otherEnd, msgs);
			case SmePackage.STATE__SRC_WEAK_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_WeakTransitions()).basicAdd(
					otherEnd, msgs);
			case SmePackage.STATE__SRC_STRONG_TRANSITIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_StrongTransitions()).basicAdd(
					otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.STATE__DST_STRONG_TRANSITIONS:
				return ((InternalEList<?>) getDst_StrongTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.STATE__DST_WEAK_TRANSITIONS:
				return ((InternalEList<?>) getDst_WeakTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.STATE__STATE_PARENT:
				return basicSetStateParent(null, msgs);
			case SmePackage.STATE__SRC_WEAK_TRANSITIONS:
				return ((InternalEList<?>) getSrc_WeakTransitions()).basicRemove(otherEnd, msgs);
			case SmePackage.STATE__SRC_STRONG_TRANSITIONS:
				return ((InternalEList<?>) getSrc_StrongTransitions()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.STATE__STATE_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.COMPOUND_STATE__STATES,
					CompoundState.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.STATE__DST_STRONG_TRANSITIONS:
				return getDst_StrongTransitions();
			case SmePackage.STATE__DST_WEAK_TRANSITIONS:
				return getDst_WeakTransitions();
			case SmePackage.STATE__NAME:
				return getName();
			case SmePackage.STATE__COMMENT:
				return getComment();
			case SmePackage.STATE__STATE_PARENT:
				return getStateParent();
			case SmePackage.STATE__SRC_WEAK_TRANSITIONS:
				return getSrc_WeakTransitions();
			case SmePackage.STATE__SRC_STRONG_TRANSITIONS:
				return getSrc_StrongTransitions();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.STATE__DST_STRONG_TRANSITIONS:
				getDst_StrongTransitions().clear();
				getDst_StrongTransitions().addAll((Collection<? extends StrongTransition>) newValue);
				return;
			case SmePackage.STATE__DST_WEAK_TRANSITIONS:
				getDst_WeakTransitions().clear();
				getDst_WeakTransitions().addAll((Collection<? extends WeakTransition>) newValue);
				return;
			case SmePackage.STATE__NAME:
				setName((String) newValue);
				return;
			case SmePackage.STATE__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.STATE__STATE_PARENT:
				setStateParent((CompoundState) newValue);
				return;
			case SmePackage.STATE__SRC_WEAK_TRANSITIONS:
				getSrc_WeakTransitions().clear();
				getSrc_WeakTransitions().addAll((Collection<? extends WeakTransition>) newValue);
				return;
			case SmePackage.STATE__SRC_STRONG_TRANSITIONS:
				getSrc_StrongTransitions().clear();
				getSrc_StrongTransitions().addAll((Collection<? extends StrongTransition>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.STATE__DST_STRONG_TRANSITIONS:
				getDst_StrongTransitions().clear();
				return;
			case SmePackage.STATE__DST_WEAK_TRANSITIONS:
				getDst_WeakTransitions().clear();
				return;
			case SmePackage.STATE__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.STATE__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.STATE__STATE_PARENT:
				setStateParent((CompoundState) null);
				return;
			case SmePackage.STATE__SRC_WEAK_TRANSITIONS:
				getSrc_WeakTransitions().clear();
				return;
			case SmePackage.STATE__SRC_STRONG_TRANSITIONS:
				getSrc_StrongTransitions().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.STATE__DST_STRONG_TRANSITIONS:
				return dst_StrongTransitions != null && !dst_StrongTransitions.isEmpty();
			case SmePackage.STATE__DST_WEAK_TRANSITIONS:
				return dst_WeakTransitions != null && !dst_WeakTransitions.isEmpty();
			case SmePackage.STATE__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.STATE__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.STATE__STATE_PARENT:
				return getStateParent() != null;
			case SmePackage.STATE__SRC_WEAK_TRANSITIONS:
				return src_WeakTransitions != null && !src_WeakTransitions.isEmpty();
			case SmePackage.STATE__SRC_STRONG_TRANSITIONS:
				return src_StrongTransitions != null && !src_StrongTransitions.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == StateOrHistory.class) {
			switch (derivedFeatureID) {
				case SmePackage.STATE__DST_STRONG_TRANSITIONS:
					return SmePackage.STATE_OR_HISTORY__DST_STRONG_TRANSITIONS;
				case SmePackage.STATE__DST_WEAK_TRANSITIONS:
					return SmePackage.STATE_OR_HISTORY__DST_WEAK_TRANSITIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.STATE__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.STATE__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == StateBase.class) {
			switch (derivedFeatureID) {
				case SmePackage.STATE__STATE_PARENT:
					return SmePackage.STATE_BASE__STATE_PARENT;
				case SmePackage.STATE__SRC_WEAK_TRANSITIONS:
					return SmePackage.STATE_BASE__SRC_WEAK_TRANSITIONS;
				case SmePackage.STATE__SRC_STRONG_TRANSITIONS:
					return SmePackage.STATE_BASE__SRC_STRONG_TRANSITIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == StateOrHistory.class) {
			switch (baseFeatureID) {
				case SmePackage.STATE_OR_HISTORY__DST_STRONG_TRANSITIONS:
					return SmePackage.STATE__DST_STRONG_TRANSITIONS;
				case SmePackage.STATE_OR_HISTORY__DST_WEAK_TRANSITIONS:
					return SmePackage.STATE__DST_WEAK_TRANSITIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.STATE__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.STATE__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == StateBase.class) {
			switch (baseFeatureID) {
				case SmePackage.STATE_BASE__STATE_PARENT:
					return SmePackage.STATE__STATE_PARENT;
				case SmePackage.STATE_BASE__SRC_WEAK_TRANSITIONS:
					return SmePackage.STATE__SRC_WEAK_TRANSITIONS;
				case SmePackage.STATE_BASE__SRC_STRONG_TRANSITIONS:
					return SmePackage.STATE__SRC_STRONG_TRANSITIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			result.append(name);
			result.append("::(| ... |)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getStateParent();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleConstants()
	 * @generated NOT
	 */
	@Override
	public EList<ConstantOrRef> getVisibleConstants() {
		EList<ConstantOrRef> list = super.getVisibleConstants();
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleConstants());
		
		return list;
	}
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleModelProcesses()
	 * @generated NOT
	 */
	@Override
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		EList<InterfaceDefinition> list = super.getVisibleModelProcesses();
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleModelProcesses());
		
		return list;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleSignals()
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		EList<SignalOrRef> list = super.getVisibleSignals();
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleSignals());
		
		return list;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleTypes()
	 * @generated NOT
	 */
	@Override
	public EList<Identifier> getVisibleTypes() {
		EList<Identifier> list = super.getVisibleTypes();
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleTypes());
		
		return list;
	}
	
	/**
	 * This method must only be called by the local buildAutomaton method.
	 * 
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#buildEquationsAST(int)
	 * @generated NOT
	 */
	@Override
	public long buildEquationsAST(long bodyTree) {
		long body = super.buildEquationsAST(PKTrees.ERROR_NODE);
		// If the state is IN an AndState, we use the name of the state as a label
		
		/*
		 * if (getParent() instanceof AndState) { Utils.TRACE(" Cas 1 ---"+ getName()); long idNode = getAST(null); if
		 * (idNode != PKTrees.ERROR_NODE) body = API.makeBinary(SignalOperators.oppetiq, idNode, body); bodyTree =
		 * API.post(bodyTree, body); } else if (getParent() instanceof Automaton) { Utils.TRACE(" Cas 2 ---"+
		 * getName()); long idNode = getAST(null); if (idNode != PKTrees.ERROR_NODE) { idNode =
		 * API.makeList(SignalOperators.oplesign, API.makeBinary(SignalOperators.opvenum, TBAPI.mkAtomNilTree(),
		 * idNode)); body = API.makeBinary(SignalOperators.opcas, idNode, body); bodyTree = API.post(bodyTree, body); }
		 * } else PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.STATE_NOT_IMPLEMENTED, this);
		 */
		return body;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getAST(java.util.Set)
	 * @generated NOT
	 */
	@Override
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE)
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
			else
				result = idNode;
		}
		else
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getDeclarationAST(long)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		
		// If the state is IN an AndState, we declare the name of the state as a label
		/*
		 * if (getParent() instanceof AndState) { long idNode = getAST(null); if (idNode != PKTrees.ERROR_NODE) declTree
		 * = API.post(declTree, API.makeUnary(SignalOperators.oplab, API.makeList(SignalOperators.oplident, idNode))); }
		 * // If the state is in an Automaton, we have nothing to declare.
		 */
		return declTree;
	}
	
	/**
	 * @ return the name of the Signal process associated with and automata.
	 * 
	 * @generated NOT
	 */
	private long getProcessName() {
		return API.string2ident("State_" + getName());
	}
	
	/**
	 * @see fr.irisa.espresso.sme.StateBase#buildAutomaton(fr.irisa.espresso.sme.polychrony.DeclarationValue,
	 *      java.lang.String)
	 * @generated NOT
	 */
	public long buildAutomaton(long bodyTree, DeclarationValue rootDeclaration, String reinitSignalName) {
		long declTree;
		int op;
		boolean b;
		
		Utils.TRACE("buildAutomaton : StateImpl : " + getName() + " start");
		long body = buildEquationsAST(bodyTree);
		Utils.TRACE("buildAutomaton : StateImpl : (1) " + API.oper(body));
		
		long mp = TBAPI.n_mkSkeletonModel(getProcessName(), SignalOperators.opprocess);
		/*
		 * long theTick = PkPlugin.getParser().parseString(Phyla.ph_dec_s, "event" + TICKNAME);
		 * TBAPI.n_addDeclarationToInterface(mp, theTick, 2);
		 */
		op = API.oper(body);
		b = true;
		if (op == SignalOperators.oppconf) {
			declTree = API.child(2, body);
			Utils.TRACE("StateImpl declTree operator  " + API.oper(declTree));
			body = API.child(1, body);
			Utils.TRACE("StateImpl body operator  " + API.oper(body));
			TBAPI.setLocalDeclarations(mp, declTree);
		}
		else if (op == SignalOperators.opnullproc) {
			body = API.makeEmptyList(SignalOperators.oplfeed);
			body = API.post(body, TBAPI.mkReferenceToModel(API.string2ident("Nil_Process")));
			b = false;
		}
		Utils.TRACE("buildAutomaton : StateImpl :  (2) " + API.oper(body));
		if (b)
			TBAPI.moveModelsFrom(mp, body);
		TBAPI.setBody(mp, body);
		Utils.TRACE("buildAutomaton : StateImpl : " + getName() + " done");
		return mp;
	}
	
}
