/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.CaseConnection;
import fr.irisa.espresso.sme.ClockRelation;
import fr.irisa.espresso.sme.ClockedExpr;
import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.Dependence;
import fr.irisa.espresso.sme.DependenceInOut;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.SubProcess;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Sub Process</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.SubProcessImpl#getSrc_ClockRelations <em>Src Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubProcessImpl#getDst_ClockRelations <em>Dst Clock Relations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubProcessImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubProcessImpl#getComment <em>Comment</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubProcessImpl#getSrc_Dependences <em>Src Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubProcessImpl#getDst_Dependences <em>Dst Dependences</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubProcessImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubProcessImpl#getDst_CaseConnection <em>Dst Case Connection</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class SubProcessImpl extends ModelImpl implements SubProcess {
	/**
	 * The cached value of the '{@link #getSrc_ClockRelations() <em>Src Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>	src_ClockRelations;
	
	/**
	 * The cached value of the '{@link #getDst_ClockRelations() <em>Dst Clock Relations</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_ClockRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<ClockRelation>	dst_ClockRelations;
	
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String	NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String				name				= NAME_EDEFAULT;
	
	/**
	 * The default value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected static final String	COMMENT_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getComment() <em>Comment</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getComment()
	 * @generated
	 * @ordered
	 */
	protected String				comment				= COMMENT_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSrc_Dependences() <em>Src Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>		src_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_Dependences() <em>Dst Dependences</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_Dependences()
	 * @generated
	 * @ordered
	 */
	protected EList<Dependence>		dst_Dependences;
	
	/**
	 * The cached value of the '{@link #getDst_CaseConnection() <em>Dst Case Connection</em>}' reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst_CaseConnection()
	 * @generated
	 * @ordered
	 */
	protected CaseConnection		dst_CaseConnection;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SubProcessImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.SUB_PROCESS;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getSrc_ClockRelations() {
		if (src_ClockRelations == null) {
			src_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__SRC);
		}
		return src_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ClockRelation> getDst_ClockRelations() {
		if (dst_ClockRelations == null) {
			dst_ClockRelations = new EObjectWithInverseResolvingEList<ClockRelation>(ClockRelation.class, this,
				SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS, SmePackage.CLOCK_RELATION__DST);
		}
		return dst_ClockRelations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SUB_PROCESS__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getComment() {
		return comment;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setComment(String newComment) {
		String oldComment = comment;
		comment = newComment;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SUB_PROCESS__COMMENT, oldComment, comment));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getSrc_Dependences() {
		if (src_Dependences == null) {
			src_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.SUB_PROCESS__SRC_DEPENDENCES, SmePackage.DEPENDENCE__SRC);
		}
		return src_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Dependence> getDst_Dependences() {
		if (dst_Dependences == null) {
			dst_Dependences = new EObjectWithInverseResolvingEList<Dependence>(Dependence.class, this,
				SmePackage.SUB_PROCESS__DST_DEPENDENCES, SmePackage.DEPENDENCE__DST);
		}
		return dst_Dependences;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.SUB_PROCESS__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.SUB_PROCESS__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.SUB_PROCESS__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__SUBPROCESSES,
					Model.class, msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SUB_PROCESS__MODEL_PARENT, newModelParent,
				newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CaseConnection getDst_CaseConnection() {
		if (dst_CaseConnection != null && dst_CaseConnection.eIsProxy()) {
			InternalEObject oldDst_CaseConnection = (InternalEObject) dst_CaseConnection;
			dst_CaseConnection = (CaseConnection) eResolveProxy(oldDst_CaseConnection);
			if (dst_CaseConnection != oldDst_CaseConnection) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE,
						SmePackage.SUB_PROCESS__DST_CASE_CONNECTION, oldDst_CaseConnection, dst_CaseConnection));
			}
		}
		return dst_CaseConnection;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public CaseConnection basicGetDst_CaseConnection() {
		return dst_CaseConnection;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst_CaseConnection(CaseConnection newDst_CaseConnection, NotificationChain msgs) {
		CaseConnection oldDst_CaseConnection = dst_CaseConnection;
		dst_CaseConnection = newDst_CaseConnection;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.SUB_PROCESS__DST_CASE_CONNECTION, oldDst_CaseConnection, newDst_CaseConnection);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst_CaseConnection(CaseConnection newDst_CaseConnection) {
		if (newDst_CaseConnection != dst_CaseConnection) {
			NotificationChain msgs = null;
			if (dst_CaseConnection != null)
				msgs = ((InternalEObject) dst_CaseConnection).eInverseRemove(this, SmePackage.CASE_CONNECTION__DST,
					CaseConnection.class, msgs);
			if (newDst_CaseConnection != null)
				msgs = ((InternalEObject) newDst_CaseConnection).eInverseAdd(this, SmePackage.CASE_CONNECTION__DST,
					CaseConnection.class, msgs);
			msgs = basicSetDst_CaseConnection(newDst_CaseConnection, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SUB_PROCESS__DST_CASE_CONNECTION,
				newDst_CaseConnection, newDst_CaseConnection));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_ClockRelations()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SUB_PROCESS__SRC_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SUB_PROCESS__DST_DEPENDENCES:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getDst_Dependences()).basicAdd(otherEnd,
					msgs);
			case SmePackage.SUB_PROCESS__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
			case SmePackage.SUB_PROCESS__DST_CASE_CONNECTION:
				if (dst_CaseConnection != null)
					msgs = ((InternalEObject) dst_CaseConnection).eInverseRemove(this, SmePackage.CASE_CONNECTION__DST,
						CaseConnection.class, msgs);
				return basicSetDst_CaseConnection((CaseConnection) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS:
				return ((InternalEList<?>) getSrc_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS:
				return ((InternalEList<?>) getDst_ClockRelations()).basicRemove(otherEnd, msgs);
			case SmePackage.SUB_PROCESS__SRC_DEPENDENCES:
				return ((InternalEList<?>) getSrc_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.SUB_PROCESS__DST_DEPENDENCES:
				return ((InternalEList<?>) getDst_Dependences()).basicRemove(otherEnd, msgs);
			case SmePackage.SUB_PROCESS__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
			case SmePackage.SUB_PROCESS__DST_CASE_CONNECTION:
				return basicSetDst_CaseConnection(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.SUB_PROCESS__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__SUBPROCESSES, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS:
				return getSrc_ClockRelations();
			case SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS:
				return getDst_ClockRelations();
			case SmePackage.SUB_PROCESS__NAME:
				return getName();
			case SmePackage.SUB_PROCESS__COMMENT:
				return getComment();
			case SmePackage.SUB_PROCESS__SRC_DEPENDENCES:
				return getSrc_Dependences();
			case SmePackage.SUB_PROCESS__DST_DEPENDENCES:
				return getDst_Dependences();
			case SmePackage.SUB_PROCESS__MODEL_PARENT:
				return getModelParent();
			case SmePackage.SUB_PROCESS__DST_CASE_CONNECTION:
				if (resolve)
					return getDst_CaseConnection();
				return basicGetDst_CaseConnection();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				getSrc_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				getDst_ClockRelations().addAll((Collection<? extends ClockRelation>) newValue);
				return;
			case SmePackage.SUB_PROCESS__NAME:
				setName((String) newValue);
				return;
			case SmePackage.SUB_PROCESS__COMMENT:
				setComment((String) newValue);
				return;
			case SmePackage.SUB_PROCESS__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				getSrc_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.SUB_PROCESS__DST_DEPENDENCES:
				getDst_Dependences().clear();
				getDst_Dependences().addAll((Collection<? extends Dependence>) newValue);
				return;
			case SmePackage.SUB_PROCESS__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.SUB_PROCESS__DST_CASE_CONNECTION:
				setDst_CaseConnection((CaseConnection) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS:
				getSrc_ClockRelations().clear();
				return;
			case SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS:
				getDst_ClockRelations().clear();
				return;
			case SmePackage.SUB_PROCESS__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.SUB_PROCESS__COMMENT:
				setComment(COMMENT_EDEFAULT);
				return;
			case SmePackage.SUB_PROCESS__SRC_DEPENDENCES:
				getSrc_Dependences().clear();
				return;
			case SmePackage.SUB_PROCESS__DST_DEPENDENCES:
				getDst_Dependences().clear();
				return;
			case SmePackage.SUB_PROCESS__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.SUB_PROCESS__DST_CASE_CONNECTION:
				setDst_CaseConnection((CaseConnection) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS:
				return src_ClockRelations != null && !src_ClockRelations.isEmpty();
			case SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS:
				return dst_ClockRelations != null && !dst_ClockRelations.isEmpty();
			case SmePackage.SUB_PROCESS__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.SUB_PROCESS__COMMENT:
				return COMMENT_EDEFAULT == null ? comment != null : !COMMENT_EDEFAULT.equals(comment);
			case SmePackage.SUB_PROCESS__SRC_DEPENDENCES:
				return src_Dependences != null && !src_Dependences.isEmpty();
			case SmePackage.SUB_PROCESS__DST_DEPENDENCES:
				return dst_Dependences != null && !dst_Dependences.isEmpty();
			case SmePackage.SUB_PROCESS__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.SUB_PROCESS__DST_CASE_CONNECTION:
				return dst_CaseConnection != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ClockedExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS:
					return SmePackage.CLOCKED_EXPR__SRC_CLOCK_RELATIONS;
				case SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS:
					return SmePackage.CLOCKED_EXPR__DST_CLOCK_RELATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.SUB_PROCESS__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (derivedFeatureID) {
				case SmePackage.SUB_PROCESS__COMMENT:
					return SmePackage.IDENTIFIER__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (derivedFeatureID) {
				case SmePackage.SUB_PROCESS__SRC_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES;
				case SmePackage.SUB_PROCESS__DST_DEPENDENCES:
					return SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ClockedExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CLOCKED_EXPR__SRC_CLOCK_RELATIONS:
					return SmePackage.SUB_PROCESS__SRC_CLOCK_RELATIONS;
				case SmePackage.CLOCKED_EXPR__DST_CLOCK_RELATIONS:
					return SmePackage.SUB_PROCESS__DST_CLOCK_RELATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.SUB_PROCESS__NAME;
				default:
					return -1;
			}
		}
		if (baseClass == Identifier.class) {
			switch (baseFeatureID) {
				case SmePackage.IDENTIFIER__COMMENT:
					return SmePackage.SUB_PROCESS__COMMENT;
				default:
					return -1;
			}
		}
		if (baseClass == DependenceInOut.class) {
			switch (baseFeatureID) {
				case SmePackage.DEPENDENCE_IN_OUT__SRC_DEPENDENCES:
					return SmePackage.SUB_PROCESS__SRC_DEPENDENCES;
				case SmePackage.DEPENDENCE_IN_OUT__DST_DEPENDENCES:
					return SmePackage.SUB_PROCESS__DST_DEPENDENCES;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", comment: ");
		result.append(comment);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			result.append(name);
			result.append("::(| ... |)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getModelParent();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleConstants()
	 * @generated NOT
	 */
	public EList<ConstantOrRef> getVisibleConstants() {
		EList<ConstantOrRef> list = super.getVisibleConstants();
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleConstants());
		
		return list;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleModelProcesses()
	 * @generated NOT
	 */
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		EList<InterfaceDefinition> list = super.getVisibleModelProcesses();
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleModelProcesses());
		
		return list;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleSignals()
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		EList<SignalOrRef> list = super.getVisibleSignals();
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleSignals());
		
		return list;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getVisibleTypes()
	 * @generated NOT
	 */
	public EList<Identifier> getVisibleTypes() {
		EList<Identifier> list = super.getVisibleTypes();
		
		SignalContainer parent = getParent();
		if (parent != null)
			list.addAll(parent.getVisibleTypes());
		
		return list;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#buildEquationsAST(int)
	 * @generated NOT
	 */
	@Override
	public long buildEquationsAST(long bodyTree) {
		PKTrees API = PkPlugin.getTreeAPI();
		
		if (this.getDst_CaseConnection() == null && bodyTree != PKTrees.ERROR_NODE) {
			// We build the structure of a Model and we add the label to it.
			long result = super.buildEquationsAST(PKTrees.ERROR_NODE);
			long idNode = getAST(null);
			if (idNode != PKTrees.ERROR_NODE)
				result = API.makeBinary(SignalOperators.oppetiq, idNode, result);
			
			// It is used as a labeled part in the specification of the process
			bodyTree = API.post(bodyTree, result);
			
			for (ClockRelation cr : getDst_ClockRelations())
				bodyTree = cr.buildEquationsAST(bodyTree);
			for (Dependence dep : getDst_Dependences())
				bodyTree = dep.buildEquationsAST(bodyTree);
		}
		else if (this.getDst_CaseConnection() != null && bodyTree == PKTrees.ERROR_NODE) {
			// The SubProcess is used as a case in a switch case constructs
			// We build the structure of a Model and we add the label to it.
			long result = super.buildEquationsAST(PKTrees.ERROR_NODE);
			long idNode = getAST(null);
			if (idNode != PKTrees.ERROR_NODE)
				result = API.makeBinary(SignalOperators.oppetiq, idNode, result);
			bodyTree = API.makeList(SignalOperators.oplfeed, result);
			
			for (ClockRelation cr : getDst_ClockRelations())
				bodyTree = cr.buildEquationsAST(bodyTree);
			for (Dependence dep : getDst_Dependences())
				bodyTree = dep.buildEquationsAST(bodyTree);
		}
		
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ModelImpl#getAST(java.util.Set)
	 * @generated NOT
	 */
	@Override
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * Returns the AST of the label of the SubProcess
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			
			declTree = API.post(declTree, API.makeUnary(SignalOperators.oplab, API.makeList(SignalOperators.oplident,
				idNode)));
		}
		return declTree;
	}
	
} // SubProcessImpl
