/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EDataTypeEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.Module;
import fr.irisa.espresso.sme.ParameterType;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalIntrinsicType;
import fr.irisa.espresso.sme.SignalType;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.SubstitutionType;
import fr.irisa.espresso.sme.VisibilityKind;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Substitution Type</b></em>'. <!-- end-user-doc
 * -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.SubstitutionTypeImpl#getSrc_ConstantAffectations <em>Src Constant Affectations
 * </em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubstitutionTypeImpl#getType <em>Type</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubstitutionTypeImpl#isIsExternal <em>Is External</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SubstitutionTypeImpl#getArrayDimensions <em>Array Dimensions</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class SubstitutionTypeImpl extends TypeDeclarationImpl implements SubstitutionType {
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The cached value of the '{@link #getArrayDimensions() <em>Array Dimensions</em>}' attribute list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getArrayDimensions()
	 * @generated
	 * @ordered
	 */
	protected EList<String>					arrayDimensions;
	
	/**
	 * The default value of the '{@link #getType() <em>Type</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getType()
	 * @generated
	 * @ordered
	 */
	protected static final String			TYPE_EDEFAULT			= null;
	
	/**
	 * The cached value of the '{@link #getType() <em>Type</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getType()
	 * @generated
	 * @ordered
	 */
	protected String						type					= TYPE_EDEFAULT;
	
	/**
	 * The default value of the '{@link #isIsExternal() <em>Is External</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #isIsExternal()
	 * @generated
	 * @ordered
	 */
	protected static final boolean			IS_EXTERNAL_EDEFAULT	= false;
	
	/**
	 * The cached value of the '{@link #isIsExternal() <em>Is External</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #isIsExternal()
	 * @generated
	 * @ordered
	 */
	protected boolean						isExternal				= IS_EXTERNAL_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SubstitutionTypeImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.SUBSTITUTION_TYPE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setType(String newType) {
		String oldType = type;
		type = newType;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SUBSTITUTION_TYPE__TYPE, oldType, type));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public boolean isIsExternal() {
		return isExternal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setIsExternal(boolean newIsExternal) {
		boolean oldIsExternal = isExternal;
		isExternal = newIsExternal;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SUBSTITUTION_TYPE__IS_EXTERNAL,
				oldIsExternal, isExternal));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getArrayDimensions() {
		if (arrayDimensions == null) {
			arrayDimensions = new EDataTypeEList<String>(String.class, this,
				SmePackage.SUBSTITUTION_TYPE__ARRAY_DIMENSIONS);
		}
		return arrayDimensions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.SUBSTITUTION_TYPE__ARRAY_DIMENSIONS:
				return getArrayDimensions();
			case SmePackage.SUBSTITUTION_TYPE__TYPE:
				return getType();
			case SmePackage.SUBSTITUTION_TYPE__IS_EXTERNAL:
				return isIsExternal() ? Boolean.TRUE : Boolean.FALSE;
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.SUBSTITUTION_TYPE__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				getArrayDimensions().addAll((Collection<? extends String>) newValue);
				return;
			case SmePackage.SUBSTITUTION_TYPE__TYPE:
				setType((String) newValue);
				return;
			case SmePackage.SUBSTITUTION_TYPE__IS_EXTERNAL:
				setIsExternal(((Boolean) newValue).booleanValue());
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.SUBSTITUTION_TYPE__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				return;
			case SmePackage.SUBSTITUTION_TYPE__TYPE:
				setType(TYPE_EDEFAULT);
				return;
			case SmePackage.SUBSTITUTION_TYPE__IS_EXTERNAL:
				setIsExternal(IS_EXTERNAL_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.SUBSTITUTION_TYPE__ARRAY_DIMENSIONS:
				return arrayDimensions != null && !arrayDimensions.isEmpty();
			case SmePackage.SUBSTITUTION_TYPE__TYPE:
				return TYPE_EDEFAULT == null ? type != null : !TYPE_EDEFAULT.equals(type);
			case SmePackage.SUBSTITUTION_TYPE__IS_EXTERNAL:
				return isExternal != IS_EXTERNAL_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ParameterType.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == SignalType.class) {
			switch (derivedFeatureID) {
				case SmePackage.SUBSTITUTION_TYPE__ARRAY_DIMENSIONS:
					return SmePackage.SIGNAL_TYPE__ARRAY_DIMENSIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ParameterType.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.SUBSTITUTION_TYPE__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == SignalType.class) {
			switch (baseFeatureID) {
				case SmePackage.SIGNAL_TYPE__ARRAY_DIMENSIONS:
					return SmePackage.SUBSTITUTION_TYPE__ARRAY_DIMENSIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (arrayDimensions: ");
		result.append(arrayDimensions);
		result.append(", type: ");
		result.append(type);
		result.append(", isExternal: ");
		result.append(isExternal);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
				result.append("private ");
			result.append("type ");
			result.append(name);
			if (isExternal) {
				result.append(" = ");
				if (getArrayDimensions().size() > 0) {
					result.append("[");
					Iterator<String> it = arrayDimensions.iterator();
					result.append(it.next());
					while (it.hasNext()) {
						result.append(", ");
						result.append(it.next());
					}
					result.append("] ");
				}
				result.append("external \"");
				result.append(type);
				result.append("\"");
			}
			else if (getType() != null && !("".equals(type))) {
				result.append(" = ");
				if (getArrayDimensions().size() > 0) {
					result.append("[");
					Iterator<String> it = arrayDimensions.iterator();
					result.append(it.next());
					while (it.hasNext()) {
						result.append(", ");
						result.append(it.next());
					}
					result.append("] ");
				}
				result.append(type);
			}
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * 
	 * A model declaration does not define equations in the body of its parent, so bodyTree is not modified.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			
			long decl;
			if (isExternal)
				decl = API.makeUnary(SignalOperators.opexte, API.makeNullary(SignalOperators.opnil));
			else {
				if (TYPE_EDEFAULT == type || type.equals(SignalIntrinsicType.UNDEFINED_LITERAL.getLiteral()))
					decl = idNode;
				else {
					if (type.equals(SignalIntrinsicType.BOOLEAN_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.oplogical);
					else if (type.equals(SignalIntrinsicType.CHAR_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opchar);
					else if (type.equals(SignalIntrinsicType.COMPLEX_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opcomplex);
					else if (type.equals(SignalIntrinsicType.DCOMPLEX_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opdcomplex);
					else if (type.equals(SignalIntrinsicType.DREAL_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opdpreal);
					else if (type.equals(SignalIntrinsicType.EVENT_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opevent);
					else if (type.equals(SignalIntrinsicType.INTEGER_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opinteger);
					else if (type.equals(SignalIntrinsicType.LONG_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.oplong);
					else if (type.equals(SignalIntrinsicType.REAL_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opreal);
					else if (type.equals(SignalIntrinsicType.SHORT_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opshort);
					else if (type.equals(SignalIntrinsicType.STRING_LITERAL.getLiteral()))
						decl = API.makeNullary(SignalOperators.opstring);
					else
						decl = API.atomString(SignalOperators.opnom, type);
					
					// Add the specification of arrayDimensions. Parse the textual expression
					if (!getArrayDimensions().isEmpty()) {
						long arrayNode = API.makeEmptyList(SignalOperators.oplesign);
						boolean allOk = true;
						for (String st : arrayDimensions) {
							long exprNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
							if (exprNode == PKTrees.ERROR_NODE) {
								PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.ARRAY_DIMENSION_VALUE + st,
									this);
								allOk = false;
							}
							else
								arrayNode = API.post(arrayNode, exprNode);
						}
						if (allOk)
							decl = API.makeBinary(SignalOperators.optvect, arrayNode, decl);
					}
					decl = API
							.makeTernary(SignalOperators.optype, idNode, decl, API.makeNullary(SignalOperators.opnil));
				}
				if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
					decl = API.makeList(SignalOperators.opltypep, decl);
				else
					decl = API.makeList(SignalOperators.opltype, decl);
			}
			declTree = API.post(declTree, decl);
		}
		return declTree;
	}
	
} // SubstitutionTypeImpl
