/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.CaseConnection;
import fr.irisa.espresso.sme.CaseKind;
import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.DataflowOperator;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.Signal;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.Switch;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Switch</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.SwitchImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SwitchImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SwitchImpl#getSignal <em>Signal</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SwitchImpl#getSrc_CaseConnections <em>Src Case Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.SwitchImpl#getSrcField <em>Src Field</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class SwitchImpl extends EObjectImpl implements Switch {
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String	NAME_EDEFAULT		= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String				name				= NAME_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getSignal() <em>Signal</em>}' reference. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSignal()
	 * @generated
	 * @ordered
	 */
	protected Signal				signal;
	
	/**
	 * The cached value of the '{@link #getSrc_CaseConnections() <em>Src Case Connections</em>}' reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_CaseConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<CaseConnection>	src_CaseConnections;
	
	/**
	 * The default value of the '{@link #getSrcField() <em>Src Field</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSrcField()
	 * @generated
	 * @ordered
	 */
	protected static final String	SRC_FIELD_EDEFAULT	= "";
	
	/**
	 * The cached value of the '{@link #getSrcField() <em>Src Field</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSrcField()
	 * @generated
	 * @ordered
	 */
	protected String				srcField			= SRC_FIELD_EDEFAULT;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected SwitchImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.SWITCH;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SWITCH__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.SWITCH__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.SWITCH__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.SWITCH__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__OPERATORS, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SWITCH__MODEL_PARENT, newModelParent,
				newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Signal getSignal() {
		if (signal != null && signal.eIsProxy()) {
			InternalEObject oldSignal = (InternalEObject) signal;
			signal = (Signal) eResolveProxy(oldSignal);
			if (signal != oldSignal) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.SWITCH__SIGNAL, oldSignal,
						signal));
			}
		}
		return signal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Signal basicGetSignal() {
		return signal;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSignal(Signal newSignal) {
		Signal oldSignal = signal;
		signal = newSignal;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SWITCH__SIGNAL, oldSignal, signal));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<CaseConnection> getSrc_CaseConnections() {
		if (src_CaseConnections == null) {
			src_CaseConnections = new EObjectWithInverseResolvingEList<CaseConnection>(CaseConnection.class, this,
				SmePackage.SWITCH__SRC_CASE_CONNECTIONS, SmePackage.CASE_CONNECTION__SRC);
		}
		return src_CaseConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getSrcField() {
		return srcField;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSrcField(String newSrcField) {
		String oldSrcField = srcField;
		srcField = newSrcField;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.SWITCH__SRC_FIELD, oldSrcField, srcField));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getModelParent();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.SWITCH__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
			case SmePackage.SWITCH__SRC_CASE_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_CaseConnections()).basicAdd(
					otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.SWITCH__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
			case SmePackage.SWITCH__SRC_CASE_CONNECTIONS:
				return ((InternalEList<?>) getSrc_CaseConnections()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.SWITCH__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__OPERATORS, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.SWITCH__NAME:
				return getName();
			case SmePackage.SWITCH__MODEL_PARENT:
				return getModelParent();
			case SmePackage.SWITCH__SIGNAL:
				if (resolve)
					return getSignal();
				return basicGetSignal();
			case SmePackage.SWITCH__SRC_CASE_CONNECTIONS:
				return getSrc_CaseConnections();
			case SmePackage.SWITCH__SRC_FIELD:
				return getSrcField();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.SWITCH__NAME:
				setName((String) newValue);
				return;
			case SmePackage.SWITCH__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.SWITCH__SIGNAL:
				setSignal((Signal) newValue);
				return;
			case SmePackage.SWITCH__SRC_CASE_CONNECTIONS:
				getSrc_CaseConnections().clear();
				getSrc_CaseConnections().addAll((Collection<? extends CaseConnection>) newValue);
				return;
			case SmePackage.SWITCH__SRC_FIELD:
				setSrcField((String) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.SWITCH__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.SWITCH__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.SWITCH__SIGNAL:
				setSignal((Signal) null);
				return;
			case SmePackage.SWITCH__SRC_CASE_CONNECTIONS:
				getSrc_CaseConnections().clear();
				return;
			case SmePackage.SWITCH__SRC_FIELD:
				setSrcField(SRC_FIELD_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.SWITCH__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.SWITCH__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.SWITCH__SIGNAL:
				return signal != null;
			case SmePackage.SWITCH__SRC_CASE_CONNECTIONS:
				return src_CaseConnections != null && !src_CaseConnections.isEmpty();
			case SmePackage.SWITCH__SRC_FIELD:
				return SRC_FIELD_EDEFAULT == null ? srcField != null : !SRC_FIELD_EDEFAULT.equals(srcField);
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == DataflowOperator.class) {
			switch (derivedFeatureID) {
				case SmePackage.SWITCH__MODEL_PARENT:
					return SmePackage.DATAFLOW_OPERATOR__MODEL_PARENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConnectableElement.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == DataflowOperator.class) {
			switch (baseFeatureID) {
				case SmePackage.DATAFLOW_OPERATOR__MODEL_PARENT:
					return SmePackage.SWITCH__MODEL_PARENT;
				default:
					return -1;
			}
		}
		if (baseClass == ConnectableElement.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(", srcField: ");
		result.append(srcField);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		PKTrees API = PkPlugin.getTreeAPI();
		
		boolean severalElse = false;
		CaseConnection elseCase = null;
		long caseNode = API.makeEmptyList(SignalOperators.oplcas);
		
		for (CaseConnection cc : getSrc_CaseConnections()) {
			try {
				if (cc.getCaseKind().equals(CaseKind.ELSE_CASE_LITERAL)) {
					if (elseCase != null) {
						PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.SWITCH_SEVERAL_ELSE + name, cc);
						severalElse = true;
					}
					else
						elseCase = cc;
				}
				else {
					long node = cc.getAST(null);
					if (node != PKTrees.ERROR_NODE)
						caseNode = API.post(caseNode, node);
				}
			}
			catch (CyclicDefinitionException cde) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		
		long elseNode;
		if (elseCase != null) {
			if (severalElse) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.SWITCH_SEVERAL_ELSE + name, elseCase);
				elseNode = API.makeNullary(SignalOperators.opnil);
			}
			else {
				try {
					long node = elseCase.getAST(null);
					if (node != PKTrees.ERROR_NODE)
						elseNode = node;
					else
						elseNode = API.makeNullary(SignalOperators.opnil);
				}
				catch (CyclicDefinitionException cde) {
					elseNode = API.makeNullary(SignalOperators.opnil);
					// cannot be thrown
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
							.getElement());
				}
			}
		}
		else
			elseNode = API.makeNullary(SignalOperators.opnil);
		
		long idNode = PKTrees.ERROR_NODE;
		if (getSignal() != null) {
			if (srcField != null && "".equals(srcField) == false)
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.SWITCH_NOT_IMPLEMENTED, this);
			
			try {
				idNode = signal.getAST(null);
			}
			catch (CyclicDefinitionException cde) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.CYCLE_DETECTION + cde.getElement(), cde
						.getElement());
			}
		}
		
		if (!API.isEmpty(caseNode) || idNode != PKTrees.ERROR_NODE) {
			long result = API.makeTernary(SignalOperators.oppcas, idNode, caseNode, elseNode);
			bodyTree = API.post(bodyTree, result);
		}
		
		return bodyTree;
	}
	/**
	 * This method is not supported. To generate AST, use buildEquationsAST
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		throw new UnsupportedOperationException();
	}
	
	/**
	 * Nothing to declare for a switch operator, so declTree is not modified.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
	
} // SwitchImpl
