/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EDataTypeEList;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ConstantAffectation;
import fr.irisa.espresso.sme.ConstantExpr;
import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.Local;
import fr.irisa.espresso.sme.Module;
import fr.irisa.espresso.sme.ParameterType;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SignalType;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.Specifications;
import fr.irisa.espresso.sme.TupleType;
import fr.irisa.espresso.sme.TypeKind;
import fr.irisa.espresso.sme.VisibilityKind;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Tuple Type</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.TupleTypeImpl#getSrc_ConstantAffectations <em>Src Constant Affectations</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.TupleTypeImpl#getArrayDimensions <em>Array Dimensions</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.TupleTypeImpl#getKind <em>Kind</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.TupleTypeImpl#getLocals <em>Locals</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.TupleTypeImpl#getSpec <em>Spec</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class TupleTypeImpl extends TypeDeclarationImpl implements TupleType {
	/**
	 * The cached value of the '{@link #getSrc_ConstantAffectations() <em>Src Constant Affectations</em>}' reference
	 * list. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_ConstantAffectations()
	 * @generated
	 * @ordered
	 */
	protected EList<ConstantAffectation>	src_ConstantAffectations;
	
	/**
	 * The cached value of the '{@link #getArrayDimensions() <em>Array Dimensions</em>}' attribute list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getArrayDimensions()
	 * @generated
	 * @ordered
	 */
	protected EList<String>					arrayDimensions;
	
	/**
	 * The default value of the '{@link #getKind() <em>Kind</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getKind()
	 * @generated
	 * @ordered
	 */
	protected static final TypeKind			KIND_EDEFAULT	= TypeKind.STRUCT_LITERAL;
	
	/**
	 * The cached value of the '{@link #getKind() <em>Kind</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getKind()
	 * @generated
	 * @ordered
	 */
	protected TypeKind						kind			= KIND_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getLocals() <em>Locals</em>}' containment reference list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getLocals()
	 * @generated
	 * @ordered
	 */
	protected EList<Local>					locals;
	
	/**
	 * The cached value of the '{@link #getSpec() <em>Spec</em>}' containment reference. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getSpec()
	 * @generated
	 * @ordered
	 */
	protected Specifications				spec;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected TupleTypeImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.TUPLE_TYPE;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<ConstantAffectation> getSrc_ConstantAffectations() {
		if (src_ConstantAffectations == null) {
			src_ConstantAffectations = new EObjectWithInverseResolvingEList<ConstantAffectation>(
				ConstantAffectation.class, this, SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS,
				SmePackage.CONSTANT_AFFECTATION__SRC);
		}
		return src_ConstantAffectations;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public TypeKind getKind() {
		return kind;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setKind(TypeKind newKind) {
		TypeKind oldKind = kind;
		kind = newKind == null ? KIND_EDEFAULT : newKind;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.TUPLE_TYPE__KIND, oldKind, kind));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getArrayDimensions() {
		if (arrayDimensions == null) {
			arrayDimensions = new EDataTypeEList<String>(String.class, this, SmePackage.TUPLE_TYPE__ARRAY_DIMENSIONS);
		}
		return arrayDimensions;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<Local> getLocals() {
		if (locals == null) {
			locals = new EObjectContainmentWithInverseEList<Local>(Local.class, this, SmePackage.TUPLE_TYPE__LOCALS,
				SmePackage.LOCAL__TYPE_PARENT);
		}
		return locals;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Specifications getSpec() {
		return spec;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSpec(Specifications newSpec, NotificationChain msgs) {
		Specifications oldSpec = spec;
		spec = newSpec;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET, SmePackage.TUPLE_TYPE__SPEC,
				oldSpec, newSpec);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSpec(Specifications newSpec) {
		if (newSpec != spec) {
			NotificationChain msgs = null;
			if (spec != null)
				msgs = ((InternalEObject) spec).eInverseRemove(this, SmePackage.SPECIFICATIONS__TYPE_PARENT,
					Specifications.class, msgs);
			if (newSpec != null)
				msgs = ((InternalEObject) newSpec).eInverseAdd(this, SmePackage.SPECIFICATIONS__TYPE_PARENT,
					Specifications.class, msgs);
			msgs = basicSetSpec(newSpec, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.TUPLE_TYPE__SPEC, newSpec, newSpec));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<ConstantOrRef> getVisibleConstants() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleConstants();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleModelProcesses();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		return new BasicEList<SignalOrRef>(this.getLocals());
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<Identifier> getVisibleTypes() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleTypes();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_ConstantAffectations()).basicAdd(
					otherEnd, msgs);
			case SmePackage.TUPLE_TYPE__LOCALS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getLocals()).basicAdd(otherEnd, msgs);
			case SmePackage.TUPLE_TYPE__SPEC:
				if (spec != null)
					msgs = ((InternalEObject) spec).eInverseRemove(this, EOPPOSITE_FEATURE_BASE
							- SmePackage.TUPLE_TYPE__SPEC, null, msgs);
				return basicSetSpec((Specifications) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return ((InternalEList<?>) getSrc_ConstantAffectations()).basicRemove(otherEnd, msgs);
			case SmePackage.TUPLE_TYPE__LOCALS:
				return ((InternalEList<?>) getLocals()).basicRemove(otherEnd, msgs);
			case SmePackage.TUPLE_TYPE__SPEC:
				return basicSetSpec(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return getSrc_ConstantAffectations();
			case SmePackage.TUPLE_TYPE__ARRAY_DIMENSIONS:
				return getArrayDimensions();
			case SmePackage.TUPLE_TYPE__KIND:
				return getKind();
			case SmePackage.TUPLE_TYPE__LOCALS:
				return getLocals();
			case SmePackage.TUPLE_TYPE__SPEC:
				return getSpec();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				getSrc_ConstantAffectations().addAll((Collection<? extends ConstantAffectation>) newValue);
				return;
			case SmePackage.TUPLE_TYPE__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				getArrayDimensions().addAll((Collection<? extends String>) newValue);
				return;
			case SmePackage.TUPLE_TYPE__KIND:
				setKind((TypeKind) newValue);
				return;
			case SmePackage.TUPLE_TYPE__LOCALS:
				getLocals().clear();
				getLocals().addAll((Collection<? extends Local>) newValue);
				return;
			case SmePackage.TUPLE_TYPE__SPEC:
				setSpec((Specifications) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS:
				getSrc_ConstantAffectations().clear();
				return;
			case SmePackage.TUPLE_TYPE__ARRAY_DIMENSIONS:
				getArrayDimensions().clear();
				return;
			case SmePackage.TUPLE_TYPE__KIND:
				setKind(KIND_EDEFAULT);
				return;
			case SmePackage.TUPLE_TYPE__LOCALS:
				getLocals().clear();
				return;
			case SmePackage.TUPLE_TYPE__SPEC:
				setSpec((Specifications) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS:
				return src_ConstantAffectations != null && !src_ConstantAffectations.isEmpty();
			case SmePackage.TUPLE_TYPE__ARRAY_DIMENSIONS:
				return arrayDimensions != null && !arrayDimensions.isEmpty();
			case SmePackage.TUPLE_TYPE__KIND:
				return kind != KIND_EDEFAULT;
			case SmePackage.TUPLE_TYPE__LOCALS:
				return locals != null && !locals.isEmpty();
			case SmePackage.TUPLE_TYPE__SPEC:
				return spec != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == SignalContainer.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ParameterType.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == SignalType.class) {
			switch (derivedFeatureID) {
				case SmePackage.TUPLE_TYPE__ARRAY_DIMENSIONS:
					return SmePackage.SIGNAL_TYPE__ARRAY_DIMENSIONS;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == SignalContainer.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ParameterType.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == ConstantExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.CONSTANT_EXPR__SRC_CONSTANT_AFFECTATIONS:
					return SmePackage.TUPLE_TYPE__SRC_CONSTANT_AFFECTATIONS;
				default:
					return -1;
			}
		}
		if (baseClass == SignalType.class) {
			switch (baseFeatureID) {
				case SmePackage.SIGNAL_TYPE__ARRAY_DIMENSIONS:
					return SmePackage.TUPLE_TYPE__ARRAY_DIMENSIONS;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (arrayDimensions: ");
		result.append(arrayDimensions);
		result.append(", kind: ");
		result.append(kind);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.Identifier#toString(boolean, boolean)
	 * @generated NOT
	 */
	public String toString(boolean asCode, boolean withComment) {
		if (asCode) {
			StringBuffer result = new StringBuffer();
			if (withComment && comment != null && ("".equals(comment) == false))
				result.append(comment + "\n\n");
			if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
				result.append("private ");
			result.append("type ");
			result.append(name);
			result.append(" = ");
			if (getArrayDimensions().size() > 0 && kind.equals(TypeKind.STRUCT_LITERAL)) {
				result.append("[");
				Iterator<String> it = arrayDimensions.iterator();
				result.append(it.next());
				while (it.hasNext()) {
					result.append(", ");
					result.append(it.next());
				}
				result.append("] ");
			}
			result.append(kind);
			result.append("{\n");
			for (Local l : getLocals()) {
				result.append(l.toString(asCode, false));
				result.append(";\n");
			}
			result.append("}");
			if (getKind().equals(TypeKind.BUNDLE_LITERAL))
				result.append(" spec (| ... |)");
			return result.toString();
		}
		else
			return toString();
	}
	
	/**
	 * 
	 * A tuple type declaration does not define equations in the body of its parent, so bodyTree is not modified.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (name != null) {
			long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, name);
			if (idNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
				result = PKTrees.ERROR_NODE;
			}
			else {
				result = idNode;
			}
		}
		else {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.NO_NAME, this);
			result = PKTrees.ERROR_NODE;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		long idNode = getAST(null);
		if (idNode != PKTrees.ERROR_NODE) {
			PKTrees API = PkPlugin.getTreeAPI();
			
			long decl;
			if (!getLocals().isEmpty()) {
				long tupleNode = API.makeEmptyList(SignalOperators.opldecs);
				for (Local l : getLocals())
					tupleNode = l.getDeclarationAST(tupleNode);
				
				if (kind.equals(TypeKind.STRUCT_LITERAL))
					tupleNode = API.makeUnary(SignalOperators.opstru, tupleNode);
				else {
					long specNode;
					if (spec != null)
						specNode = spec.buildEquationsAST(PKTrees.ERROR_NODE);
					else
						specNode = API.makeNullary(SignalOperators.opnil);
					tupleNode = API.makeBinary(SignalOperators.opbund, tupleNode, specNode);
				}
				
				// Add the specification of arrayDimensions. Parse the textual expression
				if (!getArrayDimensions().isEmpty()) {
					long arrayNode = API.makeEmptyList(SignalOperators.oplesign);
					boolean allOk = true;
					for (String st : arrayDimensions) {
						long exprNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
						if (exprNode == PKTrees.ERROR_NODE) {
							PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.ARRAY_DIMENSION_VALUE + st, this);
							allOk = false;
						}
						else
							arrayNode = API.post(arrayNode, exprNode);
					}
					if (allOk)
						tupleNode = API.makeBinary(SignalOperators.optvect, arrayNode, tupleNode);
				}
				
				decl = API.makeTernary(SignalOperators.optype, idNode, tupleNode, API
						.makeNullary(SignalOperators.opnil));
			}
			else
				decl = idNode;
			
			if (visibility.equals(VisibilityKind.PRIVATE_LITERAL) && getParent() instanceof Module)
				decl = API.makeList(SignalOperators.opltypep, decl);
			else
				decl = API.makeList(SignalOperators.opltype, decl);
			
			declTree = API.post(declTree, decl);
		}
		
		return declTree;
	}
	
} // TupleTypeImpl
