/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;

import fr.irisa.espresso.sme.ConstantOrRef;
import fr.irisa.espresso.sme.Identifier;
import fr.irisa.espresso.sme.InputPort;
import fr.irisa.espresso.sme.InterfaceDefinition;
import fr.irisa.espresso.sme.NamedElement;
import fr.irisa.espresso.sme.OperatorPort;
import fr.irisa.espresso.sme.OutputPort;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalOrRef;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.TwoInputsOperator;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Two Inputs Operator</b></em>'. <!--
 * end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.TwoInputsOperatorImpl#getName <em>Name</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.TwoInputsOperatorImpl#getFirstOperand <em>First Operand</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.TwoInputsOperatorImpl#getSecondOperand <em>Second Operand</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public abstract class TwoInputsOperatorImpl extends EObjectImpl implements TwoInputsOperator {
	/**
	 * The default value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected static final String	NAME_EDEFAULT	= null;
	
	/**
	 * The cached value of the '{@link #getName() <em>Name</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc
	 * -->
	 * 
	 * @see #getName()
	 * @generated
	 * @ordered
	 */
	protected String				name			= NAME_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getFirstOperand() <em>First Operand</em>}' containment reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getFirstOperand()
	 * @generated
	 * @ordered
	 */
	protected InputPort				firstOperand;
	
	/**
	 * The cached value of the '{@link #getSecondOperand() <em>Second Operand</em>}' containment reference. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSecondOperand()
	 * @generated
	 * @ordered
	 */
	protected InputPort				secondOperand;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected TwoInputsOperatorImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.TWO_INPUTS_OPERATOR;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setName(String newName) {
		String oldName = name;
		name = newName;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.TWO_INPUTS_OPERATOR__NAME, oldName, name));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public InputPort getFirstOperand() {
		return firstOperand;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetFirstOperand(InputPort newFirstOperand, NotificationChain msgs) {
		InputPort oldFirstOperand = firstOperand;
		firstOperand = newFirstOperand;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND, oldFirstOperand, newFirstOperand);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setFirstOperand(InputPort newFirstOperand) {
		if (newFirstOperand != firstOperand) {
			NotificationChain msgs = null;
			if (firstOperand != null)
				msgs = ((InternalEObject) firstOperand).eInverseRemove(this, EOPPOSITE_FEATURE_BASE
						- SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND, null, msgs);
			if (newFirstOperand != null)
				msgs = ((InternalEObject) newFirstOperand).eInverseAdd(this, EOPPOSITE_FEATURE_BASE
						- SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND, null, msgs);
			msgs = basicSetFirstOperand(newFirstOperand, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND,
				newFirstOperand, newFirstOperand));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public InputPort getSecondOperand() {
		return secondOperand;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSecondOperand(InputPort newSecondOperand, NotificationChain msgs) {
		InputPort oldSecondOperand = secondOperand;
		secondOperand = newSecondOperand;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND, oldSecondOperand, newSecondOperand);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSecondOperand(InputPort newSecondOperand) {
		if (newSecondOperand != secondOperand) {
			NotificationChain msgs = null;
			if (secondOperand != null)
				msgs = ((InternalEObject) secondOperand).eInverseRemove(this, EOPPOSITE_FEATURE_BASE
						- SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND, null, msgs);
			if (newSecondOperand != null)
				msgs = ((InternalEObject) newSecondOperand).eInverseAdd(this, EOPPOSITE_FEATURE_BASE
						- SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND, null, msgs);
			msgs = basicSetSecondOperand(newSecondOperand, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND,
				newSecondOperand, newSecondOperand));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public String getPortName(OperatorPort port) {
		String tmp = "";
		if (name != null)
			tmp = name;
		if (getFirstOperand().equals(port))
			return tmp += "_in0";
		if (getSecondOperand().equals(port))
			return tmp += "_in1";
		return null;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<InputPort> getInputPorts() {
		EList<InputPort> list = new BasicEList<InputPort>();
		InputPort in = getFirstOperand();
		if (in != null)
			list.add(in);
		in = getSecondOperand();
		if (in != null)
			list.add(in);
		return list;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<OutputPort> getOutputPorts() {
		return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<ConstantOrRef> getVisibleConstants() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleConstants();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<InterfaceDefinition> getVisibleModelProcesses() {
		// NO NEED for an operator to see Model Processes
		return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<SignalOrRef> getVisibleSignals() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleSignals();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public EList<Identifier> getVisibleTypes() {
		SignalContainer parent = getParent();
		if (parent != null)
			return parent.getVisibleTypes();
		else
			return ECollections.emptyEList();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public abstract SignalContainer getParent();
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public abstract long getAST(Set<SignalElement> path) throws CyclicDefinitionException;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND:
				return basicSetFirstOperand(null, msgs);
			case SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND:
				return basicSetSecondOperand(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.TWO_INPUTS_OPERATOR__NAME:
				return getName();
			case SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND:
				return getFirstOperand();
			case SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND:
				return getSecondOperand();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.TWO_INPUTS_OPERATOR__NAME:
				setName((String) newValue);
				return;
			case SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND:
				setFirstOperand((InputPort) newValue);
				return;
			case SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND:
				setSecondOperand((InputPort) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.TWO_INPUTS_OPERATOR__NAME:
				setName(NAME_EDEFAULT);
				return;
			case SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND:
				setFirstOperand((InputPort) null);
				return;
			case SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND:
				setSecondOperand((InputPort) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.TWO_INPUTS_OPERATOR__NAME:
				return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
			case SmePackage.TWO_INPUTS_OPERATOR__FIRST_OPERAND:
				return firstOperand != null;
			case SmePackage.TWO_INPUTS_OPERATOR__SECOND_OPERAND:
				return secondOperand != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (derivedFeatureID) {
				case SmePackage.TWO_INPUTS_OPERATOR__NAME:
					return SmePackage.NAMED_ELEMENT__NAME;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == NamedElement.class) {
			switch (baseFeatureID) {
				case SmePackage.NAMED_ELEMENT__NAME:
					return SmePackage.TWO_INPUTS_OPERATOR__NAME;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (name: ");
		result.append(name);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#buildEquationsAST(int)
	 * @generated NOT
	 */
	public long buildEquationsAST(long bodyTree) {
		return bodyTree;
	}
	
	/**
	 * 
	 * @param path
	 * @param opCode
	 * @return
	 * @throws CyclicDefinitionException
	 * @generated NOT
	 */
	protected long getAST(Set<SignalElement> path, int opCode) throws CyclicDefinitionException {
		if (path.contains(this))
			throw new CyclicDefinitionException(this);
		path.add(this);
		
		long firstNode = getFirstOperand().getAST(path);
		long secondNode = getSecondOperand().getAST(path);
		
		long result;
		if (firstNode == PKTrees.ERROR_NODE || secondNode == PKTrees.ERROR_NODE) {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.TWO_INPUTS_OPERATOR_BRANCHES + name, this);
			result = PKTrees.ERROR_NODE;
		}
		else
			result = PkPlugin.getTreeAPI().makeBinary(opCode, firstNode, secondNode);
		
		path.remove(this);
		return result;
	}
	
	/**
	 * An operator has no declaration AST, so declTree is not modified.
	 * 
	 * @see fr.irisa.espresso.sme.SignalElement#getDeclarationAST(int)
	 * @generated NOT
	 */
	public long getDeclarationAST(long declTree) {
		return declTree;
	}
	
} // TwoInputsOperatorImpl
