/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.StateBase;
import fr.irisa.espresso.sme.StateOrHistory;
import fr.irisa.espresso.sme.WeakTransition;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Weak Transition</b></em>'. <!-- end-user-doc
 * -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.WeakTransitionImpl#getSrc <em>Src</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.WeakTransitionImpl#getDst <em>Dst</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class WeakTransitionImpl extends TransitionImpl implements WeakTransition {
	/**
	 * The cached value of the '{@link #getSrc() <em>Src</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc()
	 * @generated
	 * @ordered
	 */
	protected StateBase			src;
	
	/**
	 * The cached value of the '{@link #getDst() <em>Dst</em>}' reference. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getDst()
	 * @generated
	 * @ordered
	 */
	protected StateOrHistory	dst;
	
	/**
	 * Name of the transition signal
	 * 
	 * @generated NOT
	 */
	private String				sigName				= null;
	
	/**
	 * Count the number of transitions
	 * 
	 * @generated NOT
	 */
	private static int			countWeakTransition	= 0;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected WeakTransitionImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.WEAK_TRANSITION;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public StateBase getSrc() {
		if (src != null && src.eIsProxy()) {
			InternalEObject oldSrc = (InternalEObject) src;
			src = (StateBase) eResolveProxy(oldSrc);
			if (src != oldSrc) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.WEAK_TRANSITION__SRC, oldSrc,
						src));
			}
		}
		return src;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public StateBase basicGetSrc() {
		return src;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetSrc(StateBase newSrc, NotificationChain msgs) {
		StateBase oldSrc = src;
		src = newSrc;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.WEAK_TRANSITION__SRC, oldSrc, newSrc);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setSrc(StateBase newSrc) {
		if (newSrc != src) {
			NotificationChain msgs = null;
			if (src != null)
				msgs = ((InternalEObject) src).eInverseRemove(this, SmePackage.STATE_BASE__SRC_WEAK_TRANSITIONS,
					StateBase.class, msgs);
			if (newSrc != null)
				msgs = ((InternalEObject) newSrc).eInverseAdd(this, SmePackage.STATE_BASE__SRC_WEAK_TRANSITIONS,
					StateBase.class, msgs);
			msgs = basicSetSrc(newSrc, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.WEAK_TRANSITION__SRC, newSrc, newSrc));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public StateOrHistory getDst() {
		if (dst != null && dst.eIsProxy()) {
			InternalEObject oldDst = (InternalEObject) dst;
			dst = (StateOrHistory) eResolveProxy(oldDst);
			if (dst != oldDst) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SmePackage.WEAK_TRANSITION__DST, oldDst,
						dst));
			}
		}
		return dst;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public StateOrHistory basicGetDst() {
		return dst;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetDst(StateOrHistory newDst, NotificationChain msgs) {
		StateOrHistory oldDst = dst;
		dst = newDst;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET,
				SmePackage.WEAK_TRANSITION__DST, oldDst, newDst);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setDst(StateOrHistory newDst) {
		if (newDst != dst) {
			NotificationChain msgs = null;
			if (dst != null)
				msgs = ((InternalEObject) dst).eInverseRemove(this, SmePackage.STATE_OR_HISTORY__DST_WEAK_TRANSITIONS,
					StateOrHistory.class, msgs);
			if (newDst != null)
				msgs = ((InternalEObject) newDst).eInverseAdd(this, SmePackage.STATE_OR_HISTORY__DST_WEAK_TRANSITIONS,
					StateOrHistory.class, msgs);
			msgs = basicSetDst(newDst, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.WEAK_TRANSITION__DST, newDst, newDst));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.WEAK_TRANSITION__SRC:
				if (src != null)
					msgs = ((InternalEObject) src).eInverseRemove(this, SmePackage.STATE_BASE__SRC_WEAK_TRANSITIONS,
						StateBase.class, msgs);
				return basicSetSrc((StateBase) otherEnd, msgs);
			case SmePackage.WEAK_TRANSITION__DST:
				if (dst != null)
					msgs = ((InternalEObject) dst).eInverseRemove(this,
						SmePackage.STATE_OR_HISTORY__DST_WEAK_TRANSITIONS, StateOrHistory.class, msgs);
				return basicSetDst((StateOrHistory) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.WEAK_TRANSITION__SRC:
				return basicSetSrc(null, msgs);
			case SmePackage.WEAK_TRANSITION__DST:
				return basicSetDst(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.WEAK_TRANSITION__SRC:
				if (resolve)
					return getSrc();
				return basicGetSrc();
			case SmePackage.WEAK_TRANSITION__DST:
				if (resolve)
					return getDst();
				return basicGetDst();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.WEAK_TRANSITION__SRC:
				setSrc((StateBase) newValue);
				return;
			case SmePackage.WEAK_TRANSITION__DST:
				setDst((StateOrHistory) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.WEAK_TRANSITION__SRC:
				setSrc((StateBase) null);
				return;
			case SmePackage.WEAK_TRANSITION__DST:
				setDst((StateOrHistory) null);
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.WEAK_TRANSITION__SRC:
				return src != null;
			case SmePackage.WEAK_TRANSITION__DST:
				return dst != null;
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#getDestination()
	 * @generated NOT
	 */
	@Override
	public ConnectableElement getDestination() {
		return getDst();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#getSource()
	 * @generated NOT
	 */
	@Override
	public ConnectableElement getSource() {
		return getSrc();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) {
		long result = PKTrees.ERROR_NODE;
		if (sigName == null)
			sigName = "__WT_" + countWeakTransition++ + "_" + name + "_P_" + getPriority();
		long idNode = PkPlugin.getParser().parseString(Phyla.ph_ident, sigName);
		if (idNode == PKTrees.ERROR_NODE) {
			PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.INCORRECT_NAME + name, this);
			result = PKTrees.ERROR_NODE;
		}
		else {
			result = idNode;
		}
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		return result;
	}
	
	/**
	 * @see fr.irisa.espresso.sme.impl.ConnectionImpl#buildEquationsAST(int)
	 * @generated NOT
	 */
	@Override
	public long buildEquationsAST(long bodyTree) {
		return buildEquationsAST1(bodyTree, false);
	}
} // WeakTransitionImpl
