/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.impl;

import java.util.Collection;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EDataTypeEList;
import org.eclipse.emf.ecore.util.EObjectWithInverseResolvingEList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.irisa.espresso.sme.ConnectableElement;
import fr.irisa.espresso.sme.DataflowConnection;
import fr.irisa.espresso.sme.DataflowOperator;
import fr.irisa.espresso.sme.Model;
import fr.irisa.espresso.sme.SignalContainer;
import fr.irisa.espresso.sme.SignalElement;
import fr.irisa.espresso.sme.SignalExpr;
import fr.irisa.espresso.sme.SmePackage;
import fr.irisa.espresso.sme.Window;
import fr.irisa.espresso.sme.polychrony.CyclicDefinitionException;
import fr.irisa.espresso.sme.polychrony.PKTrees;
import fr.irisa.espresso.sme.polychrony.Phyla;
import fr.irisa.espresso.sme.polychrony.PkPlugin;
import fr.irisa.espresso.sme.polychrony.SignalOperators;
import fr.irisa.espresso.sme.polychrony.traceability.ErrorMessages;
import fr.irisa.espresso.sme.polychrony.traceability.WarningMessages;
import fr.irisa.espresso.sme.polychrony.utils.PolychronyDiagnostic;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Window</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 * <li>{@link fr.irisa.espresso.sme.impl.WindowImpl#getSrc_DataflowConnections <em>Src Dataflow Connections</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.WindowImpl#getModelParent <em>Model Parent</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.WindowImpl#getWindowSize <em>Window Size</em>}</li>
 * <li>{@link fr.irisa.espresso.sme.impl.WindowImpl#getInitialValue <em>Initial Value</em>}</li>
 * </ul>
 * </p>
 * 
 * @generated
 */
public class WindowImpl extends SingleInputOperatorImpl implements Window {
	/**
	 * The cached value of the '{@link #getSrc_DataflowConnections() <em>Src Dataflow Connections</em>}' reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getSrc_DataflowConnections()
	 * @generated
	 * @ordered
	 */
	protected EList<DataflowConnection>	src_DataflowConnections;
	
	/**
	 * The default value of the '{@link #getWindowSize() <em>Window Size</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getWindowSize()
	 * @generated
	 * @ordered
	 */
	protected static final String		WINDOW_SIZE_EDEFAULT	= "1";
	
	/**
	 * The cached value of the '{@link #getWindowSize() <em>Window Size</em>}' attribute. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @see #getWindowSize()
	 * @generated
	 * @ordered
	 */
	protected String					windowSize				= WINDOW_SIZE_EDEFAULT;
	
	/**
	 * The cached value of the '{@link #getInitialValue() <em>Initial Value</em>}' attribute list. <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @see #getInitialValue()
	 * @generated
	 * @ordered
	 */
	protected EList<String>				initialValue;
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected WindowImpl() {
		super();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SmePackage.Literals.WINDOW;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<DataflowConnection> getSrc_DataflowConnections() {
		if (src_DataflowConnections == null) {
			src_DataflowConnections = new EObjectWithInverseResolvingEList<DataflowConnection>(
				DataflowConnection.class, this, SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS,
				SmePackage.DATAFLOW_CONNECTION__SRC);
		}
		return src_DataflowConnections;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public Model getModelParent() {
		if (eContainerFeatureID != SmePackage.WINDOW__MODEL_PARENT)
			return null;
		return (Model) eContainer();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public NotificationChain basicSetModelParent(Model newModelParent, NotificationChain msgs) {
		msgs = eBasicSetContainer((InternalEObject) newModelParent, SmePackage.WINDOW__MODEL_PARENT, msgs);
		return msgs;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setModelParent(Model newModelParent) {
		if (newModelParent != eInternalContainer()
				|| (eContainerFeatureID != SmePackage.WINDOW__MODEL_PARENT && newModelParent != null)) {
			if (EcoreUtil.isAncestor(this, newModelParent))
				throw new IllegalArgumentException("Recursive containment not allowed for " + toString());
			NotificationChain msgs = null;
			if (eInternalContainer() != null)
				msgs = eBasicRemoveFromContainer(msgs);
			if (newModelParent != null)
				msgs = ((InternalEObject) newModelParent).eInverseAdd(this, SmePackage.MODEL__OPERATORS, Model.class,
					msgs);
			msgs = basicSetModelParent(newModelParent, msgs);
			if (msgs != null)
				msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.WINDOW__MODEL_PARENT, newModelParent,
				newModelParent));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public String getWindowSize() {
		return windowSize;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public void setWindowSize(String newWindowSize) {
		String oldWindowSize = windowSize;
		windowSize = newWindowSize;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SmePackage.WINDOW__WINDOW_SIZE, oldWindowSize,
				windowSize));
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public EList<String> getInitialValue() {
		if (initialValue == null) {
			initialValue = new EDataTypeEList<String>(String.class, this, SmePackage.WINDOW__INITIAL_VALUE);
		}
		return initialValue;
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<InternalEObject>) (InternalEList<?>) getSrc_DataflowConnections()).basicAdd(
					otherEnd, msgs);
			case SmePackage.WINDOW__MODEL_PARENT:
				if (eInternalContainer() != null)
					msgs = eBasicRemoveFromContainer(msgs);
				return basicSetModelParent((Model) otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS:
				return ((InternalEList<?>) getSrc_DataflowConnections()).basicRemove(otherEnd, msgs);
			case SmePackage.WINDOW__MODEL_PARENT:
				return basicSetModelParent(null, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public NotificationChain eBasicRemoveFromContainerFeature(NotificationChain msgs) {
		switch (eContainerFeatureID) {
			case SmePackage.WINDOW__MODEL_PARENT:
				return eInternalContainer().eInverseRemove(this, SmePackage.MODEL__OPERATORS, Model.class, msgs);
		}
		return super.eBasicRemoveFromContainerFeature(msgs);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS:
				return getSrc_DataflowConnections();
			case SmePackage.WINDOW__MODEL_PARENT:
				return getModelParent();
			case SmePackage.WINDOW__WINDOW_SIZE:
				return getWindowSize();
			case SmePackage.WINDOW__INITIAL_VALUE:
				return getInitialValue();
		}
		return super.eGet(featureID, resolve, coreType);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				getSrc_DataflowConnections().addAll((Collection<? extends DataflowConnection>) newValue);
				return;
			case SmePackage.WINDOW__MODEL_PARENT:
				setModelParent((Model) newValue);
				return;
			case SmePackage.WINDOW__WINDOW_SIZE:
				setWindowSize((String) newValue);
				return;
			case SmePackage.WINDOW__INITIAL_VALUE:
				getInitialValue().clear();
				getInitialValue().addAll((Collection<? extends String>) newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS:
				getSrc_DataflowConnections().clear();
				return;
			case SmePackage.WINDOW__MODEL_PARENT:
				setModelParent((Model) null);
				return;
			case SmePackage.WINDOW__WINDOW_SIZE:
				setWindowSize(WINDOW_SIZE_EDEFAULT);
				return;
			case SmePackage.WINDOW__INITIAL_VALUE:
				getInitialValue().clear();
				return;
		}
		super.eUnset(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS:
				return src_DataflowConnections != null && !src_DataflowConnections.isEmpty();
			case SmePackage.WINDOW__MODEL_PARENT:
				return getModelParent() != null;
			case SmePackage.WINDOW__WINDOW_SIZE:
				return WINDOW_SIZE_EDEFAULT == null ? windowSize != null : !WINDOW_SIZE_EDEFAULT.equals(windowSize);
			case SmePackage.WINDOW__INITIAL_VALUE:
				return initialValue != null && !initialValue.isEmpty();
		}
		return super.eIsSet(featureID);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eBaseStructuralFeatureID(int derivedFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (derivedFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == SignalExpr.class) {
			switch (derivedFeatureID) {
				case SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS:
					return SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowOperator.class) {
			switch (derivedFeatureID) {
				case SmePackage.WINDOW__MODEL_PARENT:
					return SmePackage.DATAFLOW_OPERATOR__MODEL_PARENT;
				default:
					return -1;
			}
		}
		return super.eBaseStructuralFeatureID(derivedFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public int eDerivedStructuralFeatureID(int baseFeatureID, Class<?> baseClass) {
		if (baseClass == ConnectableElement.class) {
			switch (baseFeatureID) {
				default:
					return -1;
			}
		}
		if (baseClass == SignalExpr.class) {
			switch (baseFeatureID) {
				case SmePackage.SIGNAL_EXPR__SRC_DATAFLOW_CONNECTIONS:
					return SmePackage.WINDOW__SRC_DATAFLOW_CONNECTIONS;
				default:
					return -1;
			}
		}
		if (baseClass == DataflowOperator.class) {
			switch (baseFeatureID) {
				case SmePackage.DATAFLOW_OPERATOR__MODEL_PARENT:
					return SmePackage.WINDOW__MODEL_PARENT;
				default:
					return -1;
			}
		}
		return super.eDerivedStructuralFeatureID(baseFeatureID, baseClass);
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy())
			return super.toString();
		
		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (windowSize: ");
		result.append(windowSize);
		result.append(", initialValue: ");
		result.append(initialValue);
		result.append(')');
		return result.toString();
	}
	
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public SignalContainer getParent() {
		return getModelParent();
	}
	
	/**
	 * @see fr.irisa.espresso.sme.SignalElement#getAST(java.util.Set)
	 * @generated NOT
	 */
	public long getAST(Set<SignalElement> path) throws CyclicDefinitionException {
		if (path.contains(this))
			throw new CyclicDefinitionException(this);
		path.add(this);
		
		long nbNode;
		if (windowSize == null || "".equals(windowSize)) {
			nbNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, WINDOW_SIZE_EDEFAULT);
			PolychronyDiagnostic.add(Diagnostic.WARNING, WarningMessages.WINDOW_UNDEFINED_SIZE + WINDOW_SIZE_EDEFAULT,
				this);
		}
		else {
			nbNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, windowSize);
			if (nbNode == PKTrees.ERROR_NODE) {
				PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.WINDOW_INCORRECT_SIZE_ATTRIBUTE + name, this);
				nbNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, WINDOW_SIZE_EDEFAULT);
			}
		}
		
		PKTrees API = PkPlugin.getTreeAPI();
		long initNode;
		int size = getInitialValue().size();
		if (size > 0) {
			if (size > 1) {
				boolean allOk = true;
				initNode = API.makeEmptyList(SignalOperators.oplevect);
				// Value represents an array
				for (String st : initialValue) {
					long tmp = PkPlugin.getParser().parseString(Phyla.ph_e_sign, st);
					if (tmp == PKTrees.ERROR_NODE) {
						PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.OPERATOR_INITIAL_VALUE + st, this);
						allOk = false;
					}
					else
						initNode = API.post(initNode, tmp);
				}
				if (!allOk)
					initNode = API.makeNullary(SignalOperators.opnil);
			}
			else {
				// Value is a simple value
				initNode = PkPlugin.getParser().parseString(Phyla.ph_e_sign, initialValue.get(0));
				if (initNode == PKTrees.ERROR_NODE) {
					PolychronyDiagnostic.add(Diagnostic.ERROR, ErrorMessages.OPERATOR_INITIAL_VALUE + name, this);
					initNode = API.makeNullary(SignalOperators.opnil);
				}
			}
		}
		else
			initNode = API.makeNullary(SignalOperators.opnil);
		
		long result;
		long node = getIn().getAST(path);
		if (node == PKTrees.ERROR_NODE)
			result = PKTrees.ERROR_NODE;
		else
			result = API.makeTernary(SignalOperators.opfen, node, nbNode, initNode);
		
		path.remove(this);
		
		PkPlugin.getTraceabilityAPI().getInterfaceMapSMESignal().put(this, result);
		
		return result;
	}
	
} // WindowImpl
