/*
 * Authors: Loïc Besnard, Christian Brunette, Jean-Pierre Talpin. Copyright (C) by INRIA-CNRS - developed by Espresso
 * project - EPL License - 2010.
 * 
 * Contact: polychronycontact@listes.irisa.fr
 * 
 * This file is part of Polychrony SME Platform (website: http://www.irisa.fr/espresso/polychrony).
 * 
 * Polychrony SME Platform is free software; you can redistribute it and/or modify it under the terms of the EPL as
 * published by the Eclipse Foundation.
 * 
 * Polychrony SME Platform are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License - v 1.0
 * for more details.
 * 
 * You should have received a copy of the Eclipse Public License - v 1.0 along with Polychrony SME Platform; see the
 * file COPYING. If not, consult the url http://www.eclipse.org/org/documents/epl-v10.php, or write to the Eclipse
 * Foundation, Inc. 102 Centrepointe Drive. Ottawa, Ontario,Canada, K2G 6B1.
 */

package fr.irisa.espresso.sme.util;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

import fr.irisa.espresso.sme.*;

/**
 * <!-- begin-user-doc --> The <b>Switch</b> for the model's inheritance hierarchy. It supports the call
 * {@link #doSwitch(EObject) doSwitch(object)} to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object and proceeding up the inheritance hierarchy until a non-null result is
 * returned, which is the result of the switch. <!-- end-user-doc -->
 * 
 * @see fr.irisa.espresso.sme.SmePackage
 * @generated
 */
public class SmeSwitch<T> {
	/**
	 * The cached model package <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	protected static SmePackage	modelPackage;
	
	/**
	 * Creates an instance of the switch. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public SmeSwitch() {
		if (modelPackage == null) {
			modelPackage = SmePackage.eINSTANCE;
		}
	}
	
	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that
	 * result. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	public T doSwitch(EObject theEObject) {
		return doSwitch(theEObject.eClass(), theEObject);
	}
	
	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that
	 * result. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected T doSwitch(EClass theEClass, EObject theEObject) {
		if (theEClass.eContainer() == modelPackage) {
			return doSwitch(theEClass.getClassifierID(), theEObject);
		}
		else {
			List<EClass> eSuperTypes = theEClass.getESuperTypes();
			return eSuperTypes.isEmpty() ? defaultCase(theEObject) : doSwitch(eSuperTypes.get(0), theEObject);
		}
	}
	
	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that
	 * result. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected T doSwitch(int classifierID, EObject theEObject) {
		switch (classifierID) {
			case SmePackage.SIGNAL_ELEMENT: {
				SignalElement signalElement = (SignalElement) theEObject;
				T result = caseSignalElement(signalElement);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ROOT_MODEL: {
				RootModel rootModel = (RootModel) theEObject;
				T result = caseRootModel(rootModel);
				if (result == null)
					result = caseSignalElement(rootModel);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONNECTABLE_ELEMENT: {
				ConnectableElement connectableElement = (ConnectableElement) theEObject;
				T result = caseConnectableElement(connectableElement);
				if (result == null)
					result = caseSignalElement(connectableElement);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.NAMED_ELEMENT: {
				NamedElement namedElement = (NamedElement) theEObject;
				T result = caseNamedElement(namedElement);
				if (result == null)
					result = caseSignalElement(namedElement);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.IDENTIFIER: {
				Identifier identifier = (Identifier) theEObject;
				T result = caseIdentifier(identifier);
				if (result == null)
					result = caseNamedElement(identifier);
				if (result == null)
					result = caseConnectableElement(identifier);
				if (result == null)
					result = caseSignalElement(identifier);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SIGNAL_EXPR: {
				SignalExpr signalExpr = (SignalExpr) theEObject;
				T result = caseSignalExpr(signalExpr);
				if (result == null)
					result = caseConnectableElement(signalExpr);
				if (result == null)
					result = caseSignalElement(signalExpr);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONSTANT_EXPR: {
				ConstantExpr constantExpr = (ConstantExpr) theEObject;
				T result = caseConstantExpr(constantExpr);
				if (result == null)
					result = caseConnectableElement(constantExpr);
				if (result == null)
					result = caseSignalElement(constantExpr);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CLOCKED_EXPR: {
				ClockedExpr clockedExpr = (ClockedExpr) theEObject;
				T result = caseClockedExpr(clockedExpr);
				if (result == null)
					result = caseConnectableElement(clockedExpr);
				if (result == null)
					result = caseSignalElement(clockedExpr);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.DEPENDENCE_IN_OUT: {
				DependenceInOut dependenceInOut = (DependenceInOut) theEObject;
				T result = caseDependenceInOut(dependenceInOut);
				if (result == null)
					result = caseConnectableElement(dependenceInOut);
				if (result == null)
					result = caseSignalElement(dependenceInOut);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SIGNAL_CONTAINER: {
				SignalContainer signalContainer = (SignalContainer) theEObject;
				T result = caseSignalContainer(signalContainer);
				if (result == null)
					result = caseSignalElement(signalContainer);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.INTERFACE_DEFINITION: {
				InterfaceDefinition interfaceDefinition = (InterfaceDefinition) theEObject;
				T result = caseInterfaceDefinition(interfaceDefinition);
				if (result == null)
					result = caseSignalContainer(interfaceDefinition);
				if (result == null)
					result = caseSignalElement(interfaceDefinition);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MODEL: {
				Model model = (Model) theEObject;
				T result = caseModel(model);
				if (result == null)
					result = caseSignalContainer(model);
				if (result == null)
					result = caseSignalElement(model);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.BASIC_ITERATE: {
				BasicIterate basicIterate = (BasicIterate) theEObject;
				T result = caseBasicIterate(basicIterate);
				if (result == null)
					result = caseSubProcess(basicIterate);
				if (result == null)
					result = caseModel(basicIterate);
				if (result == null)
					result = caseClockedExpr(basicIterate);
				if (result == null)
					result = caseIdentifier(basicIterate);
				if (result == null)
					result = caseDependenceInOut(basicIterate);
				if (result == null)
					result = caseSignalContainer(basicIterate);
				if (result == null)
					result = caseConnectableElement(basicIterate);
				if (result == null)
					result = caseNamedElement(basicIterate);
				if (result == null)
					result = caseSignalElement(basicIterate);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ITERATE: {
				Iterate iterate = (Iterate) theEObject;
				T result = caseIterate(iterate);
				if (result == null)
					result = caseSubProcess(iterate);
				if (result == null)
					result = caseSignalOperator(iterate);
				if (result == null)
					result = caseModel(iterate);
				if (result == null)
					result = caseClockedExpr(iterate);
				if (result == null)
					result = caseIdentifier(iterate);
				if (result == null)
					result = caseDependenceInOut(iterate);
				if (result == null)
					result = caseSignalContainer(iterate);
				if (result == null)
					result = caseConnectableElement(iterate);
				if (result == null)
					result = caseNamedElement(iterate);
				if (result == null)
					result = caseSignalElement(iterate);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ITERATION_INIT: {
				IterationInit iterationInit = (IterationInit) theEObject;
				T result = caseIterationInit(iterationInit);
				if (result == null)
					result = caseModel(iterationInit);
				if (result == null)
					result = caseSignalContainer(iterationInit);
				if (result == null)
					result = caseSignalElement(iterationInit);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SPECIFICATIONS: {
				Specifications specifications = (Specifications) theEObject;
				T result = caseSpecifications(specifications);
				if (result == null)
					result = caseModel(specifications);
				if (result == null)
					result = caseSignalContainer(specifications);
				if (result == null)
					result = caseSignalElement(specifications);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SUB_PROCESS: {
				SubProcess subProcess = (SubProcess) theEObject;
				T result = caseSubProcess(subProcess);
				if (result == null)
					result = caseModel(subProcess);
				if (result == null)
					result = caseClockedExpr(subProcess);
				if (result == null)
					result = caseIdentifier(subProcess);
				if (result == null)
					result = caseDependenceInOut(subProcess);
				if (result == null)
					result = caseSignalContainer(subProcess);
				if (result == null)
					result = caseConnectableElement(subProcess);
				if (result == null)
					result = caseNamedElement(subProcess);
				if (result == null)
					result = caseSignalElement(subProcess);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.TYPE_DECLARATION: {
				TypeDeclaration typeDeclaration = (TypeDeclaration) theEObject;
				T result = caseTypeDeclaration(typeDeclaration);
				if (result == null)
					result = caseIdentifier(typeDeclaration);
				if (result == null)
					result = caseNamedElement(typeDeclaration);
				if (result == null)
					result = caseConnectableElement(typeDeclaration);
				if (result == null)
					result = caseSignalElement(typeDeclaration);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.PARAMETER_TYPE: {
				ParameterType parameterType = (ParameterType) theEObject;
				T result = caseParameterType(parameterType);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SIGNAL_TYPE: {
				SignalType signalType = (SignalType) theEObject;
				T result = caseSignalType(signalType);
				if (result == null)
					result = caseParameterType(signalType);
				if (result == null)
					result = caseConstantExpr(signalType);
				if (result == null)
					result = caseConnectableElement(signalType);
				if (result == null)
					result = caseSignalElement(signalType);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ENUMERATION_TYPE: {
				EnumerationType enumerationType = (EnumerationType) theEObject;
				T result = caseEnumerationType(enumerationType);
				if (result == null)
					result = caseTypeDeclaration(enumerationType);
				if (result == null)
					result = caseSignalType(enumerationType);
				if (result == null)
					result = caseIdentifier(enumerationType);
				if (result == null)
					result = caseParameterType(enumerationType);
				if (result == null)
					result = caseConstantExpr(enumerationType);
				if (result == null)
					result = caseNamedElement(enumerationType);
				if (result == null)
					result = caseConnectableElement(enumerationType);
				if (result == null)
					result = caseSignalElement(enumerationType);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MODEL_TYPE: {
				ModelType modelType = (ModelType) theEObject;
				T result = caseModelType(modelType);
				if (result == null)
					result = caseInterfaceDefinition(modelType);
				if (result == null)
					result = caseTypeDeclaration(modelType);
				if (result == null)
					result = caseParameterType(modelType);
				if (result == null)
					result = caseSignalContainer(modelType);
				if (result == null)
					result = caseIdentifier(modelType);
				if (result == null)
					result = caseSignalElement(modelType);
				if (result == null)
					result = caseNamedElement(modelType);
				if (result == null)
					result = caseConnectableElement(modelType);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SUBSTITUTION_TYPE: {
				SubstitutionType substitutionType = (SubstitutionType) theEObject;
				T result = caseSubstitutionType(substitutionType);
				if (result == null)
					result = caseTypeDeclaration(substitutionType);
				if (result == null)
					result = caseSignalType(substitutionType);
				if (result == null)
					result = caseIdentifier(substitutionType);
				if (result == null)
					result = caseParameterType(substitutionType);
				if (result == null)
					result = caseConstantExpr(substitutionType);
				if (result == null)
					result = caseNamedElement(substitutionType);
				if (result == null)
					result = caseConnectableElement(substitutionType);
				if (result == null)
					result = caseSignalElement(substitutionType);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.TUPLE_TYPE: {
				TupleType tupleType = (TupleType) theEObject;
				T result = caseTupleType(tupleType);
				if (result == null)
					result = caseTypeDeclaration(tupleType);
				if (result == null)
					result = caseSignalContainer(tupleType);
				if (result == null)
					result = caseSignalType(tupleType);
				if (result == null)
					result = caseIdentifier(tupleType);
				if (result == null)
					result = caseSignalElement(tupleType);
				if (result == null)
					result = caseParameterType(tupleType);
				if (result == null)
					result = caseConstantExpr(tupleType);
				if (result == null)
					result = caseNamedElement(tupleType);
				if (result == null)
					result = caseConnectableElement(tupleType);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MODEL_DECLARATION: {
				ModelDeclaration modelDeclaration = (ModelDeclaration) theEObject;
				T result = caseModelDeclaration(modelDeclaration);
				if (result == null)
					result = caseModel(modelDeclaration);
				if (result == null)
					result = caseInterfaceDefinition(modelDeclaration);
				if (result == null)
					result = caseIdentifier(modelDeclaration);
				if (result == null)
					result = caseConstantExpr(modelDeclaration);
				if (result == null)
					result = caseRootModel(modelDeclaration);
				if (result == null)
					result = caseSignalContainer(modelDeclaration);
				if (result == null)
					result = caseNamedElement(modelDeclaration);
				if (result == null)
					result = caseConnectableElement(modelDeclaration);
				if (result == null)
					result = caseSignalElement(modelDeclaration);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MODEL_REF: {
				ModelRef modelRef = (ModelRef) theEObject;
				T result = caseModelRef(modelRef);
				if (result == null)
					result = caseConstantExpr(modelRef);
				if (result == null)
					result = caseSignalElement(modelRef);
				if (result == null)
					result = caseConnectableElement(modelRef);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MODEL_INSTANCE: {
				ModelInstance modelInstance = (ModelInstance) theEObject;
				T result = caseModelInstance(modelInstance);
				if (result == null)
					result = caseClockedExpr(modelInstance);
				if (result == null)
					result = caseIdentifier(modelInstance);
				if (result == null)
					result = caseDependenceInOut(modelInstance);
				if (result == null)
					result = caseSignalContainer(modelInstance);
				if (result == null)
					result = caseConnectableElement(modelInstance);
				if (result == null)
					result = caseNamedElement(modelInstance);
				if (result == null)
					result = caseSignalElement(modelInstance);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MODULE: {
				Module module = (Module) theEObject;
				T result = caseModule(module);
				if (result == null)
					result = caseIdentifier(module);
				if (result == null)
					result = caseRootModel(module);
				if (result == null)
					result = caseSignalContainer(module);
				if (result == null)
					result = caseNamedElement(module);
				if (result == null)
					result = caseConnectableElement(module);
				if (result == null)
					result = caseSignalElement(module);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.USE_MODULE: {
				UseModule useModule = (UseModule) theEObject;
				T result = caseUseModule(useModule);
				if (result == null)
					result = caseSignalElement(useModule);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.DATAFLOW_CONNECTOR: {
				DataflowConnector dataflowConnector = (DataflowConnector) theEObject;
				T result = caseDataflowConnector(dataflowConnector);
				if (result == null)
					result = caseConnectableElement(dataflowConnector);
				if (result == null)
					result = caseSignalElement(dataflowConnector);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONSTANT_OR_REF: {
				ConstantOrRef constantOrRef = (ConstantOrRef) theEObject;
				T result = caseConstantOrRef(constantOrRef);
				if (result == null)
					result = caseSignalExpr(constantOrRef);
				if (result == null)
					result = caseConstantExpr(constantOrRef);
				if (result == null)
					result = caseConnectableElement(constantOrRef);
				if (result == null)
					result = caseSignalElement(constantOrRef);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONSTANT_REF: {
				ConstantRef constantRef = (ConstantRef) theEObject;
				T result = caseConstantRef(constantRef);
				if (result == null)
					result = caseConstantOrRef(constantRef);
				if (result == null)
					result = caseSignalElement(constantRef);
				if (result == null)
					result = caseConditionExpr(constantRef);
				if (result == null)
					result = caseSignalExpr(constantRef);
				if (result == null)
					result = caseConstantExpr(constantRef);
				if (result == null)
					result = caseConnectableElement(constantRef);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONSTANT: {
				Constant constant = (Constant) theEObject;
				T result = caseConstant(constant);
				if (result == null)
					result = caseConstantOrRef(constant);
				if (result == null)
					result = caseIdentifier(constant);
				if (result == null)
					result = caseSignalExpr(constant);
				if (result == null)
					result = caseConstantExpr(constant);
				if (result == null)
					result = caseNamedElement(constant);
				if (result == null)
					result = caseConnectableElement(constant);
				if (result == null)
					result = caseSignalElement(constant);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONSTANT_VALUE: {
				ConstantValue constantValue = (ConstantValue) theEObject;
				T result = caseConstantValue(constantValue);
				if (result == null)
					result = caseConstant(constantValue);
				if (result == null)
					result = caseConditionExpr(constantValue);
				if (result == null)
					result = caseConstantOrRef(constantValue);
				if (result == null)
					result = caseIdentifier(constantValue);
				if (result == null)
					result = caseConnectableElement(constantValue);
				if (result == null)
					result = caseSignalExpr(constantValue);
				if (result == null)
					result = caseConstantExpr(constantValue);
				if (result == null)
					result = caseNamedElement(constantValue);
				if (result == null)
					result = caseSignalElement(constantValue);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.LITERAL: {
				Literal literal = (Literal) theEObject;
				T result = caseLiteral(literal);
				if (result == null)
					result = caseSignalExpr(literal);
				if (result == null)
					result = caseConstantExpr(literal);
				if (result == null)
					result = caseConditionExpr(literal);
				if (result == null)
					result = caseConnectableElement(literal);
				if (result == null)
					result = caseSignalElement(literal);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ENUMERATION_VALUE: {
				EnumerationValue enumerationValue = (EnumerationValue) theEObject;
				T result = caseEnumerationValue(enumerationValue);
				if (result == null)
					result = caseLiteral(enumerationValue);
				if (result == null)
					result = caseSignalExpr(enumerationValue);
				if (result == null)
					result = caseConstantExpr(enumerationValue);
				if (result == null)
					result = caseConditionExpr(enumerationValue);
				if (result == null)
					result = caseConnectableElement(enumerationValue);
				if (result == null)
					result = caseSignalElement(enumerationValue);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.PARAMETER: {
				Parameter parameter = (Parameter) theEObject;
				T result = caseParameter(parameter);
				if (result == null)
					result = caseConstant(parameter);
				if (result == null)
					result = caseConditionExpr(parameter);
				if (result == null)
					result = caseConstantOrRef(parameter);
				if (result == null)
					result = caseIdentifier(parameter);
				if (result == null)
					result = caseConnectableElement(parameter);
				if (result == null)
					result = caseSignalExpr(parameter);
				if (result == null)
					result = caseConstantExpr(parameter);
				if (result == null)
					result = caseNamedElement(parameter);
				if (result == null)
					result = caseSignalElement(parameter);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.PARAMETER_INSTANCE: {
				ParameterInstance parameterInstance = (ParameterInstance) theEObject;
				T result = caseParameterInstance(parameterInstance);
				if (result == null)
					result = caseConstant(parameterInstance);
				if (result == null)
					result = caseConditionExpr(parameterInstance);
				if (result == null)
					result = caseConstantOrRef(parameterInstance);
				if (result == null)
					result = caseIdentifier(parameterInstance);
				if (result == null)
					result = caseConnectableElement(parameterInstance);
				if (result == null)
					result = caseSignalExpr(parameterInstance);
				if (result == null)
					result = caseConstantExpr(parameterInstance);
				if (result == null)
					result = caseNamedElement(parameterInstance);
				if (result == null)
					result = caseSignalElement(parameterInstance);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ITERATOR: {
				Iterator iterator = (Iterator) theEObject;
				T result = caseIterator(iterator);
				if (result == null)
					result = caseConstant(iterator);
				if (result == null)
					result = caseConstantOrRef(iterator);
				if (result == null)
					result = caseIdentifier(iterator);
				if (result == null)
					result = caseSignalExpr(iterator);
				if (result == null)
					result = caseConstantExpr(iterator);
				if (result == null)
					result = caseNamedElement(iterator);
				if (result == null)
					result = caseConnectableElement(iterator);
				if (result == null)
					result = caseSignalElement(iterator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.INDEX: {
				Index index = (Index) theEObject;
				T result = caseIndex(index);
				if (result == null)
					result = caseConstant(index);
				if (result == null)
					result = caseConstantOrRef(index);
				if (result == null)
					result = caseIdentifier(index);
				if (result == null)
					result = caseSignalExpr(index);
				if (result == null)
					result = caseConstantExpr(index);
				if (result == null)
					result = caseNamedElement(index);
				if (result == null)
					result = caseConnectableElement(index);
				if (result == null)
					result = caseSignalElement(index);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SIGNAL_OR_REF: {
				SignalOrRef signalOrRef = (SignalOrRef) theEObject;
				T result = caseSignalOrRef(signalOrRef);
				if (result == null)
					result = caseSignalExpr(signalOrRef);
				if (result == null)
					result = caseClockedExpr(signalOrRef);
				if (result == null)
					result = caseDependenceInOut(signalOrRef);
				if (result == null)
					result = caseDataflowConnector(signalOrRef);
				if (result == null)
					result = caseConditionExpr(signalOrRef);
				if (result == null)
					result = caseConnectableElement(signalOrRef);
				if (result == null)
					result = caseSignalElement(signalOrRef);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SIGNAL_REF: {
				SignalRef signalRef = (SignalRef) theEObject;
				T result = caseSignalRef(signalRef);
				if (result == null)
					result = caseSignalOrRef(signalRef);
				if (result == null)
					result = caseSignalElement(signalRef);
				if (result == null)
					result = caseSignalExpr(signalRef);
				if (result == null)
					result = caseClockedExpr(signalRef);
				if (result == null)
					result = caseDependenceInOut(signalRef);
				if (result == null)
					result = caseDataflowConnector(signalRef);
				if (result == null)
					result = caseConditionExpr(signalRef);
				if (result == null)
					result = caseConnectableElement(signalRef);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SIGNAL: {
				Signal signal = (Signal) theEObject;
				T result = caseSignal(signal);
				if (result == null)
					result = caseSignalOrRef(signal);
				if (result == null)
					result = caseIdentifier(signal);
				if (result == null)
					result = caseSignalExpr(signal);
				if (result == null)
					result = caseClockedExpr(signal);
				if (result == null)
					result = caseDependenceInOut(signal);
				if (result == null)
					result = caseDataflowConnector(signal);
				if (result == null)
					result = caseConditionExpr(signal);
				if (result == null)
					result = caseNamedElement(signal);
				if (result == null)
					result = caseConnectableElement(signal);
				if (result == null)
					result = caseSignalElement(signal);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.INPUT: {
				Input input = (Input) theEObject;
				T result = caseInput(input);
				if (result == null)
					result = caseSignal(input);
				if (result == null)
					result = caseSignalOrRef(input);
				if (result == null)
					result = caseIdentifier(input);
				if (result == null)
					result = caseSignalExpr(input);
				if (result == null)
					result = caseClockedExpr(input);
				if (result == null)
					result = caseDependenceInOut(input);
				if (result == null)
					result = caseDataflowConnector(input);
				if (result == null)
					result = caseConditionExpr(input);
				if (result == null)
					result = caseNamedElement(input);
				if (result == null)
					result = caseConnectableElement(input);
				if (result == null)
					result = caseSignalElement(input);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.INPUT_INSTANCE: {
				InputInstance inputInstance = (InputInstance) theEObject;
				T result = caseInputInstance(inputInstance);
				if (result == null)
					result = caseSignalOrRef(inputInstance);
				if (result == null)
					result = caseIdentifier(inputInstance);
				if (result == null)
					result = caseSignalExpr(inputInstance);
				if (result == null)
					result = caseClockedExpr(inputInstance);
				if (result == null)
					result = caseDependenceInOut(inputInstance);
				if (result == null)
					result = caseDataflowConnector(inputInstance);
				if (result == null)
					result = caseConditionExpr(inputInstance);
				if (result == null)
					result = caseNamedElement(inputInstance);
				if (result == null)
					result = caseConnectableElement(inputInstance);
				if (result == null)
					result = caseSignalElement(inputInstance);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.OUTPUT: {
				Output output = (Output) theEObject;
				T result = caseOutput(output);
				if (result == null)
					result = caseSignal(output);
				if (result == null)
					result = caseSignalOrRef(output);
				if (result == null)
					result = caseIdentifier(output);
				if (result == null)
					result = caseSignalExpr(output);
				if (result == null)
					result = caseClockedExpr(output);
				if (result == null)
					result = caseDependenceInOut(output);
				if (result == null)
					result = caseDataflowConnector(output);
				if (result == null)
					result = caseConditionExpr(output);
				if (result == null)
					result = caseNamedElement(output);
				if (result == null)
					result = caseConnectableElement(output);
				if (result == null)
					result = caseSignalElement(output);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.OUTPUT_INSTANCE: {
				OutputInstance outputInstance = (OutputInstance) theEObject;
				T result = caseOutputInstance(outputInstance);
				if (result == null)
					result = caseSignalOrRef(outputInstance);
				if (result == null)
					result = caseConstantExpr(outputInstance);
				if (result == null)
					result = caseIdentifier(outputInstance);
				if (result == null)
					result = caseSignalExpr(outputInstance);
				if (result == null)
					result = caseClockedExpr(outputInstance);
				if (result == null)
					result = caseDependenceInOut(outputInstance);
				if (result == null)
					result = caseDataflowConnector(outputInstance);
				if (result == null)
					result = caseConditionExpr(outputInstance);
				if (result == null)
					result = caseConnectableElement(outputInstance);
				if (result == null)
					result = caseNamedElement(outputInstance);
				if (result == null)
					result = caseSignalElement(outputInstance);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.LOCAL: {
				Local local = (Local) theEObject;
				T result = caseLocal(local);
				if (result == null)
					result = caseSignal(local);
				if (result == null)
					result = caseSignalOrRef(local);
				if (result == null)
					result = caseIdentifier(local);
				if (result == null)
					result = caseSignalExpr(local);
				if (result == null)
					result = caseClockedExpr(local);
				if (result == null)
					result = caseDependenceInOut(local);
				if (result == null)
					result = caseDataflowConnector(local);
				if (result == null)
					result = caseConditionExpr(local);
				if (result == null)
					result = caseNamedElement(local);
				if (result == null)
					result = caseConnectableElement(local);
				if (result == null)
					result = caseSignalElement(local);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONNECTION: {
				Connection connection = (Connection) theEObject;
				T result = caseConnection(connection);
				if (result == null)
					result = caseNamedElement(connection);
				if (result == null)
					result = caseSignalElement(connection);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MODEL_CONNECTION: {
				ModelConnection modelConnection = (ModelConnection) theEObject;
				T result = caseModelConnection(modelConnection);
				if (result == null)
					result = caseConnection(modelConnection);
				if (result == null)
					result = caseNamedElement(modelConnection);
				if (result == null)
					result = caseSignalElement(modelConnection);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CASE_CONNECTION: {
				CaseConnection caseConnection = (CaseConnection) theEObject;
				T result = caseCaseConnection(caseConnection);
				if (result == null)
					result = caseModelConnection(caseConnection);
				if (result == null)
					result = caseConnection(caseConnection);
				if (result == null)
					result = caseNamedElement(caseConnection);
				if (result == null)
					result = caseSignalElement(caseConnection);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CLOCK_RELATION: {
				ClockRelation clockRelation = (ClockRelation) theEObject;
				T result = caseClockRelation(clockRelation);
				if (result == null)
					result = caseModelConnection(clockRelation);
				if (result == null)
					result = caseConnection(clockRelation);
				if (result == null)
					result = caseNamedElement(clockRelation);
				if (result == null)
					result = caseSignalElement(clockRelation);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONDITION: {
				Condition condition = (Condition) theEObject;
				T result = caseCondition(condition);
				if (result == null)
					result = caseModelConnection(condition);
				if (result == null)
					result = caseConnection(condition);
				if (result == null)
					result = caseNamedElement(condition);
				if (result == null)
					result = caseSignalElement(condition);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONSTANT_AFFECTATION: {
				ConstantAffectation constantAffectation = (ConstantAffectation) theEObject;
				T result = caseConstantAffectation(constantAffectation);
				if (result == null)
					result = caseModelConnection(constantAffectation);
				if (result == null)
					result = caseConnection(constantAffectation);
				if (result == null)
					result = caseNamedElement(constantAffectation);
				if (result == null)
					result = caseSignalElement(constantAffectation);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.DATAFLOW_CONNECTION: {
				DataflowConnection dataflowConnection = (DataflowConnection) theEObject;
				T result = caseDataflowConnection(dataflowConnection);
				if (result == null)
					result = caseModelConnection(dataflowConnection);
				if (result == null)
					result = caseConnection(dataflowConnection);
				if (result == null)
					result = caseNamedElement(dataflowConnection);
				if (result == null)
					result = caseSignalElement(dataflowConnection);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.DEPENDENCE: {
				Dependence dependence = (Dependence) theEObject;
				T result = caseDependence(dependence);
				if (result == null)
					result = caseModelConnection(dependence);
				if (result == null)
					result = caseConnection(dependence);
				if (result == null)
					result = caseNamedElement(dependence);
				if (result == null)
					result = caseSignalElement(dependence);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.OPERATOR_PORT: {
				OperatorPort operatorPort = (OperatorPort) theEObject;
				T result = caseOperatorPort(operatorPort);
				if (result == null)
					result = caseConnectableElement(operatorPort);
				if (result == null)
					result = caseSignalElement(operatorPort);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.INPUT_PORT: {
				InputPort inputPort = (InputPort) theEObject;
				T result = caseInputPort(inputPort);
				if (result == null)
					result = caseClockedExpr(inputPort);
				if (result == null)
					result = caseOperatorPort(inputPort);
				if (result == null)
					result = caseDataflowConnector(inputPort);
				if (result == null)
					result = caseConnectableElement(inputPort);
				if (result == null)
					result = caseSignalElement(inputPort);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.OUTPUT_PORT: {
				OutputPort outputPort = (OutputPort) theEObject;
				T result = caseOutputPort(outputPort);
				if (result == null)
					result = caseIdentifier(outputPort);
				if (result == null)
					result = caseSignalExpr(outputPort);
				if (result == null)
					result = caseConstantExpr(outputPort);
				if (result == null)
					result = caseNamedElement(outputPort);
				if (result == null)
					result = caseConnectableElement(outputPort);
				if (result == null)
					result = caseSignalElement(outputPort);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SIGNAL_OPERATOR: {
				SignalOperator signalOperator = (SignalOperator) theEObject;
				T result = caseSignalOperator(signalOperator);
				if (result == null)
					result = caseSignalContainer(signalOperator);
				if (result == null)
					result = caseSignalElement(signalOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONDITION_EXPR: {
				ConditionExpr conditionExpr = (ConditionExpr) theEObject;
				T result = caseConditionExpr(conditionExpr);
				if (result == null)
					result = caseConnectableElement(conditionExpr);
				if (result == null)
					result = caseSignalElement(conditionExpr);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONDITIONNED_EXPR: {
				ConditionnedExpr conditionnedExpr = (ConditionnedExpr) theEObject;
				T result = caseConditionnedExpr(conditionnedExpr);
				if (result == null)
					result = caseConnectableElement(conditionnedExpr);
				if (result == null)
					result = caseSignalElement(conditionnedExpr);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONTROLFLOW_OPERATOR: {
				ControlflowOperator controlflowOperator = (ControlflowOperator) theEObject;
				T result = caseControlflowOperator(controlflowOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.DATAFLOW_OPERATOR: {
				DataflowOperator dataflowOperator = (DataflowOperator) theEObject;
				T result = caseDataflowOperator(dataflowOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.BOOLEAN_OPERATOR_I: {
				BooleanOperatorI booleanOperatorI = (BooleanOperatorI) theEObject;
				T result = caseBooleanOperatorI(booleanOperatorI);
				if (result == null)
					result = caseSignalExpr(booleanOperatorI);
				if (result == null)
					result = caseConstantExpr(booleanOperatorI);
				if (result == null)
					result = caseDataflowOperator(booleanOperatorI);
				if (result == null)
					result = caseConditionExpr(booleanOperatorI);
				if (result == null)
					result = caseConnectableElement(booleanOperatorI);
				if (result == null)
					result = caseSignalElement(booleanOperatorI);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.NUMERIC_OPERATOR_I: {
				NumericOperatorI numericOperatorI = (NumericOperatorI) theEObject;
				T result = caseNumericOperatorI(numericOperatorI);
				if (result == null)
					result = caseSignalExpr(numericOperatorI);
				if (result == null)
					result = caseConstantExpr(numericOperatorI);
				if (result == null)
					result = caseDataflowOperator(numericOperatorI);
				if (result == null)
					result = caseConnectableElement(numericOperatorI);
				if (result == null)
					result = caseSignalElement(numericOperatorI);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SINGLE_INPUT_OPERATOR: {
				SingleInputOperator singleInputOperator = (SingleInputOperator) theEObject;
				T result = caseSingleInputOperator(singleInputOperator);
				if (result == null)
					result = caseSignalOperator(singleInputOperator);
				if (result == null)
					result = caseNamedElement(singleInputOperator);
				if (result == null)
					result = caseSignalContainer(singleInputOperator);
				if (result == null)
					result = caseSignalElement(singleInputOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.TWO_INPUTS_OPERATOR: {
				TwoInputsOperator twoInputsOperator = (TwoInputsOperator) theEObject;
				T result = caseTwoInputsOperator(twoInputsOperator);
				if (result == null)
					result = caseSignalOperator(twoInputsOperator);
				if (result == null)
					result = caseNamedElement(twoInputsOperator);
				if (result == null)
					result = caseSignalContainer(twoInputsOperator);
				if (result == null)
					result = caseSignalElement(twoInputsOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MULTIPLE_INPUTS_OPERATOR: {
				MultipleInputsOperator multipleInputsOperator = (MultipleInputsOperator) theEObject;
				T result = caseMultipleInputsOperator(multipleInputsOperator);
				if (result == null)
					result = caseSignalOperator(multipleInputsOperator);
				if (result == null)
					result = caseNamedElement(multipleInputsOperator);
				if (result == null)
					result = caseSignalContainer(multipleInputsOperator);
				if (result == null)
					result = caseSignalElement(multipleInputsOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.AND: {
				And and = (And) theEObject;
				T result = caseAnd(and);
				if (result == null)
					result = caseMultipleInputsOperator(and);
				if (result == null)
					result = caseBooleanOperatorI(and);
				if (result == null)
					result = caseSignalOperator(and);
				if (result == null)
					result = caseNamedElement(and);
				if (result == null)
					result = caseSignalExpr(and);
				if (result == null)
					result = caseConstantExpr(and);
				if (result == null)
					result = caseDataflowOperator(and);
				if (result == null)
					result = caseConditionExpr(and);
				if (result == null)
					result = caseSignalContainer(and);
				if (result == null)
					result = caseSignalElement(and);
				if (result == null)
					result = caseConnectableElement(and);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ARITHMETIC_OPERATOR: {
				ArithmeticOperator arithmeticOperator = (ArithmeticOperator) theEObject;
				T result = caseArithmeticOperator(arithmeticOperator);
				if (result == null)
					result = caseMultipleInputsOperator(arithmeticOperator);
				if (result == null)
					result = caseNumericOperatorI(arithmeticOperator);
				if (result == null)
					result = caseSignalOperator(arithmeticOperator);
				if (result == null)
					result = caseNamedElement(arithmeticOperator);
				if (result == null)
					result = caseSignalExpr(arithmeticOperator);
				if (result == null)
					result = caseConstantExpr(arithmeticOperator);
				if (result == null)
					result = caseDataflowOperator(arithmeticOperator);
				if (result == null)
					result = caseSignalContainer(arithmeticOperator);
				if (result == null)
					result = caseSignalElement(arithmeticOperator);
				if (result == null)
					result = caseConnectableElement(arithmeticOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ARRAY_ENUMERATION: {
				ArrayEnumeration arrayEnumeration = (ArrayEnumeration) theEObject;
				T result = caseArrayEnumeration(arrayEnumeration);
				if (result == null)
					result = caseMultipleInputsOperator(arrayEnumeration);
				if (result == null)
					result = caseSignalExpr(arrayEnumeration);
				if (result == null)
					result = caseConstantExpr(arrayEnumeration);
				if (result == null)
					result = caseDataflowOperator(arrayEnumeration);
				if (result == null)
					result = caseSignalOperator(arrayEnumeration);
				if (result == null)
					result = caseNamedElement(arrayEnumeration);
				if (result == null)
					result = caseConnectableElement(arrayEnumeration);
				if (result == null)
					result = caseSignalContainer(arrayEnumeration);
				if (result == null)
					result = caseSignalElement(arrayEnumeration);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ARRAY_PRODUCT: {
				ArrayProduct arrayProduct = (ArrayProduct) theEObject;
				T result = caseArrayProduct(arrayProduct);
				if (result == null)
					result = caseMultipleInputsOperator(arrayProduct);
				if (result == null)
					result = caseNumericOperatorI(arrayProduct);
				if (result == null)
					result = caseSignalOperator(arrayProduct);
				if (result == null)
					result = caseNamedElement(arrayProduct);
				if (result == null)
					result = caseSignalExpr(arrayProduct);
				if (result == null)
					result = caseConstantExpr(arrayProduct);
				if (result == null)
					result = caseDataflowOperator(arrayProduct);
				if (result == null)
					result = caseSignalContainer(arrayProduct);
				if (result == null)
					result = caseSignalElement(arrayProduct);
				if (result == null)
					result = caseConnectableElement(arrayProduct);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ARRAY_RECOVERY: {
				ArrayRecovery arrayRecovery = (ArrayRecovery) theEObject;
				T result = caseArrayRecovery(arrayRecovery);
				if (result == null)
					result = caseTwoInputsOperator(arrayRecovery);
				if (result == null)
					result = caseSignalExpr(arrayRecovery);
				if (result == null)
					result = caseConstantExpr(arrayRecovery);
				if (result == null)
					result = caseConditionExpr(arrayRecovery);
				if (result == null)
					result = caseDataflowOperator(arrayRecovery);
				if (result == null)
					result = caseSignalOperator(arrayRecovery);
				if (result == null)
					result = caseNamedElement(arrayRecovery);
				if (result == null)
					result = caseConnectableElement(arrayRecovery);
				if (result == null)
					result = caseSignalContainer(arrayRecovery);
				if (result == null)
					result = caseSignalElement(arrayRecovery);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ARRAY_RESTRUCTURATION: {
				ArrayRestructuration arrayRestructuration = (ArrayRestructuration) theEObject;
				T result = caseArrayRestructuration(arrayRestructuration);
				if (result == null)
					result = caseTwoInputsOperator(arrayRestructuration);
				if (result == null)
					result = caseSignalExpr(arrayRestructuration);
				if (result == null)
					result = caseConstantExpr(arrayRestructuration);
				if (result == null)
					result = caseDataflowOperator(arrayRestructuration);
				if (result == null)
					result = caseSignalOperator(arrayRestructuration);
				if (result == null)
					result = caseNamedElement(arrayRestructuration);
				if (result == null)
					result = caseConnectableElement(arrayRestructuration);
				if (result == null)
					result = caseSignalContainer(arrayRestructuration);
				if (result == null)
					result = caseSignalElement(arrayRestructuration);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ASSERTION: {
				Assertion assertion = (Assertion) theEObject;
				T result = caseAssertion(assertion);
				if (result == null)
					result = caseNamedElement(assertion);
				if (result == null)
					result = caseConditionnedExpr(assertion);
				if (result == null)
					result = caseSignalElement(assertion);
				if (result == null)
					result = caseConnectableElement(assertion);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ASSERTION_CLOCK_CONSTRAINT_OPERATOR: {
				AssertionClockConstraintOperator assertionClockConstraintOperator = (AssertionClockConstraintOperator) theEObject;
				T result = caseAssertionClockConstraintOperator(assertionClockConstraintOperator);
				if (result == null)
					result = caseClockConstraintOperator(assertionClockConstraintOperator);
				if (result == null)
					result = caseClockedExpr(assertionClockConstraintOperator);
				if (result == null)
					result = caseNamedElement(assertionClockConstraintOperator);
				if (result == null)
					result = caseControlflowOperator(assertionClockConstraintOperator);
				if (result == null)
					result = caseConnectableElement(assertionClockConstraintOperator);
				if (result == null)
					result = caseSignalElement(assertionClockConstraintOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.ASSERTION_CLOCK_SPEED_OPERATOR: {
				AssertionClockSpeedOperator assertionClockSpeedOperator = (AssertionClockSpeedOperator) theEObject;
				T result = caseAssertionClockSpeedOperator(assertionClockSpeedOperator);
				if (result == null)
					result = caseClockSpeedOperator(assertionClockSpeedOperator);
				if (result == null)
					result = caseMultipleInputsOperator(assertionClockSpeedOperator);
				if (result == null)
					result = caseControlflowOperator(assertionClockSpeedOperator);
				if (result == null)
					result = caseSignalOperator(assertionClockSpeedOperator);
				if (result == null)
					result = caseNamedElement(assertionClockSpeedOperator);
				if (result == null)
					result = caseSignalContainer(assertionClockSpeedOperator);
				if (result == null)
					result = caseSignalElement(assertionClockSpeedOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.BOOLEAN_EXPRESSION: {
				BooleanExpression booleanExpression = (BooleanExpression) theEObject;
				T result = caseBooleanExpression(booleanExpression);
				if (result == null)
					result = caseNamedElement(booleanExpression);
				if (result == null)
					result = caseBooleanOperatorI(booleanExpression);
				if (result == null)
					result = caseSignalElement(booleanExpression);
				if (result == null)
					result = caseSignalExpr(booleanExpression);
				if (result == null)
					result = caseConstantExpr(booleanExpression);
				if (result == null)
					result = caseDataflowOperator(booleanExpression);
				if (result == null)
					result = caseConditionExpr(booleanExpression);
				if (result == null)
					result = caseConnectableElement(booleanExpression);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CARTESIAN_PRODUCT: {
				CartesianProduct cartesianProduct = (CartesianProduct) theEObject;
				T result = caseCartesianProduct(cartesianProduct);
				if (result == null)
					result = caseMultipleInputsOperator(cartesianProduct);
				if (result == null)
					result = caseDataflowOperator(cartesianProduct);
				if (result == null)
					result = caseSignalOperator(cartesianProduct);
				if (result == null)
					result = caseNamedElement(cartesianProduct);
				if (result == null)
					result = caseSignalContainer(cartesianProduct);
				if (result == null)
					result = caseSignalElement(cartesianProduct);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CELL: {
				Cell cell = (Cell) theEObject;
				T result = caseCell(cell);
				if (result == null)
					result = caseSingleInputOperator(cell);
				if (result == null)
					result = caseConditionnedExpr(cell);
				if (result == null)
					result = caseSignalExpr(cell);
				if (result == null)
					result = caseConditionExpr(cell);
				if (result == null)
					result = caseDataflowOperator(cell);
				if (result == null)
					result = caseSignalOperator(cell);
				if (result == null)
					result = caseNamedElement(cell);
				if (result == null)
					result = caseConnectableElement(cell);
				if (result == null)
					result = caseSignalContainer(cell);
				if (result == null)
					result = caseSignalElement(cell);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CLOCK_CONSTRAINT_OPERATOR: {
				ClockConstraintOperator clockConstraintOperator = (ClockConstraintOperator) theEObject;
				T result = caseClockConstraintOperator(clockConstraintOperator);
				if (result == null)
					result = caseClockedExpr(clockConstraintOperator);
				if (result == null)
					result = caseNamedElement(clockConstraintOperator);
				if (result == null)
					result = caseControlflowOperator(clockConstraintOperator);
				if (result == null)
					result = caseConnectableElement(clockConstraintOperator);
				if (result == null)
					result = caseSignalElement(clockConstraintOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CLOCK_RELATION_OPERATOR: {
				ClockRelationOperator clockRelationOperator = (ClockRelationOperator) theEObject;
				T result = caseClockRelationOperator(clockRelationOperator);
				if (result == null)
					result = caseMultipleInputsOperator(clockRelationOperator);
				if (result == null)
					result = caseClockedExpr(clockRelationOperator);
				if (result == null)
					result = caseControlflowOperator(clockRelationOperator);
				if (result == null)
					result = caseSignalOperator(clockRelationOperator);
				if (result == null)
					result = caseNamedElement(clockRelationOperator);
				if (result == null)
					result = caseConnectableElement(clockRelationOperator);
				if (result == null)
					result = caseSignalContainer(clockRelationOperator);
				if (result == null)
					result = caseSignalElement(clockRelationOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CLOCK_SPEED_OPERATOR: {
				ClockSpeedOperator clockSpeedOperator = (ClockSpeedOperator) theEObject;
				T result = caseClockSpeedOperator(clockSpeedOperator);
				if (result == null)
					result = caseMultipleInputsOperator(clockSpeedOperator);
				if (result == null)
					result = caseControlflowOperator(clockSpeedOperator);
				if (result == null)
					result = caseSignalOperator(clockSpeedOperator);
				if (result == null)
					result = caseNamedElement(clockSpeedOperator);
				if (result == null)
					result = caseSignalContainer(clockSpeedOperator);
				if (result == null)
					result = caseSignalElement(clockSpeedOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.COMPARISON_OPERATOR: {
				ComparisonOperator comparisonOperator = (ComparisonOperator) theEObject;
				T result = caseComparisonOperator(comparisonOperator);
				if (result == null)
					result = caseTwoInputsOperator(comparisonOperator);
				if (result == null)
					result = caseBooleanOperatorI(comparisonOperator);
				if (result == null)
					result = caseSignalOperator(comparisonOperator);
				if (result == null)
					result = caseNamedElement(comparisonOperator);
				if (result == null)
					result = caseSignalExpr(comparisonOperator);
				if (result == null)
					result = caseConstantExpr(comparisonOperator);
				if (result == null)
					result = caseDataflowOperator(comparisonOperator);
				if (result == null)
					result = caseConditionExpr(comparisonOperator);
				if (result == null)
					result = caseSignalContainer(comparisonOperator);
				if (result == null)
					result = caseSignalElement(comparisonOperator);
				if (result == null)
					result = caseConnectableElement(comparisonOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.COMPLEX_OPERATOR: {
				ComplexOperator complexOperator = (ComplexOperator) theEObject;
				T result = caseComplexOperator(complexOperator);
				if (result == null)
					result = caseTwoInputsOperator(complexOperator);
				if (result == null)
					result = caseNumericOperatorI(complexOperator);
				if (result == null)
					result = caseSignalOperator(complexOperator);
				if (result == null)
					result = caseNamedElement(complexOperator);
				if (result == null)
					result = caseSignalExpr(complexOperator);
				if (result == null)
					result = caseConstantExpr(complexOperator);
				if (result == null)
					result = caseDataflowOperator(complexOperator);
				if (result == null)
					result = caseSignalContainer(complexOperator);
				if (result == null)
					result = caseSignalElement(complexOperator);
				if (result == null)
					result = caseConnectableElement(complexOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.CONCATENATION: {
				Concatenation concatenation = (Concatenation) theEObject;
				T result = caseConcatenation(concatenation);
				if (result == null)
					result = caseMultipleInputsOperator(concatenation);
				if (result == null)
					result = caseSignalExpr(concatenation);
				if (result == null)
					result = caseConstantExpr(concatenation);
				if (result == null)
					result = caseDataflowOperator(concatenation);
				if (result == null)
					result = caseSignalOperator(concatenation);
				if (result == null)
					result = caseNamedElement(concatenation);
				if (result == null)
					result = caseConnectableElement(concatenation);
				if (result == null)
					result = caseSignalContainer(concatenation);
				if (result == null)
					result = caseSignalElement(concatenation);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.COUNTER: {
				Counter counter = (Counter) theEObject;
				T result = caseCounter(counter);
				if (result == null)
					result = caseTwoInputsOperator(counter);
				if (result == null)
					result = caseSignalExpr(counter);
				if (result == null)
					result = caseDataflowOperator(counter);
				if (result == null)
					result = caseSignalOperator(counter);
				if (result == null)
					result = caseNamedElement(counter);
				if (result == null)
					result = caseConnectableElement(counter);
				if (result == null)
					result = caseSignalContainer(counter);
				if (result == null)
					result = caseSignalElement(counter);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.DELAY: {
				Delay delay = (Delay) theEObject;
				T result = caseDelay(delay);
				if (result == null)
					result = caseSingleInputOperator(delay);
				if (result == null)
					result = caseSignalExpr(delay);
				if (result == null)
					result = caseConditionExpr(delay);
				if (result == null)
					result = caseDataflowOperator(delay);
				if (result == null)
					result = caseSignalOperator(delay);
				if (result == null)
					result = caseNamedElement(delay);
				if (result == null)
					result = caseConnectableElement(delay);
				if (result == null)
					result = caseSignalContainer(delay);
				if (result == null)
					result = caseSignalElement(delay);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.DEPENDENCE_OPERATOR: {
				DependenceOperator dependenceOperator = (DependenceOperator) theEObject;
				T result = caseDependenceOperator(dependenceOperator);
				if (result == null)
					result = caseDependenceInOut(dependenceOperator);
				if (result == null)
					result = caseConditionnedExpr(dependenceOperator);
				if (result == null)
					result = caseNamedElement(dependenceOperator);
				if (result == null)
					result = caseControlflowOperator(dependenceOperator);
				if (result == null)
					result = caseConnectableElement(dependenceOperator);
				if (result == null)
					result = caseSignalElement(dependenceOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.EXTRACTION: {
				Extraction extraction = (Extraction) theEObject;
				T result = caseExtraction(extraction);
				if (result == null)
					result = caseSingleInputOperator(extraction);
				if (result == null)
					result = caseConditionnedExpr(extraction);
				if (result == null)
					result = caseSignalExpr(extraction);
				if (result == null)
					result = caseConstantExpr(extraction);
				if (result == null)
					result = caseConditionExpr(extraction);
				if (result == null)
					result = caseDataflowOperator(extraction);
				if (result == null)
					result = caseSignalOperator(extraction);
				if (result == null)
					result = caseNamedElement(extraction);
				if (result == null)
					result = caseConnectableElement(extraction);
				if (result == null)
					result = caseSignalContainer(extraction);
				if (result == null)
					result = caseSignalElement(extraction);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.IF_THEN_ELSE: {
				IfThenElse ifThenElse = (IfThenElse) theEObject;
				T result = caseIfThenElse(ifThenElse);
				if (result == null)
					result = caseTwoInputsOperator(ifThenElse);
				if (result == null)
					result = caseConditionnedExpr(ifThenElse);
				if (result == null)
					result = caseSignalExpr(ifThenElse);
				if (result == null)
					result = caseConstantExpr(ifThenElse);
				if (result == null)
					result = caseConditionExpr(ifThenElse);
				if (result == null)
					result = caseDataflowOperator(ifThenElse);
				if (result == null)
					result = caseSignalOperator(ifThenElse);
				if (result == null)
					result = caseNamedElement(ifThenElse);
				if (result == null)
					result = caseConnectableElement(ifThenElse);
				if (result == null)
					result = caseSignalContainer(ifThenElse);
				if (result == null)
					result = caseSignalElement(ifThenElse);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.MERGE: {
				Merge merge = (Merge) theEObject;
				T result = caseMerge(merge);
				if (result == null)
					result = caseMultipleInputsOperator(merge);
				if (result == null)
					result = caseSignalExpr(merge);
				if (result == null)
					result = caseConstantExpr(merge);
				if (result == null)
					result = caseConditionExpr(merge);
				if (result == null)
					result = caseDataflowOperator(merge);
				if (result == null)
					result = caseSignalOperator(merge);
				if (result == null)
					result = caseNamedElement(merge);
				if (result == null)
					result = caseConnectableElement(merge);
				if (result == null)
					result = caseSignalContainer(merge);
				if (result == null)
					result = caseSignalElement(merge);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.NOT: {
				Not not = (Not) theEObject;
				T result = caseNot(not);
				if (result == null)
					result = caseSingleInputOperator(not);
				if (result == null)
					result = caseBooleanOperatorI(not);
				if (result == null)
					result = caseSignalOperator(not);
				if (result == null)
					result = caseNamedElement(not);
				if (result == null)
					result = caseSignalExpr(not);
				if (result == null)
					result = caseConstantExpr(not);
				if (result == null)
					result = caseDataflowOperator(not);
				if (result == null)
					result = caseConditionExpr(not);
				if (result == null)
					result = caseSignalContainer(not);
				if (result == null)
					result = caseSignalElement(not);
				if (result == null)
					result = caseConnectableElement(not);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.NULL_CLOCK: {
				NullClock nullClock = (NullClock) theEObject;
				T result = caseNullClock(nullClock);
				if (result == null)
					result = caseClockedExpr(nullClock);
				if (result == null)
					result = caseControlflowOperator(nullClock);
				if (result == null)
					result = caseSignalElement(nullClock);
				if (result == null)
					result = caseConnectableElement(nullClock);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.NUMERIC_EXPRESSION: {
				NumericExpression numericExpression = (NumericExpression) theEObject;
				T result = caseNumericExpression(numericExpression);
				if (result == null)
					result = caseNamedElement(numericExpression);
				if (result == null)
					result = caseNumericOperatorI(numericExpression);
				if (result == null)
					result = caseSignalElement(numericExpression);
				if (result == null)
					result = caseSignalExpr(numericExpression);
				if (result == null)
					result = caseConstantExpr(numericExpression);
				if (result == null)
					result = caseDataflowOperator(numericExpression);
				if (result == null)
					result = caseConnectableElement(numericExpression);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.OR: {
				Or or = (Or) theEObject;
				T result = caseOr(or);
				if (result == null)
					result = caseMultipleInputsOperator(or);
				if (result == null)
					result = caseBooleanOperatorI(or);
				if (result == null)
					result = caseSignalOperator(or);
				if (result == null)
					result = caseNamedElement(or);
				if (result == null)
					result = caseSignalExpr(or);
				if (result == null)
					result = caseConstantExpr(or);
				if (result == null)
					result = caseDataflowOperator(or);
				if (result == null)
					result = caseConditionExpr(or);
				if (result == null)
					result = caseSignalContainer(or);
				if (result == null)
					result = caseSignalElement(or);
				if (result == null)
					result = caseConnectableElement(or);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.PRAGMA: {
				Pragma pragma = (Pragma) theEObject;
				T result = casePragma(pragma);
				if (result == null)
					result = caseSignalElement(pragma);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.REPETITION_OPERATOR: {
				RepetitionOperator repetitionOperator = (RepetitionOperator) theEObject;
				T result = caseRepetitionOperator(repetitionOperator);
				if (result == null)
					result = caseTwoInputsOperator(repetitionOperator);
				if (result == null)
					result = caseSignalExpr(repetitionOperator);
				if (result == null)
					result = caseConstantExpr(repetitionOperator);
				if (result == null)
					result = caseDataflowOperator(repetitionOperator);
				if (result == null)
					result = caseSignalOperator(repetitionOperator);
				if (result == null)
					result = caseNamedElement(repetitionOperator);
				if (result == null)
					result = caseConnectableElement(repetitionOperator);
				if (result == null)
					result = caseSignalContainer(repetitionOperator);
				if (result == null)
					result = caseSignalElement(repetitionOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SEQUENTIAL_DEFINITION: {
				SequentialDefinition sequentialDefinition = (SequentialDefinition) theEObject;
				T result = caseSequentialDefinition(sequentialDefinition);
				if (result == null)
					result = caseMultipleInputsOperator(sequentialDefinition);
				if (result == null)
					result = caseSignalExpr(sequentialDefinition);
				if (result == null)
					result = caseConstantExpr(sequentialDefinition);
				if (result == null)
					result = caseDataflowOperator(sequentialDefinition);
				if (result == null)
					result = caseSignalOperator(sequentialDefinition);
				if (result == null)
					result = caseNamedElement(sequentialDefinition);
				if (result == null)
					result = caseConnectableElement(sequentialDefinition);
				if (result == null)
					result = caseSignalContainer(sequentialDefinition);
				if (result == null)
					result = caseSignalElement(sequentialDefinition);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.SWITCH: {
				Switch switch_ = (Switch) theEObject;
				T result = caseSwitch(switch_);
				if (result == null)
					result = caseNamedElement(switch_);
				if (result == null)
					result = caseDataflowOperator(switch_);
				if (result == null)
					result = caseConnectableElement(switch_);
				if (result == null)
					result = caseSignalElement(switch_);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.TRANSPOSITION: {
				Transposition transposition = (Transposition) theEObject;
				T result = caseTransposition(transposition);
				if (result == null)
					result = caseSingleInputOperator(transposition);
				if (result == null)
					result = caseSignalExpr(transposition);
				if (result == null)
					result = caseConstantExpr(transposition);
				if (result == null)
					result = caseDataflowOperator(transposition);
				if (result == null)
					result = caseSignalOperator(transposition);
				if (result == null)
					result = caseNamedElement(transposition);
				if (result == null)
					result = caseConnectableElement(transposition);
				if (result == null)
					result = caseSignalContainer(transposition);
				if (result == null)
					result = caseSignalElement(transposition);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.TUPLE_OPERATOR: {
				TupleOperator tupleOperator = (TupleOperator) theEObject;
				T result = caseTupleOperator(tupleOperator);
				if (result == null)
					result = caseMultipleInputsOperator(tupleOperator);
				if (result == null)
					result = caseSignalExpr(tupleOperator);
				if (result == null)
					result = caseConstantExpr(tupleOperator);
				if (result == null)
					result = caseDataflowOperator(tupleOperator);
				if (result == null)
					result = caseSignalOperator(tupleOperator);
				if (result == null)
					result = caseNamedElement(tupleOperator);
				if (result == null)
					result = caseConnectableElement(tupleOperator);
				if (result == null)
					result = caseSignalContainer(tupleOperator);
				if (result == null)
					result = caseSignalElement(tupleOperator);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.UNARY_MINUS: {
				UnaryMinus unaryMinus = (UnaryMinus) theEObject;
				T result = caseUnaryMinus(unaryMinus);
				if (result == null)
					result = caseSingleInputOperator(unaryMinus);
				if (result == null)
					result = caseNumericOperatorI(unaryMinus);
				if (result == null)
					result = caseSignalOperator(unaryMinus);
				if (result == null)
					result = caseNamedElement(unaryMinus);
				if (result == null)
					result = caseSignalExpr(unaryMinus);
				if (result == null)
					result = caseConstantExpr(unaryMinus);
				if (result == null)
					result = caseDataflowOperator(unaryMinus);
				if (result == null)
					result = caseSignalContainer(unaryMinus);
				if (result == null)
					result = caseSignalElement(unaryMinus);
				if (result == null)
					result = caseConnectableElement(unaryMinus);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.VARIABLE: {
				Variable variable = (Variable) theEObject;
				T result = caseVariable(variable);
				if (result == null)
					result = caseSingleInputOperator(variable);
				if (result == null)
					result = caseSignalExpr(variable);
				if (result == null)
					result = caseDataflowOperator(variable);
				if (result == null)
					result = caseSignalOperator(variable);
				if (result == null)
					result = caseNamedElement(variable);
				if (result == null)
					result = caseConnectableElement(variable);
				if (result == null)
					result = caseSignalContainer(variable);
				if (result == null)
					result = caseSignalElement(variable);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.WINDOW: {
				Window window = (Window) theEObject;
				T result = caseWindow(window);
				if (result == null)
					result = caseSingleInputOperator(window);
				if (result == null)
					result = caseSignalExpr(window);
				if (result == null)
					result = caseDataflowOperator(window);
				if (result == null)
					result = caseSignalOperator(window);
				if (result == null)
					result = caseNamedElement(window);
				if (result == null)
					result = caseConnectableElement(window);
				if (result == null)
					result = caseSignalContainer(window);
				if (result == null)
					result = caseSignalElement(window);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.XOR: {
				Xor xor = (Xor) theEObject;
				T result = caseXor(xor);
				if (result == null)
					result = caseMultipleInputsOperator(xor);
				if (result == null)
					result = caseBooleanOperatorI(xor);
				if (result == null)
					result = caseSignalOperator(xor);
				if (result == null)
					result = caseNamedElement(xor);
				if (result == null)
					result = caseSignalExpr(xor);
				if (result == null)
					result = caseConstantExpr(xor);
				if (result == null)
					result = caseDataflowOperator(xor);
				if (result == null)
					result = caseConditionExpr(xor);
				if (result == null)
					result = caseSignalContainer(xor);
				if (result == null)
					result = caseSignalElement(xor);
				if (result == null)
					result = caseConnectableElement(xor);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.STATE_OR_HISTORY: {
				StateOrHistory stateOrHistory = (StateOrHistory) theEObject;
				T result = caseStateOrHistory(stateOrHistory);
				if (result == null)
					result = caseConnectableElement(stateOrHistory);
				if (result == null)
					result = caseSignalElement(stateOrHistory);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.HISTORY: {
				History history = (History) theEObject;
				T result = caseHistory(history);
				if (result == null)
					result = caseStateOrHistory(history);
				if (result == null)
					result = caseOperatorPort(history);
				if (result == null)
					result = caseConnectableElement(history);
				if (result == null)
					result = caseSignalElement(history);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.STATE_BASE: {
				StateBase stateBase = (StateBase) theEObject;
				T result = caseStateBase(stateBase);
				if (result == null)
					result = caseStateOrHistory(stateBase);
				if (result == null)
					result = caseIdentifier(stateBase);
				if (result == null)
					result = caseConnectableElement(stateBase);
				if (result == null)
					result = caseNamedElement(stateBase);
				if (result == null)
					result = caseSignalElement(stateBase);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.STATE: {
				State state = (State) theEObject;
				T result = caseState(state);
				if (result == null)
					result = caseModel(state);
				if (result == null)
					result = caseStateBase(state);
				if (result == null)
					result = caseSignalContainer(state);
				if (result == null)
					result = caseStateOrHistory(state);
				if (result == null)
					result = caseIdentifier(state);
				if (result == null)
					result = caseSignalElement(state);
				if (result == null)
					result = caseConnectableElement(state);
				if (result == null)
					result = caseNamedElement(state);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.COMPOUND_STATE: {
				CompoundState compoundState = (CompoundState) theEObject;
				T result = caseCompoundState(compoundState);
				if (result == null)
					result = caseStateBase(compoundState);
				if (result == null)
					result = caseSignalContainer(compoundState);
				if (result == null)
					result = caseStateOrHistory(compoundState);
				if (result == null)
					result = caseIdentifier(compoundState);
				if (result == null)
					result = caseSignalElement(compoundState);
				if (result == null)
					result = caseConnectableElement(compoundState);
				if (result == null)
					result = caseNamedElement(compoundState);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.AUTOMATON: {
				Automaton automaton = (Automaton) theEObject;
				T result = caseAutomaton(automaton);
				if (result == null)
					result = caseCompoundState(automaton);
				if (result == null)
					result = caseClockedExpr(automaton);
				if (result == null)
					result = caseDependenceInOut(automaton);
				if (result == null)
					result = caseStateBase(automaton);
				if (result == null)
					result = caseSignalContainer(automaton);
				if (result == null)
					result = caseConnectableElement(automaton);
				if (result == null)
					result = caseStateOrHistory(automaton);
				if (result == null)
					result = caseIdentifier(automaton);
				if (result == null)
					result = caseSignalElement(automaton);
				if (result == null)
					result = caseNamedElement(automaton);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.AND_STATE: {
				AndState andState = (AndState) theEObject;
				T result = caseAndState(andState);
				if (result == null)
					result = caseCompoundState(andState);
				if (result == null)
					result = caseStateBase(andState);
				if (result == null)
					result = caseSignalContainer(andState);
				if (result == null)
					result = caseStateOrHistory(andState);
				if (result == null)
					result = caseIdentifier(andState);
				if (result == null)
					result = caseSignalElement(andState);
				if (result == null)
					result = caseConnectableElement(andState);
				if (result == null)
					result = caseNamedElement(andState);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.TRANSITION: {
				Transition transition = (Transition) theEObject;
				T result = caseTransition(transition);
				if (result == null)
					result = caseConnection(transition);
				if (result == null)
					result = caseNamedElement(transition);
				if (result == null)
					result = caseSignalElement(transition);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.WEAK_TRANSITION: {
				WeakTransition weakTransition = (WeakTransition) theEObject;
				T result = caseWeakTransition(weakTransition);
				if (result == null)
					result = caseTransition(weakTransition);
				if (result == null)
					result = caseConnection(weakTransition);
				if (result == null)
					result = caseNamedElement(weakTransition);
				if (result == null)
					result = caseSignalElement(weakTransition);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			case SmePackage.STRONG_TRANSITION: {
				StrongTransition strongTransition = (StrongTransition) theEObject;
				T result = caseStrongTransition(strongTransition);
				if (result == null)
					result = caseTransition(strongTransition);
				if (result == null)
					result = caseConnection(strongTransition);
				if (result == null)
					result = caseNamedElement(strongTransition);
				if (result == null)
					result = caseSignalElement(strongTransition);
				if (result == null)
					result = defaultCase(theEObject);
				return result;
			}
			default:
				return defaultCase(theEObject);
		}
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Root Model</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Root Model</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseRootModel(RootModel object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Signal Element</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Signal Element</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSignalElement(SignalElement object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Connectable Element</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Connectable Element</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConnectableElement(ConnectableElement object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Named Element</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Named Element</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseNamedElement(NamedElement object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Identifier</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Identifier</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseIdentifier(Identifier object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Signal Expr</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Signal Expr</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSignalExpr(SignalExpr object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Constant Expr</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Constant Expr</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConstantExpr(ConstantExpr object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Clocked Expr</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Clocked Expr</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseClockedExpr(ClockedExpr object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Dependence In Out</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Dependence In Out</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDependenceInOut(DependenceInOut object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Signal Container</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Signal Container</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSignalContainer(SignalContainer object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Interface Definition</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Interface Definition</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseInterfaceDefinition(InterfaceDefinition object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Model</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Model</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModel(Model object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Basic Iterate</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Basic Iterate</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseBasicIterate(BasicIterate object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Iterate</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Iterate</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseIterate(Iterate object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Iteration Init</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Iteration Init</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseIterationInit(IterationInit object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Specifications</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Specifications</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSpecifications(Specifications object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sub Process</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sub Process</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSubProcess(SubProcess object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Type Declaration</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Type Declaration</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseTypeDeclaration(TypeDeclaration object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Parameter Type</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Parameter Type</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseParameterType(ParameterType object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Signal Type</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Signal Type</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSignalType(SignalType object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Enumeration Type</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Enumeration Type</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseEnumerationType(EnumerationType object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Model Type</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Model Type</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModelType(ModelType object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Substitution Type</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Substitution Type</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSubstitutionType(SubstitutionType object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Tuple Type</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Tuple Type</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseTupleType(TupleType object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Model Declaration</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Model Declaration</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModelDeclaration(ModelDeclaration object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Model Ref</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Model Ref</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModelRef(ModelRef object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Model Instance</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Model Instance</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModelInstance(ModelInstance object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Module</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Module</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModule(Module object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Use Module</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Use Module</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseUseModule(UseModule object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Dataflow Connector</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Dataflow Connector</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDataflowConnector(DataflowConnector object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Constant Or Ref</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Constant Or Ref</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConstantOrRef(ConstantOrRef object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Constant Ref</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Constant Ref</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConstantRef(ConstantRef object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Constant</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Constant</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConstant(Constant object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Constant Value</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Constant Value</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConstantValue(ConstantValue object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Literal</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Literal</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseLiteral(Literal object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Enumeration Value</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Enumeration Value</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseEnumerationValue(EnumerationValue object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Parameter</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Parameter</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseParameter(Parameter object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Parameter Instance</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Parameter Instance</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseParameterInstance(ParameterInstance object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Iterator</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Iterator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseIterator(Iterator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Index</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Index</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseIndex(Index object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Signal Or Ref</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Signal Or Ref</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSignalOrRef(SignalOrRef object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Signal Ref</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Signal Ref</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSignalRef(SignalRef object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Signal</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Signal</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSignal(Signal object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Input</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Input</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseInput(Input object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Input Instance</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Input Instance</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseInputInstance(InputInstance object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Output</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Output</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOutput(Output object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Output Instance</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Output Instance</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOutputInstance(OutputInstance object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Local</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Local</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseLocal(Local object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Connection</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Connection</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConnection(Connection object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Model Connection</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Model Connection</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModelConnection(ModelConnection object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Case Connection</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Case Connection</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCaseConnection(CaseConnection object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Clock Relation</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Clock Relation</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseClockRelation(ClockRelation object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Condition</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Condition</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCondition(Condition object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Constant Affectation</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Constant Affectation</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConstantAffectation(ConstantAffectation object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Dependence</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Dependence</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDependence(Dependence object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Dataflow Connection</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Dataflow Connection</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDataflowConnection(DataflowConnection object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Operator Port</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Operator Port</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOperatorPort(OperatorPort object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Input Port</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Input Port</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseInputPort(InputPort object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Output Port</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Output Port</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOutputPort(OutputPort object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Signal Operator</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Signal Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSignalOperator(SignalOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Condition Expr</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Condition Expr</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConditionExpr(ConditionExpr object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Conditionned Expr</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Conditionned Expr</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConditionnedExpr(ConditionnedExpr object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Controlflow Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Controlflow Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseControlflowOperator(ControlflowOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Dataflow Operator</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Dataflow Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDataflowOperator(DataflowOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Boolean Operator I</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Boolean Operator I</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseBooleanOperatorI(BooleanOperatorI object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Numeric Operator I</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Numeric Operator I</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseNumericOperatorI(NumericOperatorI object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Single Input Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Single Input Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSingleInputOperator(SingleInputOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Two Inputs Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Two Inputs Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseTwoInputsOperator(TwoInputsOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Multiple Inputs Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Multiple Inputs Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseMultipleInputsOperator(MultipleInputsOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>And</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>And</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAnd(And object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Arithmetic Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Arithmetic Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseArithmeticOperator(ArithmeticOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Array Enumeration</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Array Enumeration</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseArrayEnumeration(ArrayEnumeration object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Array Product</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Array Product</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseArrayProduct(ArrayProduct object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Array Recovery</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Array Recovery</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseArrayRecovery(ArrayRecovery object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Array Restructuration</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Array Restructuration</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseArrayRestructuration(ArrayRestructuration object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Assertion</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Assertion</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAssertion(Assertion object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Assertion Clock Constraint Operator</em>'.
	 * <!-- begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Assertion Clock Constraint Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAssertionClockConstraintOperator(AssertionClockConstraintOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Assertion Clock Speed Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Assertion Clock Speed Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAssertionClockSpeedOperator(AssertionClockSpeedOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Boolean Expression</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Boolean Expression</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseBooleanExpression(BooleanExpression object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Cartesian Product</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Cartesian Product</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCartesianProduct(CartesianProduct object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Cell</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Cell</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCell(Cell object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Clock Constraint Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Clock Constraint Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseClockConstraintOperator(ClockConstraintOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Clock Relation Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Clock Relation Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseClockRelationOperator(ClockRelationOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Clock Speed Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Clock Speed Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseClockSpeedOperator(ClockSpeedOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Comparison Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Comparison Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseComparisonOperator(ComparisonOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Complex Operator</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Complex Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseComplexOperator(ComplexOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Concatenation</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Concatenation</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseConcatenation(Concatenation object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Counter</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Counter</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCounter(Counter object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Delay</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Delay</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDelay(Delay object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Dependence Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Dependence Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDependenceOperator(DependenceOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Extraction</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Extraction</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseExtraction(Extraction object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>If Then Else</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>If Then Else</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseIfThenElse(IfThenElse object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Merge</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Merge</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseMerge(Merge object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Not</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Not</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseNot(Not object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Null Clock</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Null Clock</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseNullClock(NullClock object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Numeric Expression</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Numeric Expression</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseNumericExpression(NumericExpression object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Or</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Or</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOr(Or object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Pragma</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Pragma</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T casePragma(Pragma object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Repetition Operator</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Repetition Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseRepetitionOperator(RepetitionOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sequential Definition</em>'. <!--
	 * begin-user-doc --> This implementation returns null; returning a non-null result will terminate the switch. <!--
	 * end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sequential Definition</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSequentialDefinition(SequentialDefinition object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Switch</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Switch</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSwitch(Switch object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Transposition</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Transposition</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseTransposition(Transposition object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Tuple Operator</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Tuple Operator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseTupleOperator(TupleOperator object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Unary Minus</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Unary Minus</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseUnaryMinus(UnaryMinus object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Variable</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Variable</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseVariable(Variable object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Window</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Window</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseWindow(Window object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Xor</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Xor</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseXor(Xor object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>State Or History</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>State Or History</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseStateOrHistory(StateOrHistory object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>History</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>History</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseHistory(History object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>State Base</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>State Base</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseStateBase(StateBase object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>State</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>State</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseState(State object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Compound State</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Compound State</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCompoundState(CompoundState object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Automaton</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Automaton</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAutomaton(Automaton object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>And State</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>And State</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAndState(AndState object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Transition</em>'. <!-- begin-user-doc -->
	 * This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Transition</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseTransition(Transition object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Weak Transition</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Weak Transition</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseWeakTransition(WeakTransition object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>Strong Transition</em>'. <!-- begin-user-doc
	 * --> This implementation returns null; returning a non-null result will terminate the switch. <!-- end-user-doc
	 * -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Strong Transition</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseStrongTransition(StrongTransition object) {
		return null;
	}
	
	/**
	 * Returns the result of interpreting the object as an instance of '<em>EObject</em>'. <!-- begin-user-doc --> This
	 * implementation returns null; returning a non-null result will terminate the switch, but this is the last case
	 * anyway. <!-- end-user-doc -->
	 * 
	 * @param object
	 *        the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject)
	 * @generated
	 */
	public T defaultCase(EObject object) {
		return null;
	}
	
} // SmeSwitch
